(*         ______   ___    ___
 *        /\  _  \ /\_ \  /\_ \
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      BMP reader.
 *
 *      By Seymour Shlien.
 *
 *      OS/2 BMP support and BMP save function by Jonas Petersen.
 *
 *      See readme.txt for copyright information.
 *)

{
  #include "allegro.h"
  #include "allegro/internal/aintern.h"
}

const
  BI_RGB          = 0;
  BI_RLE8         = 1;
  BI_RLE4         = 2;
  BI_BITFIELDS    = 3;
  _rgb_r_shift_24 = 16;
  _rgb_g_shift_24 =  8;
  _rgb_b_shift_24 =  0;

  OS2INFOHEADERSIZE = 12;
  WININFOHEADERSIZE = 40;

type
  TRGB = record
    case longint of
     0: (b,g,r: byte);
     1: (color: Longint);
  end;

  TRGBA = record
    case longint of
     0: (r,g,b,a: byte);
     1: (color: Longint);
  end;

  TRGBPalette = array[0..255] of TRGB;
  TRGBAPalette = array[0..255] of TRGBA;

type
  BITMAPFILEHEADER = record
   bfType     : Longword;
   bfSize     : Longword;
   bfReserved1: word;
   bfReserved2: word;
   bfOffBits  : Longword;
  end;


(* Used for both OS/2 and Windows BMP. 
 * Contains only the parameters needed to load the image 
 *)
type
  BITMAPINFOHEADER = record

   biWidth      : Longword;
   biHeight     : Longword;
   biBitCount   : Longword; { word }
   biCompression: Longword;
  end;


type
  WINBMPINFOHEADER = record (* size: 40 *)
   biWidth        : Longword; {alles unsigned long -> longword -> ausser biPlanes, biBitCount }
   biHeight       : Longword;
   biPlanes       : Longword; {word -> unsigned short}
   biBitCount     : Longword; {word -> unsigned short}
   biCompression  : Longword;
   biSizeImage    : Longword;
   biXPelsPerMeter: Longword;
   biYPelsPerMeter: Longword;
   biClrUsed      : Longword;
   biClrImportant : Longword;
  end;


type 
  OS2BMPINFOHEADER = record (* size: 12 *)
   biWidth    : word;
   biHeight   : word;
   biPlanes   : word;
   biBitCount : word;
  end;

type
  BITMAP = record
   line: {$ifdef OPascal}array of array of integer{$else}PDynamicArray{$endif};
  end;

type
  PPackFile = ^TPackFile;
  TPackFile = record
    {$ifdef use_stream}
    f: TFileStream;
    {$else}
    f: file;
    {$endif}
  end;

function pack_getc(f: PPackFile): Longint;
begin

end;

function pack_igetw(f: PPackFile): Word;
begin

end;

function pack_igetl(f: PPackFile): Longword;
begin

end;

function pack_mgetl(f: PPackFile): Longword;
begin

end;


(* read_bmfileheader:
 *  Reads a BMP file header and check that it has the BMP magic number.
 *)
function read_bmfileheader(f: PPACKFILE; var fileheader: BITMAPFILEHEADER): Integer;
begin
   fileheader.bfType     := pack_igetw(f);
   fileheader.bfSize     := pack_igetl(f);
   fileheader.bfReserved1:= pack_igetw(f);
   fileheader.bfReserved2:= pack_igetw(f);
   fileheader.bfOffBits  := pack_igetl(f);

   if (fileheader.bfType <> 19778)
     then read_bmfileheader := -1
   else read_bmfileheader := 0;
end;



(* read_win_bminfoheader:
 *  Reads information from a BMP file header.
 *)
function read_win_bminfoheader(f: PPACKFILE; var infoheader: BITMAPINFOHEADER): Integer;
var
  win_infoheader: WINBMPINFOHEADER;
begin
   win_infoheader.biWidth := pack_igetl(f);
   win_infoheader.biHeight := pack_igetl(f);
   win_infoheader.biPlanes := pack_igetw(f);
   win_infoheader.biBitCount := pack_igetw(f);
   win_infoheader.biCompression := pack_igetl(f);
   win_infoheader.biSizeImage := pack_igetl(f);
   win_infoheader.biXPelsPerMeter := pack_igetl(f);
   win_infoheader.biYPelsPerMeter := pack_igetl(f);
   win_infoheader.biClrUsed := pack_igetl(f);
   win_infoheader.biClrImportant := pack_igetl(f);

   infoheader.biWidth := win_infoheader.biWidth;
   infoheader.biHeight := win_infoheader.biHeight;
   infoheader.biBitCount := win_infoheader.biBitCount;
   infoheader.biCompression := win_infoheader.biCompression;

   read_win_bminfoheader := 0;
end;



(* read_os2_bminfoheader:
 *  Reads information from an OS/2 format BMP file header.
 *)
function read_os2_bminfoheader(f: PPACKFILE; infoheader: BITMAPINFOHEADER): Integer;
var
  os2_infoheader: OS2BMPINFOHEADER;
begin
   os2_infoheader.biWidth := pack_igetw(f);
   os2_infoheader.biHeight := pack_igetw(f);
   os2_infoheader.biPlanes := pack_igetw(f);
   os2_infoheader.biBitCount := pack_igetw(f);

   infoheader.biWidth := os2_infoheader.biWidth;
   infoheader.biHeight := os2_infoheader.biHeight;
   infoheader.biBitCount := os2_infoheader.biBitCount;
   infoheader.biCompression := 0;

   read_os2_bminfoheader := 0;
end;



(* read_bmicolors:
 *  Loads the color palette for 1,4,8 bit formats.
 *)
procedure read_bmicolors(ncols: integer; var pal: TRGBPalette; f: PPACKFILE; win_flag: integer);
var
   i: integer;
begin
   for i:=0 to ncols-1 do
   begin
      pal[i].b := pack_getc(f) div 4;
      pal[i].g := pack_getc(f) div 4;
      pal[i].r := pack_getc(f) div 4;
      if win_flag > 0 then
	  pack_getc(f);
   end;
end;



(* read_1bit_line:
 *  Support function for reading the 1 bit bitmap file format.
 *)
procedure read_1bit_line(length: integer; f: PPACKFILE; var bmp: BITMAP; line: integer);
var
   b: array[0..31] of byte;
   n: longint;
   i, j, k: integer;
   pix: longint;
begin
   for i:=0 to length-1 do
   begin
      j := i mod 32;
      if (j = 0) then
      begin
        n := pack_mgetl(f);
	    for k:=0 to 31 do
            begin
	      b[31-k] := n and 1;
	      n := n shr 1;
	    end;
      end;
      pix := b[j];
      bmp.line[line][i] := pix;
   end;
end;



(* read_4bit_line:
 *  Support function for reading the 4 bit bitmap file format.
 *)
procedure read_4bit_line(length: integer; f: PPACKFILE; var bmp: BITMAP; line: integer);
var
   b: array[0..7] of byte;
   n: Longint;
   i, j, k: integer;
   temp: integer;
   pix: longint;
begin
   for i:=0 to length-1 do
   begin
      j := i mod 8;
      if (j = 0) then
      begin
        n := pack_igetl(f);
        for k:=0 to 3 do
        begin
	  temp := n and 255;
          b[k*2+1] := temp and 15;
          temp := temp shr 4;
          b[k*2] := temp and 15;
	  n := n shr 8;
	end;
      end;
      pix := b[j];
      bmp.line[line][i] := pix;
   end;
end;



(* read_8bit_line:
 *  Support function for reading the 8 bit bitmap file format.
 *)
procedure read_8bit_line(length: integer; f: PPACKFILE; var bmp: BITMAP; line: integer);
var
   b: array[0..3] of byte;
   n: longint;
   i, j, k: integer;
   pix: longint;
begin
   for i:=0 to length-1 do
   begin
      j := i mod 4;
      if (j = 0) then
      begin
	n := pack_igetl(f);
	for k:=0 to 3 do
	begin
          b[k] := n and 255;
          n := n shr 8;
	end;
      end;
      pix := b[j];
      bmp.line[line][i] := pix;
   end;
end;



(* read_24bit_line:
 *  Support function for reading the 24 bit bitmap file format, doing
 *  our best to convert it down to a 256 color palette.
 *)
procedure read_24bit_line(length: integer; f: PPACKFILE; var bmp: BITMAP; line: integer);
var
   i, nbytes: integer;
   c: TRGB;
begin

   nbytes:=0;

   for i:=0 to length-1 do
   begin
      c.b := pack_getc(f);
      c.g := pack_getc(f);
      c.r := pack_getc(f);
      bmp.line[line][i*3+_rgb_r_shift_24 div 8] := c.r;
      bmp.line[line][i*3+_rgb_g_shift_24 div 8] := c.g;
      bmp.line[line][i*3+_rgb_b_shift_24 div 8] := c.b;
      inc(nbytes, 3);
   end;

   nbytes := nbytes mod 4;
   if (nbytes <> 0) then
      for i:=nbytes to 3 do
	 pack_getc(f);
end;



(* read_image:
 *  For reading the noncompressed BMP image format.
 *)
procedure read_image(f: PPACKFILE; var bmp: BITMAP; CONST infoheader: BITMAPINFOHEADER);
var
   i, line: integer;
begin
   for i:=0 to infoheader.biHeight - 1 do
   begin
      line := i;

      case infoheader.biBitCount of

	 1:
	    read_1bit_line(infoheader.biWidth, f, bmp, infoheader.biHeight-i-1);
	    {break;}

	 4:
	    read_4bit_line(infoheader.biWidth, f, bmp, infoheader.biHeight-i-1);
	    {break;}

	 8:
	    read_8bit_line(infoheader.biWidth, f, bmp, infoheader.biHeight-i-1);
	    {break;}

        24:
	    read_24bit_line(infoheader.biWidth, f, bmp, infoheader.biHeight-i-1);
	    {break;}
      end; { von case }
   end; { von for }
end;



(* read_RLE8_compressed_image:
 *  For reading the 8 bit RLE compressed BMP image format.
 *)
procedure read_RLE8_compressed_image(f: PPACKFILE; var bmp: BITMAP; CONST infoheader: BITMAPINFOHEADER);
var
   count, valx, val0: byte;
   j, pos, line: integer;
   eolflag, eopicflag: integer;
begin
   eopicflag := 0;
   line := infoheader.biHeight - 1;

   while (eopicflag = 0) do
   begin
      pos := 0;                               (* x position in bitmap *)
      eolflag := 0;                           (* end of line flag *)

      while ((eolflag = 0) and (eopicflag = 0)) do
      begin
	 count := pack_getc(f);
	 valx := pack_getc(f);

	 if (count > 0) then                 (* repeat pixel count times *)
         begin
	    for j:=0 to count-1 do  begin
	       bmp.line[line][pos] := valx;
	       inc(pos);
	    end;
	 end
	 else begin
	    case valx of

	       0:                       (* end of line flag *)
		  eolflag:=1;


	       1:                       (* end of picture flag *)
		  eopicflag:=1;


	       2:                        (* displace picture *)
                  begin
                    count := pack_getc(f);
                    valx := pack_getc(f);
                    pos := pos + count;
                    line := line - valx;
                  end;

	       else                          (* read in absolute mode *)
                 begin
                   for j:=0 to valx-1 do
                   begin
		     val0 := pack_getc(f);
		     bmp.line[line][pos] := val0;
		     inc(pos);
		   end;

		   if (j mod 2) = 1 then
		     val0 := pack_getc(f);    (* align on word boundary *)
                 end;
	    end;
	 end;

	 if (pos-1 > infoheader.biWidth) then eolflag := 1;
      end;

      dec(line);
      if (line < 0) then
	 eopicflag := 1;
   end;
end;



(* read_RLE4_compressed_image:
 *  For reading the 4 bit RLE compressed BMP image format.
 *)
procedure read_RLE4_compressed_image(f: PPACKFILE; var bmp: BITMAP; CONST infoheader: BITMAPINFOHEADER);
var
   b: array[0..7] of byte;
   count: byte;
   {unsigned short} val0, val: integer;
   j, k, pos, line: integer;
   eolflag, eopicflag: integer;
begin
   eopicflag := 0;                            (* end of picture flag *)
   line := infoheader.biHeight - 1;

   while (eopicflag = 0) do
   begin
      pos := 0;
      eolflag := 0;                            (* end of line flag *)

      while ((eolflag = 0) and (eopicflag = 0)) do
      begin
	 count := pack_getc(f);
	 val := pack_getc(f);

	 if (count > 0) then                  (* repeat pixels count times *)
         begin
	    b[1] := val and 15;
	    b[0] := (val shr 4) and 15;
	    for j:=0 to count-1 do begin
	       bmp.line[line][pos] := b[j mod 2];
	       inc(pos);
	    end;
	 end
	 else begin
	    case val of

	        0:                       (* end of line *)
		  eolflag := 1;

	        1:                       (* end of picture *)
		  eopicflag := 1;

	        2:                       (* displace image *)
                  begin
                    count := pack_getc(f);
		    val := pack_getc(f);
		    pos := pos + count;
		    line := line - val;
                  end;

	       else begin                    (* read in absolute mode *)
		  for j:=0 to val-1 do
                  begin
		     if ((j mod 4) = 0) then
                     begin
			val0 := pack_igetw(f);
			for k:=0 to 1 do
                        begin
			   b[2*k+1] := val0 and 15;
			   val0 := val0 shr 4;
			   b[2*k] := val0 mod 15;
			   val0 := val0 shr 4;
			end;
		     end;
		     bmp.line[line][pos] := b[j mod 4];
		     inc(pos);
		  end;
               end;
	    end;
	 end;

	 if (pos-1 > infoheader.biWidth) then
	    eolflag:=1;
      end;

      dec(line);
      if (line < 0) then
	 eopicflag := 1;
   end;
end;



(* load_bmp:
 *  Loads a Windows BMP file, returning a bitmap structure and storing
 *  the palette data in the specified palette (this should be an array of
 *  at least 256 RGB structures).
 *
 *  Thanks to Seymour Shlien for contributing this function.
 *)
(* BITMAP *load_bmp(AL_CONST char *filename, RGB *pal): Bitmap; *)
{
   BITMAPFILEHEADER fileheader;
   BITMAPINFOHEADER infoheader;
   PACKFILE *f;
   BITMAP *bmp;
   PALETTE tmppal;
   int ncol;
   unsigned long biSize;
   int bpp, dest_depth;

   if (!pal)
      pal = tmppal;

   f = pack_fopen(filename, F_READ);
   if (!f)
      return NULL;

   if (read_bmfileheader(f, &fileheader) != 0) {
      pack_fclose(f);
      return NULL;
   }

   biSize = pack_igetl(f);

   if (biSize == WININFOHEADERSIZE) {
      if (read_win_bminfoheader(f, &infoheader) != 0) {
	 pack_fclose(f);
	 return NULL;
      }
      /* compute number of colors recorded */
      ncol = (fileheader.bfOffBits - 54) / 4;
      read_bmicolors(ncol, pal, f, 1);
   }
   else if (biSize == OS2INFOHEADERSIZE) {
      if (read_os2_bminfoheader(f, &infoheader) != 0) {
	 pack_fclose(f);
	 return NULL;
      }
      /* compute number of colors recorded */
      ncol = (fileheader.bfOffBits - 26) / 3;
      read_bmicolors(ncol, pal, f, 0);
   }
   else {
      pack_fclose(f);
      return NULL;
   }

   if (infoheader.biBitCount == 24) {
      bpp = 24;
      generate_332_palette(pal);
   }
   else
      bpp = 8;

   dest_depth = _color_load_depth(bpp, FALSE);

   bmp = create_bitmap_ex(bpp, infoheader.biWidth, infoheader.biHeight);
   if (!bmp) {
      pack_fclose(f);
      return NULL;
   }

   clear_bitmap(bmp);

   switch (infoheader.biCompression) {

      case BI_RGB:
	 read_image(f, bmp, &infoheader);
	 break;

      case BI_RLE8:
	 read_RLE8_compressed_image(f, bmp, &infoheader);
	 break;

      case BI_RLE4:
	 read_RLE4_compressed_image(f, bmp, &infoheader);
	 break;

      default:
	 destroy_bitmap(bmp);
	 bmp = NULL;
   }

   if (dest_depth != bpp)
      bmp = _fixup_loaded_bitmap(bmp, pal, dest_depth);

   pack_fclose(f);
   return bmp;
}



(* save_bmp:
 *  Writes a bitmap into a BMP file, using the specified palette (this
 *  should be an array of at least 256 RGB structures).
 *)
(* function save_bmp(AL_CONST char *filename, BITMAP *bmp, AL_CONST RGB *pal): integer; *)
{
   PACKFILE *f;
   PALETTE tmppal;
   int bfSize;
   int depth = bitmap_color_depth(bmp);
   int bpp = (depth == 8) ? 8 : 24;
   int filler = 3 - ((bmp->w*(bpp/8)-1) & 3);
   int c, i, j;

   if (!pal) {
      get_palette(tmppal);
      pal = tmppal;
   }

   if (bpp == 8) {
      bfSize = 54                      /* header */
	       +256*4                  /* palette */
	       +bmp->w*bmp->h;         /* image data */
   }
   else {
      bfSize = 54                      /* header */
	       +bmp->w*bmp->h*3;       /* image data */
   }

   f = pack_fopen(filename, F_WRITE);
   if (!f) 
      return *allegro_errno;

   /* file_header */
   pack_iputw(0x4D42, f);              /* bfType ("BM") */
   pack_iputl(bfSize, f);              /* bfSize */
   pack_iputw(0, f);                   /* bfReserved1 */
   pack_iputw(0, f);                   /* bfReserved2 */

   if (bpp == 8)                       /* bfOffBits */
      pack_iputl(54+256*4, f); 
   else
      pack_iputl(54, f); 

   /* info_header */
   bfSize = bmp->w * bmp->h * bpp/8;

   pack_iputl(40, f);                  /* biSize */
   pack_iputl(bmp->w, f);              /* biWidth */
   pack_iputl(bmp->h, f);              /* biHeight */
   pack_iputw(1, f);                   /* biPlanes */
   pack_iputw(bpp, f);                 /* biBitCount */
   pack_iputl(0, f);                   /* biCompression */
   pack_iputl(bfSize, f);              /* biSizeImage */
   pack_iputl(0, f);                   /* biXPelsPerMeter */
   pack_iputl(0, f);                   /* biYPelsPerMeter */

   if (bpp == 8) {
      pack_iputl(256, f);              /* biClrUsed */
      pack_iputl(256, f);              /* biClrImportant */

      /* palette */
      for (i=0; i<256; i++) {
	 pack_putc(_rgb_scale_6[pal[i].b], f);
	 pack_putc(_rgb_scale_6[pal[i].g], f);
	 pack_putc(_rgb_scale_6[pal[i].r], f);
	 pack_putc(0, f);
      }
   }
   else {
      pack_iputl(0, f);                /* biClrUsed */
      pack_iputl(0, f);                /* biClrImportant */
   }

   /* image data */
   for (i=bmp->h-1; i>=0; i--) {
      for (j=0; j<bmp->w; j++) {
	 if (bpp == 8) {
	    pack_putc(getpixel(bmp, j, i), f);
	 }
	 else {
	    c = getpixel(bmp, j, i);
	    pack_putc(getb_depth(depth, c), f);
	    pack_putc(getg_depth(depth, c), f);
	    pack_putc(getr_depth(depth, c), f);
	 }
      }

      for (j=0; j<filler; j++)
	 pack_putc(0, f);
   }

   pack_fclose(f);
   return *allegro_errno;
}

