(*
/////////////////////////////////////////////////////////////////////////////////
//
//	Button Core File 
//
//	Aura Milestone 4 
//
//	(c) Copyright 2012 Finn Technologies and Chase Finn. All rights reserved.
//
/////////////////////////////////////////////////////////////////////////////////
*)
{
 .#include "kernel.h"
 .#include "widget.h"
 .#include "button.h"
 .#include "grfx.h"
 .#include "internal.h"
}
unit button;

interface

uses
  guitypes, widget, grfx, metrics, events, dynld, skin, font, allegro, text;
  
{$i cdefs.inc}

{$IFDEF LINK_CLIBS}
   {$LINK widget.o}
   {$LINK grfx.o}
   {$LINK skin.o}
{$ENDIF}  

var
  AppVersion: l_ulong;
  
const
  AppName: String       = 'Button Widget';
  nUID   : String       = 'button';
  NeededLibs: array[0..3] of String	= ( 'widget', 'skin', 'grfx', '' );

  BS_OVER	=	1;
  BS_DOWN	=	2;


type 
  PButton = ^TButton;
  TButton = record

	o: TWidget;

	Caption: l_text; ///< Caption
	
	Message: l_ulong; ///< Message send when clicked
    MessageExtra: Pointer; ///< Extra value send with message when clicked
	
	(*
		Facultative bitmap drawn on button
		@note By default, icon is on the left of the text, whole content is center
		@see buttonflags
	*)
	
	Icon: p_bitmap;
	
	State: l_ushort;
	
	(*
		Button flags
		@see buttonflags
	*)
	Flags: l_ushort;
  end;

function CreateButton( app: PApplication; r: TRect; Caption: l_text; Message: l_ulong ): PButton;
{$IFDEF LINK_CLIBS}external;{$endif}

procedure ButtonDraw ( b: PWidget; buffer: p_bitmap; w: PRect );
{$IFDEF LINK_CLIBS}external;{$endif}

function  ButtonEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
{$IFDEF LINK_CLIBS}external;{$endif}

procedure ButtonSetCaption ( b: PButton; Caption: l_text );
{$IFDEF LINK_CLIBS}external;{$endif}

procedure ButtonSetCaptionEx ( b: PButton; StrFormat: l_text; args: {$ifdef OPascal}array of const{$else}OpenArray{$endif});
{$IFDEF LINK_CLIBS}external;{$endif}

function ButtonSetRProp ( o: PWidget; Name: l_text; const Value; Redraw: l_bool ): l_bool;
{$IFDEF LINK_CLIBS}external;{$endif}

function ButtonGetRProp ( o: PWidget; Name: l_text ): Pointer;
{$IFDEF LINK_CLIBS}external;{$endif}

(* @defgroup buttonflags Buttons flags
	Values that can be set into TButton Flags field.
	Use 'OR' to associate those flags.
	@see CreateButton
	@see TButton
	@{
*)
const
 BF_TEXTUNDERICON =	$00000001; {  ///< If icon is set, draw the text under the icon and center both of them      }
 BF_DISABLED	  =	$00000002; {  ///< Disable the button, and if an icon is set, draw icon in grey              }
 BF_ALIGNLEFT	  =	$00000004; {  ///< If icon is set, draw icon on left and draw text on the right of the icon  }

implementation

{$IFNDEF LINK_CLIBS}
{$ifdef OPascal}
uses   Sysutils;
{$else}
uses   OSysutil;
{$endif}

(*
 ** @defgroup button Button widget
	@ingroup widgets
	@note Text is center by default.
	@{
*)

(*
 **	@file
	Button core file.
*)
const
  SkinButtonUp: PSkin		= NIL;
  SkinButtonDown: PSkin		= NIL;
  SkinButtonOver: PSkin		= NIL;
  SkinButtonDisabled: PSkin	= NIL;

var
  ButtonClass: PWidgetClass;

(*
 *	NAME: ButtonDraw
 *	DESCRIPTION: Default button draw function
*)
procedure ButtonDraw ( b: PWidget; buffer: p_bitmap; w: PRect );
var
  Font: PFont;
  FontColor: l_color;
  OffsetX  : l_int;
  OffsetY  : l_int;
  Shift    : l_int;
  Icon: p_bitmap;
  Face: l_color;
  Light: l_color;
  Dark: l_color;
  SkinItem: PSkin;
begin

	Font	  := default_font;
	FontColor := COL_3DTEXT;
	OffsetX	  := 5;
	OffsetY	  := 3;
	Shift	  := 0;
	
	Icon      := PButton(b)^.Icon;

   
	if (PButton(b)^.Flags and BF_DISABLED) = BF_DISABLED then
	begin
		if Assigned(Icon) then
			Icon := ImageNewGreyScaleWithMask (Icon);
	end;

	if ( UseSkins ) then
	begin
		SkinItem := SkinButtonUp;

		if ( PButton(b)^.State = BS_OVER ) then SkinItem := SkinButtonOver;
		if ( PButton(b)^.Flags and BF_DISABLED ) = BF_DISABLED then SkinItem := SkinButtonDisabled;
		if ( PButton(b)^.State = BS_DOWN ) then
		begin
			SkinItem := SkinButtonDown;
			Shift := 1;
		end;

		DrawSkin(buffer, SkinItem, b^.Absolute.a.x, b^.Absolute.a.y , b^.Absolute.b.x, b^.Absolute.b.y);

		FontColor 	:= SkinItem^.FontColor;
		Font		:= SkinItem^.Font;
		OffsetX		:= SkinItem^.Left;
		OffsetY		:= SkinItem^.Top;
	end
	else
	begin
		Face 	:= COL_3DFACE;
		Light	:= COL_3DLIGHT;
		Dark 	:= COL_3DDARK;

		if ( PButton(b)^.State = BS_OVER ) then Face := COL_3DOVER;
		if ( PButton(b)^.State = BS_DOWN ) then
		begin
			Light := COL_3DDARK;
			Dark  := COL_3DLIGHT;
			Shift := 1;
		end;

		al_rectfill(buffer, w^.a.x, w^.a.y, w^.b.x, w^.b.y, Face);
		Rect3D(buffer, b^.Absolute.a.x, b^.Absolute.a.y, b^.Absolute.b.x, b^.Absolute.b.y, Light, Dark);
	end;

	if ( PButton(b)^.Caption <> '' ) then
	begin
		if Assigned(Icon) then
		begin
			if ( b^.Flags and BF_TEXTUNDERICON ) = BF_TEXTUNDERICON then
			begin
                          al_draw_sprite(
                            buffer,
                            Icon,
                            (b^.Absolute.b.x+b^.Absolute.a.x) div 2 - (Icon^.w div 2) + Shift,
                            (b^.Absolute.a.y+b^.Absolute.b.y) div 2 + Shift - (Icon^.h + OffsetY + al_text_height(Font)) div 2
                          );
			  al_textout_centre_ex(
                            buffer,
                            Font,
                            PButton(b)^.Caption,
                            (b^.Absolute.b.x + b^.Absolute.a.x) div 2+Shift,
                            (b^.Absolute.a.y + b^.Absolute.b.y) div 2 + Shift -
                            (Icon^.h + OffsetY + al_text_height(Font)) div 2 +
                            Icon^.h + OffsetY, FontColor, -1
                            );
			end
			else if (b^.Flags and BF_ALIGNLEFT) = BF_ALIGNLEFT then
			begin
			  al_draw_sprite(
                            buffer, Icon,
                            b^.Absolute.a.x + OffsetX + Shift,
                            (b^.Absolute.b.y + b^.Absolute.a.y - Icon^.h) div 2 + Shift
                          );
                          al_textout_ex(
                            buffer, Font,
                            PButton(b)^.Caption,
                            b^.Absolute.a.x+ OffsetX + Shift + Icon^.w + OffsetX,
                            (b^.Absolute.a.y+b^.Absolute.b.y-al_text_height(Font)) div 2 + Shift, FontColor, -1
                          );
			end
			else
			begin
			  al_draw_sprite(buffer, Icon,
                            (b^.Absolute.a.x+b^.Absolute.b.x) div 2 +
                              Shift - (Icon^.w + OffsetX + al_text_length(Font, PButton(b)^.Caption)) div 2,
                            (b^.Absolute.b.y + b^.Absolute.a.y - Icon^.h) div 2 + Shift
                          );
			  al_textout_ex(buffer, Font,
                            PButton(b)^.Caption, (b^.Absolute.a.x+b^.Absolute.b.x) div 2 +
                            Shift - (Icon^.w + OffsetX + al_text_length(Font, PButton(b)^.Caption)) div 2 +
                            Icon^.w + OffsetX, (b^.Absolute.a.y+b^.Absolute.b.y-al_text_height(Font)) div 2 +
                            Shift, FontColor, -1
                          );
			end;
		end
		else
                  al_textout_centre_ex(
                    buffer,
                    Font,
                    PButton(b)^.Caption,
                    (b^.Absolute.b.x+b^.Absolute.a.x) div 2 +
                     Shift, (b^.Absolute.a.y+b^.Absolute.b.y-al_text_height(Font)) div 2 +
                     Shift, FontColor, -1
                    );
	end
	else if ( PButton(b)^.Caption = '' ) and Assigned( Icon ) then
	begin
          al_draw_sprite(buffer, Icon,
            (b^.Absolute.b.x+b^.Absolute.a.x-Icon^.w) div 2+Shift,
            (b^.Absolute.b.y+b^.Absolute.a.y-Icon^.h) div 2+Shift
          );
	end;
end;

(*
*	NAME: ButtonEventHandler
*	DESCRIPTION: Default event handler for button
*)
function  ButtonEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
begin
	if (Ev^.aType = EV_MOUSE) then
	begin
		if ((PBUTTON(o)^.Flags and BF_DISABLED)=BF_DISABLED) then
			ButtonEventHandler := true;

		if (Ev^.Message = WEvMouseEnter) then
		begin
			PBUTTON(o)^.State := BS_OVER;
			WidgetDraw(o, NIL);
			ButtonEventHandler := true;
		end;

		if (Ev^.Message = WEvMouseLeft) then
		begin
			PBUTTON(o)^.State := 0;
			WidgetDraw(o, NIL);
			ButtonEventHandler := true;
		end;

		if (Ev^.Message = WEvMouseLDown) then
		begin
			PBUTTON(o)^.State := BS_DOWN;
			WidgetDraw(o, NIL);
			ButtonEventHandler := true;
		end;

		if ((Ev^.Message = WEvMouseLUp) and (PBUTTON(o)^.State = BS_DOWN)) then
		begin
			PBUTTON(o)^.State := BS_OVER;
			WidgetDraw(o, NIL);
			if ( PBUTTON(o)^.Message <> 0 ) then
				WidgetSendSEvent(o, EV_MESSAGE, PBUTTON(o)^.Message, PBUTTON(o)^.MessageExtra);
			ButtonEventHandler := true;
		end;
	end
    else
	ButtonEventHandler := false;
end;

procedure ButtonFreeEx ( o: PWidget );
begin
	if (PBUTTON(o)^.Caption <> '') then
		PBUTTON(o)^.Caption := '';
end;

(**
	Change the caption of a button
	@param b Button to affect
	@param Caption New caption
	@note Redraw the button
	@see CreateButton
*)
procedure ButtonSetCaption ( b: PButton; Caption: l_text );
begin
	if ( b^.Caption <> '' ) then b^.Caption := '';
	b^.Caption := Caption;
	WidgetDraw(PWIDGET(b), NIL);
end;

procedure ButtonSetCaptionEx ( b: PButton; StrFormat: l_text; args: {$ifdef OPascal}array of const{$else}OpenArray{$endif} );
var CaptionText: {$ifdef OPascal}ShortString{$else}String{$endif};
begin
  {$ifdef OPascal}
  CaptionText := Format(StrFormat, args);
  {$else}
  FormatStr(CaptionText, StrFormat, args);
  {$endif}
  ButtonSetCaption(b,CaptionText);
  WidgetDraw(PWIDGET(b),NIL);
end;

(** 
	Creates a new button
	@param App Application which want to allocate the widget( use &Me in most cases)
	@param r Relative coordinates of new widget
	@param Caption Caption of button
	@param Message Message send when button is clicked
	@see WidgetInsert
	@see buttonflags
*)
function CreateButton ( App: PApplication; r: TRect; Caption:  l_text; Message: l_ulong ): PButton;
var b: PButton;
begin
	New(b);

	if not assigned(b) then CreateButton := NIL else
    begin
	fillchar(b, sizeof(TButton), 0);

	if ( Caption <> '' ) then
		b^.Caption := Caption;

	b^.Message := Message;

	IntialiseWidget(App, ButtonClass, PWIDGET(b), r );

	PWIDGET(b)^.Draw := {$ifdef fpc}@{$endif}ButtonDraw;
	PWIDGET(b)^.EventHandler := {$ifdef fpc}@{$endif}ButtonEventHandler;
	PWIDGET(b)^.FreeEx := {$ifdef fpc}@{$endif}ButtonFreeEx;

	CreateButton := b;
	end;
end;

function  StdCreateButton ( App: PApplication; r: TRect ): PWidget;
begin
	StdCreateButton := PWIDGET(CreateButton(App,r,'',0));
end;

function ButtonSetRProp ( o: PWidget; Name: l_text; const Value; Redraw: l_bool ): l_bool;
begin
	if ( TextCompare(Name,'caption') = 0) then
	begin
		if ( PBUTTON(o)^.Caption <> '' ) then PBUTTON(o)^.Caption := '';
		PBUTTON(o)^.Caption := l_text(Value);
		if ( Redraw ) then WidgetDraw(PWIDGET(o), NIL);
		ButtonSetRProp := true;
	end else
	if ( TextCompare(Name,'message') = 0) then
	begin
		PBUTTON(o)^.Message := l_ulong(Value);
		ButtonSetRProp := true;
	end else
	ButtonSetRProp := false;
end;

function ButtonGetRProp ( o: PWidget; Name: l_text ): Pointer;
begin
	if ( TextCompare(Name,'caption') = 0 ) then ButtonGetRProp := PString(PBUTTON(o)^.Caption) else
	if ( TextCompare(Name,'message') = 0 ) then ButtonGetRProp := PInteger(PBUTTON(o)^.Message) else
    ButtonGetRProp := NIL;	
end;

(**
@}
*)

function LibMain ( argc: integer; argv: l_text ): l_bool;
begin
	ButtonClass := CreateWidgetClass( ULONG_ID('B','t','n',' '), StdWidgetClass, 0 );
	ButtonClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateButton;
	ButtonClass^.SetRProp := {$ifdef fpc}@{$endif}ButtonSetRProp;
	ButtonClass^.GetRProp := {$ifdef fpc}@{$endif}ButtonGetRProp;
	
	WClassRegisterProp(ButtonClass,PTYPE_STRING,'caption');
	WClassRegisterProp(ButtonClass,PTYPE_MESSAGE,'message');

	
	SkinButtonUp		:= GetSkinItem('Button');
	SkinButtonDown		:= GetSkinItem('ButtonClick');
	SkinButtonOver		:= GetSkinItem('ButtonOver');
	SkinButtonDisabled	:= GetSkinItem('ButtonDisabled');
    (*
	APPEXPORT(CreateButton);
	APPEXPORT(ButtonSetCaption);
	APPEXPORT(ButtonSetCaptionEx);
	*)
	LibMain := true;
end;

procedure Close;
begin

end;

{$ifdef fpc}
initialization
{$else}
begin
{$endif}

  AppVersion := ULONG_ID(0,10,0,0);

{$ENDIF}
end.
