(*
////////////////////////////////////////////////////////////////////////////////
//
//  Combobox - Core File
//
//  (c) Copyright 2003 Point Mad. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
 .#include 'kernel.h'
 .#include 'combobox.h'
 .#include 'grfx.h'
*)
unit combobox;

interface

uses
  guitypes,dynld,list,events,grfx,skin,font,metrics,listview,widget,scrllbar,
  allegro;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$link grfx.o}
{$endif}

type 
  PCombobox = ^TCombobox;
  TCombobox = record

	o       :  TWidget;

	Items   : PList;
	
	Selected: PListviewItem;
	
	State   : l_uchar;
	OnUpdate: l_ulong;
	
  end;

function NewComboBox ( App: PApplication; r: TRect; OnUpdate: l_ulong ): PCombobox;
procedure ComboboxAddItem ( o: PCombobox; Caption: l_text; Icon: PIcon );

function ComboboxItemIndex 	( o: PCombobox; a: PListviewItem ): l_ulong;
procedure ComboboxSelectIndex ( o: PCombobox; Index: l_ulong );
procedure ComboboxSelectItem 	( o: PCombobox; p: PListviewItem );

implementation
{$ifndef LINK_CLIBS}

var
  AppVersion: l_ulong;
  SkinCombobox: PSkin;
  ComboboxClass: PWidgetClass;
  
const
  AppName: l_text    = 'Combobox widget';
  nUID: l_text       = 'combobox';
  NeededLibs: array[0..4] of l_text = ( 'widget','grfx','skin','listview','' );


procedure ComboboxDraw ( o: PWidget; buffer: p_bitmap; w: PRect );
begin
	if ( UseSkins ) then
	begin
		DrawSkin(buffer, SkinCombobox, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y);

		al_hline(buffer,o^.Absolute.b.x-SkinCombobox^.Right-4,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-2,o^.Absolute.b.x-SkinCombobox^.Right-11,COL_3DTEXT);
		al_hline(buffer,o^.Absolute.b.x-SkinCombobox^.Right-5,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-1,o^.Absolute.b.x-SkinCombobox^.Right-10,COL_3DTEXT);
		al_hline(buffer,o^.Absolute.b.x-SkinCombobox^.Right-6,(o^.Absolute.a.y+o^.Absolute.b.y) div 2,o^.Absolute.b.x-SkinCombobox^.Right-9,COL_3DTEXT);
		al_hline(buffer,o^.Absolute.b.x-SkinCombobox^.Right-7,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+1,o^.Absolute.b.x-SkinCombobox^.Right-8,COL_3DTEXT);
		al_hline(buffer,o^.Absolute.b.x-SkinCombobox^.Right-8,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+2,o^.Absolute.b.x-SkinCombobox^.Right-7,COL_3DTEXT);

		if assigned( PCOMBOBOX(o)^.Selected ) then
			DrawNiceText (buffer,
                        SkinCombobox^.Font,
                        o^.Absolute.a.x+SkinCombobox^.Left,
                        (o^.Absolute.a.y+o^.Absolute.b.y-al_text_height(SkinCombobox^.Font))div 2,
                        o^.Absolute.b.x-o^.Absolute.a.x-30,PCOMBOBOX(o)^.Selected^.Caption,
                        SkinCombobox^.FontColor
                        );
			//textout(buffer, SkinCombobox^.Font, COMBOBOX(o)^.Selected^.Caption,o^.Absolute.a.x+SkinCombobox^.Left, (o^.Absolute.a.y+o^.Absolute.b.y-text_height(SkinCombobox^.Font))/2, SkinCombobox^.FontColor);
	end
	else
	begin
		al_rectfill(buffer,w^.a.x,w^.a.y,w^.b.x,w^.b.y,COL_3DFACE);

		if ( PCOMBOBOX(o)^.State > 0 ) then
			Rect3D(buffer,o^.Absolute.b.x-25,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.b.y,COL_3DDARK,COL_3DLIGHT)
		else
			Rect3D(buffer,o^.Absolute.b.x-25,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.b.y,COL_3DLIGHT,COL_3DDARK);

		al_hline(buffer,o^.Absolute.b.x-5-4,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-2,o^.Absolute.b.x-5-11,COL_3DTEXT);
		al_hline(buffer,o^.Absolute.b.x-5-5,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-1,o^.Absolute.b.x-5-10,COL_3DTEXT);
		al_hline(buffer,o^.Absolute.b.x-5-6,(o^.Absolute.a.y+o^.Absolute.b.y) div 2,o^.Absolute.b.x-5-9,COL_3DTEXT);
		al_hline(buffer,o^.Absolute.b.x-5-7,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+1,o^.Absolute.b.x-5-8,COL_3DTEXT);
		al_hline(buffer,o^.Absolute.b.x-5-8,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+2,o^.Absolute.b.x-5-7,COL_3DTEXT);

		al_rectfill(buffer,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.b.x-35,o^.Absolute.b.y, COL_BOXFACE);
		Rect3D(buffer,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.b.x-35,o^.Absolute.b.y, COL_BOXDARK,COL_BOXLIGHT);

		if assigned( PCOMBOBOX(o)^.Selected ) then
			DrawNiceText (buffer,
                           default_font,
                           o^.Absolute.a.x+5,
                           (o^.Absolute.a.y+o^.Absolute.b.y-al_text_height(default_font)) div 2,
                           o^.Absolute.b.x-o^.Absolute.a.x-35-5,PCOMBOBOX(o)^.Selected^.Caption,
                           COL_3DTEXT
                        );
		{	//textout(buffer, default_font, PCOMBOBOX(o)^.Selected^.Caption,o^.Absolute.a.x+8,(o^.Absolute.a.y+o^.Absolute.b.y-text_height(default_font)) div 2, COL_3DTEXT); }
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function ComboboxEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
var
  r: TRect;
  l: PListView;
  msg: l_ulong;
begin
	if (Ev^.aType = EV_MOUSE) then
	begin
		if ( Ev^.Message = WEvMouseLDown ) then
		begin
			PCOMBOBOX(o)^.State := 1;

			WidgetDraw(o,NIL);

			RectAssign(r,
                          o^.Absolute.a.x,o^.Absolute.b.y+1,
                          o^.Absolute.b.x,o^.Absolute.b.y+min(100,ListviewEstimateHeight(PCOMBOBOX(o)^.Items)));

			l := CreateListview(o^.AppOwner,r,'',LVS_LIST,LVF_DRAWICONS);

			FreeList(l^.Items);

			l^.Items := PCOMBOBOX(o)^.Items;
			l^.TopItem := PCOMBOBOX(o)^.Selected;
			l^.OnValMsg := 1;

			InsertWidget(DeskTop,PWidget(l));

			WidgetDraw(PWidget(l), NIL);
			WidgetDraw(PWidget(SLLWDG(l)^.Vertical), NIL);

			ListviewSelectItem(l, GetSelectedItem(l), true, false);

			WidgetSetFocus(PWidget(l));
            msg := WidgetExecute(PWidget(l));
			if ( msg = 1 ) then
				PCOMBOBOX(o)^.Selected := GetSelectedItem(l);

			l^.Items := NewList();

			WidgetDispose(PWidget(SLLWDG(l)^.Vertical));
			SLLWDG(l)^.Vertical := NIL;
			WidgetDispose(PWidget(l));

			PCOMBOBOX(o)^.State := 0;

			WidgetDraw(o,NIL);

			if ( PCOMBOBOX(o)^.OnUpdate > 0 )
                                then WidgetSendSEvent(o, EV_MESSAGE, PCOMBOBOX(o)^.OnUpdate, NIL);

			ComboboxEventHandler := true;
		end;

	end else ComboboxEventHandler := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ComboboxSelectItem ( o: PCombobox; p: PListviewItem );
var a,b: PListItem;
begin
    a := o^.Items^.Last; b := a;
	if not assigned( a ) then exit;
	repeat
		if (PLISTVIEWITEM(a^.Data)^.Flags and LVI_SELECTED ) = LVI_SELECTED then
		begin
			PLISTVIEWITEM(a^.Data)^.Flags := PLISTVIEWITEM(a^.Data)^.Flags and not LVI_SELECTED;
		end;
		a := a^.Prev;
	until ( a = b );

	if assigned( p ) then
		p^.Flags := p^.Flags or LVI_SELECTED;
	

	o^.Selected	:= p;
	WidgetDraw(PWidget(o),NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ComboboxSelectIndex ( o: PCombobox; Index: l_ulong );
var a: PListviewItem;
begin
	if ( Index >= 0 ) then
	begin
		a := ListAt(o^.Items,Index);
		if assigned( a ) then ComboboxSelectItem(o,a);
	end else
		ComboboxSelectItem(o,NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function ComboboxItemIndex ( o: PCombobox; a: PListviewItem ): l_ulong;
begin
	ComboBoxItemIndex := ListFoundIndex(o^.Items,a);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ComboboxAddItem ( o: PCombobox; Caption: l_text; Icon: PIcon );
var
  p: PListviewItem;
begin
    New(p);
	if not assigned(p) then exit else
	begin
	fillchar(p^, sizeof(TListviewItem), 0);
	ListviewInitItem(PLISTVIEWITEM(p),Caption,Icon);
	ListAdd (o^.Items, Caption, p, {$ifdef fpc}@{$endif}FreeListviewItem);
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ComboBoxFree ( o: PWidget );
begin
	//FreeList(PCOMBOBOX(o)^.Items);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function NewComboBox ( App: PApplication; r: TRect; OnUpdate: l_ulong ): PCombobox;
var 
  o: PComboBox;
begin
	NEW(o);
	fillchar(o,sizeof(TCombobox),0);

	IntialiseWidget(App,ComboboxClass,PWidget(o),r);

	o^.Items := NewList;

	PWidget(o)^.Draw         := {$ifdef fpc}@{$endif}ComboboxDraw;
	PWidget(o)^.EventHandler := {$ifdef fpc}@{$endif}ComboboxEventHandler;
	PWidget(o)^.FreeEx := {$ifdef fpc}@{$endif}ComboBoxFree;
	o^.OnUpdate := OnUpdate;
	NewComboBox := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function  StdCreateCombobox ( App:PApplication; r: TRect ): PWidget;
begin
	StdCreateComboBox := PWidget(NewComboBox( App, r, 0 ));
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function LibMain ( argc: integer; argv: l_text ): l_bool;
var
  r: TRect; cbx: PComboBox;
begin
	ComboboxClass := CreateWidgetClass( ULONG_ID('C','m','b','B'), StdWidgetClass, 0 );
	ComboboxClass^.StdCreate := {(void*)}{$ifdef fpc}@{$endif}StdCreateCombobox;

	
	
	(*TRect r := { 100,100,300,120 };
	PCombobox cbx := NewComboBox(@Me,r);

	ComboboxAddItem(cbx,'Item a',NIL);
	ComboboxAddItem(cbx,'Item b',NIL);
	ComboboxAddItem(cbx,'Item c',NIL);
	ComboboxAddItem(cbx,'Item d',NIL);
	ComboboxAddItem(cbx,'Item e',NIL);
	ComboboxAddItem(cbx,'Item f',NIL);

	InsertWidget(DeskTop,PWidget(cbx));

	WidgetDraw(PWidget(cbx),NIL);*)

	SkinCombobox := GetSkinItem('Combobox');
        (*
	APPEXPORT(NewComboBox);
	APPEXPORT(ComboboxAddItem);
	APPEXPORT(ComboboxItemIndex);
	APPEXPORT(ComboboxSelectIndex);
	APPEXPORT(ComboboxSelectItem);
        *)
	LibMain := true;
end;

procedure Close;
begin

end;


{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion := ULONG_ID(0,0,0,1);
  SkinCombobox := NIL;

{$endif}  {LINK_CLIBS}
end.
