(*
////////////////////////////////////////////////////////////////////////////////
//
//	Console (Library)
//
//	Copyright (c) 2004 Point Mad. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
unit conlib;

interface

uses 
  guitypes, debug, widget, ulabel, console;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
  {$link window.o}
  {$link label.o}
  {$link console.o}
{$endif}
{
 .#include"kernel.h"
 .#include"window.h"
 .#include"label.h"
 .#include"internal.h"
 .#include"console.h"
 
 .#include "allegro/internal/aintern.h"
}

{ #define CONWDG(o) ((PConWdg)(o)) }

type
  PConsole = ^TConsole;
  
  TConWdg = ^TConWdg;
  TConWdg = record;

	w: TWidget;
	
	o: PConsole;
  end;
  
  PVideoBuf = ^TVideoBuf;
  TVideoBuf = array[0..1920] of byte;
  TVideoMem = array[0..23][0..79] of byte;
  
  TConsole = record

	Color : l_uchar;
	flags : l_long;
	cursor: l_long;
	w: PWindow;
	box: PConWdg;
	t: PTimer;
	t1: PTimer;
	VideoMem: TVideoMem;
	VideoMem1: TVideoMem;
	cur : Longint;
	cur1: Longint;
	
	KeyBuffer: array[0..255] of Integer;
	KeyNb: l_long;
	
	App: PApplication; { An application can only open one console. }
	X,Y: Integer;
  end;

  PScreenMemory32 = ^TScreenMemory32;
  TScreenMemory32 = array of Longword;
  
  PScreenMemory16 = ^TScreenMemory16;
  TScreenMemory16 = array of word;
  
  PScreenMemory8  = ^TScreenMemory8;
  PScreenMemory8  = array of byte;
  
  PScreenMemory   = ^TScreenMemory;
  TScreenMemory   = array of byte;

  PScreen = ^TScreen;
  TScreen = record
     buffer: PScreenMemory;
     clip: Integer;
     cl,
     cr,
     ct,
     cb  : Integer;
  end;

{ #define CONSOLE(o) ((PConsole)(o)) }

function GetKey ( App: PApplication ): integer;
function Printf(App: PApplication; StrFormat: l_text; args: {$ifdef OPascal}array of const{$else}PDynamicArray{$endif} ): integer;


(*
l_text txt =  Prompt ( &Me ) ;

 It will prompt user for a string allowing right and left moves, and use of del and backspace keys.
 When user press [enter] it will return string (no '\n' at end).

*)
function Prompt ( App: PApplication ): l_text;

var
  AppVersion: l_ulong;

const
  AppName: String   = 'Console Libary';
  nUID: l_uid	    = ('conlib');
  NeededLibs: l_uid = ( 'window','widget','label','' );

var
  Palette: array[0..15] of l_color;

(*////////////////////////////////////////////////////////////////////////////////*)
implementation
(*////////////////////////////////////////////////////////////////////////////////*)
{$ifndef LINK_CLIBS}

function AppEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
begin
	if ( Ev^.Type = EV_MESSAGE ) then
	begin
		if ( Ev^.Message = WM_CLOSE ) then
		begin
			//CloseApp(o^.AppOwner);
			BreakApp(o^.AppOwner);

			AppEventHandler := true;
		end;
	end;
	AppEventHandler := false;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
procedure TimerPoll(var A);
begin
	WidgetDraw(PWIDGET(PCONSOLE(A)^.box), NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TimerPoll1(var A);
begin
	PCONSOLE(A)^.flags := PCONSOLE(A)^.flags xor 1;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ConsoleDraw ( o: PWidget; buffer: p_bitmap; w: PRect );
var
	f: PFont;        { FONT *f = font; }
	Cons: PConsole;  { Cons := PCONWDG(o)^.o; }
	Col,c: byte;      { l_text c := *Cons^.VideoMem; l_text col := *Cons^.VideoMem1; }
	sx,sy,x,y: l_long;
begin
    f := font;
    Cons := PCONWDG(o)^.o;
    c := Cons^.VideoMem[0,0];     
    col := Cons^.VideoMem1[0,0];
    sx := o^.Absolute.a.x;
    sy := o^.Absolute.a.y;
    x := 0;
    y := 0;
	while ( y < 24 ) do
	begin
		sx := o^.Absolute.a.x;
		x  := 0;
		while ( x < 80 ) do
        begin
			rectfill(buffer, sx, sy, sx+8, sy+8, Palette[ ({*}col[x,y] shl 4 ) and $07 ]);
			f^.vtable^.render_char( f,{*}Cons^.VideoMem[x,y],Palette[{*}col[x,y] and $0F],-1,buffer,sx,sy);
			if ( (c[x,y] = Cons^.cur) and ((Cons^.flags and 1)=1) and (Cons^.cursor<>0) ) then
			begin
	   			xor_mode(TRUE);
	   			if ( Cons^.cursor = 1 ) then
	    			rectfill(buffer, sx, sy+6, sx+8, sy+8, Palette[Cons^.Color and $0F] )
	    		else if ( Cons^.cursor = 2 ) then
	    			rectfill(buffer, sx, sy+4, sx+8, sy+8, Palette[Cons^.Color and $0F] )
	    		else                                                              
                    rectfill(buffer, sx, sy, sx+8, sy+8, Palette[Cons^.Color and $0F] );
		  		xor_mode(FALSE);
			end;
			{inc(c);}
			{inc(col);}
			sx := sx + 8;
			inc(x);
		end;
		sy := sy + 8;
		inc(y);
	end;
end;
(*////////////////////////////// //////////////////////////////////////////////////*)
function ConsoleEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
var Cons: PConsole;
begin

	if ( Ev^.Type = EV_KEYBOARD ) then
	begin
		Cons := PCONWDG(o)^.o;

		Cons^.KeyBuffer[Cons^.KeyNb] := Ev^.Keyb.Key;

		if (SCANCODE(Ev, KEY_ENTER)) then
			Cons^.KeyBuffer[Cons^.KeyNb] := ((Cons^.KeyBuffer[Cons^.KeyNb]) and not $FF ) or ord(#10);

		inc(Cons^.KeyNb);

		ConsoleEventHandler := true;
	end;

	ConsoleEventHandler := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ClearConsole ( Cons: PConsole );
begin
	fillchar(Cons^.VideoMem , sizeof(TVideoMem), ' ');
	fillchar(Cons^.VideoMem1, sizeof(TVideoMem), ' ');
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure SetPos ( Cons: PConsole; x, y: l_long );
begin
	x := min(max(x,0),80);
	y := min(max(y,0),24);
	Cons^.cur  := Longint(Addr(Cons^.VideoMem))  {[x,y]} +x+(y*80); 
	Cons^.cur1 := Longint(Addr(Cons^.VideoMem1)) {[x,y]} +x+(y*80); 
	Cons.X := x;
	Cons.Y := y;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure CursorMoveRight ( o: PConsole );
var x,y: Integer;

	inc(o^.cur);
	inc(o^.cur1);

   { if (x >= 79) and (y >= 23) then }
	if ( o^.cur >= Longint(Addr(o^.VideoMem))+(80*24) ) then
	begin
		o^.cur  := o^.cur  - 80;
		o^.cur1 := o^.cur1 - 80;
		move( Pointer(Longint(Addr(o^.VideoMem))+80)^, Addr(o^.VideoMem)^,80*23);
		move( Pointer(Longint(Addr(o^.VideoMem1))+80)^, Addr(o^.VideoMem1)^, 80*23);
		fillchar(Pointer(Longint(Addr(o^.VideoMem))  + (80*23))^, 80, ' ');
		fillchar(Pointer(Longint(Addr(o^.VideoMem1)) + (80*23))^, 80, o^.Color);
	end;
    Cons.X := o^.cur mod 80; 
    Cons.Y := o^.cur div 80;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure CursorMoveLeft ( o: PConsole );
begin
	if o^.cur = Longint(Addr(o^.VideoMem)) then exit;

	dec(o^.cur);
	dec(o^.cur1);

    Cons.X := o^.cur mod 80; 
    Cons.Y := o^.cur div 80;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure PutChar ( o: PConsole; c: l_char );

	if ( c = chr(10) ) then
	begin
		o^.cur := o^.cur + 80-((o^.cur-Longint(Addr(o^.VideoMem))) mod 80);
		o^.cur1 := 80-((o^.cur1-Longint(Addr(o^.VideoMem1))) mod 80);

	end else begin
       {
		*o->cur = c;
		*o->cur1 = o->Color;
		o->cur++;
		o->cur1++;
	   }
	   fillchar(o^.VideoMem, 80*24, c);
	   fillchar(o^.Videomem1,80*24, o^.Color);
 	end;

	if ( o^.cur >= Longint(Addr(o^.VideoMem))+(80*24) ) then
	begin
		o^.cur := o^.cur - 80;
		o^.cur1 := o^.cur1 - 80;
		move(Pointer(Longint(Addr(o^.VideoMem))+80)^,Addr(o^.VideoMem)^,80*23);
		move(Pointer(Longint(Addr(o^.VideoMem1))+80)^,Addr(o^.VideoMem1)^,80*23);

		fillchar(Pointer(Longint(Addr(o^.VideoMem))+(80*23))^,80,' ');
		fillchar(Pointer(Longint(Addr(o^.VideoMem1))+(80*23))^,80,o^.Color);

	end;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function NewConsole ( App: PApplication ): PConsole;
var
  r: TRect;
  Cons: PConsole;
begin
	if ( App->Cons ) then NewConsole := App^.Cons;
	if ( App->Host ) then NewConsole := NewConsole(App^.Host);

	New(Cons);
	fillchar(Cons,sizeof(TConsole),0);
	Cons^.cursor := 1;
	Cons^.Color := $07;
	ClearConsole(Cons);
	RectAssign(r,0, 0, 80*8, 24*8);
	Cons^.w := CreateWindow(App, r, App^.Name, WF_FRAME or WF_CAPTION or WF_MINIMIZE or WF_CENTERED);
	PWIDGET(Cons^.w)^.AppEvHdl := {ifdef fpc}@{$endif}AppEventHandler;
	InsertWidget(PWIDGET(DeskTop), PWIDGET(Cons^.w));
	RectAssign(r,0, 0, 80*8, 24*8);
	New(Cons^.box);
	fillchar(Cons^.box,sizeof(TConWdg),0);
	IntialiseWidget(App,StdWidgetClass,PWIDGET(Cons^.box),r);
	PWIDGET(Cons^.box)^.Draw := {ifdef fpc}@{$endif}ConsoleDraw;
	PWIDGET(Cons^.box)^.EventHandler := {ifdef fpc}@{$endif}ConsoleEventHandler;
	PWIDGET(Cons^.box)^.Flags := PWIDGET(Cons^.box)^.Flags or WFForceBuffer or WFFocusable;
	Cons^.box^.o := Cons;
	InsertWidget(PWIDGET(Cons^.w), WIDGET(Cons^.box));
	WidgetDrawAll(PWIDGET(Cons^.w));
	Cons^.t := NewTimer(App,20, {ifdef fpc}@{$endif}TimerPoll,Cons);
	Cons^.t1 := NewTimer(App,500, {ifdef fpc}@{$endif}TimerPoll1,Cons);
	Cons^.cur := Longint(Addr(Cons^.VideoMem));
	Cons^.cur1 := Longint(Addr(Cons^.VideoMem1));
    Cons.X := 0;
    Cons.Y := 0;

	NewConsole := Cons;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function ConsoleStdOutFileWrite ( f: PFileStruct; var d; s: l_ulong ): integer;
var
  c: {$ifdef OPascal}String absolute d{$else}TVideoBuf absolute d{$endif};
  v: TVideoBuf;
  p: l_ulong;
  i: integer;
begin
    i := 0; p := Longint(Addr(s));
	if not assigned( PAPPLICATION(f^.Ex1)^.Cons ) then
	begin
		PAPPLICATION(f^.Ex1)^.Cons = NewConsole(PAPPLICATION(f^.Ex1));
		if not assigned( PAPPLICATION(f^.Ex1)^.Cons ) then ConsoleStdOutFileWrite := 0;
	end;
    inc(i);
    PutChar(PAPPLICATION(f^.Ex1)^.Cons, ' ');
	while ( PAPPLICATION(f^.Ex1)^.Cons^.Y <= 24 ) do
	begin
	    move(c{ifdef OPascal}[1]{$endif},v,min(s,1920));
	    move(v,PAPPLICATION(f^.Ex1)^.Cons^.VideoMem,min(s,1920)); 
	    {	PutChar(PAPPLICATION(f^.Ex1)^.Cons, c[i]); }
                inc(i);
	end;
	ConsoleStdOutFileWrite := s;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetKey ( App: PApplication ): integer;
var
  Cons:	PConsole; 
  v: integer;
begin
    Cons := App^.Cons;
	if not assigned( Cons ) then GetKey := 0 else
	begin
    	while ( Cons^.KeyNb = 0 ) do SysPoll; 
	    v := Cons^.KeyBuffer[0];
    	if ((v and $FF) = $0A) then Dec(Cons^.KeyNb);
    	if ((v and $FF) = $0D) then Dec(Cons^.KeyNb);
	    if ( Cons^.KeyNb > 0 ) then
    	begin
		  { memcpy(Cons->KeyBuffer,(void*)((l_ulong)Cons->KeyBuffer)+sizeof(int),Cons^.KeyNb*sizeof(int)); }
	      v := Cons^.KeyBuffer[Cons^.KeyNb];
        end;
	    GetKey := v;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function ConsoleStdInFileRead ( f: PFileStruct; var d; s: l_ulong ): integer;
var
  v: {$ifdef OPascal}String absolute d{$else}TVideoBuf absolute d{$endif};
  App : PApplication;
  Cons: PConsole;
  n,i : l_long;
begin
    n := 0;
    App := APPLICATION(f^.Ex1);
    Cons := PAPPLICATION(f^.Ex1)^.Cons;
	if not assigned( Cons ) then ConsoleStdInFileRead := 0;
	s := min(s,Cons^.KeyNb);
	{$ifdef OPascal}Inc(n){$endif};
	while ( n < s ) do begin
	   v[n] := Lo(Cons^.KeyBuffer[n]) and $FF;
	end;
	if ( Cons^.KeyNb - s ) > 0 then
	begin
		memcpy(Cons->KeyBuffer,(void*)((l_ulong)Cons->KeyBuffer)+(s*sizeof(int)),(Cons->KeyNb-s)*sizeof(int));
	end;
	Cons^.KeyNb := Cons^.KeyNb - s;
    i := 0; 
    {$ifdef OPascal}
    inc(i);
    SetLength(v, s);
    {$endif}
	while ( s > 0 ) do
	begin
		v[i] := GetKey(App) and $FF;
		{v[i] := GetKey(App) and $FF;}
		dec(s);
        inc(i);
	end;
	ConsoleStdInFileRead := s;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function Printf(App: PApplication; StrFormat: l_text; args: {$ifdef OPascal}array of const{$else}array of String{$endif} ): integer;
var
  i: integer;
  b: TVideoBuf;
  s: String;
begin
    i := 0;
	if not assigned( App^.Cons ) then
	begin
		App^.Cons := NewConsole(App);
		if not assigned( App^.Cons ) then Printf := 0;
	end;

    fillchar(App^.Cons^.VideoMem, SizeOf(TVideoMem), ' ');
    fillchar(App^.Cons^.VideoMem1, SizeOf(TVideoMem), App^.Cons^.Color);
    {$ifdef OPascal}
    s := Format(StrFormat, args);
    {$else}
    FormatStr(s, StrFormat, args);
	while i < Length(s) do
	begin
		{PutChar(App^.Cons,((l_text)t)[i]);}
		b[i] := s[i+1];
        inc(i);
    end;
    move(b, App^.Cons^.VideoMem, i);
	Printf := i;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function Prompt ( App: PApplication ): l_text;
var 
 t:	l_text;
 p:	l_text;
 c:	integer;
begin
    t := ''; 
    p := '';
	if not assigned( App^.Cons ) then
	begin
		App^.Cons := NewConsole(App);
		if not assigned( App^.Cons ) then Prompt := 0;
	end
    else
	while ( ((c := GetKey(App)) and $FF) <> 10 ) do
	begin

		case ( c shr 8 ) of

			KEY_LEFT : begin
				if ( p != t ) {
					CursorMoveLeft(App^.Cons);
					p--;
				}
			end;

			case KEY_RIGHT : {
				if ( p != t+TextLen(t) ) {
					CursorMoveRight(App^.Cons);
					p++;
				}
			} break;

			case KEY_BACKSPACE : {

				if ( p != t ) {

					l_text tmp = TextDup(p);
					l_text ex = realloc(t,TextLen(t));
					TextCopy(ex+(p-t-1),tmp);
					free(tmp);
					p = ex+(p-t-1);
					t = ex;

					if ( p != t+TextLen(t) ) {
						l_text a = p, b = p;
						CursorMoveLeft(App->Cons);
						while ( *a ) {
							PutChar(App->Cons,*a);
							a++;
						}
						PutChar(App->Cons,' ');
						while ( *(b++) ) CursorMoveLeft(App->Cons);
						CursorMoveLeft(App->Cons);
					} else {
						CursorMoveLeft(App->Cons);
						PutChar(App->Cons,' ');
						CursorMoveLeft(App->Cons);
					}

				}
			} break;

			case KEY_DEL : {

				if ( p != t ) {
					l_text tmp = TextDup(p+1);
					l_text ex = realloc(t,TextLen(t));
					TextCopy(ex+(p-t),tmp);
					free(tmp);
					p = ex+(p-t);
					t = ex;
					if ( p != t+TextLen(t) ) {
						l_text a = p, b = p;
						while ( *a ) {
							PutChar(App->Cons,*a);
							a++;
						}
						PutChar(App^.Cons,' ');
						while ( *(b++) ) CursorMoveLeft(App->Cons);
						CursorMoveLeft(App->Cons);
					} else {
						PutChar(App->Cons,' ');
						CursorMoveLeft(App->Cons);
					}
				}
			} break;

			else 
             if ( c and $FF ) then
             begin
				l_text tmp = TextDup(p);
				l_text ex = realloc(t,TextLen(t)+2);
				TextCopy(ex+(p-t+1),tmp);
				free(tmp);
				PutChar(App->Cons,c & 0xFF);
				*(ex+(p-t)) = c & 0xFF;
				p = ex+(p-t+1);
				t = ex;
				if ( p != t+TextLen(t) ) {
					l_text a = p, b = p;
					while ( *a ) {
						PutChar(App->Cons,*a);
						a++;
					}
					while ( *(b++) ) CursorMoveLeft(App->Cons);
				} else
					*p = 0;


			end;


		end; { case }
	end; { while }

	Prompt := t;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure _CloseConsole ( Cons: PConsole );
begin
	DebugMessage('_CloseConsole(%x)',[Cons]);
	KillTimer(Cons^.t);
	KillTimer(Cons^.t1);
	WidgetDispose(PWIDGET(Cons^.w));
	Dispose(Cons);
	DebugMessage('_CloseConsole return;',[]);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure CloseConsole ( App: PApplication );
begin
	if ( App^.Cons ) then _CloseConsole(App^.Cons);
	App^.Cons := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
const
  ConsoleStdOut: TFileDriver = ( 
    DriverName: 'Console stdout',
	FileOpen: NIL,
    FileClose: NIL,
    FileRead: NIL,
	FileWrite: {$ifdef fpc}@{$endif}ConsoleStdOutFileWrite,
	FileGetPos: NIL,
    FileSeek: NIL,
    FileEOF: NIL,
    FileDelete: NIL,
    FileRename: NIL,
    FileGetInfo: NIL,
    FileFlush: NIL,
    DirMake: NIL,
    DirDelete: NIL,
    DirRename: NIL,
    List: NIL,
    RefreshDriveInfo: NIL,
    FileSetAttributes: NIL 
 );
(*////////////////////////////////////////////////////////////////////////////////*)
  ConsoleStrOutDrv: TDrive = (
	Name: 'Console stdout',
	TotalSpace: 0,
    UsedSpace: 0,
    dType: 0,
    Ex1: NIL,
    Ex2: NIL,
	Driver: {$ifdef fpc}@{$endif}ConsoleStdOut
);
(*////////////////////////////////////////////////////////////////////////////////*)
  ConsoleStdIn: TFileDriver = ( 
    DriverName: 'Console stdin',
	FileOpen: NIL,
    FileClose: NIL,
	FileRead: {$ifdef fpc}@{$endif}ConsoleStdInFileRead,
	FileWrite: NIL,
    FileGetPos: NIL,
    FileSeek: NIL,
    FileEOF: NIL,
    FileDelete: NIL,
    FileRename: NIL,
    FileGetInfo: NIL,
    FileFlush: NIL,
    DirMake: NIL,
    DirDelete: NIL,
    DirRename: NIL,
    List. NIL,
    RefreshDriveInfo: NIL,
    FileSetAttributes: NIL 
 );
(*////////////////////////////////////////////////////////////////////////////////*)
  ConsoleStdInDrv: TDrive = (
	Name: 'Console stdin',
	TotalSpace: 0,
    UsedSpace: 0,
    dType: 0,
    Ex1: NIL,
    Ex2: NIL,
	Driver: {$ifdef fpc}@{$endif}ConsoleStdIn
);
(*////////////////////////////////////////////////////////////////////////////////*)

procedure _ExternConsolePreInit ( App: PApplication );
begin
	DebugMessage('_ExternConsolePreInit %x;',[App]);
	if assigned( App^.Host ) then begin
		App^.Cons := App^.Host^.Cons;
	end;
	New(App^.StdOut);
	fillchar(App^.StdOut^,sizeof(TFile),0);
	App^.StdOut^.Drive := {$ifdef fpc}@{$endif}ConsoleStrOutDrv;
	App^.StdOut^.Ex1 := App;

	New(App^.StdIn);
	memset(App^.StdIn,sizeof(TFile),0);
	App^.StdIn^.Drive := &ConsoleStdInDrv;
	App^.StdIn^.Ex1 := App;
end;
(*////////////////////////////////////////////////////////////////////////////////*)

procedure _ExternConsolePreUnInit ( App: PApplication );
begin
	DebugMessage('_ExternConsolePreUnInit %x;',[App]);
	if not assigned( App^.Host ) then
	begin
		if ( App^.Cons ) then _CloseConsole(App^.Cons);
		App^.Cons = NULL;
	end;

	Dispose(App^.StdOut);
	Dispose(App^.StdIn);
	App^.StdOut := NIL;
	App^.StdIn := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
var ConsoleExtender: PAppExtender;

function LibMain( argc: integer; argv: l_text ): integer;
begin

	Palette[0]  := makecol(0,0,0);
	Palette[1]  := makecol(0,0,128);
	Palette[2]  := makecol(0,128,0);
	Palette[3]  := makecol(0,128,128);
	Palette[4]  := makecol(128,0,0);
	Palette[5]  := makecol(128,0,128);
	Palette[6]  := makecol(128,128,0);
	Palette[7]  := makecol(192,192,192);
	Palette[8]  := makecol(128,128,128);
	Palette[9]  := makecol(0,0,255);
	Palette[10] := makecol(0,255,0);
	Palette[11] := makecol(0,255,255);
	Palette[12] := makecol(255,0,0);
	Palette[13] := makecol(255,0,255);
	Palette[14] := makecol(255,255,0);
	Palette[15] := makecol(255,255,255);

	ConsoleExtender := InstallAppExtender(&_ExternConsolePreInit,&_ExternConsolePreUnInit);
    
    (*
	SYSEXPORT(GetKey);
	SYSEXPORT(Printf);
	SYSEXPORT(Prompt);
	*)
	LibMain := 1;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
procedure Close;
begin
	RemoveAppExtender(ConsoleExtender);

end;
(*////////////////////////////////////////////////////////////////////////////////*)

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion	  := ULONG_ID(0,1,0,0);
  ConsoleExtender := NIL;

{$endif} {LINK_CLIBS}
end.
