(*
////////////////////////////////////////////////////////////////////////////////
//
//	Control Panel Deamon
//
//	Copyright (c) 2003 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
unit cp;
{
 .#include "kernel.h"
 .#include "menu.h"
 .#include "fms2.h"
 .#include "cp.h"
}
interface

uses guitypes, menu, fms2;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
  {$LINK menu.o}
  {$LINK fms2.o}
{$endif}

const NeededLibs: l_uid = ( 'fms2', '' );

var
 AppVersion: l_ulong;
 char    AppName[]	= "Control Panel directory deamon";
 l_uid	nUID		= "ControlPanel";

 CPPanels: PList;
 CPDriver: PFileDriver;
 CPCodec : PCodec;
 CPSubCodec: PSubCodec;
 CPDTE   : PDataTypeEx;
 CPDrive : PDrive;

const
 BTNWIDTH  = 80;
 BTNHEIGHT = 22;
 BTNSPACE  =  5;


type
  TRunFunc = fuction: l_bool;
  PCPPanel = ^TCPPanel;
  TCPPanel = record
	
	Name    : l_text Name;
	
	Command : l_text Command;
	Args    :l_text Args;
	
	(*
	Icon16  : p_bitmap;
	Icon32  : p_bitmap;
	Icon48  : p_bitmap;
	*)
	Icon    : PIcon;
	Run     : TRunFunc;
	
  end;
  
function CPGetIcons ( f: PFileInfo ): PIcon;
function CPRun ( o: PFileInfo; argc: integer; var argv: l_text; Host: PApplication; h: PErrorHandler ): l_bool;
function CPList ( d: PDrive; VDir,Dir: l_text; Attrib: l_uchar ): PList;
procedure FreeCPPanel ( o: PCPPanel o );
function CPAddPanel ( Name, Command, Args: l_text; Icon16, Icon32, Icon48: p_bitmap; Run: TRunFunc ): PCPPanel;

implementation
{$ifndef LINK_CLIBS}

(*////////////////////////////////////////////////////////////////////////////////*)
function CPGetIcons ( f: PFileInfo ): PIcon;
begin	
	CPGetIcons := DuplicateIcon(PCPPanel(f^.Ex)^.Icon);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CPRun ( o: PFileInfo; argc: integer; var argv: l_text; Host: PApplication; h: PErrorHandler ): l_bool;
begin
    CPRun := false;
	if assigned( PCPPANEL(o^.Ex)^.Run ) then
	begin
		CPRun := PCPPANEL(o^.Ex)^.Run;
	end
	else
	begin
		//RunFile2(PCPPANEL(o^.Ex)^.Command,PCPPANEL(o^.Ex)^.Args);
		CPRun := DynLdRun(PCPPANEL(o^.Ex)^.Command,PCPPANEL(o^.Ex)^.Args){ ? true : false};

	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CPList ( d: PDrive; VDir,Dir: l_text; Attrib: l_uchar ): PList;
var a,b: PListItem; l: PList;
begin
	l := NewList;

	if assigned( CPPanels^.Last ) then
	begin
		a := CPPanels^.Last^.Next; 
        b := a;

		repeat
		
			AddFileInfoToList(l,PCPPANEL(a^.Data)^.Name,VDir,0,FT_VFILE,0,0,0,0,d,CPCodec,a^.Data);

			a := a^.Next;
		
		until ( a = b );
	end;

	CPList := l;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeCPPanel ( o: PCPPanel o );
begin
	if ( o^.Name ) then o^.Name := '';
	if ( o^.Command ) then o^.Command := '';
	if ( o^.Args ) then o^.Args := '';
	if ( o^.Icon ) then FreeIcon(o^.Icon);
	Dispose(o);	
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CPAddPanel ( Name, Command, Args: l_text; Icon16, Icon32, Icon48: p_bitmap; Run: TRunFunc ): PCPPanel;
var o: PCPPanel;
begin
	New(o);
	if not assigned( o ) then CPAddPanel := NIL else
	begin
	fillchar(o, sizeof(TCPPanel), 0);

	o^.Name := Name;
	o^.Command := Command;
	
	if ( assigned(Icon16) or assigned(Icon32) or assigned(Icon48) ) then begin
		o^.Icon := NewIcon2(Icon16,Icon32,Icon48);
	end;
	o^.Run := Run;
	o^.Args := Args;

	ListAdd(CPPanels, '', o, {$ifdef fpc}@{$endif}FreeCPPanel);

	CPAddPanel := o;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)

function LibMain ( argc: integer; argv: l_text ): l_bool;
begin
	if not assigned( Me.Ressource ) then
	begin
		DebugError('CP.DL ressource is missing. Control Panel have not been loaded.',[]);
		LibMain := false;
	end;

	CPPanels := NewList();



	CPAddPanel('Keyboard', 'applications/setkeyb.app', NIL, NIL, NIL,0,0);
	CPAddPanel('Mouse', 'applications/setmouse.app', NIL, NIL, NIL,0,0);
	CPAddPanel('Display device', 'applications/setscr.app',NIL, ListKey(Me.Ressource,'SETSCR16'), ListKey(Me.Ressource,'SETSCR32'), 0,0);
	CPAddPanel('Wallpaper', 'applications/setwall.app', NULL, ListKey(Me.Ressource,'SETWALL16'), ListKey(Me.Ressource,'SETWALL32'),0,0);
	CPAddPanel('Theme manager', 'applications/settheme.app', NULL, 0, 0,0,0);

	New(CPDriver);
	fillchar(CPDriver^, sizeof(TFileDriver), 0);
	CPDriver^.List := CPList;
	CPDrive := AddDrive('Control Panel', FT_FOLDER, NULL, NULL, CPDriver);

	
	{ FMS2/DMS2 Compatiblity WIP }
	CPCodec := AddCodec('CP Codec',$FFFF0001,0,0,0,{$ifdef fpc}@{$endif}CPGetIcons,'&cpld','vfile/controlpanel',$FFFF0001,0 );
	AddNewSubCodec(CPCodec,$FFFF0001,0,0,0,0,0,0,0,0,0);
	CPDTE := AddDataTypeEx($FFFF0001,'Control panel item',0,0,0,NIL,DTEF_STRICTINTERNAL);
	AddDTAction( CPDTE,'Open',NIL, {$ifdef fpc}@{$endif}CPRun,$0FFFFFFF );

	
	
	LibMain := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Close;
begin
	RemoveDrive(CPDrive);
	FreeList(CPPanels);
	Dispose(CPDriver);
end;
(*////////////////////////////////////////////////////////////////////////////////*)


{$ifdef OPascal}
initialization
{$else}
begin
{$endif}

  AppVersion	:= ULONG_ID(0,0,0,1);
  CPDrive       := NIL;
  CPPanels      := NIL;


{$ENDIF} {LINK_CLIBS}  
end.
