//**
//*	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//*
//*	FILE:			debug.c
//*
//*	PROJECT:		Phoenix engine - Core
//*
//*	DESCRIPTION:	This is the debugging system that is used for printing
//*						error and information messages into kernel.txt
//*
//*	CONTRIBUTORS:
//*					Lukas Lipka
//*					Julien Etelain
//*
//*	TODO:			@
//*
//*	BUG:			@
//*
//*	MISTAKE:		@
//*
//*	IDEA:			@
//*/
(*
#include 'kernel.h'
#include 'internal.h'
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
*)

unit debug;

//** @defgroup debug Debug and logging system.
//	If your software or the system crashes during development use the set of debug functions to find the error.
//	@{
//
//**	@file
//	Debug core file
//*/

interface

uses
  Sysutils, ctypes, guitypes;

{$I cdefs.inc}
{$IFDEF LINK_CLIBS}
 {$LINK debug.o}
{$ENDIF}

type
  TDebugFile = TextFile;

var DebugFile: TDebugFile;

function DebugFileOpend: Boolean; {$IFDEF LINK_CLIBS}external;{$ENDIF}

procedure DebugMessage ( szFormat: String; args: array of const ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure DebugWarning ( szFormat: String; args: array of const ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure DebugError ( szFormat: String; args: array of const ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure DebugFatal ( szFormat: String; args: array of const ); {$IFDEF LINK_CLIBS}external;{$ENDIF}

procedure GSSystemDebugInstall ( szFilename: l_text ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure GSSystemDebugUninstall ( ); {$IFDEF LINK_CLIBS}external;{$ENDIF}

function DebugCheckPtr ( inf: l_text; ptr: Pointer ): l_bool; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function DCkPt( inf: l_text; ptr: Pointer ): l_bool; {$IFDEF LINK_CLIBS}external;{$ENDIF}


implementation

{$IFNDEF LINK_CLIBS}

uses
  dynld;

const
  KernelCopyright = '(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.';
  KernelName = 'Phoenix engine - Core';

function DebugFileOpend: Boolean;
var
  Buffer: array[0..127] of byte;
  Readed: Longint;
  BufPos: Longint;
begin
  {$ifdef OPascal}
  try
    Reset(DebugFile);
    DebugFileOpend := true;
  except
    DebugFileOpend := false;
  end;
  {$else}
  {$i-}
  Reset(Debugfile);
  {$i+}
  DebugFileOpend := IOResult = 0;
  {$endif}
end;

{//**
//	Writes a message into the debug file using the label 'Message:'
//*/}
procedure DebugMessage ( szFormat: String;
  args: array of const
);
begin

	if DebugFileOpend then
	begin
		Writeln(DebugFile, 'Message: ');
		Writeln(DebugFile, Format(szFormat,args));
		Writeln(DebugFile);
		//flush(DebugFile);
	end;
end;

{
	Writes a message into the debug file using the label 'Warning:'
}
procedure DebugWarning ( szFormat: String;
   args: array of const
); 
begin 
	if (DebugFileOpend) then
	begin
		Write(DebugFile, 'Warning: ');
		Write(DebugFile, Format(szFormat, args));
		Writeln(DebugFile);
		//flush(DebugFile);
	end;
end;

{
	Writes a message into the debug file using the label 'Error:'
}
procedure DebugError ( szFormat: String;
  args: array of const
); 
begin
	if (DebugFileOpend) then
	begin
		Write(DebugFile, 'ERROR: ');
		Write(DebugFile, Format(szFormat, args));
		Writeln(DebugFile);
		//flush(DebugFile);
	end;
end;

{
	Writes a message into the debug file using the label 'Fatal:' and halts the kernel.
}
procedure DebugFatal ( szFormat: String;
  args: array of const
);
begin
	if (DebugFileOpend) then
	begin
		Write(DebugFile, 'FATAL ERROR: ');
		Write(DebugFile, Format(szFormat, args));
		Writeln(DebugFile);
		Writeln(DebugFile);
		Writeln(DebugFile,'fatal error has occurred. Aborting program execution.');
		//flush(DebugFile);
	end;

	Exit;
end;

{
	Initializes the debug system
}
procedure GSSystemDebugInstall ( szFilename: l_text );
var
  t: TDateTime;
begin
	t := now;
	//DebugFile = NULL;

	if (szFilename='') then
		Close(DebugFile)
	else
	begin
        //DebugFile = fopen(szFilename, 'wt');
        Assign(DebugFile, szFilename);
        Rewrite(DebugFile);
        Append(DebugFile);
		if not DebugFileOpend then
			Write(Format('DEBUG::DEBUG - Can''t open file %S for writing\n', [szFilename]));
	end;

	///*
	//*	Print kernel information
	//*/
	{$ifdef _OZONE__WIN32_}
        (*
        DebugMessage('Starting log to %s on %s\t%s\n\t\tKERNEL: %s  VERSION: %d.%d.%d.%d\n\t\t\tWIN32 Release\n',
				[szFilename], [t], KernelCopyright, KernelName,
				 ULID_A(Me.Version),ULID_B(Me.Version),ULID_C(Me.Version),ULID_D(Me.Version)] );
        *)
	{$endif}
	{$ifdef _OZONE__DJGPP_}
		DebugMessage('Starting log to %s on %s\t%s\n\t\tKERNEL: %s  VERSION: %d.%d.%d.%d\n\t\t\tDJGPP Release\n',
				[szFilename], [ctime(t)], [KernelCopyright], [KernelName],
				[ULID_A(Me.Version)],[ULID_B(Me.Version)],[ULID_C(Me.Version)],[ULID_D(Me.Version)] );
	{$endif}
	{$ifdef _OZONE__LINUX_}
		DebugMessage('Starting log to %s on %s\t%s\n\t\tKERNEL: %s  VERSION: %d.%d.%d.%d\n\t\t\tLINUX Release\n',
				[szFilename], [ctime(t)], [KernelCopyright], [KernelName],
				[ULID_A(Me.Version)],[ULID_B(Me.Version)],[ULID_C(Me.Version)],[ULID_D(Me.Version)]);
	{$endif}

end;


function DebugCheckPtr ( inf: l_text; ptr: Pointer ): l_bool;
begin
	if ( ptr=nil ) then
	begin
		DebugWarning('NULL pointer at %s', [inf]);

		DebugCheckPtr := false;
	end;

	DebugCheckPtr := true;
end;

function DCkPt( inf: l_text; ptr: Pointer ): l_bool;
begin
   DCkPt := DebugCheckPtr(inf, ptr);
end;

///*
//	Uninstalls the debug system
//*/
procedure GSSystemDebugUninstall;
begin
	DebugMessage('Logging stopped, closing log file',[]);

	if DebugFileOpend then
	begin
		Close(DebugFile);
	end;
end;


{$ENDIF LINK_CLIBS}

end.
