////////////////////////////////////////////////////////////////////////////////
//
//	Data Management System - Core file
//
//	See also types.h for definitions
//
//	(c) Copyright 2003 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
unit dms;

interface

uses Sysutils, guitypes, list, vfile, text;

{$I cdefs.inc}
{$IFDEF LINK_CLIBS}
 {$LINK 'dms.o'}
 {$Link 'resource.o'}
{$ENDIF}

{
type
  PDataMan = ^TDataMan;
  TDataMan = record
    Type      : l_dataType;
    Name      : l_text;
    FreeData  : FreeDataProc;
    Duplicate : DuplicateFunc;
    Data      : Pointer;
  end;

  DataTypeManaged   : i_dataencodemanaged;
  DataEncodeManaged : i_dataencodemanaged;
  Extention         : l_text;
  SaveToFile        : SaveToFileFunc;
  LoadFromFile      : LoadFromFileFunc;
  Save              : SaveFunc;
  Load              : LoadFunc;
  UseForRessource   : l_bool;
  ExtraOptionsSet   : ExtraOptionsSetf;
  Icon32            : PImage;
  Icon16            : PImage;
}
{
 .#include "allegro.h"
 .#include "types.h"
 .#include "list.h"
 .#include "vfile.h"
 .#include "resource.h"
 .#include "error.h"

 .#ifndef _DMS2_H_INCLUDED_
 .#define _DMS2_H_INCLUDED_
}

type 
  PFExt = ^TFExt;
  TFExt = record
	//Extention: l_text; -> Stored in list key
	CodecId: l_ulong;
  end;

{ .#define FEXT(o) ((PFExt)(o))}

type 
  PMimeDat = ^TMimeDat;
  TMimeDat = record
	//MimeType: l_text; -> Stored in list key
	CodecId: l_ulong;
  end;

{ .#define MIMEDAT(o) ((PMimeDat)(o))}
  TDataTypeEx = Pointer; // nur für Übersetzung
  l_dataType = TDataTypeEx;

type
  PIcon = Pointer;
  i_dataencodemanaged = array of byte;
  GetIconsFunc     = function ( i: PFileInfo): PIcon;
  PCodec = ^TCodec;
  TCodec = record
     Name: l_text;
     CodecId: l_ulong;
     SubCodecs: PList;

     Default: PIcon;
	
     GetIcons: GetIconsFunc;

     DefaultExtention: l_text;
     DefaultMimeType: l_text;
     FavoriteType: l_ulong;
     Flags: l_ulong;
     DataTypeManaged   : i_dataencodemanaged;
     DataEncodeManaged : i_dataencodemanaged;
     Extention         : l_text;
     SaveToFile        : SaveToFileFunc;
     LoadFromFile      : LoadFromFileFunc;
     Save              : SaveFunc;
     Load              : LoadFunc;
     UseForRessource   : l_bool;
     ExtraOptionsSet   : ExtraOptionsSetf;
     Icon32            : PImage;
     Icon16            : PImage;

  end;

const CDCF_FREEICONS = $01;

type
  PErrorHandler = procedure (fn: String);
  scSaveToFile = function( filename: l_text; Data: l_ptr; var ExtraSave: l_ulong; h: PErrorHandler ): l_bool;
  scLoadFromFile = function( filename: l_text; var Data: l_ptr; var ExtraLoad: l_ulong; h: PErrorHandler ): l_bool;
  scSaveFunc = function( afile: PFileStruct; Data: l_ptr; var EndOffset: l_ulong; var ExtraSave: l_ulong; h: PErrorHandler ): l_bool;
  scLoadFunc = function( afile: PFileStruct; var Data: l_ptr; var EndOffset: l_ulong; var ExtraLoad: l_ulong; h: PErrorHandler ): l_bool;

type
  PSubCodec = ^TSubCodec;
  TSubCodec = record

	TypeId: l_ulong;
	CodecId: l_ulong;
	Flags: l_ulong;
    //Funktionszeiger dürfen hier nicht stehen, weil danach noch Felder
    DefaultExtraSave: ^l_ulong;
	DefaultExtraLoad: ^l_ulong;
	ExtraSaveSetApp: l_text;
	ExtraLoadSetApp: l_text;
    //deshalb jetzt hier
    SaveToFile: scSaveToFile;
	LoadFromFile: scLoadFromFile;
	Save: scSaveFunc;
	Load: scLoadFunc;

  end;

type
  dtmFreeProc = procedure(var Data: Pointer);
  dtmDuplFunc = function(var Data: Pointer): Pointer;
  PDataTypeMan = ^TDataTypeMan;
  TDataTypeMan = record
	TypeId: l_ulong;
	Free: dtmFreeProc;
	Duplicate: dtmDuplFunc;
	DefaultSubcodec: PSubCodec;
  end;

type
  SaveToFilefunc   = function ( f: l_text; Data: l_ptr; Size: l_ulong ): l_bool;
  LoadFromFilefunc = function ( f: l_text; var Data: l_ptr; var Size: l_ulong ): l_bool;
  Savefunc         = function ( f: PFileStruct; Data: l_ptr; Size: l_ulong; var EndOffset: l_ulong ): l_bool;
  Loadfunc         = function ( f: PFileStruct; var Data: l_ptr; var Size: l_ulong; var EndOffset: l_ulong ): l_bool;
  ExtraOptionsSetf = function ( o: PCodec): l_bool;
  DuplicateFunc    = function ( var d: Pointer; c: l_ulong): Pointer;

  PDataMan = ^TDataMan;
  TDataMan = record
    DType     : l_dataType;
    Name      : l_text;
    FreeData  : TFreeProc;
    Duplicate : DuplicateFunc;
    Data      : Pointer;
  end;
{ .#define DATATYPEMAN(o) ((PDataTypeMan)(o))  }
{ .#define CODEC(o) ((PCodec)(o))              }
{ .#define SUBCODEC(o) ((PSubCodec)(o))        }

//procedure DMS2Init ( );
//procedure DMS2DeInit ( );

var
  FExts, MimeDats, Codecs, DatMan: PList;
  GenericText, GenericBinary, GenericUText: PSubCodec;

function  GetDataTypeMan 		( TypeId: l_ulong ): PDataTypeMan; {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure AddMimeType 			( Mime: l_text; CodecId: l_ulong ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure AddFileExtention 		( FExt: l_text; CodecId: l_ulong ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
function  RegisterDataType 		( TypeId: l_ulong; Free: dtmFreeProc; Duplicate: dtmDuplFunc ): PDataTypeMan; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function  AddCodec			( Name: l_text; CodecId: l_ulong; Icon16, Icon32, Icon48: p_bitmap; GetIcons: GetIconsFunc; DefaultExtention: l_text; DefaultMimeType: l_text; FavoriteType, Flags: l_ulong ): PCodec; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function  GetCodec			( CodecId: l_ulong ): PCodec; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function  GetCodecFileExtention ( FExt: l_text ): PCodec; {$IFDEF LINK_CLIBS}external;{$ENDIF}

function _GetSubCodec ( o: PCodec; TypeId: l_ulong ): PSubCodec;


function AddNewSubCodec ( Codec: PCodec; TypeId, Flags: l_ulong;
                          SaveToFile: scSaveToFile;
                          LoadFromFile: scLoadFromFile;
                          Save: scSaveFunc;
                          Load: scLoadFunc;
			  DefaultExtraSave,DefaultExtraLoad: l_ulong;
                          ExtraSaveSetApp,ExtraLoadSetApp: l_text ): PSubCodec; {$IFDEF LINK_CLIBS}external;{$ENDIF}
						
function  NewSubCodec (   TypeId, CodecId, Flags: l_ulong;
                          SaveToFile: scSaveToFile;
                          LoadFromFile: scLoadFromFile;
                          Save: scSaveFunc;
                          Load: scLoadFunc;
			  DefaultExtraSave,DefaultExtraLoad: l_ulongPtr;
			  ExtraSaveSetApp,ExtraLoadSetApp: l_text ): PSubCodec; {$IFDEF LINK_CLIBS}external;{$ENDIF}
						
procedure AddSubCodec ( Codec: PCodec; o: PSubCodec ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure SetDefaultSubCodec ( TypeId: l_ulong; sc: PSubCodec ); {$IFDEF LINK_CLIBS}external;{$ENDIF}

function GetSubCodec ( CodecId, TypeId : l_ulong ): PSubCodec; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function GetSubCodecFileExtention ( FExt: l_text; TypeId: l_ulong ): PSubCodec; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function GetSubCodecMimeType ( Mime: l_text; TypeId: l_ulong ): PSubCodec; {$IFDEF LINK_CLIBS}external;{$ENDIF}

function GetFileExtention ( FileName: l_text ): l_text; {$IFDEF LINK_CLIBS}external;{$ENDIF}


function SaveDataEx ( FileName: l_text; var Data; TypeId: l_ulong; CodecId: l_ulong; ExtraSave: l_ulong; h: PErrorHandler ): l_bool; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function SaveData2 ( FileName: l_text; var Data; TypeId: l_ulong ): l_bool; {$IFDEF LINK_CLIBS}external;{$ENDIF}

function LoadDataEx ( FileName: l_text; TypeId: l_ulong; CodecId: l_ulong; var ExtraLoad: l_ulong; h: PErrorHandler ): Pointer; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function LoadData2 ( FileName: l_text; TypeId: l_ulong ): Pointer; {$IFDEF LINK_CLIBS}external;{$ENDIF}

function NewBinData ( Data: Pointer; Size: l_ulong ): PBindata; {$IFDEF LINK_CLIBS}external;{$ENDIF}

function DuplicateData2 ( TypeId: l_ulong; Data: Pointer ): Pointer; {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure FreeData2 ( TypeId: l_ulong; Data: Pointer ); {$IFDEF LINK_CLIBS}external;{$ENDIF}

function GetDefaultSubCodec ( TypeId: l_ulong ): PSubCodec; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function GetCodecFileInfo ( f: PFileInfo ): PCodec; {$IFDEF LINK_CLIBS}external;{$ENDIF}

procedure DMS2InstallIcons ( ); {$IFDEF LINK_CLIBS}external;{$ENDIF}

function NewIcon ( i16,i32,i48: p_bitmap ): PIcon; {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure FreeIcon ( i: PIcon ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
function NewIcon2 ( i16,i32,i48: p_bitmap ): PIcon; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function DuplicateIcon ( i: PIcon ): PIcon; {$IFDEF LINK_CLIBS}external;{$ENDIF}

function DuplicateFileRef ( Data: Pointer ): Pointer; {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure FreeBinData ( o: Pointer ); {$IFDEF LINK_CLIBS}external;{$ENDIF}

procedure DmsInstallBMP; {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure DmsInstallGenericBinary ( ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure DmsInstallGenericText ( ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure DmsInstallGenericUText ( ); {$IFDEF LINK_CLIBS}external;{$ENDIF}

procedure free(var o: PCodec); {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure FreeCodec ( o: Pointer ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure FreeDataMan ( o: Pointer ); {$IFDEF LINK_CLIBS}external;{$ENDIF}

function InstallNewCodec (DataTypeManaged: l_datatype;
							DataEncodeManaged: l_dataencode;
							Name: l_text;
							Extention: l_text;
							UseForRessource: l_bool;
							SaveToFile: SaveToFilefunc;
							LoadFromFile: LoadFromFilefunc;
							Save :Savefunc;
							Load :Loadfunc;
							ExtraOptionsSet :ExtraOptionsSetf): PCodec; {$IFDEF LINK_CLIBS}external;{$ENDIF}

function InstallNewDataManager (dType: l_datatype; Name: l_text; FreeData: TFreeProc; Duplicate: DuplicateFunc): PDataMan; {$IFDEF LINK_CLIBS}external;{$ENDIF}

implementation

{$IFNDEF LINK_CLIBS}

uses
  debug, recource, registry, Convert, Objects;

var
  DmsCodecs: PList;
  DmsDataManagers: PList;

procedure free(var o: PCodec);
begin
  Dispose(o);
end;

procedure FreeCodec ( o: Pointer );
begin
	DebugMessage ('DMS :: Unload Codec Name: %s', [PCodec(o)^.Name]);
    {
	//if ( PCODEC(o)^.Name ) then free(PCodec(o)^.Name);
	//if ( PCODEC(o)^.Extention ) then free(PCODEC(o)^.Extention);
    }
	free(PCodec(o));
end;

procedure FreeDataMan ( o: Pointer );
begin
	DebugMessage ('DMS :: Unload Manager Name: %s', [PDataMan(o)^.Name]);
	{//if ( PDataMan(o)^.Name ) then free(PDataMan(o)^.Name);}
	Dispose(PDataMan(o));
end;

procedure DmsInstallBMP;
begin

end;

procedure DmsInstallGenericBinary;
begin

end;

procedure DmsInstallGenericText;
begin

end;

procedure DmsInstallGenericUText;
begin

end;

function InstallNewCodec (DataTypeManaged: l_datatype;
							DataEncodeManaged: l_dataencode;
							Name: l_text;
							Extention: l_text;
							UseForRessource: l_bool;
							SaveToFile: SaveToFilefunc;
							LoadFromFile: LoadFromFilefunc;
							Save :Savefunc;
							Load :Loadfunc;
							ExtraOptionsSet :ExtraOptionsSetf): PCodec;
var
  o: PCodec;							
begin
	NEW(o);

	if (o = NIL) then InstallNewCodec := NIL;
        {
	o^.DataTypeManaged   := DataTypeManaged;
	o^.DataEncodeManaged := DataEncodeManaged;
        }
        o^.Name              := Name;
	o^.Extention         := Extention;
        {
	o^.SaveToFile        := @SaveToFile;
	o^.LoadFromFile      := @LoadFromFile;
	o^.Save              := @Save;
	o^.Load              := @Load;
	o^.UseForRessource   := UseForRessource;
	o^.ExtraOptionsSet   := @ExtraOptionsSet;
        }
	//o^.Icon32			 := LoadImage("SYSTEM/ICONS/file32.bmp");
	//o^.Icon16			 := LoadImage("SYSTEM/ICONS/file16.bmp");

	ListAdd(DmsCodecs, NULL, o, @FreeCodec);

	DebugMessage ('DMS :: New Codec installed (Name: %s  Extension: %s)', [o^.Name, o^.Extention]);

	InstallNewCodec := o;
end;

function InstallNewDataManager (dType: l_datatype; Name: l_text; FreeData: TFreeProc; Duplicate: DuplicateFunc): PDataMan;
{//function InstallNewDataManager (l_datatype Type,l_text Name,void (*FreeData)(void*),void*(*Duplicate)(void*,l_ulong) ): PDataMan;}
var
  o: PDataMan;
begin
	NEW(o);
	if not Assigned(o) then InstallNewDataManager := NIL;
	o^.DType     := dType;
	o^.Name      := Name;
	o^.FreeData  := FreeData;
	o^.Duplicate := Duplicate;
	ListAdd(DmsDataManagers, NULL, o, @FreeDataMan);
	DebugMessage ('DMS :: New Manager installed (%s)', [o^.Name]);
	InstallNewDataManager := o;
end;

procedure UnInstallCodec ( o: PCodec );
var
  i: PListItem;
begin
  i := ListFoundItem(DmsCodecs,o);
	if Assigned( i ) then ListRemoveItem(DmsCodecs,i);
end;

function FoundCodecByExtention ( DataTypeAccepted: l_datatype; Extention: l_text ) : PCodec;
var a, b: PListItem;
begin
  FoundCodecByExtention := NIL;
  if ( DmsCodecs^.Last=nil ) then FoundCodecByExtention := NIL else
  begin
    a := DmsCodecs^.Last^.Next;
    b := DmsCodecs^.Last^.Next;
    if (Assigned( a )) and Assigned( a^.Data ) then
    while ( a <> b ) do
    begin
      if ( Pointer(PCODEC(a^.Data)^.DataTypeManaged) = DataTypeAccepted) and

         (PCODEC(a^.Data)^.Extention = Extention) or

         (PCODEC(a^.Data)^.Extention = '') then

         FoundCodecByExtention := PCODEC(a^.Data);
	 a := a^.Next;
    end;
  end;
end;

function FoundCodecByEncoder ( DataTypeAccepted: l_datatype; DataEncode: l_dataencode ): PCodec;
var a,b: PListItem;
begin
  if ( DmsCodecs^.Last = nil ) then Result := NIL;
	a := DmsCodecs^.Last^.Next;
    b := DmsCodecs^.Last^.Next;
  if Assigned( a ) then 
  while a<>b do begin
    if ( Pointer(PCODEC(a^.Data)^.DataTypeManaged) = Pointer(DataTypeAccepted) ) and
       ( Pointer(PCODEC(a^.Data)^.DataEncodeManaged) = Pointer(DataEncode) ) or
         not Assigned( PCODEC(a^.Data)^.DataEncodeManaged ) then
      FoundCodecByEncoder := PCODEC(a^.Data);
	  a := a^.Next;
  end; 
  FoundCodecByEncoder := NIL;
end;

function DefaultCodec ( DataTypeAccepted : l_datatype ): PCodec;
var a,b: PListItem;
begin
  if ( DmsCodecs^.Last = NIL ) then DefaultCodec := NIL;
	a := DmsCodecs^.Last;
    b := DmsCodecs^.Last;
  if Assigned( a ) then
  while a<>b do
  begin
    if ( Pointer(PCODEC(a^.Data)^.DataTypeManaged) = Pointer(DataTypeAccepted) ) then
      DefaultCodec := PCODEC(a^.Data);
	  a := a^.Prev;
  end;
  DefaultCodec := NIL;
end;

function DefaultRessourceCodec ( DataTypeAccepted: l_datatype ): PCodec;
var a, b: PListItem;
begin
  if ( DmsCodecs^.Last = NIL ) then DefaultRessourceCodec := NIL;
     a := DmsCodecs^.Last;
     b := a;
  if Assigned( a ) then
  while a <> nil do
  begin
    if {( Longint(PCODEC(a^.Data)^.DataTypeManaged) = Longint(DataTypeAccepted) ) and}

       ( PCODEC(a^.Data)^.UseForRessource ) then

       DefaultRessourceCodec := PCODEC(a^.Data);
       a := a^.Prev;
  end;
  DefaultRessourceCodec := NIL;
end;

function GetDataManager ( aType: l_datatype ): PDataMan; 
var a, b: PListItem;
begin
  GetDataManager := NIL;
  if ( DmsDataManagers^.Last = NIL ) then GetDataManager := NIL;
	a := DmsDataManagers^.Last^.Next;
    b := DmsDataManagers^.Last^.Next;
  if Assigned( a ) then 
  while a<>b do
  begin
    if ( PDATAMAN(a^.Data)^.DType = aType ) then GetDataManager := PDATAMAN(a^.Data);
    a := a^.Next;
  end;
end;

function GetFileExtention ( FileName: l_text ): l_text;
var AFileExt: l_text;
begin
  AFileExt := ExtractFileExt ( FileName );
  if AFileExt <> '' then System.Delete(AFileExt, 1, 1);
  GetFileExtention := AFileExt;
end;

function SaveSizedData ( aType: l_datatype; var Data; Size: l_ulong; FileName :l_text ): l_bool;
var ext: l_text; dm: PCodec;
begin	
    ext := GetFileExtention(FileName);
	dm  := NIL;
	if ( ext <> '' ) then
		dm := FoundCodecByExtention(aType,ext)
	else
		dm := DefaultCodec(aType);
	if ( dm = nil ) then SaveSizedData := false else
	//SaveSizedData := dm^.SaveToFile(FileName,Data,Size);

        FileWrite(Data,1,Size,FileOpen(FileName,FSF_WRITE));

        SaveSizedData := true
end;

function SaveData ( aType: l_datatype; var Data; FileName: l_text ): l_bool; 
begin
	SaveData := SaveSizedData(aType,Data,0,FileName);
end;


const
   CachedRes: PList = NIL;
   CachedResFile: l_text = '';


function LoadSizedData ( ExecptedType: l_datatype; FileName: l_text; var Size: l_ulong ): Pointer;
var
  Item,ResFile: l_text;
  Data: Pointer;
  ext: l_text;
  dm: PCodec;
begin
    LoadSizedData := NIL;

    Data := nil;
	if ( Item[Length(Item)] = '#' ) then
	begin
		ResFile := System.Copy(FileName, 1, Length(FileName)-1);
		//Item++;
		//Data := LoadSizedDataFromRessource(ExecptedType,ResFile,Size,Item);

                FileRead(Data,1,Size,FileOpen(ResFile,FSF_READ));

		//free(ResFile);
		LoadSizedData := Data;
	end
    else
    begin
		Data := nil;
		ext := GetFileExtention(FileName);
		dm := NIL;
		if ( ext <> '' ) then
			dm := FoundCodecByExtention(ExecptedType,ext)
		else
			dm := DefaultCodec(ExecptedType);
		if ( dm = NIL ) then LoadSizedData := NIL else
                begin
                  getmem(Data,Size);
                  FileRead(Data,1,Size,FileOpen(ResFile,FSF_READ));
                  LoadSizedData := Data;
	{	//if ( dm^.LoadFromFile(FileName,Data,Size) ) then LoadSizedData := Data; }
                end;
	end;

end;

function LoadSizedDataFromRessource ( ExecptedType: l_datatype; FileName: l_text; var Size: l_ulong; Item: l_text ) : Pointer;
var a: PListItem;
begin	
	if ( CachedResFile = '' ) then 
	begin
		CachedResFile := FileName;
		CachedRes := LoadSizedData(ExecptedType, CachedResFile, Size);
	end
    else 
    if ( TextCompare(CachedResFile,FileName)=0 ) then 
    begin
		//if ( CachedResFile ) then free(CachedResFile);
		if Assigned( CachedRes ) then FreeList(CachedRes);
		CachedResFile := FileName;
                getmem(CachedRes,Size);
                FileRead(CachedRes, 1, Size, FileOpen(CachedResFile,FSF_READ));
                LoadSizedDataFromRessource := CachedRes;
		//CachedRes := LoadResourceFile(CachedResFile);
		if ( CachedRes = Nil ) then LoadSizedDataFromRessource := NIL;
	end;
	a := ListKeyItem(CachedRes,Item);
	if ( a = NIL ) then LoadSizedDataFromRessource := NIL;
    {
	//if ( a^.DataType <> ExecptedType ) then LoadSizedDataFromRessource := NIL;
	//if Assigned( Size ) then Size := a^.DataSize;
	//LoadSizedDataFromRessource := DataDuplicate(a^.DataType,a^.Data,a^.DataSize);
	}
end;

function LoadData ( ExecptedType: l_datatype; FileName: l_text ): Pointer;
var Size: l_ulong = 0;
begin	
  LoadData := LoadSizedData(ExecptedType,FileName,Size);
end;

procedure DataFree ( aType: l_ulong; var Data );
var m: PDataMan;
begin
  {
    m := GetDataManager(Pointer(aType));
	if Assigned( m ) then
	begin
	  if ( m^.FreeData ) then m^.FreeData(Pointer(Data));
	end;
        }
end;

function DataDuplicate ( aType: l_ulong; var Data; Size: l_ulong ): Pointer;
var m: PDataMan; 
begin
  {
    m = GetDataManager(Pointer(aType));
	if Assigned( m ) begin
	  if ( m^.FreeData ) then DataDuplicate := m^.Duplicate(Data,Size);
	end;
	DataDuplicate := NIL;
        }
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function _DuplicateData ( var Data; Size: l_ulong ): Pointer;
var o: Pointer;
begin
	GetMem(o, Size);
	if not Assigned( o ) then _DuplicateData := NIL;
	move(Data,o^,Size);
	_DuplicateData := o;
end;

function _DuplicateText (var Data; Size: l_ulong ): Pointer; 
var SData: String absolute Data;
begin
  _DuplicateText := @SData;
end;

function _DuplicateUText (var Data; Size: l_ulong ): Pointer;
var UTF8Str: UTF8String;
begin
  { //UTF8Str := TUTF8String(Data); }
   move(Data, UTF8Str, Size); 
  _DuplicateUText := @UTF8Str; //UTextDup((l_utext)Data); schauen, wie UTF8-Strings in FPC behandelt werden
end;

function _DuplicateImage (var Data; Size: l_ulong ) : Pointer;
var s,b: P_Bitmap;
begin
  s := P_BITMAP(Data);
  //b := create_bitmap(s^.w,s^.h);
  //if Assigned( b ) then blit(s,b,0,0,0,0,s^.w,s^.h);
  _DuplicateImage := b;
end;

{
procedure _DuplicateFileRef (var Data; Size: l_ulong ): Pointer; 
var
  s: PFileRef;
  d: PFileRef;
begin
	s := PFileRef(Data);
	d := Getmem(d, sizeof(TFileRef));
	if not Assigned( d ) then _DuplicateFileRef := NIL;
	
	d^.Drive 		:= s^.Drive;
	d^.Size			:= s^.Size; 			
	d^.Attributes	:= s^.Attributes;
	d^.LocalName	:= TextDup(s^.LocalName);
	d^.SName 		:= TextDup(s^.SName);
	
	_DuplicateFileRef := d;
end;
}
{ .$define TEST}
(*////////////////////////////////////////////////////////////////////////////////*)

(*////////////////////////////////////////////////////////////////////////////////*)
procedure InitDMS;
{$ifdef TEST}
var Tst: PList;
{$endif}
begin
	DmsCodecs := NewList();
	if ( DmsCodecs  = NIL ) then DebugFatal('DMS::NOTENOUGHMEMORY',[]);
	DmsDataManagers := NewList();
	if ( DmsDataManagers = NIL ) then DebugFatal('DMS::NOTENOUGHMEMORY',[]);

	// Generic Manager (free,duplicate)
        {
        InstallNewDataManager(TYPE_UNKNOW_BINARY,'Unknown Binary',free,_DuplicateData);
	InstallNewDataManager(TYPE_TEXT,'Text (ASCII/UTF-8)',free,_DuplicateText);
	InstallNewDataManager(TYPE_UTEXT,'Unicode Text (UTF-16)',free,_DuplicateUText);

	InstallNewDataManager(TYPE_IMAGE,'Image',Pointer(destroy_bitmap),_DuplicateImage);
	InstallNewDataManager(TYPE_FILEREF,'File Refrence',Pointer(FreeFileRef),_DuplicateFileRef);
	InstallNewDataManager(TYPE_FILEREFLIST,'File Refrence Listing',Pointer(FreeList),NIL);
        }
	//DmsInstallGenericBinary();
	//DmsInstallGenericText();
	//DmsInstallGenericUText();
	//DmsInstallBMP();
	{$ifdef TEST}
		Tst = NewList();
		ListAddItem(Tst,NewListItemEx('TEST','DATADATADATA',NULL,0,13));
		ListAddItem(Tst,NewListItemEx('TEST2','DATADATADATADATA',NULL,0,17));
		ListAddItem(Tst,NewListItemEx('TESTTxt','This is a nice and lovely text',NULL,TYPE_TEXT,0));
		SaveRessourceFile('test.res',Tst);
		FreeList(Tst);
		Tst := LoadRessourceFile('test.res');
		FreeList(Tst);
	{$endif}
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ShutDownDMS;
begin
	DebugMessage ('Shutting down DMS...',[]);
	FreeList(DmsCodecs);
	FreeList(DmsDataManagers);
	//if Assigned( CachedResFile ) then free(CachedResFile);
	if Assigned( CachedRes ) then FreeList(CachedRes);
	DebugMessage ('DMS Stopped',[]);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
{$i dms2.inc}

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
   GenericText  := NIL;
   GenericBinary:= NIL;
   GenericUText := NIL;


{$ENDIF} {LINK_CLIBS}
end.
