(*
////////////////////////////////////////////////////////////////////////////////
//
//      Data Management System V2 - Core file
//
//      (c) Copyright 2004 Point Mad. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
{ #define DMS2}

(** @defgroup dmsfms File and data management
        Libraries to manage file and data interractions
        @{
*)

(** @defgroup dms Data Management System (DMS)
        Load, save data, and get icons from files. Duplicates and free data.
        @{
*)
(**     @file
        DMS core file (provided into kernel)
*)
{
 .#include"kernel.h"
 .#include "string.h
}

(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeFExt ( o: PFExt );
begin
       { //if ( o->Extention ) free(o->Extention); }
        dispose(o);
end;

procedure FreeMimeDat( o: PMimeDat  );
begin
       { //if ( o->MimeType ) free(o->MimeType); }
        dispose(o);
end;

procedure FreeCodec2( o: PCodec );
begin
        if assigned( o^.Default ) then FreeIcon(o^.Default);
        if ( o^.Name <> '' ) then o^.Name := '';
        if ( o^.SubCodecs <> NIL ) then FreeList(o^.SubCodecs);
        if ( o^.DefaultExtention <> '' ) then o^.DefaultExtention := '';
        if ( o^.DefaultMimeType <> '') then o^.DefaultMimeType := '';

        dispose(o);
end;

procedure FreeSubCodec ( o: PSubCodec );
begin
        if ( o^.ExtraSaveSetApp <> '' ) then o^.ExtraSaveSetApp := '';
        if ( o^.ExtraLoadSetApp <> '' ) then o^.ExtraLoadSetApp := '';
        dispose(o);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeFExt(p: Pointer);
begin
  Dispose(PFExt(p));
end;

procedure AddFileExtention ( FExt: l_text; CodecId: l_ulong );
var c: PCodec; o: PFExt;
begin
        New(o);
        if assigned( o ) then
        begin
          o^.CodecId := CodecId;
          ListAdd(FExts,FExt,o,nil(*{$ifdef fpc}@{$endif}FreeFExt*));
          DebugMessage('DMS2 : File extention %s -> Codec %08x',[FExt,CodecId]);

          if ( c = GetCodec(CodecId) ) then
          begin
                if ( c^.DefaultExtention = '' ) then c^.DefaultExtention := FExt;
          end;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeMimeDat(p: Pointer);
begin
  Dispose(PMimeDat(p));
end;

procedure AddMimeType ( Mime: l_text; CodecId: l_ulong );
var c: PCodec; o: PMimeDat;
begin
        New(o);
        if assigned( o ) then
        begin
          o^.CodecId := CodecId;
          ListAdd(MimeDats,Mime,o,nil(*{$ifdef fpc}@{$endif}FreeMimeDat*));
          if ( c = GetCodec(CodecId) ) then
                if not( c^.DefaultMimeType <> '' ) then c^.DefaultMimeType := Mime;
        
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure freedtm(p:Pointer);
begin
  Dispose(PDataTypeMan(p));
end;

function _GetDataTypeMan ( TypeId: l_ulong ): PDataTypeMan;
var
        a,b: PListItem;
        o: PDataTypeMan;
begin
        if assigned( DatMan^.Last ) then
        begin
                a := DatMan^.Last^.Next;
                b := DatMan^.Last^.Next;
                repeat
                        if ( PDATATYPEMAN(a^.Data)^.TypeId = TypeId ) then _GetDataTypeMan := PDataTypeMan(a^.Data) else
                        a := a^.Next;
                until ( a = b );
        end;
        New(o);
        if assigned( o ) then
        begin
          fillchar(o,sizeof(TDataTypeMan),0);
          o^.TypeId               := TypeId;
          ListAdd(DatMan,NULL,o,nil(*{$ifdef fpc}&{$endif}freedtm*));
        end;
        _GetDataTypeMan := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetDataTypeMan ( TypeId: l_ulong ): PDataTypeMan;
var
        a,b:PListItem;
begin   
        GetDataTypeMan := NIL;
        if assigned( DatMan^.Last ) then
        begin
          a := DatMan^.Last^.Next;
          b := DatMan^.Last^.Next;
          repeat
                if ( PDATATYPEMAN(a^.Data)^.TypeId = TypeId ) then GetDataTypeMan := PDATATYPEMAN(a^.Data);
                a := a^.Next;
          until ( a = b );
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
{ TDuplicateFunc = function (p: Pointer): Pointer; }
function RegisterDataType ( TypeId: l_ulong; Free: dtmFreeProc; Duplicate: dtmDuplFunc ): PDataTypeMan;
var o: PDataTypeMan;
begin
        o := _GetDataTypeMan(TypeId);
        RegisterDataType := NIL;
        if assigned( o ) then
        begin
        o^.Free          := {$ifdef fpc}{$endif}Free;
        o^.Duplicate     := {$ifdef fpc}{$endif}Duplicate;
        RegisterDataType := o;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function AddCodec( Name: l_text; CodecId: l_ulong; Icon16,Icon32,Icon48: p_bitmap; GetIcons: GetIconsFunc; DefaultExtention, DefaultMimeType: l_text; FavoriteType: l_ulong; Flags : l_ulong ): PCodec;
var o: PCodec;
begin
   AddCodec := NIL;
   New(o);
   if assigned( o ) then
   begin
        fillchar(o,sizeof(TCodec),0);
        o^.Name := Name;
        o^.CodecId := CodecId;

        if ( assigned(Icon16) or assigned(Icon32) or assigned(Icon48) ) then
        begin
                if (( Flags and CDCF_FREEICONS ) = CDCF_FREEICONS) then
                        o^.Default := NewIcon(Icon16,Icon32,Icon48)
                else
                        o^.Default := NewIcon2(Icon16,Icon32,Icon48);
        end;
        o^.GetIcons := GetIcons;
        o^.DefaultExtention := TextDup(DefaultExtention);
        o^.DefaultMimeType := TextDup(DefaultMimeType);
        o^.SubCodecs := NewList;
        o^.FavoriteType := FavoriteType;
        o^.Flags := Flags;

        ListAdd(Codecs,'',o,nil(*{$ifdef fpc}@{$endif}FreeCodec2*));

        if ( DefaultExtention <> '' ) then AddFileExtention(DefaultExtention,CodecId);
        if ( DefaultMimeType <> '' ) then AddMimeType(DefaultMimeType,CodecId);

        DebugMessage('DMS2 : New codec %08x : %s (%s,%s,%08x)',[CodecId,Name,DefaultExtention,DefaultMimeType,FavoriteType]);

        AddCodec := o;
   end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetCodec ( CodecId: l_ulong ): PCodec;
var a,b: PListItem;
begin
        GetCodec := NIL;
        if assigned( Codecs^.Last ) then
        begin
          a := Codecs^.Last^.Next;
          b := Codecs^.Last^.Next;
          repeat
                if ( PCODEC(a^.Data)^.CodecId = CodecId ) then begin GetCodec := PCODEC(a^.Data); exit end else
                a := a^.Next;
          until ( a = b );
          DebugWarning('DMS2 : Codec %08x not loaded',[CodecId]);
        end;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetCodecFileExtention ( FExt: l_text ): PCodec;
var o: PFExt;
begin
        GetCodecFileExtention := NIL;
        if ( FExt<>'' ) then
        begin
                o := ListKeyCase(FExts,FExt);
                if not assigned( o ) then
                begin
                        DebugWarning('DMS2 : No codec assosciated to file extention %s',[FExt]);
                        GetCodecFileExtention := NIL;
                end else GetCodecFileExtention :=  GetCodec(o^.CodecId);
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetCodecMimeType ( Mime: l_text ): PCodec;
var o: PMimeDat;
begin
        GetCodecMimeType := NIL;
        o := ListKeyCase(MimeDats,Mime);
        if not assigned( o ) then
        begin
                DebugWarning('DMS2 : No codec assosciated to mime type %s',[Mime]);
                GetCodecMimeType := NIL;
        end else
        GetCodecMimeType := GetCodec(o^.CodecId);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetCodecFileInfo ( f: PFileInfo ): PCodec;
begin
        if assigned( f^.FMS ) then
                GetCodecFileInfo := PCodec(f^.FMS)
        else if ( f^.fType = FT_HDD ) then
                GetCodecFileInfo := GetCodecFileExtention('&hdd')
        else if ( f^.fType = FT_FOLDER ) then
                GetCodecFileInfo := GetCodecFileExtention('&folder')
        else
                GetCodecFileInfo := GetCodecFileExtention(GetFileExtention(f^.Name));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function NewSubCodec ( TypeId, CodecId, Flags: l_ulong;
                                                SaveToFile: scSaveToFile;
                                                LoadFromFile: scLoadFromFile;
                                                Save: scSaveFunc;
                                                Load: scLoadFunc;
                                                DefaultExtraSave, DefaultExtraLoad: l_ulongPtr;
                                                ExtraSaveSetApp,ExtraLoadSetApp: l_text ): PSubCodec;
var
  o: PSubCodec;
begin
     NewSubCodec := NIL;
     New(o);
     if assigned( o ) then
     begin
        fillchar(o,sizeof(TSubCodec),0);

        o^.TypeId := TypeId;
        o^.CodecId := CodecId;
        o^.Flags := Flags;
        o^.SaveToFile := SaveToFile;
        o^.LoadFromFile := LoadFromFile;
        o^.Save := Save;
        o^.Load := Load;
        o^.DefaultExtraSave := DefaultExtraSave;
        o^.DefaultExtraLoad := DefaultExtraLoad;
        o^.ExtraSaveSetApp := TextDup(ExtraSaveSetApp);
        o^.ExtraLoadSetApp := TextDup(ExtraLoadSetApp);

        if ( assigned(GetDefaultSubCodec(TypeId)) and assigned(Save) and assigned(Load) ) then SetDefaultSubCodec(TypeId,o);

        NewSubCodec := o;
     end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function AddNewSubCodec ( Codec: PCodec; TypeId,Flags: l_ulong;
                                                SaveToFile: scSaveToFile;
                                                LoadFromFile: scLoadFromFile;
                                                Save: scSaveFunc;
                                                Load: scLoadFunc;
                                                DefaultExtraSave, DefaultExtraLoad: l_ulong;
                                                ExtraSaveSetApp,ExtraLoadSetApp: l_text ): PSubCodec;

var o: PSubCodec;
begin
        if ( not(o = _GetSubCodec(Codec,TypeId )) ) then
        begin
                o := NewSubCodec(TypeId,Codec^.CodecId,Flags,SaveToFile,LoadFromFile,Save,Load,@DefaultExtraSave,@DefaultExtraLoad,ExtraSaveSetApp,ExtraLoadSetApp);
                ListAdd(Codec^.SubCodecs,'',o,nil(*{$ifdef fpc}@{$endif}FreeSubCodec*));
                if ( Codec^.FavoriteType = 0 ) then Codec^.FavoriteType := TypeId;
        end else
                DebugWarning('DMS2 : Codec %08x has already a sub codec for type %08x',[Codec^.CodecId,TypeId]);
        AddNewSubCodec := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure AddSubCodec ( Codec:PCodec; o:PSubCodec );
begin
        if assigned( _GetSubCodec(Codec,o^.TypeId ) ) then
                DebugWarning('DMS2 : Codec %08x has already a sub codec for type %08x',[Codec^.CodecId,o^.TypeId])
        else
                ListAdd(Codec^.SubCodecs,'',o,NIL);
        if ( Codec^.FavoriteType = 0 ) then Codec^.FavoriteType := o^.TypeId;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure SetDefaultSubCodec ( TypeId: l_ulong; sc: PSubCodec );
var o: PDataTypeMan;
begin
        o := _GetDataTypeMan(TypeId);
        if assigned( o ) then
                o^.DefaultSubcodec := sc
        else
                DebugError('DMS2 : Unable to set default sub codec for type %08x',[TypeId]);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function _GetSubCodec ( o: PCodec; TypeId: l_ulong ): PSubCodec;
var a,b: PListItem; c: PSubCodec;
begin
       _GetSubCodec := nil; c := nil;
        if assigned( o^.SubCodecs^.Last ) then
        begin
          a := o^.SubCodecs^.Last^.Next;
          b := o^.SubCodecs^.Last^.Next;
          repeat
                if ( PSUBCODEC(a^.Data)^.TypeId = TypeId ) then begin c := PSUBCODEC(a^.Data); a := b; end else
                a := a^.Next;
          until ( a = b );

          if c = NIL then DebugWarning('DMS2 : Codec %x do not support data type %x',[o^.CodecId,TypeId]);
          
         _GetSubCodec := c;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetSubCodec ( CodecId: l_ulong; TypeId: l_ulong ): PSubCodec;
var
      o  :  PCodec;
      sc :  PSubCodec; 
      dtm:  PDataTypeMan;
begin
        o := GetCodec(CodecId); sc := NIL;
        if assigned( o ) then sc := _GetSubCodec(o,TypeId);
        if not assigned( sc ) then
        begin
                dtm := GetDataTypeMan(TypeId);
                if assigned( dtm ) then
                begin
                        sc := dtm^.DefaultSubcodec;
                        if assigned( sc ) then
                                DebugWarning('DMS2 : Use default sub-codec instaed',[])
                        else
                                DebugError('DMS2 : No subcodec found for data type %x',[TypeId]);
                end else
                        DebugError('DMS2 : No subcodec found for data type %x',[TypeId]);
        end;
        GetSubCodec := sc;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetSubCodecFileExtention ( FExt: l_text; TypeId: l_ulong ): PSubCodec;
var
        o  : PCodec; 
        sc : PSubCodec;
        dtm: PDataTypeMan;
begin
        o := GetCodecFileExtention(FExt); sc := NIL;
        if assigned( o ) then sc := _GetSubCodec(o,TypeId);
        if not assigned( sc ) then
        begin
                dtm := GetDataTypeMan(TypeId);
                if assigned( dtm ) then
                begin
                        sc := dtm^.DefaultSubcodec;
                        if assigned( sc ) then
                                DebugWarning('DMS2 : Use default sub-codec instaed',[])
                        else
                                DebugError('DMS2 : No subcodec found for data type %x',[TypeId])
                end else
                        DebugError('DMS2 : No subcodec found for data type %x',[TypeId]);
        end;
        GetSubCodecFileExtention := sc;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetSubCodecMimeType (  Mime:l_text; TypeId:l_ulong ): PSubCodec;
var
        o  : PCodec; 
        sc : PSubCodec;
        dtm: PDataTypeMan;
begin        
        o := GetCodecMimeType(Mime); sc := NIL;
        if assigned( o ) then sc := _GetSubCodec(o,TypeId);
        if not assigned( sc ) then
        begin
                dtm := GetDataTypeMan(TypeId);
                if assigned( dtm ) then
                begin
                        sc := dtm^.DefaultSubcodec;
                        if assigned( sc ) then
                                DebugWarning('DMS2 : Use default sub-codec instaed',[])
                        else
                                DebugError('DMS2 : No subcodec found for data type %x',[TypeId]);
                end else
                        DebugError('DMS2 : No subcodec found for data type %x',[TypeId]);
        end;
        GetSubCodecMimeType := sc;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(**
        Found the sub-codec for the specified file and data type.
        Determines the codec using the file extention.
        @param File File path
        @param TypeId Data type identifier
        @return Pointer to sub-codec, or null if not none corresponding
*)
function GetSubCodecFile ( Filename:l_text; TypeId:l_ulong ): PSubCodec;
begin
        GetSubCodecFile := GetSubCodecFileExtention(GetFileExtention(Filename),TypeId);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(**
        Get default sub-codec for specified data type
        @param TypeId Type
        @return Pointer to sub-codec, or NULL if none installed
*)
function GetDefaultSubCodec ( TypeId: l_ulong ): PSubCodec;
var o: PDataTypeMan;
begin 
        o := _GetDataTypeMan(TypeId);
        if assigned( o ) then GetDefaultSubCodec := o^.DefaultSubcodec else GetDefaultSubCodec := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Save data functions                                                          *)
(*////////////////////////////////////////////////////////////////////////////////*)
function SubCodecSaveEmu ( o:PSubCodec; Filename: l_text; Data:l_ptr; var ExtraSave: l_ulong; h: PErrorHandler ): l_bool;
var f: PFileStruct; r: l_bool; exsave: l_ulong;
begin
        exsave := 0;
        r := false;
        f := FileOpen(Filename,FSF_Write);
        if assigned( f )then begin
                r := o^.Save(f,Data,ExtraSave,exsave,h);
                FileClose(f);
        end else
                {CreateError(h,ERROR_CANTOPENTOWRITE,'File not found or read-only file');}
        SubCodecSaveEmu := r;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function SaveDataEx ( Filename: l_text; var Data; TypeId: l_ulong; CodecId: l_ulong; ExtraSave: l_ulong; h: PErrorHandler ): l_bool;
var sc: PSubCodec;
begin
        sc := NIL;
        SaveDataEx := false;
        if ( CodecId > 0 ) then
                sc := GetSubCodec(CodecId,TypeId)
        else
                sc := GetSubCodecFile(Filename,TypeId);
        if not assigned( sc ) then SaveDataEx := false;
        if ( ExtraSave = 0) then ExtraSave := sc^.DefaultExtraSave^;
        if assigned( sc^.SaveToFile ) then
                SaveDataEx := sc^.SaveToFile(Filename,@Data,ExtraSave,h)
        else if assigned( sc^.Save ) then
                SaveDataEx := SubCodecSaveEmu(sc,Filename,@Data,ExtraSave,h);
        DebugMessage('DMS 2 : %s : Codec is unable to save %x data type',[Filename,TypeId]);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(** Saves specified data to a file.
        @param File Path to the file
        @param Data Data to write
        @param TypeId Type of data
        @return true on sucess, else returns false (no codec, or codec can't write, or file error)
        @note use SaveDataEx for advanced options
        Similar to SaveDataEx(File,Data,TypeId,0,NULL);
        @see SaveDataEx
        @see LoadData2
        @note This function doesn't support ressources adressing.
*)
function SaveData2 ( Filename: l_text; var Data; TypeId: l_ulong ): l_bool;
var exsave: l_ulong;
begin
  exsave := 0; SaveData2 := SaveDataEx(Filename,Data,TypeId,0,exsave,NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Load data functions                                                          *)
(*////////////////////////////////////////////////////////////////////////////////*)
function SubCodecLoadEmu ( o: PSubCodec; Filename: l_text; Data: l_ptr; var ExtraLoad: l_ulong; h: PErrorHandler ): l_bool;
var     f: PFileStruct;
        r: l_bool;
        e: Longword;
begin
        r := false;
        f := FileOpen(Filename,FSF_READ);
        if assigned( f ) then
        begin
                e := 0;
                r := o^.Load(f,Data,ExtraLoad,e,h);
                FileClose(f);
        end else
                {CreateError(h,ERROR_FILENOTFOUND,'File not found');}
        SubCodecLoadEmu := r;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(** Loads data from a file as spefied type usign the specified codec.
        @param File Path to the file
        @param TypeId Type of data to return
        @param CodecId Identifier of codec to use (if 0 autodetect codec)
        @param ExtraLoad Pointer to extra load options of codec (if NULL autofill values)
        @param h Error handler to get information on eventual error
        @return On sucess returns pointer to data, else return NULL if sytem was unable to load file as the specified data type (no codec or data type not avaible for this codec or file error)
        @see LoadData2
        @see SaveData2
        @note This function doesn't support ressources adressing.
*)
function LoadDataEx ( Filename: l_text; TypeId, CodecId: l_ulong; var ExtraLoad: l_ulong; h: PErrorHandler ): Pointer;
var
  sc: PSubCodec;
  Data: Pointer;
begin
  sc := NIL;
  Data := NIL;
        
  if ( CodecId > 0) then
                sc := GetSubCodec(CodecId,TypeId)
  else
                sc := GetSubCodecFile(Filename,TypeId);
  if ( not assigned(sc) or (Filename = '')) then LoadDataEx := NIL else
  begin
        if ( ExtraLoad = 0 ) then ExtraLoad := sc^.DefaultExtraLoad^;
        DebugMessage('DMS2 : Load file %s using codec %08x to type %08x',[Filename,sc^.CodecId,sc^.TypeId]);
        if assigned( sc^.Load ) or assigned( sc^.LoadFromFile ) then
        begin
          if assigned( sc^.LoadFromFile ) then
          begin
            if ( sc^.LoadFromFile(Filename,Data,ExtraLoad,h) ) then
            begin
              DebugMessage('DMS2 : Load file %s ', [Filename]);
              LoadDataEx := Data;
            end;
          end else 
          begin
            if ( SubCodecLoadEmu(sc,Filename,Data,ExtraLoad,h) ) then
            begin
              DebugMessage('DMS2 : Sub codec load emu %s ', [Filename]);
              LoadDataEx := Data;
            end;
          end; (* else
          begin
            DebugMessage('DMS2 : Codec is unable to load file',[]);
            LoadDataEx := NIL;
          end;
        end else
        begin
          DebugMessage('DMS2 : %s Codec is unable to load %x data type',[Filename,TypeId]);
          LoadDataEx := NIL;
        end;
        *)
  end;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*////////////////////////////////////////////////////////////////////////////////*)
function LoadDataFromRessource ( TypeId: l_ulong; Filename: l_text; Item: l_text ): Pointer;
var a: PListItem;
begin
        if ( CachedResFile = '' ) then begin
                CachedResFile := Filename;
                CachedRes := LoadRessourceFile(CachedResFile);
        end else if ( TextCompare(CachedResFile,Filename)>0 ) then
        begin
                if ( CachedResFile <> '' ) then {free(CachedResFile)};
                if assigned( CachedRes ) then FreeList(CachedRes);
                CachedResFile := Filename;
                CachedRes := LoadRessourceFile(CachedResFile);
                if not assigned( CachedRes ) then LoadDataFromRessource := NIL;
        end;
        a := ListKeyItem(CachedRes,Item);
        if not assigned( a ) then LoadDataFromRessource := NIL else
        if ( a^.DataType <> TypeId ) then LoadDataFromRessource := NIL else        
        LoadDataFromRessource := DuplicateData2(a^.DataType,a^.Data);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(** Loads data from a file as specified type.
        @param File Path to the file to load
        @param TypeId Identifier of data type to return
        @return On sucess returns pointer to data, else return NULL if sytem was unable to load file as the specified data type (no codec or data type not avaible for this codec or file error)
        @note Don't forget to free data : you can use FreeData2(TypeId, Data);
        You can load an item form a ressource susing this syntax : "/path/to/file.res#item"; ex: "/system/xlib/cons.dl#ICON32"
        @see LoadDataEx
        @see SaveData2
*)
function LoadData2 ( Filename: l_text; TypeId: l_ulong ): Pointer;
var Item,ResFile: l_text; Data: Pointer; exload: l_ulong;
begin
        Item := IntToStr(TextRChr(Filename,'#')); 
        if ( Item > '0' ) then
        begin
                ResFile := {TextNDup(Filename,Item-Filename);}  system.copy(Filename, 1, system.POS('#',Filename)-1);
                Item := Copy(Item, system.POS('#',Item)+1, Length(Item));
                Data := LoadDataFromRessource(TypeId,ResFile,Item);
                ResFile := '';
                LoadData2 := Data;
        end else
        exload := 0;
        LoadData2 := LoadDataEx(Filename,TypeId,0,exload,NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Binary data                                                                  *)
(*////////////////////////////////////////////////////////////////////////////////*)
function NewBinData ( Data: Pointer; Size: l_ulong ): PBinData;
var o: PBinData;
begin
        New(o);
        o^.Data := Data;
        o^.Size := Size;
        NewBinData := o;
end;
////////////////////////////////////////////////////////////////////////////////
function DuplicateBinData ( o: Pointer ): Pointer;
var
  nd: Pointer;
   n: PBinData;
begin    
        if assigned( o ) then
        begin
                getmem(nd,PBinData(o)^.Size);
                
                if not assigned( nd ) then DuplicateBinData := NIL else
                begin

                  move(PBINDATA(o)^.Data^,nd^,PBINDATA(o)^.Size);
                  n := NewBinData(nd,PBinData(o)^.Size);
                  if not assigned( n ) then freemem(nd,PBinData(o)^.Size);

                  DuplicateBinData := n;
                end;
        end else
        DuplicateBinData := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeBinData ( o: Pointer );
begin
        if assigned( PBinData(o)^.Data ) then freemem(PBinData(o)^.Data, PBinData(o)^.Size);
        Dispose(PBindata(o));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Duplicators                                                                  *)
(*////////////////////////////////////////////////////////////////////////////////*)
{$i bmp.pas}
function DuplicateImage ( Data: Pointer ): Pointer;
var s,b: {P}Bitmap;
begin
(*        if assigned( Data ) then
        begin
                s := PBitmap(Data);
                b := create_bitmap(s^.w,s^.h);
                if assigned( b ) then blit(s,b,0,0,0,0,s^.w,s^.h);
                DuplicateImage := b;
        end else
        DuplicateImage := NIL; *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DuplicateFileRef (Data: Pointer): Pointer;
var
  s: PFileRef;
  d: PFileRef;
begin
   DuplicateFileRef := NIL;

   s := PFileRef(Data);

   New(d);

   if assigned( d ) then
   begin
     move(s^, d^, sizeof(TFileRef));
     DuplicateFileRef := d;
   end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DuplicateFileRefList (Data: Pointer): Pointer;
var
  n, l: PList;
  a, b: PListItem;
begin   
        l := PList(Data);
        n := NewList;
        b := l^.last;
        a := l^.Last;
        repeat
                ListAdd(n,a^.Key,DuplicateFileRef(PFILEREF(a^.Data)),{$ifdef fpc}@{$endif}FreeFileRef);
                a := a^.Next;
        until ( a = b);
        DuplicateFileRefList := n;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Icons managers                                                               *)
(*////////////////////////////////////////////////////////////////////////////////*)
function NewRleSpriteStretch ( ico: PImage; size: Integer ): PRLESprite;
var
        s: PRleSprite;
        buffer: PImage; 
begin
        (*
        buffer := create_bitmap(size,size);
        clear_to_color(buffer, ico^.vtable^.mask_color);
        stretch_sprite(buffer, ico, 0, 0, size, size);
        s := get_rle_sprite(buffer);
        destroy_bitmap(buffer);
        NewRleSpriteStretch := s;
        *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function NewIcon2 ( i16, i32, i48: p_bitmap ): PIcon;
var i: PIcon;
begin
        (*
        if assigned( i ) then
        begin
          i^.Instances := 1;

          if assigned( i16 ) then
                i^.Icon16 := get_rle_sprite(i16);
          else if assigned( i32 ) then
                i^.Icon16 := NewRleSpriteStretch(i32,16);
          else
                i^.Icon16 := NewRleSpriteStretch(i48,16);

          if assigned( i32 ) then
                i^.Icon32 := get_rle_sprite(i32);
          else if assigned( i48 ) then
                i^.Icon32 := NewRleSpriteStretch(i48,32);
          else
                i^.Icon32 := NewRleSpriteStretch(i16,32);

          if assigned( i48 ) then
                i^.Icon48 := get_rle_sprite(i48);
          else if assigned( i32 ) then
                i^.Icon48 := NewRleSpriteStretch(i32,48);
          else
                i^.Icon48 := NewRleSpriteStretch(i16,48);

          NewIcon2 := i;
          *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function NewIcon ( i16, i32, i48: p_bitmap ): PIcon;
var i: PIcon;
begin
        (*
        i := NewIcon2(i16,i32,i48);
        destroy_bitmap(i16);
        destroy_bitmap(i32);
        destroy_bitmap(i48);
        NewIcon := i;
        *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DuplicateIcon ( i: PIcon ): PIcon; {  Will help to save memory }
begin   (*
        if assigned( i ) then
                inc(i^.Instances);
        DuplicateIcon := i; *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeIcon ( i: PIcon );
begin   (*
        Dec(i^.Instances);
        if ( i^.Instances = 0 ) then
        begin
                if ( i^.Icon16 ) then destroy_rle_sprite(i^.Icon16);
                if ( i^.Icon32 ) then destroy_rle_sprite(i^.Icon32);
                if ( i^.Icon48 ) then destroy_rle_sprite(i^.Icon48);
                Dispose(i);
        end; *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Data operators                                                               *)
(*////////////////////////////////////////////////////////////////////////////////*)
(** Frees specified data, characterized by its type
        @param TypeId Data type identifier
        @param Data Pointer to data to free
*)
procedure FreeData2 ( TypeId: l_ulong; Data: Pointer );
var o: PDataTypeMan;
begin
        o := GetDataTypeMan(TypeId);
        if assigned( o ) then
                if assigned( o^.Free ) then
                        o^.Free(Data);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(* * Ducplicates specified data, characterized by its type
        @param TypeId Data type identifier
        @param Data Pointer to data to duplicate
        @return On sucess, pointer to duplicated data, or NULL on error (not enough memory most of times)
*)
function DuplicateData2 ( TypeId: l_ulong; Data: Pointer ): Pointer;
var o: PDataTypeMan;
begin
        o := GetDataTypeMan(TypeId);
        if Assigned( o ) then
                if Assigned( o^.Duplicate ) then
                        DuplicateData2 := o^.Duplicate(Data);
        DuplicateData2 := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure InstallGatewayFromReg ( o: PRegKey );
var
  id,x : l_ulong;
  a,b: PRegKey;
  c  : PCodec;
begin
        Id := HexToLongInt(o^.Name);
        DebugMessage('DMS2 : Ignore next warning message',[]);
        c := GetCodec(Id);
        if not assigned( c ) then c := AddCodec('',Id,NIL,NIL,NIL,NIL,'','',0,CDCF_FREEICONS);
        if ( o^.aType = RKT_TEXT) then
        begin
                {if ( c^.Name ) then free(c^.Name);}
                c^.Name := PString(o^.Data)^;
        end;

        if assigned( o^.Last ) then
        begin
                a := o^.Last^.Next;
                b := o^.Last^.Next;
                repeat
                        if ( TextCaseCompare(a^.Name,'FavoriteType' )=0) and (a^.aType = RKT_LONG) then
                                c^.FavoriteType := PInteger(a^.Data)^
                        else begin
                                if ( a^.aType = RKT_LONG ) then
                                begin
                                        DebugMessage('DMS2 : New gateway from codec %08x -> to data type %08x',[c^.CodecId,l_ulong(a^.Data)]);
                                        x := 0;
                                        AddNewSubCodec (c,PInteger(a^.Data)^,0,nil,nil,nil,nil,x,x,'','');
                                end else if ( a^.aType = RKT_TEXT ) then
                                begin
                                        DebugMessage('DMS2 : Sub codec %s (into %08x)',[a^.Data,c^.CodecId]);
                                        if ( TextCaseCompare(PString(a^.Data)^,'GenericBinary' ) = 0 ) then
                                                AddSubCodec(c,GenericBinary)
                                        else if ( TextCaseCompare(PString(a^.Data)^,'GenericText' ) = 0 ) then
                                                AddSubCodec(c,GenericText)
                                        else if ( TextCaseCompare(PString(a^.Data)^,'GenericUText' ) = 0 ) then
                                                AddSubCodec(c,GenericUText);
                                end;
                        end;
                        a := a^.Next;
                until ( a = b );
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure InstallCodecsIcons ( o: PRegKey );
var
  Id: l_ulong; 
  a, b: PRegKey;
  c: PCodec;
  Img16,Img32,Img48: PImage;

begin
    Id := HexToLongInt(o^.Name);
    Img16 := NIL;
    Img32 := NIL;
    Img48 := NIL;
    c := GetCodec(Id);
    if assigned( c ) then
    begin
        if assigned( o^.Last ) then
        begin     
                a := o^.Last^.Next;
                b := o^.Last^.Next;
                repeat
                        if ( TextCaseCompare(a^.Name,'Icon16' ) = 0) and ( a^.aType = RKT_TEXT ) then
                                Img16 := LoadData2(PString(a^.Data)^,DTYPE_IMAGE)
                        else if ( TextCaseCompare(a^.Name,'Icon32' ) = 0) and ( a^.aType = RKT_TEXT ) then
                                Img32 := LoadData2(PString(a^.Data)^,DTYPE_IMAGE)
                        else if ( TextCaseCompare(a^.Name,'Icon48' ) = 0) and ( a^.aType = RKT_TEXT ) then
                                Img48 := LoadData2(PString(a^.Data)^,DTYPE_IMAGE);
                        a := a^.Next;
                until ( a = b );
        end;
        if ( assigned(Img48) or assigned(Img32) or assigned(Img16) ) then
        begin
                if assigned( c^.Default ) then FreeIcon(c^.Default);
                c^.Default := NewIcon(Img16,Img32,Img48);
        end;
    end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*
function GetFileExtention ( FileName: l_text ): l_text;
begin
  GetFileExtention := ExtractFileExt(Filename);
end;
*)
(*////////////////////////////////////////////////////////////////////////////////*)
const ResCdc: PCodec = NIL;
(*////////////////////////////////////////////////////////////////////////////////*)
function RESLoadFromFile ( filename: l_text; var Data: l_ptr; var Extra: l_ulong; h: PErrorHandler ): l_bool;
begin
  Data := LoadRessourceFile(filename);
  if assigned( Data ) then
                RESLoadFromFile := true
        else
                RESLoadFromFile := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function RESSaveToFile   ( filename: l_text;  Data: l_ptr; {l_ulong*} Extra: PLongword; h: PErrorHandler ): l_bool;
begin
        SaveRessourceFile(filename,PList(Data));
        RESSaveToFile := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DMS2DeInit;
begin

        if assigned( FExts )    then FreeList(FExts);
        if assigned( MimeDats ) then FreeList(MimeDats);
        if assigned( Codecs )   then FreeList(Codecs);
        if assigned( DatMan )   then FreeList(DatMan);

end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DMS2Init;
var 
  o,a,b: PRegKey; 
  FExts,
  MimeDats,
  Codecs,
  Datman: PList;
  x: Integer;
begin
        FExts    := NewList;
        MimeDats := NewList;
        Codecs   := NewList;
        DatMan   := NewList;

        if ( FExts=NIL) or (MimeDats=NIL) or (Codecs=NIL) or (DatMan=NIL) then
        begin
                DMS2DeInit;
                DebugFatal('DMS2 :: NOT ENOUGH MEMORY',[]);
                exit;
        end;
        { InitGenericSubCodecs(); }

        { Generic and default handlers }

        { NB: If a data type is not registred clipboard can't use it. }
        {
        RegisterDataType(DTYPE_BINDATA,{$ifdef fpc}@{$endif}FreeBinData,{$ifdef fpc}@{$endif}DuplicateBinData);
        RegisterDataType(DTYPE_TEXT,{$ifdef fpc}@{$endif}free,{$ifdef fpc}@{$endif}TextDup);
        RegisterDataType(DTYPE_UTEXT,{$ifdef fpc}@{$endif}free,{$ifdef fpc}@{$endif}UTextDup);
        RegisterDataType(DTYPE_IMAGE,{$ifdef fpc}@{$endif}destroy_bitmap,{$ifdef fpc}@{$endif}DuplicateImage);
        RegisterDataType(TYPE_FILEREF,{$ifdef fpc}@{$endif}FreeFileRef,{$ifdef fpc}@{$endif}DuplicateFileRef);
        RegisterDataType(TYPE_FILEREFLIST,{$ifdef fpc}@{$endif}FreeList,{$ifdef fpc}@{$endif}DuplicateFileRefList);
        RegisterDataType(DTYPE_ICON,{$ifdef fpc}@{$endif}FreeIcon,{$ifdef fpc}@{$endif}DuplicateIcon);
        }
        SetDefaultSubCodec(DTYPE_BINDATA,GenericBinary);
        SetDefaultSubCodec(DTYPE_TEXT,GenericText);
        SetDefaultSubCodec(DTYPE_UTEXT,GenericUText);

        { Registry }

        o := ResolveKey('/SYSTEM/DMS/Codecs/Codecs');

        if assigned( o ) then
        begin
                
                if assigned( o^.Last ) then
                begin
                        a := o^.Last^.Next;
                        b := o^.Last^.Next;
                        repeat
                                InstallGatewayFromReg(a);
                                a := a^.Next;
                        until ( a = b );
                end;
        end;

        o := ResolveKey('/SYSTEM/DMS/Extentions');

        if assigned( o ) then
        begin
                
                if assigned( o^.Last ) then
                begin
                        a := o^.Last^.Next;
                        b := o^.Last^.Next;
                        repeat
                                if ( a^.aType = RKT_LONG ) then
                                        AddFileExtention(a^.Name,PInteger(a^.Data)^);
                                a := a^.Next;
                        until ( a = b );
                end;
        end;
        x := 0;
        {
        ResCdc := AddCodec('oZone DMS ressource file',ULONG_ID('D','M','S','R'),0,0,NIL,NIL,'res','binary/ressource',DTYPE_RESSOURCE,0 );
        AddSubCodec(ResCdc,GenericBinary);
        AddNewSubCodec(ResCdc,DTYPE_RESSOURCE,0,@RESSaveToFile,@RESLoadFromFile,0,0,0,0,0,0 );
        }


end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DMS2InstallIcons;
var o,a,b: PRegKey;
begin

        o := ResolveKey('/SYSTEM/DMS/Codecs/Codecs');

        if assigned( o ) then
        begin
                
                if assigned( o^.Last ) then
                begin
                        a := o^.Last^.Next;
                        b := o^.Last^.Next;
                        repeat
                                InstallCodecsIcons(a);
                                a := a^.Next;
                        until ( a = b );
                end;
        end;


end;
(*////////////////////////////////////////////////////////////////////////////////*)

