(*////////////////////////////////////////////////////////////////////////////////
//
//      Dynamic Loading System - Loader
//
//      (c) Copyright 2003,2004 Point Mad. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////*)
unit dynld;

interface

uses
  guitypes, list, text, events, error, recource;
  
{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$LINK dynld.o}
{$endif}  
  
{
 .#include <setjmp.h>

 .#include "types.h"
 .#include "list.h"
 .#include "stdio.h"
 .#include "vfile.h"
 .#include "error.h"
}



{	DynLD version      }

var
  DYNLDVERSION: Integer;


{	Application states }


const
  DynLdStateStarting = $00;
  DynLdStateRunning  = $01;
  DynLdStateShutDown = $02;
  DynLdStateDied     = $03;

  DynLdStateToFree   = $FF;

  DYNLD_STRATING     = $01;
  DYNLD_RUNNING      = $02;

(*
	Output types
*)
  DYNLD_TYPELIB      = $00;
  DYNLD_TYPEAPP      = $01;

{type PConsole = ^TConsole;}

(*////////////////////////////////////////////////////////////////////////////////*)
(*typedef struct PDynLdEvent = ^TDynLdEvent;                                      *)
type
  PDynLdEvent = ^TDynLdEvent;
  TRedirectProc = procedure(var p: Pointer; ptrEv: PDynLdEvent);
  TDynLdEvent = record

	Ev: TEvent;

    Dest: Pointer;
    Redirect: TRedirectProc;
  end;
(* define DYNLDEVENT(o) ((PDynLdEvent)(o)) *)

(*////////////////////////////////////////////////////////////////////////////////*)
type
  Ptr_l_uid = ^l_uid;
  TMainfunc = function(argc: integer; argv: l_text): Integer;
  TClosefunc = procedure;
  PApplication = ^TApplication;
  TApplication = record
	Name: l_text;
	FileName: l_text;
  	Version: l_ulong;
	Data: Pointer;

	Main: TMainfunc;
	Close: TClosefunc;

	Ressource: PList;

	State: l_uchar;         { What i am doing ? }
	aType: byte;            { App or Lib ?      }
	ThreadId: Integer;      { If App, Thread Id }
	
	(* ** Due to unknown threading bugs, this section is unactivated **
  EventsWaitList: PList; // If App, Events waiting for app widegts...
  *)

  	UID: l_uid;
  	Libs: Ptr_l_uid;
  	Users: l_ulong;

	Cons: Pointer;         {^TConsole -> Desktop }
	StdIn: PFileStruct;
	StdOut: PFileStruct;
	
{$ifdef ALLEGRO_H}
  	Icon: PIcon;
{$else}
	Icon: Pointer;  
{$endif}

	argc: Integer;
	argv: TArguments;

	Host: PApplication;
	
	
  end;

{#define APPLICATION(o) ((PApplication)(o))}


type
  PDynLdHeader = ^TDynLdHeader;
  TDynLdHeader = record

	Magic                  : l_ulong;
	FileFormatVersion      : l_ulong;
	SupportedApiVersion    : l_ulong;
	Time                   : l_ulong;

	Relocations            : l_ulong;
	Importations           : l_ulong;
	Size                   : l_ulong;

	//l_ulong	UID;
	UID                    : l_uid;
	FileVersion            : l_ulong;
	Flags                  : l_ulong;

	aType                  : byte;
	MainOffset             : l_ulong;
	CloseOffset            : l_ulong;
	LibsOffset             : l_ulong;

	Compression            : l_uchar;
	OriginalSize           : l_ulong;
	
	RessourceOffset        : l_ulong;
	RessourceEntries       : l_ulong;
	
  end;

type
  PDynLdReloc = ^TDynLdReloc;
  TDynLdReloc = record

	aType   : l_ulong;
	Address : l_ulong;
	Symbol  : l_ulong;
  end;

type
  PDynLdExt = ^TDynLdExt;
  TDynLdExt = record

	Name    : l_text;
	Symbol  : l_ulong;
  end; 

type 
  PSymbol = ^TSymbol;
  TSymbol = record
	Itm : TListItem;
	App : PApplication;
  end;

type
  PTransfer = ^TTransfer;
  TTransfer = record
    Filename: l_text;
    argc: Integer;
    argv: TArguments;
    Host: PApplication;
    h: PErrorHandler;
  end;

{#define SYMBOL(o) ((PSymbol)(o))}

const
  NoneOffset = $FFFFFFFF;

  REL32_ABSOLUTE = $01;
  REL32_RELATIVE = $02;


(**
*	DynLD system
*)
procedure InitDynLd;
procedure ShutDownDynLd;

(**
*	Event managment
*)

(* ** Due to unknown threading bugs, this section is unactivated **)
(*
procedure AppSendEvent ( App: PApplication; aType, Msg: l_ulong; Extra: Pointer; Dest: Pointer; Redirect: TRedirectProc );
{$ifdef LINK_CLIBS}external;{$endif}
function AppGetNextEvent ( App: PApplication; Ev: PDynLdEvent ): l_bool;
{$ifdef LINK_CLIBS}external;{$endif}
procedure AppRedirectEvent ( App: PApplication; Ev: PDynLdEvent );
{$ifdef LINK_CLIBS}external;{$endif}
procedure AppSendEventEv ( App: PApplication; Ev: TEvent; Dest: Pointer; Redirect: TRedirectProc );
{$ifdef LINK_CLIBS}external;{$endif}
*)


(**
*	System functions
*)
function DynLdRun ( Filename: l_text; Args: l_text ): PApplication;
{$ifdef LINK_CLIBS}external;{$endif}
function DynLdRun2 ( Filename: l_text; Args: l_text ): PApplication;
{$ifdef LINK_CLIBS}external;{$endif}
(*function Run_Ex( Filename:l_text; argc:Integer; argv:TArguments; Host:PApplication; h:PErrorHandler ): PApplication;*)
(*function DynLdRunEx1 ( Filename: l_text; argc: Integer; argv: TArguments; Host: PApplication; h: PErrorHandler ): PApplication;*)
(*function DynLdRunEx ( Filename: l_text; argc: Integer; argv: TArguments; Host: PApplication; h: PErrorHandler ): PApplication;*)
(*function ExecutePDAction2( a: PDTAction; f: PFileInfo; argc: integer; argv: TArguments; Host: PApplication; h: PErrorHandler ): l_bool;*)
{$ifdef LINK_CLIBS}external;{$endif}
procedure CloseApp ( App: PApplication );
{$ifdef LINK_CLIBS}external;{$endif}
function AddSymbol ( Name: l_text; Value: Pointer; App: PApplication ): PSymbol;
{$ifdef LINK_CLIBS}external;{$endif}
function ResolveSymbol ( App: PApplication; Name: l_text ): Longint;
{$ifdef LINK_CLIBS}external;{$endif}

(**
*	Export macros
*)

(*
#define APPEXPORT(Name)			AddSymbol(#Name, &Name, &Me)
#define APPEXPORTAS(Func, Name)	AddSymbol(Name, &Func, &Me)
#define SYSEXPORT(Name)			AddSymbol(#Name, (void * )&Name, NULL)
#define SYSEXPORTAS(Func, Name)	AddSymbol(Name, (void * )&Func, NULL)
*)

(**
*	Application handle
*)
var Me: TApplication;

(**
*	Export system
*)
(*procedure RegisterExports; {$ifdef LINK_CLIBS}external;{$endif}*)


function DynLdInstallLibrary ( Filename: l_text ): l_bool;
{$ifdef LINK_CLIBS}external;{$endif}

procedure _ParseArgs ( Args, Initial: l_text; var argc: Integer; var argv: TArguments );
{$ifdef LINK_CLIBS}external;{$endif}
function DuplicateArgs ( argc: Integer; argv: TArguments ): TArguments;
{$ifdef LINK_CLIBS}external;{$endif}
procedure FreeArgs ( argc: Integer; argv: TArguments );
{$ifdef LINK_CLIBS}external;{$endif}

procedure BreakApp ( App: PApplication );
{$ifdef LINK_CLIBS}external;{$endif}
function DynLdCommand ( Command: l_text ): PApplication;
{$ifdef LINK_CLIBS}external;{$endif}

procedure RegistryLoadLibraries ( Name: l_text );
{$ifdef LINK_CLIBS}external;{$endif}


type
  TAppExtInit = procedure(App: PApplication);
  TAppExtDone = procedure(App: PApplication);
  PAppExtender = ^TAppExtender;
  TAppExtender = record
	
	Init: TAppExtInit;
	
	DeInit: TAppExtDone;
	
  end;

function InstallAppExtender ( Init: TAppExtInit; DeInit: TAppExtDone ): PAppExtender;
{$ifdef LINK_CLIBS}external;{$endif}
procedure RemoveAppExtender ( o: PAppExtender );
{$ifdef LINK_CLIBS}external;{$endif}

(**
*       Global variables
*)
var
  Symbols      : PList;
  Applications : PList;
  AppExtenders : PList;
  Application  : PApplication;
  Libraries    : PApplication;

function getargs(from: l_text): TArguments;

var MainPoint: jmp_buf;

(*procedure _RunExDynld_(var App: PApplication; Filename: l_text; argc: Integer; argv: TArguments; Host: PApplication; h: PErrorHandler);*)

implementation

{$ifndef LINK_CLIBS}

uses vfile, debug, dms, tasks, fms2;

{
 .#define  KERNEL_FILE
 .#include "kernel.h"
 .#include "string.h"
}

(* function UnLoadLibraries ( var Id: l_uid ): l_bool; *)

(**
*       Kernel indetification
*)
(*TApplication Me =

        "Codename: Phoenix Kernel (update)",
        "",
        ULONG_ID(0,5,8,0),
        NULL,
        NULL,
        NULL,
        NULL,
        0,
        0,
        0,
        "kernel",
        0,
        1
;*)

{ below is better/right? I dont know... Florian Xaver  }

(**
*       NAME: AddSymbol
*       DESCRIPTION: Used by the macros to add or export a symbol
*       RETURN: The exported symbol
*)
function AddSymbol ( Name: l_text; Value: Pointer; App: PApplication ): PSymbol;
var
  o: PSymbol;
begin
   if ( o = PSymbol(ListKeyItem(Symbols, Name)) ) then
   begin
      DebugWarning('Duplicate symbol: %s orignaly registered by %x', [Name, o^.App]);
      AddSymbol := NIL;
   end else
   begin
        o := NEW(PSymbol);

        if ( (o=NIL) or (Name='') ) then AddSymbol := NIL else
        begin
        fillchar(o, sizeof(TSymbol), 0);
        o^.Itm.Key              := Name;
        o^.Itm.Data             := Value;
        o^.Itm.FreeData         := NIL;
        o^.App                  := App;

        ListAddItem(Symbols, PListItem(o));
        end;
        AddSymbol := o;
   end;
end;

(**
*       NAME: ResolveSymbol
*       DESCRIPTION: Resolves the symbol in a DynLD app
*       RETURN: The address of the symbol
*)
function ResolveSymbol ( App: PApplication; Name: l_text ): Longint;
var Address: Longint;
begin
        Address := 0;

        if ( Name = 'Me' ) then
                Address := Longint(App)
        else
                Address := Longint(ListKey(Symbols, Name));

        if not Assigned( Pointer(Address) ) then
        begin
                DebugError ('0x%08x :: Unresolved external %s', [App, Name]);
        end;

        ResolveSymbol := Address;
end;

procedure FreeApplication ( var o: Pointer );
var App: PApplication;
begin
        App := PApplication(o);

        DebugMessage('Free Application :: %s',[App^.Name]);

        if Assigned( App^.Close ) then App^.Close;
        //if Assigned( App^.ThreadId ) then KillThread(App^.ThreadId);
       {
        // in Pascal not nessessary
        if ( App->Name )        free(App->Name);
        if ( App->FileName )    free(App->FileName);
       } 
        if assigned( App^.Ressource ) then  FreeList(App^.Ressource);
        if assigned( App^.Icon )      then  {FreeIcon(App^.Icon);} App^.Icon := nil;
        if assigned( App^.argv)       then  FreeArgs(App^.argc,App^.argv);
       { free(App^.Data); }

(* ** Due to unknown threading bugs, this section is unactivated **

        FreeList(App^.EventsWaitList);

        *)
        Dispose(App);
end;
(*
////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
*)
procedure FreeAppExt(p: Pointer);
begin
  Dispose(PAppExtender(p));
end;

function InstallAppExtender ( Init: TAppExtInit; DeInit: TAppExtDone ): PAppExtender;
var o: PAppExtender; itm: PListItem;
begin
  o := NIL;
  New(o);
  if Assigned( o ) then
        begin
          o^.Init := Init;
          o^.DeInit := DeInit;
          itm := ListAdd(AppExtenders,'',o,nil(*{$ifdef fpc}@{$endif}FreeAppExt*));
          //itm^.FreeData := {$ifdef fpc}@{$endif}FreeAppExt;
        end;
  InstallAppExtender := o;
end;
(*
////////////////////////////////////////////////////////////////////////////////
*)
procedure RemoveAppExtender (o: PAppExtender);
var a: PListItem;
begin
        a := ListFoundItem(AppExtenders,o);
        if Assigned( a ) then ListRemoveItem(AppExtenders,a);
end;
(*
////////////////////////////////////////////////////////////////////////////////
*)
procedure AppExtendersInit ( App: PApplication );
var a,b: PListItem; o: PAppExtender;
begin
        if Assigned ( AppExtenders^.Last ) then
        begin
                a := AppExtenders^.Last^.Next;
                b := AppExtenders^.Last^.Next;
                repeat
                        o := PAppExtender(a^.Data);
                        if Assigned( o^.Init ) then o^.Init(App);
                        a := a^.Next;
                until ( a = b );
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure AppExtendersDeInit ( App: PApplication );
var a,b: PListItem; o: PAppExtender;
begin
        if  assigned( AppExtenders^.Last ) then
        begin
                a := AppExtenders^.Last;
                b := AppExtenders^.Last;
                repeat
                        o := PAppExtender(a^.Data);
                        if Assigned( o^.DeInit ) then o^.DeInit(App);
                        a := a^.Prev;
                until ( a = b );
        end;
end;
(*
////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////
*)
function GetLibrary ( id: l_uid ): PApplication;
var a,b: PListItem; App: PApplication;
begin
        if Assigned( Applications^.Last ) then
        begin
                a := Applications^.Last;                                                          
                
                b := a;
                
                repeat
                        App := PApplication(a^.Data);
                        if ( TextCompare(l_text(App^.UID),l_text(id)) = 0 ) then
                                GetLibrary := App
                        else a := a^.Next;
                until ( a = b );
        end else GetLibrary := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function LoadLibrary ( id: l_uid ): l_bool;
var App: PApplication; Key, filename: l_text;
begin        
        app := GetLibrary(id);
        if Assigned( app ) then
        begin
                Key := TextArgs('/SYSTEM/LIBRARIES/%s',[l_text('id')]);
                if ( key <> '' ) then
                begin
                        LoadLibrary := true;
                        filename := App^.FileName;{KeyGetText(key,NIL);}
                        Key := '';
                        if ( filename = '' ) then
                        begin
                                filename := TextArgs('/system/xlib/%s.dl',[l_text(id)]);
                                if ( filename = '' ) then LoadLibrary := false;
                        end else
                        app := DynLdRun(filename,'');
                        if not Assigned( app ) then LoadLibrary := false;
                end else
                        LoadLibrary := false;
        end else
            begin Inc(app^.Users); if Libraries <> NIL then Inc(Libraries^.Users) else Libraries := App; end;
        
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function UnLoadLibrary ( id: l_uid ): l_bool;
var App: PApplication;
begin
        app := GetLibrary(id);

        if not assigned( app ) then UnloadLibrary := false else
        begin
        Dec(app^.Users);
        Dec(Libraries^.Users);
        if ( Libraries{app}^.Users = 0 ) then CloseApp(app);

        UnLoadLibrary := true;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function LoadLibraries ( ids: l_uid ): l_bool;
var ix: Integer;
begin
        ix := 0;
        while ( ids[ix] <> ' ' ) and (ix < 13) do
        begin
                if ( not LoadLibrary(ids) ) then  LoadLibraries := false;
                Inc(ix);
        end;
        LoadLibraries := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function UnLoadLibraries ( var ids: l_uid ): l_bool;
var ix: Integer;
begin
        ix := 0;
        while ( ids[ix] <> ' ' ) and (ix < 13) do
        begin
                if ( not UnLoadLibrary(ids)) then DebugWarning('Unable to unload %s',[ids]);
                Inc(ix);
        end;
        UnLoadLibraries := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeHostedApp ( App: PApplication );
var 
  A: PApplication;
  l: PListItem;
  n: PListItem;
begin
        if Assigned ( Applications^.Last ) then
        begin
                l := Applications^.Last;
                repeat
                
                        n := l^.Next;
                        A := PApplication(l^.Data);
                        if ( A^.Host = App ) then
                        begin
                                FreeHostedApp(A);
                                ListRemoveItem(Applications, l);
                        end;
                        l := n;
                until ( l = Applications^.Last );

        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*
function DynLdRunEx ( Filename: l_text; argc: Integer; argv: TArguments; Host: PApplication; h: PErrorHandler; xarg: l_bool = false ): PApplication;
begin

end;
*)
function DynLdRunEx( Filename:l_text; argc:Integer; argv:TArguments; Host:PApplication; h:PErrorHandler ): PApplication;
var
        App     : PApplication;
        TempData: Pointer;
        Ext     : PDynLdExt;
        i       : l_ulong;
        DynERROR: l_ulong;
        DynReloc: TDynLdReloc;
        DynHead : TDynLdHeader;
        itm     : PListItem;
        f       : PFileStruct;
        j, addr : l_ulong; 
        i16     : PImage;
        i32     : PImage;
        i48     : PImage;
        argvstr : l_Text;
        argvaa  : {$ifdef OPascal}array of l_text{$else}PDynamicArray{$endif};
begin
        App := NIL; TempData := NIL; Ext := NIL; i:=0; DynError := 0;
        {$ifdef OPascal}
        if (argc = 0) or (argv[0] = '') then
        {$else}
        if (argc = 0) or ( (argv = NIL) and (argv^[0]^ = '') ) then
        {$endif}
        begin
                DebugError('DYNLD :: %s : argv is NULL !', [Filename]);

                DynLdRunEx := NIL;

                exit;
        end;


        (*
        *       Open application file
        *)

        f := FileOpen(Filename, FSF_READ);

        if not Assigned( f ) then
        begin
                {CreateError(h, ERROR_FILENOTFOUND,'File not found');}
                DebugError('DYNLD :: File not found - %s', [Filename]);
                DynLdRunEx := NIL;
                exit;
        end;

        FileRead(DynHead, 1, sizeof(TDynLdHeader), f);

        if ( DynHead.Magic <> ULONG_ID('D','n','L','d') ) then
        begin
                FileClose(f);
                {CreateError(h,ERROR_INVALIDFILE,'Application is not a valid DynLD');}
                DebugError('DYNLD :: Application is not a valid DynLD.',[]);

                DynLdRunEx := NIL;
                exit;
        end;

        if ( DynHead.FileFormatVersion <> DYNLDVERSION ) then
        begin
                FileClose(f);
                {CreateError(h,ERROR_INVALIDFILE,'Application has invalid version number of DynLD linker');}
                DebugError('DYNLD :: Application has invalid version number of DynLD linker.', []);

                DynLdRunEx := NIL;
                exit;
        end;

        New(App); { => (PApplication)malloc(sizeof(TApplication)); }

        if not assigned( App ) then
        begin
                FileClose(f);
                {CreateError(h,ERROR_NOTENOUGHMEMORY,'');}
                DebugError('DYNLD :: Not enough memory to load application.',[]);
                DynLdRunEx := NIL;
                exit;
        end;

        fillchar(App, sizeof(TApplication), 0);

        App^.Name           := FileReadBinString(f);
        App^.FileName       := Filename;
        move(DynHead.UID,App^.UID,sizeof(l_uid));
        App^.Version        := DynHead.FileVersion;
        
        (* ** Due to unknown threading bugs, this section is unactivated **

        App^.EventsWaitList := NewList;
        *)
        
        App^.aType          := DynHead.aType;
        App^.Host           := Host;


        DebugMessage ('Loading application %s (%x), FileName - %s',[App^.Name,App,App^.FileName]);
        DebugMessage ('         UID - %s Version - 0x%08x',[App^.UID,App^.Version]);

        (*if ( DynHead.Compression <> 0) then
        begin
                getmem(TempData, DynHead.Size);
                FileRead(TempData, 1, DynHead.Size, f);

                getmem(App^.Data, DynHead.OriginalSize);
                uncompress(App^.Data, DynHead.OriginalSize, TempData, DynHead.Size);
                freemem(TempData, DynHead.Size);
                DynHead.Size := DynHead.OriginalSize;
        end
        else
        begin*)
                getmem(App^.Data, DynHead.Size);
                FileRead(App^.Data^, 1, DynHead.Size, f);
        //end;

        if ( DynHead.MainOffset  <> NoneOffset ) then Pointer(App^.Main)  := {(void * )(((long)App->Data)+DynHead.MainOffset);}
           Pointer(Longint(App^.Data)+DynHead.MainOffset);
        if ( DynHead.CloseOffset <> NoneOffset ) then Pointer(App^.Close) := {(void * )(((long)App->Data)+DynHead.CloseOffset);}
           Pointer(Longint(App^.Data)+DynHead.CloseOffset);
        if ( DynHead.LibsOffset  <> NoneOffset ) then App^.Libs  := {(void * )(((long)App->Data)+DynHead.LibsOffset);}
           Ptr_l_uid(Longint(App^.Data)+DynHead.LibsOffset);


        if Assigned( App^.Libs ) then
                if ( not LoadLibraries(App^.Libs^) ) then
                begin
                        freemem(App^.Data, DynHead.Size);
                        Dispose(App);
                        FileClose(f);
                        DynLdRunEx := NIL;
                        exit;
                end;


        getmem(Ext, sizeof(TDynLdExt) * DynHead.Importations);


        for i := 0 to DynHead.Importations - 1 do
        begin
                Ext[i].Name := FileReadBinString(f);
                FileRead(Ext[i].Symbol, 1, 4, f);
                if ( ResolveSymbol(App,Ext[i].Name) = 0 ) then
                begin
                        {CreateError(h,ERROR_INVALIDFILE,'Unresolved extrenal');}
                        DebugError('%s not found\n',[Ext[i].Name,h]);
                        Inc(DynERROR);
                end;
        end;

        if ( DynERROR > 0 ) then
        begin
                App^.Close := NIL;
                DebugError ('0x%08x :: %d error(s) while loading. Application has not started', [App, DynERROR]);
                FreeApplication(App);
                DynLdRunEx := NIL;
                exit;
        end;

        for i := 0 to DynHead.Relocations - 1 do
        begin
                FileRead(DynReloc, 1, sizeof(TDynLdReloc), f);
                if ( DynReloc.aType = REL32_ABSOLUTE ) or ( DynReloc.aType = REL32_RELATIVE ) then
                begin
                        addr := l_ulong(App^.Data);

                        for j := 0 to DynHead.Importations - 1 do
                                if (Ext[j].Symbol = DynReloc.Symbol) then
                                begin
                                        addr := ResolveSymbol(App,Ext[j].Name);
                                        break;
                                end;

                        if ( DynReloc.aType = REL32_ABSOLUTE ) then
                        begin
                                { *(long*)(((((long)App^.Data))+DynReloc.Address)) += (long)addr; }
                                Longint(App^.Data) := Longint(App^.Data) + DynReloc.Address;
                                Longint(App^.Data) := Longint(App^.Data) + DynReloc.Address + Longint(addr);
                        end
                        else if ( DynReloc.aType = REL32_RELATIVE ) then
                        begin
                                { *(long*)(((((long)App->Data))+DynReloc.Address)) += (long)addr-(long)App->Data-(long)DynReloc.Address; }
                                Longint(App^.Data) := Longint(App^.Data) + DynReloc.Address;
                                Longint(App^.Data) := Longint(addr) - Longint(App^.Data) - Longint(DynReloc.Address);
                        end;

                end else begin
                        {CreateError(h,ERROR_INVALIDFILE,'Unknown reloaction type');}
                        DebugError('DYNLD : Unknown reloaction type',[]);
                        Inc(DynERROR);

                end;
        end;

        if ( DynHead.RessourceEntries > 0 ) then
                App^.Ressource := DMSLoadRessource(f,DynHead.RessourceEntries);


        if Assigned( App^.Ressource ) then
        begin        
                i16 := ListKeyCase(App^.Ressource,'ICON16');
                i32 := ListKeyCase(App^.Ressource,'ICON32');
                i48 := ListKeyCase(App^.Ressource,'ICON48');
                if ( Assigned(i16) or Assigned(i32) or Assigned(i48) ) then
                        App^.Icon := NewIcon2(i16,i32,i48);
        end;

        FileClose(f);

        {for (i:=0 to DynHead.Importations - 1 do free(Ext[i].Name);}

        freemem(Ext, sizeof(TDynLdExt) * DynHead.Importations);

        App^.argc := argc;
        App^.argv := DuplicateArgs(argc,argv);

        if not Assigned( App^.argv ) then
        begin
                DebugError('0x%08x :: argv is null',[App]);
                {CreateError(h,ERROR_NOTENOUGHMEMORY,'argv is null');}
                Inc(DynERROR);
        end;

        if ( DynERROR > 0 ) then
        begin
                App^.Close := NIL;
                DebugError ('0x%08x :: %d error(s) while loading. Application has not started', [App, DynERROR]);
                FreeApplication(App);
                DynLdRunEx := NIL;
        end;

        (* ** Due to unknown threading bugs, this section is unactivated **

        if ( App^.aType = DYNLD_TYPEAPP ) then
        begin
                { Stack = 10k }
                App^.ThreadId := NewThread(App^.Main,Args,10240,1);
                
                {DebugMessage ("         Thread - 0x%08x",App->ThreadId);}

                while ( App^.State = DynLdStateStarting ) do Yield;

                {DebugMessage ("         Started!");}
        end
        else
        {*)


        DebugMessage('AppExtendersInit',[]);

        AppExtendersInit(App);


        DebugMessage('Adding app to list',[]);
        ListAdd(Applications,'',App,nil(*FreeApplication*));

        DebugMessage('Calling @App^.Main(Args)',[]);
        App^.State := DYNLD_STRATING;
        if assigned( App^.Main ) then
        begin
           {$ifdef OPascal}
           SetLength(argvaa, App^.argc);
           move(App^.argv, argvaa, App^.argc);
           {$else}
           argvaa := MakeArray(argvaa, App^.argc);
           move(App^.argc^[0], argvaa^[0], App^.argc);
           {$endif}
           argvstr := '';
           for i:=0 to App^.argc-1 do argvstr := argvstr + App^.argv[i] + ',';
           if argvstr[length(argvstr)] = ',' then system.delete(argvstr,Length(argvstr),1);
           if not Assigned( App^.Main{(App^.argc, argvstr)} ) then
           begin
                   App^.Close := NIL;
                   CloseApp(App);
                   if assigned( App^.Libs ) then UnLoadLibraries(App^.Libs^);
                   if ( itm = ListFoundItem(Applications,App) ) then
                           ListRemoveItem(Applications, itm)
                   else begin
                           DebugWarning('Application was not in list',[]);
                           FreeApplication(App);
                   end;
                   DynLdRunEx := NIL;
           end;
        end;
        App^.State := DYNLD_RUNNING;

        App^.Users := 1;


        DynLdRunEx := App;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function Run_Ex( Filename:l_text; argc:Integer; argv:TArguments; Host:PApplication; h:PErrorHandler ): PApplication;
begin
  Run_Ex := DynLdRunEx(Filename,argc,argv,Host,h);
end;

procedure _RunExDynld_(var App: PApplication; Filename: l_text; argc: Integer; argv: TArguments; Host: PApplication; h: PErrorHandler);
begin

end;

function ExecutableRun ( f: PFileInfo; argc: integer; argv: TArguments; Host: PApplication; h: PErrorHandler ): l_bool;
var
	r: l_bool; App: PApplication;
begin
   (*
	int nargc = argc+1;
	l_text *nargv = malloc((nargc+1)*sizeof(l_text));
	nargv[0] = TextDup(f^.VPathName);
	while ( argc ) {
		nargv[argc] = TextDup(argv[argc-1]);
		argc--;
	}
	nargv[nargc] = 0;
   *)
	App := DynLdRunEx(f^.VPathName,argc,argv,Host,h);
	{ FreeArgs(nargc,nargv); }
	ExecutableRun := assigned(App);
end;

function getargs(from: l_text): TArguments;
var
  {$ifdef OPascal}
  args : array of l_text;
  {$else}
  args : PDynamicArray;
  {$endif}
  index: Integer;
  count: Integer;
  cmark: Integer;
  strg : l_text;
begin
  strg  := '';
  index := 1; count := 0;
  while (Length(from) > 0) and ((from[1] = ' ') or (from[1] = #9)) do System.Delete(from,1,1);
  while index <= Length(from) do
  begin
    if (from[index] <> ' ') and (from[index] <> #9) then strg := strg + from[index] else begin Inc(count); Strg := '' end; 
    Inc(index);
  end;
  cmark := Count;
  cmark := 0;
  strg := '';
  {$ifdef OPascal}
  SetLength(args, (Count + 1) * sizeof(l_text));
  {$else}
  args := MakeArray(args, (count + 1) * Sizeof(Pointer));
  {$endif}
  while index <= Length(from) do
  begin
    if (from[index] <> ' ') and (from[index] <> #9) then strg := strg + from[index] else
    begin
      {$ifdef OPascal}
      args[cmark] := strg; strg := ''; inc(cmark);
      {$else}
      args^[cmark] := NewStr(strg); strg := ''; inc(cmark);
      {$endif}
    end;
    inc(index);
  end;
  {$ifdef OPascal}
  args[count] := #0
  {$else}
  args^[count] := NewStr(#0);
  {$endif}
end;

function DynLdRun ( Filename: l_text; Args: l_text ): PApplication;
var 
  argv: TArguments;
  argx: TArguments;
  argp: Pointer;
  App : PApplication;
  argc: Integer;
begin
  {l_int argc = Args ? 2 : 1;}
  {l_text *argv = malloc(sizeof(l_text)*(argc+1));}

  {$ifdef OPascal}
    SetLength(argv, succ(App^.argc)*sizeof(l_text));
    SetLength(argx, succ(App^.argc)*sizeof(l_text));
  {$else}
    argv := MakeArray(argv,succ(App^.argc)*sizeof(l_text)); 
    argx := MakeArray(argx,succ(App^.argc)*sizeof(l_text)); 
  {$endif} 
  
    argv := getargs(Args);
    
    {for argc := 0 to App^.argc - 1 do argv[argc] := argx[argc];}
    
    App := DynLdRunEx(Filename,argc,argv,NIL,NIL);

    FreeArgs(argc,argv);

    DynLdRun := App;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DynLdRun2 ( Filename, Args: l_text ): PApplication;
var
  App : PApplication;
  argc: Integer;
  argv: TArguments; 
begin
        _ParseArgs(Args,Filename,argc,argv);
        App := DynLdRunEx(Filename,argc,argv,NIL,NIL);
        FreeArgs(argc,argv);
        DynLdRun2 := App;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DynLdCommand ( Command: l_text ): PApplication;
var
  App: PApplication;
  argc: Integer;
  argv: TArguments;
begin
   App := NIL; argc := 0; argv := NIL;
  _ParseArgs(Command,'',argc,argv);
   if assigned( argv ) then
   begin
     App := DynLdRunEx(argv[0],argc,argv,NIL,NIL);
     FreeArgs(argc,argv);
   end;
   DynLdCommand := App;
end;
(*////////////////////////////////////////////////////////////////////////////////*)

(* ** Due to unknown threading bugs, this section is unactivated **

procedure AppSendEvent ( App: PApplication; aType, Msg: l_ulong; Extra: Pointer; var Dest: Pointer; Redirect: TRedirctProc );
var Ev: PDynLdEvent;
begin
        New(Ev); { = (PDynLdEvent)malloc(sizeof(TDynLdEvent)); }

        Ev^.Ev.aType    := aType;
        Ev^.Ev.Message  := Msg;
        Ev^.Ev.Extra    := Extra;
        Ev^.Dest        := Dest;
        Ev^.Redirect    := Redirect;

        ListAdd(App^.EventsWaitList,NIL,Ev,free);
end;

procedure AppSendEventEv ( App: PApplication; Ev: TEvent; Dest: Pointer; Redirect: TRedirectProc );
var Evt: PDynLdEvent;
begin
        New(Evt); { = (PDynLdEvent)malloc(sizeof(TDynLdEvent)); }

        Evt^.Ev  := Ev;

        Evt^.Dest     := Dest;
        Evt^.Redirect := Redirect;

        ListAdd(App^.EventsWaitList,NIL,Evt,free);
end;

function AppGetNextEvent ( App: PApplication; Ev: PDynLdEvent ): l_bool;
begin
        while not Assigned(App^.EventsWaitList^.Last) do Yield;

        move(PDynLdEvent(App^.EventsWaitList^.Last^.Next^.Data)^, Ev, sizeof(TDynLdEvent));

        ListRemoveItem(App^.EventsWaitList, App^.EventsWaitList^.Last^.Next);

        if ( Ev^.Ev.Message = MSG_KILLAPP ) and ( Ev^.Ev.Type = EV_MESSAGE ) then AppGetNextEvent := false; else

        AppGetNextEvent := true;
end;

procedure AppRedirectEvent ( App: PApplication; Ev: PDynLdEvent );
begin
        if assigned( Ev^.Redirect ) Ev^.Redirect(Ev^.Dest, Ev);
end;*)
(*////////////////////////////////////////////////////////////////////////////////*)
procedure BreakApp ( App: PApplication );
var itm: PListItem; temp: Pointer;
begin
        DebugMessage ('Break :: %s (0x%08x)',[App^.Name,App]);
        DebugMessage ('Try to call close function.',[]);
        CloseApp(App);
        DebugWarning('Closed.',[]);
        if assigned( App^.Libs ) then UnLoadLibraries(App^.Libs^);
        itm := ListFoundItem(Applications,Pointer(App));
        if assigned( itm ) then
                ListRemoveItem(Applications, itm)
        else begin
                DebugWarning('Application was not in list',[]);
                FreeApplication(App);
        end;
        longjmp(MainPoint, 1);
        DebugWarning('Should never came here, calling object may no more exists',[]);
end;

function DynLdRunEx1(Filename: l_text; argc: Integer; argv: TArguments;
  Host: PApplication; h: PErrorHandler): PApplication;
begin
  DynLdRunEx1 := DynLdRunEx(Filename,argc,argv,Host,h);
end;

(*////////////////////////////////////////////////////////////////////////////////*)
procedure CloseApp ( App: PApplication );
var a,b,n: PListItem;
begin
        DebugMessage ('Close :: %s',[App^.Name]);

        FreeHostedApp(App);

        if Assigned( App^.Close ) then
        begin

                App^.Close;
                App^.Close := NIL;
        end;
        DebugMessage('AppExtendersDeInit',[]);

        AppExtendersDeInit(App);


        if Assigned( Symbols^.Last ) then
        begin

                a := Symbols^.Last^.Next;
                b := Symbols^.Last^.Next;

                repeat
                        
                                n := a^.Next;

                                if ( PSymbol(a)^.App = App ) then
                                begin
                                        ListRemoveItem(Symbols, a);
                                        if ( a = b ) then b := Symbols^.Last^.Next;
                                end;

                                a := n;
                until ( a = b );
        end;


        App^.State := DynLdStateToFree; { App will be free by DYNLDTCP Thread }


end;
(*////////////////////////////////////////////////////////////////////////////////*)
const DynLdTCPId: PTask = NIL;

var GSExit: l_bool;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DynLdTCP ( o: PTask );
var a,n: PListItem; App: PApplication;
begin

        if Assigned( Applications^.Last ) then
        begin
                a := Applications^.Last;
                
                repeat
                
                        n := a^.Next;
                        App := PApplication(a^.Data);
                        if ( App^.State = DynLdStateToFree ) then
                        begin
                                if assigned( App^.Libs ) then UnLoadLibraries(App^.Libs^);
                                ListRemoveItem(Applications, a);

                        end;
                        if ( (App^.State = DYNLD_STRATING) and GSExit ) then begin
                                BreakApp(App);
                        end;

                        a := n;
                until ( a = Applications^.Last );
        end;




end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DynLdInstallLibrary ( filename: l_text ): l_bool;
var
  key,n: l_text;
  DynHead: TDynLdHeader;
  f: PFileStruct;
begin   
        f := FileOpen(filename, FSF_READ);

        if not assigned( f ) then
        begin
                DebugError('DYNLD :: File not found - %s', [filename]);
                DynLdInstallLibrary := false;
        end;

        FileRead(DynHead, 1, sizeof(TDynLdHeader), f);
        if ( DynHead.Magic <> ULONG_ID('D','n','L','d') ) then
        begin
                FileClose(f);
                DebugError('DYNLD :: Library is not a valid DynLD. Library not installed.',[]);
                DynLdInstallLibrary := false;
        end;

        if ( DynHead.FileFormatVersion > DYNLDVERSION ) then
        begin
                FileClose(f);
                DebugError('DYNLD :: Library has invalid version number of DynLD linker. Library not installed.',[]);
                DynLdInstallLibrary := false;
        end;

        FileClose(f);

        key := TextArgs('/SYSTEM/LIBRARIES/%s',[DynHead.UID]);
        if ( key = '' ) then DynLdInstallLibrary := false;
        n       := TextArgs('%s',[DynHead.UID]);
        if ( n='' ) then DynLdInstallLibrary := false;
        (*
        NewKey('/SYSTEM/LIBRARIES',n);

        KeySetText(key,filename);
        *)
        DynLdInstallLibrary := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure _ParseArgs ( Args, Initial: l_text; var argc: Integer; var argv: TArguments );
var
  n,g,i: Integer; p: Integer;
  Buffer: array[0..255] of l_text;
begin                                   
        p := 1;
        n := 0; 
        g := 0;
        i := 1;
        if ( Initial <> '' ) then while i <= Length(Initial) do
              begin  Buffer[n] := Initial[i]; inc(n); inc(i); end;
        
        n := 0; i := 1;
        if ( Args <> '' ) then
        begin
                while i <= Length(Args) do
                begin
                        if (i<=Length(Args)) and (Args[i]='"') then
                        begin
                           Buffer[n] := args[i]; inc(n); 
                                if ( g > 0 ) then if ( Args[i] <> ' ' )  then Buffer[n] := Args[i];
                                p := p+1;
                                g := 1;
                        end;

                        if (i<=Length(Args)) and (Args[i]=' ') and ( g = 0 ) then
                        begin
                                if {((i-p)>=1) and} (Args[i{-p}]=' ') then Buffer[n] := Args[i{-p}];
                                p := p+1;
                        end;
                        {Args++;}
                        inc(i); 
                end;

                {if ( Args-p ) Buffer[n++] = TextNDup(p,Args-p);}
        end;
        
        if ( argc > 0 ) and (assigned (argv)) then
        begin
                Buffer[Length(Initial)] := #0;
                argc := Length(Initial);
                {$ifdef OPascal}
                  SetLength(argv, (n+1) * sizeof(l_text));
                  move(Buffer, argv, (n+1)*sizeof(l_text));
                {$else}
                  argv := MakeArray(argv, (n+1) * sizeof(l_text));
                  move(Buffer, argv^, (n+1)*sizeof(l_text));
                {$endif}
                { * argv = malloc((n+1) * sizeof(l_text));}
                
        end;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DuplicateArgs (  argc: Integer; argv: TArguments ): TArguments;
var n: Integer; nargv: TArguments;
begin
   n := 0;
        
   {l_text *nargv = malloc(sizeof(l_text)*(argc+1));}
   
   {$ifdef OPascal}
      SetLength(nargv, succ(argc) * Sizeof(l_text));
      if not Assigned( nargv ) then DuplicateArgs := NIL else
      while ( n < argc ) do
      begin
         nargv[n] := argv[n];
         inc(n);
      end;
   {$else}
      nargv := MakeArray(nargv, succ(argc) * sizeof(Pointer));
      if not Assigned( nargv ) then DuplicateArgs := NIL else
      while ( n < argc ) do
      begin
         nargv^[n] := NewStr(argv[n]);
         inc(n);
      end;
   {$endif}    

   nargv[argc] := {$ifdef OPascal}#0{$else}NewStr(#0){$endif};

   DuplicateArgs := nargv;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeArgs (  argc: Integer; argv: TArguments );
{$ifdef Opascal} var n: Integer; {$endif}
begin
  {$ifdef OPascal}
   SetLength(argv, 0);
  {$else}
   n := 0;
   while ( n < argc ) do
   begin
      DisposeStr(PString(argv^[n]));
      inc(n);
   end;
   Dispose(argv);
  {$endif}
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure RegistryLoadLibraries ( Name: l_text );
{var o,a,b: PRegKey; id: l_uid; i: Integer;}
begin
  (*
        o := ResolveKey(Name);
        if assigned(o) then
        begin
          DebugMessage('Run registry entries form %s',[Name]);
          if assigned(o^.Last) then
          begin
                a := o^.Last^.Next;
                b := a;
          
                repeat
                   fillchar(id,sizeof(l_uid),0);
                   {TextCopy(l_text(id),a^.Name);}
                   i := 1;
                   {move(a^.Name[1], id, sizeof(id)-1);}
                   while (i < Length(a^.Name)) and (i < sizeof(l_uid)) do begin id[i] := a^.Name[i]; inc(i) end;
                   LoadLibrary(id);
                   a := a^.Next;
                until ( a = b );
          end;
        end;
        *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure InitDynLd;
begin
        Symbols := NewList;

        if not assigned( Symbols ) then DebugFatal('DynLd::ERROR_NOTENOUGHMEMORY',[]);

        Applications := NewList;

        if not assigned( Applications ) then DebugFatal('DynLd::ERROR_NOTENOUGHMEMORY',[]);

        AppExtenders := NewList;

        if not assigned( AppExtenders ) then DebugFatal('DynLd::ERROR_NOTENOUGHMEMORY',[]);
       (*
        SYSEXPORT(AddSymbol);
        SYSEXPORT(CloseApp);
        SYSEXPORT(ResolveSymbol);
        SYSEXPORT(DynLdInstallLibrary);

        SYSEXPORT(InstallAppExtender);
        SYSEXPORT(RemoveAppExtender);

        SYSEXPORT(_ParseArgs);
        SYSEXPORT(FreeArgs);
        SYSEXPORT(BreakApp);

        SYSEXPORTAS (Applications, "DynLdApplications");
        SYSEXPORT(DynLdRun);
        SYSEXPORT(DynLdRun2);
        SYSEXPORT(DynLdRunEx);
        SYSEXPORT(DynLdCommand);

        /* ** Due to unknown threading bugs, this section is unactivated **

        SYSEXPORT(AppSendEvent);
        SYSEXPORT(AppSendEventEv);
        SYSEXPORT(AppGetNextEvent);
        SYSEXPORT(AppRedirectEvent);*/
        *)
        {DynLdTCPId := InstallTask(DynLdTCP);}

end;

procedure ShutDownDynLd;
var
  App: PApplication;
  a,b: PListItem;
begin
        DebugMessage ('Shutting down DynLd...',[]);

        RemoveTask(DynLdTCPId);


        if Assigned( Applications^.Last ) then
        begin
                a := Applications^.Last;
                b := a;

                repeat
                
                        App := PApplication(a^.Data);

                        (* ** Due to unknown threading bugs, this section is unactivated **

                        if ( App^.aType = DYNLD_TYPEAPP ) then
                        begin
                                DebugMessage('Kill thread :: 0x%08x : %s : %d', [App, App^.Name, App^.ThreadId]);
                                AppSendEvent(App, EV_MESSAGE, MSG_KILLAPP, NIL, NIL, NIL);

                                while ( App^.State <> DynLdStateDied ) do Yield;

                                KillThread(App^.ThreadId);

                                App^.ThreadId := 0;
                                App^.Close := NIL;
                        end;
                        else *)
                        if Assigned( App^.Close ) then
                        begin
                                DebugMessage ('Close :: %s',[App^.Name]);

                                App^.Close;
                                App^.Close := NIL;

                        end;
                        DebugMessage('AppExtendersDeInit',[]);
                        AppExtendersDeInit(App);

                        a := a^.Prev;
                until ( a = b );
        end;

        FreeList(Symbols);
        FreeList(Applications);
        FreeList(AppExtenders);

        DebugMessage ('DynLd Stopped',[]);
end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}

  Me.Name      := 'Codename: Phoenix Kernel (update)';
  Me.FileName  := '';
  Me.Version   := ULONG_ID(0,5,8,0);
  Me.Data      := NIL;
  Me.Main      := NIL;
  Me.Close     := NIL;
  Me.Ressource := NIL;
  Me.State     := 0;
  Me.aType     := 0;
  Me.ThreadId  := 0;
  {Me.UID       := (0,0,0,0,0,0,0,0,0,0,0,0,0);}
  {Me.libs      := 0;} {NIL ???}
  Me.Users     := 0;
  Me.Cons      := NIL;
  Me.StdIn     := NIL;
  Me.StdOut    := NIL;
  Me.Icon      := NIL;
  Me.argc      := 0;
  Me.argv      := NIL;
  Me.Host      := NIL;

  DYNLDVERSION := ULONG_ID(0,6,3,0);
  Symbols      := NIL;
  Applications := NIL;
  AppExtenders := NIL;
  
{$endif} {LINK_CLIBS}
end.
