(**
*       (c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
*
*       FILE:                   error.c
*
*       PROJECT:                Phoenix engine - Core
*
*       DESCRIPTION:    Error and exception handling functions
*
*       CONTRIBUTORS:
*                                       Lukas Lipka
*
*       TODO:                   @ Add ASM interupt reprogramming code.
*
*       BUG:                    @
*
*       MISTAKE:                @
*
*       IDEA:                   @
*)
{
 .#include "kernel.h"
 .#include "internal.h"
 .#include <string.h>
}
unit error;

interface

uses
  guitypes, list, Sysutils;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$LINK errors.o}
{$endif}


const
 ERROR_NOTENOUGHMEMORY	= $FF0FF001;
 ERROR_ENABLETOSTARTLWP = $FF0FF002;

 ERROR_FILENOTFOUND  	= $FF0FF003;
 ERROR_UNABLETOREAD  	= $FF0FF004;
 ERROR_UNABLETOWRITE  	= $FF0FF005;
 ERROR_INVALIDFILE  	= $FF0FF006;
 ERROR_CANTOPENTOWRITE  = $FF0FF007;

procedure ExceptionHandlerInstall;
procedure ExceptionHandlerUninstall;

type
  PError = ^TError;
  TError = record
	
	l: TListItem;
	
	ErrorCode: Longint;
	Description: l_text;
	Log: l_text;
	
  end;

{ #define ERROR(o) ((PError)(o)) }

type
  PErrorHandler = ^TErrorHandler;
    TErrorHandler = record	
    
	l: TList;
	e: TError;
	
  end;
  
procedure  EXCEPTION_isr0;
procedure  EXCEPTION_isr1;
procedure  EXCEPTION_isr2;
procedure  EXCEPTION_isr3;
procedure  EXCEPTION_isr4;
procedure  EXCEPTION_isr5;
procedure  EXCEPTION_isr6;
procedure  EXCEPTION_isr7;
procedure  EXCEPTION_isr8;
procedure  EXCEPTION_isr9;
procedure  EXCEPTION_isr10;
procedure  EXCEPTION_isr11;
procedure  EXCEPTION_isr12;
procedure  EXCEPTION_isr13;
procedure  EXCEPTION_isr14;
procedure  EXCEPTION_isr15;
procedure  EXCEPTION_isr16;

procedure CreateError ( h: PErrorHandler; ErrorCode: Longint; Description: l_text );
procedure ErrorLogEx ( h: PErrorHandler; txt: l_text );
procedure ErrorLog ( h: PErrorHandler; fmtxt: l_text; args: array of {$ifdef OPascal}const{$else}PString{$endif} );
function CreateErrorHandler: PErrorHandler;
procedure FreeErrorHandler(h: PErrorHandler);

function EHandlerIsError(h: PErrorHandler): l_bool;



implementation
{$ifndef LINK_CLIBS}
uses
  debug;

(**
*       NAME: ERROR_SystemHalt
*       DESCRIPTION: Halts the whole system, cannot be undone
*)
procedure  ERROR_SystemHalt;
begin
        writeln('[EXCEPTION] => SYSTEM HALTED!');
        DebugFatal('SYSTEM HALTED!',[]);
        Halt;
end;

(**
*       NAME: ERROR_SystemPause
*       DESCRIPTION: Pauses the whole system, while we handle the exception
*)
procedure  ERROR_SystemPause;
begin

end;

(**
*       NAME: ERROR_SystemResume
*       DESCRIPTION: Resumes the whole system from a previous pause
*)
procedure  ERROR_SystemResume;
begin

end;

(**
 *      Exception functions
*)
procedure  EXCEPTION_isr0;
begin
        DebugError('0 Division by zero',[]);
end;

procedure  EXCEPTION_isr1;
begin
        DebugError('1 Debug exception',[]);
end;

procedure  EXCEPTION_isr2;
begin
        DebugError('2 Intel reserved',[]);
end;

procedure  EXCEPTION_isr3;
begin
        DebugError('3 Breakpoint',[]);
end;

procedure  EXCEPTION_isr4;
begin
        DebugError('4 Overflow',[]);
end;

procedure  EXCEPTION_isr5;
begin
        DebugError('5 Bounds check',[]);
end;

procedure  EXCEPTION_isr6;
begin
        DebugError('6 Invalid OPCODE',[]);
end;

procedure  EXCEPTION_isr7;
begin
        DebugError('7 Coprocessor Not Available',[]);
end;

procedure  EXCEPTION_isr8;
begin
        DebugError('8 Double Fault',[]);
end;

procedure  EXCEPTION_isr9;
begin
        DebugError('9 Coprocessor Segment Overrun',[]);
end;

procedure  EXCEPTION_isr10;
begin
        DebugError('10 Invalid TSS',[]);
end;

procedure  EXCEPTION_isr11;
begin
        DebugError('11 Segment Not Present',[]);
end;

procedure  EXCEPTION_isr12;
begin
        DebugError('12 Stack Exception',[]);
end;

procedure  EXCEPTION_isr13;
begin
        DebugError('13 General Protection Exception (Triple Fault)',[]);
end;

procedure  EXCEPTION_isr14;
begin
        DebugError('14 Page Fault',[]);
end;

procedure  EXCEPTION_isr15;
begin
        DebugError('15 Intel reserved',[]);
end;

procedure  EXCEPTION_isr16;
begin
        DebugError('16 Coprocessor Error',[]);
end;

procedure CreateError ( h: PErrorHandler; ErrorCode: Longint; Description: l_text);
var e: PError;
begin
        if assigned( h ) then begin
                
                if assigned( h^.l.Last ) then
                begin
                        e := @h^.e;
                        if ( e^.ErrorCode = ErrorCode ) then ;
                end else 
                begin
                
                  New(e);
                
                  if assigned( e ) then
                  begin
                    fillchar(e,sizeof(TError),0);
                    e^.ErrorCode := ErrorCode;
                    e^.Description := Description;
                    h^.e := e^;
                    BListAddItem(@h^.l,@h^.e.l);
                  end;
                end;
        end;
end;

procedure ErrorLogEx ( h: PErrorHandler; txt: l_text ); 
var e: PError; l: Integer; Ex: l_text;
begin
        if assigned( h ) then
                if assigned( h^.l.Last ) then begin
                        e := @h^.e;
                        if ( e^.Log <> '' ) then begin
                                Ex := e^.Log + ' ' + txt;                                
                                e^.Log := Ex;
                        end else
                                e^.Log := txt;
                end;
end;

procedure ErrorLog ( h: PErrorHandler; fmtxt: l_text; args: array of {$ifdef OPascal}const{$else}PString{$endif} );
var fmtstr: String;
begin
        if assigned( h ) then
                if assigned( h^.l.Last ) then
                begin
                  {$ifdef OPascal}
                    writeln(Format(fmtxt, args));
                    ErrorLogEx(h,Format(fmtxt, args));
                  {$else}
                    FormatStr(fmtStr, fmtxt, args);
                    writeln(fmtstr);
                    ErrorLogEx(h,fmtstr);
                  {$endif}
                end;
end;

procedure FreeError ( o: PError ); 
begin
  {
  if ( o^.Description ) then DisposeStr(o^.Description);
  if ( o^.Log ) then DisposeStr(o^.Log);
  }
  dispose(o);
end;

function CreateErrorHandler: PErrorHandler; 
var o: PErrorHandler;
begin
    New(o);
    if not assigned( o ) then CreateErrorHandler := NIL else
    begin
      fillchar(o,sizeof(TErrorHandler),0);
      CreateErrorHandler := o;
    end;
end;

procedure FreeErrorHandler(h: PErrorHandler);
var a,b,n: PListItem;
begin
        if assigned( h^.l.Last ) then begin
                
                a := h^.l.Last;
                b := h^.l.Last;
                repeat
                        n := a^.Prev;
                        dispose(a);
                        a := n;
                until ( a = b );
        end;
        dispose(h);
end;

procedure  ExceptionHandlerInstall;
begin
        DebugMessage('Starting exception handler for exceptions (0-16)',[]);

        (**
        *       Reprogram int 8, for ISR 0-16
        *)
        DebugMessage('Reprogramming interrupt 8 => 0-16 ISR',[]);

        DebugMessage('Reprogramming interrupt successful',[]);
end;

procedure  ExceptionHandlerUninstall;
begin
        DebugMessage('Reprogramming interrupt 8 => NORMAL',[]);

        DebugMessage('Reprogramming interrupt successful',[]);
end;

function EHandlerIsError(h: PErrorHandler): l_bool;
begin
  EHandlerIsError := assigned(h^.l.last);
end;


{$endif}
end.
