(**
*	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
*
*	FILE:			events.c
*
*	PROJECT:		Phoenix engine - Core
*
*	DESCRIPTION:	Main event handler system for handling events
*
*	CONTRIBUTORS:
*					Lukas Lipka
*
*	TODO:			@
*
*	BUG:			@
*
*	MISTAKE:		@
*
*	IDEA:			@
*)
{
 .#define KERNEL_FILE
 .#include "kernel.h"
}
unit events;

interface

uses
  guitypes, metrics;
  
{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$LINK events.o}
{$endif}  


(**
*	Mouse State
*)
type
  PMouseState = ^TMouseState;
  TMouseState = record

	p: TPoint;
	z: integer;
	b: l_ulong;
	
  end;
 

(**
*	Keyboard State
*)
type
  PKeybState = ^TKeybState;
  TKeybState = record
	Key  : Longint;
	Shift: Longint;
  end;

(**
*	Basic Event
*)
type
  PEvent = ^TEvent;
  TEvent = record

	aType  : Integer;
	Message: l_ulong;
	Extra  : Pointer;

	Keyb   : TKeybState;
	Mouse  : TMouseState;

  end;

{#define CLEAR_EVENT(e)  CLEAR(e)}

type
  TEventGenProc = procedure(Event: PEvent);
  PEventGenerators = ^TEventGenerators;
  TEventGenerators = record

	Generator: TEventGenProc;
	Enabled  : l_bool;

	Next     : PEventGenerators;

  end;

type
  TEventHandlerProc = procedure(Event: PEvent);
  PEventHandlers = ^TEventHandlers;
  TEventHandlers = record

	Handler: TEventHandlerProc;

	Prev: PEventHandlers;
	Next: PEventHandlers;

  end;

(**
*	Standard event type
*)
const
  EV_NOTHING    =  $00000000;
  EV_MESSAGE    =  $00000001;
  EV_KEYBOARD   =  $00000002;
  EV_MOUSE      =  $00000004;


(**
*	Standard messageges
*)
  MSG_NOTHING   =  0;
  MSG_QUIT      =  1;
  MSG_KILLAPP   =  2;

  { See <docs/events.txt> for events messages codes attribution }

  { 0xFFFF50XX Reserved to common operations ( file, clipboard, drag and drop ...) }

  MSG_COPY     	= $FFFF5000;
  MSG_PASTE    	= $FFFF5001;
  MSG_CUT     	= $FFFF5002;
  MSG_DELETE    = $FFFF5003;
  
  MSG_IODLG_NEWDIR      = $FFFF5004;
  MSG_IOMENU_RENAME     = $FFFF5005;
  MSG_IOMENU_DELETE     = $FFFF5006;
  MSG_IOMENU_COPY       = $FFFF5007;
  MSG_IOMENU_CUT        = $FFFF5008;
  MSG_IOMENU_PASTE      = $FFFF5009;
  MSG_IOMENU_PROPERTIES = $FFFF500A;

const
  EventGenerators    : PEventGenerators	= NIL;
  LastEventGenerator : PEventGenerators	= NIL;
  EventHandlers      : PEventHandlers	= NIL;
  LastEventHandler   : PEventHandlers 	= NIL;
  Event              : PEvent			= NIL;
  GSExit             : l_bool			= false;


procedure InstallEventGenerator(Generator: TEventGenProc);  {$ifdef LINK_CLIBS}external;{$endif}
procedure InstallEventHandler(Handler: TEventHandlerProc);  {$ifdef LINK_CLIBS}external;{$endif}
procedure RemoveEventHandler(Handler: TEventHandlerProc);   {$ifdef LINK_CLIBS}external;{$endif}
procedure GenerateEvent(Event: PEvent);                     {$ifdef LINK_CLIBS}external;{$endif}
procedure HandleEvent(Event: PEvent);                       {$ifdef LINK_CLIBS}external;{$endif}
procedure EventsThread(arg: Pointer);                       {$ifdef LINK_CLIBS}external;{$endif}
procedure PollEvents;                                       {$ifdef LINK_CLIBS}external;{$endif}
procedure InitializeEventsSystem;                           {$ifdef LINK_CLIBS}external;{$endif}
procedure ShutdownEventsSystem;                             {$ifdef LINK_CLIBS}external;{$endif}

procedure EventAssign ( var Ev: TEvent; aType: Integer; Message: l_ulong; Extra: Pointer; Keyb: TKeybState; Mouse: TMouseState );
{$ifdef LINK_CLIBS}external;{$endif}

function NewEvent ( aType: Integer; Message: l_ulong; Extra: Pointer; Keyb: TKeybState; Mouse: TMouseState ): PEvent;
{$ifdef LINK_CLIBS}external;{$endif}

function ScanCode(Ev: PEvent; Key: longint): l_bool;

implementation

{$ifndef LINK_CLIBS}

uses debug;

function ScanCode(Ev: PEvent; Key: longint): l_bool;
begin
  if Ev = NIL then ScanCode := false else
  ScanCode := Ev^.Keyb.Key = Key;
end;

procedure InstallEventGenerator(Generator: TEventGenProc);
var
  p: PEventGenerators;
begin
	if not assigned(Generator) then

	if not assigned(EventGenerators) then
	begin
		New(EventGenerators); {:= NEW(TEventGenerators);}
		EventGenerators^.Generator := Generator;
		EventGenerators^.Enabled := TRUE;
		EventGenerators^.Next := NIL;
		LastEventGenerator := EventGenerators;
	end
	else
    begin
        NEW(p);
    	p^.Generator := Generator;
	    p^.Enabled := TRUE;
    	p^.Next := NIL;
    	LastEventGenerator^.Next := p;
    	LastEventGenerator := p;
	end;
end;

procedure InstallEventHandler(Handler: TEventHandlerProc);
var p:	PEventHandlers;
begin
	if assigned(Handler) then

	if not assigned(EventHandlers) then
	begin
		NEW(EventHandlers);
		EventHandlers^.Handler := Handler;
		EventHandlers^.Prev := LastEventHandler;
		EventHandlers^.Next := NIL;
		LastEventHandler := EventHandlers;
	end
    else
    begin
    	NEW(p);
        p^.Handler := Handler;
    	p^.Prev := LastEventHandler;
        p^.Next := NIL;
    	LastEventHandler^.Next := p;
    	LastEventHandler := p;
	end;
end;

procedure RemoveEventHandler(Handler: TEventHandlerProc);
var 
  p: PEventHandlers;
  
begin
	p := EventHandlers;

	if assigned(p) then

	repeat
	
		if (p^.Handler = Handler) then
		begin
			if assigned(p^.Next) then p^.Next^.Prev := p^.Prev else LastEventHandler := p^.Prev;
			if assigned(p^.Prev) then p^.Prev^.Next := p^.Next else EventHandlers := p^.Next;

			Dispose(p);
			
		end;

		p := p^.Next;
	
	until (p = nil);
end;

procedure EventAssign ( var Ev: TEvent; aType: Integer; Message: l_ulong; Extra: Pointer; Keyb:TKeybState; Mouse:TMouseState );
begin
	Ev.aType   := aType;
	Ev.Message := Message;
	Ev.Extra   := Extra;
	Ev.Keyb    := Keyb;
	Ev.Mouse   := Mouse;
end;

function NewEvent ( aType: Integer; Message: l_ulong; Extra: Pointer; Keyb: TKeybState; Mouse: TMouseState ): PEvent;
var Ev: PEvent;
begin
	New(Ev); { = malloc(sizeof(TEvent)); }
	Ev^.aType   := aType;
	Ev^.Message := Message;
	Ev^.Extra   := Extra;
	Ev^.Keyb    := Keyb;
	Ev^.Mouse   := Mouse;
	NewEvent := Ev;
end;

procedure GenerateEvent(Event: PEvent);
var
	p : PEventGenerators; 
	Ev: TEvent;
begin
    p := EventGenerators;

	Event^.aType := EV_NOTHING;
	Event^.Extra := NIL;
	Event^.Message := MSG_NOTHING;

	if Assigned(EventGenerators) then

	repeat
	
		ev.aType := EV_NOTHING;
		ev.Extra := NIL;

		if (p^.Enabled) then
	    begin
			p^.Generator(@Ev);
			{ SET(Event->Type, ev.Type); }
			Event^.aType := Ev.aType;

			Event^.Message := ev.Message;

			if (ev.aType and EV_KEYBOARD) = EV_KEYBOARD then
			begin
				Event^.Keyb := ev.Keyb;

			end;

			if Assigned(ev.Extra) then
				Event^.Extra := ev.Extra;
		end;

		p := p^.Next;
	
	until (p = NIL);

  { DebugMessage('+++ GenerateEvent...',[]); }

end;

procedure HandleEvent(Event: PEvent);
var
  p, next: PEventHandlers; 
begin
  p := EventHandlers;
  if (not assigned(EventHandlers) or (Event^.aType = EV_NOTHING)) then
  begin
    { DebugMessage('+++ HandleEvent: EV_NOTHING 1',[]); }
	{	return; }
  end
  else
  begin
	repeat
	
	{	next := p^.Next;  commented code has been improved }
		if assigned(Event) and assigned(p) then p^.Handler(Event);

		if (Event^.aType = EV_NOTHING) then ;
                {
                   DebugMessage('+++ HandleEvent: EV_NOTHING 2',[]);
                   return;
                }

                {p := next;}
       
		p := p^.Next;
	until (p = NIL);

	if (Event^.aType = EV_MESSAGE) and (Event^.Message = MSG_QUIT) then
	begin
		GSExit := TRUE;
		DebugMessage('+++ SYSTEM GOING DOWN FOR HALT +++'+#10#13,[]);
	end;

   { DebugMessage('+++ HandleEvent END',[]); }
  end;
end;

procedure PollEvents;
begin
	if Assigned ( Event ) then
	begin
		GenerateEvent(Event);
		HandleEvent(Event);
	end;

end;

procedure EventsThread(arg: Pointer);
begin
	while true do
	begin
		PollEvents;
		//Yield;
	end;
end;

procedure InitializeEventsSystem;
begin
	NEW(Event);
end;

procedure ShutdownEventsSystem;
begin
	if assigned(Event)
		then Dispose(Event);
end;

begin
  EventHandlers := NIL;
  EventGenerators := NIL;

{$endif}
end.
