(*
////////////////////////////////////////////////////////////////////////////////
//
//	File Management System v 2 - Core file
//
//	(c) Copyright 2004 Point Mad. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
unit fms2;
{
 .#include "kernel.h"
 .#include "iodlg.h"
 .#include "fms2.h"
 .#include "window.h"
}

interface

uses
  guitypes,vfile,list,text,dynld,error,menu,dms,events,convert,{iodlg,}window,
  registry,allegro,udaction,debug;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
  {$link iodlg.o}
  {$link fms2.o}
  {$link window.o}
{$endif}


type
  GetIconsFunc = function(i: PFileInfo): PIcon;
  PDataTypeEx = ^TDataTypeEx;
  TDataTypeEx = record

	TypeId   : l_ulong;

	TypeName : l_text;
	
	Default  : PIcon;
	GetIcons : GetIconsFunc;

	
	Actions  : PList;

	Flags    : l_ulong;
	
  end;
  DATATYPEEX = PDataTypeEx;

const	
  DTEF_FREEICONS      = $01;
  DTEF_STRICTINTERNAL = $02;

var DTEs: PList;


function FileHandleMsg ( f: PFileInfo; Msg: l_ulong ): l_bool;
{$ifdef LINK_CLIBS}external;{$endif}

function FileGetMenu2 ( f: PFileInfo; Flags: l_ulong ): PMenu;
{$ifdef LINK_CLIBS}external;{$endif}

function FileGetIcon2 ( f: PFileInfo; Size: l_ulong; var ToFree: l_ulong ): p_bitmap;
{$ifdef LINK_CLIBS}external;{$endif}

procedure FileGetIcons2 ( f: PFileInfo; var i16, i32, i48: p_bitmap; var fi: l_ulong );
{$ifdef LINK_CLIBS}external;{$endif}

function GetCodecFileInfo2( f: PFileInfo; var lnk: PFileInfo ): PCodec;

function RunFile2 ( f: PFileInfo; argc: integer; argv: TArguments ): l_bool;
{$ifdef LINK_CLIBS}external;{$endif}

function AddDTAction ( dte: PDataTypeEx; Name: l_text; Runner: l_text; Run: RunFunc; FavoriteWeight: l_ulong ): PDTAction;
{$ifdef LINK_CLIBS}external;{$endif}

function AddDataTypeEx( TypeId: l_ulong; TypeName: l_text; Icon16, Icon32, Icon48: p_bitmap; GetIcons: GetIconsFunc; Flags: l_ulong ): PDataTypeEx;
{$ifdef LINK_CLIBS}external;{$endif}

function _GetDataTypeEx( TypeId: l_ulong ): PDataTypeEx;
{$ifdef LINK_CLIBS}external;{$endif}

function GetTypeDefaultMsg ( TypeId: l_ulong ): l_ulong;
{$ifdef LINK_CLIBS}external;{$endif}

function ExecutePDAction ( a: PDTAction; f: PFileInfo; argc: integer; argv: TArguments ): l_bool;
{$ifdef LINK_CLIBS}external;{$endif}

function FileGetIconsEx ( f: PFileInfo ): PIcon;
{$ifdef LINK_CLIBS}external;{$endif}

function FileRun ( filename: l_text; Args: TArguments ): l_bool;
{$ifdef LINK_CLIBS}external;{$endif}

(*
 // Advanced run-times
 // - Custom error handler, do not display error messages
 // - Allow host application (console based apps)
*)
function RunFile3 ( f: PFileInfo; argc: integer; argv: TArguments; Host: PApplication; h: PErrorHandler ): l_bool;
{$ifdef LINK_CLIBS}external;{$endif}

function FileHandleMsg2 ( f: PFileInfo; Msg: l_ulong; h: PErrorHandler ): l_bool;
{$ifdef LINK_CLIBS}external;{$endif}

implementation
{$ifndef LINK_CLIBS}
{$ifdef OPascal}
uses Sysutils;
{$else}
uses OSysutil;
{$endif}

var 
  AppVersion: l_ulong;
  GAction: PList;
  BinaryExtender: PDataTypeEx;
  
const
  AppName: l_text = 'File Management System v2';
  nUID: l_text = 'fms2';
  NeededLibs: array[0..2] of String = ( 'menu','window', '' );

(** @defgroup fms File Management System (FMS)
	Manages file actions and informations. Provides generic icons for data.
	@ingroup dmsfms
	@{
*)
(**	@file
	FMS core file (uid:'fms2')
*)

(*////////////////////////////////////////////////////////////////////////////////*)

const NextActionMsg: l_ulong = $FFFE0000;

(*////////////////////////////////////////////////////////////////////////////////*)
function LinkGetLinkedFile ( f: PFileInfo ): PFileInfo;
var
  filename: l_text;
  o: PFileInfo;
  Buffer: Pointer;
begin
    o := NIL;
    Buffer := LoadData2(f^.VPathName,TYPE_TEXT);
	if ( Buffer <> NIL ) then
	begin
	  o := FileGetInfo(filename);
	  filename := '';
	end;
	LinkGetLinkedFile := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function __GetCodecFileInfo ( f: PFileInfo; var lnk: PFileInfo ): PCodec;
var c: PCodec; lf: PFileInfo;
begin
	c := GetCodecFileInfo(f);
	if assigned( lnk ) then lnk := NIL;
	if not assigned( c ) then __GetCodecFileInfo := NIL;
	if ( c^.FavoriteType = DTYPE_FILEREF ) then
	begin
		lf := LinkGetLinkedFile(f);
		if assigned( lf ) then
		begin
			lnk := lf;
			__GetCodecFileInfo := GetCodecFileInfo(lf);
		end else
			DebugWarning('FMS2 : Invalid link',[]);
	end;
	__GetCodecFileInfo := c;
end;

function GetCodecFileInfo2( f: PFileInfo; var lnk: PFileInfo ): PCodec;
begin
  GetCodecFileInfo2 := __GetCodecFileInfo(f, lnk);
end;

(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeDataTypeEx ( o: Pointer );
begin
	if ( PDataTypeEx(o)^.TypeName <> '' ) then PDataTypeEx(o)^.TypeName := '';
	if assigned( PDataTypeEx(o)^.Actions ) then FreeList(PDataTypeEx(o)^.Actions);
	if assigned( PDataTypeEx(o)^.Default ) then FreeIcon(PDataTypeEx(o)^.Default);

	dispose(PDataTypeEx(o));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeDTAction ( o: Pointer );
begin
	if ( PDTAction(o)^.Name<>'' ) then PDTAction(o)^.Name := '';
	if ( PDTAction(o)^.Runner<>'' ) then PDTAction(o)^.Runner := '';
	dispose(PDTAction(o));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function _GetDataTypeEx( TypeId: l_ulong ): PDataTypeEx;
var a,b: PListItem;
begin
    _GetDataTypeEx := NIL;
	if assigned( DTEs^.Last ) then
	begin
		a := DTEs^.Last^.Next; b := a;
		repeat
			if ( DATATYPEEX(a^.Data)^.TypeId = TypeId ) then begin _GetDataTypeEx := DATATYPEEX(a^.Data); break; end;
			a := a^.Next;
		until ( a = b );
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function AddDataTypeEx( TypeId:l_ulong; TypeName:l_text; Icon16,Icon32,Icon48:p_bitmap; GetIcons: GetIconsFunc; Flags:l_ulong ): PDataTypeEx; 
var
  o: PDataTypeEx;
begin	
	DebugMessage('FMS2 : DTE %08x %s',[TypeId,TypeName]);

	if ( o = _GetDataTypeEx(TypeId) ) then
	begin
		if ( o^.TypeName<>'' ) then o^.TypeName := '';
		if assigned( o^.Default ) then FreeIcon(o^.Default);
		if ( assigned(Icon16) or assigned(Icon32) or assigned(Icon48) ) then
		begin
			if ( Flags and DTEF_FREEICONS ) = DTEF_FREEICONS then
				o^.Default := NewIcon(Icon16,Icon32,Icon48)
			else
				o^.Default := NewIcon2(Icon16,Icon32,Icon48);
		end else
			o^.Default := NIL;
		o^.TypeName := TypeName;
		DebugWarning('FMS2 : Data Type Extender %08x yet declared : Overwrite icons and name',[TypeId]);
		AddDataTypeEx := o;
	end;
	New(o);
	if not assigned( o ) then AddDataTypeEx := NIL else
	begin
    	fillchar(o,sizeof(TDataTypeEx),0);
	    o^.TypeId := TypeId;
    	o^.TypeName := TypeName;
	
    	if ( assigned(Icon16) or assigned(Icon32) or assigned(Icon48) ) then
    	begin
	    	if ( Flags and DTEF_FREEICONS ) = DTEF_FREEICONS then
		    	o^.Default := NewIcon(Icon16,Icon32,Icon48)
     		else
		    	o^.Default := NewIcon2(Icon16,Icon32,Icon48);
    	end;
    	o^.GetIcons := GetIcons;
    	o^.Flags := Flags;
    	o^.Actions := NewList;
    	ListAdd(DTEs,'',o,{$ifdef fpc}@{$endif}FreeDataTypeEx);
    	AddDataTypeEx := o;
    end;	
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function AddDTAction ( dte:PDataTypeEx; Name,Runner: l_text; Run: RunFunc; FavoriteWeight: l_ulong ): PDTAction;
var o: PDTAction;
begin
	New(o);
	if not assigned( o ) then AddDTAction := NIL else
	fillchar(o,sizeof(TDTAction),0);
	
	if assigned( Run ) then
		DebugMessage('FMS2 :     %08x : New action %s (%d) ^. Internal %08x',[dte^.TypeId,Name,FavoriteWeight,Run])
	else
		DebugMessage('FMS2 :     %08x : New action %s (%d) ^. %s',[dte^.TypeId,Name,FavoriteWeight,Runner]);
	
	o^.Name := Name;
	o^.Runner := Runner;
	o^.Run := Run;
	o^.FavoriteWeight := FavoriteWeight;
	o^.ActionMsg := NextActionMsg;
        inc(NextActionMsg);
	
	ListAdd(dte^.Actions,'',o,{$ifdef fpc}@{$endif}FreeDTAction);
	ListAdd(GAction,'',o,NIL);
	AddDTAction := o;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
procedure _CollectTypeActions ( TypeId: l_ulong; Lst: PList );
var dte: PDataTypeEx; a,b: PListItem;
begin
    dte := _GetDataTypeEx(TypeId);
	if ( (dte=NIL) and (Lst^.Last=NIL) ) then dte := BinaryExtender;
	if assigned( dte ) then
	if assigned( dte^.Actions^.Last ) then
	begin
		a := dte^.Actions^.Last^.Next; b := a;
		repeat
			ListAdd(Lst,'',a^.Data,NIL);
			a := a^.Next;
		until ( a = b ); 
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DTActionFavComparator ( a, b: Pointer {PDTAction} ): Longint;
begin
	DTActionFavComparator := DTACTION(b)^.FavoriteWeight-DTACTION(a)^.FavoriteWeight;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CollectFileActions ( f: PFileInfo ): PList;
var
	lnk: PFileInfo;
	c: PCodec;
	Lst: PList;
	a,b: PListItem;
begin
	lnk := NIL;
	c := __GetCodecFileInfo(f,lnk);
	Lst := NewList;
	if assigned( lnk ) then FreeFileInfo(lnk);
	if not assigned( Lst ) then CollectFileActions := NIL;
	if assigned( c ) then
	begin
		if assigned( c^.SubCodecs^.Last ) then
		begin
			a := c^.SubCodecs^.Last^.Next; b := a;
			repeat
				_CollectTypeActions(PSUBCODEC(a^.Data)^.TypeId,Lst);
				a := a^.Next;
			until ( a = b );
		end else
			_CollectTypeActions(BinaryExtender^.TypeId,Lst);
	end else
		_CollectTypeActions(BinaryExtender^.TypeId,Lst);
	ListOrderByData(Lst,{$ifdef fpc}@{$endif}DTActionFavComparator);
	CollectFileActions := Lst;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CollectTypeActions ( TypeId:l_ulong ): PList;
var
  Lst: PList;
begin
    Lst := NewList;
	if not assigned( Lst ) then CollectTypeActions := NIL else
	begin
    	_CollectTypeActions(TypeId,Lst);
    	ListOrderByData(Lst,{$ifdef fpc}@{$endif}DTActionFavComparator);
    	CollectTypeActions := Lst;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetActionByMsg ( Msg: l_ulong ): PDTAction;
var a,b: PListItem;
begin
	GetActionByMsg := NIL;
	if assigned( GAction^.Last ) then
	begin
		a := GAction^.Last^.Next; b := a;
		repeat
			if ( DTACTION(a^.Data)^.ActionMsg = Msg ) then begin GetActionByMsg := DTACTION(a^.Data); break; end;
			a := a^.Next;
		until ( a = b );
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*////////////////////////////////////////////////////////////////////////////////*)
function ExecutePDAction ( a: PDTAction; f: PFileInfo; argc: integer; argv: TArguments ): l_bool;
var exresult: l_bool; h: PErrorHandler;
begin
   (*
	h := {$ifdef fpc}@{$endif}CreateErrorHandler;
	exresult := ExecutePDAction2(a,f,argc,argv,NIL,h);
	DebugMessage('Could i report ?',[]);
	if ( EHandlerIsError(h) ) then WindowNotifyError(Me,h);
	FreeErrorHandler(h);
	ExecutePDAction := exresult;
   *)
        exresult := ExecutePDAction2(a,f,argc,argv,NIL,@h);
        ExecutePDAction := exresult;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetFileDefaultAction ( f: PFileInfo ): PDTAction;
var
  Lst: PList;
begin    
    Lst := CollectFileActions(f);
	if assigned( Lst ) then
		if assigned( Lst^.Last ) then GetFileDefaultAction := DTACTION(Lst^.Last^.Next^.Data) else
	GetFileDefaultAction := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetTypeDefaultMsg ( TypeId:l_ulong ): l_ulong;
var	Lst: PList; 
begin
    Lst := CollectTypeActions(TypeId);
	if assigned( Lst ) then
		if assigned( Lst^.Last ) then GetTypeDefaultMsg := DTACTION(Lst^.Last^.Next^.Data)^.ActionMsg else
	GetTypeDefaultMsg := 0;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function RunFile3 ( f:PFileInfo; argc: integer; argv:TArguments; Host:PApplication; h:PErrorHandler ): l_bool;
var a: PDTAction;
begin
	a := GetFileDefaultAction(f);
	if assigned( a ) then RunFile3 := ExecutePDAction2(a,f,argc,argv,Host,@h) else
	RunFile3 := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function RunFile2 ( f: PFileInfo; argc: integer; argv: TArguments ): l_bool;
var a: PDTAction;
begin	
    a := GetFileDefaultAction(f);
	if assigned( a ) then RunFile2 := ExecutePDAction(a,f,argc,argv) else
	RunFile2 := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function FileRun ( filename: l_text; Args: TArguments ): l_bool;
var 
  f: PFileInfo;
  r: l_bool;
  argv: array[0..1] of integer;
begin
	f := FileGetInfo(filename);
	if assigned( f ) then
	begin
		argv[0] := 0; argv[1] := 0;
		
		{$ifdef OPascal}
		 argv[0] := 0;
		 argv[1] := 0;
        {$else}
		 argv[0] := 0;
		 argv[1] := 0;
		{$endif}
		
		if assigned( Args ) then move(args[0], argv[0], 2);
		r := RunFile2(f,StrToInt(Args[0]),args);
		{if assigned( Args ) then SetLength(argv, 0);}
		FreeFileInfo(f);
		FileRun := r;
	end
	else FileRun := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function RunMessage ( Msg: l_ulong; f: PFileInfo; argc: integer; argv: TArguments ): l_bool;
var a: PDTAction;
begin
	a := GetActionByMsg(Msg);
	if assigned( a ) then RunMessage := ExecutePDAction(a,f,argc,argv) else
	RunMessage := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*////////////////////////////////////////////////////////////////////////////////*)
function FileGetIconsEx ( f: PFileInfo ): PIcon;
var
  lnk: PFileInfo;
  i  : PIcon;
  c  : PCodec;
  dte: PDataTypeEx;
	
begin
	lnk := NIL;
	i   := NIL;
	c   := __GetCodecFileInfo(f, lnk);
	dte := NIL;

	if assigned( lnk ) then f := lnk;

	if assigned( c ) then
	begin
		if assigned( c^.GetIcons ) then
			i := c^.GetIcons(f);

		if not assigned( i ) then i := DuplicateIcon(c^.Default);
	
		dte := _GetDataTypeEx(c^.FavoriteType);
	end;
	if not assigned( dte ) then dte := BinaryExtender;
	
	if not assigned( i ) then
		if assigned( dte^.GetIcons ) then
			i := dte^.GetIcons(f);
		
	if not assigned( i ) then i := DuplicateIcon(dte^.Default);
	
	if not assigned( i ) then i := DuplicateIcon(BinaryExtender^.Default);
	
	if assigned( lnk ) then FreeFileInfo(lnk);
	FileGetIconsEx := i;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function RleSpriteToImage ( ico: PRleSprite ): PImage;
var
  img: PImage;
begin
	img := al_create_bitmap(AL_RLE_SPRITEptr(ico)^.w,AL_RLE_SPRITEptr(ico)^.h);
	al_clear_to_color(img, img^.vtable^.mask_color);
	al_draw_rle_sprite(img,AL_RLE_SPRITEptr(ico),0,0);
	RleSpriteToImage := img;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FileGetIcons2 ( f: PFileInfo; var i16,i32,i48: p_bitmap; var fi: l_ulong );
var i: guitypes.PIcon;
begin	
    i := FileGetIconsEx(f);
	if not assigned( i ) then exit;
	if ( i^.Instances = 0 ) then begin
		DebugError('FMS2 : MAY CRASH : FREEED ICON RETURNED !',[]);
	end else begin
		if ( fi<>0 ) then fi := 1;
		if assigned( i16 ) then i16 := RleSpriteToImage(i^.Icon16);
		if assigned( i32 ) then i32 := RleSpriteToImage(i^.Icon32);
		if assigned( i48 ) then i48 := RleSpriteToImage(i^.Icon48);
	end;
	FreeIcon(i);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function FileGetIcon2 ( f: PFileInfo; Size: l_ulong; var ToFree: l_ulong ): p_bitmap;
var
  i: guitypes.PIcon; bmp: p_bitmap;
begin
    i := FileGetIconsEx(f);
	if not assigned( i ) then FileGetIcon2 := NIL else
	if ( i^.Instances = 0 ) then begin
		DebugError('FMS2 : MAY CRASH : FREEED ICON RETURNED !',[]);
	end else begin
		bmp := NIL;
		if ( ToFree <> 0 ) then ToFree := 1;
		if ( Size = 16 ) then bmp := RleSpriteToImage(i^.Icon16);
		if ( Size = 32 ) then bmp := RleSpriteToImage(i^.Icon32);
		if ( Size = 48 ) then bmp := RleSpriteToImage(i^.Icon48);
		FileGetIcon2 := bmp;
	end;
	FreeIcon(i);
	FileGetIcon2 := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function FileGetMenu2 ( f:PFileInfo; Flags:l_ulong ): PMenu;
var
  m: PMenu;
  Lst: PList;
  a,b: PListItem;
begin
    m := NewMenu(NIL);
    Lst := CollectFileActions(f);
	if assigned( Lst ) then
	begin
		if assigned( Lst^.Last ) then
		begin
			a := Lst^.Last^.Next;
            b := Lst^.Last^.Next;
			repeat
				AddMenuItem (m,DTACTION(a^.Data)^.Name,NIL,DTACTION(a^.Data)^.ActionMsg,0,NIL);
				a := a^.Next;
			until ( a = b);
		end;
		FreeList(Lst);
	end;
	AddMenuItemSeparator(m);
	if ( (f^.Attributes and FA_DIREC) = FA_DIREC ) and (( FA_DIREC and Flags ) = Flags ) then
		AddMenuItem(m,'New directory',NIL,MSG_IODLG_NEWDIR,0,NIL);
	if ( Flags = 0 )  then
		AddMenuItem(m,'Rename',NIL,MSG_IOMENU_RENAME,0,NIL);
	AddMenuItem (m,'Delete',NIL,MSG_IOMENU_DELETE,0,NIL);
	AddMenuItemSeparator(m);
	
	AddMenuItem (m,'Copy',NIL,MSG_IOMENU_COPY,0,NIL);
	AddMenuItem (m,'Cut',NIL,MSG_IOMENU_CUT,0,NIL);
	if ( f^.Attributes and FA_DIREC ) = FA_DIREC then
		AddMenuItem (m,'Paste',NIL,MSG_IOMENU_PASTE,0,NIL);
	AddMenuItemSeparator(m);
	AddMenuItem (m,'Propreties',NIL,MSG_IOMENU_PROPERTIES,0,NIL);
	FileGetMenu2 := m;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function FileHandleMsg2 ( f: PFileInfo; Msg: l_ulong; h: PErrorHandler ): l_bool;
var a: PDTAction; App: PApplication;
begin
        App := NIL;
	if ( Msg = MSG_IOMENU_PROPERTIES ) then
        begin
           App := DynLdRun('xlib/fprop.dl',f^.VPathName);
           FileHandleMsg2 := assigned(App);
        end;
        a := GetActionByMsg(Msg);
	if not assigned( a ) then FileHandleMsg2 := false;
	FileHandleMsg2 := ExecutePDAction2(a,f,0,NIL,NIL,@h);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function FileHandleMsg ( f: PFileInfo; Msg: l_ulong ): l_bool;
var a: PDTAction; App: PApplication;
begin
	App := NIL;
	if ( Msg = MSG_IOMENU_PROPERTIES ) then
        begin
          App := DynLdRun('xlib/fprop.dl',f^.VPathName);
          FileHandleMsg := assigned(App);
        end;
        a := GetActionByMsg(Msg);
	if not assigned( a ) then FileHandleMsg := false;
	FileHandleMsg := ExecutePDAction(a,f,0,NIL);
	
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Registry setup                                                               *)
(*////////////////////////////////////////////////////////////////////////////////*)
procedure InstallActionFromReg( dte: PDataTypeEx; o: PRegKey );
var
	act: PDTAction; 
	a,b: PRegKey;
begin
    act := AddDTAction(dte,o^.Name,l_text(o^.Data),NIL,0);
	if assigned( o^.Last ) then
	begin
		a := o^.Last^.Next; b := a;
		repeat
			if ( (TextCaseCompare(a^.Name,'FavoriteWeight') = 0) and (a^.aType = RKT_LONG) ) then
				act^.FavoriteWeight := Longint(a^.Data);
			a := a^.Next;
		until ( a = b );
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure InstallActionsFromReg( dte: PDataTypeEx; o: PRegKey );
var a,b: PRegKey;
begin
	
	if assigned( o^.Last ) then
	begin
		a := o^.Last^.Next; b := a;
		repeat
			if ( a^.aType = RKT_TEXT ) then begin
				InstallActionFromReg(dte,a);
			end;
			a := a^.Next;
		until ( a = b );
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure InstallDTEFromReg ( o: PRegKey );
var 
  dte: PDataTypeEx;
  a,b: PRegKey;
  img16,img32,img48: PImage;
  TypeId: l_ulong;
begin
	
	if ( assigned(o^.Last) and ( o^.aType = RKT_TEXT ) ) then
	begin
		TypeId := HexToLongint(o^.Name);
		
		Img16 := NIL; Img32 := NIL; Img48 := NIL;
		a := o^.Last^.Next; 
        b := a;
		
		if ( not(dte = _GetDataTypeEx(TypeId)) ) then
			dte := AddDataTypeEx(TypeId,l_text(o^.Data),NIL,NIL,NIL,NIL,DTEF_FREEICONS);
	
		repeat
			if ( (TextCaseCompare(a^.Name,'Icon16' ) = 0) and ( a^.aType = RKT_TEXT ) ) then
				Img16 := LoadData2(l_text(a^.Data),DTYPE_IMAGE)
			else if ( (TextCaseCompare(a^.Name,'Icon32' ) = 0) and ( a^.aType = RKT_TEXT ) ) then
				Img32 := LoadData2(l_text(a^.Data),DTYPE_IMAGE)
			else if ( (TextCaseCompare(a^.Name,'Icon48' ) = 0) and ( a^.aType = RKT_TEXT ) ) then
				Img48 := LoadData2(l_text(a^.Data),DTYPE_IMAGE)
			else if ( TextCaseCompare(a^.Name,'Actions' ) = 0 ) then
				InstallActionsFromReg(dte,a);
			a := a^.Next;
		until ( a = b );
		
		if ( assigned(Img48) or assigned(Img32) or assigned(Img16) ) then
		begin
			if assigned( dte^.Default ) then FreeIcon(dte^.Default);
			dte^.Default := NewIcon(Img16,Img32,Img48);
		end;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function LibMain ( argc: integer; argv: l_text ): l_bool;
var
	t: PDataTypeEx;
	a,b,o: PRegKey;
begin	
	DTEs := NewList;
	
	if not assigned( DTEs ) then
	begin
		DebugError('FMS2 : Unable to initialise',[]);
		LibMain := false;
	end;

	GAction := NewList;
	
	if not assigned( GAction ) then
	begin
		FreeList(DTEs);
		DebugError('FMS2 : Unable to initialise',[]);
		LibMain := false;
	end;

	DMS2InstallIcons();
	(*
	SYSEXPORT(FileHandleMsg2);
	SYSEXPORT(FileHandleMsg);
	SYSEXPORT(FileGetMenu2);
	SYSEXPORT(FileGetIcon2);
	SYSEXPORT(FileGetIcons2);
	SYSEXPORT(RunFile2);
	SYSEXPORT(RunFile3);

	SYSEXPORT(AddDTAction);
	SYSEXPORT(AddDataTypeEx);
	SYSEXPORT(_GetDataTypeEx);
	SYSEXPORT(GetTypeDefaultMsg);
	SYSEXPORT(DTEs);
	SYSEXPORT(ExecutePDAction);
	SYSEXPORT(ExecutePDAction2);
	SYSEXPORT(FileGetIconsEx);
	SYSEXPORT(FileRun);
    *)
	BinaryExtender := AddDataTypeEx(DTYPE_BINDATA,'Unknown binary',ListKey(Me.Ressource,'FILE16'),ListKey(Me.Ressource,'FILE32'),ListKey(Me.Ressource,'FILE48'),NIL,0);
	AddDTAction(BinaryExtender,'Open with...','Applications/openwth.app',NIL,0);
	
	
	t := AddDataTypeEx(
                DTYPE_DYNLD,
                'DynLd executable',
                ListKey(Me.Ressource,'APP16'),
                ListKey(Me.Ressource,'APP32'),
                ListKey(Me.Ressource,'APP48'),nil
                (*{$ifdef fpc}@{$endif}GetDynLdIcon*),0
        );
	AddDTAction(
          t,
          'Execute',
          '',nil,
          (*{$ifdef fpc}@{$endif}ExecutableRun,*)
          100
        );
	o := ResolveKey('/SYSTEM/FMS');
	
	
	if assigned( o ) then 
	begin
		if assigned( o^.Last ) then
		begin
			a := o^.Last^.Next;
                        b := o^.Last^.Next;
			repeat
				if ( a^.aType = RKT_TEXT ) then InstallDTEFromReg(a);
				a := a^.Next;
			until ( a = b );
		end;
	end;
	
	
	LibMain := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Close;
begin
	if assigned( DTEs ) then FreeList(DTEs);
	if assigned( GAction ) then FreeList(GAction);
end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion := ULONG_ID(0,0,0,1);
  DTEs := NIL;
  GAction := NIL;

{$endif}   {LINK_CLIBS}
end.
