(*
*       (c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
*
*       FILE:                   font.c
*
*       PROJECT:                Phoenix engine - Core
*
*       DESCRIPTION:    Exetended font manipulation
*
*       CONTRIBUTORS:
*                                       Lukas Lipka
*
*       TODO:                   @
*
*       BUG:                    @
*
*       MISTAKE:                @
*
*       IDEA:                   @
*)
unit font;
{
  #include "kernel.h"
  ...#include "alfont.h"...
  #include "allegttf.h"
}
(** @defgroup font Font loader for allegro based text output
        @{
*)
(**     @file
        Font core file
*)

(***
        Global font variable, which is used as the default system font.
*)
interface

uses
  guitypes,
  list,
  freetype,
  ttobjs,
  allegro;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$LINK font.o}
   {$LINK ft2build.o}
{$endif}

{$define NEWFONTSYS}

function FontLoad ( szFilename: l_text ): PFont; {$ifdef link_clibs}external;{$endif}

(**
*	Global main system font
*)

const  default_font: PFONT = NIL;

{$ifdef NEWFONTSYS}
{
  #include <ft2build.h>  
  #include FT_FREETYPE_H
}

type 
  PFontFace    = ^TFontFace;
  PFontSize    = ^TFontSize;
  PFontGlyph   = ^TFontGlyph;

  PFT_Library = ^FT_Library;
  //PPFT_Library = ^PFT_Library;
  PFT_Face = ^FT_Face;
  //PPFT_Face = ^PFT_Face;
  PFT_Charmap = ^TFT_Charmap;
  PPFT_Charmap = ^PFT_Charmap;
  PFT_GlyphSlot = ^FT_GlyphSlot;
  PFT_Subglyph = ^FT_Subglyph;
  PFT_Size = ^FT_Size;

  {$if defined(cpu64) and not(defined(win64) and defined(cpux86_64))}
  FT_Long = int64;
  FT_ULong = qword;
  FT_Pos = int64;
  {$ELSE}
  FT_Long = longint;
  FT_ULong = longword;
  FT_Pos = longint;
  {$ENDIF}

  FT_Vector = record
    x : FT_Pos;
    y : FT_Pos;
  end;
  PFT_Vector = ^FT_Vector;

  PFT_Glyph = ^FT_Glyph;
  //PPFT_Glyph = ^PFT_Glyph;
  PFT_BitmapGlyph = ^FT_BitmapGlyph;
  PFT_OutlineGlyph = ^FT_OutlineGlyph;
  FT_Glyph = record
    FTlibrary : PFT_Library;
    clazz : pointer;
    aFormat : longword;
    advance : FT_Vector;
  end;
  FT_Bitmap = record
    rows : integer;
    width : integer;
    pitch : integer;
    buffer : pointer;
    num_grays : shortint;
    pixel_mode : char;
    palette_mode : char;
    palette : pointer;
  end;

  FT_Outline = record
    n_contours,
    n_points : smallint;
    points : PFT_Vector;
    tags : pchar;
    contours : ^smallint;
    flags : integer;
  end;
  PFT_Outline = ^FT_Outline;

  FT_BitmapGlyph = record
    root : FT_Glyph;
    left, top : Longint;
    bitmap : FT_Bitmap;
  end;

  FT_OutlineGlyph = record
    root : FT_Glyph;
    outline : FT_Outline;
  end;

  FT_Library   =  record
  end;

  FT_Bool = boolean;
  FT_FWord = smallint;
  FT_UFWord = word;
  FT_Char = char;
  FT_Byte = byte;
  FT_String = char;
  FT_Short = smallint;
  FT_UShort = word;
  FT_Int = longint;
  FT_UInt = longword;
  FT_F2Dot14 = smallint;
  FT_F26Dot6 = longint;
  FT_Fixed = FT_Long;
  FT_Error = longint;
  FT_Pointer = pointer;
  //FT_Offset = size_t;
  //FT_PtrDist = size_t;

  FT_Render_Mode = (FT_RENDER_MODE_NORMAL, FT_RENDER_MODE_LIGHT,
      FT_RENDER_MODE_MONO, FT_RENDER_MODE_LCD, FT_RENDER_MODE_LCD_V,
      FT_RENDER_MODE_MAX);

  FT_UnitVector_ = record
      x : FT_F2Dot14;
      y : FT_F2Dot14;
   end;
  FT_UnitVector = FT_UnitVector_;

  FT_Matrix = record
      xx : FT_Fixed;
      xy : FT_Fixed;
      yx : FT_Fixed;
      yy : FT_Fixed;
   end;
  PFT_Matrix = ^FT_Matrix;

  FT_Data = record
      pointer : ^FT_Byte;
      length : FT_Int;
   end;

  FT_Generic_Finalizer = procedure (AnObject:pointer);cdecl;

  FT_Generic = record
      data : pointer;
      finalizer : FT_Generic_Finalizer;
   end;

  FT_Glyph_Metrics = record
    width : FT_Pos;
    height : FT_Pos;
    horiBearingX : FT_Pos;
    horiBearingY : FT_Pos;
    horiAdvance : FT_Pos;
    vertBearingX : FT_Pos;
    vertBearingY : FT_Pos;
    vertAdvance : FT_Pos;
  end;

  FT_Bitmap_Size = record
    height : FT_Short;
    width : FT_Short;
  end;
  AFT_Bitmap_Size = array [0..1023] of FT_Bitmap_Size;
  PFT_Bitmap_Size = ^AFT_Bitmap_Size;

  FT_BBox = record
    xMin, yMin : FT_Pos;
    xMax, yMax : FT_Pos;
  end;
  PFT_BBox = ^FT_BBox;


  FT_Outline_MoveToFunc = function(const to_: PFT_Vector; user: Pointer): integer;
  FT_Outline_LineToFunc = function(const to_: PFT_Vector; user: Pointer): integer;
  FT_Outline_ConicToFunc = function(const control, to_: PFT_Vector; user: Pointer): integer;
  FT_Outline_CubicToFunc = function(const control1, control2, to_: PFT_Vector; user: Pointer): integer;

  FT_Outline_Funcs = record
    move_to: FT_Outline_MoveToFunc;
    line_to: FT_Outline_LineToFunc;
    conic_to: FT_Outline_ConicToFunc;
    cubic_to: FT_Outline_CubicToFunc;
    shift: integer;
    delta: FT_Pos;
  end;
  PFT_Outline_Funcs = ^FT_Outline_Funcs;

  FT_Size_Metrics = record
    x_ppem : FT_UShort;
    y_ppem : FT_UShort;
    x_scale : FT_Fixed;
    y_scale : FT_Fixed;
    ascender : FT_Pos;
    descender : FT_Pos;
    height : FT_Pos;
    max_advance : FT_Pos;
  end;

  FT_Encoding = array[0..3] of char;

  TFT_Charmap = record
    face : PFT_Face;
    encoding : FT_Encoding;
    platform_id, encoding_id : FT_UShort;
  end;

  FT_Size = record
    face : PFT_Face;
    generic : FT_Generic;
    metrics : FT_Size_Metrics;
    //internal : FT_Size_Internal;
  end;

  FT_Subglyph = record  // TODO
  end;

  FT_GlyphSlot = record
    alibrary : PFT_Library;
    face : PFT_Face;
    next : PFT_GlyphSlot;
    flags : FT_UInt;
    generic : FT_Generic;
    metrics : FT_Glyph_Metrics;
    linearHoriAdvance : FT_Fixed;
    linearVertAdvance : FT_Fixed;
    advance : FT_Vector;
    format : longword;
    bitmap : FT_Bitmap;
    bitmap_left : FT_Int;
    bitmap_top : FT_Int;
    outline : FT_Outline;
    num_subglyphs : FT_UInt;
    subglyphs : PFT_SubGlyph;
    control_data : pointer;
    control_len : longint;
    other : pointer;
  end;

  FT_Face = record
    num_faces : FT_Long;
    face_index : FT_Long;
    face_flags : FT_Long;
    style_flags : FT_Long;
    num_glyphs : FT_Long;
    family_name : pchar;
    style_name : pchar;
    num_fixed_sizes : FT_Int;
    available_sizes : PFT_Bitmap_Size;     // is array
    num_charmaps : FT_Int;
    charmaps : PPFT_CharMap;               // is array
    generic : FT_Generic;
    bbox : FT_BBox;
    units_per_EM : FT_UShort;
    ascender : FT_Short;
    descender : FT_Short;
    height : FT_Short;
    max_advance_width : FT_Short;
    max_advance_height : FT_Short;
    underline_position : FT_Short;
    underline_thickness : FT_Short;
    glyph : PFT_GlyphSlot;
    size : PFT_Size;
    charmap : PFT_CharMap;
  end;

  (*
  FT_Face      =  FT_FaceRec_
  FT_FaceRec   =  record

     { FT_Long          } num_faces       : TT_Long;
     { FT_Long          } face_index      : TT_Long;

     { FT_Long          } face_flags      : TT_Long;
     { FT_Long          } style_flags     : TT_Long;

     { FT_Long          } num_glyphs      : TT_Long;

     { FT_String*       } family_name     : String;
      {FT_String*       } style_name      : String;

     { FT_Int           } num_fixed_sizes : TT_Int;
     { FT_Bitmap_Size*  } available_sizes : ;

     { FT_Int           } num_charmaps    : TT_Int;
     { FT_CharMap*      } charmaps        : TT_CharMap;

     { FT_Generic       } generic         : TT_Generic;

      {*# The following member variables (down to `underline_thickness') *}
      {*# are only relevant to scalable outlines; cf. @FT_Bitmap_Size    *}
      {*# for bitmap fonts.                                              *}
     { FT_BBox          } bbox                : TT_Box;

     { FT_UShort        } units_per_EM        : TT_UShort;
     { FT_Short         } ascender            : TT_Short;
     { FT_Short         } descender           : TT_Short;
     { FT_Short         } height              : TT_Short;

     { FT_Short         } max_advance_width   : TT_Short;
     { FT_Short         } max_advance_height  : TT_Short;

     { FT_Short         } underline_position  : TT_Short;
     { FT_Short         } underline_thickness : TT_Short;

     { FT_GlyphSlot     } glyph               : TT_Glyph{Slot};
     { FT_Size          } size                : TT_Size;
     { FT_CharMap       } charmap             : TT_CharMap;

     { *@private begin * }

     { FT_Driver        } driver              : TT_Driver;
     { FT_Memory        } memory              : TT_Memory;
     { FT_Stream        } stream              : TT_Stream;

     { FT_ListRec       } sizes_list          : TT_ListRec;

     { FT_Generic       } autohint            : TT_Generic; {* face-specific auto-hinter data *}
     { void*            } extensions          : Pointer;    {* unused                         *}

     { FT_Face_Internal } internal            : TT_Face_Internal;

     {* @private end *}

    end;
    FT_FaceRec = FT_FaceRec_;
    *)




  TFontFace = record
	
	Name: l_text;
	
	face: FT_Face{FT_Face}; 
	
	FontSizesCache: PList;
	
  end;

  TFontGlyph = record
	
	aType: l_uchar;
	
	Width :l_ushort;
	Height: l_ushort;
	
	DeltaX: l_ushort;
	
	Data: Pointer;
  end;
  

  TFontSize = record
	
	Face: PFontFace;
	
	PixelSize: l_ushort;
    DeltaY: l_ushort;
	
	Glyphs: PList;
	
  end;

var 
  FreeTypeLibrary: FT_Library;

  FontFaces: PList;
  DefaultFontFace: PFontFace;
  DefaultFontSize: PFontSize;

{$endif} {NEWFONTSYS}


implementation
{$IFNDEF LINK_CLIBS}
uses
vfile
{$ifdef OPascal}
,Sysutils
{$else}
,OSysutil
{$endif}
;
(**
        Loads a font
        @param szFilename Path to the font to load
        @return The loaded font, or default font on failure.
*)
function FontLoad ( szFilename: l_text ): PFONT;
var
   ext: l_text;
        palette: AL_PALETTE;
begin
        ext := ExtractFileExt(szFilename);
        if (ext='') then Fontload := default_font;

        if (not fopen(szFilename, FSF_READ)) then FontLoad := default_font;

        if UpperCase(ext) = '.FNT' then
	  FontLoad := al_load_font(szFilename, @palette, NIL)
        else if UpperCase(ext) = '.TTF' then
	  FontLoad := al_load_font (szFilename, @palette, NIL)
        else
	  FontLoad := default_font;
end;

{$ifdef NEWFONTSYS}

(*

        New font system

        Work In Progress, !! not working !!

*)

function LoadFontFace ( Filename: l_text ): PFontFace; 
var
   Error: Integer;
   o: PFontFace;
begin
    {$ifdef use_freetype}
    New(o);
    if assigned( o ) then
    begin
        Error := FT_New_Face( FreeTypeLibrary, Filename, 0, o^.face );

        if l_bool( Error ) then
        begin
                dispose(o);
                LoadFontFace := NIL;

        end;


        LoadFontFace := o;
    end;
    {$else}
     {$ifdef use_allegro}
     {$else}
     {$endif}
    {$endif}
end;


function InitFontSystem: l_bool;
var
  Error: integer;
begin
  {$ifdef use_freetype}
  FontFaces := NewList;

  Error := FT_Init_FreeType( FreeTypeLibrary );

  InitFontSystem := l_bool( Error );
  {$else}
    {$ifdef use_allegro}
    {$else}
    {$endif}
  {$endif}
end;

procedure UnInitFontSystem;
begin

  if assigned( FontFaces ) then FreeList(FontFaces);

end;

{$endif} {NEWFONTSYS}

{$ENDIF} {LINK_CLIBS}
end.
