(*
////////////////////////////////////////////////////////////////////////////////
//
//	File Properties 0.01
//
//	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
program fprop;
{
 #include 'kernel.h'
 #include 'widget.h'
 #include 'menu.h'
 #include 'button.h'
 #include 'window.h'
 #include 'label.h'
 #include 'fms2.h'
 #include 'checkbox.h'
 #include 'tabbook.h'
 #include 'textbox.h'
 #include 'iodlg.h'
}

uses
  Sysutils,dos,
  guitypes,error,vfile,dms,events,text,widget,menu,button,window,ulabel,fms2,
  dynld,timer,list,metrics,checkbox,tabbook,textbox,scrllbar,skin,iodlg;

{$i cdefs.inc}

const
 AppName: l_text	= 'File Properties';

 nUID: l_text = 'app:fprop';
 NeededLibs: array[0..7] of String = ( 'label','checkbox','button','window','tabbook','textbox','iodlg','' );

 filename: l_text = '';
 w: PWindow = NIL;
 Mode: l_ulong = 0;

var
 l1: PLabel;
 l2: PLabel;
 l3: PLabel;
 l4: PLabel;
 l5: PLabel;
 l6: PLabel;
 l7: PLabel;

 icon: PImageLabel;

 tl: PTextbox;

 AppVersion: l_ulong;

 c1: PCheckbox;
 c2: PCheckbox;
 c3: PCheckbox;
 c4: PCheckbox;

const
  MSG_F_OK 	 = $00010001;
  MSG_SELECTFILE = $00010002;



function AppEventHandler ( o: PWidget; Event: PEvent ): l_bool;
var
  Attr: Byte;
  lfile, filename: l_text;
begin
	AppEventHandler := false;
	if ( Event^.aType = EV_MESSAGE ) then
	begin
		case Event^.Message of
		
			WM_CLOSE:
			begin
				WidgetDispose(PWIDGET(o));
				CloseApp(@Me);
				AppEventHandler := true;
			end;
			
			MSG_SELECTFILE:
			begin
                                lfile := '';
				lfile := IOBox('Select linked file', IOBOX_OPEN, lfile, NIL, true);

				if ( lfile<>'' ) then 
				begin
					TextBoxSetTextEx(tl,lfile);
					lfile := '';
				end;
			end;
			
			MSG_F_OK:
			begin
				Attr := 0;

				if ( c1^.Checked ) then Attr := Attr or FA_RDONLY;
				if ( c2^.Checked ) then Attr := Attr or FA_HIDDEN;
				if ( c3^.Checked ) then Attr := Attr or FA_ARCH;
				if ( c4^.Checked ) then Attr := Attr or FA_SYSTEM;

				if ( filename<>'' ) then
				begin
					FileSetAttributes(filename,Attr);
					if ( Mode = 1 ) then SaveData2(filename,tl^.Text,TYPE_TEXT);
				end;

				WidgetDispose(PWIDGET(o));
				CloseApp(@Me);
				AppEventHandler := true;
			end;
			
			WM_ABOUT:
			begin
				MessageBox(@Me, 'File properties', 'File properties part of FMS 2\n\n(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.', MBB_OK);
				AppEventHandler := true;
			end;
			

		end; { case }
	end;  { if }

end;

function format_size (size: l_ulong): l_text;
begin
	if (size < 1024) then
		format_size := TextArgs ('%d bytes', [size])
	else if (size < l_ulong(1024*1024)) then
		format_size := TextArgs ('%d KB', [size div 1024])
	else if (size < l_ulong(1024*1024*1024)) then
		format_size := TextArgs ('%d MB', [size div (1024*1024)])
	else
		format_size := TextArgs ('%d GB', [size div (1024*1024*1024)]);
end;

function Main ( argc: integer; argv: l_text ): integer;
var
	l: PLabel;
	b: PButton;
	r: TRect;
	info: PFileInfo;
	bk: PTabBook;
	t: PTab;
	c: PCodec;
	
	tme: timer.PDateTimeRec;
        tm: dos.DateTime;
	v: l_ulong;
	link: l_text;
begin
    AppVersion := ULONG_ID(0,0,0,1);

	if ( argc = 1 ) then Main := 0;
	
	filename := ParamStr(1); //TextDup(argv[1]); 
	if system.Pos('.',filename) > 0 then
	begin
	    ExtractFileExt(filename);
    	if system.Pos('.ln',filename) > 0 then Mode := 1;
	end;
	info := FileGetInfo(filename);
	if not assigned( info ) then
	begin
		filename := '';
		MessageBox(@Me, 'Error', 'Unable to load file propreties.', MBB_OK or MBI_ERROR);
		Main := 0;
	end;

	c := GetCodecFileInfo(info);
	
	
	RectAssign(r,0, 0, 285, 375);
	w := CreateWindow(@Me, r, 'Properties', WF_NORMAL or WF_CENTERED);
	PWIDGET(w)^.AppEvHdl := {$ifdef fpc}@{$endif}AppEventHandler;

	InsertWidget(PWIDGET(DeskTop), PWIDGET(w));

	RectAssign(r, 5, 5, 45, 45);
	v := 0; icon := CreateImageLabel(@Me,r,FileGetIcon2(info,32,v));
	PWIDGET(icon)^.BackgroundColor := COL_3DFACE;
	InsertWidget(PWIDGET(w), PWIDGET(icon));

	RectAssign(r, 50, 0, 250, 20);
	l1 := CreateLabel(@Me,r,'');
	PWIDGET(l1)^.BackgroundColor := COL_3DFACE;
	InsertWidget(PWIDGET(w), PWIDGET(l1));

	RectAssign(r, 50, 20, 250, 40);
	l2 := CreateLabel(@Me,r,'');
	PWIDGET(l2)^.BackgroundColor := COL_3DFACE;
	InsertWidget(PWIDGET(w), PWIDGET(l2));


	RectAssign(r, 5, 350, 95, 370);
	b := CreateButton(@Me,r,'Ok',MSG_F_OK);
	InsertWidget(PWIDGET(w), PWIDGET(b));

	RectAssign(r, 100, 350, 190, 370);
	b := CreateButton(@Me,r,'Cancel',WM_CLOSE);
	InsertWidget(PWIDGET(w), PWIDGET(b));


	RectAssign(r, 5, 50, 280, 345);
	bk := PTabBook(CreateTabBook(@Me,r));
	InsertWidget(PWIDGET(w), PWIDGET(bk));

	t := TabBookAddTab(bk,'Informations',NIL);

	RectAssign(r, 0, 0, 250, 15);
	l3 := CreateLabel(@Me,r,'');
	PWIDGET(l3)^.BackgroundColor := COL_3DFACE;
	InsertWidget(PWIDGET(t), PWIDGET(l3));
	
	RectAssign(r, 0, r.a.y+20, 250, r.b.y+20);
	l7 := CreateLabel(@Me,r,'File type : Unknown binary');
	PWIDGET(l7)^.BackgroundColor := COL_3DFACE;
	InsertWidget(PWIDGET(t), PWIDGET(l7));
	
	RectAssign(r, 0, r.a.y+20, 250, r.b.y+20);
	l4 := CreateLabel(@Me,r,'Created  : (unavaible)');
	PWIDGET(l4)^.BackgroundColor := COL_3DFACE;
	InsertWidget(PWIDGET(t), PWIDGET(l4));

	RectAssign(r, 0, r.a.y+20, 250, r.b.y+20);
	l5 := CreateLabel(@Me,r,'Modified : (unavaible)');
	PWIDGET(l5)^.BackgroundColor := COL_3DFACE;
	InsertWidget(PWIDGET(t), PWIDGET(l5));

	RectAssign(r, 0, r.a.y+20, 250, r.b.y+20);
	l6 := CreateLabel(@Me,r,'Accessed : (unavaible)');
	PWIDGET(l6)^.BackgroundColor := COL_3DFACE;
	InsertWidget(PWIDGET(t), PWIDGET(l6));

	RectAssign(r, 0, r.a.y+20, 120, r.b.y+20);
	c1 := CreateCheckbox(@Me,r,'Read only');
	InsertWidget(PWIDGET(t), PWIDGET(c1));

	RectAssign(r, 125, r.a.y, 250, r.b.y);
	c2 := CreateCheckbox(@Me,r,'Hidden');
	InsertWidget(PWIDGET(t), PWIDGET(c2));

	RectAssign(r, 0, r.a.y+20, 120, r.b.y+20);
	c3 := CreateCheckbox(@Me,r,'Archive');
	InsertWidget(PWIDGET(t), PWIDGET(c3));

	RectAssign(r, 125, r.a.y, 250, r.b.y);
	c4 := CreateCheckbox(@Me,r,'System');
	InsertWidget(PWIDGET(t), PWIDGET(c4));

	if ( Mode<>0 ) then
	begin

		t := TabBookAddTab(bk,'Link',NIL);

		RectAssign(r, 0, 0, 250, 15);
		l := CreateLabel(@Me,r,'Linked file');
		PWIDGET(l)^.BackgroundColor := COL_3DFACE;
		InsertWidget(PWIDGET(t), PWIDGET(l));

		RectAssign(r, 0, r.a.y+20, 225, r.b.y+20);
		tl := CreateTextbox(@Me,r,0);
		InsertWidget(PWIDGET(t), PWIDGET(tl));

		RectAssign(r, 230, r.a.y, 250, r.b.y);
		b := CreateButton(@Me,r,'...',MSG_SELECTFILE);
		InsertWidget(PWIDGET(t), PWIDGET(b));

	end;

	c1^.Checked := (info^.Attributes and FA_RDONLY) = FA_RDONLY;
	c2^.Checked := (info^.Attributes and FA_HIDDEN) = FA_HIDDEN;
	c3^.Checked := (info^.Attributes and FA_ARCH  ) = FA_ARCH;
	c4^.Checked := (info^.Attributes and FA_SYSTEM) = FA_SYSTEM;

	WidgetDrawAll(PWIDGET(w));
	TabBookSelectTab(bk,t);

	if ( Mode = 1 ) then
	begin
		link := l_text(LoadData2(filename,TYPE_TEXT));
		TextBoxSetTextEx(tl,link);
		link := '';
	end;

	LabelSetText(l1,'%s',[info^.Name]);
	LabelSetText(l2,'%s',[info^.VPathName]);
	LabelSetText(l3,'Size: %s', [format_size(info^.Size)]);

	UnpackTime(info^.CTime,tm); move(tm,tme,sizeof(tm)); tme := @tm;
	if assigned( tme ) then
	LabelSetText(l4,'Created: %02d/%02d/%04d %02d:%02d:%02d',[tme^.tm_mday,tme^.tm_mon+1,tme^.tm_year+1900,tme^.tm_hour,tme^.tm_min,tme^.tm_sec]);
	UnPackTime(info^.MTime,tm); move(tm,tme,sizeof(tm)); tme := @tm;
	if assigned( tme ) then
	LabelSetText(l5,'Modified: %02d/%02d/%04d %02d:%02d:%02d',[tme^.tm_mday,tme^.tm_mon+1,tme^.tm_year+1900,tme^.tm_hour,tme^.tm_min,tme^.tm_sec]);
	UnPackTime(info^.ATime,tm); move(tm,tme,sizeof(tm)); tme := @tm;
	if assigned( tme ) then
	LabelSetText(l6,'Accessed: %02d/%02d/%04d %02d:%02d:%02d',[tme^.tm_mday,tme^.tm_mon+1,tme^.tm_year+1900,tme^.tm_hour,tme^.tm_min,tme^.tm_sec]);

	if assigned( c ) then
	begin
		if ( c^.Name <> '' ) then
			LabelSetText(l7,'File type : %s',[c^.Name])
		else
			LabelSetText(l7,'File type : Id 0x%08x',[c^.CodecId]);
	end;
	
	FreeFileInfo(info);
	Main := 1;
end;

procedure Close;
begin
  filename := '';
end;

begin
  Main(1, '');
end.


