(*
////////////////////////////////////////////////////////////////////////////////
//
//	Graphics Extender
//
//	Aura Milestone 3 
//
//	(c) Copyright 2012 Finn Technologies and Chase Finn. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
unit grfx;
{
 .#include 'kernel.h'
 .#include 'allegro/internal/aintern.h'
}
interface

uses
  guitypes,metrics,text,allegro,gdrivers,gfonts;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
  {$link grfx.o}
{$endif}

type
  p_bitmap = AL_BITMAPptr;
  PBITMAP  = Pointer;

function getr(Color: Longint): byte;
function getb(Color: Longint): byte;
function getg(Color: Longint): byte;
function makecol(r,g,b: Longint): Longint;

procedure DrawDotLineVertical ( place: PBITMAP; x, y, h: integer; c: l_color );  {$ifdef LINK_CLIBS}external;{$endif}
procedure DrawDotLineHorizontal ( place: PBITMAP; x, y, w: integer; c: l_color ); {$ifdef LINK_CLIBS}external;{$endif}
procedure DrawDotRect ( place: PBITMAP; r: TRect; c: l_color ); {$ifdef LINK_CLIBS}external;{$endif}

procedure Rect3D ( bmp: p_bitmap; x1, y1, x2, y2, c1, c2: integer ); {$ifdef LINK_CLIBS}external;{$endif}

function FontCharLength ( f: PFONT; ch: Char ): l_ulong; {$ifdef LINK_CLIBS}external;{$endif}
function FontTextLengthEx ( f: PFONT; Txt, tEnd: l_text; p: l_bool ): l_ulong; {$ifdef LINK_CLIBS}external;{$endif}
procedure DrawTextEx ( bmp: p_bitmap; f: PFONT; x, y: l_long; Txt, tEnd: l_text; color: integer; p: l_bool ); {$ifdef LINK_CLIBS}external;{$endif}

function WhereInText ( f: PFONT; x: l_long; Txt, tEnd: l_text ): l_text; {$ifdef LINK_CLIBS}external;{$endif}
function WhereInTextMultiLines ( f: PFONT; x, y: l_long; Txt, tEnd: l_text; p: l_bool ): l_text; {$ifdef LINK_CLIBS}external;{$endif}
procedure LocatePointText ( f: PFONT; Txt, p: l_text; var dy, dx: l_long; pw: l_bool ); {$ifdef LINK_CLIBS}external;{$endif}

procedure DrawNiceTextCenter ( bmp: p_bitmap; f: PFONT; x, y, w: l_long; Txt: l_text; color: integer ); {$ifdef LINK_CLIBS}external;{$endif}
procedure DrawNiceText ( bmp: p_bitmap; f: PFONT; x, y, w: l_ulong; Txt: l_text; color: integer ); {$ifdef LINK_CLIBS}external;{$endif}

procedure TextSize ( f: PFONT; Txt: l_text; var w, h: l_ulong ); {$ifdef LINK_CLIBS}external;{$endif}

procedure ImageGreyScale ( s: p_bitmap );
{$ifdef LINK_CLIBS}external;{$endif}

function ImageNewGreyScaled ( s: p_bitmap ): {$ifdef use_allegro}p_bitmap{$else}PCanvas{$endif};
{$ifdef LINK_CLIBS}external;{$endif}

function ImageNewGreyScaleWithMask ( s: p_bitmap ): {$ifdef use_allegro}p_bitmap{$else}PCanvas{$endif};
{$ifdef LINK_CLIBS}external;{$endif}

procedure GetClip ( bmp: p_bitmap; var r: TRect ); {$ifdef LINK_CLIBS}external;{$endif}


//procedure draw_sprite_greyscale ( dest, src: p_bitmap; d_x, d_y: integer ); {$ifdef LINK_CLIBS}external;{$endif}
//procedure draw_sprite_tile  ( dest, src: p_bitmap; x, y, w, h: integer ); {$ifdef LINK_CLIBS}external;{$endif}

procedure SetClip(bmp: p_bitmap; r: TRect);

procedure DrawIcon16(bmp: p_bitmap; icon: PIcon; x,y: integer);

procedure DrawIcon32(bmp: p_bitmap; icon: PIcon; x,y: integer);

procedure DrawIcon48(bmp: p_bitmap; icon: PIcon; x,y: integer);

implementation
{$ifndef LINK_CLIBS}
{$ifdef OPascal}
uses Sysutils;
{$else}
uses OSysutil;
{$endif}

var	AppVersion: l_ulong;

const
	nUID: l_text    = 'grfx';
	AppName: l_text = 'Graphics Extender';

    TabWidth: l_long = 6;
    
    TAB  = #9;
    CR   = #13;
    LF   = #10;
    CRLF = #13#10;

function getr(Color: Longint): byte;
var c: gdrivers.TColor;
begin
  c.Color := Color;
  getr := c.Red;
end;

function getb(Color: Longint): byte;
var c: gdrivers.TColor;
begin
  c.Color := Color;
  getb := c.Blue;
end;

function getg(Color: Longint): byte;
var c: gdrivers.TColor;
begin
  c.Color := Color;
  getg := c.Green;
end;

function makecol(r,g,b: Longint): Longint;
var c: gdrivers.TColor;
begin
  c.Red := byte(r);
  c.Blue := byte(b);
  c.Green := byte(g);
  makecol := c.Color;
end;

procedure SetClip(bmp: p_bitmap; r: TRect);
begin
  al_set_clip_rect(bmp, r.a.x, r.a.y, r.b.x, r.b.y);
end;

procedure DrawIcon16(bmp: p_bitmap; icon: PIcon; x,y: integer);
begin  
  if assigned( icon ) then
	if assigned( icon ) then
		al_draw_rle_sprite(bmp,icon^.Icon16,x,y); 
end;

procedure DrawIcon32(bmp: p_bitmap; icon: PIcon; x,y: integer);
begin  
  if assigned( icon ) then
	if assigned( icon^.Icon32 ) then
		al_draw_rle_sprite(bmp,icon^.Icon32,x,y); 
end;

procedure DrawIcon48(bmp: p_bitmap; icon: PIcon; x,y: integer);
begin
  if assigned( icon ) then
	if assigned( icon^.Icon48 ) then
		al_draw_rle_sprite(bmp,icon^.Icon48,x,y); 
end;


(**
*	NAME: Rect3D
*	DESCRIPTION: Draws a beveled rectangle
*)
procedure Rect3D ( bmp: p_bitmap; x1, y1, x2, y2: integer; c1, c2: l_color );
begin
	al_vline(Bmp,x1,y1,y2,c1);
	al_hline(Bmp,x1,y1,x2,c1);
	al_vline(Bmp,x2,y1,y2,c2);
	al_hline(Bmp,x1,y2,x2,c2);
end;

(**
*	NAME: DrawDotLineVertical
*	DESCRIPTION: Draws a vertical dotted line seperated every pixel
*					on the specified bitmap.
*)
procedure DrawDotLineVertical ( place: PBITMAP; x, y, h: integer; c: l_color );
var a,b: integer;
begin
	a := MIN(y,h);
	b := MAX(y,h);

	while ( a <= b ) do
	begin
		al_putpixel(place, x, a, c);
		a := a + 2;
	end;
end;

(**
*	NAME: DrawDotLineHorizontal
*	DESCRIPTION: Draws a horizontal dotted line seperated every pixel
*					on the specified bitmap.
*)
procedure DrawDotLineHorizontal ( place: PBITMAP; x, y, w: integer; c: l_color );
var a,b: integer;
begin
	a := MIN(x,w);
	b := MAX(x,w);

	while ( a <= b ) do
	begin
		al_putpixel(place, a, y, c);
		a := a + 2;
	end;
end;

(**
*	NAME: DrawDotrect
*	DESCRIPTION: Draws a dotted rectangle seperated every pixel
*					on the specified bitmap.
*)
procedure DrawDotRect ( place: PBITMAP; r: TRect; c: l_color );
begin
	DrawDotLineHorizontal ( place, r.a.x, r.a.y, r.b.x, c );
	DrawDotLineVertical	( place, r.a.x, r.a.y, r.b.y, c );
	DrawDotLineHorizontal	( place, r.a.x, r.b.y, r.b.x, c );
	DrawDotLineVertical ( place, r.b.x, r.a.y, r.b.y, c );
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function FontCharLength ( f: PFONT; ch: Char ): l_ulong;
begin	
    if ( ch <> CR ) then
		FontCharLength := f^.CharWidth(ch)
	else
		FontCharLength := 0;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TextGetTabWidth ( f: PFONT; sx, x: l_long ): integer;
var w: l_long;
begin
	w := FontCharLength(f,' ')*TabWidth;
	TextGetTabWidth := sx+((((x-sx) div w)+1)*w)-x;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function FontTextLengthEx ( f: PFONT; Txt, tEnd: l_text; p: l_bool ): l_ulong;
var l,ix: l_ulong;
begin
        l := 0; ix := 1;
	while ( (ix <= Length(Txt)) and (Txt <> tEnd) ) do
	begin	
		if ( Txt[ix] = TAB ) then
		 	l := l + TextGetTabWidth(f,0,l)
		else if ( p ) then
			l := l + f^.CharWidth('*')
		else if ( Txt[ix] <> CR ) then
			l := l + f^.CharWidth(Txt[ix]);
			
	    inc(ix);
	end;
	FontTextLengthEx := l;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DrawTextEx ( bmp:p_bitmap; f:PFONT; x, y: l_long; Txt, tEnd: l_text; color: integer; p: l_bool ); 
var sx,ix: l_long; canvas: PCanvas;
begin
	sx := x; ix := 1;
        canvas := New(PCanvas, Init(bmp^.w, bmp^.h, {$ifdef DOS}4{$else}32{$endif}));
	while (ix <= Length(Txt)) and (Txt[ix] <> tEnd)  do
	begin
		if ( Txt[ix] = TAB ) then
		 	x := x + TextGetTabWidth(f,sx,x)
		else if ( Txt[ix] <> CR ) then
		begin
			if ( p ) then
                        begin
				f^.OutChar( canvas, x, y, '*', color );
                                x := x + f^.CharWidth('*');
                        end
                        else
                        begin
				f^.OutChar( canvas, x, y, Txt[ix], color );
                                x := x + f^.CharWidth(Txt[ix]);
                        end;
                end;
	  inc(ix);
    end;
    Dispose(canvas,done);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function WhereInText ( f: PFONT; x: l_long; Txt, tEnd: l_text ): l_text;
var ix: integer;
begin	
    ix := 1;
    if ( x <= 0 ) then begin WhereInText := Txt; exit; end;
	while ( (ix <= Length(txt)) and (Txt <> tEnd) ) do
	begin
		if ( Txt[ix] <> CR ) then
			x := x - f^.CharWidth(Txt[ix]);
		if ( x <= 0 ) then WhereInText := Txt;
	  inc(ix);
    end;
    WhereInText := Txt;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function WhereInTextMultiLines ( f: PFONT; x, y: l_long; Txt, tEnd: l_text; p: l_bool ): l_text;
var w,sx,ix: l_ulong;
begin
	w := 0; sx := x; ix := 1;
	y := y - f^.TextHeight+1;
	if ( (x <= 0) and (y <= 0) ) then WhereInTextMultiLines := '1';
	while ( Txt[ix] <> tEnd ) do
	begin	
		if ( Txt[ix] = LF ) then begin
			if ( y <= 0 ) then WhereInTextMultiLines := IntToStr(ix+w);
			y := y-f^.TextHeight+1; ix := 1;
		end;
		
		if ( y <= 0 ) then 
		begin
			if ( Txt[ix] <> CR ) then
			if ( Txt[ix] <> LF ) then 
			begin
				if ( Txt[ix] = TAB ) then
					w := TextGetTabWidth(f,sx,x)
				else if ( p ) then
					w := f^.CharWidth('*')
				else
					w := f^.CharWidth(Txt[ix]);
				if ( x <= w div 2 ) then WhereInTextMultiLines := IntToStr(ix+w);
				if ( Txt[ix] <> LF ) then x := x - w;
			end else if ( x <= 0 ) then
				WhereInTextMultiLines := Txt;
		end;
  	
  	    inc(ix);
    end;
  WhereInTextMultiLines := IntToStr(ix+w);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TextSize ( f: PFONT; Txt: l_text; var w, h: l_ulong );
var x, y, ix: integer;
begin
	x := 0; y := 0; ix := 1;
    h := f^.TextHeight;
	w := 0;
	while ( ix <= length(Txt) ) do
	begin
		if ( Txt[ix] = LF ) then
		begin
			h := h + f^.TextHeight+1;
			x := 0;	
		end else if ( Txt[ix] <> CR ) then begin
			x := x + f^.CharWidth(Txt[ix]);
		end;
		w := max(w,x);
  		inc(ix);
  	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure LocatePointText ( f: PFONT; Txt, p: l_text; var dy, dx: l_long; pw: l_bool );
var x,y,ix: integer;
begin
	x := 0; y := 0; ix := 1;
	while ( ix <= Length(Txt) ) do
	begin
		if TextCompare(Txt, p) = 0 then
		begin
			dy := y;
			dx := x;
			exit;
		end;
		if ( Txt[ix] <> CR ) then
		if ( Txt[ix] = LF ) then
		begin
			y := y + f^.TextHeight+1;
			x := 0; ix := 1;
		end else
			if ( Txt[ix] = TAB ) then
				x := x + TextGetTabWidth(f,0,x)
			else if ( pw ) then
				x := x + f^.CharWidth('*')
			else
				x := x + f^.CharWidth(Txt[ix]);
			
  	   inc(ix);
    end;
	if ( Txt = p ) then
	begin
		dy := y;
		dx := x;		
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DrawNiceTextCenter ( bmp: p_bitmap; f: PFONT; x, y, w: l_long; Txt: l_text; color: l_int );
var l,ix: l_ulong; canvas: PCanvas;
begin
        canvas := New(PCanvas, Init(bmp^.w, bmp^.h, {$ifdef DOS}4{$else}32{$endif}));

	l := f^.TextWidth(Txt);
	if ( l > w ) then
	begin
		x := x - (w div 2);
		w  := w+x-f^.TextWidth('...');
		while ( (ix <= Length(Txt)) and (x+f^.CharWidth(Txt[ix]) < w) ) do
                begin
		  f^.OutText( canvas, x, y, Txt, color );
                  x := x + f^.TextWidth(Txt);
	  	  inc(ix);
  	        end;
		{textout_ex(bmp, f, '...', x, y, color, -1);}
                f^.OutText(canvas, x, y, '...', color);
	end else
	{textout_ex(bmp, f, Txt, x - l div 2, y, color, -1);}
        f^.OutText(canvas, x,y,Txt, color);

        Dispose(canvas,done);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DrawNiceText ( bmp: p_bitmap; f: PFONT; x, y, w: l_ulong; Txt: l_text; color: l_int );
var l,ix: l_ulong; canvas: PCanvas;
begin
    canvas := New(PCanvas, Init(bmp^.w, bmp^.h, {$ifdef DOS}4{$else}32{$endif}));
    ix := 1;
	l := f^.TextWidth(Txt);
	if ( w <= f^.TextWidth('...') ) then exit;
	if ( l > w ) then begin
		w  := w+x-f^.TextWidth('...');
		while ( (ix <= Length(Txt)) and (x+f^.CharWidth(Txt[ix]) < w) ) do begin
		  f^.OutText( canvas, x, y, Txt, color );
                  x := x + f^.TextWidth(Txt);
	  	  inc(ix);
  	    end;
  	    {textout_ex(bmp, f, '...', x, y, color, -1);}
            f^.OutText(canvas,x,y,'...',color);
	end else
	    {textout_ex(bmp, f, Txt, x , y, color, -1);}
            f^.OutText(canvas,x,y,Txt,color);
    Dispose(canvas,done);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure GetClip ( bmp: p_bitmap; var r: TRect );
begin	
    if ( bmp^.clip ) then
	begin
		r.a.x := bmp^.cl;
		r.a.y := bmp^.ct;
		r.b.x := bmp^.cr;
		r.b.y := bmp^.cb;
	end else begin
		r.a.x := 0;
		r.a.y := 0;
		r.b.x := 0;
		r.b.y := 0;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ImageGreyScale ( s: p_bitmap );
var x,y: l_ulong; i,c: integer; canvas: PCanvas;
begin
	x := 0; y := 0; 
	i := 0; c := 0;
        {$ifndef use_allegro}
        canvas := New(PCanvas, Init(s^.w, s^.h, {$ifdef DOS}4{$else}32{$endif}));
        {$endif}
	while ( y < s^.h ) do
	begin
		x := 0;
		while ( x < s^.w ) do
		begin
                        {$ifdef use_allegro}
			c := al_getpixel(s,x,y);
                        {$else}
                        c := canvas^.getpixel(x,y);
                        {$endif}
			i := (getr(c)+getg(c)+getb(c)) div 3;
                        {$ifdef use_allegro}
                        al_putpixel(s,x,y,makecol(i,i,i));
                        {$else}
                        canvas^.putpixel(x,y,makecol(i,i,i));
                        {$endif}
			inc(x);
		end;
		inc(y);
	end;
        {$ifndef use_allegro}
        Dispose(canvas, done);
        {$endif}
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function ImageNewGreyScaled ( s: p_bitmap ): {$ifdef use_allegro}p_bitmap{$else}PCanvas{$endif};
var
  b: {$ifdef use_allegro}p_bitmap{$else}PCanvas{$endif};
  x,y: l_ulong;
  i,c: longint;
  canvas: PCanvas;
  _width,_height: l_ulong;
begin
        {$ifdef use_allegro}
        b := al_create_bitmap(s^.w,s^.h);
        {$endif}
	x := 0; y:=0; 
	i := 0; c:=0;
	{$ifndef use_allegro}
        canvas := New(PCanvas, Init(s^.w, s^.h, {$ifdef DOS}4{$else}32{$endif}));
        b := canvas;
        _width := canvas^.Width;
        _height := canvas^.Height;
        {$else}
        _width := b^.w;
        _height := b^.h;
        {$endif}
	while ( y < _height ) do
	begin
		x := 0;
		while ( x < _width ) do
		begin
                        {$ifdef use_allegro}
                        c := al_getpixel(s,x,y);
                        {$else}
                        c := canvas^.getpixel(x,y);
                        {$endif}
			i := (getr(c)+getg(c)+getb(c)) div 3;
                        {$ifdef use_allegro}
                        al_putpixel(b,x,y,makecol(i,i,i));
                        {$else}
			canvas^.putpixel(x,y,makecol(i,i,i));
                        {$endif}
			inc(x);
		end;
		inc(y);
	end;
	{$ifndef use_allegro}
        Dispose(canvas, done);
        {$endif}
	ImageNewGreyScaled := b;
end;

function ImageNewGreyScaleWithMask ( s: p_bitmap ): {$ifdef use_allegro}p_bitmap{$else}PCanvas{$endif};
var
  b: {$ifdef use_allegro}p_bitmap{$else}PCanvas{$endif};
  i,c: Longint;
  x,y,mask: l_ulong;
  canvas: PCanvas;
  _width,_height: l_ulong;
begin
        {$ifdef use_allegro}
	b := al_create_bitmap (s^.w, s^.h);
        _width := b^.w;
        _height := b^.h;
        {$else}
        canvas := New(PCanvas, Init(s^.w, s^.h, {$ifdef DOS}4{$else}32{$endif}));
        b := canvas;
        _width := canvas^.Width;
        _height := canvas^.Height;
        {$endif}
	x := 0; y := 0; 
	i := 0; c := 0; mask := makecol (255, 0, 255);
	
	while (y < _height) do
	begin
		x := 0;
		
		while (x < _width) do
		begin
                        {$ifdef use_allegro}
                        c := al_getpixel(s,x,y);
                        {$else}
                        c := canvas^.getpixel(x,y);
                        {$endif}
			if (c <> mask) then
			begin
				i := (getr(c)+getg(c)+getb(c)) div 3;
				i := makecol (i,i,i);
			end
			else i := mask;
                        {$ifdef use_allegro}
			al_putpixel(b,x,y,i);
                        {$else}
                        canvas^.putpixel(x,y,i);
                        {$endif}
			inc(x);
		end;

		inc(y);
	end;
        {$ifndef use_allegro}
	Dispose(canvas, done);
        {$endif}
	ImageNewGreyScaleWithMask := b;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function LibMain ( argc: integer; argv: l_text ): l_bool;
begin
   (*
	APPEXPORT(Rect3D);
	APPEXPORT(DrawDotLineHorizontal);
	APPEXPORT(DrawDotLineVertical);
	APPEXPORT(DrawDotRect);

	APPEXPORT(FontCharLength);
	APPEXPORT(FontTextLengthEx);
	APPEXPORT(DrawTextEx);
	APPEXPORT(WhereInText);
	APPEXPORT(WhereInTextMultiLines);
	APPEXPORT(LocatePointText);
	APPEXPORT(DrawNiceTextCenter);
	APPEXPORT(DrawNiceText);
	APPEXPORT(GetClip);
	APPEXPORT(TextSize);
	APPEXPORT(ImageGreyScale);
	APPEXPORT(ImageNewGreyScaled);
	APPEXPORT(ImageNewGreyScaleWithMask);
   *)
	LibMain := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Close;
begin

end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
   AppVersion := ULONG_ID(0,5,0,0);

{$endif}  {LINK_CLIBS}
end.
