(**
*	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
*
*	FILE:			registry.c
*
*	PROJECT:		Phoenix engine - Core
*
*	DESCRIPTION:	Used for reading and writing into the system registry.
*
*	CONTRIBUTORS:
*					Lukas Lipka
*
*	TODO:			@ Add writing support
*
*	BUG:			@
*
*	MISTAKE:		@
*
*	IDEA:			@
*)
unit ini;
{
 .#include <string.h>
 .#include <stdio.h>
 .#include <stdlib.h>
 .#include "kernel.h"
 .#include "ini.h"
}

interface

uses
  guitypes,vfile,list,registry;

{$i cdefs.inc}

{$ifdef link_clibs}
    {$LINK kernel.o}
    {$LINK ini.o}
{$endif}

const REGISTRY_FILE = './SYSTEM/REGS/xsystem.reg';

function RGB(r,g,b: byte): Longword;

var
  cBuffer : array[0..{$ifdef OPascal}65535{$else}255{$endif}] of l_char;
  cBuff2  : array[0..255] of l_char;

function INISectionExists (szFileName, szSection: l_text): l_bool; {$ifdef link_clibs}external;{$endif}
function RegistryGetKey ( path,key: l_text ): l_text;              {$ifdef link_clibs}external;{$endif}
function RegistryGetInteger ( path,key: l_text ): Integer;         {$ifdef link_clibs}external;{$endif}
(*function RegistryGetColor(l_char *name): l_color;                 {$ifdef link_clibs}external;{$endif}*)
function INIGet ( filename, path, key: l_text ): l_text;           {$ifdef link_clibs}external;{$endif}
function INIGetInteger(szFileName, szSection, szItem: l_text; iDefault: integer): integer; 
{$ifdef link_clibs}external;{$endif}

procedure INIGetText(szFileName, szSection, szItem: l_text; var szValue: array of char; szDefault: l_text);
{$ifdef link_clibs}external;{$endif}

function INIGetNew(szFileName, szSection, szItem, szDefault: l_text): l_text;
{$ifdef link_clibs}external;{$endif}

function INIGetColor(szFileName, szSection, szItem: l_text; colDefault: l_color): Longword;
{$ifdef link_clibs}external;{$endif}

implementation
{$ifndef link_clibs}
uses
{$ifdef OPascal}
   Sysutils;
{$else}
   OSysutil;
{$endif}

const
  bufsize = {$ifdef OPascal}65536{$else}1024{$endif};

type
  rgbr = record
    case Longword of
     0: (r,g,b,a: byte);
     1: (c: l_color);
  end;

function RGB(r,g,b: byte): Longword;
var rec: rgbr;
begin
  rec.r := r; rec.g := g; rec.b := b;
  RGB := rec.c;
end;


type
  PFOpenRec = ^TFOpenRec;
  TFOpenRec = record
     f: file;
     opend: l_bool;
  end;

{$ifdef OPascal}
function fopen(Filename: l_text; mode: integer): TFOpenRec;
var f: file; rec: TFOpenRec;
begin
  Assign(f, Filename); 
  {$i-}
  Reset(f, 1);
  {$i+}
  if ioresult = 0 then
  begin
    rec.f := f;
    rec.opend := true;
    fopen := rec;
  end
  else
  begin
    rec.opend := false;
    fopen := rec;
  end;
end;
{$else}
function fopen(Filename: l_text; mode: integer): PFOpenRec;
var f: file; rec: PFOpenRec;
begin
  New(rec);
  Assign(f, Filename);
  {$i-}
  Reset(f, 1);
  {$i+}
  if ioresult = 0 then
  begin
    rec^.f := f;
    rec^.opend := true;
    fopen := rec;
  end
  else
  begin
    rec^.opend := false;
    fopen := rec;
  end;
end;
{$endif}

procedure fclose(var f: file);
begin
  Close(f);
end;

function fgets(var s: array of char; Count: integer; var f: file): l_bool;
var readed: Integer;
begin
  BlockRead(f,s,Count,readed);
end;

function CharsToString(a: array of char): l_text;
var i: integer; s: l_text;
begin
  for i:=0 to high(a) do
    if a[i] <> #0 then s := s + a[i] else break;
  CharsToString := s;
end;

function strlen(a: array of char): longint;
var cnt: Longint;
begin
  cnt := 0;
  while (a[cnt] <> #0) and (cnt <= high(a)) do inc(cnt);
  strlen := cnt;
end;


function stricmp(c: array of char; s: l_text): Integer;
var v: l_text; i,j: integer;
begin
  v := ''; i := 1; j := 0;
  for j := 0 to high(c) do
    if c[j] <> #0 then v := v + c[j] else exit;
  if v < s then stricmp := -1 else
  if v = s then stricmp :=  0 else stricmp := +1;
end;


procedure RemoveSpaces(szString: l_text);
begin
  {
	while (szString[1] = ' ') do
		move(szString[2], szString[1], length(szString));
	while (szString[Length(szString)-1] = ' ') do
		szString[Length(szString)-1] := #0;
  }
  while szString[1] = ' ' do system.Delete(szString, 1, 1);
  while szString[Length(szString)] = ' ' do system.Delete(szString, Length(szString), 1);
end;

function INISectionExists (szFileName, szSection: l_text): l_bool;
var f: file; rec: TFOpenRec;
begin
	if (szFileName='') or (szSection='') then
		INISectionExists := false;

	rec := fopen(szFileName, FSF_READ){$ifndef OPascal}^{$endif};
        f := rec.f;
	if not rec.opend then
		INISectionExists := false;

	while not(eof(f)) do
	begin
		fgets(cBuffer, bufsize, f);

		if (cBuffer[0] = ';') then break;

		if (cBuffer[0] = '[') and (cBuffer[strlen(cBuffer)-2] = ']') then { note \r\n }
		begin
			cBuffer[strlen(cBuffer)-2] := #0;

			if stricmp(cBuffer[1], PChar(szSection))=0 then
				begin INISectionExists := true; fclose(f); exit; end;
		end;
	end;

	fclose(f);

	INISectionExists := false;
end;

procedure INIGetText(szFileName, szSection, szItem: l_text; var szValue: array of char; szDefault: l_text);
var
  rec: TFOpenRec; equalpos: integer; eqplen: integer; temp: l_text;
begin
	if (szFileName<>'') and (szSection<>'') and (szItem<>'') and (szValue<>'') then
	begin

    { DebugMessage('INIGetText: Open %s...', [szFileName]); }
	rec := fopen(szFileName, FSF_READ){$ifndef OPascal}^{$endif};

	if not rec.opend then
	begin
		if ( szDefault <> '' ) then
                begin
                        move(szDefault[1], szValue[0], Length(szDefault));
                end
                else
			fillchar(szValue, sizeof(szValue), 0);

		exit;
	end;

	while not(eof(rec.f)) do
	begin
		fgets(cBuffer, bufsize, rec.f);

		if (cBuffer[0] = ';') then break;

		if (cBuffer[0] = '[') and (cBuffer[strlen(cBuffer)-2] = ']') then { note \r\n }
		begin
			cBuffer[strlen(cBuffer)-2] := #0;

			if stricmp(cBuffer[1], szSection)=0 then
			begin
				while not(eof(rec.f)) do
				begin
					
					fgets(cBuffer, bufsize, rec.f);

					if (cBuffer[0] = '[') and (cBuffer[strlen(cBuffer)-2] = ']') then break;

					equalpos := system.pos('=', String(cBuffer));

					if (equalpos > 0) then
					begin

						RemoveSpaces(CharsToString(cBuffer));

						if stricmp(cBuffer, szItem)=0 then
						begin
							
							fclose(rec.f);
							inc(equalpos);
							{
							equalpos[strlen(equalpos)-1] := 0;
							RemoveSpaces(equalpos);							
							eqplen = strlen(equalpos);
                            }
							if (eqplen > 1) and (cbuffer[equalpos] = '"') and (cbuffer[eqplen-1] = '"') then
							begin
								{
                                memmove(equalpos, equalpos+1, eqplen);
								eqplen -= 2;
								equalpos[eqplen] := 0;
								}
								temp := system.copy(szValue,equalpos,Length(szValue)-equalpos);
                                                                move(temp[1], szValue[0], length(temp));
							end;

							{strcpy(szValue, equalpos);}

							{return;}
						end;
					end;
				end;
			end;
		end;
	end;

	if (szDefault <> '') then
		move(szDefault[1], szValue[0], Length(szDefault))
	else
		fillchar(szValue, sizeof(szValue), 0);

	end;
		fclose(rec.f);
end;

function INIGetNew(szFileName, szSection, szItem, szDefault: l_text): l_text;
var 
	value: array[0..{$ifdef OPascal}65535{$else}255{$endif}] of char; S: String;
begin
	INIGetText(szFileName, szSection, szItem, value, szDefault);

	INIGetNew := StrPas(value);
end;

function INIGetInteger( szFileName, szSection, szItem: l_text; iDefault: l_int ): Integer;
begin
	if (szFileName = '') or (szSection = '') or (szItem = '') then
		INIGetInteger := iDefault;

	INIGetText(szFileName, szSection, szItem, cBuff2, '\001\002\003\004');

	if stricmp(cBuff2, '\001\002\003\004') = 0 then
		INIGetInteger := iDefault
	else
		 INIGetInteger := StrToInt(CharsToString(cBuff2));
end;

function INIGetColor(szFileName, szSection, szItem: l_text; colDefault: l_color): Longword;
var
  i,j : l_int;
  _rgb: array[0..9] of rgbr;
begin
	if (szFileName = '') or (szSection = '') or (szItem = '') then
		INIGetColor := RGB(0, 0, 0);

	INIGetText(szFileName, szSection, szItem, cBuff2, '001,002,003,004');

	if stricmp(cBuff2, '001,002,003,004')=0 then
		INIGetColor := colDefault
	else
	begin
		i := 0; 
        j := 0;
		fillchar(_rgb, sizeof(_rgb), 0);

		for i := 0 to 9 do
			if (cBuff2[i] = ',') then
			begin
				if (j = 2) then
					break
				else
					inc(j);
			end
			else if (cBuff2[i] <> ' ') then ;
			{	_rgb[j][strlen(rgb[j])] = cBuff2[i]; }

		INIGetColor := 0;
	end;
end;

function INIGet ( filename, path, key: l_text ): l_text;
begin
	IniGet := INIGetNew(filename, path, key, '');
end;

function RegistryGetKey ( path, key: l_text ): l_text;
begin
	RegistryGetKey := INIGetNew(REGISTRY_FILE, path, key, '');
end;

function RegistryGetInteger ( path, key: l_text ): integer;
begin
	RegistryGetInteger := INIGetInteger(REGISTRY_FILE, path, key, 0);
end;


{$endif}
end.
