(*
////////////////////////////////////////////////////////////////////////////////
//
//  I/O Dialogs - Core File
//
//  (c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
unit iodlg;
(*
  #include "kernel.h"
  #include "iodlg.h"
  #include "label.h"
  #include "window.h"
  #include "button.h"
  #include "textbox.h"
  #include "combobox.h"
  #include "fms2.h"
  #include "clipboard.h"
  #include "listbox.h"
  #include "checkbox.h"
  #include "string.h"
*)
interface

uses
  guitypes,dms,fms2,vfile,events,list,dynld,text,metrics,widget,window,listview,
  treeview,colsel,progress,scrllbar,menu,keyboard,mousedrv,clipbrd,button,image,
  listbox,checkbox,textbox,combobox,ulabel,skin,timer,allegro;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
  {$LINK kernel.o}
  {$LINK iodlg.o}
  {$LINK label.o}
  {$LINK window.o}
  {$LINK button.o}
  {$LINK textbox.o}
  {$LINK combobox.o}
  {$LINK fms2.o}
  {$LINK clipboard.o}
  {$LINK listbox.o}
  {$LINK checkbox.o}
  {$LINK string.o}
{$endif}

type
  PTreeDirview = ^TTreeDirview;
  PDirview     = ^TDirview;
  PFileTypes   = ^TFileTypes;

  TFileTypes = record

	Title : l_text;

	Extentions : PList;

	Next : PFileTypes;

  end;

  TDirview = record

	o: TListview;

	Directory: l_text;

	Flags: l_ulong;

	Extentions: PList;

	Dir: PFileInfo;

  end;


  PDirviewItem = ^TDirviewItem;
  TDirviewItem = record

	o: TListviewItem;

	f: PFileInfo;

  end;

  DIRVIEWITEM = PDirviewItem;
  TTreeDirview = record

	o      : TTreeView;

	Msg    : l_ulong;

	Linked : PDirview;

  end;

  PTreeDirviewItem = ^TTreeDirviewItem;
  TTreeDirviewItem = record

	o: TTreeItem;

	f: PFileInfo;

  end;
  TREEDIRVIEW = PTreeDirview;
  TREEDIRVIEWITEM = PTreeDirviewItem;


{ Flag - disables the parent dir icon when set }
const
 DVF_NOPARICON		= $00000001;
 DVF_MULTISELECT        = $00000002;
 DVF_PREDRAG            = $00000004;
                                      
 DVF_ORDERBYNAME        = $00000008;
 DVF_ORDERBYSIZE        = $00000010;
 DVF_ORDERBYMTIME       = $00000020;
 DVF_ORDERBYCTIME       = $00000040;
 DVF_ORDERBYATIME       = $00000080;
                                      
{ 0xFFFF51XX Reserved to I/O Dialogs}
 MSG_IODLG_ITEMFILE_VAL	= $FFFF5100;
 MSG_IODLG_ITEMFILE_SEL	= $FFFF5102;
 MSG_IODLG_OPENPARENT	= $FFFF5104;

 MSG_IODLG_BOX_OK    	= $FFFF5105;
 MSG_IODLG_ITEMFILE_MENU= $FFFF5106;

 MSG_IOMENU_OPEN	= $FFFF5108;
 MSG_IOMENU_PROPERTIES	= $FFFF5109;
 MSG_IOMENU_DELETE	= $FFFF510A;

 MSG_IOMENU_COPY	= $FFFF510B;
 MSG_IOMENU_CUT		= $FFFF510C;
 MSG_IOMENU_PASTE	= $FFFF510D;

 MSG_IODLG_NEWDIR	= $FFFF510E;
 MSG_IOMENU_RENAME	= $FFFF510F;

 MSG_TREEVIEW_ITEM	= $FFFF5110;
 MSG_TREEVIEW_ITEM_MENU	= $FFFF5111;
 MSG_TREEVIEW_EXPEND	= $FFFF5112;

(**
 * Flags for IOBox()
 *)
 IOBOX_SAVE		= $00000001;
 IOBOX_OPEN		= $00000002;

function IOBox ( Title:l_text; Flags:l_ulong; var Dir:l_text; Types:PFileTypes; FExists:l_bool ): l_text;
function CreateDirview ( App:PApplication; r:TRect; dir:l_text; Extentions:PList; Flags:l_ulong ): PDirview;
procedure DirviewOpenDirectory ( o:PDirview; dir:l_text );
procedure TreeDirViewOpenDir ( o:PTreeDirview; dir:l_text );
function GenerateFileRefs ( f:PFileInfo ): PList;
function CollectFilesFromRef ( l: PList ): PList;
procedure CopyFilesFromList ( l: PList; Dest: l_text );
procedure DeleteFilesFromList ( l: PList );

procedure FreeFileTypes ( o:PFileTypes );
function NewFileTypes ( Title:l_text; txtexts:l_text; Next:PFileTypes ): PFileTypes;

function CreateTreeDirview ( App: PApplication; r: TRect ): PTreeDirview;

implementation
{$ifndef LINK_CLIBS}
uses
  debug;
(*////////////////////////////////////////////////////////////////////////////////*)
var
  AppVersion: l_ulong;

const
  AppName: l_text = 'I/O Dialogs';
  nUID   : l_text = 'iodlg';
  NeededLibs: array[0..11] of l_text = (
    'widget','listview','fms2','textbox','combobox',
    'button','window','clipbrd','listbox','ulabel','checkbox',''
  );

const
 ParentDirIcon: PIcon = NIL;
 GlobalDir: l_text = '';

var
 DirOpenMSG : l_ulong;
 IconOpen   : PImage;
 IconSave   : PImage;
 IconAdd    : PImage;
 IconUp     : PImage;
 IconRemove : PImage;
 IconCancel : PImage;

 TreeDirviewClass: PWidgetClass;
 DirviewClass: PWidgetClass;


(*////////////////////////////////////////////////////////////////////////////////*)
(*// File types                                                                   *)
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeFileTypes ( o:PFileTypes );
begin
        if ( o^.Title <> '' ) then o^.Title := '';
        if assigned( o^.Extentions ) then FreeList(o^.Extentions);
        if assigned( o^.Next ) then FreeFileTypes(o^.Next);
        Dispose(o);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function NewFileTypes ( Title:l_text; txtexts:l_text; Next:PFileTypes ): PFileTypes;
var o: PFileTypes; s,e,n: l_text; spos,npos,epos: l_ulong;
begin
        New(o); spos := 0; npos := 0; epos := 0;
        if not assigned( o ) then NewFileTypes := NIL else
        begin
          fillchar(o^,sizeof(TFileTypes),0);

          o^.Title := TextDup(Title);

          if ( txtexts <> '' ) then begin
                s := txtexts; e := txtexts;
                o^.Extentions := NewList;
                repeat
                        spos := TextChr(e,',');
                        if ( spos > 0 ) then begin
                                n := TextNDup(e,spos-epos);
                                ListAdd(o^.Extentions,n,NIL,NIL);
                                n := '';
                                epos := spos+1;
                        end else
                          ListAdd(o^.Extentions,e,NIL,NIL);
                until ( spos = 0 );
          end;
          o^.Next := Next;
          NewFileTypes := o;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function FileTypesAt ( o: PFileTypes; index: l_ulong ): PFileTypes;
begin
        if not assigned( o ) then FileTypesAt := NIL else
        if ( index <= 0 ) then FileTypesAt := o else
        FileTypesAt := FileTypesAt(o^.Next,index-1);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Standart IODlg Actions                                                       *)
(*////////////////////////////////////////////////////////////////////////////////*)
function IODlgDropAccept ( DestItemInfo:PFileInfo; aType:l_ulong; var Data; SupportedMethods: l_ulong; var Method: l_ulong ): l_bool;
var
 MoveThem: l_bool;
 a,b: PListItem;
 sc: PSubCodec;
begin
        DebugMessage ('Drag n Drop: IODlgDropAccept: %s', [DestItemInfo^.Name] );

        if not((DestItemInfo^.Attributes and FA_DIREC) = FA_DIREC) then IODlgDropAccept := false else

        if ( aType = DTYPE_FILEREFLIST ) then begin
                MoveThem := true;
                b := PList(Data)^.Last;
                a := PList(Data)^.Last;
                if assigned( a ) then repeat
                        if ( PFILEREF(a^.Data)^.Drive <> DestItemInfo^.Drive ) then
                                MoveThem := false;
                        a := a^.Next;
                until ( (a = b) and MoveThem );
                if ( MoveThem and ((SupportedMethods and DMETHOD_MOVE)=DMETHOD_MOVE)) then
                        Method := DMETHOD_MOVE
                else
                        Method := DMETHOD_COPY;
                IODlgDropAccept := true;
        end else begin
                sc := GetDefaultSubCodec(aType);
                if assigned( sc ) then
                        if ( assigned(sc^.Save) or assigned(sc^.SaveToFile) ) then
                                if ( SupportedMethods and DMETHOD_COPY ) = DMETHOD_COPY then begin
                                        Method := DMETHOD_COPY;
                                        IODlgDropAccept := true;
                                end;
        end;
        IODlgDropAccept := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure IODlgDropDataToFileInfo ( DestItemInfo: PFileInfo; var Data; aType,Method: l_ulong );
var
  FileExtention,Filename,Name,nname: l_text;
  a,b: PListItem; l,nl: PList;
  nb: l_ulong;
  sc: PSubCodec; c: PCodec;
  dest,f: PFileInfo;
begin
        if ( aType = DTYPE_FILEREFLIST ) then begin

                l := Pointer(Data);

                if ( Method = DMETHOD_MOVE ) then begin {// Section to improve !}
                        dest := DestItemInfo;
                        b := l^.Last; a := l^.Last;
                        if assigned( a ) then repeat
                                nname := FileNameToPath(dest^.LocalName,PFILEREF(a^.Data)^.SName);

                                if ( PFILEREF(a^.Data)^.Attributes and FA_DIREC ) = FA_DIREC then begin
                                        if assigned( dest^.Drive^.Driver^.DirRename ) then
                                                dest^.Drive^.Driver^.DirRename(dest^.Drive,PFILEREF(a^.Data)^.LocalName,nname);
                                end else begin
                                        if assigned( dest^.Drive^.Driver^.FileRename ) then
                                                dest^.Drive^.Driver^.FileRename(dest^.Drive,PFILEREF(a^.Data)^.LocalName,nname);
                                end;
                                nname := '';
                                a := a^.Next;
                        until ( a = b );
                end else if ( Method = DMETHOD_COPY ) then begin
                        nl := CollectFilesFromRef(l);
                        if assigned( nl ) then begin   {// was l....now right? Florian Xaver }
                          CopyFilesFromList ( nl, DestItemInfo^.VPathName );
                          FreeList(nl);
                        end;
                end;

        end else begin
                sc := GetDefaultSubCodec(aType);
                if assigned( sc ) then begin
                        if ( assigned(sc^.Save) or  assigned(sc^.SaveToFile) ) then begin

                                FileExtention := 'dat';
                                nb := 0;
                                if ( sc^.CodecId > 0 ) then begin
                                        c := GetCodec(sc^.CodecId);
                                        if assigned( c ) then
                                                if( c^.DefaultExtention <> '' ) then
                                                        FileExtention := c^.DefaultExtention;
                                end else begin {// May be a generic sub codec }
                                        c := GetCodec(aType); {// Most of time, generic codecs has as codec id the type id }
                                        if assigned( c ) then
                                                if ( c^.FavoriteType = aType ) then {// Check if codec is _realy_ type favorite }
                                                        if( c^.DefaultExtention <> '' ) then
                                                                FileExtention := c^.DefaultExtention;
                                end;
                                Name := TextArgs('data.%s',[FileExtention]);
                                FileName := FileNameToPath(DestItemInfo^.VPathName,Name);
                                while ( f = FileGetInfo(FileName) ) do begin
                                        FreeFileInfo(f);
                                        inc(nb);
                                        FileName := '';
                                        Name := '';
                                        Name := TextArgs('data(%d).%s',[nb,FileExtention]);
                                        FileName := FileNameToPath(DestItemInfo^.VPathName,Name);
                                end;
                                SaveData2(FileName,Data,aType);
                                {//f := FileGetInfo(FileName);}
                                FileName := '';
                                Name     := '';
                                {//FreeFileInfo(f);}
                        end;
                end;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Dirview                                                                      *)
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirviewItemFree(o: Pointer);
begin
        FreeFileInfo(PDirviewItem(o)^.f);
        FreeListviewItem(PLISTVIEWITEM(o));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
FUNCTION DirviewNewItem ( o: PDirview ): PDirviewItem;
var p: PDirviewItem;
begin
        New(p);
        if not assigned(p) then DirviewNewItem := NIL else
        begin
        fillchar(p^, sizeof(TDirviewItem), 0);
        ListAdd (PLISTVIEW(o)^.Items,'' , p, {$ifdef fpc}@{$endif}DirviewItemFree);
        DirviewNewItem := p;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function format_size (size: l_ulong): l_text;
begin
        if size < 1024 then
                format_size := TextArgs ('%d bytes', [l_ulong(size)])
        else if (size < l_ulong(1024*1024)) then
                format_size := TextArgs ('%d KB', [size div 1024])
        else if (size < l_ulong(1024*1024*1024)) then
                format_size := TextArgs ('%d MB', [size div (1024*1024)])
        else
                format_size := TextArgs ('%d GB', [size div (1024*1024*1024)]);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
{
 datum := Trunc( Stempel);
 zeit := Frac( Stempel);
}
function _DirviewAddItem ( o:PDirview; f:PFileInfo ): PDirviewItem;
var
  l: l_text;
  i: PIcon;
  p: PDirviewItem;
  tmer: PDateTimeRec;
  tme: TDateTime;
begin
        New(p);
        if not assigned(p) then _DirviewAddItem := NIL else
        begin
        fillchar(p^, sizeof(TDirviewItem), 0);
        i := FileGetIconsEx(f);
        ListviewInitItem(PLISTVIEWITEM(p),f^.Name,i);
        if assigned( i ) then FreeIcon(i);
        p^.f := f;


        l := format_size(f^.Size);
        ListviewItemPushCol(PLISTVIEWITEM(p),l);
        l := '';
        //tme := now; {localtime((const time_t *)@f^.CTime);}
        if (f^.CTime>0)
          then l := TextArgs('%02d/%02d/%04d %02d:%02d:%02d',
                      [tmer^.tm_mday,tmer^.tm_mon+1,tmer^.tm_year+1900,tmer^.tm_hour,tmer^.tm_min,tmer^.tm_sec])
        else l := TextArgs('-',[]);
        ListviewItemPushCol(PLISTVIEWITEM(p),l);
        l := '';
        //tme := now; {localtime((const time_t *)@f^.MTime);}
        if (f^.MTime>0)
          then l := TextArgs('%02d/%02d/%04d %02d:%02d:%02d',
                      [tmer^.tm_mday,tmer^.tm_mon+1,tmer^.tm_year+1900,tmer^.tm_hour,tmer^.tm_min,tmer^.tm_sec])
        else l := TextArgs('-',[]);
        ListviewItemPushCol(PLISTVIEWITEM(p),l);
        l := '';
        //tme := now; {localtime((const time_t *)@f^.ATime);}
        if (f^.ATime>0)
          then l := TextArgs('%02d/%02d/%04d %02d:%02d:%02d',
                                 [tmer^.tm_mday,tmer^.tm_mon+1,tmer^.tm_year+1900,tmer^.tm_hour,tmer^.tm_min,tmer^.tm_sec])
        else l := TextArgs('-',[]);
        ListviewItemPushCol(PLISTVIEWITEM(p),l);
        l := '';
        (*l := TextArgs('%d', [f^.Size]);
          ListviewItemPushCol(LISTVIEWITEM(p),l);  {// for attributes, maybe in future}
          free(l);*)

        ListAdd (PLISTVIEW(o)^.Items, f^.Name, p, {$ifdef fpc}@{$endif}DirviewItemFree);
        {// ListviewItemSetColCaption(p,1,l); }

        _DirviewAddItem := p;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DirviewItemOrderyByName ( a:Pointer; b:Pointer ): integer;
begin
{ // improved by Florian Xaver }

        if ((PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
           ((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC) then
                DirviewItemOrderyByName := TextCaseCompare(PDirviewItem(a)^.f^.Name,PDirviewItem(b)^.f^.Name)
        else if (not((PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
                    ((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC)) then
                DirviewItemOrderyByName := 1
        else if not((PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
                not((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC) then
                DirviewItemOrderyByName := TextCaseCompare(PDirviewItem(a)^.f^.Name,PDirviewItem(b)^.f^.Name)
        else
                DirviewItemOrderyByName := -1;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DirviewItemOrderyBySize ( a:Pointer; b:Pointer ): integer;
begin
{// new by Florian Xaver }

        if ((PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
           ((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC) then
                DirviewItemOrderyBySize := TextCaseCompare(PDirviewItem(a)^.f^.Name,PDirviewItem(b)^.f^.Name)
        else if (not((PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
                    ((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC)) then
                DirviewItemOrderyBySize := 1
        else if not((PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
                not((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC) then
                if (PDirviewItem(a)^.f^.Size >= PDirviewItem(b)^.f^.Size)
                  then DirviewItemOrderyBySize := 1 else DirviewItemOrderyBySize := -1
        else
                DirviewItemOrderyBySize := -1;

end;

(*////////////////////////////////////////////////////////////////////////////////*)
function DirviewItemOrderyByCTime ( a:Pointer; b:Pointer ): integer;
begin
 {// new by Florian Xaver}

        if ( (PDirviewItem(a)^.f^.Attributes and FA_DIREC) and
             (PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC ) then
           begin if (PDirviewItem(a)^.f^.CTime >= PDirviewItem(b)^.f^.CTime)
                   then DirviewItemOrderyByCTime := 1
                     else DirviewItemOrderyByCTime := -1; end
        else if ((not(PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
                    ((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC)) then
                DirviewItemOrderyByCTime := 1
        else if (not(PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
                 not((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC) then
           begin if (PDirviewItem(a)^.f^.CTime >= PDirviewItem(b)^.f^.CTime)
                    then DirviewItemOrderyByCTime := 1
                      else DirviewItemOrderyByCTime := -1; end
        else
                DirviewItemOrderyByCTime := -1;

end;

(*////////////////////////////////////////////////////////////////////////////////*)
function DirviewItemOrderyByATime ( a:Pointer; b:Pointer ): integer;
begin
{// new by Florian Xaver}

        if ((PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
           ((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC) then
           begin if (PDirviewItem(a)^.f^.ATime >= PDirviewItem(b)^.f^.ATime)
                    then DirviewItemOrderyByATime := 1 else DirviewItemOrderyByATime := -1; end
        else if ((not(PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
                    ((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC) ) then
                DirviewItemOrderyByATime := 1
        else if (not(PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
                 not((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC ) then
           begin if (PDirviewItem(a)^.f^.ATime >= PDirviewItem(b)^.f^.ATime)
                    then DirviewItemOrderyByATime := 1
                      else DirviewItemOrderyByATime := -1; end
        else
                DirviewItemOrderyByATime := -1;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function DirviewItemOrderyByMTime ( a:Pointer; b:Pointer ): integer;
begin
        { // new by Florian Xaver }

        if ((PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
           ((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC) then
           begin if (PDirviewItem(a)^.f^.MTime >= PDirviewItem(b)^.f^.MTime)
                    then DirviewItemOrderyByMTime := 1
                      else DirviewItemOrderyByMTime := -1; end
        else if ((not(PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
                    ((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC) ) then
                DirviewItemOrderyByMTime := 1
        else if (not(PDirviewItem(a)^.f^.Attributes and FA_DIREC)=FA_DIREC) and
                 not((PDirviewItem(b)^.f^.Attributes and FA_DIREC)=FA_DIREC) then
           begin if (PDirviewItem(a)^.f^.MTime >= PDirviewItem(b)^.f^.MTime)
                    then DirviewItemOrderyByMTime := 1
                      else DirviewItemOrderyByMTime := -1; end
        else
                DirviewItemOrderyByMTime := -1;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DirviewAddItem ( o: PDirview; f: PFileInfo ): PDirviewItem;
var p: PDirviewItem;
begin
   p := _DirviewAddItem(o,f);

   if (o^.Flags and DVF_ORDERBYNAME) = DVF_ORDERBYNAME then
          ListOrderByData(PLISTVIEW(o)^.Items,{$ifdef fpc}@{$endif}DirviewItemOrderyByName)
   else if (o^.Flags and DVF_ORDERBYSIZE) = DVF_ORDERBYSIZE then
          ListOrderByData(PLISTVIEW(o)^.Items,{$ifdef fpc}@{$endif}DirviewItemOrderyBySize)
   else if (o^.Flags and DVF_ORDERBYCTIME) = DVF_ORDERBYCTIME then
          ListOrderByData(PLISTVIEW(o)^.Items,{$ifdef fpc}@{$endif}DirviewItemOrderyByCTime)
   else if (o^.Flags and DVF_ORDERBYMTIME) = DVF_ORDERBYMTIME then
          ListOrderByData(PLISTVIEW(o)^.Items,{$ifdef fpc}@{$endif}DirviewItemOrderyByMTime)
   else if (o^.Flags and DVF_ORDERBYATIME) = DVF_ORDERBYATIME then
          ListOrderByData(PLISTVIEW(o)^.Items,{$ifdef fpc}@{$endif}DirviewItemOrderyByATime)
   else
          ListOrderByData(PLISTVIEW(o)^.Items,{$ifdef fpc}@{$endif}DirviewItemOrderyByName);

        DirviewAddItem := p;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirviewGenerateSubs ( o: PDirview );
var
  i16: p_bitmap;
  i32: p_bitmap;
  fi : l_ulong;
  l: PList;
  a,b: PListItem;
  pi: PDirviewItem;
  pf: PFileinfo;
  ext: l_text;
  pex: l_ulong;

begin
        if assigned( o^.Dir ) then FreeFileInfo(o^.Dir);
        o^.Dir := FileGetInfo(PDIRVIEW(o)^.Directory);

        ListviewRemoveAllItems(PLISTVIEW(o));

        if (not(o^.Flags and DVF_NOPARICON)=DVF_NOPARICON) then
                if ( o^.Directory <> '' ) then begin
                        pf := FileGetInfo(GetPathParent(o^.Directory));
                        pi := DirviewNewItem(o);
                        ListviewInitItem(PLISTVIEWITEM(pi),'..',ParentDirIcon);
                        pi^.f := pf;
                end;

        l := ListDir(o^.Directory,FA_DIREC or FA_LABEL);
        if assigned( l ) then begin

           if assigned( l^.Last ) then
           begin
                a := l^.Last^.Next;
                b := l^.Last^.Next;

                repeat
                        if ( PFILEINFO(a^.Data)^.Attributes and FA_DIREC ) = FA_DIREC then
                                DirviewAddItem(o,PFILEINFO(a^.Data));
                        a := a^.Next;

                until ( a = b );

                repeat
                        if (not(PFILEINFO(a^.Data)^.Attributes and FA_DIREC ))=FA_DIREC then
                        if assigned( o^.Extentions ) then begin
                                pex := system.pos('.',PFILEINFO(a^.Data)^.Name);
                                ext := system.Copy(PFILEINFO(a^.Data)^.Name,pex+1,Length(PFILEINFO(a^.Data)^.Name)-pex);
                                {ext := TextRChr(PFILEINFO(a^.Data)^.Name,'.');}
                                if ( ext <> '' ) then
                                        if assigned(ListKeyCaseItem(o^.Extentions,ext))
                                           then DirviewAddItem(o,PFILEINFO(a^.Data));
                        end else
                                DirviewAddItem(o,PFILEINFO(a^.Data));
                        a := a^.Next;
                until ( a = b );
           end;
           KickList(l);
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirviewOpenDirectory ( o: PDirview; dir: l_text );
begin
        if ( o^.Directory <> '' ) then o^.Directory := '';
        o^.Directory := dir;
	{
        if (o^.Flags and DVF_DESKTOPICON) = DVF_DESKTOPICON then
	begin
	  FileRun('applications/nav.app', TextDup(dir));
	end
	else 
	begin
        }
	  DirviewGenerateSubs(o);
	  ListviewUpdateAndReDraw(PLISTVIEW(o));
        {
        end;
        }
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirviewOpenParentDirectory ( o: PDirview );
var dir: l_text;
begin
        dir := GetPathParent(o^.Directory);
        DirviewOpenDirectory(o,dir);
        dir := '';
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirviewRefreshData ( o: PDirview );
begin
        DirviewGenerateSubs(o);
        ListviewUpdateAndReDraw(PLISTVIEW(o));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
{l_bool CollectCallBack ( void* id, l_text curfile );}
type
  TCollectCallBackFunc = function(id: Pointer; curfile: l_text): l_bool;

function CollectCallBack(id: Pointer; curfile: l_text): l_bool; forward;

(*////////////////////////////////////////////////////////////////////////////////*)
function DirviewCollectSelctedFiles ( o: PDirview ): PList;
var
  Lst: PList;
  p  : PProgressPop;
  a  : PListItem;
begin
        a := PLISTVIEW(o)^.Items^.Last;

        if assigned( a ) then DirviewCollectSelctedFiles := NIL;

        Lst := NewList;

        if assigned( Lst ) then DirviewCollectSelctedFiles := NIL;

        p := ProgressPopUp(@Me,1,'I/O','Please wait. Collecting files...',0,'');

        repeat

                if (PLISTVIEWITEM(a^.Data)^.Flags and LVI_SELECTED) = LVI_SELECTED then begin
                        if ( not _GenerateFileRefs( Lst, DIRVIEWITEM(a^.Data)^.f, '', {$ifdef fpc}@{$endif}CollectCallBack, p) ) then
                        begin
                                FreeList(Lst);
                                ProgressPopKick(p);
                                DirviewCollectSelctedFiles := NIL;
                        end;
                end;
                a := a^.Prev;

        until ( a = PLISTVIEW(o)^.Items^.Last );
        ProgressPopKick(p);
        DirviewCollectSelctedFiles := Lst;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DirviewGetSelectedFiles ( o: PDirview ): PList;
var
  Lst: PList;
  a: PListItem;
begin
        a := PLISTVIEW(o)^.Items^.Last;
        if not assigned( a ) then DirviewGetSelectedFiles := NIL;
        Lst := NewList;
        if not assigned( Lst ) then DirviewGetSelectedFiles := NIL;

        repeat
                if (PLISTVIEWITEM(a^.Data)^.Flags and LVI_SELECTED) = LVI_SELECTED then
                        ListAdd(Lst,'',NewFileReference(DIRVIEWITEM(a^.Data)^.f,DIRVIEWITEM(a^.Data)^.f^.Name),{$ifdef fpc}@{$endif}FreeFileRef);
                a := a^.Prev;

        until ( a = PLISTVIEW(o)^.Items^.Last );
        DirviewGetSelectedFiles := Lst;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirviewItemStartDrag ( o: PWidget );
var l: PList;
begin
        DebugMessage ('Drag n Drop: DirviewItemStartDrag',[]);
        l := DirviewGetSelectedFiles(PDIRVIEW(o));
        if assigned( l ) then begin
                DragStart(o,DTYPE_FILEREFLIST,l,DMETHOD_COPY or DMETHOD_MOVE);
                FreeList(l);
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DirviewItemDropAccept (o,Caller:PWidget; Item:PListviewItem; aType:l_ulong; Data:Pointer; SupportedMethods:l_ulong; Method:l_ulong): l_bool;
var
  DestItemInfo: PFileInfo;
begin
        DebugMessage ('Drag n Drop: DirviewItemDropAccept',[]);
        DestItemInfo := PDIRVIEW(o)^.Dir;
        if assigned( Item ) then DestItemInfo := PDIRVIEWITEM(Item)^.f;

        DirviewItemDropAccept := IODlgDropAccept(DestItemInfo,aType,Data,SupportedMethods,Method);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DirviewItemDragBeforeDrop (o,Dest:PWidget; Item:PListviewItem; Data: Pointer; aType,Method:l_ulong): l_bool;
begin
        DebugMessage ('Drag n Drop: DirviewItemDragBeforeDrop',[]);
        DirviewItemDragBeforeDrop := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirviewItemDragAfterDrop (o,Dest:PWidget; Item:PListviewItem; Data: Pointer; aType,Method:l_ulong);
begin
        DebugMessage ('Drag n Drop: DirviewItemDragAfterDrop',[]);
        DirviewRefreshData(PDIRVIEW(o));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirviewItemDropData (o,Caller:PWidget; Item:PListviewItem; Data: Pointer; aType,Method:l_ulong);
var
  DestItemInfo: PFileInfo;
begin
        DebugMessage ('Drag n Drop: DirviewItemDropData',[]);
        DestItemInfo := PDIRVIEW(o)^.Dir;
        if assigned( Item ) then DestItemInfo := PDIRVIEWITEM(Item)^.f;

        IODlgDropDataToFileInfo( DestItemInfo, Data, aType, Method );

        DirviewRefreshData(PDIRVIEW(o));
        WidgetDraw(o,NIL);
        ScrollBarRedraw(SLLWDG(o));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DirviewEventHandler ( o:PWidget; Ev:PEvent ): l_bool;
var
  NewName,Outp,NewVPathName: l_text;
  fi,f: PFileInfo;
  nb,msg: l_ulong;
  l,el : PList;
  p : PListViewItem;
  r: l_bool;
  m: PMenu;
begin

        if (Ev^.aType = EV_KEYBOARD) then begin

                if ( SCANCODE(@Ev^.Keyb,AL_KEY_DEL) ) then begin
                        p := GetSelectedItem(PLISTVIEW(o));
                        if assigned( p ) then begin
                                l := DirviewCollectSelctedFiles(PDIRVIEW(o));
                                if assigned( l ) then begin
                                        DeleteFilesFromList(l);
                                        FreeList(l);
                                        DirviewRefreshData(PDIRVIEW(o));
                                end;
                        end;
                        DirViewEventHandler := true;
                end else if ( Ev^.Keyb.Shift and KB_CTRL ) = KB_CTRL then begin

                        if ( SCANCODE(@Ev^.Keyb,AL_KEY_C) ) then begin
                                p := GetSelectedItem(PLISTVIEW(o));
                                if assigned( p ) then begin
                                        { l := DirviewCollectSelctedFiles(PDIRVIEW(o)); }
                                        l := DirviewGetSelectedFiles(PDIRVIEW(o));
                                        if assigned( l ) then
                                                ClipboardPut(TYPE_FILEREFLIST,l,false)
                                        else
                                                MessageBox(@Me, 'I/O', 'Unable to copy files to clipboard.', 0);
                                end;

                        end else if ( SCANCODE(@Ev^.Keyb,AL_KEY_V) ) then begin

                                WidgetSendSEvent(o, EV_MESSAGE, MSG_IOMENU_PASTE, Ev^.Extra);

                        end;
                        DirViewEventHandler := true;
                end else if ( SCANCODE(@Ev^.Keyb,AL_KEY_F5) ) then begin
                        DirviewRefreshData(PDIRVIEW(o));
                        DirViewEventHandler := true;
                end;
        end;
        if ( Ev^.aType = EV_MESSAGE ) then begin
                if ( Ev^.Message = MSG_IODLG_ITEMFILE_VAL ) then begin
                        if ( PDIRVIEWITEM(Ev^.Extra)^.f^.Attributes and FA_DIREC ) = FA_DIREC then begin
                                DirviewOpenDirectory(PDIRVIEW(o),PDIRVIEWITEM(Ev^.Extra)^.f^.VPathName);
                                DirViewEventHandler := true;
                        end;
                end;
                if ( Ev^.Message = MSG_IODLG_OPENPARENT ) then begin
                        DirviewOpenParentDirectory(PDIRVIEW(o));
                        DirViewEventHandler := true;
                end;
                if ( Ev^.Message = MSG_IODLG_ITEMFILE_MENU ) then begin
                        if ( ListviewCountSelectedFiles(PLISTVIEW(o)) > 1 ) then begin

                                MessageBox(@Me, 'I/O', 'Unsupported. Even use CTRL+C to copy selected files.', 0);

                        end else begin
                                m := FileGetMenu2(PDIRVIEWITEM(Ev^.Extra)^.f,0);
                                msg := PopUpMenuWait(o^.AppOwner,Mouse^.State.p,m,MVF_FREEMENU);
                                if ( msg = MSG_IOMENU_COPY ) then begin
                                        {//PList l = GenerateFileRefs(PDIRVIEWITEM(Ev^.Extra)^.f);}
                                        l := DirviewGetSelectedFiles(PDIRVIEW(o));
                                        if assigned( l ) then
                                                ClipboardPut(TYPE_FILEREFLIST,l,false)
                                        else {// System was unable to collect files, so we put only file reference}
                                                MessageBox(@Me, 'I/O', 'Unable to copy files to clipboard.', 0);

                                end else if ( msg = MSG_IOMENU_DELETE ) then begin
                                        l := GenerateFileRefs(PDIRVIEWITEM(Ev^.Extra)^.f);
                                        if assigned( l ) then
                                        begin DeleteFilesFromList(l); FreeList(l); DirviewRefreshData(PDIRVIEW(o)); end;
                                end else if ( msg = MSG_IOMENU_PASTE ) then begin
                                        el := ClipboardGet(TYPE_FILEREFLIST);
                                        if assigned( el ) then begin
                                                IODlgDropDataToFileInfo( PDIRVIEWITEM(Ev^.Extra)^.f, el, TYPE_FILEREFLIST, DMETHOD_COPY );
                                        end;
                                end else if ( (*msg = MSG_IOMENU_OPEN*) msg = DirOpenMSG ) then begin
                                        WidgetSendSEvent(o, EV_MESSAGE, MSG_IODLG_ITEMFILE_VAL, Ev^.Extra);
                                end else if ( msg = MSG_IODLG_ITEMFILE_VAL ) then begin

                                        WidgetSendSEvent(o, EV_MESSAGE, MSG_IODLG_ITEMFILE_VAL, Ev^.Extra);

                                end else if ( msg = MSG_IOMENU_RENAME ) then begin

                                        NewName := ListviewRenameItemBox(PLISTVIEW(o),PLISTVIEWITEM(Ev^.Extra));

                                        if ( NewName <> '' ) then begin
                                                NewVPathName := FileNameToPath(PDIRVIEW(o)^.Directory,NewName);

                                                r := false;

                                                if (PDIRVIEWITEM(Ev^.Extra)^.f^.Attributes and FA_DIREC)=FA_DIREC then
                                                        r := DirRename(PDIRVIEWITEM(Ev^.Extra)^.f^.VPathName,NewVPathName)
                                                else
                                                        r := FileRename(PDIRVIEWITEM(Ev^.Extra)^.f^.VPathName,NewVPathName);

                                                if ( r ) then begin
                                                        FreeFileInfo(PDIRVIEWITEM(Ev^.Extra)^.f);
                                                        DIRVIEWITEM(Ev^.Extra)^.f := FileGetInfo(NewVPathName);
                                                        PLISTVIEWITEM(Ev^.Extra)^.Caption := '';
                                                        PLISTVIEWITEM(Ev^.Extra)^.Caption := TextDup(NewName);
                                                        WidgetDraw(o,NIL);
                                                        ScrollBarRedraw(SLLWDG(o));
                                                end;
                                                NewVPathName := '';
                                                NewName := '';
                                        end;
                                end else if ( msg <> 0 ) then
                                        FileHandleMsg(PDIRVIEWITEM(Ev^.Extra)^.f,msg);
                                DirViewEventHandler := true;
                        end;
                end;


                if ( Ev^.Message = MSG_IOMENU_PASTE ) then begin
                        el := ClipboardGet(TYPE_FILEREFLIST);
                        if assigned( el ) then begin
                                IODlgDropDataToFileInfo( PDIRVIEW(o)^.Dir, el, TYPE_FILEREFLIST, DMETHOD_COPY );
                                DirviewRefreshData(PDIRVIEW(o));
                        end;
                        DirViewEventHandler := true;
                end;

                if ( Ev^.Message = MSG_IODLG_NEWDIR ) then begin
                        nb := 0;
                        Outp := FileNameToPath(PDIRVIEW(o)^.Directory,'New directory');

                        while ( fi = FileGetInfo(Outp) ) do begin
                                Outp := '';
                                FreeFileInfo(fi);
                                inc(nb);
                                NewName := TextArgs('New directory (%d)',[nb]);
                                Outp := FileNameToPath(PDIRVIEW(o)^.Directory,NewName);
                                NewName := '';
                        end;
                        MakeDir(Outp);
                        fi := FileGetInfo(Outp);
                        Outp := '';
                        if assigned( fi ) then begin
                                DirviewAddItem(PDIRVIEW(o),fi);
                                WidgetDraw(o,NIL);
                                ScrollBarRedraw(SLLWDG(o));
                        end;
                        DirViewEventHandler := true;
                end;

        end;

        if ( ListviewEventHandler(o,Ev) ) then DirViewEventHandler := true;

        if ( Ev^.Message = WEvMouseRUp ) then begin
                f := FileGetInfo(PDIRVIEW(o)^.Directory);
                if assigned( f ) then begin
                        m := FileGetMenu2(f,1);
                        msg := PopUpMenuWait(o^.AppOwner,Mouse^.State.p,m,MVF_FREEMENU);

                        if ( (msg = MSG_IOMENU_PASTE) or (msg = MSG_IODLG_NEWDIR) ) then begin
                                WidgetSendSEvent(o, EV_MESSAGE, msg, Ev^.Extra);

                        end else
                                FileHandleMsg(f,msg);
                end;
                DirViewEventHandler := true;
        end else

        DirViewEventHandler := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirviewSetExtentions ( o: PDirview; Extentions: PList );
begin
        o^.Extentions := Extentions;
        DirviewGenerateSubs(o);
        ListviewUpdateAndReDraw(PLISTVIEW(o));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirviewFree ( o: PWidget );
begin
        if assigned( PDIRVIEW(o)^.Dir ) then FreeFileInfo(PDIRVIEW(o)^.Dir);
        if ( PDIRVIEW(o)^.Directory <> '' ) then PDIRVIEW(o)^.Directory := '';
        ListviewFree(o);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CreateDirview ( App:PApplication; r:TRect; dir: l_text; Extentions:PList; Flags:l_ulong ): PDirview;
var o: PDirView;
begin
           { //FreeFileInfo(dest); }

        New(o);

        if not assigned(o) then CreateDirview := NIL;
        fillchar(o^, sizeof(TDirview), 0);
        InitListview(
           PLISTVIEW(o),
           DirviewClass,
           App, r,
           'File name',
           LVS_ICONS,LVF_DRAWICONS or LVF_COLUMS or LVF_ALLOWDRAG or LVF_COLRESIZE
        );

        ListviewAddColum ( PLISTVIEW(o), 'Size',NIL, 40, 0 );
        ListviewAddColum ( PLISTVIEW(o), 'Created',NIL, 125, 0 );
        ListviewAddColum ( PLISTVIEW(o), 'Modified',NIL, 125, 0 );
        ListviewAddColum ( PLISTVIEW(o), 'Accessed',NIL, 125, 0 );
        //ListviewAddColum ( o, 'Attributes',NIL, 75, 0 );

        o^.Directory := TextDup(dir);
        o^.Extentions := Extentions;

        PWIDGET(o)^.EventHandler := {$ifdef fpc}@{$endif}DirviewEventHandler;
        PWIDGET(o)^.FreeEx := {$ifdef fpc}@{$endif}DirviewFree;

        PLISTVIEW(o)^.OnSelMsg := MSG_IODLG_ITEMFILE_SEL;
        PLISTVIEW(o)^.OnValMsg := MSG_IODLG_ITEMFILE_VAL;
        PLISTVIEW(o)^.OnRClkMsg := MSG_IODLG_ITEMFILE_MENU;

        if ( Flags and DVF_MULTISELECT ) = DVF_MULTISELECT then
        PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags or LVF_MULTISELECT;

        

        o^.Flags := Flags;
        PLISTVIEW(o)^.ItemStartDrag := {$ifdef fpc}@{$endif}DirviewItemStartDrag;
        PLISTVIEW(o)^.ItemDropAccept      := {$ifdef fpc}@{$endif}DirviewItemDropAccept;
        PLISTVIEW(o)^.ItemDragBeforeDrop  := {$ifdef fpc}@{$endif}DirviewItemDragBeforeDrop;
        PLISTVIEW(o)^.ItemDropData        := {$ifdef fpc}@{$endif}DirviewItemDropData;
        PLISTVIEW(o)^.ItemDragAfterDrop  := {$ifdef fpc}@{$endif}DirviewItemDragAfterDrop;


        DirviewGenerateSubs(o);
        ListviewCalculateMax(PWIDGET(o));

        CreateDirview := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// TreeDirview                                                                  *)
(*////////////////////////////////////////////////////////////////////////////////*)
function DirStrDelta(DirStr: l_text): l_ulong;
begin
  DirStrDelta := Length(DirStr);
end;

function DirStrFileSepPos(DirStr: l_text): l_ulong;
begin
  DirStrFileSepPos := system.pos(FileSeparator,DirStr);
end;

function DirStrNetWDrivePos(DirStr: l_text): l_ulong;
begin
  DirStrNetWDrivePos := system.pos(NetWDriveSep,DirStr);
end;

function TreeDirViewOpenDirEx ( t:PTreeView; i:PTreeItem; dir:l_text ): PTreeItem;
var S,temp: l_text; a,b: PTreeItem; slpos: l_ulong;
begin
        i^.Flags := i^.Flags or TIF_EXPENDED;

        if not assigned( i^.Last ) then
                if assigned( i^.GenerateSubs ) then
                        i^.GenerateSubs(i);

        if assigned( i^.Last ) then begin

                a := i^.Last;
                b := a;

                slpos := 1;
                while (slpos <= Length(dir))and((dir[slpos] = '\')or(dir[slpos] = '/')) do inc(slpos);
                dec(slpos); move(dir[slpos],temp[1],Length(dir)-slpos); SetLength(temp,Length(dir)-slpos);
                dir := temp;
                (*
                slpos := system.pos(NetWDriveSep,dir);
                if slpos > 0 then
                begin

                  Inc(slpos,2);
                end;
                begin
                  slpos := system.pos(FileSeparator,dir);
                  if slpos > 0 then inc(slpos);
                  {if ( * dir='\\' || *dir = '/' ) dir++;}
                  system.Delete(dir,slpos,1);
                end;
                *)
                if (slpos = Length(s)) then TreeDirViewOpenDirEx := i;
                S := temp{dir};
        end;

        while ( slpos <= Length(dir) ) do begin
                if ((S[slpos]='\') or (S[slpos]='/')) then begin
                   repeat
                        if ( TextSqNCaseCompare(dir,a^.Caption,Length(S)-Length(dir)) = 0 ) then
                                TreeDirViewOpenDirEx := TreeDirViewOpenDirEx( t,a,S[slpos+1]);
                        a := a^.Next;
                   until ( a = b );
                end;
                inc(slpos);

           repeat
                if ( TextCaseCompare(dir,a^.Caption)=0 ) then TreeDirViewOpenDirEx := TreeDirViewOpenDirEx( t,a,'');
                a := a^.Next;
           until ( a = b );
        end;

        TreeDirViewOpenDirEx := i;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeDirViewOpenDir ( o: PTreeDirview; dir: l_text );
var i: PTreeItem;
begin
        i := TreeDirViewOpenDirEx(PTREEVIEW(o),PTREEVIEW(o)^.Items^.Last,dir);
        TreeviewScrollBarRecalculate(PTREEVIEW(o));
        if assigned( i ) then TreeViewSelectItem(PTREEVIEW(o),i);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function DirTreeViewEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
var
  f: PFileInfo;
  m: PMenu;
  msg: l_ulong;
begin
        if ( Ev^.aType = EV_MESSAGE ) then begin
                if ( Ev^.Message = MSG_TREEVIEW_ITEM ) then begin

                        if not assigned( PTREEVIEW(o)^.Sel ) then DirTreeViewEventHandler := true;

                        if assigned( PTREEDIRVIEW(o)^.Linked ) then begin
                                DirviewOpenDirectory(PTREEDIRVIEW(o)^.Linked,PTREEDIRVIEWITEM(PTREEVIEW(o)^.Sel)^.f^.VPathName);
                        end;

                        if ( PTREEDIRVIEW(o)^.Msg <> 0 ) then begin
                                if ( PTREEDIRVIEW(o)^.Msg <> MSG_TREEVIEW_ITEM ) {// Prevent infinite loops}
                                   then WidgetSendSEvent(o, EV_MESSAGE, PTREEDIRVIEW(o)^.Msg, PTREEVIEW(o)^.Sel);
                        end;

                        DirTreeViewEventHandler := true;
                end;
                if ( (Ev^.Message = MSG_TREEVIEW_ITEM_MENU) and assigned(PTREEVIEW(o)^.Sel) ) then begin
                        f := PTREEDIRVIEWITEM(PTREEVIEW(o)^.Sel)^.f;
                        if assigned( f ) then begin
                                m := FileGetMenu2(f,1);
                                if assigned( m ) then begin

                                        m^.Items := NewMenuItem('Explore',NIL,MSG_TREEVIEW_ITEM,0,NIL,
                                                NewMenuItem('Expend',NIL,MSG_TREEVIEW_EXPEND,0,NIL,
                                                NewMenuItemSeparator(
                                                m^.Items)));

                                        msg := PopUpMenuWait(o^.AppOwner,Mouse^.State.p,m,MVF_FREEMENU);

                                        if ( msg = MSG_TREEVIEW_ITEM ) then begin
                                                if assigned( PTREEDIRVIEW(o)^.Linked ) then begin
                                                        DirviewOpenDirectory(PTREEDIRVIEW(o)^.Linked,PTREEDIRVIEWITEM(PTREEVIEW(o)^.Sel)^.f^.VPathName);
                                                end;

                                                if (PTREEDIRVIEW(o)^.Msg <> 0) then begin
                                                        if ( PTREEDIRVIEW(o)^.Msg <> MSG_TREEVIEW_ITEM ) {// Prevent infinite loops}
                                                           then WidgetSendSEvent(o, EV_MESSAGE, PTREEDIRVIEW(o)^.Msg, PTREEVIEW(o)^.Sel);
                                                end;

                                        end else if ( msg = MSG_TREEVIEW_EXPEND ) then begin
                                                PTREEVIEW(o)^.Sel^.Flags := PTREEVIEW(o)^.Sel^.Flags or TIF_EXPENDED;
                                                if not assigned(PTREEVIEW(o)^.Sel^.Last) then
                                                        if assigned( PTREEVIEW(o)^.Sel^.GenerateSubs ) then
                                                                PTREEVIEW(o)^.Sel^.GenerateSubs(PTREEVIEW(o)^.Sel);
                                                TreeviewScrollBarRecalculate(PTREEVIEW(o));
                                                WidgetDraw(PWIDGET(o),NIL);
                                                ScrollBarRedraw(SLLWDG(o));
                                        end else

                                                FileHandleMsg(f,msg);
                                end;
                        end;
                        DirTreeViewEventHandler := true;
                end;
        end;

        DirTreeViewEventHandler := TreeViewEventHandler(o,Ev);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeDirviewItemFreeEx ( i: PTreeviewItem );
begin
  if assigned( PTREEDIRVIEWITEM(i)^.f ) then FreeFileInfo(PTREEDIRVIEWITEM(i)^.f);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirTreeViewGenerateSubs ( o: PTreeItem  ); forward;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeDirviewAddItem ( Parent: PTreeItem; f: PFileInfo ): PTreeDirviewItem;
var Icon: PIcon;
    Name: l_text;
    i: PTreeDirviewItem;
begin
        Name := f^.Name;
        New(i);
        if not assigned( i ) then TreeDirviewAddItem := nil else
        fillchar(i,sizeof(TTreeDirviewItem),0);
        Icon := FileGetIconsEx(f);
        if ( TextCompare(Name,'') = 0 ) then Name := 'Computer';
        TreeviewInitItem(
           PTREEVIEWITEM(i),
           Name,'',Icon,
           {$ifdef fpc}@{$endif}DirTreeViewGenerateSubs,
           {$ifdef fpc}@{$endif}TreeDirviewItemFreeEx
        );
        FreeIcon(Icon);

        i^.f := f;

        TreeviewAddItem(Parent,PTREEVIEWITEM(i));


        TreeDirViewAddItem := i;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeDirviewItemOrderyByName (a: Pointer; b: Pointer ): Integer;
begin
// improved by Florian Xaver

        if ((PFileInfo(a)^.Attributes and FA_DIREC)=FA_DIREC) and
           ((PFileInfo(b)^.Attributes and FA_DIREC) = FA_DIREC) then
                TreeDirviewItemOrderyByName := TextCaseCompare(PFileInfo(a)^.Name,PFileInfo(b)^.Name)
        else
                TreeDirviewItemOrderyByName := -1;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DirTreeViewGenerateSubs ( o: PTreeItem );
var a,b: PListItem; l: PList;
begin
        l := ListDir(PTREEDIRVIEWITEM(o)^.f^.VPathName,FA_DIREC or FA_LABEL);
        EmptySubTreeItems(o);
        if ( l = nil ) then exit;

        if assigned( l^.Last ) then begin
           ListOrderByData(l, {$ifdef fpc}@{$endif}TreeDirviewItemOrderyByName);
           a := l^.Last^.Next; b := a;
           repeat
                if ( PFILEINFO(a^.Data)^.Attributes and FA_DIREC ) = FA_DIREC then begin
                        TreeDirviewAddItem(o,PFILEINFO(a^.Data));

                end;
                a := a^.Next;
           until ( a = b );
        end;
        KickList(l);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeDirviewItemStartDrag ( o: PWidget );
var l: PList;
begin
        DebugMessage ('Drag n Drop: TreeDirviewItemStartDrag',[]);
(*      l := DirviewGetSelectedFiles(PDIRVIEW(o));
        if assigned( l ) then begin
                DragStart(o,DTYPE_FILEREFLIST,l,DMETHOD_COPY or DMETHOD_MOVE);
                FreeList(l);
        end;*)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeDirviewItemDropAccept(
  o,Caller:PWidget;
  Item:PTreeviewItem;
  aType:l_ulong;
  Data: Pointer;
  SupportedMethods:l_ulong;
  var Method:l_ulong
): l_bool;
begin
        DebugMessage ('Drag n Drop: TreeDirviewItemDropAccept',[]);
        if assigned( Item ) then
           TreeDirviewItemDropAccept := IODlgDropAccept(PTREEDIRVIEWITEM(Item)^.f,aType,Data,SupportedMethods,Method)
        else TreeDirviewItemDropAccept := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeDirviewItemDragBeforeDrop(o,Dest:PWidget; Item:PTreeviewItem; Data:Pointer; aType,Method:l_ulong): l_bool;
begin
        DebugMessage ('Drag n Drop: TreeDirviewItemDragBeforeDrop',[]);
        TreeDirviewItemDragBeforeDrop := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeDirviewItemDragAfterDrop(o,Dest:PWidget; Item:PTreeviewItem; Data:Pointer; aType,Method:l_ulong);
begin
        DebugMessage ('Drag n Drop: TreeDirviewItemDragAfterDrop',[]);
        if ( Method = DMETHOD_MOVE ) then
                TreeViewDeleteItem(PTREEVIEW(o),Item);
        WidgetDraw(o,NIL);
        ScrollBarRedraw(SLLWDG(o));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeDirviewItemDropData (o,Caller:PWidget; Item:PTreeviewItem; Data:Pointer; aType,Method:l_ulong);
var DestItemInfo: PFileInfo;
begin
        DestItemInfo := PTREEDIRVIEWITEM(Item)^.f;
        DebugMessage ('Drag n Drop: TreeDirviewItemDropData: Destination: %s', [DestItemInfo^.Name]);

        IODlgDropDataToFileInfo( DestItemInfo, Data, aType, Method );

        TreeviewUnExpendItem(PTREEVIEW(o),Item);
        WidgetDraw(o,NIL);
        ScrollBarRedraw(SLLWDG(o));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CreateTreeDirview ( App: PApplication; r:TRect ): PTreeDirview;
var
  fi: PFileInfo;
  Icon: PIcon;
  o: PTreeDirview;

begin
        New(o);

        if not assigned( o ) then CreateTreeDirview := NIL else
        begin
          fillchar(o,sizeof(TTreeDirview),0);

          InitTreeView(PTREEVIEW(o),TreeDirviewClass,App,r,MSG_TREEVIEW_ITEM);

          PTREEVIEW(o)^.Flags := PTREEVIEW(o)^.Flags or TVF_ALLOWDRAG;

          PTREEVIEW(o)^.ItemStartDrag      := {$ifdef fpc}@{$endif}TreeDirviewItemStartDrag;
          PTREEVIEW(o)^.ItemDropAccept     := {$ifdef fpc}@{$endif}TreeDirviewItemDropAccept;
          PTREEVIEW(o)^.ItemDragBeforeDrop := {$ifdef fpc}@{$endif}TreeDirviewItemDragBeforeDrop;
          PTREEVIEW(o)^.ItemDropData       := {$ifdef fpc}@{$endif}TreeDirviewItemDropData;
          PTREEVIEW(o)^.ItemDragAfterDrop  := {$ifdef fpc}@{$endif}TreeDirviewItemDragAfterDrop;

          PTREEVIEW(o)^.OnMenuMsg := MSG_TREEVIEW_ITEM_MENU;

          PWIDGET(o)^.EventHandler := {$ifdef fpc}@{$endif}DirTreeViewEventHandler;

          TreeDirviewAddItem(PTREEVIEW(o)^.Items,FileGetInfo('/'));

          CreateTreeDirview := o;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// IO Box                                                                       *)
(*////////////////////////////////////////////////////////////////////////////////*)
const
  CB_FILTER   = $FF00FFB0;
  TB_ADD      = $FF00FFB1;
  TB_REMOVE   = $FF00FFB2;
  TB_GOUP     = $FF00FFB4;
  LB_CHANGE   = $FF00FFB8;

function IOBox ( Title:l_text; Flags:l_ulong; var Dir:l_text; Types: PFileTypes; FExists:l_bool ): l_text;
var
        r: TRect;
        DefExt: PList;
        a: PListviewItem;

        Icon: p_bitmap;

        w: PWindow;
        b: PButton;
        BAdd: PButton;
        BRemove: PButton;
        BUp: PButton;
        l: PListbox;
        dirv: PDirview;
        c: PCheckbox;
        t: PLabel;

        FileName: PTextbox;
        tsel: PCombobox;
        msg: l_ulong;
        OpenFile: l_text;

        fi: PFileInfo;
        ft: PFileTypes;
begin
        DefExt := NIL;
        msg := 0;
        OpenFile := '';
        Icon := LoadImage('./SYSTEM/ICONS/folder16.bmp');

        if ( Dir = '' ) then Dir := GlobalDir;

        if assigned( Types ) then DefExt := Types^.Extentions; {// DO NOT MOVE THIS !}

        RectAssign(r, 0, 0, 600, 440);
        w := CreateWindow(@Me, r, Title, WF_CAPTION or WF_FRAME or WF_CENTERED or WF_MINIMIZE);
        InsertWidget(DeskTop, PWIDGET(w));

        RectAssign(r, 10, 10, 70, 40);
        BAdd := CreateButton(@Me,r,'Add',TB_ADD);
        BAdd^.Icon := IconAdd;
        InsertWidget(PWIDGET(w), PWIDGET(BAdd));

        RectAssign(r, 75, 10, 160, 40);
        BRemove := CreateButton(@Me,r,'Remove',TB_REMOVE);
        BRemove^.Icon := IconRemove;
        BRemove^.Flags := BRemove^.Flags or BF_DISABLED;
        InsertWidget(PWIDGET(w), PWIDGET(BRemove));

        RectAssign(r, 165, 10, 215, 40);
        BUp := CreateButton(@Me,r,'Up',TB_GOUP);
        BUp^.Icon := IconUp;
        InsertWidget(PWIDGET(w), PWIDGET(BUp));

        RectAssign(r, 10, 50, 215, 295);
        l := CreateListbox(@Me, r);
        l^.AddItem(l, 'Computer', PChar('/'), Icon);
        l^.AddSeperator(l);
        l^.AddItem(l, 'Home', PChar('/system/User/'), Icon);
        l^.AddItem(l, 'Desktop', PChar('/system/User/Desktop/'), Icon);
        l^.AddItem(l, 'Pictures', PChar('/system/User/Pictures/'), Icon);
        l^.AddItem(l, 'Music', PChar('/system/User/Music/'), Icon);
        l^.AddItem(l, 'Documents', PChar('/system/User/Documents/'), Icon);
        l^.OnSelect := LB_CHANGE;
        InsertWidget(PWIDGET(w), PWIDGET(l));

        RectAssign(r, 225, 50, 590, 295);
        dirv := CreateDirview(@Me, r, Dir, DefExt, DVF_NOPARICON or DVF_ORDERBYNAME);
        PLISTVIEW(dir)^.Style := LVS_LIST;
        InsertWidget(PWIDGET(w), PWIDGET(dir));

        RectAssign(r, 10, 315, 80, 340);
        t := CreateLabel(@Me,r,'Filename:');
        PWIDGET(t)^.BackgroundColor := COL_3DFACE;
        InsertWidget(PWIDGET(w), PWIDGET(t));

        RectAssign(r, 90, 315, 430, 340);
        FileName := CreateTextbox(@Me,r,TBF_EDITABLE);
        InsertWidget(PWIDGET(w), PWIDGET(FileName));

        RectAssign(r, 440, 315, 590, 340);
        tsel := NewComboBox(@Me, r, CB_FILTER);
        InsertWidget(PWIDGET(w), PWIDGET(tsel));

        if assigned( Types ) then
        begin
                ft := Types;

                while assigned( ft ) do
                begin
                        ComboboxAddItem(tsel,ft^.Title,NIL);
                        ft := ft^.Next;
                end;

        end
        else
        begin
                ComboboxAddItem(tsel, 'All files (*.*)', NIL);
        end;

        ComboboxSelectIndex(tsel, 1);


        RectAssign(r, 405, 400, 490, 430);
        b := CreateButton(@Me,r,'Cancel',WM_CLOSE);
        b^.Icon := IconCancel;
        InsertWidget(PWIDGET(w), PWIDGET(b));

        RectAssign(r ,505, 400, 590, 430);
        if ( Flags and IOBOX_SAVE ) = IOBOX_SAVE then
        begin
                b := CreateButton(@Me, r, 'Save', MSG_IODLG_BOX_OK);
                b^.Icon := IconSave;
        end
        else
        begin
                b := CreateButton(@Me, r, 'Open', MSG_IODLG_BOX_OK);
                b^.Icon := IconOpen;
        end;

        InsertWidget(PWIDGET(w), PWIDGET(b));
        WidgetDrawAll(PWIDGET(w));

        msg := not WM_CLOSE;
        while ( msg <> WM_CLOSE ) do
        begin
                msg := WidgetExecute(PWIDGET(w));
                case msg of
                        WM_CLOSE: break;

                        TB_GOUP:
                                begin
                                DirviewOpenParentDirectory(dirv);
                                TextBoxSetTextEx(FileName, '');
                                end;

                        LB_CHANGE:
                                if assigned(PListboxItem(l^.SelectedRow)) then
                                begin
                                        if ( BRemove^.Flags and BF_DISABLED ) = BF_DISABLED then
                                        begin
                                                BRemove^.Flags := BRemove^.Flags and not BF_DISABLED;
                                                WidgetDraw(PWIDGET(BRemove), NIL);
                                        end;

                                        if assigned( l^.SelectedRow^.Data ) then
                                                DirviewOpenDirectory(dirv, l_text(l^.SelectedRow^.Data));

                                end
                                else
                                begin
                                        BRemove^.Flags := BRemove^.Flags or BF_DISABLED;
                                        WidgetDraw(PWIDGET(BRemove), NIL);
                                end;

                        MSG_IODLG_BOX_OK:
                        begin
                                if ( TextCompare(FileName^.Text, '') = 0 ) then
                                        OpenFile := FileNameToPath(dirv^.Directory,FileName^.Text);
                        end;

                        CB_FILTER:
                                if ( assigned(Types) and assigned(tsel^.Selected) ) then
                                begin
                                        ft := FileTypesAt(Types,ComboboxItemIndex(tsel,tsel^.Selected)-1);
                                        if assigned( ft ) then DirviewSetExtentions(dirv,ft^.Extentions);
                                end;

                        MSG_IODLG_ITEMFILE_SEL:
                        begin
                                a := GetSelectedItem(PLISTVIEW(dirv));
                                if assigned( a ) then
                                begin
                                        if ( PDIRVIEWITEM(a)^.f^.Attributes and FA_DIREC ) = FA_DIREC then
                                                TextBoxSetTextEx(FileName,'')
                                        else
                                                TextBoxSetTextEx(FileName,a^.Caption);
                                end;
                        end;

                        MSG_IODLG_ITEMFILE_VAL:
                        begin
                                a := GetSelectedItem(PLISTVIEW(dirv));
                                if assigned( a ) then
                                begin
                                        if ( not(PDIRVIEWITEM(a)^.f^.Attributes and FA_DIREC) = FA_DIREC) then
                                        begin
                                                OpenFile := FileNameToPath(dirv^.Directory,a^.Caption);
                                        end;
                                end;
                        end;
                end;

                (**
                 * TODO: Clean this crap
                 *)
                if ( OpenFile <> '' ) then
                begin
                        fi := FileGetInfo(OpenFile);

                        if assigned( fi ) then
                        begin
                                if ( fi^.Attributes and FA_DIREC ) = FA_DIREC then
                                begin
                                        DirviewOpenDirectory(dirv,OpenFile);
                                        TextBoxSetTextEx(FileName,'');
                                        OpenFile := '';
                                        FreeFileInfo(fi);
                                end
                                else if ( not FExists ) then
                                begin
                                        msg := MessageBox(@Me,'Warning',
                                                 'File already exists. Do you want to replace it?',
                                                  MBB_YESNOCANCEL or MBI_WARNING
                                               );
                                        FreeFileInfo(fi);
                                        if ( msg = MSG_YES ) then break;
                                        OpenFile := '';
                                        if ( msg = MSG_CANCEL ) then break;
                                end
                                else
                                begin
                                        FreeFileInfo(fi);
                                        {break;}
                                end;
                        end
                        else
                        begin
                                if ( FExists ) then
                                begin
                                        {free(OpenFile);}
                                        OpenFile := '';
                                end
                                else break;
                        end;
                end;
        end;

        if ( Dir <> '' ) then Dir := '';
        Dir := TextDup(dirv^.Directory);

        WidgetDispose(PWIDGET(w));
        DestroyImage(Icon);



        IOBox := OpenFile;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Files manipulation GUI progress info                                         *)
(*////////////////////////////////////////////////////////////////////////////////*)
function CollectCallBack ( id: Pointer; curfile: l_text ): l_bool;
begin
        SysPoll;

        if ( PWIDGET(PPROGRESSPOP(id)^.W)^.MessageBuffer = WM_CLOSE ) then begin
                ProgressPopSetInfoEx(PProgressPop(id),'Stopping...');
                CollectCallBack := false;
        end else
        begin
                ProgressPopSetInfo(PProgressPop(id),'Adding %s',[curfile]);
                DebugMessage('Adding %s...', [curfile]);
        end;
        CollectCallBack := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GenerateFileRefs ( f: PFileInfo ): PList;
var
  Lst: PList; p: PProgressPop;

begin
   Lst := NewList;
   p := ProgressPopUp(@Me,1,'I/O','Please wait. Collecting files...',0,'');
   if ( not _GenerateFileRefs( Lst, f, '', {$ifdef fpc}@{$endif}CollectCallBack, p ) ) then
   begin
     FreeList(Lst);
     ProgressPopKick(p);
     GenerateFileRefs := NIL;
   end;
   ProgressPopKick(p);
   GenerateFileRefs := Lst;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CollectFilesFromRef ( l:PList ): PList;
var
        Lst: PList;
        p: PProgressPop;
        a: PListItem;

begin
  a := l^.Last;
  CollectFilesFromRef := NIL;
  if not assigned(a) then CollectFilesFromRef := NIL else
  begin
    Lst := NewList;
    if not assigned( Lst ) then CollectFilesFromRef := NIL else
    begin
        p := ProgressPopUp(@Me,1,'I/O','Please wait. Collecting files...',0,'');
        repeat
           {  // below generates a null-pointer-warning in kernel.txt: filetopath.dir  }
           if ( not _GenerateFileRefs2( Lst, PFileRef(a^.Data), '', {$ifdef fpc}@{$endif}CollectCallBack, p ) ) then begin
                        FreeList(Lst);
                        ProgressPopKick(p);
                        CollectFilesFromRef := NIL;
           end;
           a := a^.Prev;

        until ( a = l^.Last );
        ProgressPopKick(p);
        CollectFilesFromRef := Lst;
    end;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function FileCopier ( p:PProgressPop; s:PFileStruct; d:PFileStruct ): l_bool;
var
   Buffer: array[0..4095] of char;
   r: Longint;
begin
        r := 0;
        while ( not l_bool(FileEOF(s)) ) do
        begin
                r := FileRead(Buffer,1,4096,s);
                SysPoll;
                FileWrite(Buffer,1,r,d);
                ProgressPopStep(p,r div 1024);
                SysPoll;
                if ( PWIDGET(p^.W)^.MessageBuffer = WM_CLOSE )
                         then ProgressPopSetInfoEx(p,'Stopping...');
        end;
        if ( PWIDGET(p^.W)^.MessageBuffer = WM_CLOSE ) then FileCopier := false else
        FileCopier := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure CopyFilesFromList ( l:PList; Dest:l_text );
{ // here is a bug! crashs pc if drag n drop of directories... Florian Xaver }
var
  GlobalSize: l_ulong;
  p: PProgressPop;
  r: PFileRef;
  f: PFileInfo;
  a,b: PListItem;
  s,d: PFileStruct;
  Text: l_text;
  outt: l_text;
  Ignore: l_bool;
  NewSName: l_text;
  ReplaceAll: l_bool;
  SamePlace,msg: l_ulong;
begin
  if assigned( l^.Last ) then begin
        GlobalSize := 0;
        p := NIL;
        a := l^.Last^.Next; b := a;
        Ignore := false;
        ReplaceAll := false;
        SamePlace := 0;

        repeat
                r := PFILEREF(a^.Data);

                if ( r^.Attributes and FA_DIREC ) = FA_DIREC then
                        GlobalSize := GlobalSize + 1
                else
                        GlobalSize += GlobalSize + r^.Size div 1024;

                a := a^.Next;
        until ( a = b );

        p := ProgressPopUp(@Me,7,'I/O','Please wait. Copying files...',GlobalSize,'...');
        DebugMessage('Copying files...',[]);

        repeat
                r := PFILEREF(a^.Data);
                if SamePlace > 0 then Outt := '' else Outt := FileNameToPath(Dest,r^.SName);
                if ( ReplaceAll or (SamePlace > 0) ) then f := NIL else f := FileGetInfo(Outt);
                ProgressPopSetInfo(p,'%s',[r^.SName]);
                SysPoll;

                Ignore := false;

                if assigned( f ) then begin

                        if (f^.Drive = r^.Drive) and
                           (TextCompare(f^.LocalName,r^.LocalName)=0) then begin
                                NewSName := TextArgs('Copy of %s',[r^.SName]);
                                Outt := '';
                                FreeFileInfo(f);
                                SamePlace := 1;
                                Outt := FileNameToPath(Dest,NewSName);
                                NewSName := '';
                                while ( f = FileGetInfo(Outt) ) do begin
                                        Outt := '';
                                        FreeFileInfo(f);
                                        NewSName := TextArgs('Copy(%d) of %s',[SamePlace-1,r^.SName]);
                                        Outt := FileNameToPath(Dest,NewSName);
                                        NewSName := '';
                                        Inc(SamePlace);
                                end;
                        end else begin

                                text := TextArgs('File %s (%d bytes) yet exists.\nDo you want to replace it with %s (%d bytes) ?',[Outt,f^.Size,r^.SName,r^.Size]);
                                msg := MessageBox(@Me,'I/O',text,MBB_YESYESALLNOCANCEL);

                                if ( msg = MSG_CANCEL ) then break;
                                if ( msg = MSG_NO ) then Ignore := true;
                                if ( msg = MSG_YESALL ) then ReplaceAll := true;

                                text := '';
                                FreeFileInfo(f); f := NIL;

                        end;
                end;

                if ( not Ignore ) then begin

                   if ( SamePlace > 0 ) then begin

                        if ( SamePlace > 1 ) then
                                NewSName := TextArgs('Copy(%d) of %s',[SamePlace-1,r^.SName])
                        else
                                NewSName := TextArgs('Copy of %s',[r^.SName]);
                        Outt := FileNameToPath(Dest,NewSName);
                   end;


                   if ( r^.Attributes and FA_DIREC ) = FA_DIREC then begin
                        MakeDir(Outt);
                        SysPoll;
                        ProgressPopStep(p,1);
                   end else if assigned( r^.Drive^.Driver^.FileOpen ) then begin
                        s := NIL;
                        d := NIL;
                        s := r^.Drive^.Driver^.FileOpen(r^.Drive,r^.LocalName,FSF_READ);
                        d := FileOpen(Outt,FSF_WRITE);
                        if ( assigned(d) and assigned(s) ) then begin
                                FileCopier(p,s,d);
                        end else
                        begin
                                DebugMessage ('Could not copy file - reason unknown! :-(',[]);
                                ProgressPopStep(p,r^.Size div 1024);
                        end;
                        if assigned( s ) then FileClose(s);
                        if assigned( d ) then FileClose(d);
                        DebugMessage ('File is copied!',[]);
                       { MessageBox(@Me,'I/O','ok',MBB_OK); }


                   end;

                end;
                if (Outt <> '') then Outt := '';
                if ( PWIDGET(p^.W)^.MessageBuffer = WM_CLOSE ) then break;
                a := a^.Next;
        until ( a = b );

        ProgressPopKick(p);
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DeleteFilesFromList ( l: PList );
var
        Nb: l_ulong = 0;
        p: PProgressPop;
        r: PFileRef;
        a: PListItem;
        b: PListItem;
begin
        a := l^.Last; b := a;

        Nb := 0;
        repeat
                Inc(Nb);
                a := a^.Prev;
        until ( a = b );

        p := ProgressPopUp(@Me,7,'I/O','Please wait. Removing files...',Nb,'...');

        repeat
                r := PFILEREF(a^.Data);

                ProgressPopSetInfo(p,'%s',[r^.SName]);

                SysPoll;

                if ( r^.Attributes and FA_DIREC ) = FA_DIREC then begin
                        if assigned( r^.Drive^.Driver^.DirDelete ) then
                                r^.Drive^.Driver^.DirDelete(r^.Drive,r^.LocalName);
                end else begin
                        if assigned( r^.Drive^.Driver^.FileDelete ) then
                                r^.Drive^.Driver^.FileDelete(r^.Drive,r^.LocalName);
                end;

                if ( PWIDGET(p^.W)^.MessageBuffer = WM_CLOSE ) then break;

                ProgressPopStep(p,1);
                a := a^.Prev;
        until ( a = b );

        ProgressPopKick(p);

end;
(*////////////////////////////////////////////////////////////////////////////////*)
function StdCreateDirview ( App: PApplication; r: TRect ): PWidget ;
begin
   StdCreateDirview := PWIDGET(CreateDirview (  App,  r, '/', NIL, 0));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function StdCreateTreeDirview ( App: PApplication; r: TRect ): PWidget;
begin
        StdCreateTreeDirview := PWIDGET(CreateTreeDirview( App, r ));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function LibMain ( argc: integer; argv: l_text ): l_bool;
begin
        DirviewClass := CreateWidgetClass( ULONG_ID('D','r','V','w'), ListviewClass, 0 );
        DirviewClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateDirview;

        TreeDirviewClass := CreateWidgetClass( ULONG_ID('D','T','r','V'), TreeviewClass, 0 );
        TreeDirviewClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateTreeDirview;


        GlobalDir := TextDup('/');

        ParentDirIcon := NewIcon(LoadImage('SYSTEM/ICONS/folpar16.bmp'),LoadImage('SYSTEM/ICONS/folpar32.bmp'),NIL);
        (*
        APPEXPORT(IOBox);
        APPEXPORT(CreateDirview);
        APPEXPORT(DirviewOpenDirectory);
        APPEXPORT(TreeDirViewOpenDir);
        APPEXPORT(FreeFileTypes);
        APPEXPORT(NewFileTypes);
        APPEXPORT(DirviewOpenParentDirectory);//
        APPEXPORT(CreateTreeDirview);
        APPEXPORT(CollectFilesFromRef);
        APPEXPORT(DirviewRefreshData);
        *)
        (*MouseShow();
        IOBox ( 'Select a file', 'Open', '/', NIL, true );*)

        DirOpenMSG := GetTypeDefaultMsg(DTYPE_DIRECTORY);

        IconOpen := LoadImage('./SYSTEM/ICONS/open.bmp');
        IconSave := LoadImage('./SYSTEM/ICONS/save.bmp');
        IconAdd := LoadImage('SYSTEM/ICONS/add.bmp');
        IconUp := LoadImage('SYSTEM/ICONS/up.bmp');
        IconRemove := LoadImage('SYSTEM/ICONS/remove.bmp');
        IconCancel := LoadImage('./SYSTEM/ICONS/cancel.bmp');



        LibMain := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Close;
begin
        if ( GlobalDir <> '' ) then GlobalDir := '';
        if assigned( ParentDirIcon ) then FreeIcon(ParentDirIcon);

        if assigned( IconOpen ) then al_destroy_bitmap(IconOpen);
        if assigned( IconSave ) then al_destroy_bitmap(IconSave);
        if assigned( IconAdd ) then al_destroy_bitmap(IconAdd);
        if assigned( IconUp ) then al_destroy_bitmap(IconUp);
        if assigned( IconRemove ) then al_destroy_bitmap(IconRemove);
        if assigned( IconCancel ) then al_destroy_bitmap(IconCancel);

end;
(*////////////////////////////////////////////////////////////////////////////////*)
{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion := ULONG_ID(0,0,0,1);

{$endif}   {LINK_CLIBS}
end.
