(**
*	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
*
*	FILE:			keyboard.c
*
*	PROJECT:		Phoenix engine - Core
*
*	DESCRIPTION:	The keyboard event handler
*
*	CONTRIBUTORS:
*					Lukas Lipka
*					Julien Etelain
*
*	TODO:			@
*
*	BUG:			@
*
*	MISTAKE:		@
*
*	IDEA:			@
*)
(*////////////////////////////////////////////////////////////////////////////////*)
{
 .#include "kernel.h"
 .#include "allegro/internal/aintern.h"
}
unit keyboard;

interface
 
uses guitypes, events, List, vfile, text;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {LINK keyboard.o}
{$endif}

const
  KEY_MAX = 256;
{$ifndef _OZONE__LINUX_}

var 
  std_key_ascii_table    : array[0..KEY_MAX-1] of l_ushort;
  std_key_capslock_table : array[0..KEY_MAX-1] of l_ushort;
  std_key_shift_table    : array[0..KEY_MAX-1] of l_ushort;
  std_key_control_table  : array[0..KEY_MAX-1] of l_ushort;
  std_key_empty_table    : array[0..KEY_MAX-1] of l_ushort;

{$endif}

var KState: TKeybState;

(* Keyboard support *)
const
  KB_ALT_FLAG   =  $00000008;
  KB_CTRL_FLAG  =  $00000004;
  KB_SHIFT_FLAG =  $00000003;

  KB_ALT        =  KB_ALT_FLAG;
  KB_CTRL       =  KB_CTRL_FLAG;
  KB_SHIFT      =  KB_SHIFT_FLAG;

function ASCII(e: PKeybState): Longint;     { (e->Keyb.Key & 0xFF)       }
function KEY(e: PKeybState; k: Longint): l_bool;                  { (ASCII(e) == k)            }
function SCANCODE(e: PKeybState; k: Longint): l_bool;             { ((e->Keyb.Key >> 8) == k)  }
function ALT(e: PKeybState; k: Longint): l_bool;                  { (((e->Keyb.Key >> 8) == k) && (e->Keyb.Shift & KB_ALT))   }
function CTRL(e: PKeybState; k: Longint): l_bool;                 { (((e->Keyb.Key >> 8) == k) && (e->Keyb.Shift & KB_CTRL))  }
function SHIFT(e: PKeybState; k: Longint): l_bool;                { (((e->Keyb.Key >> 8) == k) && (e->Keyb.Shift & KB_SHIFT)) }
function COMBINE(e: PKeybState; k,c: Longint): l_bool;            { (((e->Keyb.Key >> 8) == k) && (e->Keyb.Shift & (c)))      }

(* event generator for keyboard events *)
procedure KeyboardEventGenerator(Event: PEvent); {$ifdef LINK_CLIBS}external;{$endif}

(* installs the keyboard system *)
procedure GSSystemKeyboardInstall;     {$ifdef LINK_CLIBS}external;{$endif}
procedure GSSystemKeyboardUnInstall;   {$ifdef LINK_CLIBS}external;{$endif}


type 
  PKeyboardLayout = ^TKeyboardLayout;
  TKeyboardLayout = record
	
	accent1: Integer;
	accent2: Integer;
	accent3: Integer;
	accent4: Integer;
	
	accent1_flag: Integer;
	accent2_flag: Integer;
	accent3_flag: Integer;
	accent4_flag: Integer;
	
	ascii_table        : array[0..KEY_MAX-1] of l_ushort;
	capslock_table     : array[0..KEY_MAX-1] of l_ushort;
	shift_table        : array[0..KEY_MAX-1] of l_ushort;
	control_table      : array[0..KEY_MAX-1] of l_ushort;
	altgr_lower_table  : array[0..KEY_MAX-1] of l_ushort;
	altgr_upper_table  : array[0..KEY_MAX-1] of l_ushort;
	accent1_lower_table: array[0..KEY_MAX-1] of l_ushort;
	accent1_upper_table: array[0..KEY_MAX-1] of l_ushort;
	accent2_lower_table: array[0..KEY_MAX-1] of l_ushort;
	accent2_upper_table: array[0..KEY_MAX-1] of l_ushort;
	accent3_lower_table: array[0..KEY_MAX-1] of l_ushort;
	accent3_upper_table: array[0..KEY_MAX-1] of l_ushort;
	accent4_lower_table: array[0..KEY_MAX-1] of l_ushort;
	accent4_upper_table: array[0..KEY_MAX-1] of l_ushort;
	
	Name: array[0..31] of l_char;
	Code: array[0..4] of l_char;
	
  end;

const CurrentLayout: PKeyboardLayout = NIL;


procedure KeyboardSetLayout ( l: PKeyboardLayout );   {$ifdef LINK_CLIBS}external;{$endif}
function KeyboardGetLayout: PKeyboardLayout;          {$ifdef LINK_CLIBS}external;{$endif}
procedure KeyboardInstallLayout;                      {$ifdef LINK_CLIBS}external;{$endif}

function __KeyboardStop: l_bool;                      {$ifdef LINK_CLIBS}external;{$endif}
function __KeyboardRestart: l_bool;                   {$ifdef LINK_CLIBS}external;{$endif}

implementation
{$ifndef LINK_CLIBS}
(*////////////////////////////////////////////////////////////////////////////////*)
var
  _key_ascii_table         : array[0..KEY_MAX-1] of l_ushort;
  _key_capslock_table      : array[0..KEY_MAX-1] of l_ushort;
  _key_shift_table         : array[0..KEY_MAX-1] of l_ushort;
  _key_control_table       : array[0..KEY_MAX-1] of l_ushort;
  _key_altgr_lower_table   : array[0..KEY_MAX-1] of l_ushort;
  _key_altgr_upper_table   : array[0..KEY_MAX-1] of l_ushort;
  _key_accent1_lower_table : array[0..KEY_MAX-1] of l_ushort;
  _key_accent1_upper_table : array[0..KEY_MAX-1] of l_ushort;
  _key_accent2_lower_table : array[0..KEY_MAX-1] of l_ushort;
  _key_accent2_upper_table : array[0..KEY_MAX-1] of l_ushort;
  _key_accent3_lower_table : array[0..KEY_MAX-1] of l_ushort;
  _key_accent3_upper_table : array[0..KEY_MAX-1] of l_ushort;
  _key_accent4_lower_table : array[0..KEY_MAX-1] of l_ushort;
  _key_accent4_upper_table : array[0..KEY_MAX-1] of l_ushort;
  _key_accent1             : Integer;
  _key_accent2             : Integer;
  _key_accent3             : Integer;
  _key_accent4             : Integer;
  _key_accent1_flag        : Integer;
  _key_accent2_flag        : Integer;
  _key_accent3_flag        : Integer;
  _key_accent4_flag        : Integer;
  _key_standard_kb         : l_bool;
  {
  std_key_ascii_table 	   : array[0..KEY_MAX-1] of l_ushort;
  std_key_capslock_table   : array[0..KEY_MAX-1] of l_ushort;
  std_key_shift_table 	   : array[0..KEY_MAX-1] of l_ushort;
  std_key_control_table    : array[0..KEY_MAX-1] of l_ushort;
  std_key_empty_table 	   : array[0..KEY_MAX-1] of l_ushort;
  }

procedure KeyboardSetLayout ( l: PKeyboardLayout );
begin	
{$ifndef _OZONE__LINUX_}
	
	if Assigned( CurrentLayout ) then Dispose(CurrentLayout);
	New(CurrentLayout); { = malloc(sizeof(TKeyboardLayout)); }
	if Assigned(CurrentLayout ) then
	begin
	move(l^,CurrentLayout^,sizeof(TKeyboardLayout));
	
  _key_ascii_table         := CurrentLayout^.ascii_table;
  _key_capslock_table      := CurrentLayout^.capslock_table;
  _key_shift_table         := CurrentLayout^.shift_table;
  _key_control_table       := CurrentLayout^.control_table;
  _key_altgr_lower_table   := CurrentLayout^.altgr_lower_table;
  _key_altgr_upper_table   := CurrentLayout^.altgr_upper_table;
  _key_accent1_lower_table := CurrentLayout^.accent1_lower_table;
  _key_accent1_upper_table := CurrentLayout^.accent1_upper_table;
  _key_accent2_lower_table := CurrentLayout^.accent2_lower_table;
  _key_accent2_upper_table := CurrentLayout^.accent2_upper_table;
  _key_accent3_lower_table := CurrentLayout^.accent3_lower_table;
  _key_accent3_upper_table := CurrentLayout^.accent3_upper_table;
  _key_accent4_lower_table := CurrentLayout^.accent4_lower_table;
  _key_accent4_upper_table := CurrentLayout^.accent4_upper_table;
	_key_accent1 := CurrentLayout^.accent1;
	_key_accent2 := CurrentLayout^.accent2;
	_key_accent3 := CurrentLayout^.accent3;
	_key_accent4 := CurrentLayout^.accent4;
	_key_accent1_flag := CurrentLayout^.accent1_flag;
	_key_accent2_flag := CurrentLayout^.accent2_flag;
	_key_accent3_flag := CurrentLayout^.accent3_flag;
	_key_accent4_flag := CurrentLayout^.accent4_flag;
  _key_standard_kb := FALSE;
    end;
{$endif}

end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeyboardGetLayout: PKeyboardLayout;
begin
  KeyboardGetLayout := CurrentLayout;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure KeyboardEventGenerator ( Event: PEvent );
begin
   Event^.aType := EV_NOTHING;

   KState.Shift := 3{key_shifts};
   {$ifdef _OZONE__DJGPP_}
   if keypressed then
   begin
		Event^.aType := EV_KEYBOARD;
		KState.Key   := ord(readkey);

		Event->Keyb  := KState;

   end else
		KState.Key := 0;
   {$endif}

   {$ifdef _OZONE__WIN32_}
   {$endif}

   {$ifdef _OZONE__LINUX_}
   {$endif}
end;

(*////////////////////////////////////////////////////////////////////////////////*)
procedure KeyboardDefaultLayout;
begin	
	{$ifndef _OZONE__LINUX_}
	_key_ascii_table 		 := std_key_ascii_table;
	_key_capslock_table      := std_key_capslock_table;
	_key_shift_table         := std_key_shift_table;
	_key_control_table       := std_key_control_table;
	_key_altgr_lower_table   := std_key_empty_table;
	_key_altgr_upper_table   := std_key_empty_table;
	_key_accent1_lower_table := std_key_empty_table;
	_key_accent1_upper_table := std_key_empty_table;
	_key_accent2_lower_table := std_key_empty_table;
	_key_accent2_upper_table := std_key_empty_table;
	_key_accent3_lower_table := std_key_empty_table;
	_key_accent3_upper_table := std_key_empty_table;
	_key_accent4_lower_table := std_key_empty_table;
	_key_accent4_upper_table := std_key_empty_table;
	_key_accent1 := 0;
	_key_accent2 := 0;
	_key_accent3 := 0;
	_key_accent4 := 0;
	_key_accent1_flag := 0;
	_key_accent2_flag := 0;
	_key_accent3_flag := 0;
	_key_accent4_flag := 0;
	_key_standard_kb := TRUE;
	{$endif}
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeyGetText(Path,Text: l_text): l_text;
begin

end;

procedure KeyboardInstallLayout;
var 
 LayoutFile: l_text;
 f: PFileStruct;
 CL: PKeyboardLayout;
begin
	
	{$ifndef _OZONE__LINUX_}
	
	LayoutFile := KeyGetText('/SYSTEM/KEYBOARD/LAYOUT', '');

	if ( TextCompare(LayoutFile,'' ) > 0 ) then
	begin
		f := FileOpen(LayoutFile,FSF_READ);
		if Assigned( f ) then
		begin
			New(CL); { = malloc(sizeof(TKeyboardLayout));			 }
			if Assigned( CL ) then
			begin
				FileRead(CL^,1,sizeof(TKeyboardLayout),f);
				KeyboardSetLayout(CL);
				Dispose(CL);
			end;
			FileClose(f);
		end;
	end	else if Assigned( CurrentLayout ) then
	begin
		Dispose(	CurrentLayout);
		CurrentLayout := NIL;
		KeyboardDefaultLayout;
	end;
	
	{$endif}
	
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure GSSystemKeyboardInstall;
begin
	{$ifndef _OZONE__LINUX_}
	std_key_ascii_table 	:= _key_ascii_table;
	std_key_capslock_table 	:= _key_capslock_table;
	std_key_shift_table 	:= _key_shift_table;
	std_key_control_table 	:= _key_control_table;
	std_key_empty_table 	:= _key_altgr_lower_table;
	{$endif}
	
	{install_keyboard;}
	InstallEventGenerator({$ifdef fpc}@{$endif}KeyboardEventGenerator);
	
	{$ifndef _OZONE__LINUX_}
	KeyboardInstallLayout();
	{$endif}
	
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure GSSystemKeyboardUnInstall;
begin
	if Assigned( CurrentLayout ) then Dispose(CurrentLayout);
	CurrentLayout := NIL;
	{$ifndef _OZONE__LINUX_}
	KeyboardDefaultLayout;
	{$endif}
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function __KeyboardStop: l_bool;
begin
	{remove_keyboard;}
	__KeyboardStop := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function __KeyboardRestart: l_bool;
begin
	{install_keyboard;}
	KeyboardInstallLayout;
	__KeyboardRestart := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)

function ASCII(e: PKeybState): Longint;
begin
  ASCII := (e^.Key and $FF);
end;


function KEY(e: PKeybState; k: Longint): l_bool;
begin
  Key := (ASCII(e) = k);
end;

function SCANCODE(e: PKeybState; k: Longint): l_bool;
begin
   SCANCODE :=((e^.Key shl 8) = k);
end;

function ALT(e: PKeybState; k: Longint): l_bool;
begin
  ALT := ((e^.Key shl 8) = k) and ((e^.Shift and KB_ALT) = KB_ALT);
end;

function CTRL(e: PKeybState; k: Longint): l_bool;
begin
  CTRL := ((e^.Key shl 8) = k) and ((e^.Shift and KB_CTRL) = KB_CTRL);
end;       
       
function  SHIFT(e: PKeybState; k: Longint): l_bool;
begin
  SHIFT := ((e^.Key shl 8) = k) and ((e^.Shift and KB_SHIFT) = KB_SHIFT);
end;
  
function COMBINE(e: PKeybState; k: Longint; c: Longint): l_bool;
var tempA,tempB: l_bool;
begin
  {
  tempA := SHIFT(e,k);
  tempB := (e^.Shift and c) = c;
  COMBINE := tempA and tempB);
  }
  COMBINE := ((e^.Key shl 8) = k) and ((e^.Shift and c) = c);
end;


{$endif}

end.
