////////////////////////////////////////////////////////////////////////////////
//
//    List Manager - Core file
//
//  (c) Copyright 2003 Point Mad. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
unit list;
{
 .#include <stdio.h>
 .#include "types.h"
 .#include "list.h"
 .#include "text.h"
 .#include <stdlib.h>
 .#include <string.h>
}

interface

uses Sysutils, guitypes;

{$i cdefs.inc}

{$IFDEF LINK_CLIBS}
  {$LINK list.o}
  {$LINK text.o}
{$ENDIF}

{$undef _include_in_vfile_}

type
  {$i vfile.inc}
  TFreeProc  = procedure (p: Pointer);
  PListItem = ^TListItem;
  TListItem = record
	Next: PListItem;
	Prev: PListItem;
	Key: AnsiString;
	Data: Pointer;
	FreeData: TFreeProc;

	DataType: l_ulong;
        DataSize: l_ulong;
  end;

  PList = ^TList;
  TList = record
	Last: PListItem;
	Current: PListItem;	
  end;

  //ListFunc = function(d: PDrive; VDir, Dir: l_text; Attrib: l_uchar): PList;

  TComparePT = function (p1, p2: Pointer): Longint;
  TCompareLI = function (p1, p2: PListItem): Longint;

  //ListFunc = function(d: PDrive; VDir, Dir: l_text; Attrib: l_uchar): Pointer;
{$i vfiledrv.inc}

type
  PBListItem = ^TBListItem;
  PSListItem = ^TSListItem;
///**
//	Basic List Item
//*/
  TBListItem = record
	Next: PBListItem;
	Prev: PBListItem;
	//SLIt: PSListItem
    //Data: Pointer;
  end;

{ .#define BLISTITEM(o) ((PBListItem)(o))}

///**
//	Simple List Item 
//*/

  TSListItem = record
	Next: PSListItem;
	Prev: PSListItem;
	o: PBListItem;
	Data: Pointer;
	//Link: PBListItem;     //hinzugefgt am 20.01.2016, war voher -> Data: Pointer
  end;

{ .#define SLISTITEM(o) ((PSListItem)(o))}

///**
//	Basic and Simple List (identical) 
//*/
type 
  PBList = ^TBList;
  TBList = record
	Last: PSListItem;
	Link: PSListItem;     //hinzugefgt am 20.01.2016
  end; 
  PSList = PBList;
  //FreeProc = procedure (Data: Pointer);

{ .#define BLIST(o) ((PBList)(o))}

procedure BListRemoveItemEx ( o: PBList; i: PBListItem ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
function SListAdd ( o: PBList; var Data ): PSListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function BListAddItem ( o: PBList; i: PBListItem ): PBListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure FreeSList ( var o: PBList; FreeData: TFreeProc ); {$IFDEF LINK_CLIBS}external;{$ENDIF}
function NewBList ( ): PBList; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function SListFoundItem ( l: PBList; var Data: Pointer ): PSListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
  

function NewListItemEx   ( Key: AnsiString; var Data: Pointer; FreeData: TFreeProc; DataType, DataSize: l_ulong ): PListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function NewListItem     ( Key: AnsiString; var Data: Pointer; FreeData: TFreeProc ): PListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListAddItem     ( o: PList; i: PListItem ): PListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListAdd         ( o: PList; Key: AnsiString; Data: Pointer; FreeData: TFreeProc ): PListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListRemoveItem  ( o: PList; i: PListItem ): Byte; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListAtItem      ( o: PList; Index: Longword ): PListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListAt          ( o: PList; Index: Longword ): Pointer; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListKeyItem     ( o: PList; Key: Ansistring ): PListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListKey         ( o: PList; Key: AnsiString ): Pointer; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListKeyCaseItem     ( o: PList; Key: AnsiString ): PListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListKeyCase         ( o: PList; Key: AnsiString ): Pointer; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListFoundItem   ( o: PList; var Data: Pointer ): PListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function NewList         (  ): PList; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListFoundIndex ( o: PList; var Data: Pointer ): LongWord; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListAddItemBefore ( o: PList; i, b: PListItem ): PListItem; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListRemoveItemEx ( o: PList; i: PListItem ): Byte; {$IFDEF LINK_CLIBS}external;{$ENDIF}

// /!\ This 2 functions dot no order Items in List, but Data /!\ ->-> To be improved ...
function ListOrderByData ( o: PList; comparator: TComparePT ): Byte; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListOrderByItem ( o: PList; comparator: TCompareLI ): Byte; {$IFDEF LINK_CLIBS}external;{$ENDIF}

// free list, all items and data. Ideal for most uses.
procedure FreeList ( o: PList ); {$IFDEF LINK_CLIBS}external;{$ENDIF}

// Same as freelist but do not free data. (skip DataFree)
procedure KickList ( o: PList ); {$IFDEF LINK_CLIBS}external;{$ENDIF}

implementation

{$IFNDEF LINK_CLIBS}

function SListFoundItem ( l: PBList; var Data: Pointer ): PSListItem; inline;
var a,b: PSListItem; c: PSListItem;
begin
  SListFoundItem := NIL;
  if not Assigned( l^.Last ) then SListFoundItem := NIL else
  begin
    a := l^.Last^.Next;
    b := l^.Last^.Next;
    if Assigned( a ) then
    while a<>nil do {vorher a<>b}
    begin
      if ( a^.Data = Data ) then begin SListFoundItem := a; break; end;
      a := a^.Next;
    end; 
  end;
end;

function NewBList ( ): PBList; inline;
var o: PBList;
begin
  New(o);
  o^.Last := NIL;
  NewBList := o;
end;

procedure FreeSList ( var o: PBList; FreeData: TFreeProc ); inline;
var a,b,n: PSListItem;
begin
	if Assigned( o^.Last ) then
	begin
		a := o^.Last;
        b := o^.Last;
		while a<>nil do {vorher a<>b}
        begin 
			n := a^.Prev;
			//if ( FreeData && SLISTITEM(a)->Data ) FreeData(SLISTITEM(a)->Data);
			if Assigned(FreeData) then if Assigned(a^.Data) then FreeData(a^.Data);
			Dispose(a);
			a := n;
		end;
	end;
	Dispose(o);
end;

function BListAddItem ( o: PBList; i: PBListItem ): PBListItem; inline;
begin
	if Assigned( o^.Last ) then
	begin
		i^.Next := o^.Last^.o^.Next;
		i^.Prev := o^.Last^.o;
		o^.Last^.o^.Next^.Prev := i;
		o^.Last^.o^.Next := i;
		o^.Last^.o := i;
	end else 
	begin
		i^.Next := i;
		i^.Prev := i;
		o^.Last^.o := i;
	end;
	BListAddItem := i;
end;

function SListAdd ( o: PBList; var Data ): PSListItem; inline;
var
  i: PSListItem;
begin	
  //PSListItem i = (PSListItem)malloc(sizeof(TSListItem));
  New(i);
  i^.Data := Pointer(Data);
  o^.Last := i;
  o^.Last := i;
  //o^.Last^.SLIt := i;
  //BListAddItem(o,BLISTITEM(i));
  if Assigned( o^.Last ) then
  begin
	i^.o^.Next := o^.Last^.o^.Next;  //Ursprnglich nicht i^.Link, sondern nur i.
	i^.o^.Prev := o^.Last^.o;
	o^.Last^.o^.Next^.Prev := i^.o;
	o^.Last^.o^.Next := i^.o;
	o^.Last := i;
  end else 
  begin
	i^.Next := i;
	i^.Prev := i;
	o^.Last := i;
  end;
  SListAdd := i;
end;

procedure BListRemoveItemEx ( o: PBList; i: PBListItem ) ; inline;
begin
  if ( i^.Next = i ) then
	  o^.Last := NIL
    else 
    begin
	  i^.Next^.Prev := i^.Prev;
	  i^.Prev^.Next := i^.Next;
	  if ( o^.Last^.o = i ) then o^.Last^.o := i^.Prev;
    end;
end;

///** @defgroup list List system.
//        Widely used in kernel and libraries, PBList and derivated are very usefull.
//        PList is the most used system as it is the oldest and have interresting functionalities.
//        PBList and PSList provide an inline module, so very fast, and lightweight structures but have little funtionalities.
//        @n PSList is derivated form PBList : PBList functions can be used for PSList.
//        @n PList is derivated from PSList : PBList and PSList functions can be used for PList (to get better performances).
//        @n *BUT* do not allocate a PSList item in a PList, system will crash.
//        @{
//*/
///**     @file
//        List core file
//*/

////////////////////////////////////////////////////////////////////////////////
function NewListItem ( Key: AnsiString; var Data: Pointer; FreeData: TFreeProc ): PListItem;
var i: PListItem;
begin
  New(i);
  if not Assigned( i ) then NewListItem := NIL else
  begin
    fillchar(i^, sizeof(TListItem), 0);
    i^.Data := Data;

    if ( Key <> '' ) then i^.Key := Key;

    i^.FreeData := FreeData;
    NewListItem := i;
  end;
end;
////////////////////////////////////////////////////////////////////////////////
function NewListItemEx ( Key: AnsiString; var Data: Pointer; FreeData: TFreeProc; DataType, DataSize: l_ulong ): PListItem; 
var i: PListItem;
begin
  New(i);
  if not Assigned( i ) then NewListItemEx := NIL else
  begin
        fillchar(i^, sizeof(TListItem), 0);
        i^.Data := Data;

        if ( Key <> '' ) then i^.Key := Key;

        i^.FreeData := FreeData;
        i^.DataType := DataType;
        i^.DataSize := DataSize;
        NewListItemEx := i;
  end;
end;
////////////////////////////////////////////////////////////////////////////////
function ListAddItemBefore ( o: PList; i, b: PListItem ): PListItem;
begin
        if not Assigned( o^.Last ) then
        begin
                i^.Next := i;
                i^.Prev := i;
                o^.Last := i;
        end
        else
        begin
                if Assigned( b ) then
                begin
                        i^.Prev := b^.Prev;
                        i^.Next := b;
                        b^.Prev^.Next := i;
                        b^.Prev := i;
                end
                else
                begin
                        i^.Next := o^.Last^.Next;
                        i^.Prev := o^.Last;
                        o^.Last^.Next^.Prev := i;
                        o^.Last^.Next := i;
                        o^.Last := i;
                end;
        end;
        ListAddItemBefore := i;
end;
////////////////////////////////////////////////////////////////////////////////
function ListAddItem ( o: PList; i: PListItem ): PListItem;
begin
	if Assigned( o^.Last ) then
	begin
		i^.Next := o^.Last^.Next;
		i^.Prev := o^.Last;
		o^.Last^.Next^.Prev := i;
		o^.Last^.Next := i;
		o^.Last := i;
	end else 
	begin
		i^.Next := i;
		i^.Prev := i;
		o^.Last := i;
	end;
        //BListAddItem(BLIST(o),BLISTITEM(i));
    ListAddItem := i;
end;
////////////////////////////////////////////////////////////////////////////////
function ListAdd ( o: PList; Key: AnsiString; Data: Pointer; FreeData: TFreeProc ): PListItem;
var i: PListItem;
begin  
  i := NewListItem(Key,Data,FreeData);      
  if not Assigned( i ) then ListAdd :=  NIL else ListAdd := ListAddItem( o, i);
  //return (PListItem)BListAddItem(BLIST(o),BLISTITEM(i));
end;
////////////////////////////////////////////////////////////////////////////////
function ListRemoveItemEx ( o: PList; i: PListItem ): Byte;
begin  
  //BListRemoveItemEx(BLIST(o),BLISTITEM(i));
  if ( i^.Next = i ) then
	  o^.Last := NIL
    else 
    begin
	  i^.Next^.Prev := i^.Prev;
	  i^.Prev^.Next := i^.Next;
	  if ( o^.Last = i ) then o^.Last := i^.Prev;
    end;
        
   ListRemoveItemEx := 1;
end;
////////////////////////////////////////////////////////////////////////////////
function ListRemoveItem ( o: PList; i: PListItem ): Byte;
begin 
  //BListRemoveItemEx(BLIST(o),BLISTITEM(i));
  if ( i^.Next = i ) then
	  o^.Last := NIL
    else 
    begin
	  i^.Next^.Prev := i^.Prev;
	  i^.Prev^.Next := i^.Next;
	  if ( o^.Last = i ) then o^.Last := i^.Prev;
    end;
  
  //if Assigned( i^.Key ) then i^.Key := '';
  if Assigned( i^.Data ) and Assigned( i^.FreeData ) then i^.FreeData(i^.Data);
  Dispose(i);
  ListRemoveItem := 1;
end;
////////////////////////////////////////////////////////////////////////////////
//**
//        Found the item that index is specified one.
//        @param o List containing the item
//        @param Index Index to found
//        @return NULL on error, or searched data
//*/
function ListAtItem ( o: PList; Index: Longword ): PListItem;
var a, b: PListItem;
begin
  ListAtItem := NIL;
  if not Assigned( o^.Last ) then ListAtItem := NIL;
  a := o^.Last^.Next;
  b := o^.Last^.Next;
  Dec(Index);
  if Assigned( a ) then 
  while (a <> NIL) and (Index >= 0) do
  begin
    if ( Index = 0 ) then begin ListAtItem := a; break; end else
    begin
      Dec(Index);
      a := a^.Next;
    end;
  end;                           // while ( a != b );
end;
////////////////////////////////////////////////////////////////////////////////
//**
//        Found the data associated to item that index is specified one.
//        @param o List containing the item
//        @param Index Index to found
//        @return NULL on error, or searched data
//*/
function ListAt ( o: PList; Index: Longword ): Pointer; 
var a: PListItem;
begin
  ListAt := NIL;
  a := ListAtItem(o,Index);
  if Assigned( a ) then ListAt := a^.Data;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Found the item that Key is specified one. Case sensitive.
//        @param o List containing the item
//        @param Key Item key to found
//        @return NULL on error, or a pointer to item
//*/
//Def a item as marker first
function DefItemAsMarker(i: PListItem): PListItem;
var Data: PInteger; tmpdata: Pointer; notnil: Boolean;
begin
  notnil := false;
  if i^.Data <> NIL then
  begin
    if i^.DataSize > 0 then
    begin
      getmem(tmpdata, i^.DataSize);
      move(i^.Data^, tmpdata^, i^.DataSize);
      freemem(i^.Data,i^.DataSize);
      i^.Data := NIL;
      New(Data); Data^ := $77;
      i^.Data := Data;
      notnil := true;
    end
  end
  else
  begin
    tmpData := i^.Data;
    New(Data); Data^ := $37;
    i^.Data := Data;
  end;


  if i^.DataSize > 0 then
  begin
    getmem(i^.Data, i^.DataSize);
    move(tmpdata^, i^.Data^, i^.DataSize);
  end else if notnil then i^.Data := tmpData;
end;

function ListKeyItem ( o: PList; Key: AnsiString ): PListItem;
var a, b: PListItem; Data: PInteger; tmpdata: Pointer;
begin
  ListKeyItem := nil;
  if not Assigned( o^.Last ) then ListKeyItem := NIL else
  begin
    a := o^.Last^.Next;
    b := o^.Last^.Next;
    if Assigned( a ) then
    begin
      if ( a^.Key <> '' ) then if ( Key = a^.Key) then ListKeyItem := a else a := a^.Next;
      while a <> b do
      begin
        if ( a^.Key <> '' ) then
        if ( Key = a^.Key) then
        begin
          ListKeyItem := a;
          break;
        end else
        a := a^.Next;
      end;    //while ( a != b );
    end;
  end;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Found the data associated to item that Key is specified one. Case sensitive.
//        @param o List containing the item
//        @param Key Item key to found
//        @return NULL on error, or searched data
//*/
function ListKey ( o: PList; Key: AnsiString ): Pointer;
var a: PListItem; 
begin
  a := ListKeyItem(o,Key);
  if Assigned( a ) then ListKey := a^.Data else ListKey := NIL;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Found the item that Key is specified one ignoring the case.
//        @param o List containing the item
//        @param Key Item key to found
//        @return NULL on error, or a pointer to item
//*/
function ListKeyCaseItem ( o: PList; Key: AnsiString ): PListItem;
var a,b: PListItem;
begin
  ListKeyCaseItem := NIL;
  if not Assigned( o^.Last ) then ListKeyCaseItem := NIL else
  begin
    a := o^.Last^.Next;
    b := o^.Last^.Next;
    if Assigned( a ) then
    if ( a^.Key <> '' ) then if ( AnsiUpperCase(Key) = AnsiUpperCase(a^.Key)) then ListKeyCaseItem := a else a := a^.Next;
    while a <> b do
    begin
      if ( a^.Key <> '' ) then
       if ( AnsiUpperCase(Key) = AnsiUpperCase(a^.Key))  then
       begin
         ListKeyCaseItem := a;
         break;
       end;
      a := a^.Next;
    end; //  while ( a != b );
  end;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Found the data associated to item that Key is specified one ignoring the case.
//        @param o List containing the item
//        @param Key Item key to found
//        @return NULL on error, or searched data
//*/
function ListKeyCase ( o: PList; Key: AnsiString ): Pointer;
var a: PListItem;        
begin
  a := ListKeyCaseItem(o,Key);
  if Assigned( a ) then ListKeyCase := a^.Data else ListKeyCase := NIL;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Found the item that Data is specified one.
//        @param o List containing the item
//        @param Data Data to found
//        @return NULL on error, or a pointer to item
//        @note Similar to ((PListItem)SListFoundItem((PSList)o,Data))
//*/
function ListFoundItem ( o: PList; var Data: Pointer ): PListItem;
var a, b: PListItem;
begin
  ListFoundItem := NIL;
  if not Assigned( o^.Last ) then ListFoundItem := NIL else
  begin
    a := o^.Last^.Next;
    b := o^.Last^.Next;
    if Assigned( a ) then
    if ( a^.Data = Data ) then ListFoundItem := a else a := a^.Next;
    while a <> b do
    begin
      if ( a^.Data = Data ) then begin ListFoundItem := a; break; end;
      a := a^.Next;
    end;   // while ( a != b );
  end;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Determines the index of item that Data is specified one.
//        @param o List containing the item
//        @param Data Data to found
//        @return -1 on error, or item index
//*/
function ListFoundIndex ( o: PList; var Data: Pointer ): Longword;
var a, b: PListItem; i: l_ulong;
begin
  ListFoundIndex := $FFFFFFFF;
  i := 1;
  if (not Assigned( o^.Last )) then ListFoundIndex := 0 else
  begin
    a := o^.Last^.Next;
    b := o^.Last^.Next;
    if Assigned( a ) then
    if ( a^.Data = Data ) then ListFoundIndex := i else begin a := a^.Next; Inc(i); end;
    while a <> b do
    begin
      if ( a^.Data = Data ) then begin ListFoundIndex := i; break; end;
      a := a^.Next;
      Inc(i);
    end; // while ( a != b );
  end;
end;
////////////////////////////////////////////////////////////////////////////////
function ListOrderByData ( o: PList; comparator: TComparePT ): Byte;
var 
  c, a, b: PListItem;
  t: TListItem;
  aData: Pointer;
  aType: l_ulong;
  aSize: l_ulong;
  aKey: AnsiString;
begin
  if (not Assigned( o )) and
     (not Assigned(o^.Last)) and
     (not Assigned(o^.Last^.Next)) and
     (not Assigned(comparator)) then ListOrderByData := 0
  //fix 27.01.2007
  else
  begin
    c := o^.Last^.Next;
    a := o^.Last^.Next;
    while a <> NIL {c != a} do 
    begin
      b := a;
      while b <> NIL do 
      begin
        if ( Assigned(a^.Data) and Assigned(b^.Data) and Assigned(comparator) ) then
        if comparator( a^.Data, b^.Data ) > 0 then
        begin
        //* char * aKey = aKey;
        aKey  := a^.Key;
        aData := a^.Data;
        aType := a^.DataType;
        aSize := a^.DataSize;

        a^.Key  := b^.Key;
        a^.Data := b^.Data;
        a^.DataType := b^.DataType;
        a^.DataSize := b^.DataSize;

        b^.Key  := aKey;
        b^.Data := aData;
        b^.DataType := aType;
        b^.DataSize := aSize;
        {
        memcpy((void*)(((long)&t)+8),(void*)(((long)a)+8),sizeof(TListItem)-8);
        memcpy((void*)(((long)a)+8),(void*)(((long)b)+8),sizeof(TListItem)-8);
        memcpy((void*)(((long)b)+8),(void*)(((long)&t)+8),sizeof(TListItem)-8);
        }
        end;
        b := b^.Next;
      end;  // while ( c != b );
      a := a^.Next;
    end; // while ( c != a );
    ListOrderByData := 1;
  end;
end;
////////////////////////////////////////////////////////////////////////////////
function ListOrderByItem ( o: PList; comparator: TCompareLI ): Byte;
var
  c, a, b: PListItem;
  t: TListItem;
  aKey : AnsiString;
  aData: Pointer;
  aType: l_ulong;
  aSize: l_ulong;
begin
        
  if (not Assigned(o^.Last)) and (not Assigned(comparator)) then ListOrderByItem := 0 else
  begin
        c := o^.Last^.Next;
        a := o^.Last^.Next;
        while a <> nil do 
        begin
          b := a;
          while b <> NIL do 
          begin
            if ( comparator( a, b ) > 0 ) then
            begin
               
              aKey        := a^.Key;
              aData       := a^.Data;
              aType       := a^.DataType;
              aSize       := a^.DataSize;
              a^.Key      := b^.Key;
              a^.Data     := b^.Data;
              a^.DataType := b^.DataType;
              a^.DataSize := b^.DataSize;
              b^.Key      := aKey;
              b^.Data     := aData;
              b^.DataType := aType;
              b^.DataSize := aSize;

              {
              memcpy((void * )(((long)&t)+8),(void * )(((long)a)+8),sizeof(TListItem)-8);
              memcpy((void * )(((long)a)+8),(void * )(((long)b)+8),sizeof(TListItem)-8);
              memcpy((void * )(((long)b)+8),(void * )(((long)&t)+8),sizeof(TListItem)-8);
              } 
            end;
            b := b^.Next;
          end;  //while ( c != b );
          a := a^.Next;
        end;    // while ( c != a );
   ListOrderByItem := 1;
  end;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Creates a PList
//        @note Never forget to free the returned list after use.
//        @see FreeList
//*/
function NewList() : PList;
var o: PList;
begin
  New( o );
  if not Assigned( o ) then NewList := NIL else
  begin
    fillchar(o^,sizeof(TList),0);
    NewList := o;
  end;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Free a list and items data (using the FreeData procedure if specified, else doesn't free data)
//        @param o List to free
//*/
procedure FreeList ( o: PList );
var a, b,n: PListItem;
begin
  if Assigned( o^.Last ) then
  begin
    a := o^.Last;
    b := o^.Last;
    while a <> NIL do //repeat
    begin
      n := a^.Prev;
      //if ( a^.Key ) then free(a^.Key);
      if Assigned( a^.Data) and Assigned(a^.FreeData) then a^.FreeData(a^.Data);
      Dispose(a);
      a := n;
    end;  //while ( a != b ); //until a = b;
  end;
  Dispose(o);
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Free a List but do not free items data.
//        @note Similar to FreeSList((PSList)o,NULL)
//*/
procedure KickList ( o: PList );
var a, b, n: PListItem;
begin
  if Assigned( o^.Last ) then
  begin
    a := o^.Last;
    b := o^.Last;
    while a <> nil do  //repeat
    begin 
      n := a^.Prev;
      //if ( a^.Key ) then free(^.Key);
      Dispose(a);
      a := n;
    end;   // while ( a != b ); //until a = b;
  end;
  Dispose(o);
end;
////////////////////////////////////////////////////////////////////////////////
{$ENDIF}

end.
