(*
/////////////////////////////////////////////////////////////////////////////////
//
//	Listbox Core File 
//
//	Aura Milestone 3 
//
//	(c) Copyright 2012 Finn Technologies and Chase Finn. All rights reserved.
//
/////////////////////////////////////////////////////////////////////////////////
*)
unit listbox;
{
 .#include 'kernel.h'
 .#include 'widget.h'
 .#include 'listbox.h'
}
(*
 TODO
	+ Memory leaks
	+ Document
  + Messages
*)
interface

uses
  guitypes,dynld,debug,list,text,mousedrv,events,metrics,grfx,widget,skin,font,
  allegro;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$link widget.o}
   {$link listbox.o}
{$endif}

var   ListboxClass: PWidgetClass;

const SkinListbox : PSkin = NIL;

type
  PListboxItem = ^TListboxItem;
  TListboxItem = record

	Caption : l_text;
	Icon    : p_bitmap;

	State   : l_bool;

	Offset  : longint;
  end;

type
  PListboxRow = ^TListboxRow;
  AddLBoxRowFunc = function(Row: PListBoxRow; Offset: Integer; Caption: l_text): PListboxItem;
  TListboxRow = record

	ItemList : PList;

	State    : l_bool;
	BufAbs   : TRect;

	Data     : l_ptr;
	Icon     : p_bitmap;

	AddItem  : AddLBoxRowFunc;
  end;

type
  PListbox = ^TListbox;
  AddLbxRowFunc = function(List: PListBox; Data: l_ptr): PListboxRow;
  AddLbxItemFunc = function(List: PListBox; Caption: l_text; Data: Pointer; Icon: p_bitmap): PListboxItem;
  AddLbxSepFunc = function(List: PListBox): PListboxItem;
  TListbox = record

	o: TWidget;

	RowList     : PList;

	SelectedRow : PListboxRow;

	AddRow      : AddLbxRowFunc;
	AddItem     : AddLbxItemFunc;
	AddSeperator: AddLbxSepFunc;

	OnSelect    : l_ulong;
	OnValidate  : l_ulong;
  end;

function CreateListbox ( App: PApplication; r: TRect ): PListbox;
function LBAddRow ( List: PListbox; Data: Pointer ): PListboxRow;
function LBAddSeperator ( List: PListbox ): PListboxItem;

implementation
{$ifndef LINK_CLIBS}

var
	AppVersion: l_ulong;

const
	nUID: l_text = 'listbox';
	AppName: l_text = 'Listbox Widget';

(**
*	NAME: LBAddItem
*	DESCRIPTION: Adds an item to a listbox (doesnt depend on row)
*	RETURN: Pointer to the added item
*)
function LBAddItem ( List: PListbox; Caption: l_text; Data: Pointer; Icon: p_bitmap ): PListboxItem;
var
  Row: PListboxRow;
begin
  Row := NIL;

  Row := List^.AddRow(List, Data);
  Row^.Icon := Icon;

  LBAddItem := Row^.AddItem(Row, 0, Caption);
end;

(**
*	NAME: RAddItem
*	DESCRIPTION:
*	RETURN:
*)
function  RAddItem ( Row: PListboxRow; Offset: Integer; Caption: l_text ): PListboxItem;
var
  o: PListBoxItem;
begin
	NEW(o);
	fillchar(o^,Sizeof(TListBoxItem),0);

	o^.Offset	:= Offset;
	o^.Caption	:= Caption;

	ListAdd(Row^.ItemList, '', o, NIL);

	RAddItem := o;
end;

(**
*	NAME: LBAddRow
*	DESCRIPTION:
*	RETURN:
*)
function LBAddRow ( List: PListbox; Data: Pointer ): PListboxRow;
var
  o: PListBoxRow;
begin
	NEW(o);
	fillchar(o^,sizeof(TListboxRow),0);

	o^.ItemList := NewList;
	{o^.AddItem  := {$ifdef fpc}{$endif}RAddItem;}
	o^.Data	    := Data;

	ListAdd(List^.RowList, '', o, NIL);

	LBAddRow := o;
end;

(**
*	NAME: LBAddRow
*	DESCRIPTION:
*	RETURN:
*)
function LBAddSeperator ( List: PListbox ): PListboxItem;
begin
   LBAddSeperator := List^.AddItem(List, '\001\002\003', NIL, NIL);
end;

procedure ListboxUpdateBufferAbs ( o: PListbox );
var
  a: PListItem;
  y: Integer;
  yy: Integer;
begin
	y  := 0;
	yy := 0;

	a := PLISTBOX(o)^.RowList^.Last^.Next;

	if not assigned(a) then exit;

        repeat
		if assigned ( (PListboxRow((a^.Data)))^.Icon ) then
			yy := max((PListboxRow((a^.Data)))^.Icon^.w, al_text_height(default_font)+2)
		else
			yy := al_text_height(default_font) + 2;

		RectAssign((PListboxRow((a^.Data)))^.BufAbs, PWIDGET(o)^.Absolute.a.x + 5, PWIDGET(o)^.Absolute.a.y+5+y, PWIDGET(o)^.Absolute.b.x - 5, PWIDGET(o)^.Absolute.a.y + 5 + y + yy - 1);

		y := y + yy;

		a := a^.Next;

	until (a = PLISTBOX(o)^.RowList^.Last^.Next);
end;

(**
*	NAME: LBDraw
*	DESCRIPTION: Default draw function
*)
procedure LBDraw ( o: PWidget; buffer: p_bitmap; w: PRect );
var
  a,b: PListItem;
  y: Integer;
  yy,xx: Integer;
begin
	if not assigned(w) then w := @o^.Absolute;

	if ( UseSkins ) then
		DrawSkin(buffer, SkinListbox, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y)
	else begin
		al_rectfill(buffer,w^.a.x,w^.a.y,w^.b.x,w^.b.y,COL_BOXFACE);
		Rect3D(buffer, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y,COL_BOXLIGHT,COL_BOXDARK);
	end;
	
	y := 0;
	yy := 0;

	if not assigned(PLISTBOX(o)^.RowList) then exit;

	a := PLISTBOX(o)^.RowList^.Last^.Next;

	if not assigned(a) then exit;

	ListboxUpdateBufferAbs(PLISTBOX(o));

	repeat

		b := (PListboxRow((a^.Data)))^.ItemList^.Last^.Next;
		if not assigned(b) then exit;

		if ( TextCompare((PListboxItem((b^.Data)))^.Caption, '\001\002\003') = 0 ) then
		begin
			al_hline(buffer, o^.Absolute.a.x+5, ((PListboxRow((a^.Data)))^.BufAbs.a.y+(PListboxRow((a^.Data)))^.BufAbs.b.y) div 2, o^.Absolute.b.x-5, makecol(0,0,0));
			yy := al_text_height(default_font) + 2;
		end
		else
		begin
			if assigned( (PListboxRow((a^.Data)))^.Icon ) then
				yy := max((PListboxRow((a^.Data)))^.Icon^.w, al_text_height(default_font)+2)
			else
				yy := al_text_height(default_font) + 2;

			xx := 0;

			if ((PListboxRow((a^.Data)))^.State) then
				al_rectfill(buffer, (PListboxRow(a^.Data))^.BufAbs.a.x, (PListboxRow(a^.Data))^.BufAbs.a.y, (PListboxRow(a^.Data))^.BufAbs.b.x, (PListboxRow(a^.Data))^.BufAbs.b.y, makecol(200,200,200));

			if assigned((PListboxRow((a^.Data)))^.Icon) then
			begin
				al_draw_sprite(buffer, (PListboxRow(a^.Data))^.Icon, o^.Absolute.a.x+5, o^.Absolute.a.y + 5 + y);
				xx := (PListboxRow(a^.Data))^.Icon^.w + 5;
			end;

			(**
			 * Draw all cells in this row
			 *)

			repeat
				if (b^.Next <> PListboxRow(a^.Data)^.ItemList^.Last^.Next) and
                                   ( PListboxItem(b^.Data)^.Offset + al_text_length(default_font, PListboxItem(b^.Data)^.Caption) >= PListboxItem(b^.Next^.Data)^.Offset) then
					DrawNiceText(
                                           buffer,
                                           default_font,
                                           o^.Absolute.a.x+5+xx+PListboxItem(b^.Data)^.Offset,
                                           o^.Absolute.a.y + 5 + y,
                                           PListboxItem(b^.Data)^.Offset-5,
                                           PListboxItem(b^.Data)^.Caption,
                                           makecol(0,0,0))
				else    {
					al_textout_ex(
                                           buffer,
                                           default_font,
                                           PListboxItem(b^.Data))^.Caption,
                                           o^.Absolute.a.x+5+xx+{PListboxItem(b^.Data)^.Offset},
                                           o^.Absolute.a.y + 5 + y,
                                           makecol(0,0,0),
                                           -1
                                        )};

				b := b^.Next;

			until (b = (PListboxRow(a^.Data))^.ItemList^.Last^.Next);
		end;

		y := y + yy;

		a := a^.Next;

	until (a = PLISTBOX(o)^.RowList^.Last^.Next);
end;

(**
*	NAME: LBEventHandler
*	DESCRIPTION: Default event handler
*)
function LBEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
var
  a: PListItem;
begin
	if (Ev^.aType = EV_MOUSE) then
	begin
		if (Ev^.Message = WEvMouseLDown) or
                   (Ev^.Message = WEvMouseMove) and
                   ((Mouse^.State.b and BUTTON_LEFT)=BUTTON_LEFT) then
		begin
			a := PLISTBOX(o)^.RowList^.Last^.Next;
			if not assigned(a) then LBEventHandler := false;

			repeat

				if CURSOR_IN_RECT( PListboxRow(a^.Data)^.BufAbs ) then
				begin
					if PListboxRow(a^.Data) = PLISTBOX(o)^.SelectedRow then
					begin
						if assigned(PLISTBOX(o)^.SelectedRow) then
						begin
							PLISTBOX(o)^.SelectedRow^.State := false;
							WidgetDraw(o, @PLISTBOX(o)^.SelectedRow^.BufAbs);
						end;

						if (
                                                  TextCompare(
                                                    PListboxItem(PListboxRow(a^.Data)^.ItemList^.Last^.Data)^.Caption,
                                                    '\001\002\003'
                                                  ) = 0) then
						begin
							PLISTBOX(o)^.SelectedRow := PListboxRow(a^.Data);
							PLISTBOX(o)^.SelectedRow^.State := true;
							WidgetDraw(o, @PLISTBOX(o)^.SelectedRow^.BufAbs);
						end
						else PLISTBOX(o)^.SelectedRow := NIL;
					end;

					LBEventHandler := true;
				end;
				a := a^.Next;

			until (a = PLISTBOX(o)^.RowList^.Last^.Next);

			if assigned(PLISTBOX(o)^.SelectedRow) then
			begin
				PLISTBOX(o)^.SelectedRow^.State := false;
				WidgetDraw(o, @PLISTBOX(o)^.SelectedRow^.BufAbs);
				PLISTBOX(o)^.SelectedRow := NIL;
			end;
		end;

		if ( Ev^.Message = WEvMouseLUp ) then
		begin
			if (PLISTBOX(o)^.OnSelect > 0) and assigned(PLISTBOX(o)^.SelectedRow) then
				WidgetSendSEvent(o, EV_MESSAGE, PLISTBOX(o)^.OnSelect, PLISTBOX(o)^.SelectedRow^.Data);
		end;

		if ( Ev^.Message = WEvMouseLDClk ) then
		begin
			if (PLISTBOX(o)^.OnValidate > 0) and assigned(PLISTBOX(o)^.SelectedRow) then
				WidgetSendSEvent(o, EV_MESSAGE, PLISTBOX(o)^.OnValidate, PLISTBOX(o)^.SelectedRow^.Data);
		end;
	end;

	LBEventHandler := false;
end;

procedure ListboxFreeEx ( o: PWidget );
begin
   DebugMessage('PLISTBOX::Leaking memory :P',[]);
end;

(**
*	NAME: CreateListbox
*	DESCRIPTION: Creates a new listbox widget
*	RETURN: Pointer to the new listbox
*)
function CreateListbox ( App: PApplication; r: TRect ): PListbox;
var
  p: PListBox;
begin
	New(p);

	if not assigned(p) then CreateListBox := NIL else

	fillchar(p^,sizeof(TListbox),0);

	IntialiseWidget(App, ListboxClass, PWIDGET(p), r );

	PWIDGET(p)^.Draw := {$ifdef fpc}@{$endif}LBDraw;
	{PWIDGET(p)^.EventHandler := {$ifdef fpc}@{$endif}LBEventHandler;}
	{PWIDGET(p)^.FreeEx := {$ifdef fpc}@{$endif}ListboxFreeEx;}

	p^.AddItem		:= {$ifdef fpc}@{$endif}LBAddItem;
	p^.AddRow		:= {$ifdef fpc}@{$endif}LBAddRow;
	p^.AddSeperator	:= {$ifdef fpc}@{$endif}LBAddSeperator;

	p^.RowList := NewList;

	CreateListBox := p;
end;

function LibMain ( argc: integer; argv: l_text): l_bool;
var t: TRect; l: PListBox; r: PListboxRow;
begin
	
	ListboxClass := CreateWidgetClass( ULONG_ID('L','s','t','B'), StdWidgetClass, 0 );
	{ListboxClass^.StdCreate := {$ifdef fpc}@{$endif}CreateListbox;}
	

	SkinListbox := GetSkinItem('Listbox');

{	APPEXPORT(CreateListbox); }

	(* t.a.x := 20; t.a.y := 50; t.b.x := 440; t.b.y := 280;

	l := NIL;
	r := NIL;

	l := CreateListbox(@Me, t);

	r := l^.AddRow(l);
	r^.AddItem(r, 0, 'ASM', NIL); r^.AddItem(r, 50, 'hardcore', NIL);
	r := l^.AddRow(l);
	r^.AddItem(r, 0, 'C', NIL); r^.AddItem(r, 50, 'medium', NIL);
	r := l^.AddRow(l);
	r^.AddItem(r, 0, 'C++', NIL); r^.AddItem(r, 50, 'medium/hard', NIL);
	r := l^.AddRow(l);
	r^.AddItem(r, 0, 'C#', NIL); r^.AddItem(r, 50, 'piece of cake', NIL);
	l^.AddItem(l, 'VB .NET', NIL, NIL);
	l^.AddItem(l, 'ASP', NIL, NIL);
	l^.AddItem(l, 'HTML', NIL, NIL);
	r := l^.AddRow(l);
	r^.AddItem(r, 0, 'XML', NIL); r^.AddItem(r, 50, 'oZone GUI will be the best', NIL); r^.AddItem(r, 100, 'Screw me', NIL);
	l^.AddItem(l, 'Other ...', NIL, NIL);

	InsertWidget(DeskTop, PWIDGET(l));
	WidgetDraw(WIDGET(l), NIL);*)

	LibMain := true;
end;

procedure Close;
begin

end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion := ULONG_ID(0,0,0,1);
  
{$endif}   {LINK_CLIBS}
end.
