////////////////////////////////////////////////////////////////////////////////
//
//	Listview - Core file
//
//	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
unit listview;
(*
 #include "kernel.h"
 #include "widget.h"
 #include "listview.h"
 #include "textbox.h"
 #include "scrollbar.h"
 #include "menu.h"
*)

interface

uses
  guitypes,dynld,events,keyboard,mousedrv,list,metrics,font,skin,widget,
  scrllbar,grfx,dms,text,textbox,allegro;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$link kernel.o}
   {$link widget.o}
   {$link listview.o}
   {$link textbox.o}
   {$link scrollbar.o}
   {$link menu.o}
{$endif}

(*
//#define LISTVIEW(w) ((PListview)(w))
//#define LISTVIEWITEM(w) ((PListviewItem)(w))
//#define LISTVIEWCOL(w) ((PListviewCol)(w))
*)
type 
  PListviewItem = ^TListviewItem;
  TListviewItem = record

	Caption: l_text;
	
	ColumsData: PList;
	
	Icon  : PIcon;
	
	BufAbs: TRect;
	BufTop: TRect;
	
	RLabel: TRect;
	RIcon : TRect;
	RChk  : TRect;
	
	Flags : l_ulong;
	
  end;

  TComparatorFunc = function ( a, b: PListviewItem ): l_int;
  PListviewCol = ^TListviewCol;
  TListviewCol = record
	
	Caption: l_text;
	Icon   : PIcon;
	Width  : l_int;
	Flags  : l_ulong;
	
	Comparator: TComparatorFunc;
	
  end;

  TItemStartDragProc  = procedure (o: PWidget);
  TItemDropAcceptFunc = function (o,Caller: PWidget; i:PListviewItem; t:l_ulong; d:Pointer; sm,m:l_ulong): l_bool;
  TItemDragBeforeDropFunc = function (o,Dest:PWidget;i:PListViewItem;d:Pointer; t,m: l_ulong): l_bool;
  TItemDropDataProc = procedure (o,Caller: PWidget; i:PListviewItem; d:Pointer; t,m:l_ulong);
  TItemDragAfterDropProc = procedure (o,Dest: PWidget; i:PListviewItem; d:Pointer; t,m:l_ulong);

  PListview = ^TListview;
(**
	Listview structure
*)
  TListview = record

	o: TScrollWidget;

	Items: PList;

	Colums: PList;
	
	TopItem: PListviewItem;
	LPointed: PListviewItem;

	OnSelMsg: l_ulong;  { ///< If set, message sent when an item is selected. Item is set into @a Extra field of event.}
	OnValMsg: l_ulong;  { ///< If set, message sent when a double click (or assimilated) event happen on an item. Item is set into @a Extra field of event.}
	OnRClkMsg: l_ulong; { ///< If set, message sent when a right click event happen on an item. Item is set into @a Extra field of event.}
	
	(**
		Listview flags.
		@see listviewflags
	*)
	Flags: l_ulong;
	
	Style: l_uchar;
	
	Background: l_uchar;

	TransBG: l_uchar;

	Noscroll: l_uchar;
	
	SelColum: PListviewCol; { // Internal use only }
	
	DragItem: PListviewItem;
	
	ItemStartDrag: TItemStartDragProc;
	ItemDropAccept: TItemDropAcceptFunc;
	ItemDragBeforeDrop: TItemDragBeforeDropFunc;
	ItemDropData: TItemDropDataProc;
	ItemDragAfterDrop: TItemDragAfterDropProc;
  end;

const
 LVI_SELECTED   = $01;
 LVI_CHKDISABLE = $04;
 LVI_CHECKED    = $08;

{ Styles }
 LVS_ICONS      = $00;
 LVS_LIST       = $01;
 LVS_MOSC       = $02;

(** @defgroup listviewflags Listview flags
	Values that can be set into TListview Flags field.
	Use 'OR' to associate those flags.
	@{
*)

{ // Flags }
 LVF_MULTISELECT = $0001; { ///< Allow multiple items selection }
 LVF_ALLOWDRAG	 = $0040; { ///< Allow items to be dragged throw global system, incompatible with LVF_ORDER }

{ // Flags : LVS_LIST mode }
 LVF_ORDER 	   = $0002; { ///< Allow user to change items order by dragging them, incompatible with LVF_ALLOWDRAG }
 LVF_CHECKBOXES = $0004; { ///< Show checkbowes before items caption                                               }
 LVF_DRAWICONS  = $0008; { ///< Show icons                                                                         }
 LVF_COLUMS	   = $0010; { ///< Show colums                                                                        }
 LVF_COLRESIZE  = $0020; { ///< Allow colums resize                                                                }
                                                                                                                         

{ // Flags : Internal use only }
 LVF_ENDODRAG   = $1000;
 LVF_POINTDOWN  = $2000;
 LVF_SIZINGCOL  = $4000;
 LVF_HAVESELECT = $8000; 


function CreateListview( App: PApplication; r:TRect; CaptionColumName:l_text; Style:l_uchar; Flags:l_ulong ): PListview;
function InitListview( p:PListview; aClass:PWidgetClass; App:PApplication; r:TRect; CaptionColumName:l_text; Style:l_uchar; Flags:l_ulong): PListview;

function ListviewAddItem ( o: PListview; Caption: l_text; Icon: PIcon ): PListviewItem;
function ListviewInitItem ( p: PListviewItem; Caption: l_text; Icon: PIcon ): PListviewItem;
procedure ListviewItemPushCol ( p: PListviewItem; Text: l_text );
procedure FreeListviewItem ( p: Pointer );


procedure ListviewRemoveAllItems ( o: PListview );

function ListviewEventHandler ( o: PWidget; Ev: PEvent ): l_bool;

function GetSelectedItem ( o: PListview ): PListviewItem;

procedure ListviewSelectItem ( o: PListview; p: PListviewItem; Scrol, Multi: l_bool );


procedure ListviewSelectIndex ( o: PListview; Index: l_ulong; Scrol, Multi: l_bool );
procedure ListviewSelectItemAndTell ( o:PListview; p:PListviewItem; Scrol, Multi: l_bool );

function ListviewItemIndex ( o: PListview; a:PListviewItem ): l_ulong;
function GetSelectedListItem ( o:PListview ): PListItem;
function ListviewRenameItemBox ( o:PListview; a:PListviewItem ): l_text;
procedure ListviewItemRemoveItem ( o:PListview; p:PListviewItem );

function ListviewCountSelectedFiles ( o:PListview ): l_ulong;
function ListviewGetItemUnderPoint ( o:PListview; p:TPoint ): PListviewItem;

function ListviewEstimateHeight ( Items: PList ): l_ulong;

procedure ListviewAddColum ( p: PListview; Caption:l_text; Icon: PIcon; Width:l_int; Flags: l_ulong );
procedure ListviewSetColSize( p: PListview; col:l_int; Width: l_ulong );


procedure ListviewItemSetColCaption ( p:PListviewItem; col: l_int; Caption: l_text );

procedure ListviewUpdateScrollbars ( o:PListview );

var
  ListviewClass: PWidgetClass;

procedure ListviewFree ( o: PWidget );
procedure ListviewUpdateAndReDraw ( o: PListview );
procedure ListviewCalculateMax ( o: PWidget );
function ListviewScrollTo ( o: PListview; i: PListviewItem ): l_bool;

implementation
{$ifndef LINK_CLIBS}
(** @defgroup listview Listview widget
	Display icons lists in differents styles.
	@ingroup widgets
	@{
*)
(**	@file
	Listvisw core file.
*)

////////////////////////////////////////////////////////////////////////////////
var
  AppVersion: l_ulong;
  ListviewFontColor: l_int;
  ListviewFont: AL_FONTptr;

const
  AppName: l_text = 'Listview Widget';
  nUID   : l_text = 'listview';
  NeededLibs: array[0..3] of l_text = ('widget','skin','textbox','');

////////////////////////////////////////////////////////////////////////////////
const
  ITEMWIDTH = 65;
  ITEMHEIGHT = 65;
  BORDEROFFSET = 2;
  BORDERTOPOFFSET = 1;
////////////////////////////////////////////////////////////////////////////////
const
  SkinListview: PSkin = NIL;
  TransBG: l_uchar = 0;

////////////////////////////////////////////////////////////////////////////////
function ListviewDropAccept (o,Caller: PWidget; aType:l_ulong; Data: Pointer; SupportedMethods: l_ulong; var Method: l_ulong ): l_bool;
var old,p: PListViewItem; LFlags: l_ulong;
begin
	if ( PLISTVIEW(o)^.Flags and LVF_ALLOWDRAG ) = LVF_ALLOWDRAG then
        begin

		p := ListviewGetItemUnderPoint(PLISTVIEW(o), Mouse^.State.p);
		LFlags := PLISTVIEW(o)^.Flags;
		
		PLISTVIEW(o)^.TopItem := p;
		if ( p <> PLISTVIEW(o)^.DragItem ) then begin
			old := PLISTVIEW(o)^.DragItem;
			PLISTVIEW(o)^.DragItem := p;
			if assigned( old ) then WidgetDraw(PWIDGET(o), @(old^.BufTop));
			if assigned( PLISTVIEW(o)^.DragItem ) then WidgetDraw(PWIDGET(o), @(PLISTVIEW(o)^.DragItem^.BufTop));
                end;
		
		if (o = Caller) then begin
			if not assigned( p ) then ListviewDropAccept := false;
			if ( p^.Flags and LVI_SELECTED ) = LVI_SELECTED then ListviewDropAccept := false;
                end;
		
		if assigned( PLISTVIEW(o)^.ItemDropAccept ) then
			ListviewDropAccept := PLISTVIEW(o)^.ItemDropAccept(o,Caller,PLISTVIEW(o)^.DragItem,aType,Data,SupportedMethods,Method);
	
	end else
		
	ListviewDropAccept := false;
end;
////////////////////////////////////////////////////////////////////////////////
function ConditionedListViewItem(Condition: l_bool; Item1,Item2: PListViewItem): PListViewItem;
begin
  if Condition
    then ConditionedListViewItem := Item1
  else ConditionedListViewItem := Item2;
end;

function ListviewDragBeforeDrop(o,Dest: PWidget; Data: Pointer; aType, Method: l_ulong ): l_bool;
begin
  if ( o = Dest ) then
	if assigned( PLISTVIEW(o)^.DragItem ) then
		if ( PLISTVIEW(o)^.DragItem^.Flags and LVI_SELECTED ) = LVI_SELECTED then
				ListviewDragBeforeDrop := false;
  if assigned( PLISTVIEW(o)^.ItemDragBeforeDrop )
    then
      ListviewDragBeforeDrop :=
      PLISTVIEW(o)^.ItemDragBeforeDrop(o,Dest,ConditionedListViewItem(o=Dest,PLISTVIEW(o)^.DragItem,NIL),Data,aType,Method);
	
	ListviewDragBeforeDrop := false;
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewDropData(o,Caller: PWidget; Data: Pointer; aType, Method: l_ulong );
var old: PListViewItem;
begin
	old := PLISTVIEW(o)^.DragItem;
	
	PLISTVIEW(o)^.DragItem := NIL;
	PLISTVIEW(o)^.TopItem := NIL;
	
	ListviewSelectItem(PLISTVIEW(o),old,false,false);
	
	if assigned( PLISTVIEW(o)^.ItemDropData ) then
		PLISTVIEW(o)^.ItemDropData(o,Caller,old,Data,aType,Method);
	
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewDragAfterDrop(o,Dest: PWidget; Data: Pointer; aType,Method:l_ulong);
begin
  if assigned(PLISTVIEW(o)^.ItemDragAfterDrop) then
      PLISTVIEW(o)^.ItemDragAfterDrop(o,Dest,ConditionedListViewItem(o=Dest,PLISTVIEW(o)^.DragItem,NIL),Data,aType,Method);
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewDrawItems (o:PWidget; buffer:p_bitmap; w:TRect);
var
  Clip,OldClip: TRect;
  a,b,ca,cb,cia,cib: PListItem;
  i,point: PListViewItem;
  SelColor,TextColor,first: l_int;
  x,y,yy,ii,cx,l: l_int;
begin
	a := PLISTVIEW(o)^.Items^.Last;
	i := NIL;
	{ SelColor := ( o^.Flags and WFFocused ) = WFFocused ? COL_BOXSEL : makecol(215,215,215); }
        if ( o^.Flags and WFFocused ) = WFFocused then SelColor := COL_BOXSEL else SelColor := makecol(215,215,215);
	
	x := o^.Absolute.a.x+BORDEROFFSET;
	y := o^.Absolute.a.y+BORDERTOPOFFSET-SLLWDG(o)^.Pos.y;
	yy:= o^.Absolute.a.y+2;
        l := 0;

  	if assigned( a ) then begin
		a := a^.Next;
		b := a;
	end;

	GetClip(buffer,OldClip);
	
	if ( PLISTVIEW(o)^.Style = LVS_ICONS ) then
	begin	
		if not assigned( a ) then exit;
		
		RectAssign(Clip,o^.Absolute.a.x+1,o^.Absolute.a.y+1,o^.Absolute.b.x-1,o^.Absolute.b.y-1);
		RectIntersept(Clip,Clip,w);
		al_set_clip_rect(buffer,o^.Absolute.a.x+1,o^.Absolute.a.y+1,o^.Absolute.b.x-1,o^.Absolute.b.y-1);
		
		yy := yy + ITEMHEIGHT;

		repeat
			i := PListviewItem(a^.Data);

			l := al_text_length(ListviewFont, i^.Caption)+4;

			RectAssign ( i^.BufAbs, x, y, x + ITEMWIDTH,	y + ITEMHEIGHT);
			
			RectAssign ( i^.RIcon, i^.BufAbs.a.x+(ITEMWIDTH div 2)-16,i^.BufAbs.a.y+(ITEMHEIGHT div 2)-16, i^.BufAbs.a.x+(ITEMWIDTH div 2)+16,i^.BufAbs.a.y+(ITEMHEIGHT div 2)+16);
			
			if ( l+4 > ITEMWIDTH ) then begin
				if ( x+(ITEMWIDTH div 2)-(l div 2)-2 < o^.Absolute.a.x+BORDEROFFSET ) then
					RectAssign ( i^.BufTop,o^.Absolute.a.x+BORDEROFFSET,y,o^.Absolute.a.x+BORDEROFFSET+l+4,y+ITEMHEIGHT)
				else if ( x+(ITEMWIDTH div 2)+(l div 2)+2 > o^.Absolute.b.x-BORDEROFFSET ) then
					RectAssign ( i^.BufTop,o^.Absolute.b.x-BORDEROFFSET-l-4,y,o^.Absolute.b.x-BORDEROFFSET,y+ITEMHEIGHT)
				else
					RectAssign ( i^.BufTop, x+(ITEMWIDTH div 2)-(l div 2)-3, y, x+(ITEMWIDTH div 2)+(l div 2)+3,y + ITEMHEIGHT);
			end else
				RectAssign ( i^.BufTop,x,y,x+ITEMWIDTH,y+ITEMHEIGHT);
			
			
			l := min(l,ITEMWIDTH);	
			
			RectAssign(i^.RLabel,	i^.BufAbs.a.x+(ITEMWIDTH div 2)-(l div 2)-2,
									i^.BufAbs.a.y+(ITEMHEIGHT div 2)+(16)+2,
 									i^.BufAbs.a.x+(ITEMWIDTH div 2)+(l div 2)+2,
									i^.BufAbs.a.y+(ITEMHEIGHT div 2)+(16)+al_text_height(ListviewFont)+2);
													 
			if (RectOverlay(i^.BufAbs,w) and (i <> PLISTVIEW(o)^.TopItem)) then
			begin
				if (((i^.Flags and LVI_SELECTED)=LVI_SELECTED) or (i = PLISTVIEW(o)^.DragItem)) then
				begin
					al_rectfill(buffer,i^.RLabel.a.x,i^.RLabel.a.y,i^.RLabel.b.x,i^.RLabel.b.y,SelColor);
					TextColor := makecol(255,255,255);
				end else
					TextColor := ListviewFontColor;
				
				DrawIcon32(buffer,i^.Icon,i^.RIcon.a.x,i^.RIcon.a.y);
					
				DrawNiceTextCenter(buffer, ListviewFont,
					x+(ITEMWIDTH div 2), y+(ITEMHEIGHT div 2+16)+2,
					ITEMWIDTH, i^.Caption, TextColor);
			end;

			x := x + ITEMWIDTH;
			if ( x > o^.Absolute.b.x-BORDEROFFSET-ITEMWIDTH ) then begin
				x := o^.Absolute.a.x+BORDEROFFSET;
				y := y + ITEMHEIGHT;
				yy := yy + ITEMHEIGHT;
			end;

			a := a^.Next;

		until ( a = b );

		i := PLISTVIEW(o)^.TopItem;
		
		if assigned( i ) then
		if (RectOverlay(i^.BufAbs,w)) then
		begin
			l := al_text_length(ListviewFont, i^.Caption);
			
			if (i^.BufAbs.a.x <> i^.BufTop.a.x) or (i^.BufAbs.b.x <> i^.BufTop.b.x) then
			begin	
				if (((i^.Flags and LVI_SELECTED)=LVI_SELECTED) or (i = PLISTVIEW(o)^.DragItem)) then
				begin
					al_rectfill(buffer,
					i^.BufTop.a.x-2,
					i^.BufTop.a.y+(ITEMHEIGHT div 2)+(16)+2,
	 				i^.BufTop.b.x+2,
					i^.BufTop.a.y+(ITEMHEIGHT div 2)+(16)+al_text_height(ListviewFont)+2,
					SelColor);
					TextColor := makecol(255,255,255);
				end else
					TextColor := ListviewFontColor;
				
				DrawIcon32(buffer,i^.Icon,i^.RIcon.a.x,i^.RIcon.a.y);
				
				al_textout_centre_ex(buffer, ListviewFont, i^.Caption,
				(i^.BufTop.a.x+i^.BufTop.b.x) div 2,i^.BufAbs.a.y+(ITEMHEIGHT div 2+16)+2, TextColor, -1);

				
			end else begin
				if (((i^.Flags and LVI_SELECTED)=LVI_SELECTED) or (i = PLISTVIEW(o)^.DragItem)) then
				begin
					al_rectfill(buffer,i^.RLabel.a.x,i^.RLabel.a.y,i^.RLabel.b.x,i^.RLabel.b.y,SelColor);
					TextColor := makecol(255,255,255);
				end else
					TextColor := ListviewFontColor;
				
				DrawIcon32(buffer,i^.Icon,i^.RIcon.a.x,i^.RIcon.a.y);
				
				al_textout_centre_ex(buffer, ListviewFont, i^.Caption,
				i^.BufAbs.a.x+(ITEMWIDTH div 2),i^.BufAbs.a.y+(ITEMHEIGHT div 2+16)+2, TextColor, -1);
			end;
			
		end;


	end else if ( PLISTVIEW(o)^.Style = LVS_LIST ) then begin
		

		point := NIL;
		ca := NIL; cb := NIL; cia := NIL; cib := NIL;
		
		if ( PLISTVIEW(o)^.Flags and LVF_ENDODRAG ) = LVF_ENDODRAG then
			point := ListviewGetItemUnderPoint(PLISTVIEW(o), Mouse^.State.p);

		if ( ((PLISTVIEW(o)^.Flags and LVF_COLUMS)=LVF_COLUMS) and assigned(PLISTVIEW(o)^.Colums^.Last) ) then
		begin	
			cx := o^.Absolute.a.x+1-SLLWDG(o)^.Pos.x; first := 1;
			
			ca := PLISTVIEW(o)^.Colums^.Last^.Next;
                        cb := PLISTVIEW(o)^.Colums^.Last^.Next;
			
			if ( PLISTVIEW(o)^.Flags and LVF_DRAWICONS ) = LVF_DRAWICONS then begin
				cx := cx + 18;
				first := 0;
			end;
			
			al_rectfill(buffer,o^.Absolute.a.x+1,o^.Absolute.a.y+1,o^.Absolute.b.x-1,o^.Absolute.a.y+18,COL_BOXLIGHT);
			
			repeat
				if ( first > 0 ) then
					first := 0
				else begin
					al_vline(buffer,cx,max(w.a.y,o^.Absolute.a.y+1),min(w.b.y,o^.Absolute.b.y-1),COL_BOXLIGHT);
					al_vline(buffer,cx,o^.Absolute.a.y+1,o^.Absolute.a.y+18,COL_BOXFACE);
				end;
				DrawNiceText ( buffer, ListviewFont, cx+2, o^.Absolute.a.y+1, PLISTVIEWCOL(ca^.Data)^.Width-3, PLISTVIEWCOL(ca^.Data)^.Caption, COL_BOXTEXT );
				cx := cx + PLISTVIEWCOL(ca^.Data)^.Width+1;
				al_vline(buffer,cx,o^.Absolute.a.y+1,o^.Absolute.a.y+18,COL_BOXFACE);
				ca := ca^.Next;
			until ( ca = cb );
						
			y := y + 18;
			RectAssign(Clip,o^.Absolute.a.x+1,o^.Absolute.a.y+19,o^.Absolute.b.x-1,o^.Absolute.b.y-1);
			RectIntersept(Clip,Clip,w);
			al_set_clip_rect(buffer,o^.Absolute.a.x+1,o^.Absolute.a.y+19,o^.Absolute.b.x-1,o^.Absolute.b.y-1);
		end else begin
			RectAssign(Clip,o^.Absolute.a.x+1,o^.Absolute.a.y+1,o^.Absolute.b.x-1,o^.Absolute.b.y-1);
			RectIntersept(Clip,Clip,w);
			al_set_clip_rect(buffer,o^.Absolute.a.x+1,o^.Absolute.a.y+1,o^.Absolute.b.x-1,o^.Absolute.b.y-1);
		end;
		
		if assigned( a ) then
		repeat
			i := PListviewItem(a^.Data);
			
			RectAssign ( i^.BufAbs, o^.Absolute.a.x+1, y, max(o^.Absolute.b.x-1,o^.Absolute.a.x+22),	y + 17);
			
			i^.BufTop := i^.BufAbs;
			
			
			cx := o^.Absolute.a.x+1-SLLWDG(o)^.Pos.x;
			
			if ( PLISTVIEW(o)^.Flags and LVF_DRAWICONS ) = LVF_DRAWICONS then begin
				RectAssign ( i^.RIcon,cx+1,y+1,cx+19,y+19);
				cx := cx + 18;
			end;
			if ( PLISTVIEW(o)^.Flags and LVF_CHECKBOXES ) = LVF_CHECKBOXES then begin
				RectAssign ( i^.RChk,cx+2,y+4,cx+12,y+14);
				cx := cx + 14;
			end;
			
			if ( PLISTVIEW(o)^.Flags and LVF_COLUMS ) = LVF_COLUMS then begin
				RectAssign (
                                  i^.RLabel,
                                  cx+3,
                                  y+((16-al_text_height(ListviewFont)) div 2)+1,
                                  cx+3+min(al_text_length(ListviewFont,i^.Caption),
                                  PLISTVIEWCOL(ca^.Data)^.Width-3),
                                  y+((16+al_text_height(ListviewFont)) div 2)+1
                                );
			end else begin
				RectAssign (
                                  i^.RLabel, cx+1,
                                  y+((16-al_text_height(ListviewFont)) div 2)+1,cx+1+al_text_length(ListviewFont,i^.Caption),
                                  y+((16+al_text_height(ListviewFont)) div 2)+1
                                );
			end;
			
			if (RectOverlay(i^.BufAbs,w)) then
			begin
				first := 1;
				if (((i^.Flags and LVI_SELECTED)=LVI_SELECTED) or (i = PLISTVIEW(o)^.DragItem)) then
				begin
					al_rectfill(buffer,
					i^.RLabel.a.x,
					i^.RLabel.a.y,
					i^.RLabel.b.x,
					i^.RLabel.b.y,
					SelColor);
					TextColor := ListviewFontColor;
				end else
					TextColor := ListviewFontColor;

				if ( i = point ) then begin
					if ( PLISTVIEW(o)^.Flags and LVF_POINTDOWN ) = LVF_POINTDOWN then
						al_rectfill(buffer,	i^.BufAbs.a.x,
						i^.BufAbs.a.y,
						i^.BufAbs.b.x,
						i^.BufAbs.a.y+1,makecol(128,128,255))
					else
						al_rectfill(buffer,	i^.BufAbs.a.x,
						i^.BufAbs.b.y-1,
						i^.BufAbs.b.x,
						i^.BufAbs.b.y,makecol(128,128,255));
				end;

				cx := o^.Absolute.a.x+1-SLLWDG(o)^.Pos.x;
				
				if ( PLISTVIEW(o)^.Flags and LVF_DRAWICONS )=LVF_DRAWICONS then begin
					DrawIcon16(buffer,i^.Icon,cx+1,y+1);
					cx := cx + 18;
					first := 0;	
				end;
				
				if ( ((PLISTVIEW(o)^.Flags and LVF_COLUMS) = LVF_COLUMS) and assigned(i^.ColumsData^.Last) and assigned(ca)) then
                                begin
					cia := i^.ColumsData^.Last^.Next;
                                        cib := i^.ColumsData^.Last^.Next;
					
					repeat
						if ( first > 0 ) then
							first := 0
						else
						  al_vline(buffer,cx,i^.BufAbs.a.y,i^.BufAbs.b.y,COL_BOXLIGHT);
						if ( cia = cib ) then
							if ( PLISTVIEW(o)^.Flags and LVF_CHECKBOXES ) = LVF_CHECKBOXES then begin
								al_rect( buffer,cx+2,y+4,cx+12,y+14,makecol(0,0,0));
								al_rectfill( buffer,cx+3,y+5,cx+11,y+13,makecol(255,255,255));
								if ( i^.Flags and LVI_CHECKED ) = LVI_CHECKED then
                                                                  al_rectfill( buffer,cx+4,y+6,cx+10,y+12,makecol(0,0,0));
								cx :=  cx + 14;
							end;
						if assigned( cia^.Data ) then
                                                  DrawNiceText ( buffer, ListviewFont, cx+3, y+((16-al_text_height(ListviewFont)) div 2)+1, PLISTVIEWCOL(ca^.Data)^.Width-3, l_text(cia^.Data), TextColor );
						cx := cx + PLISTVIEWCOL(ca^.Data)^.Width+1;
						ca := ca^.Next;

					until ( (cia = cib) and (ca = cb) );
					
				end else begin
					if ( PLISTVIEW(o)^.Flags and LVF_CHECKBOXES ) = LVF_CHECKBOXES then begin
						al_rect( buffer,cx+2,y+4,cx+12,y+14,makecol(0,0,0));
						al_rectfill( buffer,cx+3,y+5,cx+11,y+13,makecol(255,255,255));
						if ( i^.Flags and LVI_CHECKED ) = LVI_CHECKED then al_rectfill( buffer,cx+4,y+6,cx+10,y+12,makecol(0,0,0));
						cx := cx + 14;
					end;
					al_textout_ex(buffer, ListviewFont, i^.Caption,cx+1,y+((16-al_text_height(ListviewFont)) div 2)+1, TextColor, -1);
				end;
			end;
			y := y + 18;
			yy := yy + 18;
			a := a^.Next;

		until ( a = b );

	end else if ( PLISTVIEW(o)^.Style = LVS_MOSC ) then begin

                if assigned( a ) then
		repeat
			i := PListviewItem(a^.Data);
			
			RectAssign ( i^.BufAbs, x, y, x+200,	 y+50);
			
			RectAssign ( i^.RIcon, i^.BufAbs.a.x+1, i^.BufAbs.a.y+1, i^.BufAbs.a.x+49, i^.BufAbs.a.y+49);
			RectAssign ( i^.RLabel,
                           i^.BufAbs.a.x+52,
                           (i^.BufAbs.a.y+i^.BufAbs.b.y-al_text_height(ListviewFont)) div 2,
                           i^.BufAbs.a.x+52+al_text_length(ListviewFont,i^.Caption),
                           (i^.BufAbs.a.y+i^.BufAbs.b.y+al_text_height(ListviewFont)) div 2
                        );
			
			i^.BufTop := i^.BufAbs;
			
			if (RectOverlay(i^.BufAbs,w)) then
			begin
				if (((i^.Flags and LVI_SELECTED)=LVI_SELECTED) or (i = PLISTVIEW(o)^.DragItem)) then
				begin
					al_rectfill(buffer,i^.RLabel.a.x,i^.RLabel.a.y,i^.RLabel.b.x,i^.RLabel.b.y,SelColor);
					al_rectfill(buffer,i^.RIcon.a.x,i^.RIcon.a.y,i^.RIcon.b.x,i^.RIcon.b.y,SelColor);
					TextColor := makecol(255,255,255);
				end else
					TextColor := ListviewFontColor;
				
				DrawIcon48(buffer,i^.Icon,i^.RIcon.a.x,i^.RIcon.a.y);
				
				al_textout_ex(buffer, ListviewFont, i^.Caption,i^.RLabel.a.x,i^.RLabel.a.y,TextColor, -1);
				
			end;
			
			x := x + 202;
			if ( x+200 > o^.Absolute.b.x ) then begin
				x  := o^.Absolute.a.x+BORDEROFFSET;
				y := y + 52;
				yy := yy + 52;
			end;

			a := a^.Next;

		until ( a = b );
        end;

	al_set_clip_rect(buffer,o^.Absolute.a.x+1,o^.Absolute.a.y+1,o^.Absolute.b.x-1,o^.Absolute.b.y-1);


	if ( ((PLISTVIEW(o)^.Flags and LVF_COLUMS)=LVF_COLUMS) and (PLISTVIEW(o)^.Style = LVS_LIST) ) then begin

		if ( yy > o^.Absolute.b.y-20 ) then
			ScrollBarSetMaxY(SLLWDG(o),yy-(o^.Absolute.b.y-20))
		else
			ScrollBarSetMaxY(SLLWDG(o),0);
		
	end else if ( yy > o^.Absolute.b.y ) then
		ScrollBarSetMaxY(SLLWDG(o),yy-o^.Absolute.b.y)
	else
		ScrollBarSetMaxY(SLLWDG(o),0);
			

end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewCalculateMax ( o:PWidget );
var
  a,b,ca,cb: PListItem;
  x,yy,width,cw: l_int;
begin
        a := PLISTVIEW(o)^.Items^.Last;
	x := o^.Relative.a.x+BORDEROFFSET;
	yy:= o^.Relative.a.y+2;
	width := 0;
	if assigned( a ) then begin
		a := a^.Next;
		b := a;
	end;
	
	if ( PLISTVIEW(o)^.Style = LVS_ICONS ) then begin
	  if assigned( a ) then begin
		yy := yy + ITEMHEIGHT;
		repeat

		  x := x + ITEMWIDTH;
		  if ( x > o^.Relative.b.x-BORDEROFFSET-ITEMWIDTH ) then begin
			x  := o^.Relative.a.x+BORDEROFFSET;
			yy := yy + ITEMHEIGHT;
		  end;

		  a := a^.Next;

		until ( a = b );
	  end
	end else if ( PLISTVIEW(o)^.Style = LVS_LIST ) then begin
		
		if assigned( a ) then repeat
			yy := yy + 18;
			a := a^.Next;
		until ( a = b );
		
		if ( PLISTVIEW(o)^.Flags and LVF_COLUMS ) = LVF_COLUMS then begin
			cw := 1;
			ca := PLISTVIEW(o)^.Colums^.Last^.Next;
                        cb := PLISTVIEW(o)^.Colums^.Last^.Next;
			if ( PLISTVIEW(o)^.Flags and LVF_DRAWICONS ) = LVF_DRAWICONS
                           then cw := cw + 18;
			repeat
				cw := cw + PLISTVIEWCOL(ca^.Data)^.Width+1;
				ca := ca^.Next;
			until ( ca = cb );
			width := max(width,cw+2);
		end;
	end else if ( PLISTVIEW(o)^.Style = LVS_MOSC ) then begin
		if assigned( a ) then
                repeat
			x := x + 100;
			if ( x+100 > o^.Absolute.b.x ) then begin
				x  := o^.Absolute.a.x+BORDEROFFSET;
				yy := yy + 50;
			end;
			a := a^.Next;

		until ( a = b );
	end;

	

	if ( width > (o^.Relative.b.x-o^.Relative.a.x) ) then
		ScrollBarSetMaxX(SLLWDG(o),width-(o^.Relative.b.x-o^.Relative.a.x))
	else
		ScrollBarSetMaxX(SLLWDG(o),0);
	
	if ( ((PLISTVIEW(o)^.Flags and LVF_COLUMS) = LVF_COLUMS) and (PLISTVIEW(o)^.Style = LVS_LIST) ) then
        begin
		if ( yy > o^.Relative.b.y-20 ) then
			ScrollBarSetMaxY(SLLWDG(o),yy-o^.Relative.b.y+20)
		else
			ScrollBarSetMaxY(SLLWDG(o),0);
		
	end else if ( yy > o^.Relative.b.y ) then
		ScrollBarSetMaxY(SLLWDG(o),yy-o^.Relative.b.y)
	else
		ScrollBarSetMaxY(SLLWDG(o),0);


end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewUpdateScrollbars ( o: PListview );
begin
		ListviewCalculateMax(PWIDGET(o));
		if assigned( SLLWDG(o)^.Horizontal ) then
			WidgetDraw(PWIDGET(SLLWDG(o)^.Horizontal),NIL);
		if assigned( SLLWDG(o)^.Vertical ) then
			WidgetDraw(PWIDGET(SLLWDG(o)^.Vertical),NIL);
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewUpdateAndReDraw ( o: PListview );
begin
	ListviewUpdateScrollbars(o);
	WidgetDraw(PWIDGET(o),NIL);
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewDraw ( o: PWidget; buffer: p_bitmap; w: PRect );
begin
  if ( UseSkins and assigned(SkinListview) ) then
     DrawSkin(buffer, SkinListview, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y)
  else begin
     al_rectfill(buffer,w^.a.x,w^.a.y,w^.b.x,w^.b.y,COL_BOXFACE);
     Rect3D(buffer, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y,COL_BOXLIGHT,COL_BOXDARK);
  end;
  ListviewDrawItems(o, buffer, w^);
end;
////////////////////////////////////////////////////////////////////////////////
function ListviewGetItemUnderPoint ( o: PListview; p: TPoint ): PListviewItem;
var
  a: PListItem;
  i: PListViewItem;
begin
        ListviewGetItemUnderPoint := NIL;

	a := PLISTVIEW(o)^.Items^.Last;

	if ( ((PLISTVIEW(o)^.Flags and LVF_COLUMS)=LVF_COLUMS) and (PLISTVIEW(o)^.Style = LVS_LIST) ) then
		if ( p.x < PWIDGET(o)^.Absolute.a.y+19 ) then ListviewGetItemUnderPoint := NIL;
	
	if assigned( a ) then
	repeat
		if (PointInRect(p, PLISTVIEWITEM(a^.Data)^.BufAbs)) then
		begin
			i := PLISTVIEWITEM(a^.Data);
			if ( PointInRect(p, i^.RLabel) ) then
				ListviewGetItemUnderPoint := i;
			if( (o^.Style = LVS_ICONS) or (o^.Style = LVS_MOSC) ) then
                        begin
				if ( PointInRect(p, i^.RIcon) ) then
					ListviewGetItemUnderPoint := i;
			end else begin
				if ( PLISTVIEW(o)^.Flags and LVF_DRAWICONS ) = LVF_DRAWICONS then
					if ( PointInRect(p, i^.RIcon) ) then
						ListviewGetItemUnderPoint := i;
				if ( PLISTVIEW(o)^.Flags and LVF_CHECKBOXES ) = LVF_CHECKBOXES then
					if ( PointInRect(p, i^.RChk) ) then
						ListviewGetItemUnderPoint := i;
			end;
			ListviewGetItemUnderPoint := NIL;
		end;

		a := a^.Prev;
	until ( a = PLISTVIEW(o)^.Items^.Last );

end;
////////////////////////////////////////////////////////////////////////////////
function ListviewGetTopListItem ( o: PListview ): PListItem;
var
  a,b: PListItem;
begin
  ListviewGetTopListItem := NIL;
	a := o^.Items^.Last; b := a;
	if not assigned( a ) then ListviewGetTopListItem := NIL else
	repeat
          if (PLISTVIEWITEM(a^.Data) = o^.TopItem ) then
            begin ListviewGetTopListItem := a; exit; end;
	  a := a^.Prev;
	until ( a = b );
end;
////////////////////////////////////////////////////////////////////////////////
function ListviewGetTopItem ( o: PListview ): PListviewItem;
begin
  ListviewGetTopItem  := o^.TopItem;
end;
////////////////////////////////////////////////////////////////////////////////
function ListviewEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
var
  a,b,ca,cb: PListItem; sel: l_text;
  s,p,old: PListViewItem;
  cx: l_int;
  r: TRect;
  LFlags: l_ulong;
begin
  if (Ev^.aType = EV_KEYBOARD) then
  begin
    if ( SCANCODE(@Ev^.Keyb, AL_KEY_RIGHT)) or ((PLISTVIEW(o)^.Style = LVS_LIST) and SCANCODE(@Ev^.Keyb,AL_KEY_DOWN)) then
    begin
      a := ListviewGetTopListItem(PLISTVIEW(o));
      if assigned( a ) then begin
	if ( a <> PLISTVIEW(o)^.Items^.Last ) then
          ListviewSelectItemAndTell(PLISTVIEW(o),PLISTVIEWITEM(a^.Next^.Data),true,(Ev^.Keyb.Shift and KB_CTRL)=KB_CTRL);
      end else if assigned(PLISTVIEW(o)^.Items^.Last) then begin
	ListviewSelectItemAndTell(PLISTVIEW(o),PLISTVIEWITEM(PLISTVIEW(o)^.Items^.Last^.Next^.Data),true,(Ev^.Keyb.Shift and KB_CTRL)=KB_CTRL);
      end;
			
    end else if ( SCANCODE(@Ev^.Keyb,AL_KEY_LEFT) or
                ((PLISTVIEW(o)^.Style = LVS_LIST) and SCANCODE(@Ev^.Keyb,AL_KEY_UP) )) then
    begin
			a := ListviewGetTopListItem(PLISTVIEW(o));
			if assigned( a ) then begin
				if ( a <> PLISTVIEW(o)^.Items^.Last^.Next ) then
					ListviewSelectItemAndTell(PLISTVIEW(o),PLISTVIEWITEM(a^.Prev^.Data),true,(Ev^.Keyb.Shift and KB_CTRL)=KB_CTRL);
			end else if assigned(PLISTVIEW(o)^.Items^.Last) then begin
				ListviewSelectItemAndTell(PLISTVIEW(o),PLISTVIEWITEM(PLISTVIEW(o)^.Items^.Last^.Next^.Data),true,(Ev^.Keyb.Shift and KB_CTRL)=KB_CTRL);
			end;
					
					
    end else if ( SCANCODE(@Ev^.Keyb,AL_KEY_HOME)) then begin
			a := ListviewGetTopListItem(PLISTVIEW(o));
			if ( assigned(a) or assigned(PLISTVIEW(o)^.Items^.Last)  ) then
				if ( a <> PLISTVIEW(o)^.Items^.Last^.Next ) then
					ListviewSelectItemAndTell(PLISTVIEW(o),PLISTVIEWITEM(PLISTVIEW(o)^.Items^.Last^.Next^.Data),true,(Ev^.Keyb.Shift and KB_CTRL)=KB_CTRL);
					
    end else if ( SCANCODE(@Ev^.Keyb,AL_KEY_END)) then begin
			a := ListviewGetTopListItem(PLISTVIEW(o));
			if ( assigned(a) or assigned(PLISTVIEW(o)^.Items^.Last) ) then
				if ( a <> PLISTVIEW(o)^.Items^.Last ) then
					ListviewSelectItemAndTell(PLISTVIEW(o),PLISTVIEWITEM(PLISTVIEW(o)^.Items^.Last^.Data),true,(Ev^.Keyb.Shift and KB_CTRL)=KB_CTRL);
			
    end else if ( SCANCODE(@Ev^.Keyb,AL_KEY_ENTER)) then begin
			if ( assigned(PLISTVIEW(o)^.TopItem) and (PLISTVIEW(o)^.OnValMsg<>0) ) then
				WidgetSendSEvent(o, EV_MESSAGE, PLISTVIEW(o)^.OnValMsg, PLISTVIEW(o)^.TopItem);
				
    end else if not l_bool((Ev^.Keyb.Shift and not KB_CTRL))
                  and assigned(PLISTVIEW(o)^.Items^.Last) then begin
			sel := ToLower(Chr(Ev^.Keyb.key));
			a := ListviewGetTopListItem(PLISTVIEW(o)); b := a;
			if not assigned(a) then a := PLISTVIEW(o)^.Items^.Last;
			a := a^.Next;
			b := a;
			repeat
				if ( LowerCase(PLISTVIEWITEM(a^.Data)^.Caption) = sel ) then begin
					ListviewSelectItemAndTell(PLISTVIEW(o),PLISTVIEWITEM(a^.Data),true,(Ev^.Keyb.Shift and KB_CTRL)=KB_CTRL);
					ListViewEventHandler := true;
				end;
				a := a^.Next;
			until ( a = b);
    end; { von Ev^.Keyb.Shift }
  end else if (Ev^.aType = EV_MESSAGE) then begin
 
		if ( Ev^.Message = WEvNotifyLostFocus ) then begin
			WidgetDraw(o, NIL);
			ListviewEventHandler := true;
		end;
		if ( Ev^.Message = WEvNotifyFocused ) then begin
			WidgetDraw(o, NIL);
			ListviewEventHandler := true;
		end;
		if ( Ev^.Message = WEvNotifySameFocus ) then begin
			
			ListviewEventHandler := true;
		end;
		if ( Ev^.Message = MSG_NOTIFY_AUTO_RESIZE ) then begin
			ListviewCalculateMax(o);
			ListviewEventHandler := true;
		end;
	end else if (Ev^.aType = EV_MOUSE) then
	begin	
		if ( (Ev^.Message = MSG_DRAGCANCEL) or ((Ev^.Message = WEvMouseLeft) and ((o^.Flags and WF_DRAGFOCUS)=WF_DRAGFOCUS)) ) then
		begin
			old := PLISTVIEW(o)^.DragItem;
			PLISTVIEW(o)^.DragItem := NIL;
			PLISTVIEW(o)^.TopItem := NIL;
			if assigned( old ) then WidgetDraw(PWIDGET(o), @(old^.BufTop));
			ListviewEventHandler := true;
		end;
	
		if (Ev^.Message = WEvMouseLDClk) then begin
			p := ListviewGetItemUnderPoint(PLISTVIEW(o), Mouse^.State.p);
			
			if ( ((PLISTVIEW(o)^.Flags and LVF_CHECKBOXES)=LVF_CHECKBOXES) and (PLISTVIEW(o)^.Style = LVS_LIST) ) then
				if ( PointInRect(Mouse^.State.p, p^.RChk) ) then
					ListviewEventHandler := true;
			if assigned( p ) then
				if ( PLISTVIEW(o)^.OnValMsg <> 0 ) then
					WidgetSendSEvent(o, EV_MESSAGE, PLISTVIEW(o)^.OnValMsg, p);
			ListviewEventHandler := true;
		end;

		if ( Ev^.Message = WEvMouseLeft ) then begin
			PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags and not (LVF_SIZINGCOL or LVF_ENDODRAG or LVF_HAVESELECT);
			ListviewEventHandler := true;
		end;

		if (Ev^.Message = WEvMouseLDown) or (Ev^.Message = WEvMouseRDown) then
		begin
			if (((PLISTVIEW(o)^.Flags and LVF_COLUMS)=LVF_COLUMS) and (PLISTVIEW(o)^.Style = LVS_LIST) and ( Mouse^.State.p.y < o^.Absolute.a.y+18)) then
                        begin
				cx := o^.Absolute.a.x+1-SLLWDG(o)^.Pos.x;
				if assigned( PLISTVIEW(o)^.Colums^.Last ) then begin
					ca := PLISTVIEW(o)^.Colums^.Last^.Next; cb := ca;
					if ((PLISTVIEW(o)^.Flags and LVF_DRAWICONS)=LVF_DRAWICONS) then cx := cx + 18;
					repeat
						cx := cx + PLISTVIEWCOL(ca^.Data)^.Width+1;
						if ( (Mouse^.State.p.x > cx-2) and (Mouse^.State.p.x < cx+2) ) then begin
							PLISTVIEW(o)^.SelColum := PLISTVIEWCOL(ca^.Data);
							if ( PLISTVIEW(o)^.Flags and LVF_COLRESIZE ) = LVF_COLRESIZE then
								PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags or LVF_SIZINGCOL;
						end;
						ca := ca^.Next;
					until ( ca = cb );
				end;
			end else begin
				{PListviewItem p = 0;}
				p := ListviewGetItemUnderPoint(PLISTVIEW(o), Mouse^.State.p);
				
				if assigned( p ) then begin
					
					if (((PLISTVIEW(o)^.Flags and LVF_CHECKBOXES)=LVF_CHECKBOXES) and
                                             (PLISTVIEW(o)^.Style = LVS_LIST) ) then
						if ( PointInRect(Mouse^.State.p, p^.RChk) ) then
							ListviewEventHandler := true;
					
					if not(( p^.Flags and LVI_SELECTED)=LVI_SELECTED) then begin
						PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags or LVF_HAVESELECT;
						ListviewSelectItem(PLISTVIEW(o),p,false,(Ev^.Keyb.Shift and KB_CTRL)=KB_CTRL);
					end;
					
				end;
			end;
			ListviewEventHandler := true;
		end;
				
		if ( (Ev^.Message = WEvMouseMove) and
                   ((Mouse^.State.b and BUTTON_LEFT)=BUTTON_LEFT) and
                   not((o^.Flags and WF_DRAGFOCUS)=WF_DRAGFOCUS)) then
		begin
			if ((PLISTVIEW(o)^.Flags and LVF_SIZINGCOL)=LVF_SIZINGCOL) then
			begin
				cx := o^.Absolute.a.x+1-SLLWDG(o)^.Pos.x;
				ca := PLISTVIEW(o)^.Colums^.Last;
				if ((PLISTVIEW(o)^.Flags and LVF_DRAWICONS)=LVF_DRAWICONS) then cx := cx + 18;
				repeat
					ca := ca^.Next;
					cx := cx + PLISTVIEWCOL(ca^.Data)^.Width+1;
				until (ca^.Data <> PLISTVIEWCOL(PLISTVIEW(o)^.SelColum));
			
				if ( (PLISTVIEW(o)^.SelColum^.Width+Mouse^.State.p.x-cx) > 20 ) then begin
					r := o^.Absolute;
					{//r.a.x := cx-(PLISTVIEWCOL(ca->Data)^.Width+1);}
					PLISTVIEW(o)^.SelColum^.Width := PLISTVIEW(o)^.SelColum^.Width + Mouse^.State.p.x-cx;
					
					WidgetDraw(o,@r);
					ListviewUpdateScrollbars(PLISTVIEW(o));
				end;
			end else if not(((PLISTVIEW(o)^.Flags and LVF_ORDER)=LVF_ORDER) and
                                         ((PLISTVIEW(o)^.Flags and LVF_ALLOWDRAG)=LVF_ALLOWDRAG)) then begin
				p := ListviewGetItemUnderPoint(PLISTVIEW(o), Mouse^.State.p);
				if assigned( p ) then
					if (p^.Flags and LVI_SELECTED)=LVI_SELECTED then begin
						PLISTVIEW(o)^.DragItem := NIL;
						if assigned(PLISTVIEW(o)^.ItemStartDrag) then
							PLISTVIEW(o)^.ItemStartDrag(o);
						(*else // Testing purposes ONLY
							DragStart(o,0,0,DMETHOD_COPY|DMETHOD_MOVE);*)
					end;
			end else if ((PLISTVIEW(o)^.Flags and LVF_ENDODRAG)=LVF_ENDODRAG) then begin
				p := ListviewGetItemUnderPoint(PLISTVIEW(o), Mouse^.State.p);
				LFlags := PLISTVIEW(o)^.Flags;
				if assigned( p ) then begin
					if ( Mouse^.State.p.y < (p^.BufAbs.a.y+p^.BufAbs.b.y) div 2 ) then
						PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags or LVF_POINTDOWN
					else
						PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags and not LVF_POINTDOWN;
					
				end;
				if ( (PLISTVIEW(o)^.Flags <> LFlags) or (p <> PLISTVIEW(o)^.LPointed) ) then begin
					if ( p = PLISTVIEW(o)^.LPointed ) then begin
						if assigned( p ) then WidgetDraw(PWIDGET(o), @p^.BufAbs)
					end else begin
						if assigned( PLISTVIEW(o)^.LPointed ) then WidgetDraw(PWIDGET(o), @(PLISTVIEW(o)^.LPointed^.BufAbs));
						if assigned( p ) then WidgetDraw(PWIDGET(o), @p^.BufAbs);
						
				 	end;
				end;
				PLISTVIEW(o)^.LPointed := p;
			end else if (((PLISTVIEW(o)^.Flags and LVF_ORDER)=LVF_ORDER) and (PLISTVIEW(o)^.Style = LVS_LIST)) then
                        begin
				p := ListviewGetItemUnderPoint(PLISTVIEW(o), Mouse^.State.p);
				
				if assigned( p ) then if ( p^.Flags and LVI_SELECTED ) = LVI_SELECTED then begin
					PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags or LVF_ENDODRAG;
					if ( Mouse^.State.p.y < (p^.BufAbs.a.y+p^.BufAbs.b.y) div 2 ) then
						PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags or LVF_POINTDOWN
					else
						PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags and not LVF_POINTDOWN;
					PLISTVIEW(o)^.LPointed := p;
					WidgetDraw(o, NIL);
				end;
			end;
			
			
		end;
		
		if ( (Ev^.Message = WEvMouseLUp) or (Ev^.Message = WEvMouseRUp) ) then
		begin	
			p := ListviewGetItemUnderPoint(PLISTVIEW(o), Mouse^.State.p);
			s := p;
			if ((PLISTVIEW(o)^.Flags and LVF_SIZINGCOL) = LVF_SIZINGCOL) then
			begin	
				PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags and not LVF_SIZINGCOL;
				PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags and not LVF_HAVESELECT;

			end else if ( PLISTVIEW(o)^.Flags and LVF_ENDODRAG ) = LVF_ENDODRAG then
                        begin
				a := ListFoundItem(PLISTVIEW(o)^.Items,p);
				b := GetSelectedListItem(PLISTVIEW(o));
				
				if ( a <> b ) then begin
					ListRemoveItemEx(PLISTVIEW(o)^.Items,b);
					if not((PLISTVIEW(o)^.Flags and LVF_POINTDOWN) = LVF_POINTDOWN) then begin
						if (a = PLISTVIEW(o)^.Items^.Last) then PLISTVIEW(o)^.Items^.Last := b;
						ListAddItemBefore(PLISTVIEW(o)^.Items,b,a^.Next);
					end else
						ListAddItemBefore(PLISTVIEW(o)^.Items,b,a);
				end;
				PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags and not LVF_ENDODRAG;
				PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags and not LVF_HAVESELECT;

				WidgetDraw(o, NIL);
			end else if assigned( p ) then
			begin	
				if not((PLISTVIEW(o)^.Flags and LVF_HAVESELECT)=LVF_HAVESELECT) then
					ListviewSelectItem(PLISTVIEW(o),p,false,(Ev^.Keyb.Shift and KB_CTRL)=KB_CTRL);
				
				PLISTVIEW(o)^.Flags := PLISTVIEW(o)^.Flags and not LVF_HAVESELECT;

				
				if ( Ev^.Message = WEvMouseLUp ) then
				begin	
					if (((PLISTVIEW(o)^.Flags and LVF_CHECKBOXES)=LVF_CHECKBOXES)) and
                                             (PLISTVIEW(o)^.Style = LVS_LIST) then
                                        begin
					   if ( PointInRect(Mouse^.State.p, p^.RChk) ) then
                                           begin
						p^.Flags := p^.Flags xor LVI_CHECKED;
						WidgetDraw(PWIDGET(o), @p^.BufAbs);
						ListviewEventHandler := true;
					   end;
					end;
					
					if ( o^.Flags and WFExecuting ) = WFExecuting then begin
						if ( PLISTVIEW(o)^.OnValMsg > 0 ) then WidgetSendSEvent(o, EV_MESSAGE, PLISTVIEW(o)^.OnValMsg, p);
					end else
						if ( PLISTVIEW(o)^.OnSelMsg > 0 ) then WidgetSendSEvent(o, EV_MESSAGE, PLISTVIEW(o)^.OnSelMsg, p);
				
				end else if ( Ev^.Message = WEvMouseRUp and PLISTVIEW(o)^.OnRClkMsg ) then
					WidgetSendSEvent(o, EV_MESSAGE, PLISTVIEW(o)^.OnRClkMsg, p);
				ListviewEventHandler := true;
			end;

		end;
	end;
	ListviewEventHandler := ScrollWidgetEventHandler(o,Ev);
end;
////////////////////////////////////////////////////////////////////////////////
procedure FreeListviewItem ( p: Pointer );
begin
	if ( PListviewItem(p)^.Caption <> '') then PListviewItem(p)^.Caption := '';
	if assigned( PListviewItem(p)^.Icon ) then FreeIcon(PListviewItem(p)^.Icon);
	if assigned( PListviewItem(p)^.ColumsData ) then FreeList(PListviewItem(p)^.ColumsData);
	Dispose(PListviewItem(p));
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewItemPushCol ( p: PListviewItem; Text: l_text );
begin
   ListAdd( p^.ColumsData, '', @Text, nil);
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewItemSetColCaption( p:PListviewItem; Col:l_int; Caption:l_text );
var a: PListItem;
begin
	a := ListAtItem(p^.ColumsData,col+1);
	if assigned( a ) then begin
		if assigned( a^.Data ) then Dispose(PString(a^.Data));
		a^.Data := @Caption;
		if ( col = 0 ) then begin
			if ( p^.Caption <> '') then p^.Caption := '';
			p^.Caption := Caption;
                end;
	end;
end;
////////////////////////////////////////////////////////////////////////////////
function ListviewInitItem ( p:PListviewItem; Caption:l_text; Icon: PIcon ): PListviewItem;
begin
	fillchar(p^, sizeof(TListviewItem), 0);
	p^.Caption := Caption;
	p^.Icon := DuplicateIcon(Icon);
	p^.ColumsData := NewList;
	ListviewItemPushCol(p, Caption);
	ListviewInitItem := p;
end;
////////////////////////////////////////////////////////////////////////////////
function ListviewAddItem ( o:PListview; Caption:l_text; Icon:PIcon ): PListviewItem;
var
  p: PListviewItem; l: PListItem;
begin
	New(p);
	if not assigned( p ) then ListviewAddItem := NIL else
	ListviewInitItem(p,Caption,Icon);
	l := ListAdd (o^.Items, Caption, Pointer(p), nil);
        {l^.FreeData := {$ifdef fpc}@{$endif}FreeListviewItem;}
	ListviewAddItem := p;
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewItemRemoveItem ( o:PListview; p:PListviewItem );
var
  i: PListItem;
begin
  i := ListFoundItem(o^.Items,p);
  if not assigned( i ) then exit;
  ListRemoveItem(o^.Items,i);
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewRemoveAllItems ( o:PListview );
begin
	ScrollBarSetMaxY(SLLWDG(o),0);
	ScrollBarSetMaxX(SLLWDG(o),0);

	if assigned(o^.Items) then FreeList(o^.Items);
	o^.Items := NewList;
	o^.TopItem := NIL;
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewFree ( o: PWidget );
begin
	FreeList(PLISTVIEW(o)^.Items);
	FreeList(PLISTVIEW(o)^.Colums);
end;
////////////////////////////////////////////////////////////////////////////////
procedure Scroll ( o:PScrollWidget; p:TPoint );
begin
	if ( PWIDGET(o)^.Flags and WFForceBuffer ) = WFForceBuffer then
		WidgetDraw(PWIDGET(o),NIL)
	else
	begin
		PWIDGET(o)^.Flags := PWIDGET(o)^.Flags or WFForceBuffer;
		WidgetDraw(PWIDGET(o),NIL);
		PWIDGET(o)^.Flags := PWIDGET(o)^.Flags and not WFForceBuffer;
	end;
end;
////////////////////////////////////////////////////////////////////////////////
function GetSelectedItem ( o: PListview ): PListviewItem;
var
  a: PListItem;
  p: PListviewItem;
begin
	GetSelectedItem := NIL;
	a := o^.Items^.Last;
	p := NIL;
	if not assigned( a ) then GetSelectedItem := NIL else
	repeat
          if (PLISTVIEWITEM(a^.Data)^.Flags and LVI_SELECTED)= LVI_SELECTED then
          begin
            GetSelectedItem := a^.Data;
            exit;
          end;
          a := a^.Prev;
	until ( a = o^.Items^.Last );
end;
////////////////////////////////////////////////////////////////////////////////
function GetSelectedListItem ( o: PListview ): PListItem;
var
  a: PListItem;
  p: PListViewItem;
begin
        GetSelectedListItem := NIL;
	a := o^.Items^.Last;
	p := NIL;
	if assigned( a ) then GetSelectedListItem := NIL else
	repeat

		if (PLISTVIEWITEM(a^.Data)^.Flags and LVI_SELECTED)=LVI_SELECTED then
                begin
                  GetSelectedListItem := a;
                  exit;
                end;
                a := a^.Prev;

	until ( a = o^.Items^.Last );
end;
////////////////////////////////////////////////////////////////////////////////
function ListviewScrollTo ( o:PListview; i:PListviewItem ): l_bool;
var Old: TPoint;
begin
  Old := SLLWDG(o)^.Pos;
	
  if not assigned(i) then ListviewScrollTo := false else begin
	
	if ( ((PLISTVIEW(o)^.Flags and LVF_COLUMS)=LVF_COLUMS) and (PLISTVIEW(o)^.Style = LVS_LIST) ) then
	begin	
		ScrollBarSetPosY(SLLWDG(o),i^.BufAbs.a.y-PWIDGET(o)^.Absolute.a.y+SLLWDG(o)^.Pos.y-20);
        end else begin
		ScrollBarSetPosX(SLLWDG(o),i^.BufAbs.a.x-PWIDGET(o)^.Absolute.a.x+SLLWDG(o)^.Pos.x);
		ScrollBarSetPosY(SLLWDG(o),i^.BufAbs.a.y-PWIDGET(o)^.Absolute.a.y+SLLWDG(o)^.Pos.y);
        end;
	
	ListviewScrollTo := (Old.x <> SLLWDG(o)^.Pos.x) or (Old.y <> SLLWDG(o)^.Pos.y);

  end;
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewSelectItemAndTell ( o:PListview; p:PListviewItem; Scrol:l_bool; Multi:l_bool );
begin
	ListviewSelectItem(o,p,Scrol,Multi);
	if ( o^.OnSelMsg <> 0) then WidgetSendSEvent(PWIDGET(o), EV_MESSAGE, o^.OnSelMsg, p);
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewSelectItem ( o:PListview; p:PListviewItem; Scrol:l_bool; Multi:l_bool );
var
  RedrawAll: l_bool;
  a,b: PListItem;
begin
        if Scrol then RedrawAll := ListviewScrollTo(o,p) else RedrawAll := false;
	
	a := o^.Items^.Last; b := a;

	if not assigned( a ) then exit;

	o^.TopItem := NIL;
	if (not Multi) or not((o^.Flags and LVF_MULTISELECT)=LVF_MULTISELECT) then begin
		repeat
			if ( PLISTVIEWITEM(a^.Data)^.Flags and LVI_SELECTED ) = LVI_SELECTED then begin
				PLISTVIEWITEM(a^.Data)^.Flags := PLISTVIEWITEM(a^.Data)^.Flags and not LVI_SELECTED;
				if ( not RedrawAll ) then WidgetDraw(PWIDGET(o), @PLISTVIEWITEM(a^.Data)^.BufTop);
		 	end;
			a := a^.Prev;
		until ( a = b );
	end;
	if assigned( p ) then begin
		p^.Flags := (p^.Flags or LVI_SELECTED);
		o^.TopItem := p;
		if ( RedrawAll ) then begin
			Scroll( SLLWDG(o), SLLWDG(o)^.Pos);
			ScrollBarRedraw(SLLWDG(o));
		end else
			WidgetDraw(PWIDGET(o), @p^.BufTop);
	end;
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewSelectIndex ( o:PListview; Index:l_ulong; Scrol:l_bool; Multi:l_bool );
var
  a: PListViewItem;
begin
        if ( Index > 0) then begin
		a := ListAt(o^.Items,Index);
		if assigned( a ) then ListviewSelectItem(o,a,Scrol,Multi);
	end else
		ListviewSelectItem(o,NIL,Scrol,Multi);
end;
////////////////////////////////////////////////////////////////////////////////
function ListviewItemIndex ( o:PListview; a:PListviewItem ): l_ulong;
begin
	ListviewItemIndex := ListFoundIndex(o^.Items,a);
end;
////////////////////////////////////////////////////////////////////////////////
function ListviewRenameItemBox ( o:PListview; a:PListviewItem ): l_text;
var
  tb: PTextBox;
   r: TRect;
 ret: l_text;
begin
	ret := '';

	if ( o^.Style = LVS_MOSC ) then
		RectAssign(r,a^.BufTop.a.x+50,(a^.BufTop.a.y+a^.BufTop.b.y-al_text_height(ListviewFont)-4) div 2,a^.BufTop.b.x,(a^.BufTop.a.y+a^.BufTop.b.y+al_text_height(ListviewFont)+4) div 2)
	else if ( o^.Style = LVS_LIST ) then begin
		RectAssign(r,a^.BufTop.a.x,a^.BufTop.a.y,a^.BufTop.b.x,a^.BufTop.b.y);
		if (PLISTVIEW(o)^.Flags and LVF_CHECKBOXES)=LVF_CHECKBOXES then r.a.x := r.a.x + 14;
		if (PLISTVIEW(o)^.Flags and LVF_DRAWICONS)=LVF_DRAWICONS then r.a.x := r.a.x + 18;
		if (((PLISTVIEW(o)^.Flags and LVF_COLUMS)=LVF_COLUMS) and assigned(PLISTVIEW(o)^.Colums^.Last)) then
                r.b.x := r.a.x + PLISTVIEWCOL(PLISTVIEW(o)^.Colums^.Last^.Next^.Data)^.Width;
        end else
		RectAssign(r,a^.BufTop.a.x,a^.BufTop.b.y-al_text_height(ListviewFont)-4,a^.BufTop.b.x,a^.BufTop.b.y);
	
	tb := CreateTextbox(@Me,r,TBF_EDITABLE);
	InsertWidget(DeskTop, PWIDGET(tb));
	
	TextBoxSetTextEx(tb,a^.Caption);

	tb^.SelPos := Longint(Addr(tb^.Text));
	tb^.SelPosEnd := tb^.SelPos+TextLen(a^.Caption);
	WidgetDraw(PWIDGET(tb), NIL);
	
	
	tb^.ValidMsg := MSG_OK;

	WidgetSetFocus(PWIDGET(tb));
	
	if ( WidgetExecute(PWIDGET(tb)) <> MSG_CANCEL ) then begin
		if ( TextCaseCompare(tb^.Text,a^.Caption) > 0 ) then
			ret := tb^.Text;
        end;

	WidgetDispose(PWIDGET(tb));

	ListviewRenameItemBox := ret;
end;
////////////////////////////////////////////////////////////////////////////////
function ListviewCountSelectedFiles ( o: PListview ): l_ulong;
var
 Nb: l_ulong; a: PListItem;
begin
	Nb := 0;
	a := o^.Items^.Last;
	if not assigned(a) then ListviewCountSelectedFiles := 0 else
	repeat
	  if (PLISTVIEWITEM(a^.Data)^.Flags and LVI_SELECTED) = LVI_SELECTED then inc(Nb);
	  a := a^.Prev;
	until ( a = o^.Items^.Last );
	ListviewCountSelectedFiles := Nb;
end;
////////////////////////////////////////////////////////////////////////////////
function ListviewEstimateHeight ( Items: PList ): l_ulong;
var
 H: l_ulong; a: PListItem;
begin
  H := 4;
  a := Items^.Last;
  if not assigned(a) then ListviewEstimateHeight := 0 else
  begin
	repeat
		H := H + 18;
		a := a^.Prev;
	until ( a = Items^.Last );
	ListviewEstimateHeight := H;
  end;
end;
////////////////////////////////////////////////////////////////////////////////
procedure FreeListviewCol ( o: Pointer );
begin	
	if ( PListviewCol(o)^.Caption <> '' ) then PListviewCol(o)^.Caption := '';
	if assigned( PListviewCol(o)^.Icon ) then FreeIcon(PListviewCol(o)^.Icon);
	Dispose(PListviewCol(o));
end;
////////////////////////////////////////////////////////////////////////////////
procedure ListviewSetColSize( p: PListview; Col: l_int; Width: l_ulong );
var c: PListviewCol;
begin
	c := ListAt(p^.Colums,col+1);
	if assigned( c ) then begin
		c^.Width := Width;
		ListviewCalculateMax(PWIDGET(p));
        end;
end;
////////////////////////////////////////////////////////////////////////////////
(* Add a colum to a listview
	@param p Listview to affect
	@param Caption Caption of the new colum
	@param Icon Icon of the new colum
	@param Width Width in pixel of the new colum
	@param Flags Flags of the new colum 
*)
procedure ListviewAddColum(p:PListview; Caption:l_text; Icon: PIcon; Width:l_int; Flags:l_ulong);
var o: PListViewCol;
begin
	New(o);
	if not assigned(o) then exit;
	fillchar(o^,sizeof(TListviewCol),0);
	o^.Caption := TextDup(Caption);
	o^.Icon := DuplicateIcon(Icon);
	o^.Width := Width;
	o^.Flags := Flags;
	ListAdd(p^.Colums,'',o,nil(*{$ifdef fpc}@{$endif}FreeListviewCol*));
	ListviewCalculateMax(PWIDGET(p));
end;
////////////////////////////////////////////////////////////////////////////////
(* Change the displaying style of a listview
	@param p Listview to affect
	@param Style New style
	@param Flags If set, change the flags to the specified value
	@note Redraw widget and scrollbars
*)
function ColumnStyle(Condition:l_bool; v1,v2: l_int): l_ulong;
begin
  if Condition then ColumnStyle := v1 else ColumnStyle := v2;
end;

procedure ListviewSetStyle ( p: PListview; Style, Flags: l_int );
var c: l_bool;
begin
        c := ((p^.Flags and LVF_COLUMS)=LVF_COLUMS) and (p^.Style = LVS_LIST);
        p^.Style := Style;
	if (Flags > 0) then p^.Flags := Flags;
	ScrollBarChangeType(SLLWDG(p),ColumnStyle(c,3,1));
end;
////////////////////////////////////////////////////////////////////////////////
(* Initialise a listview widget, or a listview derivated widget.
	@param p listview to initialise
	@param Class Class of widget (ListviewClass by default)
	@param App Application which want to allocate the widget( use &Me in most cases)
	@param r Relative coordinates of new widget
	@param CaptionColumName First colum caption, the one wich is associated to all item 'caption' field
	@param Style Displaying style of items
	@param Flags Flags
*)
function InitListview(
  p: PListview;
  aClass: PWidgetClass;
  App: PApplication;
  r: TRect;
  CaptionColumName: l_text;
  Style: l_uchar;
  Flags: l_ulong
): PListview;
var c: l_bool;
begin
  if not assigned(p) then InitListview := NIL else
  begin
	fillchar(p^, sizeof(TListview), 0);
	p^.Items  := NewList;
	p^.Colums := NewList;
	p^.Style := Style;
	p^.Flags := Flags;
        c := ((Flags and LVF_COLUMS) = LVF_COLUMS) and (Style = LVS_LIST);
	InitScrollWidget(SLLWDG(p), aClass, App, r, ColumnStyle(c,3,1));
	PWIDGET(p)^.Flags := PWIDGET(p)^.Flags or WFFocusable;
	SLLWDG(p)^.Delta.y := 20;
	SLLWDG(p)^.Delta.x := 20;
	PWIDGET(p)^.Draw := {$ifdef fpc}@{$endif}ListviewDraw;
(*
{//	SLLWDG(p)^.Scrolled := {$ifdef fpc}@{$endif}Scroll; }
{//	PWIDGET(p)^.EventHandler := {$ifdef fpc}@{$endif}ListviewEventHandler; }
{//	PWIDGET(p)^.FreeEx := {$ifdef fpc}@{$endif}ListviewFree;
{//	PWIDGET(p)^.DropAccept 		:= {$ifdef fpc}@{$endif}ListviewDropAccept; }
{//	PWIDGET(p)^.DragBeforeDrop 	:= {$ifdef fpc}@{$endif}ListviewDragBeforeDrop; }
{//	PWIDGET(p)^.DropData 		:= {$ifdef fpc}@{$endif}ListviewDropData;       }
{//	PWIDGET(p)^.DragAfterDrop 	:= {$ifdef fpc}@{$endif}ListviewDragAfterDrop;  }
*)
        (* ListviewAddColum(p,CaptionColumName?CaptionColumName:'Caption',NIL,200,0); *)
	InitListview := p;
  end;
end;
////////////////////////////////////////////////////////////////////////////////
(* Creates a listview widget.
	@param App Application which want to allocate the widget( use &Me in most cases)
	@param r Relative coordinates of new widget
	@param CaptionColumName First colum caption, the one wich is associated to all item 'caption' field
	@param Style Displaying style of items
	@param Flags Flags
	@see WidgetInsert	
*)
function CreateListview(
  App:PApplication;
  r:TRect; CaptionColumName:l_text;
  Style:l_uchar;
  Flags:l_ulong
): PListview;
var
 p: PListView;
begin
	New(p);
	if not assigned(p) then CreateListview := NIL else
	CreateListview := InitListview(p,ListviewClass,App,r,CaptionColumName,Style,Flags);
end;
////////////////////////////////////////////////////////////////////////////////
function StdCreateListview ( App: PApplication; r: TRect ): PWidget;
begin
  StdCreateListview := PWIDGET(CreateListview( App, r, 'Caption', 0, 0));
end;
////////////////////////////////////////////////////////////////////////////////
function LibMain ( argc: integer; argv: l_text ): l_bool;
begin
	ListviewClass := CreateWidgetClass( ULONG_ID('L','s','t','V'), ScrollWidgetClass, 0 );
	{ListviewClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateListview;}
	
	SkinListview := GetSkinItem('Listview');
	
	if ( UseSkins and assigned(SkinListview) ) then begin
		ListviewFont := SkinListview^.Font;
		ListviewFontColor := SkinListview^.FontColor;
	end else begin
		ListviewFont := default_font;	
		ListviewFontColor := makecol(0,0,0);
	end;
	(*
	APPEXPORT(CreateListview);
	APPEXPORT(InitListview);
	APPEXPORT(ListviewAddItem);
	APPEXPORT(ListviewRemoveAllItems);
	APPEXPORT(ListviewEventHandler);
	APPEXPORT(GetSelectedItem);
	APPEXPORT(GetSelectedListItem);
	APPEXPORT(ListviewSelectItem);
	APPEXPORT(ListviewSelectIndex);
	APPEXPORT(ListviewItemIndex);
	APPEXPORT(ListviewSelectItemAndTell);
	APPEXPORT(ListviewRenameItemBox);
	APPEXPORT(FreeListviewItem);
	APPEXPORT(ListviewItemRemoveItem);
	APPEXPORT(ListviewCountSelectedFiles);
	APPEXPORT(ListviewGetItemUnderPoint);
	APPEXPORT(ListviewEstimateHeight);
	
	APPEXPORT(ListviewInitItem);
	APPEXPORT(ListviewItemPushCol);
	APPEXPORT(ListviewAddColum);
	APPEXPORT(ListviewItemSetColCaption);
	
	APPEXPORT(ListviewUpdateScrollbars);
	APPEXPORT(ListviewClass);
	APPEXPORT(ListviewSetStyle);
	APPEXPORT(ListviewFree);
	APPEXPORT(ListviewSetColSize);
	APPEXPORT(ListviewUpdateAndReDraw);
	APPEXPORT(ListviewCalculateMax);
	APPEXPORT(ListviewScrollTo);
        *)
	LibMain := true;
end;
////////////////////////////////////////////////////////////////////////////////
procedure Close;
begin

end;
////////////////////////////////////////////////////////////////////////////////
{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
    AppVersion := ULONG_ID(0,0,0,1);


{$endif}   {LINK_CLIBS}
end.
