(************************************************************************
// File lwp.h - Headers for lwp.c
// Copyright (C) 1997 Paolo De Marino
//
// Original Source Code by Sengan Short (sengan.short@durham.ac.uk)
// and Josh Turpen (snarfy@goodnet.com).
//
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Library General Public
//  License as published by the Free Software Foundation; either
//  version 2 of the License, or (at your option) any later version,
//  with the only exception that all the people in the THANKS file
//  must receive credit.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Library General Public License for more details.
//
//  You should have received a copy of the GNU Library General Public
//  License along with this library; see the file COPYING.LIB.
//  If not, write to the Free Software Foundation, Inc., 675 Mass Ave,
//  Cambridge, MA 02139, USA.
//
//  For contacting the author send electronic mail to
//     paolodemarino@usa.net
//
//  Or paper mail to
//
//     Paolo De Marino
//     Via Donizetti 1/E
//     80127 Naples
//     Italy
//
// History: see history.txt
************************************************************************)
unit lwp;
{
 .#include <dpmi.h>
 .#include <signal.h>
 .#include <string.h>
 .#include <stdlib.h>
 .#include <time.h>
}
const
  LWP_MAIN       = -1;  (* Id of the MAIN thread. Cannot be killed!    *)
  LWP_TCP        = -2;  (* Id of the TCP thread. Cannot be killed too! *)

(* for programming IRQ8 *)

  RTC2           = $0F;
  RTC4           = $0E;
  RTC8           = $0D;
  RTC16          = $0C;
  RTC32          = $0B;
  RTC64          = $0A;
  RTC128         = $09;
  RTC256         = $08;
  RTC512         = $07;
  RTC1024        = $06;
  RTC2048        = $05;
  RTC4096        = $04;
  RTC8192        = $03;

{
 $ifdef __cplusplus
  extern "C" 
 $endif
}

type {volatile}  
  lwp_semaphore_ptr = ^lwp_semaphore;
  lwp_semaphore = record      
     owned    : Integer;     (* Is it owned or not? *)
     owner_id : Integer;  (* If yes, by whom?    *)
  end;

type
  lwp_time = record
    secs  : time_t;          // Seconds from 0:0:0 1/1/70 GMT
    msecs : short;         // As from uclock()
  end;

(* Don't change this struct.  The scheduler needs to know their positions. *)
(* If you have to add anything, add it AFTER stack *)

type lwp_atkill_fct = procedure;
type lwp_spawn_fct  = procedure ( p: Pointer );

const MAX_ATKILL = 32;

type 
  Tlwp_Status = (
  		LWP_RUNNING,
		LWP_SLEEPING,
		LWP_WAIT_SEMAPHORE,
		LWP_WAIT_TRUE,
	    LWP_WAIT_FALSE,
		LWP_DEAD
        );
  TWaitingStatus = record
	{case lwp_semaphore of}
    what_sema :  lwp_semaphore_ptr; (* If status==WAIT_SEMAPHORE		*)
    what_int  :  PInteger;          (* If status==WAIT_TRUE|WAIT_FALSE	*) {volatile}
    wakeup_time:  lwp_time;         (* If status==SLEEPING				*)
  end;

  lwp_struct_ptr = ^lwp_struct;
  lwp_struct = record
	lwpid : integer;
	next  : lwp_struct_ptr;
	stklen: integer;
    stack : PLUINT;
(* --------------From now on, can be freely modified--------------*)
    stackTop: PLUINT;

	priority : unsigned;        (* Number of time slices for this thread *)

	status : Tlwp_Status;

    waiting : TWaitingStatus;

	userptr : Pointer;                (* User per-thread pointer			*)

	atkill: array[0..MAX_ATKILL-1] of lwp_atkill_fct;
  end; 
  lwp: lwp_struct;

(* User functions *)
function lwp_init(irq, speed: Integer): Integer;
function lwp_spawn(proc: lwp_spawn_fct; arg: Pointer; stack_length: Integer; priority: unsigned): Integer;
function lwp_atkill(proc: lwp_atkill_fct): Integer;
function lwp_kill(lwpid: Integer): Integer;
function lwp_getpid: Integer;
procedure lwp_setuserptr(usrdata: Pointer);
function lwp_getuserptr: Pointer;
procedure lwp_yield;
function lwp_getactive: Integer;

procedure lwp_set_priority(priority: unsigned);
function lwp_get_priority: unsigned;

procedure lwp_sleep(secs: l_uint; msecs: Byte);
procedure lwp_wait_true({volatile} what: PInteger);   (* Wait until what <> 0 *)

procedure lwp_wait_false({volatile} what: PInteger);  (* Wait until what = 0 *)
procedure lwp_pulse_true({volatile} what: PInteger);  (* Free all the tasks waiting for *)
procedure lwp_pulse_false({volatile} what: PInteger); (* what to become true/false *)
procedure lwp_wait_semaphore(sema: lwp_semaphore_ptr);
function lwp_release_semaphore(sema: lwp_semaphore_ptr): Integer;
procedure lwp_init_semaphore(sema: lwp_semaphore_ptr);

procedure _lwp_pm_irq8_timer_hook; {external;}
procedure _lwp_pm_irq0_timer_hook; {external;}

(* for locking critical code *)
var 
 {long}  _lwpasm_start: Longint; external;
 {long}  _lwpasm_end: Longint; external;
 
function _lwp_lock_data( lockaddr: Pointer; locksize: l_ulong ): Integer;
function _lwp_unlock_data( lockaddr: Pointer; locksize: l_ulong ): Integer;
function _lwp_lock_code( lockaddr: Pointer; locksize: l_ulong ): Integer;
function _lwp_unlock_code( lockaddr: Pointer; locksize: l_ulong ): Integer;

var
 _lwp_cur: lwp_struct_ptr; external;
 _lwp_heavy_dbg : integer; external; (* NOTE - TO BE REMOVED IN FINAL RELEASE! *)

(* for blocking a task switch *)
procedure lwp_thread_enable;  {volatile}
procedure lwp_thread_disable; {volatile}

implementation
{
 .#include "lwpconio.h"
 .#include "lwpstdio.h"
 .#include "lwppc.h"
 .#include "lwpstdlib.h"
}
{$ifndef LINK_CLIBS}

function lwp_init(irq, speed: Integer): Integer;
begin

end;

function lwp_spawn(proc: lwp_spawn_fct; arg: Pointer; stack_length: Integer; priority: unsigned): Integer;
begin

end;

function lwp_atkill(proc: lwp_atkill_fct): Integer;
begin

end;

function lwp_kill(lwpid: Integer): Integer;
begin

end;

function lwp_getpid: Integer;
begin

end;

procedure lwp_setuserptr(usrdata: Pointer);
begin

end;

function lwp_getuserptr: Pointer;
begin

end;

procedure lwp_yield;
begin

end;

function lwp_getactive: Integer;
begin

end;


procedure lwp_set_priority(priority: unsigned);
begin

end;

function lwp_get_priority: unsigned;
begin

end;


procedure lwp_sleep(secs: l_uint; msecs: Byte);
begin

end;

procedure lwp_wait_true({volatile} what: PInteger);   (* Wait until what <> 0 *)
begin

end;


procedure lwp_wait_false({volatile} what: PInteger);  (* Wait until what = 0 *)
begin

end;

procedure lwp_pulse_true({volatile} what: PInteger);  (* Free all the tasks waiting for *)
begin

end;

procedure lwp_pulse_false({volatile} what: PInteger); (* what to become true/false *)
begin

end;

procedure lwp_wait_semaphore(sema: lwp_semaphore_ptr);
begin

end;

function lwp_release_semaphore(sema: lwp_semaphore_ptr): Integer;
begin

end;

procedure lwp_init_semaphore(sema: lwp_semaphore_ptr);
begin

end;


procedure _lwp_pm_irq8_timer_hook; {external;}
begin

end;

procedure _lwp_pm_irq0_timer_hook; {external;}
begin

end;

 
function _lwp_lock_data( lockaddr: Pointer; locksize: l_ulong ): Integer;
begin

end;

function _lwp_unlock_data( lockaddr: Pointer; locksize: l_ulong ): Integer;
begin

end;

function _lwp_lock_code( lockaddr: Pointer; locksize: l_ulong ): Integer;
begin

end;

function _lwp_unlock_code( lockaddr: Pointer; locksize: l_ulong ): Integer;
begin

end;



procedure lwp_thread_enable;  {volatile}
begin

end;

procedure lwp_thread_disable; {volatile}
begin

end;

{$endif}
end.
