(*
////////////////////////////////////////////////////////////////////////////////
//
//  Memory files - Core File
//
//	(c) Copyright 2004 Point Mad. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
{
 .#include"kernel.h"
 .#include"memfile.h"
}
unit memfile;

interface

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$link memfile.o}
{$endif}

function MemFileNewEmpty: PFileStruct;

function FileExtractToBinData ( f: PFileStruct ): PBinData; // Works for all PFile,

function MemFileFromBinData ( b: PBinData ): PFileStruct;

function MemFileFromFreeBinData ( b: PBinData ): PFileStruct; // Use less memory

function MemFileCloseToBinData ( f: PFileStruct ): PBinData; // Works only for memfiles, use less memory

implementation
{$ifndef LINK_CLIBS}

var
  AppVersion: l_ulong;
  NeededLibs: l_uid;

const
  AppName: l_text = 'Memory files';
  nUID   : l_text = 'memfile';

(*////////////////////////////////////////////////////////////////////////////////*)
var
  MemFileDriver: PFileDriver;
  MemFileDrive : PDrive;
(*////////////////////////////////////////////////////////////////////////////////*)
function MemFileFromBinData ( b: PBinData ): PFileStruct;
var
	f: PFileStruct 
begin 
  New(f);
  if not assigned( f ) then MemFileFromBinData := NIL else
  begin

	fillchar(f,sizeof(TFileStruct),0);
	
	f^.Drive := MemFileDrive;

	f^.FileSize := b^.Size;
	f^.BufferPos := 0;
	f^.Pos := 0;
	
	f^.BufferSize := ((b^.Size div 1024)+1)*1024;
	getmem(f^.Buffer, f^.BufferSize);

	if not assigned( f^.Buffer ) then
	begin
    	dispose(f);
		MemFileFromBinData := NIL;
	end
	else
	begin
      move(b^.Data,f^.Buffer,b^.Size);

	  MemFileFromBinData := f;
	end;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function MemFileFromFreeBinData ( b: PBinData ): PFileStruct;
var
  f: PFileStruct;
begin
  New(f);
  if not assigned( f ) then MemFileFromFreeBinData := NIL else
  begin

	fillchar(f,sizeof(TFile),0);
	
	f^.Drive := MemFileDrive;

	f^.FileSize := b^.Size;
	f^.BufferPos := 0;
	f^.Pos = 0;
	
	f^.BufferSize := b^.Size;
	f^.Buffer := b^.Data;
	
	dispose(b);

	MemFileFromFreeBinData := f;
  end;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function MemFileNewEmpty: PFileStruct;
var
  f: PFileStruct;
begin
  New(f);
  if not assigned( f ) then MemFileNewEmpty := NIL else
  begin

	fillchar(f,sizeof(TFile),0);
	
	f^.Drive := MemFileDrive;

	f^.FileSize := 0;
	f^.BufferPos := 0;
	f^.Pos := 0;
	
	f^.BufferSize := 1024;
	getmem(f^.Buffer, f^.BufferSize);
	if not assigned( f^.Buffer ) then
	begin
		dispose(f);
		MemFileNewEmpty := NIL;
	end;

	MemFileNewEmpty := f;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure  MemFileClose ( f: PFileStruct );
begin
	if assigned( f^.Buffer ) then dispose(f^.Buffer);
	dispose(f);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function MemFileCloseToBinData ( f: PFileStruct ): PBinData;
var
  o: PBinData;
begin
  o := NewBinData(f^.Buffer,f^.FileSize);
  dispose(f);
  MemFileCloseToBinData := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function MemFileRead ( f: PFileStruct; d: Pointer; s: l_ulong s ): l_ulong;
var toRead:	l_ulong;
begin
	if ((s + f^.Pos) > f^.FileSize ) then
		toRead := f^.FileSize-f^.Pos;
	else
		toRead := s;
	move(f^.Buffer^,d^,s);
	
	f^.Pos := f^.Pos + toRead;
	
	DebugMessage('Read %d bytes, filesize : %d, pos : %d, buffer : %d',[toRead,f^.FileSize,f^.Pos,f^.BufferSize]);
	
	MemFileRead := toRead;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function MemFileWrite ( f: PFileStruct; d: Pointer; s: l_ulong ): l_ulong;
var tempbuffer: Pointer; size: l_ulong;
begin
	if not assigned( f^.Buffer ) then MemFileWrite := 0;

	if ( s > (f^.BufferSize - f^.Pos) ) then
	begin
	    size := f^.Buffersize;
	    
        getmem(tempbuffer,f^.BufferSize);
	    
        move(Buffer^, tempbuffer^, f^.Buffersize);

		f^.BufferSize := (((s + f^.Pos)/1024)+1)*1024;

		{f^.Buffer = realloc(f^.Buffer,f^.BufferSize);}

		freemem(f^.Buffer,f^.BufferSize); f^.Buffer := nil;
		
        getmem(f^.Buffer,f^.BufferSize);
		
        if f^.BufferSize >= size 
          then move(tempbuffer^,f^.Buffer^,size)
        else move(tempbuffer^,f^.Buffer^, f^.BufferSize);
		
		if not assigned( f^.Buffer ) then MemFileWrite := 0;
		
		DebugMessage('Buffer size changed to %d, to store %d bytes',[f^.BufferSize,s]);
	end;

	if ( s > (f^.FileSize - f^.Pos) ) then f^.FileSize := f^.Pos+s;
	
	move(d, f^.Buffer^, s);

	f^.Pos := f^.Pos + s;
	
	DebugMessage('Write %d bytes, filesize : %d, pos : %d, buffer : %d',[s,f^.FileSize,f^.Pos,f^.BufferSize]);
	
	MemFileWrite := s;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function MemFileGetPos ( f: PFileStruct; var Pos: l_ulong ): integer;
begin

   if assigned(f) then begin Pos = f^.Pos; MemFileGetPos := Pos end else MemFileGetPos := -1;
	
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function MemFileSeek ( f: PFileStruct; aType: Integer; Pos: l_long ): integer;
var
  newPos: l_ulong; tempbuffer: Pointer; size: l_ulong;
    
begin    	
	if ( aType = SEEK_SET ) then newPos := Pos;
	if ( aType = SEEK_CUR ) then newPos := Pos + f^.Pos;
	if ( aType = SEEK_END ) then newPos := f^.FileSize + Pos;
	
	if ( newPos > f^.BufferSize ) then
	begin
	    size := f^.Buffersize;
	    getmem(tempbuffer,f^.BufferSize);
	    move(Buffer^, tempbuffer^, f^.Buffersize);
		f^.BufferSize := ((Pos/1024)+1)*1024;
		
		freemem(f^.Buffer,f^.BufferSize); f^.Buffer := nil;
		getmem(f^.Buffer,f^.BufferSize);
		if f^.BufferSize >= size 
          then move(tempbuffer^,f^.Buffer^,size)
        else move(tempbuffer^,f^.Buffer^, f^.BufferSize);
		
		if not assigned( f^.Buffer ) then MemFileSeek := -1;
		
		DebugMessage('Buffer size changed to %d, to reach %d',[f->BufferSize,Pos]);
	end;
	
	f^.Pos := newPos;
	if ( f^.Pos > f^.FileSize ) then f^.FileSize := f^.Pos;
	
	DebugMessage('Moved, filesize : %d, pos : %d, buffer : %d',[f^.FileSize,f^.Pos,f^.BufferSize]);
	
	MemFileSeek := 0;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function	MemFileEOF ( f: PFileStruct ): integer;
begin

	if ( f^.Pos < f^.FileSize ) then MemFileEOF := 0 else
	MemFileEOF := 1;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
function	MemFileFlush ( f: PFileStruct ): integer;
begin

	MemFileFlush := 0;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
function FileExtractToBinData ( f: PFileStruct ): PBinData;
var
	Size  : l_ulong;
	OldPos: l_ulong;
	Data  : Pointer;
begin	
    Size := 0;
    OldPos := 0;
	MemFileGetPos(f,OldPos);
	
	FileSeek(f, 0, SEEK_END);
	MemFileGetPos(f,Size);
	
	FileSeek(f, 0, SEEK_SET);
	
	DebugMessage('Size = %d',[Size]);
	
	getmem(Data, Size);
	if not assigned( Data ) then FileExtractToBinData := NIL;
	
	Size := FileRead(Data,1,Size,f);
	
	DebugMessage('Size = %d',[Size]);
	
	FileSeek (f, OldPos, SEEK_SET);
	
	FileExtractToBinData := NewBinData(Data,Size);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function LibMain ( argc: Integer; argv: l_text ): l_bool;
begin
	New(MemFileDriver);
	fillchar(MemFileDriver,sizeof(TFileDriver),0);
	
	MemFileDriver^.DriverName	:= 'mem drive driver';
	MemFileDriver^.FileClose	:= {$ifdef fpc}@{$endif}MemFileClose;
	MemFileDriver^.FileRead		:= {$ifdef fpc}@{$endif}MemFileRead;
	MemFileDriver^.FileWrite	:= {$ifdef fpc}@{$endif}MemFileWrite;
	MemFileDriver^.FileGetPos	:= {$ifdef fpc}@{$endif}MemFileGetPos;
	MemFileDriver^.FileSeek		:= {$ifdef fpc}@{$endif}MemFileSeek;
	MemFileDriver^.FileFlush	:= {$ifdef fpc}@{$endif}MemFileFlush;
	MemFileDriver^.FileEOF		:= {$ifdef fpc}@{$endif}MemFileEOF;
	
	New(MemFileDrive);
	fillchar(MemFileDrive,sizeof(TDrive),0);

	MemFileDrive^.Name := TextDup("&__memory");
	MemFileDrive^.aType := FT_RAM;
	MemFileDrive^.Driver := MemFileDriver;
	(*
	APPEXPORT(MemFileNewEmpty);
	APPEXPORT(FileExtractToBinData);
	APPEXPORT(MemFileCloseToBinData);
	APPEXPORT(MemFileFromBinData);
	APPEXPORT(MemFileFromFreeBinData);
	*)
	LibMain := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Close;
begin

end;
(*////////////////////////////////////////////////////////////////////////////////*)

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion := ULONG_ID(0, 1, 0, 0);
  fillchar(NeededLibs, Sizeof(NeededLibs), 0);

{$endif} {LINK_C_LIBS}

end.
