(*
////////////////////////////////////////////////////////////////////////////////
//
//  Menu Widget - Core File
//
//  (c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
unit menu;
{
 .#include "kernel.h"
 .#include "widget.h"
 .#include "menu.h"
 .#include "window.h"
}
interface

uses
  guitypes,vfile,list,text,dynld,events,metrics,widget,skin,grfx,allegro{,window},
  keyboard,mousedrv,screen,font,debug;

{$i cdefs.inc}

{$ifdef link_clibs}
  {$link widget.o}
  {$link menu.o}
  {$link window.o}
{$endif}

type
  PMenu = ^TMenu;
  PMenuItem = ^TMenuItem;

  TMenu = record

    Items  : PMenuItem;
    Current: PMenuItem;

  end;

  FreeArgsProc = procedure(p: pointer);
  GenerateSubMenuProc = procedure(item: PMenuitem; p: Pointer);

  TMenuItem = record

    Caption:	l_text;
    Name:	l_text;
    Icon:	p_bitmap;
    Message:l_ulong;

    State:	byte; // 0 -> Nothing, 1 -> Selected, 2 -> Down

    Flags:	l_ulong;

    Next:	PMenuItem;
    SubMenu:	PMenu;

    BufAbs:	TRect;

    Args:   Pointer;
    FreeArgs: FreeArgsproc;

    GenerateSubMenu: GenerateSubMenuProc;

  end;

const MIF_FREEICON  =  $08;


{#define MENUITEM(o) ((PMenuItem)(o))}



type
  PMenuView = ^TMenuView;

  TMenuView = record

    o       : TWidget;

    Style   : byte;

    Menu    : PMenu;

    Child   : PMenuView;
    Parent  : PMenuView;

    Flags   : l_ulong;

    MsgDest : PWidget;

  end;

{#define MENUVIEW(o) ((PMenuView)(o))}

const
 MenuViewStyleHorizontal = $00;
 MenuViewStyleVertical   = $01;
 MenuViewStyleDock       = $00;

 MNF_DISPOSEONEVENT      = $01;
 MVF_FREEMENU            = $02;

 MI_DISABLE              = $01;

function PopUpMenu ( App:PApplication; p:TPoint; M:PMenu; MsgDest: PWidget;Options:l_ulong ): PMenuView;
function PopUpMenuWait ( App:PApplication; p:TPoint; M:PMenu; Options:l_ulong ): l_ulong;

{#define NewMenuView CreateMenuView}

function NewMenuView ( A:PApplication; r:TRect; M:PMenu; S:Byte; F:l_ulong ): PMenuView;
function CreateMenuView ( A:PApplication; r:TRect; M:PMenu; S:Byte; F:l_ulong ): PMenuView;
function NewMenu ( Itm:PMenuItem ): PMenu;
function NewMenuItem ( Caption:l_text; Icon:p_bitmap; Message,Flags:l_ulong; SubMenu:PMenu; Nxt:PMenuItem ): PMenuItem;
function NewMenuItemEx ( Caption:l_text; Icon:p_bitmap; Message,Flags:l_ulong; SubMenu:PMenu; GenerateSubMenu: GenerateSubMenuProc; Args: Pointer; Nxt:PMenuItem ): PMenuItem;

function NewMenuItemSeparator ( Next:PMenuItem ): PMenuItem;

function AddMenuItemEx ( m:PMenu; Caption:l_text; Icon:p_bitmap; Message,Flags:l_ulong; SubMenu:PMenu; GenerateSubMenu: GenerateSubMenuProc; Args: Pointer ): PMenuItem;
function AddMenuItem ( m:PMenu; Caption:l_text; Icon:p_bitmap; Message,Flags:l_ulong; SubMenu:PMenu ): PMenuItem;
function AddMenuItemSeparator ( m:PMenu ): PMenuItem;

function DuplicateMenuItem ( i:PMenuItem ): PMenuItem;
function DuplicateMenu ( m: Pointer ): Pointer;

procedure FreeMenu ( o:Pointer );

implementation

(*////////////////////////////////////////////////////////////////////////////////*)
var
  UID: l_ulong;
  AppVersion: l_ulong;
  MenuClass: PWidgetClass;

const
  AppName   : l_text = 'Menu Widget';
  nUID      : l_text = 'menu';
  NeededLibs: l_text = 'widget';

  SkinMenuviewBackground: PSkin	= NIL;
  SkinMenuviewSelect: PSkin	= NIL;
  SkinMenuBackground: PSkin	= NIL;
  SkinMenuSelect: PSkin		= NIL;

function PopUpMenuEx (App:PApplication; Parent:PMenuView; p:TPoint; M:PMenu; F,Options:l_ulong ): PMenuView; forward;

(*
	Notice : Menu duplicators support only basics menus

*)

function DuplicateMenu ( m: Pointer ): Pointer;
var n: PMenu;
begin
  DuplicateMenu := NIL;
  if assigned( m ) then
        begin
	  New(n);
	  if not assigned(n) then DuplicateMenu := NIL else
          begin
	    fillchar(n^,sizeof(TMenu),0);
	    n^.Items := DuplicateMenuItem(PMenu(m)^.Items);
	    DuplicateMenu := n;
          end;
        end;
end;

function DuplicateMenuItem ( i:PMenuItem  ): PMenuItem;
var n: PMenuItem;
begin
	New(n);
	if not assigned(n) then DuplicateMenuItem := NIL else
        begin
	fillchar(n^,sizeof(TMenuItem),0);
	
	n^.Caption := i^.Caption;
	n^.Name := i^.Name;
	n^.Message := i^.Message;
	
	if assigned( i^.Next ) then n^.Next := DuplicateMenuItem(i^.Next);
	if assigned( i^.SubMenu ) then n^.SubMenu := DuplicateMenu(i^.SubMenu);
	DuplicateMenuItem := n;
        end;
end;


procedure FreeMenuItem ( o: PMenuItem );
begin
	if ( o^.Caption <> '' ) then o^.Caption := '';
	if ( assigned(o^.FreeArgs) and assigned(o^.Args) ) then o^.FreeArgs(o^.Args);
	if ( o^.Flags and MIF_FREEICON ) = MIF_FREEICON then al_destroy_bitmap(o^.Icon);
	Dispose(o);

end;

procedure FreeMenu ( o: Pointer );
var a,n: PMenuItem;
begin
	a := PMenu(o)^.Items;
	while assigned( a ) do
        begin
		n := a^.Next;
		if assigned( a^.SubMenu ) then FreeMenu(a^.SubMenu);
		FreeMenuItem(a);
		a := n;
        end;
	Dispose(PMenu(o));
end;

const
 MSG_TREEVIEW_ONSEL  =	$1256;
 MSG_DBLCLK	     =	$2266;
 MSG_TB_GOUP	     =	$4266;

 MSG_VIEWICONS	     =	$4267;
 MSG_VIEWLIST	     =	$4268;
 MSG_VIEWMOSC	     =	$4269;

procedure WriteMenu ( f: PFileStruct; m: Pointer ); forward;

function this_text(b: l_bool; txt1,txt2: l_text): l_text;
begin
        if b then this_text := txt1 else this_text := txt2;
end;

procedure WriteMenuItem ( f:PFileStruct; i:PMenuItem );
begin
	FileWriteBinString(f,this_text((i^.Caption<>''),i^.Caption,''));
	FileWriteBinString(f,this_text((i^.Name<>''),i^.Name,''));
	file_write_long(f^.Handle,i^.Message);
	if assigned( i^.Next ) then begin
		file_write_char(f^.Handle,#1);
		WriteMenuItem(f,i^.Next);
	end else
		file_write_char(f^.Handle,#0);
		
	if assigned( i^.SubMenu ) then begin
		file_write_char(f^.Handle,#1);
		WriteMenu(f,i^.SubMenu);
	end else
		file_write_char(f^.Handle,#0);
end;


procedure WriteMenu ( f:PFileStruct; m:Pointer );
begin
	if assigned( PMenu(m)^.Items ) then begin
		file_write_char(f^.Handle,#1);
		WriteMenuItem(f,PMenu(m)^.Items);
        end else
		file_write_char(f^.Handle,#0);
end;

function ReadMenu ( f:PFileStruct ): Pointer; forward;

function ReadMenuItem ( f:PFileStruct ): PMenuItem;
var
n: PMenuItem;
begin
  New(n);
  if not assigned(n) then ReadMenuItem := NIL else
     begin
	fillchar(n,sizeof(TMenuItem),0);
	n^.Caption := FileReadBinString(f);
	n^.Name := FileReadBinString(f);
	if ( TextCompare('',n^.Name) = 0 ) then n^.Name := '';

	FileRead(n^.Message,1,4,f);
	if ( file_get_char(f^.Handle) = 1 ) then
		n^.Next := ReadMenuItem(f);
	if ( file_get_char(f^.Handle) = 1 ) then
		n^.SubMenu := ReadMenu(f);
	ReadMenuItem := n;

     end;
end;

function ReadMenu ( f:PFileStruct ): Pointer;
var n: PMenu;
begin
  New(n);
  if not assigned(n) then ReadMenu := NIL else
  begin
    fillchar(n^,sizeof(TMenu),0);
    if file_get_char(f^.Handle) = 1 then n^.Items := ReadMenuItem(f);
    ReadMenu := n;
  end;
end;

function NewMenu ( Itm: PMenuItem ): PMenu;
var o: PMenu;
begin
	New( o );

	if not assigned( o ) then NewMenu := NIL else
        begin

	fillchar(o^, sizeof(TMenu), 0);

	o^.Items := Itm;
	o^.Current := NIL;

	NewMenu := o;
        end;
end;

function MenuGetLatest ( m:PMenu ): PMenuItem;
var i: PMenuItem;
begin
	i := m^.Items;
	if assigned( i ) then while assigned( i^.Next ) do i := i^.Next;
	MenuGetLatest := i;
end;

function NewMenuItem ( Caption:l_text; Icon:p_bitmap; Message, Flags: l_ulong; SubMenu:PMenu; Nxt:PMenuItem ): PMenuItem;
var o: PMenuItem;
begin
	New(o);

	if not assigned( o ) then NewMenuItem := NIL else
        begin
	fillchar(o, sizeof(TMenuItem), 0);

	o^.Caption  := TextDup(Caption);
	o^.Icon     := Icon;
	o^.Message  := Message;
	o^.Flags    := Flags;
	o^.State := 0;
	o^.SubMenu  := SubMenu;
	o^.Next := Nxt;

	NewMenuItem := o;
        end;
end;

function AddMenuItem ( m:PMenu; Caption:l_text; Icon:p_bitmap; Message, Flags:l_ulong; SubMenu:PMenu ): PMenuItem;
var l,o: PMenuItem;
begin
	l := MenuGetLatest(m);
	o := NewMenuItem(Caption,Icon,Message,Flags,SubMenu,NIL);
	if assigned(l) then
		l^.Next := o
	else
		m^.Items := o;
	AddMenuItem := o;
end;

function NewMenuItemEx(Caption:l_text; Icon:p_bitmap; Message, Flags:l_ulong; SubMenu:PMenu; GenerateSubMenu:GenerateSubMenuProc; Args: Pointer; Nxt:PMenuItem): PMenuItem;
var o: PMenuItem;
begin
	New(o);

	if not assigned( o ) then NewMenuItemEx := NIL else
        begin
	fillchar(o, sizeof(TMenuItem), 0);

	o^.Caption  := TextDup(Caption);
	o^.Icon     := Icon;
	o^.Message  := Message;
	o^.Flags    := Flags;
	o^.State := 0;
	o^.SubMenu  := SubMenu;
	o^.Next := Nxt;
	o^.GenerateSubMenu := {$ifdef fpc}{$endif}GenerateSubMenu;
	o^.Args := Args;

	NewMenuItemEx := o;
        end;
end;

function AddMenuItemEx(m:PMenu; Caption:l_text; Icon:p_bitmap; Message,Flags:l_ulong; SubMenu: PMenu; GenerateSubMenu:GenerateSubMenuProc; Args: Pointer): PMenuItem;
var l,o: PMenuItem;
begin
  l := MenuGetLatest(m);
  o := NewMenuItemEx(Caption,Icon,Message,Flags,SubMenu,GenerateSubMenu,Args,NIL);
  if assigned( l ) then
    l^.Next := o
  else
    m^.Items := o;
  AddMenuItemEx := o;
end;

function NewMenuItemSeparator ( Next: PMenuItem ): PMenuItem;
begin
	NewMenuItemSeparator := NewMenuItem('---', NIL, 0, 0, NIL, Next);
end;

function AddMenuItemSeparator ( m: PMenu ): PMenuItem;
var l,o: PMenuItem;
begin
	l := MenuGetLatest(m);
	o := NewMenuItemSeparator(NIL);
	if assigned( l ) then
		l^.Next := o
	else
		m^.Items := o;
	AddMenuItemSeparator := o;
end;

procedure DrawSubMenuArrow ( buffer:p_bitmap; p:TPoint );
begin
  al_putpixel(buffer, p.x, p.y, makecol(0,0,0));
  al_putpixel(buffer, p.x, p.y+1, makecol(0,0,0));
  al_putpixel(buffer, p.x+1, p.y+1, makecol(0,0,0));
  al_putpixel(buffer, p.x, p.y+2, makecol(0,0,0));
  al_putpixel(buffer, p.x+1, p.y+2, makecol(0,0,0));
  al_putpixel(buffer, p.x+2, p.y+2, makecol(0,0,0));
  al_putpixel(buffer, p.x, p.y+3, makecol(0,0,0));
  al_putpixel(buffer, p.x+1, p.y+3, makecol(0,0,0));
  al_putpixel(buffer, p.x, p.y+4, makecol(0,0,0));
end;

procedure MenuDraw ( o:PWidget; buffer:p_bitmap; w:PRect );
var
  FontColor: l_color;
  m: PMenu; i: PMenuItem;
  x,y,dx,dy,ln: l_ulong;
  pp: TPoint;
begin
	m := PMENUVIEW(o)^.Menu;

	if not assigned(m) then exit;
	if not assigned(w) then
	begin
		DebugWarning('MENU::w is NIL!',[]);
		w := @o^.Absolute;
        end;

	if ( UseSkins ) then
		FontColor := SkinMenuviewBackground^.FontColor
	else
		FontColor := COL_3DTEXT;

	al_rectfill(buffer,w^.a.x,w^.a.y,w^.b.x,w^.b.y,o^.BackgroundColor);

	if ( PMENUVIEW(o)^.Style = MenuViewStyleHorizontal ) then
	begin
		i := m^.Items;
		x := o^.Absolute.a.x;
		y := o^.Absolute.a.y;

		while assigned( i ) do
		begin
			ln := al_text_length(default_font,i^.Caption);
			RectAssign(i^.BufAbs,x,y,x+ln+9,y+al_text_height(default_font)+6);

			if ( RectOverlay(i^.BufAbs, w^) ) then
			begin
				if ( i^.State = 1 ) then
				begin
					if (UseSkins) then
					begin
						FontColor := SkinMenuviewBackground^.FontColor;
						DrawSkin(buffer, SkinMenuviewBackground, i^.BufAbs.a.x,i^.BufAbs.a.y,i^.BufAbs.b.x,i^.BufAbs.b.y);
					end
					else
					begin
						FontColor := COL_3DTEXT;
						Rect3D(buffer,i^.BufAbs.a.x,i^.BufAbs.a.y,i^.BufAbs.b.x,i^.BufAbs.b.y,makecol(255,255,255),COL_3DDARK);
					end;
				end
				else if ( i^.State = 2 ) then
				begin
					if (UseSkins) then
					begin
						FontColor := SkinMenuviewSelect^.FontColor;
						DrawSkin(buffer, SkinMenuviewSelect, i^.BufAbs.a.x,i^.BufAbs.a.y,i^.BufAbs.b.x,i^.BufAbs.b.y);
					end
					else
					begin
						FontColor := COL_3DTEXT;
						Rect3D(buffer,i^.BufAbs.a.x,i^.BufAbs.a.y,i^.BufAbs.b.x,i^.BufAbs.b.y,makecol(128,128,128),COL_3DLIGHT);
					end;
				end;

				if ( i^.Flags and MI_DISABLE ) = MI_DISABLE then
				begin
					al_textout_ex(buffer, default_font, i^.Caption, i^.BufAbs.a.x+6, i^.BufAbs.a.y+4, COL_3DLIGHT, -1 );
					al_textout_ex(buffer, default_font, i^.Caption, i^.BufAbs.a.x+5, i^.BufAbs.a.y+3, COL_3DDARK, -1 );
		  		end
		  		else
		  		begin
			  		al_textout_ex(buffer, default_font, i^.Caption, i^.BufAbs.a.x+5, i^.BufAbs.a.y+3, FontColor, -1 );
		  		end;
			end;

			x := x+ln+10;
			i := i^.Next;
		end;
	end
	else
	begin
		i := m^.Items;
		x := o^.Absolute.a.x+3;
		y := o^.Absolute.a.y+3;
		dx := 0;
                dy := 0;

		if ( UseSkins ) then
		begin
			DrawSkin(buffer, SkinMenuBackground, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y);
		end
		else
		begin
			Rect3D(buffer,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.b.y,COL_3DLIGHT,COL_3DDARK);
		end;


		while assigned( i ) do
		begin
			if assigned( i^.Icon ) then dx := max(dx,i^.Icon^.w+2);
			i := i^.Next;
		end;

		i := m^.Items;

		while assigned( i ) do
		begin
			if ( TextCompare(i^.Caption,'---' ) = 0 ) then
			begin
				RectAssign(i^.BufAbs,0,0,0,0);
				al_hline(buffer,x,y+3,o^.Absolute.b.x-3,COL_3DDARK);
				al_hline(buffer,x,y+4,o^.Absolute.b.x-3,COL_3DLIGHT);
				y := y + 6;
			end
			else
			begin
				dy := al_text_height(default_font)+2;
				if assigned( i^.Icon ) then dy := i^.Icon^.h+2;
				ln := al_text_length(default_font,i^.Caption);
				RectAssign(i^.BufAbs,x,y,o^.Absolute.b.x-3,y+dy);
				if ( RectOverlay(i^.BufAbs, w^) ) then
				begin
					if ( i^.State <> 0 ) then
					begin
						if ( UseSkins ) then
						begin
							FontColor := SkinMenuSelect^.FontColor;
							DrawSkin(buffer, SkinMenuSelect, i^.BufAbs.a.x,i^.BufAbs.a.y,i^.BufAbs.b.x,i^.BufAbs.b.y);
						end
						else
						begin
							FontColor := COL_3DTEXT;
							al_rectfill(buffer,i^.BufAbs.a.x,i^.BufAbs.a.y,i^.BufAbs.b.x,i^.BufAbs.b.y,COL_3DOVER);
						end;
					end
					else
					begin
						if ( UseSkins ) then
						begin
							FontColor := SkinMenuBackground^.FontColor;
						end
						else
						begin
							FontColor := COL_3DTEXT;
						end;
					end;

					if assigned( i^.Icon ) then
						al_draw_sprite(buffer,i^.Icon, i^.BufAbs.a.x+1, i^.BufAbs.a.y+1);

					if ( i^.Flags and MI_DISABLE ) = MI_DISABLE then
					begin
						al_textout_ex(buffer, default_font, i^.Caption, i^.BufAbs.a.x+6+dx, i^.BufAbs.a.y+2, COL_3DLIGHT, -1 );
						al_textout_ex(buffer, default_font, i^.Caption, i^.BufAbs.a.x+5+dx, i^.BufAbs.a.y+1, COL_3DDARK, -1 );
					end
					else al_textout_ex(buffer, default_font, i^.Caption, i^.BufAbs.a.x+5+dx, i^.BufAbs.a.y+1, FontColor, -1 );

					if ( assigned(i^.SubMenu) or assigned(i^.GenerateSubMenu) ) then
					begin
                                            PointAssign(pp,i^.BufAbs.b.x-6, i^.BufAbs.a.y+(i^.BufAbs.b.y-i^.BufAbs.a.y) div 2 - 1);
					    DrawSubMenuArrow(buffer, pp);
					end;
			  	end;

			  	y := y+dy+1;
			end;

			i := i^.Next;
		end;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure MenuDisposeBackMe ( o: PMenuView );
var Parent: PMenuView;
begin
  if assigned( o^.Parent ) then
  begin
	  Parent := o^.Parent;
	  WidgetDispose(PWidget(o));
	  Parent^.Child := NIL;
	  MenuDisposeBackMe(Parent);
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure MenuDisposeFromMe ( o:PMenuView );
var
  Child : PMenuView;
  Parent: PMenuView;
begin
  Child  := o^.Child;
  Parent := o^.Parent;
  if assigned( o^.Parent ) then begin
	  o^.Parent^.Child := NIL;
	  WidgetDispose(PWidget(o));
	  WidgetSetFocusExNoNotify(PWIDGET(Parent));
  end else begin
	  o^.Child := NIL;
	  if assigned( Child ) then WidgetDispose(PWidget(Child));
	  if ( o^.Flags and MNF_DISPOSEONEVENT ) = MNF_DISPOSEONEVENT then
	  	WidgetDispose(PWidget(o))
	  else
	  	WidgetSendSEvent(PWIDGET(o),EV_MESSAGE,MSG_MSGCANCEL,NIL);
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function MenuGetActiveItem ( o:PMenuView ): PMenuItem;
var m: PMenu; i: PMenuItem;
begin
  m := PMENUVIEW(o)^.Menu;
  if assigned( m ) then
  begin
    i := m^.Items;
    while assigned( i ) do
    begin
      if ( i^.State = 1 ) then
      begin
        MenuGetActiveItem := i;
        i := i^.Next;
      end;
    end;
    MenuGetActiveItem := NIL;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function MenuGetBeforeActiveItem ( o:PMenuView ): PMenuItem;
var
  m: PMenu;
  p: PMenuItem;
  i: PMenuItem;
begin
  MenuGetBeforeActiveItem := NIL;
  m := PMENUVIEW(o)^.Menu;
  if assigned( m ) then
  begin
    p := NIL;
    i := m^.Items;
    while assigned( i ) do
    begin
      if ( i^.State = 1 ) then MenuGetBeforeActiveItem := p;
      p := i;
      i := i^.Next;
    end;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure MenuRunItem ( o: PMenuView; i: PMenuItem );
var
  Msg: l_ulong;
  SM : PMenuView;
  Dest: PWidget;
begin
  if not assigned(i^.SubMenu) and not assigned(i^.GenerateSubMenu) then
  begin
    SM := o;

    if ( i^.Flags and MI_DISABLE ) = MI_DISABLE then exit;
    while assigned( SM^.Parent ) do SM := SM^.Parent;
    if ( i^.Message <> 0 ) then begin
      Msg  := i^.Message;
      if assigned( SM^.MsgDest ) then begin
	  Dest := SM^.MsgDest;
	  MenuDisposeFromMe(SM);
          WidgetSendSEvent(Dest,EV_MESSAGE,Msg,i);
      end else if ( (SM^.Flags and MNF_DISPOSEONEVENT) <> MNF_DISPOSEONEVENT ) then begin
			  MenuDisposeFromMe(SM);
		  	WidgetSendSEvent(PWIDGET(SM),EV_MESSAGE,Msg,i);
      end else begin {// This should never happen ! }
        DebugMessage('TIP : Disposing on Event Menus should have a widget (MsgDest) to recolt messages',[]);
        WidgetMakeSync(PWidget(SM));
    	WidgetSendSEvent(PWIDGET(SM),EV_MESSAGE,Msg,i);
        if assigned( SM ) then MenuDisposeFromMe(SM); {// Oups... and if handler killed me ????}
        WidgetUnMakeSync(PWidget(SM));
      end;
    end;

  end;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure MenuExpendItem ( o:PMenuView; i:PMenuItem; delay:l_bool );
var pp: Tpoint;
begin
  if assigned(i^.SubMenu) or assigned(i^.GenerateSubMenu ) then
  begin
    if (i^.Flags and MI_DISABLE ) = MI_DISABLE then exit;
        if assigned( i^.GenerateSubMenu ) then begin
	  	i^.GenerateSubMenu(i,i^.Args);
	  	i^.GenerateSubMenu := NIL;
  	end;
    if ( o^.Style = MenuViewStyleVertical ) then
    begin
       PointAssign(pp,i^.BufAbs.b.x,i^.BufAbs.a.y);
       PopUpMenuEx(PWIDGET(o)^.AppOwner,o,pp,i^.SubMenu,0,0);
    end else if ( not delay   ) then begin
       PointAssign(pp,i^.BufAbs.a.x,i^.BufAbs.b.y);
       PopUpMenuEx(PWIDGET(o)^.AppOwner,o,pp,i^.SubMenu,0,0);
    end;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function MenuEventHandler ( o:PWidget; Ev:PEvent ): l_bool;
var
  i: PMenuItem;
  p: PMenuItem;
  m: PMenu;
  Parent: PMenuView; SM: PMenuView;
begin
        MenuEventHandler := false;

	if (Ev^.aType = EV_KEYBOARD) then
        begin
		if ( Ev^.Keyb.Key = AL_KEY_DOWN ) then begin

			i := MenuGetActiveItem(PMENUVIEW(o));

			if assigned( i ) then begin
				if assigned( i^.Next ) then begin

					i^.State := 0;
					i^.Next^.State := 1;
					WidgetDraw(o, @i^.BufAbs);
					WidgetDraw(o, @i^.Next^.BufAbs);

				end;
			end else begin
				PMENUVIEW(o)^.Menu^.Items^.State := 1;
				WidgetDraw(o, @PMENUVIEW(o)^.Menu^.Items^.BufAbs);
			end;
			MenuEventHandler := true;
                end else if ( Ev^.Keyb.Key = AL_KEY_UP ) then begin
			i := MenuGetActiveItem(PMENUVIEW(o));
			p := MenuGetBeforeActiveItem(PMENUVIEW(o));
			if ( assigned(i) and assigned(p) ) then begin
				i^.State := 0;
				p^.State := 1;
				WidgetDraw(o, @i^.BufAbs);
				WidgetDraw(o, @p^.BufAbs);
			end else if not assigned( i ) then begin
				PMENUVIEW(o)^.Menu^.Items^.State := 1;
				WidgetDraw(o, @PMENUVIEW(o)^.Menu^.Items^.BufAbs);
			end;
			MenuEventHandler := true;
                end else if (Ev^.Keyb.Key = AL_KEY_LEFT) then begin
			Parent := PMENUVIEW(o)^.Parent;
			if assigned( Parent ) then begin
				MenuDisposeFromMe(PMENUVIEW(o));
			end;
			MenuEventHandler := true;
                end else if ( Ev^.Keyb.Key = AL_KEY_RIGHT ) then begin
			i := MenuGetActiveItem(PMENUVIEW(o));
			if assigned( i ) then MenuExpendItem(PMENUVIEW(o),i,false);
			MenuEventHandler := true;
                end else if (Ev^.Keyb.Key = AL_KEY_ENTER) then begin
			i := MenuGetActiveItem(PMENUVIEW(o));
			if assigned( i ) then MenuRunItem(PMENUVIEW(o),i);
			MenuEventHandler := true;
                end;
	end;

	if ( Ev^.aType = EV_MESSAGE ) then
	begin
		if ( Ev^.Message = WEvNotifySameFocus  ) then begin
			SM := PMenuView(o);
			while assigned( SM^.Parent ) do SM := SM^.Parent;
			MenuDisposeFromMe(SM);
		  MenuEventHandler := true;
	        end;
		if ( Ev^.Message = WEvNotifyLostFocus ) then begin
			SM := PMenuView(o);
			while ( PWIDGET(SM^.Parent) <> FocusedWidget ) do SM := SM^.Parent;
			MenuDisposeFromMe(SM);
		  MenuEventHandler := true;
	        end;
	end;

	if (Ev^.aType = EV_MOUSE) then
	begin

		m := PMENUVIEW(o)^.Menu;
		if not assigned(m) then MenuEventHandler := false;

		if (Ev^.Message = WEvMouseMove) then
		begin
			i := m^.Items;

			while assigned( i ) do
			begin
				if ( PointInRect(Mouse^.State.p,i^.BufAbs) ) then
				begin
					if ( i^.State <> 1 ) then
					begin
						i^.State := 1;
						WidgetDraw(o, @i^.BufAbs);
					end;
				end
				else if ( i^.State = 1 ) then
				begin
					i^.State := 0;
					WidgetDraw(o, @i^.BufAbs);
				end;

				i := i^.Next;
			end;

			MenuEventHandler := true;
		end;

		if (Ev^.Message = WEvMouseLeft) then
		begin
			i := m^.Items;

			while assigned( i ) do
			begin
		    	  if ( i^.State <> 0 ) then
		    	  begin
			    i^.State := 0;
			    WidgetDraw(o, @i^.BufAbs);
			  end;

			  i := i^.Next;
			end;

			MenuEventHandler := true;
		end;

		if ( (Ev^.Message = WEvMouseLDown) or (Ev^.Message = WEvMouseLUp) or (Ev^.Message = WEvMouseEnterDelay) ) then
		begin
			i := m^.Items;

			while assigned( i ) do
			begin
				if ( PointInRect(Mouse^.State.p,i^.BufAbs) ) then
				begin
					if ( Ev^.Message = WEvMouseLDown ) then
						i^.State := 2
					else if ( Ev^.Message = WEvMouseLUp ) then
						i^.State := 1;

					WidgetDraw(o, @i^.BufAbs);

					if ( (Ev^.Message = WEvMouseLUp) or (Ev^.Message = WEvMouseEnterDelay) ) then
					begin
						if ( assigned(i^.SubMenu) or assigned(i^.GenerateSubMenu) ) then
						begin
							{ PopUp Menu ! }
							MenuExpendItem(PMENUVIEW(o),i,Ev^.Message = WEvMouseEnterDelay);
							MenuEventHandler := true;
						end
						else
						begin
							if ( Ev^.Message = WEvMouseLUp ) then
							begin
								MenuRunItem(PMENUVIEW(o),i);
							end
							else
							begin
								if assigned(PMENUVIEW(o)^.Child) then MenuDisposeFromMe(PMENUVIEW(o)^.Child);
							end;

							MenuEventHandler := true;
						end;
				        end;
				end
				else if ( i^.State <> 0 ) then
				begin
					i^.State := 0;
					WidgetDraw(o, @i^.BufAbs);
				end;

				i := i^.Next;
			end;

			MenuEventHandler := true;
		end;
	end;


end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure MenuViewFreeEx ( o:PWidget );
begin
	if assigned( PMENUVIEW(o)^.Child ) then WidgetDispose(PWIDGET(PMENUVIEW(o)^.Child));
	if((PMENUVIEW(o)^.Flags and MVF_FREEMENU)=MVF_FREEMENU) and
           not assigned(PMENUVIEW(o)^.Parent) then FreeMenu(PMENUVIEW(o)^.Menu);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function NewMenuView ( A:PApplication; r:TRect; M:PMenu; S:Byte; F:l_ulong ): PMenuView;
begin
  CreateMenuView(A, r, M, S, F);
end;

function CreateMenuView ( A:PApplication; r:TRect; M:PMenu; S:Byte; F:l_ulong ): PMenuView;
var
  v: PMenuView;
  i: PMenuItem;
begin
  New(v);
  if not assigned(v) then CreateMenuView := NIL else
  begin
	fillchar(v,sizeof(TMenuView),0);

	IntialiseWidget(A, MenuClass, PWIDGET(v), r );

	v^.Menu := M;
	v^.Style := S;
	v^.Flags := F;
	v^.Child := NIL;

	PWIDGET(v)^.Draw := {$ifdef fpc}@{$endif}MenuDraw;
	PWIDGET(v)^.EventHandler := {$ifdef fpc}@{$endif}MenuEventHandler;
	PWIDGET(v)^.FreeEx := {$ifdef fpc}@{$endif}MenuViewFreeEx;
	PWIDGET(v)^.BackgroundColor := COL_3DFACE;

	if assigned( M ) then
	begin
	  i := M^.Items;

	  while assigned( i ) do
	  begin
	    i^.State := 0;
	    i := i^.Next;
	  end;
	end;
  end;
  CreateMenuView := v;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function PopUpMenuEx ( App:PApplication; Parent:PMenuView; p:TPoint; M:PMenu; F:l_ulong;  Options:l_ulong ): PMenuView;
var
  w: l_ulong;
  h: l_ulong;
  t: l_ulong;
  d: l_ulong;
  o: PMenuView;
  i: PMenuItem;
  r: TRect;
begin
        w := 10; h := 6; t := 0; d := 0;
        i := M^.Items;
	while assigned( i ) do
	begin
		if assigned( i^.Icon ) then d := max(d,i^.Icon^.w);
		if assigned( i^.SubMenu) or assigned(i^.GenerateSubMenu) then t := 1;
		i := i^.Next;
        end;

	(**
	 * Submenu arrow width
	 *)
	if ( t <> 0 ) then d := d + 6;

	i := M^.Items;

	while assigned( i ) do
	begin
		w := max(w, al_text_length(default_font,i^.Caption)+16+d);

		if assigned( i^.Icon ) then
			h := h + i^.Icon^.h+3
		else if TextCompare(i^.Caption,'---' ) = 0 then
			h := h + 6
		else
			h := h + al_text_height(default_font)+3;

		i := i^.Next;
        end;

	if ( Options = 1 ) then begin
		if ( p.y > GSScreenHeight) then
			RectAssign(r,p.x,GSScreenHeight-h,p.x+w,GSScreenHeight)
		else
			RectAssign(r,p.x,p.y-h,p.x+w,p.y);
        end else begin
		if ( p.y+h > GSScreenHeight) then
			RectAssign(r,p.x,GSScreenHeight-h,p.x+w,GSScreenHeight)
		else
			RectAssign(r,p.x,p.y,p.x+w,p.y+h);
	end;



	o := NewMenuView(App,r,M,MenuViewStyleVertical,F);

	if assigned( Parent ) then
	begin
		o^.Parent := Parent;
		if assigned( Parent^.Child ) then MenuDisposeFromMe(Parent^.Child );
		Parent^.Child := o;
        end;

	InsertWidget(DeskTop, PWIDGET(o));

	PWIDGET(o)^.Flags := PWIDGET(o)^.Flags or WFFocusable;

	if assigned( Parent ) then
	begin
		if ( Parent^.Style = MenuViewStyleVertical ) then
			WidgetSetFocusExNoNotify(PWIDGET(o))
		else
			WidgetSetFocusEx(PWIDGET(o));
	end
	else
		WidgetSetFocusEx(PWIDGET(o));

	WidgetDraw(PWIDGET(o), NIL);

	PopUpMenuEx := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function PopUpMenuWait ( App:PApplication; p:TPoint; M:PMenu;  Options:l_ulong ): l_ulong;
var
  mview: PMenuView;
  Message: l_ulong;

begin
	mview := PopUpMenuEx(App,NIL,p,M,0,Options);

	Message := WidgetExecute(PWidget(mview));

	WidgetDispose(PWidget(mview));

	if ( Message = MSG_MSGCANCEL ) then
		PopUpMenuWait := 0
	else
		PopUpMenuWait := Message;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function PopUpMenu ( App:PApplication; p:TPoint; M:PMenu; MsgDest:PWidget; Options:l_ulong ): PMenuView;
var o: PMenuView;
begin
	o := PopUpMenuEx(App,NIL,p,M,MNF_DISPOSEONEVENT,Options);
	o^.MsgDest := MsgDest; { Tell menu to send his messages to widget MsgDest and not desktop... }

	PopUpMenu := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function StdCreateMenu ( App:PApplication; r:TRect ): PWidget;
begin
	StdCreateMenu := PWIDGET(CreateMenuView( App, r, NIL, 0, 0));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function MenuSetRProp ( o:PWidget; Name:l_text; const Value; Redraw:l_bool ): l_bool;
begin	
	if ( TextCompare(Name,'menu') = 0 ) then
        begin
		if assigned( PMENUVIEW(o)^.Menu ) then FreeMenu(PMENUVIEW(o)^.Menu);
		if assigned( PMenu(Value) ) then
			PMENUVIEW(o)^.Menu := DuplicateMenu(PMenu(Value))
		else 
			PMENUVIEW(o)^.Menu := NIL;
		if ( Redraw ) then WidgetDraw(PWIDGET(o), NIL);
		MenuSetRProp := true;
	end else MenuSetRProp := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function MenuGetRProp ( o:PWidget; Name: l_text ): Pointer;
begin
	if (TextCompare(Name,'menu') = 0) and assigned(PMENUVIEW(o)^.Menu) then
          MenuGetRProp := DuplicateMenu(PMENUVIEW(o)^.Menu)
	else MenuGetRProp := NIL;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function LibMain ( argc: Integer; argv: l_text ): l_bool;
begin
	MenuClass := CreateWidgetClass( ULONG_ID('M','e','n','u'), StdWidgetClass, 0 );
	MenuClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateMenu;
	MenuClass^.SetRProp := {$ifdef fpc}@{$endif}MenuSetRProp;
	MenuClass^.GetRProp := {$ifdef fpc}@{$endif}MenuGetRProp;

	CreatePropType(PTYPE_MENU,
           {$ifdef fpc}@{$endif}DuplicateMenu,
           {$ifdef fpc}@{$endif}FreeMenu,
           {$ifdef fpc}@{$endif}WriteMenu,
           {$ifdef fpc}@{$endif}ReadMenu
        );

	WClassRegisterProp(MenuClass,PTYPE_MENU,'menu');
	
	SkinMenuviewBackground	:= GetSkinItem('MenuviewBackground');
	SkinMenuviewSelect	:= GetSkinItem('MenuviewSelect');
	SkinMenuBackground	:= GetSkinItem('MenuBackground');
	SkinMenuSelect		:= GetSkinItem('MenuSelect');

        (*
	APPEXPORT(PopUpMenu);
	APPEXPORT(PopUpMenuWait);
	APPEXPORT(CreateMenuView);
	APPEXPORT(NewMenu);
	APPEXPORT(NewMenuItem);
	APPEXPORT(NewMenuItemEx);
	APPEXPORT(NewMenuItemSeparator);
	
	APPEXPORT(AddMenuItem);
	APPEXPORT(AddMenuItemEx);
	APPEXPORT(AddMenuItemSeparator);
	APPEXPORT(FreeMenu);
	*)
	
	LibMain := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Close;
begin

end;

begin
  UID := ULONG_ID('M','E','N','U');
  AppVersion := ULONG_ID(0,3,0,0);



end.
