(*
*	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
*
*	FILE:			rect.c
*
*	PROJECT:		Phoenix engine - Core
*
*	DESCRIPTION:	Metric functions (rect and point calculations). Version 2
*
*	CONTRIBUTORS:
*					Lukas Lipka
*         Etelain Julien
*
*	TODO:			@
*
*	BUG:			@
*
*	MISTAKE:		@
*
*	IDEA:			@
*)
unit metrics;
(** @defgroup metrics Metrics
	Rect, point and rect list calculations.
	@{
*)
(**	@file
	Metric functions 
	Rect, point and rect list calculations.
*)
interface

uses
  guitypes, List;
  
{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$LINK metrics.o}
{$endif}  

type
  PPoint = ^TPoint;
  TPoint = record
	x: Integer; {///< x position (horizontal)}
	y: Integer; {///< y position (vertical)  }
  end;

(***
	 3 dimension point
*)
type
  PSPoint = ^TSPoint;
  TSPoint = record
	x: Integer; {///< x position}
	y: Integer; {///< y position}
	z: Integer; {///< z position}
  end;

(**
	2 dimension rectangle
*)
type
  PRect = ^TRect;
  TRect = record

	a: TPoint; {///< Minimal point (Minimal value of x and y)}
  	b: TPoint; {///< Maximum point (Maximum value of x and y)}

  end;

(**
	2 dimension rectange size
*)
type
  PSize = ^TSize;
  TSize = record
	w: Integer; {///< Width }
	h: Integer; {///< Height}
  end;

function PointInRect	( p: TPoint; r: TRect ): l_bool;       {$ifdef LINK_CLIBS}external;{$endif}
function RectOverlay ( r: TRect; d: TRect ): l_bool;           {$ifdef LINK_CLIBS}external;{$endif}

procedure RectIntersept( var i: TRect; r, d: TRect );          {$ifdef LINK_CLIBS}external;{$endif}
procedure RectMove ( var d: TRect; r: TRect; p: TPoint );      {$ifdef LINK_CLIBS}external;{$endif}
procedure RectUnMove ( var d: TRect; r: TRect; p: TPoint );    {$ifdef LINK_CLIBS}external;{$endif}
procedure RectAssign( var r: TRect; ax, ay, bx, by: Integer ); {$ifdef LINK_CLIBS}external;{$endif}
procedure PointAssign( var p: TPoint; x, y: Integer );         {$ifdef LINK_CLIBS}external;{$endif}
procedure SizeAssign(var p: TSize; x,y: Integer);

function GetXDelta		( r: TRect ): Integer;                 {$ifdef LINK_CLIBS}external;{$endif}
function GetYDelta		( r: TRect ): Integer;                 {$ifdef LINK_CLIBS}external;{$endif}
function GetWidth		( r: TRect ): Integer;                 {$ifdef LINK_CLIBS}external;{$endif}
function GetHeight		( r: TRect ): Integer;                 {$ifdef LINK_CLIBS}external;{$endif}
function MakeSize(w, h: Integer): {$ifdef OPascal}TSize{$else}PSize{$endif}; {$ifdef LINK_CLIBS}external;{$endif}


function NewListFromRect ( r: TRect ): PList; {$ifdef LINK_CLIBS}external;{$endif}
function NewListFromRectList ( l: PList ): PList; {$ifdef LINK_CLIBS}external;{$endif}
function NewListFromRectListIntersept ( l: PList; r: TRect ): PList; {$ifdef LINK_CLIBS}external;{$endif}
procedure AddRectInList ( List: PList; ax, ay, bx, by: Integer ); {$ifdef LINK_CLIBS}external;{$endif}
procedure RemoveZoneInListItem ( List: PList; A: PListItem; b: TRect ); {$ifdef LINK_CLIBS}external;{$endif}

procedure RemoveZoneInList ( l: PList; r: TRect );  {$ifdef LINK_CLIBS}external;{$endif}
procedure RemoveZoneInListV ( l: PList; r: TRect ); {$ifdef LINK_CLIBS}external;{$endif}

procedure AddZoneInList ( l: PList; r: TRect ); {$ifdef LINK_CLIBS}external;{$endif}
procedure AddZoneInListEx ( List: PList; ax, ay, bx, by: Integer ); {$ifdef LINK_CLIBS}external;{$endif}
function RectOverlayList ( List: PList; r: TRect ): l_bool; {$ifdef LINK_CLIBS}external;{$endif}

{#define SizeAssign(s,w,h) PointAssign((TPoint*)s,w,h)}

(**
	Inline version of PointInRect (faster, use only if you need high performances)
	@see PointInRect
*)
{#define _PointInRect(p,r) (p.x >= r.a.x && p.y >= r.a.y && p.x <= r.b.x && p.y <= r.b.y)}
(**
	Inline version of RectOverlay (faster, use only if you need high performances)
	@see RectOverlay
*)
{#define _RectOverlay(r,d) !( (r).a.x > (d).b.x || (r).a.y > (d).b.y || (r).b.x < (d).a.x || (r).b.y < (d).a.y )}
(**
	Inline version of RectIntersept (faster, use only if you need high performances)
	@see RectIntersept
*)
{#define _RectIntersept(i,r,d) 
 (i)->b.x = max((r).a.x, min((r).b.x, (d).b.x)); 
 (i)->b.y = max((r).a.y, min((r).b.y, (d).b.y)); 
 (i)->a.x = min((r).b.x, max((r).a.x, (d).a.x)); 
 (i)->a.y = min((r).b.y, max((r).a.y, (d).a.y));
}
function min(a,b: Integer): Integer;
function max(a,b: Integer): Integer;

implementation
{$ifndef LINK_CLIBS}
{
 .#include "kernel.h"
 .#include <stdio.h>
 .#include <stdarg.h>
}
(*
	Point functions
*)

(**
	Assign a point with the specified x and y coordinates.
	@param p Pointer to the point to assign
	@param x Value of x
	@param y Value of y
*)
procedure PointAssign( var p: TPoint; x, y: Integer );
begin
	p.x := x;
 	p.y := y;
end;

procedure SizeAssign(var p: TSize; x,y: Integer);
begin
        p.w := x;
        p.h := y;
end;

(**
	Calculates wherever a point is located in the specified rectangle.
	@return true if the rect contains the point, otherwise false
*)
function PointInRect ( p: TPoint; r: TRect ): l_bool;
begin
	PointInRect := ((p.x >= r.a.x) and (p.y >= r.a.y) and (p.x <= r.b.x) and (p.y <= r.b.y));
end;

(*
	Rect functions
*)

(**
	Assigns the specified coordinates to a rectangle structure.
	Passed values are not checked.
	@param r Pointer to the rect to assign
	@param ax Minimal value of x
	@param ay Minimal value of y
	@param bx Maximal value of x
	@param by Maximal value of y
*)
procedure RectAssign( var r: TRect; ax, ay, bx, by: Integer );
begin
	r.a.x := ax;
	r.a.y := ay;
	r.b.x := bx;
	r.b.y := by;
end;


(**
	Moves a rectangle using a point to define x and y move.
	@param d Rect where result will be assigned
	@param r Rect to move
	@param p Point defining the move
	@see RectUnMove
*)
procedure RectMove ( var d: TRect; r: TRect; p: TPoint);
begin
	d := r;

	d.a.x := d.a.x + p.x;
	d.a.y := d.a.y + p.y;
	d.b.x := d.b.x + p.x;
	d.b.y := d.b.y + p.y;
end;

(**
	Moves a rectangle back from a prevoius call of RectMove. It corresponds to a negative move.
	@param d Rect where result will be assigned
	@param r Rect to move
	@param p Point defining the move
	@see RectMove
*)
procedure RectUnMove ( var d: TRect; r: TRect; p: TPoint );
begin
	d := r;

	d.a.x := d.a.x - p.x;
	d.a.y := d.a.y - p.y;
	d.b.x := d.b.x - p.x;
	d.b.y := d.b.y - p.y;
end;

(*
	Size functions for rect
*)

(**
	Calculates the position of the x coordinate of the rectangle
	@return x varation between r.b and r.a
*)
function GetXDelta(r: TRect): Integer;
begin
	GetXDelta := r.b.x-r.a.x;
end;

(**
	Calculates the position of the y coordinate of the rectangle
	@return y varation between r.b and r.a
*)
function GetYDelta(r: TRect): Integer;
begin
	GetYDelta := r.b.y-r.a.y;
end;

(**
	Calculates the width of the rectangle
	@return width of rectangle
*)
function GetWidth(r: TRect): Integer;
begin
	GetWidth := r.b.x-r.a.x+1;
end;

(**
	Calculates the height of the rectangle
	@return height of rectangle
*)
function GetHeight(r: TRect): Integer;
begin
	GetHeight := r.b.y-r.a.y+1;
end;

function MakeSize(w, h: Integer): {$ifdef OPascal}TSize{$else}PSize{$endif};
begin
  MakeSize{$ifndef OPascal}^{$endif}.w := w;
  MakeSize{$ifndef OPascal}^{$endif}.h := h;
end;

(**
	Dertermines if rects @a r and @a d has a common part
	@return 'true' if both rects has a common part, else returns false.
*)
function  RectOverlay ( r: TRect; d: TRect ): l_bool;
begin
	if ( r.a.x > d.b.x ) or ( r.a.y > d.b.y ) or ( r.b.x < d.a.x ) or ( r.b.y < d.a.y ) then
		RectOverlay := false;

	{ RectOverlay := _RectOverlay(r, d); }
end;

(**
	Calculate the common part of @a r and @a d ans assign it into @a i
	@param i Rect to assign the result in
	@param r First rect
	@param d Second rect
	@warning @a r and @a d MUST have a common part (use RectOverlay to be sure)
*)
procedure  RectIntersept ( var i: TRect; r, d: TRect );
begin
	i.b.x := max(r.a.x, min(r.b.x, d.b.x));
	i.b.y := max(r.a.y, min(r.b.y, d.b.y));
	i.a.x := min(r.b.x, max(r.a.x, d.a.x));
	i.a.y := min(r.b.y, max(r.a.y, d.a.y));
end;

(*////////////////////////////////////////////////////////////////////////////////*)
(*// Rect List system, mostly used by widget system                               *)


function RectOverlayList ( List: PList; r: TRect ): l_bool;
var
	a, b: PListItem;
	t: TRect;
begin	
	if assigned( List^.Last ) then
	begin
		a := List^.Last;
        b := List^.Last;
		repeat
			t := PRect(a^.Data)^;
			if RectOverlay(r,t) then RectOverlayList := true;
			a := a^.Next;
		until ( a = b );
	end;
    RectOverlayList := false;
end;

procedure FreeRIL(p: Pointer);
begin
  Dispose(PRect(p));
end;

procedure _AddRectInList ( List: PList; ax, ay, bx, by: Integer ); {$ifdef OPascal}inline;{$endif}
var
  r: PRect;
begin
	if ( ax <= bx ) and ( ay <= by ) then
	begin
		New(r);
		r^.a.x := ax; r^.a.y := ay;
		r^.b.x := bx; r^.b.y := by;
		ListAdd(List,'',r,{$ifdef fpc}@{$endif}FreeRIL);
	end;
end;

function NewListFromRect ( r: TRect ): PList;
var List: PList;
begin
	List := NewList;
	
	_AddRectInList(List,r.a.x,r.a.y,r.b.x,r.b.y);

	NewListFromRect := List;
end;

function NewListFromRectList ( l: PList ): PList;
var
  a,b: PListItem; t: TRect; List: PList;
begin
	List := NewList();
	if assigned( l^.Last ) then
	begin
		a := l^.Last;
        b := l^.Last;
		repeat
			t := PRect(a^.Data)^;
			_AddRectInList(List,t.a.x,t.a.y,t.b.x,t.b.y);
			a := a^.Next;
		
		until ( a = b );
	end;
	NewListFromRectList := List;
end;

function NewListFromRectListIntersept ( l: PList; r: TRect ): PList;
var a,b: PListItem; t: TRect; List: PList;
begin
	List := NewList();
	if assigned( l^.Last ) then
	begin
		a := l^.Last;
        b := l^.Last;
		repeat
			t := PRect(a^.Data)^;
			if RectOverlay (t,r) then
			begin
				_AddRectInList(List,min(r.b.x, max(r.a.x, t.a.x)),min(r.b.y, max(r.a.y, t.a.y)),
				max(r.a.x, min(r.b.x, t.b.x)),max(r.a.y, min(r.b.y, t.b.y)));
			end;
			
			a := a^.Next;
		
		until ( a = b );
	end;
	NewListFromRectListIntersept := List;
end;

procedure AddRectInList ( List: PList; ax, ay, bx, by: Integer );
begin
  _AddRectInList(List, ax, ay, bx, by);
end;

type
  TAddProc = procedure ( List: PList; ax, ay, bx, by: Integer );

procedure AddBminusA(aa,bb: TRect; List: PList; func: TAddProc);
begin
if ( aa.a.x < bb.a.x ) then
	func(List,aa.a.x,aa.a.y,bb.a.x-1,aa.b.y); 
if ( aa.a.y < bb.a.y ) then
	func(List,max(bb.a.x,aa.a.x),aa.a.y,aa.b.x,bb.a.y-1); 
if ( aa.b.x > bb.b.x ) then
	func(List,bb.b.x+1,max(aa.a.y,bb.a.y),aa.b.x,min(bb.b.y,aa.b.y)); 
if ( aa.b.y > bb.b.y ) then
	func(List,max(aa.a.x,bb.a.x),bb.b.y+1,aa.b.x,aa.b.y);
end;
	
procedure AddBminusAHPref(aa,bb: TRect; List: PList; func: TAddProc);
begin
if ( aa.a.x < bb.a.x ) then
	func(List,aa.a.x,max(aa.a.y,bb.a.y),bb.a.x-1,min(bb.b.y,aa.b.y)); 
if ( aa.a.y < bb.a.y ) then
	func(List,aa.a.x,aa.a.y,aa.b.x,bb.a.y-1); 
if ( aa.b.x > bb.b.x ) then
	func(List,bb.b.x+1,max(aa.a.y,bb.a.y),aa.b.x,min(bb.b.y,aa.b.y)); 
if ( aa.b.y > bb.b.y ) then
	func(List,aa.a.x,bb.b.y+1,aa.b.x,aa.b.y);
end;
	
procedure AddBminusAVPref(aa,bb: TRect; List: PList; func: TAddProc);
begin
if ( aa.a.x < bb.a.x ) then
	func(List,aa.a.x,aa.a.y,bb.a.x-1,aa.b.y); 
if ( aa.a.y < bb.a.y ) then
	func(List,max(aa.a.x,bb.a.x),aa.a.y,min(aa.b.x,bb.b.x),bb.a.y-1); 
if ( aa.b.x > bb.b.x ) then
	func(List,bb.b.x+1,aa.a.y,aa.b.x,aa.b.y); 
if ( aa.b.y > bb.b.y ) then
	func(List,max(aa.a.x,bb.a.x),bb.b.y+1,min(aa.b.x,bb.b.x),aa.b.y);
end;

procedure RemoveZoneInListItemEx ( List: PList; A: PListItem; b: TRect ); {$ifdef OPascal}inline;{$endif}
var r: TRect;
begin
	r := PRect(A^.Data)^;

	if RectOverlay (r, b) then
	begin
		AddBminusAHPref(r, b, List,{$ifdef fpc}@{$endif}_AddRectInList);
		ListRemoveItem(List,A);
	end;
end;

procedure RemoveZoneInListItem ( List: PList; A: PListItem; b: TRect );
begin
  RemoveZoneInListItemEx(List, A, b);
end;

procedure RemoveZoneInList ( l: PList; r: TRect );
var
	a, b, n: PListItem;
	c: Integer;
	t: TRect;
begin	
    c := 0;
	if assigned( l^.Last ) then
	begin
		a := l^.Last;
        b := l^.Last;

		repeat
			Inc(c);
			a := a^.Next;
		until ( a = b );
		
		repeat
		
			t := PRect(a^.Data)^;
			if RectOverlay (t,r) then
			begin
				n := a^.Next;
				AddBminusAHPref(t,r,l,{$ifdef fpc}@{$endif}_AddRectInList);
				ListRemoveItem(l,a);
				a := n;
			end else
				a := a^.Next;
			Dec(c);
		
		until ( c = 0 );
	end;
end;

procedure RemoveZoneInListV ( l: PList; r: TRect );
var
  a, b, n: PListItem;
  c: INteger;
  t: TRect;
begin	
	c := 0;
	if assigned( l^.Last ) then
	begin
		a := l^.Last;
        b := l^.Last;

		repeat
		
			Inc(c);
			a := a^.Next;
		
		until ( a = b );
		
		repeat
		
			t := PRect(a^.Data)^;
			if RectOverlay (t,r) then
			begin
				n := a^.Next;
				AddBminusAVPref(t,r,l,{$ifdef fpc}@{$endif}_AddRectInList);
				ListRemoveItem(l,a);
				a := n;
			end else
				a := a^.Next;
			Dec(c);
		
		until ( c = 0 );
	end;
end;

procedure AddZoneInListEx ( List: PList; ax, ay, bx, by: Integer );
var r: TRect;
begin
	r.a.x := ax; 
	r.a.y := ay;
	r.b.x := bx; 
	r.b.y := by;
	AddZoneInList(List,r);
end;

procedure AddZoneInList ( l: PList; r: TRect );
var
	a, b: PListItem;
	c: Integer;
	t: TRect;
begin	
	c := 0;
	if assigned( l^.Last ) then
	begin
		a := l^.Last;
        b := l^.Last;

		repeat
		
			Inc(c);
			a := a^.Next;
		
		until ( a = b );
		
		repeat		
			t := PRect(a^.Data)^;
			if RectOverlay(t,r) then
			begin
				AddBminusAHPref(r,t,l,{$ifdef fpc}@{$endif}AddZoneInListEx); c := 0;
			end else
			begin
			  a := a^.Next;
			  Dec(c);
		    end;
		until ( c = 0 );
	end;
	
	_AddRectInList(l,r.a.x,r.a.y,r.b.x,r.b.y);
end;

function min(a,b: Integer): Integer;
begin
  if a < b then min := a else min := b;
end;

function max(a,b: Integer): Integer;
begin
  if a > b then max := a else max := b;
end;
{$endif}

end.
