(**
*       (c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
*
*       FILE:                   mouse.c
*
*       PROJECT:                Phoenix engine - Core
*
*       DESCRIPTION:    Mouse event handler.
*
*       CONTRIBUTORS:
*                                       Lukas Lipka
*
*       TODO:                   @
*
*       BUG:                    @
*
*       MISTAKE:                @
*
*       IDEA:                   @
*)
unit mousedrv;
(** @defgroup mouse Mouse manager
        @{

        @file
        Mouse core file. Before named mouse instead of mousedrv

 .#include "kernel.h"
 .#include <string.h>
*)

interface

uses
  guitypes,metrics,list,screen
  {$ifdef _OZONE_DJGPP_},msmouse{$endif}
  ,allegro
  ,thread
  ,events
  ,registry
  ,debug
  ,text
  ,image
  ;
  
{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$LINK mouse{drv}.o}
{$endif}  

const Buttons: Integer = -1;

type
  PCursor = ^TCursor;
  TCursor = record

	Image: p_bitmap;
	SpotX: l_ulong;
	SpotY: l_ulong;
  end;

type
  //TMouseState = (msNone, msDown, msMoving);
  (*
  TMouseState = record
     p: TPoint;
     z: Integer;
     b: Integer;
  end;
  *)
  {
  Location: TPoint  andere Definition TMouseState
  Button: Integer
  }
  PMouse = ^TMouse;
  TMouse = record

	
	State: TMouseState;
	{
	Where: TPoint;
	Wheel: Integer;
	Button: Integer;
	}


	Moved: l_bool;

	Visible: l_bool;

	CursorPosition: TRect;

	ButtonChanges: Integer;



	Cur: PCursor;
	
	{ Private Data }
	PrevScr: p_bitmap;  { Private Data }
	Speed: Integer;		{ Change this through API functions, }
	DblClkSpeed: l_int;	{ otherwise it wont have any efect.  }
  end;

const
  cursor_pointer: PCursor = NIL;
  cursor_resize: PCursor = NIL;
  cursor_clock: PCursor = NIL;
  cursor_pencil: PCursor = NIL;
  old_button: Integer = 0;
  hides: Integer = 1;
  Mouse: PMouse = NIL;
  OldPos: TPoint = ( X:0; Y:0 );
  OldZ: Integer = 0;


  MSG_MOUSE_SCLLUP 	= $FFFF1901;
  MSG_MOUSE_SCLLDWN = $FFFF1902;

(*
*	Mouse buttons
*)
  BUTTON_LEFT       = $00000001;
  BUTTON_RIGHT      = $00000002;
  BUTTON_MIDDLE     = $00000004;

(*
*	Default mouse cursors (defined below)
*)

const
  CUR_POINTER : PCursor =   NIL;
  CUR_RESIZE  : PCursor =   NIL;
  CUR_CLOCK   : PCursor =   NIL;
  CUR_PENCIL  : PCursor =   NIL;

function CURSOR_OVER_RECT(r: TRect): l_bool;   {PointInRect(Mouse->State.p, (r))}
function CURSOR_IN_RECT(r: TRect): l_bool;   {PointInRect(Mouse->State.p, (r))}

procedure  GSSystemMouseInstall();    {$ifdef LINK_CLIBS}external;{$endif}
procedure  GSSystemMouseUninstall();  {$ifdef LINK_CLIBS}external;{$endif}

procedure MouseShow();  {$ifdef LINK_CLIBS}external;{$endif}
procedure MouseHide();  {$ifdef LINK_CLIBS}external;{$endif}

procedure MouseSetCursor ( Cur: PCursor );       {$ifdef LINK_CLIBS}external;{$endif}
procedure MouseSetCursorNamed ( Name: l_text );  {$ifdef LINK_CLIBS}external;{$endif}

procedure MouseSetCursorSpeed ( Speed: Integer ); {$ifdef LINK_CLIBS}external;{$endif}
procedure MouseSetDblClkSpeed ( Speed: Integer ); {$ifdef LINK_CLIBS}external;{$endif}


(**
	System cursors collection.
	Found cursors using the item key.
	@see ListKey
*)
var
  Cursors: PList;

(*
*	Why hide/show the mouse each time you draw somewhere under the mouse
*	when you can recap the mouse? Recap will redraw the mouse on rect 'r'
*	on 'from' and before update it PrevScr buffer. <= WTF?
*)
procedure _MouseRecap ( from: p_bitmap; r: TRect ); {$ifdef LINK_CLIBS}external;{$endif}

{#define MouseRecap(from,r) if (_RectOverlay(Mouse->CursorPosition,r )) _MouseRecap(from,r)}

{var Mouse: PMouse;}

function  __MouseStop: l_bool;     {$ifdef LINK_CLIBS}external;{$endif}
function  __MouseRestart: l_bool;  {$ifdef LINK_CLIBS}external;{$endif}

function NewCursor ( img: p_bitmap; SpotX, SpotY: Integer ): PCursor; {$ifdef LINK_CLIBS}external;{$endif}


implementation

{$ifndef LINK_CLIBS}

function CURSOR_OVER_RECT(r: TRect): l_bool;   {PointInRect(Mouse->State.p, (r))}
begin
  CURSOR_OVER_RECT := PointInRect(Mouse^.State.p, r);
end;

function CURSOR_IN_RECT(r: TRect): l_bool;   {PointInRect(Mouse->State.p, (r))}
begin
  CURSOR_IN_RECT := PointInRect(Mouse^.State.p, r);
end;
(**
        Changes the mouse speed
        Larger value of Speed reprents slower mouse mouvement
        At startup, value read from registry (/SYSTEM/MOUSE/speed)
        @param Speed Speed, default : 0
*)
procedure MouseSetCursorSpeed ( Speed: Integer );
begin
        Mouse^.Speed := Speed;
        {$ifdef _OZONE_DJGPP_}
        SetMouseSpeed(Speed, Speed);
        {$endif}
end;

(**
        Changes the mouse double click speed
        Set the maximal time between 2 click to be interpreted as a double click (in deciseconds)
        At startup, value read from registry (/SYSTEM/MOUSE/dblclk)
        @param Speed Maximal time between 2 click (in deciseconds), default : 4
*)
procedure MouseSetDblClkSpeed ( Speed: Integer );
begin
        Mouse^.DblClkSpeed := Speed;
end;

procedure MouseDrawCursor ( Del: l_bool; Add: l_bool );
var
  cf : Integer;
  cl : Integer;
  cr : Integer;
  ct : Integer;
  cb : Integer;
begin
        cf := Integer(Scrn^.clip);
        cl := Scrn^.cl;
        cr := Scrn^.cr;
        ct := Scrn^.ct;
        cb := Scrn^.cb;


        Scrn^.clip := TRUE;
        Scrn^.cl := 0;
        Scrn^.ct := 0;
        Scrn^.cr := Scrn^.w;
        Scrn^.cb := Scrn^.h;

        PauseMultitasking();

        if ( Del ) then
           al_blit(Mouse^.PrevScr,Scrn,0,0,OldPos.x-Mouse^.Cur^.SpotX,OldPos.y-Mouse^.Cur^.SpotY,Mouse^.Cur^.Image^.w,Mouse^.Cur^.Image^.h);
        

        if ( Add ) then begin
           al_blit(
             Scrn,
             Mouse^.PrevScr,
             al_mouse_x-Mouse^.Cur^.SpotX,
             al_mouse_y-Mouse^.Cur^.SpotY,
             0,
             0,
             Mouse^.Cur^.Image^.w,
             Mouse^.Cur^.Image^.h
           );
           al_draw_sprite(
             Scrn,
             Mouse^.Cur^.Image,
             al_mouse_x-Mouse^.Cur^.SpotX,
             al_mouse_y-Mouse^.Cur^.SpotY
           );
        end;

        OldPos.x := al_mouse_x;
        OldPos.y := al_mouse_y;

        UnPauseMultitasking();

        Scrn^.clip := l_bool(cf);
        Scrn^.cl := cl; Scrn^.cr := cr;
        Scrn^.ct := ct; Scrn^.cb := cb;
end;

procedure _MouseRecap ( from: p_bitmap; r: TRect );
var
  cf : Integer;
  cl : Integer;
  cr : Integer;
  ct : Integer;
  cb : Integer;
   i : TRect;
begin
        if ( Mouse^.Visible ) then
                (*if ( _RectOverlay (Mouse->CursorPosition,r ) ) then *) begin

                        cf := Longint(from^.clip);
                        cl := from^.cl;
                        cr := from^.cr;
                        ct := from^.ct;
                        cb := from^.cb;

                        RectIntersept(i,r,Mouse^.CursorPosition);

                        from^.clip := TRUE;

                        from^.cl := max(i.a.x,0);
                        from^.ct := max(i.a.y,0);

                        from^.cr := min(i.b.x+1,Scrn^.w);
                        from^.cb := min(i.b.y+1,Scrn^.h);

                        al_blit( from, Mouse^.PrevScr,i.a.x, i.a.y,i.a.x-Mouse^.CursorPosition.a.x,i.a.y-Mouse^.CursorPosition.a.y,i.b.x-i.a.x+1,i.b.y-i.a.y+1 );
                        al_draw_sprite (
                          from,
                          Mouse^.Cur^.Image,
                          al_mouse_x-Mouse^.Cur^.SpotX,
                          al_mouse_y-Mouse^.Cur^.SpotY
                        );

                        from^.clip := l_bool(cf);
                        from^.cl := cl;
                        from^.cr := cr;
                        from^.ct := ct;
                        from^.cb := cb;
                end;
end;

(**
        Changes the cursor to the specified one.
        @param Cur Pointer to the cursor to use
*)
procedure MouseSetCursor ( Cur: PCursor );
begin
        if ( Mouse^.Visible and Assigned(Mouse^.Cur) and Assigned(Mouse^.PrevScr) ) then
                MouseDrawCursor(true,false);

        if Assigned( Mouse^.PrevScr ) then al_destroy_bitmap(Mouse^.PrevScr);

        Mouse^.PrevScr := al_create_bitmap(Cur^.Image^.w+2,Cur^.Image^.h+2);
        Mouse^.Cur := Cur;

        if ( Mouse^.Visible ) then MouseDrawCursor(false,true);
end;

(**
        Changes the cursor to the specified one.
        @param Name Name of the pointer to use
*)
procedure MouseSetCursorNamed ( Name: l_text );
var c: PCursor;
begin
        c := ListKey(Cursors,Name);
        if Assigned( c ) then MouseSetCursor(c);
end;

(**
        Makes the mouse cursor visible
*)
procedure MouseShow;
begin
        if ( not Mouse^.Visible ) then
        begin
                Mouse^.Visible := true;

                if Assigned( Mouse^.Cur ) then MouseDrawCursor(false, true);
        end;
end;

(**
        Hides the mouse cursor
*)
procedure MouseHide;
begin
        if ( Mouse^.Visible ) then
        begin
                Mouse^.Visible := false;

                if Assigned( Mouse^.Cur ) then MouseDrawCursor(true,false);
        end;
end;

(**
        Allocates a new cursor form a bitmap
        @param Img Bitmap of the cursor
        @param SpotX x coordinates in Img of spot point
        @param SpotY y coordinates in Img of spot point
        @see FreeCursor
        @see MouseSetCursor
*)
function NewCursor ( Img: p_bitmap; SpotX, SpotY: integer ): PCursor;
var c: PCursor;
begin
        New(c);

        if not assigned( c ) then NewCursor := NIL else
        begin
          c^.Image := Img;
          c^.SpotX := SpotX;
          c^.SpotY := SpotY;
        end;
        
        NewCursor := c;
end;

(**
        Free a cursor
        @param c Cursor to free
        @see NewCursor
*)
procedure FreeCursor ( c: PCursor );
begin
        if assigned( c^.Image ) then al_destroy_bitmap(c^.Image);
        Dispose(c);
end;

procedure MouseEventGenerator ( Event: PEvent );
begin
  Event^.aType := EV_NOTHING;
  Mouse^.Moved := FALSE;

  Mouse^.ButtonChanges  := 0;

  Mouse^.State.z := al_mouse_z;

  if ( OldZ <> Mouse^.State.z ) then
  begin

    Event^.aType := EV_MOUSE;



          if ( OldZ-Mouse^.State.z < 0 ) then
                        Event^.Message := MSG_MOUSE_SCLLUP
          else
                Event^.Message := MSG_MOUSE_SCLLDWN;

          OldZ := Mouse^.State.z;
  end;
  (*
  if ( (al_mouse_x <> Mouse^.State.p.x) or (al_mouse_y <> Mouse^.State.p.y) ) then
  begin

    Event^.aType := EV_MOUSE;
    Mouse^.Moved := TRUE;

    Mouse^.State.p.x := al_mouse_x;
    Mouse^.State.p.y := al_mouse_y;

    CursorSize(r,x,y,w,h);
    RectAssign(r,x,y,x+w,y+h);

    if ( Mouse^.Visible ) then
    begin
        CursorSize(Mouse^.CursorPosition,Mouse^.State.p.x-Mouse^.Cur^.SpotX,Mouse^.State.p.y-Mouse^.Cur^.SpotY,Mouse^.Cur^.Image^.w-1,Mouse^.Cur^.Image^.h-1);
        MouseDrawCursor(true,true);
    end else
        RectAssign(Mouse^.CursorPosition,-1,-1,-1,-1);

  end;
  *)

  Mouse^.State.b := al_mouse_b;

  if ( (Mouse^.ButtonChanges and Mouse^.State.b) = old_button ) then
  begin

    Event^.aType := EV_MOUSE;

    old_button := Mouse^.State.b;
  end;
end;

procedure FreeCursor(p: pointer);
begin
  Dispose(PCursor(p));
end;

procedure  GSSystemMouseInstall;
var
  a,b,i,o: PRegKey;
  img: PImage;
  c: PCursor;
  sx,sy: l_ulong;
begin
        Cursors := NewList;

        Buttons := al_install_mouse;


        if ( Buttons > 0 ) then
        begin

                o :=     ResolveKey('/SYSTEM/MOUSE/CURSORS');
                if not assigned( o ) then
                begin
                  DebugMessage('No cursor declared in registry. Mouse cannot start',[]);
                end else

                if Assigned( o^.Last ) then
                begin
                        a := o^.Last^.Next;
                        b := a;
                        repeat
                                if ( a^.aType = RKT_TEXT ) then
                                begin
                                        sx := 0; sy := 0;

                                        if Assigned( a^.Last ) then
                                        begin
                                                i := a^.Last;
                                                repeat
                                                        if ( (TextCaseCompare(i^.Name,'sy' ) = 0) and ( i^.aType = RKT_INT ) ) then
                                                                sy := PInteger(i^.Data)^
                                                        else if ( (TextCaseCompare(i^.Name,'sx' ) = 0) and ( i^.aType = RKT_INT ) ) then
                                                                sx := PInteger(i^.Data)^;
                                                        i := i^.Next;
                                                until ( i = a^.Last );
                                        end;

                                        Img := LoadImage(PString(a^.Data)^);

                                        if Assigned( Img ) then
                                        begin

                                                c := NewCursor(Img,sx,sy);

                                                if Assigned( c ) then
                                                        ListAdd(Cursors,a^.Name,c,nil(*{$ifdef fpc}@{$endif}FreeCursor*))
                                                else
                                                        al_destroy_bitmap(Img);
                                        end
                                          else begin
                                              DebugMessage('No cursor declared in registry. Mouse cannot start',[]);
   { good idea?? }                         
                                          end;
                                end;
                                a := a^.Next;
                        until ( a = b );
                end;

                NEW(Mouse);
                fillchar(Mouse^,sizeof(TMouse),0);

                Mouse^.State.p.x := al_mouse_x;
                Mouse^.State.p.y := al_mouse_y;
                Mouse^.State.z := al_mouse_z;
                Mouse^.State.b := al_mouse_b;

                cursor_pointer := ListKey(Cursors,'arrow');
                cursor_resize  := ListKey(Cursors,'resize');

                if not Assigned( cursor_pointer ) then
                begin
                        DebugMessage('Arrow not set, mouse cannot start',[]);
                        exit;
                end;

                MouseSetCursorSpeed(KeyGetInt('/SYSTEM/MOUSE/speed', 0));
                MouseSetDblClkSpeed(KeyGetInt('/SYSTEM/MOUSE/dblclk', 4));
                MouseSetCursor(CUR_POINTER);

                InstallEventGenerator({$ifdef fpc}@{$endif}MouseEventGenerator);
        end
        else
        begin
                DebugError('Mouse not found',[]);
        end;
end;

procedure  GSSystemMouseUninstall;
begin
        if ( Buttons > 0 ) then
        begin

                MouseHide;
                Mouse^.Cur := NIL;
                Dispose(Mouse);
                Mouse := NIL;

                al_remove_mouse;
        end;

        if assigned( Cursors ) then FreeList(Cursors);

end;

(**
        Removes temporary mouse support by kernel, and allow extra software to uses it directly
        Never uses it in multitask mode.
        @see __MouseRestart
*)
function  __MouseStop: l_bool;
begin

        if ( Buttons > 0 ) then
        begin
                MouseHide;
                al_remove_mouse;
        end;
        __MouseStop := true;
end;

(**
        Restores mouse after a __MouseStop call
        Do not show mouse, use MouseShow(); after it.
        @see MouseShow
        @see __MouseStop
*)
function  __MouseRestart: l_bool;
begin

        Buttons := al_install_mouse;

         __MouseRestart := true;
end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  CUR_POINTER  :=   cursor_pointer;
  CUR_RESIZE   :=   cursor_resize;
  CUR_CLOCK    :=   cursor_clock;
  CUR_PENCIL   :=   cursor_pencil;
  Cursors      := NIL;

{$endif}

end.
