(**
*       (c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
*
*       FILE:                   progress.c
*
*       PROJECT:                Phoenix engine - Core
*
*       DESCRIPTION:    Progress bar widget.
*
*       CONTRIBUTORS:
*                                       Lukas Lipka
*
*       TODO:                   @
*
*       BUG:                    @
*
*       MISTAKE:                @
*
*       IDEA:                   @
*)
{
 .#include "kernel.h"
 .#include "widget.h"
 .#include "progress.h"
 .#include "string.h"
}
unit progress;

interface

uses guitypes,events,dynld,text,metrics,widget,skin,font,allegro;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
  {$link widget.o}
  {$link progress.o}
{$endif}

type
  PProgressBar = ^TProgressBar;
  StepFunc = function(b: PProgressBar; Steps: integer): integer;
  TProgressBar = record

	(** Inherited functions and variables from TWidget **)
	o:	TWidget;

	(** Number of steps to be done till 100% **)
	Steps: l_ulong;

	(** Number of steps done, since start **)
	Promile: l_ulong;

	(** Increases the promile variable, adding %Steps each time **)
	Step: Stepfunc;

  end;
  PROGRESSBAR = PProgressBar;

procedure PBDraw(o: PWidget; buffer: p_bitmap; w: PRect); {$ifdef LINK_CLIBS}external;{$endif}
function PEventHandler(o: PWidget; Ev: PEvent): l_bool; {$ifdef LINK_CLIBS}external;{$endif}
function PBStep(b: PProgressBar; Steps: integer): Integer; {$ifdef LINK_CLIBS}external;{$endif}
function CreateProgressBar(App: PApplication; r: TRect; Steps: l_uint): PProgressBar; {$ifdef LINK_CLIBS}external;{$endif}

implementation
{$ifndef LINK_CLIBS}
{$ifdef Opascal}
uses Sysutils;
{$else}
uses OSysutil;
{$endif}

var
  AppVersion: l_ulong;

const
  AppName   : l_text = 'Progressbar Widget';
  nUID      : l_text = 'progress';
  NeededLibs: array[0..2] of ShortString  = ( 'widget', '', '' );

var
  SkinBar: PSkin;
  SkinBG: PSkin;

  ProgressClass: PWidgetClass;

(**
*       NAME: PBDraw
*       DESCRIPTION: Draws the Progressbar
*)
procedure PBDraw(o: PWidget; buffer: p_bitmap; w: PRect);
var StrFormat: String;
begin
        if ( UseSkins and assigned(SkinBG) and assigned(SkinBar) ) then
        begin
                { Background }
                DrawSkin(buffer, SkinBG, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y);

                { Line indicating percentage }
                if (PROGRESSBAR(o)^.Promile > 1) and (PROGRESSBAR(o)^.Steps <> 0) then
                        DrawSkin(buffer, SkinBar, 
                          o^.Absolute.a.x + SkinBG^.Left, 
                          o^.Absolute.a.y + SkinBG^.Top, (
                          o^.Absolute.a.x + SkinBG^.Left) + ((PROGRESSBAR(o)^.Promile * ((
                          o^.Absolute.b.x - SkinBG^.Right) - (
                          o^.Absolute.a.x + SkinBG^.Left))) div PROGRESSBAR(o)^.Steps), o^.Absolute.b.y  - SkinBG^.Bottom);
                (**
                *       Number of percent
                *)
                {$ifdef OPascal}
                StrFormat := Format('%i%%',[]);
                {$else}
                FormatStr(StrFormat,'%i%%',[]);
                {$endif}
                (*
                if ( PROGRESSBAR(o)^.Steps <> 0 ) then
                        al_textout_centre_ex(
                         buffer,
                         default_font,
                         StrFormat,
                         (PROGRESSBAR(o)^.Promile * 100) div PROGRESSBAR(o)^.Steps, NIL, 10),
                         o^.Absolute.a.x+((o^.Absolute.b.x-o^.Absolute.a.x) div 2),
                         o^.Absolute.a.y + ((o^.Absolute.b.y - o^.Absolute.a.y) div 6),
                         makecol(255,255,255),
                         -1
                        );
                  *)
        end else begin
                { Background }
                al_rect(buffer, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y, COL_BOXFACE);
                al_rectfill(buffer, o^.Absolute.a.x+1, o^.Absolute.a.y+1, o^.Absolute.b.x-1, o^.Absolute.b.y-1, COL_BOXDARK);

                { Line indicating percentage }
                if (PROGRESSBAR(o)^.Promile > 1) and (PROGRESSBAR(o)^.Steps <> 0) then
                        al_rectfill(buffer, 
                          o^.Absolute.a.x+2, 
                          o^.Absolute.a.y+2, (
                          o^.Absolute.a.x + 2) + 
                          ((PROGRESSBAR(o)^.Promile * (
                            o^.Absolute.b.x - 
                            o^.Absolute.a.x-4)) div PROGRESSBAR(o)^.Steps), o^.Absolute.b.y-2, COL_BOXSEL);
                (**
                *       Number of percent
                *)
                (*
                if ( PROGRESSBAR(o)^.Steps <> 0 ) then
                        al_textout_centre_ex(buffer, default_font, StrFormat,
                         (PROGRESSBAR(o)^.Promile * 100) div PROGRESSBAR(o)^.Steps, NIL, 10), 
                           o^.Absolute.a.x+((o^.Absolute.b.x-o^.Absolute.a.x) div 2),
                           o^.Absolute.a.y + ((o^.Absolute.b.y - o^.Absolute.a.y) div 6), COL_BOXTEXT,-1);
                *)
        end;
end;

(**
*       NAME: PEventHandler
*       DESCRIPTION: Event handler for the Progressbar
*)
function PEventHandler(o: PWidget; Ev: PEvent): l_bool;
begin
        if (Ev^.aType = EV_MOUSE) then
        begin

        end;

        PEventHandler := false;
end;

(**
*       NAME: PBStep
*       DESCRIPTION: Steps @Steps forward
*       EXPORTED: In the TProgressbar structure
*)
function adjust_steps(Steps1,Steps2: integer): l_bool;
begin
  adjust_steps := (Steps1 <= Steps2);
end;

function PBStep(b: PProgressBar; Steps: integer): Integer;
begin
        if adjust_steps(b^.Promile + Steps, b^.Steps) then
        {if (b^.Promile + Steps) < b^.Steps or ((b^.Promile + Steps) = b^.Steps)) then}
                b^.Promile := b^.Promile + Steps;

        WidgetDraw(PWIDGET(b), NIL);

        PBStep := b^.Promile;
end;

(**
*       NAME: CreateProgressbar
*       DESCRIPTION: Creates a new Progressbar
*       EXPORTED: Core
*)
function CreateProgressBar(App: PApplication; r: TRect; Steps: l_uint): PProgressBar;
var b: PProgressBar;
begin
   New(b); 

   if not assigned(b) then CreateProgressBar := NIL else
   begin

        fillchar(b, sizeof(TProgressBar), 0);

        b^.Steps := Steps;
        b^.Promile := 0;

        b^.Step := {$ifdef fpc}@{$endif}PBStep;

        PWIDGET(b)^.EventHandler := {$ifdef fpc}@{$endif}PEventHandler;
        PWIDGET(b)^.Draw := {$ifdef fpc}@{$endif}PBDraw;
        IntialiseWidget(App, ProgressClass, PWIDGET(b), r);

        CreateProgressBar := b;
   end;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function StdCreateProgress ( App: PApplication; r: TRect ): PWidget;
begin
  StdCreateProgress := PWIDGET(CreateProgressBar( App, r, 100));
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function ProgressSetRProp ( o: PWidget; Name: l_text; const Value;  Redraw: l_bool ): l_bool;
begin
        ProgressSetRProp := false;
        if ( TextCompare(Name,'value')=0 ) then
        begin
                PROGRESSBAR(o)^.Promile := l_ulong(Value);
                if ( PROGRESSBAR(o)^.Promile > PROGRESSBAR(o)^.Steps ) then PROGRESSBAR(o)^.Promile := PROGRESSBAR(o)^.Steps;
                if ( Redraw ) then WidgetDraw(PWIDGET(o), NIL);
                ProgressSetRProp := true;
        end;
        if ( TextCompare(Name,'max')=0 ) then
        begin
                PROGRESSBAR(o)^.Steps := l_ulong(Value);
                if ( PROGRESSBAR(o)^.Promile > PROGRESSBAR(o)^.Steps ) then PROGRESSBAR(o)^.Promile := PROGRESSBAR(o)^.Steps;
                if ( Redraw ) then WidgetDraw(PWIDGET(o), NIL);
                ProgressSetRProp := true;
        end;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
function ProgressGetRProp ( o: PWidget; Name: l_text ): Pointer;
begin
        if ( TextCompare(Name,'value') = 0 ) then ProgressGetRProp := DuplicateInt((PROGRESSBAR(o)^.Promile));
        if ( TextCompare(Name,'max') = 0 ) then ProgressGetRProp := DuplicateInt((PROGRESSBAR(o)^.Steps));
        ProgressGetRProp := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)

function LibMain (Args: l_text): l_bool;
var
  p: PProgressBar;
  r: TRect;
begin
        ProgressClass := CreateWidgetClass( ULONG_ID('P','r','g','s'), StdWidgetClass, 0 );
        ProgressClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateProgress;
        ProgressClass^.SetRProp := {$ifdef fpc}@{$endif}ProgressSetRProp;
        ProgressClass^.GetRProp := {$ifdef fpc}@{$endif}ProgressGetRProp;

        WClassRegisterProp(ProgressClass,PTYPE_INT,'value');
        WClassRegisterProp(ProgressClass,PTYPE_INT,'max');

        (**
        *       Skin items sizes
        *)
        SkinBar         := GetSkinItem('ProgressBar');
        SkinBG          := GetSkinItem('ProgressBackground');
        (*
        APPEXPORT(CreateProgressBar);
        SYSEXPORT(CreateProgressBar);
        *)
(*        TRect r;
        RectAssign(r, 50,50,250,70);
        PProgressBar p := CreateProgressBar(Me, r, 10);
        p^.Promile := 0;
        InsertWidget(DeskTop, WIDGET(p));
        WidgetDraw(WIDGET(p), NIL); *)

        LibMain := true;
end;

procedure Close;
begin

end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion  := ULONG_ID(0, 9, 0, 0);
  SkinBar     := NIL;
  SkinBG      := NIL;
  
{$endif}  {LINK_CLIBS}

end.
  
