(*
////////////////////////////////////////////////////////////////////////////////
//
//      Resource files - DMS V2 - Core file
//
//      (c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
unit recource;

interface

uses ctypes, Types, guitypes, classes, vfile, list, dms, math;

{$define DMS2}

{..#include "kernel.h"}
{..#include <string.h>}

{$i cdefs.inc}
{$ifdef LINK_CLIBS}
  {$link resource.o}
{$endif}

(*////////////////////////////////////////////////////////////////////////////////*)


type
  TResEntry = packed record
	Name: array[0..31] of char;
	DataType: l_ulong;
	DataEncode: l_ulong;
	DataSize: l_ulong;
  end;

type 
  TResHead = packed record
	Magic         : l_ulong;
	FormatVersion : l_ulong;
	Entries       : l_ulong;
  end;


function	LoadRessourceFile	( aFile: l_text ): PList; {$ifdef LINK_CLIBS}external;{$endif}
function	SaveRessourceFile	( aFile: l_text; Data: PList ): l_bool; {$ifdef LINK_CLIBS}external;{$endif}
procedure	DMSWriteRessource	( f: PFileStruct; Lst: PList; var Entries: l_ulong ); {$ifdef LINK_CLIBS}external;{$endif}
function	DMSLoadRessource	( f: PFileStruct; Entries: l_ulong ): PList; {$ifdef LINK_CLIBS}external;{$endif}

function GetDynLdIcon ( f: PFileInfo ): PIcon; {$ifdef LINK_CLIBS}external;{$endif}
function GetDynLdIconEx ( aFile: l_text ): PIcon; {$ifdef LINK_CLIBS}external;{$endif}

(*////////////////////////////////////////////////////////////////////////////////*)

implementation
{$ifndef LINK_CLIBS}
uses
  debug, text, dynld;

function DMSLoadRessource ( f: PFileStruct; Entries: l_ulong ): PList;
var
  //PList Lst = NewList();
  Lst: PList;
  E: TResEntry;
  //M: PCodec;
  M: PSubCodec;
  O: l_ulong;
  i: PListItem;
  Data: Pointer;
  mng: PDataTypeMan;
  fr: Pointer;
begin
  if ( Lst=nil ) then DMSLoadRessource := nil;
  {$ifdef TEST}
    DebugMessage ('LoadRessource :: Starting loading 0x%08x from file 0x%08x',[Lst,f]);
  {$endif}
  while ( Entries > 0 ) do
  begin
    FileRead(E,sizeof(TResEntry),1,f);
    //M := FoundCodecByEncoder(E.DataType,E.DataEncode);
    M := GetSubCodec(E.DataEncode,E.DataType);
    if assigned( M ) then
    begin
            FileGetPos(f,O);
            O := O + E.DataSize;
            if ( M^.Load(f, Data{ *,(E.DataSize)* }, O,M^.DefaultExtraLoad^,NIL) ) then
            begin
              mng := GetDataTypeMan(E.DataType);
              fr := NIL;
              if assigned( mng ) then fr := mng^.Free;
              i :=  NewListItemEx(E.Name,Data,TFreeProc(fr),E.DataType,0{E.DataSize});
              ListAddItem(Lst,i);
              {$ifdef Debug}
                 DebugMessage ('LoadRessource :: 0x%08x :: %d '%s' loaded, %s',[Lst,Entrys,E.Name,Data]);
              {$endif}
            end else begin
              DebugError ('LoadRessource :: 0x%08x :: DataType/DataEncode do not support loading from ressource file ! : 0x%08x/0x%08x',[Lst,E.DataType,E.DataEncode]);
          end;
          DebugError ('LoadRessource :: 0x%08x :: Unknow DataType/DataEncode : 0x%08x/0x%08x',[Lst,E.DataType,E.DataEncode]);
          Dec(Entries);
    end;
    DMSLoadRessource := Lst;
  end;
end;

function DMSLoadSpecificRessource ( f: PFileStruct; Entrys: l_ulong; Name: l_text; aType: l_ulong ) : Pointer;
var
  E: TResEntry;
  M: PSubCodec;
  O: l_ulong;
  Data: Pointer;
begin        
        while ( Entrys > 0 ) do
        begin
                FileRead(&E,sizeof(TResEntry),1,f);

                if ( TextCaseCompare(E.Name,Name) > 0 ) then
                begin
                        FileGetPos(f,O);
                        O := O + E.DataSize;
                        FileSetPos(f,O);
                end else 
                begin
                        if ( E.DataType <> aType ) then DMSLoadSpecificRessource := NIL;

                        M := GetSubCodec(E.DataEncode,E.DataType);
                        if Assigned( M ) then
                        begin
                                FileGetPos(f,O);
                                O := O + E.DataSize;
                                if ( M^.Load(f,Data,O,M^.DefaultExtraLoad^,NIL) ) then begin
                                        DMSLoadSpecificRessource := Data;
                                end;
                        end;
                end;
                Dec(Entrys);
        end;
        DMSLoadSpecificRessource := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DMSWriteRessource ( f: PFileStruct; Lst: PList; var Entries: l_ulong );
var 
  a,b: PListItem;
  E:TResEntry;
  {M: PCodec;}
  M: PSubCodec;

  O, St, En: l_ulong;
begin      
  O := 0;
  {$ifdef TEST}
     DebugMessage("WriteRessource :: 0x%08x :: START",Lst);
  {$endif}
  Entries := 0;
  if Assigned( Lst^.Last ) then
  begin
    a := Lst^.Last^.Next;
    b := Lst^.Last^.Next;

    repeat
       {M := DefaultRessourceCodec(a^.DataType);}
        M := GetDefaultSubCodec(a^.DataType);

        if Assigned( M ) then
        begin
          if Assigned( M^.Save ) then
          begin
            fillchar(E,sizeof(TResEntry),0);
            (*
               E.DataType := M^.DataTypeManaged;
               E.DataEncode := M^.DataEncodeManaged;
            *)
            E.DataType := M^.TypeId;
            E.DataEncode := M^.CodecId;
            move(a^.Key,E.Name,min(31,TextLen(a^.Key)));

            FileGetPos(f,En);
            FileWrite(E,sizeof(TResEntry),1,f);
            FileGetPos(f,St);
            if ( M^.Save(f,a^.Data{,a^.DataSize},O,M^.DefaultExtraSave^,NIL) ) then
            begin
              E.DataSize := O - St;
              {$ifdef TEST}
                 DebugMessage('WriteRessource :: Wrote... %d %d 0x%08x 0x%08x '%s', %s',[E.DataSize,a^.DataSize,E.DataType,E.DataEncode,E.Name, a^.Data]);
              {$endif}
              { fsetpos(f,En); }
              FileSeek(f, En, SEEK_SET);
              FileWrite(E,sizeof(TResEntry),1,f);
              { fsetpos(f,O); }
              FileSeek(f, O, SEEK_SET);
            end;
          end else DebugError ('WriteRessource :: 0x%08x :: DataType not supported : 0x%08x',[Lst,a^.DataType]);
        end;
        Inc(Entries);
        a := a^.Next;
    until ( a = b );
    {$ifdef Debug}
          DebugMessage('WriteRessource :: 0x%08x :: END',[Lst]);
    {$endif}
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function SaveRessourceFile ( afile: l_text; Data : PList  ): l_bool;
var
  f: PFileStruct;
  a: l_text;
  i: Integer;
  Head: TResHead;
  assigned_file: boolean;
  DynHead: TDynLdHeader;
begin
  assigned_file := true;
        i := system.pos('.',afile);
        if (i > 0) then
        begin
                if ( (TextCaseCompare(a,'.app')=0) or (TextCaseCompare(a,'.dl')=0 )) then
                begin
                        f := FileOpen(aFile,FSF_READ);
                        if not Assigned( f ) then SaveRessourceFile := false else
                           FileRead(DynHead, 1, sizeof(TDynLdHeader), f);
                        if ( DynHead.Magic <> ULONG_ID('D','n','L','d') ) then
                        begin
                                FileClose(f);
                                DebugError('DYNLD/RESSOURCE :: Application is not a valid DynLD.',[]);
                                SaveRessourceFile := false;
                                assigned_file := false;
                        end;
                        if ( DynHead.FileFormatVersion > DYNLDVERSION ) then
                        begin
                                FileClose(f);
                                DebugError('DYNLD/RESSOURCE :: Application has invalid version number of DynLD linker.',[]);
                                SaveRessourceFile := false;
                                assigned_file := false;
                        end;
                        if assigned_file then
                        begin
                          FileSetPos(f, DynHead.RessourceOffset);
                          DMSWriteRessource(f,Data, DynHead.RessourceEntries);
                          FileSeek(f, 0, SEEK_SET);
                          FileWrite(DynHead,sizeof(TDynLdHeader),1,f);
                          FileClose(f);
                          SaveRessourceFile := true;
                        end;
                end;
        end;

        f := FileOpen(aFile,FSF_WRITE);
        if not assigned( f ) then SaveRessourceFile := false else
        begin
          Head.Magic := ULONG_ID('D','M','S','R');
          Head.FormatVersion := ULONG_ID(0,0,0,1);
          FileWrite(Head,sizeof(TResHead),1,f);

          DMSWriteRessource(f,Data, Head.Entries);

          FileSeek(f, 0, SEEK_SET);
          FileWrite(Head,sizeof(TResHead),1,f);
          FileClose(f);

          SaveRessourceFile := true;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function LoadRessourceFile ( aFile: l_text ): PList;
var
  a: l_text;
  f: PFileStruct;
  Data: PList;
  Head: TResHead;
  load: Boolean;
  DynHead: TDynLdHeader;
begin        
  Load := true;
  Data := NIL;
  if (aFile <> '') then
  begin
        f := FileOpen(aFile, FSF_READ);

        if Assigned( f ) then

        if ( system.pos('.',afile) > 0 ) then
        begin
                if ( (TextCaseCompare(a,'.app')=0) or (TextCaseCompare(a,'.dl')=0) ) then
                begin
                        FileRead(&DynHead, 1, sizeof(TDynLdHeader), f);
                        if ( DynHead.Magic <> ULONG_ID('D','n','L','d') ) then
                        begin
                                FileClose(f);
                                DebugError('DYNLD/RESSOURCE :: Application is not a valid DynLD.',[]);
                                LoadRessourceFile := NIL; Load := false;
                        end;
                        if ( DynHead.FileFormatVersion > DYNLDVERSION ) then
                        begin
                                FileClose(f);
                                DebugError('DYNLD/RESSOURCE :: Application has invalid version number of DynLD linker.',[]);
                                LoadRessourceFile := NIL; Load := false;
                        end;
                        if Load then
                        begin
                        FileSetPos(f, DynHead.RessourceOffset);
                        Data := DMSLoadRessource(f,DynHead.RessourceEntries);
                        FileClose(f);
                        LoadRessourceFile := Data;
                        end;
                end;
        end;
        
        if Load then FileRead(Head,sizeof(TResHead),1,f);
        if ( Head.Magic = ULONG_ID('D','M','S','R') ) then
        begin
                if ( Head.FormatVersion = ULONG_ID(0,0,0,1) ) then
                begin
                        Data := DMSLoadRessource(f,Head.Entries);
                end;
        end;

        FileClose(f);

        LoadRessourceFile := Data;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetDynLdIconEx ( aFile: l_text ): PIcon;
var  
  Data: PList;
  f: PFileStruct;
  i16,i32,i48: p_bitmap;
  DynHead: TDynLdHeader;
begin
  Data := NIL;
  f := FileOpen(aFile, FSF_READ);
  if assigned( f ) then
     begin

        FileRead(&DynHead, 1, sizeof(TDynLdHeader), f);
        if ( DynHead.Magic <> ULONG_ID('D','n','L','d') ) then
        begin
                FileClose(f);
                DebugError('DYNLD/RESSOURCE :: Application is not a valid DynLD.',[]);
                GetDynLdIconEx := NIL;
        end;
        if ( DynHead.FileFormatVersion <> DYNLDVERSION ) then
        begin
                FileClose(f);
                DebugError('DYNLD/RESSOURCE :: Application has invalid version number of DynLD linker.',[]);
                GetDynLdIconEx := NIL;
        end;
        if (DynHead.RessourceEntries = 0) then
        begin
                FileClose(f);
                GetDynLdIconEx := NIL;
        end;

        FileSetPos(f,DynHead.RessourceOffset);
        i16 := DMSLoadSpecificRessource(f,DynHead.RessourceEntries,'ICON16',DTYPE_IMAGE);
        FileSetPos(f,DynHead.RessourceOffset);
        i32 := DMSLoadSpecificRessource(f,DynHead.RessourceEntries,'ICON32',DTYPE_IMAGE);
        FileSetPos(f,DynHead.RessourceOffset);
        i48 := DMSLoadSpecificRessource(f,DynHead.RessourceEntries,'ICON48',DTYPE_IMAGE);
        FileClose(f);
        if ( assigned(i16) or assigned(i32) or assigned(i48) ) then
                GetDynLdIconEx := NewIcon(i16,i32,i48);

        GetDynLdIconEx := NIL;
     end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetDynLdIcon ( f: PFileInfo ) : PIcon;
begin
  GetDynLdIcon := GetDynLdIconEx(f^.VPathName);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
{$i resource.inc}
{$endif}
end.
