(*
////////////////////////////////////////////////////////////////////////////////
//
//  Registry - Core File
//
//  Copyright (c) 2003 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
{
 .#include"kernel.h"
 .#include"registry.h"
 .#include"internal.h"
}
unit registry;

interface

uses
  guitypes, list, vfile, text;

const
  RDSizes: array[0..11] of shortint = (
   0, { RKT_NODATA }
   1, { RKT_CHAR   }
   1, { RKT_BOOL   }
   2, { RKT_SHORT  }
   4, { RKT_INT    }
   4, { RKT_LONG   }
   8, { RKT_BIG    }
   4, { RKT_FLOAT  }
   8, { RKT_DOUBLE }
   16,{ RKT_RECT   }
   4, { RKT_POINT  }
   0  { RKT_TEXT   }
);


var NREGISTRY_MAGIC: Longint;
 
type
  PRegKey = ^TRegKey;
  TRegKey = record

	 Name   : l_text;
	 aType  : byte;
	 Data   : Pointer;

	 Parent : PRegKey;
	 Last   : PRegKey;
	 Next   : PRegKey;
	 Prev   : PRegKey;

	 TmpId  : l_ulong;

	 __e    : byte; // Do not use, reserved.

  end;

const
  MainKey: PRegKey = NIL;

procedure FreeKey( o: PRegKey );


const
  RKT_NODATA  = $00;
  RKT_CHAR    = $01;
  RKT_BOOL    = $02;
  RKT_SHORT   = $03;
  RKT_INT     = $04;
  RKT_LONG    = $04;
  RKT_BIG     = $05;
  RKT_FLOAT   = $06;
  RKT_DOUBLE  = $07;
  RKT_RECT    = $08;
  RKT_POINT   = $09;
  RKT_TEXT    = $0A;
  RKT_LINK    = $0B;

type
  PRegHead = ^TRegHead;
  TRegHead = record
	Magic: l_ulong;
	Keys: l_ulong;
  end; 

procedure NRegistryInit;
procedure NRegistryUnInit;

procedure RegistryReLoad; // Do not save changes...
procedure RegistrySave;


function ResolveKey ( Name: l_text ): PRegKey;
function  KeyExists ( Name: l_text ): l_bool;
function NewKey ( Parent, Name: l_text ): Byte; { l_uchar }

procedure DeleteKey( Name: l_text );
procedure DeleteKeyEx( o: PRegKey );

function KeySetText ( Name: l_text; Val: l_text ): l_bool;
function KeyGetText ( Name: l_text; Def: l_text ): l_text;
function KeySetLong ( Name: l_text; Val: l_long ): l_bool;
function KeyGetLong ( Name: l_text; Def: l_long ): longint;

function KeySetChar ( Name: l_text; Val: l_char ): l_bool;
function KeyGetChar ( Name: l_text; Def: l_char ): char;
function KeySetBool ( Name: l_text; Val: l_bool ): l_bool;
function KeyGetBool ( Name: l_text; Def: l_bool ): l_bool;
function KeySetShort ( Name: l_text; Val: l_short ): l_bool;
function KeyGetShort ( Name: l_text; Def: l_short ): l_short;
function KeySetInt ( Name: l_text; Val: l_int ): l_bool;
function KeyGetInt ( Name: l_text; Def: l_int ): integer;
function KeySetBig ( Name: l_text; Val: l_ubig ): l_bool;
function KeyGetBig ( Name: l_text; Def: l_ubig ): l_ubig;

function KeySetNothing ( Name: l_text ): l_bool;

function GetParentKeyName ( Key: l_text ): l_text;
function RenameKey ( Key, NewName: l_text ): l_bool; { l_uchar }

procedure RegistryRunEntries ( Name: l_text );

procedure CreateKey ( key: l_text );
function GetOrCreateKey ( key: l_text ): PRegKey;

function GetColorFromHex ( Code: l_text ): integer;


implementation
{$IFNDEF LINK_CLIBS}
uses
  Objects, debug, Convert, dynld;
(*
////////////////////////////////////////////////////////////////////////////////
// Registry loader
////////////////////////////////////////////////////////////////////////////////
*)
function LoadRegistry: l_bool; 
var
	i,Tmp: l_ulong;
	Reg: PRegKey;
	f: PFileStruct; 
	Head: TRegHead;

begin
    i := 0; Reg := NIL;

    f := FileOpen('system/settings.dat', FSF_READ);

	if not assigned( f ) then LoadRegistry := false else
	begin

	if assigned( MainKey ) then FreeKey(MainKey);


	FileRead(Head,1,sizeof(Head),f);

	getmem(Reg, Head.Keys*sizeof(TRegKey));

	fillchar(Reg,Head.Keys*sizeof(TRegKey),0);

	while ( i < Head.Keys ) do
	begin

		Reg[i].Name := FileReadBinString(f);
		FileRead(Reg[i].aType,1,1,f);

		if ( Reg[i].aType = RKT_TEXT ) then
		  Reg[i].Data := NewStr(FileReadBinString(f));
        end;
        if ( Reg[i].aType = RKT_TEXT ) then
        begin
			FileRead(Tmp,1,4,f);
			if ( Tmp > 0 ) then Reg[i].Data := @Reg[Tmp-1];
		end else if ( Reg[i].aType > 0 ) then
		begin
			getmem(Reg[i].Data, RDSizes[Reg[i].aType]);
			FileRead(Reg[i].Data,1,RDSizes[Reg[i].aType],f);
		end;

		FileRead(Tmp,1,4,f);
		if (Tmp > 0) then Reg[i].Parent := @Reg[Tmp-1];
		FileRead(&Tmp,1,4,f);
		if (Tmp > 0) then Reg[i].Next := @Reg[Tmp-1];
		FileRead(&Tmp,1,4,f);
		if (Tmp > 0) then Reg[i].Prev := @Reg[Tmp-1];
		FileRead(&Tmp,1,4,f);
		if (Tmp > 0) then Reg[i].Last := @Reg[Tmp-1];
	  inc(i);
	end;

	MainKey := Reg;
	MainKey^.__e := 1;
	FileClose(f);
	LoadRegistry := true;
end;
(*
////////////////////////////////////////////////////////////////////////////////
// Registry saving functions
////////////////////////////////////////////////////////////////////////////////
*)
procedure GenerateTmpId ( o: PRegKey; var Id: l_ulong );
var a,b: PRegKey;
begin
	o^.TmpId := Id;
	inc(Id);
	if assigned( o^.Last ) then
	begin
	  a := o^.Last;
	  b := a;
	  repeat
		  GenerateTmpId(a,Id);
		  a := a^.Next;
	  until ( a = b );
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
const  Nothing: l_ulong = 0;
procedure WriteKey ( f: PFileStruct; o: PRegKey );
var a,b: PRegKey;
begin
	FileWriteBinString(f,o^.Name);
	FileWrite(o^.aType,1,1,f);
	if ( o^.aType = RKT_TEXT ) then
		FileWriteBinString(f,PString(o^.Data)^)
	else if ( o^.aType = RKT_LINK ) then
		FileWrite(PREGKEY(o^.Data)^.TmpId,1,4,f)
	else if ( o^.aType > 0 ) then
		FileWrite(o^.Data,1,RDSizes[o^.aType],f);
    {
	FileWrite(o^.Parent?&o^.Parent^.TmpId:&Nothing,1,4,f);
	FileWrite(o^.Next?&o^.Next^.TmpId:&Nothing,1,4,f);
	FileWrite(o^.Prev?&o^.Prev^.TmpId:&Nothing,1,4,f);
	FileWrite(o^.Last?&o^.Last^.TmpId:&Nothing,1,4,f);
	}
	if assigned( o^.Last ) then
	begin
	  a := o^.Last;
	  b := a;
	  repeat
		  WriteKey(f,a);
		  a := a^.Next;
	  until ( a = b );
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure SaveRegistry;
var	Head: TRegHead;
	f: PFileStruct;
begin
   f := FileOpen('system/settings.dat', FSF_WRITE);
   if not assigned(f) then
     DebugMessage('Cannot open xsystem.dat!',[])
   else begin
     Head.Magic := NREGISTRY_MAGIC;
     Head.Keys  := 1;
     GenerateTmpId(MainKey,Head.Keys);
     dec(Head.Keys);
     FileWrite(Head,1,sizeof(TRegHead),f);
     WriteKey(f,MainKey);
     FileClose(f);
   end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function ResolveKeyEx ( Name: l_text; o: PRegKey ): PRegKey;
var	S: l_text;
	a: PRegKey;
	b: PRegKey;
        i: integer;
        j: integer;
begin
        ResolveKeyEx := NIL;
        a := o^.Last;
        b := a;
	if ( o^.aType = RKT_LINK ) then o := PRegKey(o^.Data);
	{if ( *Name=='\\' || *Name == '/' ) Name++;}
        i := 1;
        while (Name[i]='\') or (Name[i]='/') do inc(i);
	if ( i = Length(Name) ) then ResolveKeyEx := o;
	if not assigned( a ) then ResolveKeyEx := NIL else
        begin
          S := Name; i := 1;
          while i <= Length(S) do
          begin

            if system.Pos(NetWDriveSep,S)  > 0 then j := i+2;
            if system.Pos(FileSeparator,S) > 0 then j := i+1;

            if (system.Pos(NetWDriveSep,S) > 0) or (S[i] = FileSeparator) then
            begin
	      repeat
	         if (TextSqNCaseCompare(system.Copy(Name,j,Length(Name)),system.Copy(a^.Name,j,Length(a^.Name)),Length(Name)-j) = 0 ) then
                 begin
                   ResolveKeyEx := ResolveKeyEx( system.Copy(S, j, Length(S)),a);
                   break;
                 end;
                 a := a^.Next;
	      until ( a = b );
	     {DebugMessage('Errrrr %s',[Name]);}
	      ResolveKeyEx := NIL;
            end;
	    inc(i);
          end;
        end;
        repeat
		if ( TextCompare(Name,a^.Name) = 0 ) then
                begin
                  ResolveKeyEx := a;
                  break;
                end;
                a := a^.Next;
        until ( a = b );
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function ResolveKey ( Name: l_text ): PRegKey;
var o: PRegKey;
begin
   ResolveKey := NIL;

   if ( DCkPt('ResolveKey.Name',PString(Name)) ) then
   begin
      o := ResolveKeyEx(Name,MainKey);
      if not assigned(o) then DebugWarning('REGISTRY : Unknow key %s', [Name]);
      ResolveKey := o;
   end;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeyExists ( Name: l_text ): l_bool;
begin
  KeyExists := Assigned(ResolveKey(Name));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeKey( o: PRegKey );
var a,b,n: PRegKey;
begin
	if ( o^.Data <> NIL ) and (o^.aType <> RKT_LINK) and (o^.aType <> RKT_NODATA) then freemem(o^.Data,o^.aType);
	if ( o^.Name <> '' ) then o^.Name := '';
	if ( o^.Last <> NIL) then
        begin
	  a := o^.Last;
	  b := a;
	  repeat
		  n := a^.Next;
		  FreeKey(a);
		  a := n;
	  until ( a = b );
        end;

	if ( o^.__e <> 0) then dispose(o);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DeleteKeyEx( o: PRegKey );
var p: PRegKey;
begin
	if assigned( o ) then
	if assigned( o^.Parent ) then p := o^.Parent;
	if ( o^.Next = o ) then begin
	  p^.Last := NIL;
        end
        else
        begin
	  o^.Next^.Prev := o^.Prev;
	  o^.Prev^.Next := o^.Next;
	  if ( p^.Last = o ) then p^.Last := o^.Prev;
        end;
	FreeKey(o);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DeleteKey( Name: l_text );
begin
  DeleteKeyEx(ResolveKey(Name));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function NewKeyEx ( p: PRegKey; Name: l_text ): PRegKey;
var o:	PRegKey;
begin
	if not assigned( p ) then NewKeyEx := NIL;
	if assigned( ResolveKeyEx(Name,p) ) then begin
		DebugWarning('REGISTRY : Key %s do already exists',[Name]);
		NewKeyEx := NIL;
        end;

	New(o);

        if not assigned( o ) then begin
		DebugError('REGISTRY : Not enough memory to create key %s',[Name]);
		NewKeyEx := NIL;
        end;

	fillchar(o,sizeof(TRegKey),0);
	o^.aType := RKT_NODATA;
	o^.Name := Name;
	if ( o^.Name = '' ) then
        begin
		dispose(o);
		DebugError('REGISTRY : Not enough memory to create key %s',[Name]);
		NewKeyEx := NIL;
	end;
	o^.Parent := p;
	if assigned( p^.Last ) then begin
		o^.Next := p^.Last^.Next;
		o^.Prev := p^.Last;
		p^.Last^.Next^.Prev := o;
		p^.Last^.Next := o;
		p^.Last := o;

        end else begin
	  o^.Next := o;
	  o^.Prev := o;
	  p^.Last := o;
       end;
       o^.__e := 1;

       NewKeyEx := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function NewKey ( Parent: l_text; Name: l_text ): l_uchar;
var p: PRegKey;
begin
	p := GetOrCreateKey(Parent);
	if not assigned(p) then NewKey := 0 else
	if assigned( NewKeyEx(p,Name) ) then NewKey := 1;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetOrCreateKeyEx ( o: PRegKey; Name: l_text ): PRegKey;
var	S,t: l_text; a,b: PRegKey;  i,j: integer;

begin
	a := o^.Last;
	b := a;

        i := 1; j := 1;

	if ( o^.aType = RKT_LINK ) then o := PRegKey(o^.Data);
	if system.Pos(NetWDriveSep , Name) > 0 then j := system.Pos(NetWDriveSep, Name) + 2;
        if system.Pos(FileSeparator, Name) > 0 then j := system.Pos(FileSeparator,Name) + 1;
	if (j >= Length(Name)) then GetOrCreateKeyEx := o;

	S := system.Copy(Name,j, Length(Name));
        while ( i <= Length(S) ) do
        begin
	  if system.Pos(NetWDriveSep, S) > 0 then i := i + 2;
          if system.Pos(FileSeparator,S) > 0 then i := i + 1;
          begin
	    if assigned( a ) then
            repeat
	      if ( TextSqNCaseCompare(system.Copy(Name,i, Length(Name)),a^.Name,succ(Length(S)-i)) = 0 )
                then GetOrCreateKeyEx := GetOrCreateKeyEx(a,system.Copy(S,i, Length(S)));
	      a := a^.Next;
	    until ( a = b );
	    t := system.Copy(Name,i,Length(Name));  { TextNDup(Name,S-Name); }
	    a := NewKeyEx(o,t);
	    GetOrCreateKeyEx := GetOrCreateKeyEx(a,system.Copy(S,i, Length(S)));
	  end;
	  inc(i);
        end;

        if assigned( a ) then
        repeat
	  if ( TextCompare(Name,a^.Name) = 0 ) then GetOrCreateKeyEx := a;
	  a := a^.Next;
	until ( a = b );

	GetOrCreateKeyEx := NewKeyEx(o,Name);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetOrCreateKey ( key: l_text ): PRegKey;
begin
	GetOrCreateKey := GetOrCreateKeyEx(MainKey,key);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure CreateKey ( key: l_text );
begin
	GetOrCreateKeyEx(MainKey,key);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function RenameKey ( Key, NewName: l_text ): l_bool;
var o: PRegKey;
begin
  o := ResolveKey(Key);
  if not assigned( o ) then RenameKey := false else
  begin
    if ( o^.Name <> '' ) then o^.Name := '';
    o^.Name   := NewName;
    RenameKey := true;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function _GetKeyName ( o: PRegKey ): l_text;
var KName,OName: l_text;
begin
	if not assigned( o^.Parent ) then begin
		_GetKeyName := FileSeparator;
        end else begin
		KName := '';

		while assigned( o^.Parent ) do
                begin
			OName := KName;
			KName := FileNameToPath(o^.Name,OName);
			OName := '';
			o := o^.Parent;
		end;
		_GetKeyName := KName;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetParentKeyName ( Key: l_text ): l_text;
var n: l_text;
    o: PRegKey;
begin
   o := ResolveKey(Key);
   if not assigned( o ) then GetParentKeyName := '' else
   if not assigned( o^.Parent ) then GetParentKeyName := '' else
   begin
     n := _GetKeyName(o^.Parent);
     DebugMessage('REGISTRY : %s parent is %s',[key,n]);
     GetParentKeyName := n;
   end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*////////////////////////////////////////////////////////////////////////////////*)
function KeySetData ( Name: l_text; DType: l_uchar; Data: Pointer ): l_bool;

var	o: PRegKey;

begin
        o := GetOrCreateKey(Name);

        if not assigned( o ) then KeySetData := false;
	if ( assigned(o^.Data) and (o^.aType <> RKT_LINK) ) then begin
		freemem(o^.Data, RDSizes[o^.aType]);
	  o^.Data := NIL;
	end;
	o^.aType := DType;
	if ( DType = RKT_LINK ) then begin
		o^.Data := Data;
        end else if ( DType > 0 ) then begin
		if ( DType = RKT_TEXT ) then begin
			o^.Data := PString(Data);
                end else begin
			getmem(o^.Data, RDSizes[DType]);
			move(Data^,o^.Data^,RDSizes[DType]);
		end;
		if not assigned( o^.Data ) then begin
			DebugError('REGISTRY : Not enough memory to set data into key %s',[Name]);
			KeySetData := false;
		end;
	end;

	KeySetData := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeyGetData( Name: l_text; DType: l_uchar; Def: Pointer ): Pointer;
var o: PRegKey;
begin
	o := ResolveKey(Name);
	if not assigned( o ) then KeyGetData := Def else
	if ( o^.aType <> DType ) then KeyGetData := Def else
	KeyGetData := o^.Data;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeySetText ( Name: l_text; Val: l_text ): l_bool;
begin
	KeySetText := KeySetData(Name,RKT_TEXT,PString(Val));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeySetNothing ( Name: l_text ): l_bool;
begin
	KeySetNothing := KeySetData(Name,0,NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeyGetText ( Name, Def: l_text ): l_text;
begin
	KeyGetText := PString(KeyGetData(Name,RKT_TEXT,PString(Def)))^;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeySetLong ( Name: l_text; Val: l_long ): l_bool;
type P_l_long = ^l_long;
begin
	KeySetLong := KeySetData(Name,RKT_LONG, @Val);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeyGetLong ( Name: l_text; Def: l_long ): l_long;
type P_l_long = ^l_long;
begin
	KeyGetLong := P_l_long(KeyGetData(Name,RKT_LONG,@Def))^;
end;
(*////////////////////////////////////////////////////////////////////////////////*)

function KeySetChar ( Name: l_text; Val: Char ): l_bool;
begin
	KeySetChar := KeySetData(Name,RKT_CHAR,@Val);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeyGetChar ( Name: l_text; def: char ): Char;
begin
	KeyGetChar :=  PChar(KeyGetData(Name,RKT_CHAR,@Def))^;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeySetBool ( Name: l_text; Val: l_bool ): l_bool;
type P_l_ulong = ^l_ulong;
begin
     KeySetBool := KeySetData(Name,RKT_BOOL, @Val);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeyGetBool ( Name: l_text; Def: l_bool ): l_bool;
type P_l_ulong = ^l_ulong;
begin
	KeyGetBool := l_bool(P_l_ulong(KeyGetData(Name,RKT_BOOL, @Def))^);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeySetShort ( Name:l_text; Val: l_short ): l_bool;
type P_l_short = ^l_short;
begin
       KeySetShort := KeySetData(Name,RKT_SHORT,@Val);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeyGetShort ( Name: l_text; Def: l_short ): l_short;
type P_l_short = ^l_short;
begin
	KeyGetShort := P_l_short(KeyGetData(Name,RKT_SHORT,@Def))^;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeySetInt ( Name: l_text; Val: l_int ): l_bool;
type PInteger = ^Integer;
begin
	KeySetInt := KeySetData(Name,RKT_INT,PInteger(Val));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeyGetInt ( Name: l_text; Def: l_int ): Integer;
type PInteger = ^Integer;
begin
	KeyGetInt := PInteger(KeyGetData(Name,RKT_INT,PInteger(Def)))^;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeySetBig ( Name: l_text; Val: l_ubig ): l_bool;
type P_l_ubig = ^l_ubig;
begin
	KeySetBig := KeySetData(Name,RKT_BIG,P_l_ubig(Val));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KeyGetBig ( Name: l_text; Def: l_ubig ): l_ubig;
type P_l_ubig = ^l_ubig;
begin
	KeyGetBig := P_l_ubig(KeyGetData(Name,RKT_BIG,P_l_ubig(Def)))^;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetColorFromHex ( Code: l_text ): Integer;
begin
	if ( Length(Code) = 6 )
          then GetColorFromHex := HexToLongInt(Code)
	 else GetColorFromHex := 0;
end;
(*////////////////////////////////////////////////////////////////////////////////*)

procedure RegistryRunEntries ( Name: l_text );
var o,a,b: PRegKey;
begin
   o :=	ResolveKey(Name);
   if assigned( o ) then
   begin
      DebugMessage('Run registry entries form %s',[Name]);
      if assigned( o^.Last ) then begin
          a := o^.Last^.Next;
	  b := a;
	  repeat
	    if ( a^.aType = RKT_TEXT ) then
            DebugMessage('->Run %s...', [l_text(a^.Data)]);
	    DynLdRun(l_text(a^.Data),'');
	    a := a^.Next;
	  until ( a = b );
      end;
   end;
end;

procedure RegistryReLoad;
begin
	DebugMessage('ReLoading registry...',[]);
	LoadRegistry;
	DebugMessage('Done.',[]);
end;

procedure RegistrySave;
begin
	DebugMessage('Saving registry...',[]);
	SaveRegistry;
	DebugMessage('Done.',[]);
end;

procedure NRegistryInit;
var
 	Generate: l_bool;
begin
        Generate := false;

	DebugMessage('Loading registry...',[]);

	if ( KernelMode <> KERNEL_RESETUP ) then
        begin
		if ( not LoadRegistry ) then Generate := true;
	end else
		Generate := true;

	if ( Generate ) then
        begin

		DebugMessage('Registry file not found. Set default registry',[]);
		New(MainKey);
		if not assigned(MainKey) then
                begin
		  DebugFatal('REGISTRY: Not enough memory',[]);
                  exit;
                end;
		fillchar(MainKey,sizeof(TRegKey),0);
		MainKey^.Name := '';

                KeySetText('/SYSTEM/LIBRARIES/jpg','xlib/jpg.dl');
		KeySetText('/SYSTEM/LIBRARIES/bmp','xlib/bmp.dl');
		KeySetText('/SYSTEM/LIBRARIES/png','xlib/png.dl');
		KeySetText('/SYSTEM/LIBRARIES/grfx','xlib/grfx.dl');
		KeySetText('/SYSTEM/LIBRARIES/clipbrd','xlib/clipbrd.dl');
		KeySetText('/SYSTEM/LIBRARIES/skin','xlib/skin.dl');
		KeySetText('/SYSTEM/LIBRARIES/widget','xlib/widget.dl');
		KeySetText('/SYSTEM/LIBRARIES/button','xlib/button.dl');
		KeySetText('/SYSTEM/LIBRARIES/label','xlib/label.dl');
		KeySetText('/SYSTEM/LIBRARIES/progress','xlib/progress.dl');
		KeySetText('/SYSTEM/LIBRARIES/checkbox','xlib/checkbox.dl');
		KeySetText('/SYSTEM/LIBRARIES/menu','xlib/menu.dl');
		KeySetText('/SYSTEM/LIBRARIES/canvas','xlib/canvas.dl');
		KeySetText('/SYSTEM/LIBRARIES/scrllbar','xlib/scrllbar.dl');
		KeySetText('/SYSTEM/LIBRARIES/textbox','xlib/textbox.dl');
		KeySetText('/SYSTEM/LIBRARIES/window','xlib/window.dl');
		KeySetText('/SYSTEM/LIBRARIES/listbox','xlib/listbox.dl');
		KeySetText('/SYSTEM/LIBRARIES/treeview','xlib/treeview.dl');
		KeySetText('/SYSTEM/LIBRARIES/groupbox','xlib/groupbox.dl');
		KeySetText('/SYSTEM/LIBRARIES/slider','xlib/slider.dl');
		KeySetText('/SYSTEM/LIBRARIES/listview','xlib/listview.dl');
		KeySetText('/SYSTEM/LIBRARIES/combobox','xlib/combobox.dl');
		KeySetText('/SYSTEM/LIBRARIES/toolbar','xlib/toolbar.dl');
		KeySetText('/SYSTEM/LIBRARIES/fms2','xlib/fms2.dl');
		KeySetText('/SYSTEM/LIBRARIES/iodlg','xlib/iodlg.dl');
		KeySetText('/SYSTEM/LIBRARIES/tabbook','xlib/tabbook.dl');
		KeySetText('/SYSTEM/LIBRARIES/cp','xlib/cp.dl');
		KeySetText('/SYSTEM/LIBRARIES/sound','xlib/sound.dl');
		KeySetText('/SYSTEM/LIBRARIES/mp3','xlib/mp3.dl');
		KeySetText('/SYSTEM/LIBRARIES/memfile','xlib/memfile.dl');

		CreateKey('/SYSTEM/BASICDRIVERS/bmp');
		CreateKey('/SYSTEM/BASICDRIVERS/png');
		CreateKey('/SYSTEM/BASICDRIVERS/jpg');

		CreateKey('/SYSTEM/DRIVERS/grfx');
		CreateKey('/SYSTEM/DRIVERS/clipbrd');
		CreateKey('/SYSTEM/DRIVERS/sound');

		CreateKey('/SYSTEM/AUTOLIB/skin');
		CreateKey('/SYSTEM/AUTOLIB/widget');
		CreateKey('/SYSTEM/AUTOLIB/button');
		CreateKey('/SYSTEM/AUTOLIB/label');
		CreateKey('/SYSTEM/AUTOLIB/progress');
		CreateKey('/SYSTEM/AUTOLIB/checkbox');
		CreateKey('/SYSTEM/AUTOLIB/menu');
		CreateKey('/SYSTEM/AUTOLIB/canvas');
		CreateKey('/SYSTEM/AUTOLIB/scrllbar');
		CreateKey('/SYSTEM/AUTOLIB/textbox');
		CreateKey('/SYSTEM/AUTOLIB/window');
		CreateKey('/SYSTEM/AUTOLIB/listbox');
		CreateKey('/SYSTEM/AUTOLIB/treeview');
		CreateKey('/SYSTEM/AUTOLIB/groupbox');
		CreateKey('/SYSTEM/AUTOLIB/slider');
		CreateKey('/SYSTEM/AUTOLIB/listview');
		CreateKey('/SYSTEM/AUTOLIB/combobox');
		CreateKey('/SYSTEM/AUTOLIB/toolbar');
		CreateKey('/SYSTEM/AUTOLIB/fms2');
		CreateKey('/SYSTEM/AUTOLIB/iodlg');
		CreateKey('/SYSTEM/AUTOLIB/tabbook');
		CreateKey('/SYSTEM/AUTOLIB/cp');


		KeySetText('/SYSTEM/STARTUP/desktop','xapps/desktop.app');


		KeySetInt('/SYSTEM/MOUSE/speed', 0);
		KeySetInt('/SYSTEM/MOUSE/dblclk', 4);
		KeySetText('/SYSTEM/MOUSE/CURSORS/arrow','SYSTEM/CURSORS/ARROW.BMP');
		KeySetText('/SYSTEM/MOUSE/CURSORS/resize','SYSTEM/CURSORS/RESIZE.BMP');

		KeySetInt('/SYSTEM/SCREEN/WIDTH',1024);
		KeySetInt('/SYSTEM/SCREEN/HEIGHT',768);
		KeySetInt('/SYSTEM/SCREEN/DEPTH',32);
		KeySetInt('/SYSTEM/SCREEN/REFRESH',80);

		KeySetText('/SYSTEM/KEYBOARD/LAYOUTS/France','SYSTEM/KEYBOARD/fr.okl');

		KeySetText('/SYSTEM/FONTS','SYSTEM/FONTS/HELV13.FNT');

		CreateKey('/USER/STARTUP');

		KeySetText('/USER/DESKTOP/color', '3D61AD');
		KeySetInt ('/USER/DESKTOP/alignment', 0);
		KeySetText('/USER/DESKTOP/wallpaper', './DESKTOP/start.bmp');
		KeySetText('/USER/DESKTOP/theme', './DESKTOP/plex.ini');

		CreateKey('/USER/GUI');
		KeySetText('/USER/GUI/SKIN', '(none)');
		KeySetText('/USER/GUI/SKINS/CLASSIX', 'classix.ini');
		KeySetText('/USER/GUI/SKINS/NONE', '');
		KeySetText('/USER/GUI/SKINPATH', './SYSTEM/SKINS/');


		KeySetText('/USER/COLOR/OSD', '888888');

		CreateKey('/SYSTEM/FMS');
		CreateKey('/SYSTEM/DMS');

		KeySetInt('/SYSTEM/DMS/Extentions/ln', $00000007);
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/00000007','Links');
		KeySetInt('/SYSTEM/DMS/Codecs/Codecs/00000007/FavoriteType',$00000007);
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/00000007/text','GenericText');
		KeySetInt('/SYSTEM/DMS/Codecs/Codecs/00000007/link',$00000007);


		KeySetInt('/SYSTEM/DMS/Extentions/dl', $44796e4c);
		KeySetInt('/SYSTEM/DMS/Extentions/app', $44796e4c);

		KeySetText('/SYSTEM/DMS/Codecs/Codecs/44796e4c','DynLd executales');
		KeySetInt('/SYSTEM/DMS/Codecs/Codecs/44796e4c/virtual',$44796e4c);
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/44796e4c/binary','GenericBinary');
		KeySetInt('/SYSTEM/DMS/Codecs/Codecs/44796e4c/FavoriteType',$44796e4c);

		KeySetInt('/SYSTEM/DMS/Extentions/exe', $44457865);
		KeySetInt('/SYSTEM/DMS/Extentions/com', $44457865);
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/44457865','Dos executable');
		KeySetInt('/SYSTEM/DMS/Codecs/Codecs/44457865/virtual',$44457865);
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/44457865/binary','GenericBinary');
		KeySetInt('/SYSTEM/DMS/Codecs/Codecs/44457865/FavoriteType',$44457865);

		KeySetText('/SYSTEM/FMS/44457865', 'Dos execultable');
		KeySetText('/SYSTEM/FMS/44457865/Icon16', '/system/SYSTEM/ICONS/app16.bmp');
		KeySetText('/SYSTEM/FMS/44457865/Icon32', '/system/SYSTEM/ICONS/app32.bmp');

		CreateKey ('/SYSTEM/FMS/44457865/Actions');
		KeySetText('/SYSTEM/FMS/44457865/Actions/Execute', '/system/xapps/rundos.app');
		KeySetInt ('/SYSTEM/FMS/44457865/Actions/Execute/FavoriteWeight', $FFFFFFF);

		KeySetInt('/SYSTEM/DMS/Extentions/&folder', $44697220);
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/44697220','Directory (VFile)');
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/44697220/Icon16', '/system/SYSTEM/ICONS/folder16.bmp');
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/44697220/Icon32', '/system/SYSTEM/ICONS/folder32.bmp');
		KeySetInt('/SYSTEM/DMS/Codecs/Codecs/44697220/virtual',$44697220);
		KeySetInt('/SYSTEM/DMS/Codecs/Codecs/44697220/FavoriteType',$44697220);


		KeySetInt('/SYSTEM/DMS/Extentions/&hdd', $48444420);
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/48444420','Hard Drive (VFile)');
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/48444420/Icon16', '/system/SYSTEM/ICONS/drive16.bmp');
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/48444420/Icon32', '/system/SYSTEM/ICONS/drive32.bmp');
		KeySetInt('/SYSTEM/DMS/Codecs/Codecs/48444420/virtual',$44697220);
		KeySetInt('/SYSTEM/DMS/Codecs/Codecs/48444420/FavoriteType',$44697220);

		KeySetText('/SYSTEM/FMS/44697220', 'Directory');
		KeySetText('/SYSTEM/FMS/44697220/Icon16', '/system/SYSTEM/ICONS/folder16.bmp');
		KeySetText('/SYSTEM/FMS/44697220/Icon32', '/system/SYSTEM/ICONS/folder32.bmp');

		CreateKey ('/SYSTEM/FMS/44697220/Actions');
		KeySetText('/SYSTEM/FMS/44697220/Actions/Open', '/system/xapps/nav.app');
		KeySetInt ('/SYSTEM/FMS/44697220/Actions/Open/FavoriteWeight', $FFFFFFF);

		KeySetText('/SYSTEM/FMS/00000003', 'Image');
		KeySetText('/SYSTEM/FMS/00000003/Icon16', '/system/SYSTEM/ICONS/img16.bmp');
		KeySetText('/SYSTEM/FMS/00000003/Icon32', '/system/SYSTEM/ICONS/img32.bmp');

		CreateKey ('/SYSTEM/FMS/00000003/Actions');
		KeySetText('/SYSTEM/FMS/00000003/Actions/Open', '/system/xapps/immagine.app');
		KeySetInt ('/SYSTEM/FMS/00000003/Actions/Open/FavoriteWeight', $FFFFFFF);

		KeySetInt('/SYSTEM/DMS/Extentions/txt', $00000001);
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/00000001','Text file');
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/00000001/text','GenericText');
		KeySetText('/SYSTEM/DMS/Codecs/Codecs/00000001/binary','GenericBinary');
		KeySetInt('/SYSTEM/DMS/Codecs/Codecs/00000001/FavoriteType',$00000001);

		KeySetText('/SYSTEM/FMS/00000001', 'Text');
		KeySetText('/SYSTEM/FMS/00000001/Icon16', '/system/SYSTEM/ICONS/txt16.bmp');
		KeySetText('/SYSTEM/FMS/00000001/Icon32', '/system/SYSTEM/ICONS/txt32.bmp');

		CreateKey ('/SYSTEM/FMS/00000001/Actions');
		KeySetText('/SYSTEM/FMS/00000001/Actions/Open', '/system/xapps/o3pad.app');
		KeySetInt ('/SYSTEM/FMS/00000001/Actions/Open/FavoriteWeight', $FFFFFFF);

		KeySetText('/SYSTEM/FMS/00000006', 'Ressource');
		KeySetText('/SYSTEM/FMS/00000006/Icon16', '/system/SYSTEM/ICONS/res16.bmp');
		KeySetText('/SYSTEM/FMS/00000006/Icon32', '/system/SYSTEM/ICONS/res32.bmp');
		CreateKey ('/SYSTEM/FMS/00000006/Actions');
		KeySetText('/SYSTEM/FMS/00000006/Actions/Open', '/system/xapps/resedit.app');
		KeySetInt ('/SYSTEM/FMS/00000006/Actions/Open/FavoriteWeight', $FFFF);

	end;
        (*
	SYSEXPORT(RegistryReLoad);
	SYSEXPORT(RegistrySave);
	SYSEXPORT(ResolveKey);
	SYSEXPORT(KeyExists);
	SYSEXPORT(NewKey);
	SYSEXPORT(DeleteKey);
	SYSEXPORT(DeleteKeyEx);
	SYSEXPORT(KeySetText);
	SYSEXPORT(KeyGetText);
	SYSEXPORT(KeySetLong);
	SYSEXPORT(KeyGetLong);
	SYSEXPORT(KeySetChar);
	SYSEXPORT(KeyGetChar);
	SYSEXPORT(KeySetBool);
	SYSEXPORT(KeyGetBool);
	SYSEXPORT(KeySetShort);
	SYSEXPORT(KeyGetShort);
	SYSEXPORT(KeySetInt);
	SYSEXPORT(KeyGetInt);
	SYSEXPORT(KeySetBig);
	SYSEXPORT(KeyGetBig);
	SYSEXPORT(KeySetNothing);
	SYSEXPORT(GetParentKeyName);
	SYSEXPORT(RenameKey);
	SYSEXPORT(CreateKey);
	SYSEXPORT(GetOrCreateKey);
	SYSEXPORT(GetColorFromHex);
        *)
	DebugMessage('Registry loaded',[]);
end;

procedure NRegistryUnInit;
begin
	DebugMessage('Saving registry...',[]);
	SaveRegistry;
	DebugMessage('Registry saved',[]);
	FreeKey(MainKey);
	DebugMessage('Registry freed',[]);
end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  NREGISTRY_MAGIC := LONG_ID('P','h','R','g');

{$ENDIF} {LINK_CLIBS}
end.
