(**
*	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
*
*	FILE:			screen.c
*
*	PROJECT:		Phoenix engine - Core
*
*	DESCRIPTION:	Screen initializations.
*
*	CONTRIBUTORS:
*					Lukas Lipka
*
*	TODO:			@
*
*	BUG:			@
*
*	MISTAKE:		@
*
*	IDEA:			@
*)
unit screen;
{
 .#include "kernel.h"
 .#include "internal.h"
}
interface

uses crt,guitypes,metrics,allegro,gdrivers,timer,image,debug;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
  {$LINK screen.o}
{$ENDIF}

var
 GSScreenWidth	   : integer	= 0;
 GSScreenHeight	   : integer	= 0;
 GSScreenDepth	   : Integer	= 0;
 GSScreenRefreshRate: integer	= 0;

 scrn: {$ifdef use_allegro}AL_BitmapPtr{$else}PCanvas{$endif};

const
  StartLogo: p_bitmap = NIL;
  QuitLogo : p_bitmap = NIL;
  FadeTime : {$ifdef OPascal}TDateTime{$else}word{$endif}  = 250;

var
  StartRect: TRect;
  QuitRect : TRect;
  ATime    : {$ifdef OPascal}TDateTime;{$else}word;{$endif}

(*======================================================================================*)


function GSSystemScreenInit (): integer;      {$ifdef LINK_CLIBS}external;{$endif}
function __ScreenRestartGraphics (): l_bool;  {$ifdef LINK_CLIBS}external;{$endif}
function __ScreenTextMode (): l_bool;         {$ifdef LINK_CLIBS}external;{$endif}



(*======================================================================================*)


implementation

{$ifndef LINK_C_LIBS}

{$ifdef OPascal}uses Sysutils,grfx;{$endif}

procedure ScreenCrazyFadeToBlack;
var t: {$ifdef OPascal}TDateTime;{$else}word{$endif} a,r: l_ulong;
begin
  t := {$ifdef OPascal}now{$else}ATime{$endif};
  randomize;
  a := 0;
  r:=random(6);
	
  if ( FadeTime <> 0 ) then
  begin
	
	al_set_clip_rect(scrn,0,0,GSScreenWidth, GSScreenHeight);
	al_set_clip_state(scrn, TRUE);
	
        al_acquire_bitmap(scrn);
    
	if ( r = 0 ) then 
	begin
    	while ( a < (GSScreenWidth shl 1) ) do
    	begin
			al_line(scrn,0,a,a,0,makecol(0,0,0));
			al_line(scrn,0,a+1,a+1,0,makecol(0,0,0));
			if ( ATime-t < a*FadeTime/(GSScreenWidth shl 1) ) then ThreadTimerWait(1);
			a:=a+2;	
		end;
		
	end else if ( r = 5 ) then
	begin
		a := GSScreenWidth shl 1;
		while ( a > 0 ) do
		begin
			al_line(scrn,0,a,a,0,makecol(0,0,0));
			al_line(scrn,0,a-1,a-1,0,makecol(0,0,0));
			if ( ATime-t < ((GSScreenWidth shl 1)-a)*FadeTime/(GSScreenWidth shl 1) ) then ThreadTimerWait(1);
			a:=a-2;	
		end;
		
	end else if ( r = 1 ) then
	begin
		while ( a < GSScreenWidth ) do
		begin
			al_vline(scrn,a,0,GSScreenHeight,makecol(0,0,0));
			if ( ATime-t < a*FadeTime/GSScreenWidth ) then ThreadTimerWait(1);
			Inc(a);	
		end
	end else if ( r = 2 ) then
		a := GSScreenWidth;
		while ( a > 0  ) do
		begin
			al_vline(scrn,a,0,GSScreenHeight,makecol(0,0,0));
			if ( ATime-t < (GSScreenWidth-a)*FadeTime/GSScreenWidth ) then ThreadTimerWait(1);
			Dec(a);	
		end;
	end else if ( r = 3 ) then
	begin
		a := GSScreenHeight;
		while ( a > 0 ) do
		begin
			al_hline(scrn,0,a,GSScreenWidth,makecol(0,0,0));
			if ( ATime-t < (GSScreenHeight-a)*FadeTime/GSScreenHeight ) then ThreadTimerWait(1);
			Dec(a);	
		end;		
	end else if ( r = 4 ) then
	begin
		while ( a < GSScreenHeight ) do
		begin
			al_hline(scrn,0,a,GSScreenWidth,makecol(0,0,0));
			if ( ATime-t < a*FadeTime/GSScreenHeight ) then ThreadTimerWait(1);
			Inc(a);	
		end;	
	end;
        al_release_bitmap(scrn);
	DebugMessage('Fade results : Fade n%d Time:%d msec',[r,ATime-t]);

end;



function __ScreenRestartGraphics: l_bool;
begin	
	al_set_color_depth(GSScreenDepth);
	al_request_refresh_rate(GSScreenRefreshRate);
	if not al_set_gfx_mode(AL_GFX_AUTODETECT, GSScreenWidth, GSScreenHeight, 0, 0) then
	begin
		DebugFatal('Unable to restart scrn...',[]);
		__ScreenRestartGraphics := false;
	end;
	__ScreenRestartGraphics := true;
end;

function __ScreenTextMode: l_bool;
begin	
	ScreenCrazyFadeToBlack;
	al_set_gfx_mode(AL_GFX_TEXT, 0, 0, 0, 0);

	__ScreenTextMode := true;
end;


function GSSystemScreenQuit: Integer;
begin
	QuitLogo := LoadImage('system/shutdown.png');

        RectAssign(
           QuitRect,
           (GSScreenWidth-QuitLogo^.w) div 2,
           (GSScreenHeight-QuitLogo^.h) div 2,
           ((GSScreenWidth-QuitLogo^.w) div 2)+QuitLogo^.w-1,
           ((GSScreenHeight-QuitLogo^.h) div 2)+QuitLogo^.h-1);
	al_set_clip_rect(scrn,QuitRect.a.x,QuitRect.a.y,QuitRect.b.x,QuitRect.b.y);
	al_set_clip_state(scrn, TRUE);
	al_draw_sprite(scrn,QuitLogo,QuitRect.a.x,QuitRect.a.y);
end;


(**
*	NAME:	GSSystemScreenInit
*	DESCRIPTION: Initializes the scrn and graphics mode using the values set
*					in the registry.
*	RETURN: 0 on success, else -1
*)
function GSSystemScreenInit: Integer;
var cc: Longint; c: char;
begin

	if ( KernelMode = KERNEL_RESETUP ) then
	begin
		c := #0;
		writeln('re-setup mode.');
		writeln('Please select your scrn configuration. You hardware may not support all listed modes.');
		writeln;
        writeln('Please select scrn definition :');
        writeln;
		writeln('1   640x480');
		writeln('2   800x600');
		writeln('3  1024x768');
		writeln('4  1152x864');
		writeln('5  1280x1024');

		while ( c = #0 ) do
		begin
			cc := (al_readkey and $FF);
                        c  := chr(byte(cc));
			if ( c = '1' ) then begin GSScreenWidth := 640;  GSScreenHeight := 480; end
			else if ( c = '2' ) then begin GSScreenWidth := 800;  GSScreenHeight := 600; end
            else if ( c = '3' ) then begin GSScreenWidth := 1024; GSScreenHeight := 768; end
			else if ( c = '4' ) then begin GSScreenWidth := 1152; GSScreenHeight := 864; end
			else if ( c = '5' ) then begin GSScreenWidth := 1280; GSScreenHeight := 1024; end
			else c := #0;
		end;

		writeln('Please select scrn color depth :');
		writeln;
		writeln('1  256 Colors  (8 bpp)');
		writeln('2  High Colors (15 bpp)');
		writeln('3  High Colors (16 bpp)');
		writeln('4  True Colors (24 bpp)');
		writeln('5  True Colors (32 bpp)');

		c := #0;

		while ( c <> #0 ) do
		begin
			cc := (al_readkey and $FF);
                        c  := Chr(byte(cc));
			if ( c = '1' ) then GSScreenDepth := 8
			else if ( c = '2' ) then GSScreenDepth := 15
			else if ( c = '3' ) then GSScreenDepth := 16
			else if ( c = '4' ) then GSScreenDepth := 24
			else if ( c = '5' ) then GSScreenDepth := 32
			else c := #0;
		end;

	end
	else if ( KernelMode = KERNEL_SAFE ) then
	begin
		GSScreenWidth 	:= 1024;
		GSScreenHeight 	:= 768;
		GSScreenDepth 	:= 16;
		GSScreenRefreshRate := 60;
	end
	else
	begin
		GSScreenWidth 		:= 640; {KeyGetInt('/SYSTEM/scrn/WIDTH',640);}
		GSScreenHeight 		:= 480; {KeyGetInt('/SYSTEM/scrn/HEIGHT',480);}
		GSScreenDepth 		:=   8; {KeyGetInt('/SYSTEM/scrn/DEPTH',8);}
		GSScreenRefreshRate	:=  60; {KeyGetInt('/SYSTEM/scrn/REFRESH',60);}
	end;


	DebugMessage('Initializing scrn by these parameters:',[]);
	DebugMessage('- Width:\t\t%i', [GSScreenWidth]);
	DebugMessage('- Height:\t\t%i', [GSScreenHeight]);
	DebugMessage('- Depth:\t\t%i', [GSScreenDepth]);
	DebugMessage('- Refresh Rate:\t\t%i', [GSScreenRefreshRate]);
	DebugMessage('Installing scrn...',[]);

	(**
	 * Set color depth
	 *)
	al_set_color_depth(GSScreenDepth);

	(**
	 * Request refresh rate
	 *)
	al_request_refresh_rate(GSScreenRefreshRate);

	if not al_set_gfx_mode(AL_GFX_AUTODETECT, GSScreenWidth, GSScreenHeight, 0, 0) then
	begin
		(**
		*	If user required mode fails, try a standard one
		*)
		al_message('Could not set requested graphics mode.\n');

		GSScreenWidth 	:= 640;
		GSScreenHeight 	:= 480;
		GSScreenDepth 	:= 8;

		al_set_color_depth(GSScreenDepth);
		if not al_set_gfx_mode(AL_GFX_AUTODETECT, GSScreenWidth, GSScreenHeight, 0, 0) then
		begin
			(**
			*	No graphic mode has been able to be set
			*)
			DebugFatal('Unable to initialise graphics mode',[]);
			GSSystemScreenInit := -1;
		end;
	end;

	if (al_get_refresh_rate <> GSScreenRefreshRate) then
	begin
		DebugError('Could not set requested refresh rate, using %iHz instead', [al_get_refresh_rate]);
	end;

	al_set_color_conversion(AL_COLORCONV_TOTAL or AL_COLORCONV_KEEP_TRANS);

	StartLogo := LoadImage('/system/start.png');


	if assigned( StartLogo ) then
	begin
		RectAssign(
                  StartRect,
                  (GSScreenWidth-StartLogo^.w) div 2,
                  (GSScreenHeight-StartLogo^.h) div 2,
                  ((GSScreenWidth-StartLogo^.w) div 2)+StartLogo^.w-1,
                  ((GSScreenHeight-StartLogo^.h) div 2)+StartLogo^.h-1);
		al_set_clip_rect(scrn,StartRect.a.x,StartRect.a.y,StartRect.b.x,StartRect.b.y);
		al_set_clip_state(scrn, TRUE);
		al_draw_sprite(scrn,StartLogo,StartRect.a.x,StartRect.a.y);

	end;
	

	GSSystemScreenInit := 0;
end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}

  GSScreenWidth		   := 0;
  GSScreenHeight	   := 0;
  GSScreenDepth		   := 0;
  GSScreenRefreshRate  := 0;

{$endif}

end.
