////////////////////////////////////////////////////////////////////////////////
//
//  Scrollbars - Core File
//
//	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
unit scrllbar;
{
 .#include "kernel.h"
 .#include "widget.h"
 .#include "scrollbar.h"
 .#include "widget.h"
}

interface

uses guitypes,metrics,dynld,events,mousedrv,widget,skin,grfx,allegro;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$LINK widget.o}
   {$LINK scrollbar.o}
{$endif}

type
  PScrollWidget = ^TScrollWidget;
  PScrollbar    = ^TScrollbar;
  ScrolledProc  = procedure(o: PScrollWidget; Pos: TPoint);
  TScrollWidget = record
	o:	TWidget;

	Scrolled: ScrolledProc;

	OldPos  : TPoint;
	Pos     : TPoint;
	RPos    : TPoint;
	Max     : TPoint;
	Delta   : TPoint;

        Vertical   : PScrollbar;
	Horizontal : PScrollbar;

  end;

  TScrollbar = record
	o        : TWidget;
	Scll     : PScrollWidget;
	aType    : l_uchar;
	BarCache : TRect;
	W        : l_ulong;
	BtnState : l_uchar;
  end;

  SLLWDG = PScrollWidget;
  SLLBAR = PScrollbar;

function InitScrollWidget( o:PScrollWidget; aClass:PWidgetClass; App:PApplication; r:TRect; aType:l_uchar ): PScrollWidget;

function ScrollWidgetEventHandler ( o:PWidget; Ev:PEvent ): l_bool;

procedure ScrollBarChangeType ( o:PScrollWidget; aType:l_uchar );
procedure ScrollBarChangeTypeUninserted ( o:PScrollWidget; aType:l_uchar );

procedure ScrollBarRedraw ( o:PScrollWidget );


procedure ScrollBarScrollTo ( i:PScrollWidget; Pos:TPoint );
procedure ScrollBarSetPosX ( i:PScrollWidget; Pos:l_ulong );
procedure ScrollBarSetPosY ( i:PScrollWidget; Pos:l_ulong );
procedure ScrollBarSetMaxX ( i:PScrollWidget; Max:l_ulong );
procedure ScrollBarSetMaxY ( i:PScrollWidget; Max:l_ulong );

var
  ScrollWidgetClass: PWidgetClass;

implementation
{$ifndef LINK_CLIBS}
{$ifdef OPascal}
uses math;
{$else}

{$endif}
var AppVersion: l_ulong;

const
 AppName: l_text = 'Scrollbar Widget';
 nUID   : l_text = 'scrllbar';
 NeededLibs: array[0..2] of String = ( 'widget', 'skin', '' );

var ScrollbarClass: PWidgetClass;

const
 SkinVScrollbarUpUp: PSkin		= NIL;
 SkinVScrollbarUpDown: PSkin		= NIL;
 SkinVScrollbarDownUp: PSkin		= NIL;
 SkinVScrollbarDownDown: PSkin  	= NIL;
 SkinVScrollbarBackground: PSkin	= NIL;
 SkinVScrollbarTracker: PSkin		= NIL;

 SkinHScrollbarLeftUp: PSkin		= NIL;
 SkinHScrollbarLeftDown: PSkin  	= NIL;
 SkinHScrollbarRightUp: PSkin		= NIL;
 SkinHScrollbarRightDown: PSkin  	= NIL;
 SkinHScrollbarBackground: PSkin	= NIL;
 SkinHScrollbarTracker: PSkin		= NIL;

 MINIMAL_BAR_SIZE = 10;

function SIZEWIDTH(o: PWidget): l_ulong;
begin
  SIZEWIDTH:=(o^.Relative.b.x-o^.Relative.a.x-(SkinHScrollbarLeftUp^.Skin^.w+SkinHScrollbarRightUp^.Skin^.w));
end;
function SIZEHEIGHT(o: PWidget): l_ulong;
begin
  SIZEHEIGHT:=(o^.Relative.b.y-o^.Relative.a.y-(SkinVScrollbarUpUp^.Skin^.h+SkinVScrollbarDownUp^.Skin^.h));
end;

function SIZEWIDTHNOSKIN(o: PWidget): l_ulong;
begin
  SIZEWIDTHNOSKIN := (o^.Relative.b.x-o^.Relative.a.x-30);
end;
function SIZEHEIGHTNOSKIN(o: PWidget): l_ulong;
begin
  SIZEHEIGHTNOSKIN := (o^.Relative.b.y-o^.Relative.a.y-30);
end;

(*
 #define SIZEWIDTH(o) SIZEWIDTH(o)
 #define SIZEHEIGHT(o) SIZEHEIGHT(o)
*)
////////////////////////////////////////////////////////////////////////////////
function GetBarSize ( aMax,Size: l_ulong ): l_ulong;
begin
  GetBarSize := max(MINIMAL_BAR_SIZE, Size*Size div (Size+1+aMax));
end;
////////////////////////////////////////////////////////////////////////////////
function GetBaRPos ( aMax, Size, Pos: l_ulong ): l_ulong;
begin
  GetBaRPos := (Size-GetBarSize(aMax,Size))*Pos div (1+aMax);
end;
////////////////////////////////////////////////////////////////////////////////
function CalcPos ( aMax, Size: l_ulong; PosPx: l_long ): l_ulong;
begin
  CalcPos := min(aMax, (1+aMax)*max(0,PosPx) div (Size-GetBarSize(aMax,Size)));
end;
////////////////////////////////////////////////////////////////////////////////
procedure NotifyScroll ( i:PScrollWidget );
begin
	if ( (i^.Pos.y <> i^.OldPos.y) or (i^.Pos.x <> i^.OldPos.x) ) then
        begin
	  if assigned( i^.Scrolled ) then
	    i^.Scrolled(i,i^.Pos);
        end;
	i^.OldPos := i^.Pos;
end;
////////////////////////////////////////////////////////////////////////////////
procedure VerticalScrollbarDraw ( o:PWidget; buffer:p_bitmap; w:PRect );
var
  i: PScrollWidget;
  r: TRect;
  size: l_ulong;
begin
  i := SLLBAR(o)^.Scll;
  if ( i^.RPos.y < 0 ) then i^.RPos.y := 0;

  if ( UseSkins ) then
  begin
    size := GetBarSize(
              i^.Max.y,
              o^.Absolute.b.y-
              o^.Absolute.a.y-(SkinVScrollbarUpUp^.Skin^.h+SkinVScrollbarDownUp^.Skin^.h)
             );

    if ( i^.RPos.y > SIZEHEIGHT(o)-size ) then
    i^.RPos.y := o^.Absolute.b.y-
                 o^.Absolute.a.y-size-
                 (SkinVScrollbarUpUp^.Skin^.h+SkinVScrollbarDownUp^.Skin^.h);

    DrawSkin(
       buffer,
       SkinVScrollbarBackground,
       o^.Absolute.a.x,
       o^.Absolute.a.y,
       o^.Absolute.b.x,
       o^.Absolute.b.y
    );
    RectAssign(r,
      o^.Absolute.a.x,
      o^.Absolute.a.y+SkinVScrollbarUpUp^.Skin^.h+i^.RPos.y,
      o^.Absolute.b.x,
      o^.Absolute.a.y+SkinVScrollbarUpUp^.Skin^.h+i^.RPos.y+size
    );
    SLLBAR(o)^.BarCache := r;
    DrawSkin(buffer, SkinVScrollbarTracker, r.a.x, r.a.y, r.b.x, r.b.y+1);
    if ( SLLBAR(o)^.BtnState = 1 ) then
	DrawSkin(
           buffer,
           SkinVScrollbarUpDown,
           o^.Absolute.a.x,
           o^.Absolute.a.y,
           o^.Absolute.b.x,
           o^.Absolute.a.y+SkinVScrollbarUpDown^.Skin^.h-1
        )
    else
	DrawSkin(
           buffer,
           SkinVScrollbarUpUp,
           o^.Absolute.a.x,
           o^.Absolute.a.y,
           o^.Absolute.b.x,
           o^.Absolute.a.y+SkinVScrollbarUpUp^.Skin^.h-1
        );
	if ( SLLBAR(o)^.BtnState = 2 ) then
           DrawSkin(
             buffer,
             SkinVScrollbarDownDown,
             o^.Absolute.a.x,o^.Absolute.b.y-SkinVScrollbarDownDown^.Skin^.h+1,
             o^.Absolute.b.x,o^.Absolute.b.y
           )
	else
	   DrawSkin(
             buffer,
             SkinVScrollbarDownUp,
             o^.Absolute.a.x,
             o^.Absolute.b.y-SkinVScrollbarDownUp^.Skin^.h+1,
             o^.Absolute.b.x,o^.Absolute.b.y
           );
  end else begin
		size := GetBarSize( i^.Max.y, o^.Absolute.b.y-o^.Absolute.a.y-30);
		if ( i^.RPos.y > SIZEHEIGHTNOSKIN(o)-size ) then
                        i^.RPos.y := o^.Absolute.b.y-o^.Absolute.a.y-size-30;
	  RectAssign(r,o^.Absolute.a.x,o^.Absolute.a.y+15+i^.RPos.y,o^.Absolute.b.x,o^.Absolute.a.y+15+i^.RPos.y+size);
		SLLBAR(o)^.BarCache:=r;
		al_rectfill(buffer,w^.a.x,w^.a.y,w^.b.x,w^.b.y,COL_3DFACE);
		Rect3D(buffer,r.a.x,r.a.y,r.b.x,r.b.y+1,COL_3DLIGHT,COL_3DDARK);
		if ( SLLBAR(o)^.BtnState = 1 ) then begin
			Rect3D(buffer,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.a.y+14,COL_3DDARK,COL_3DLIGHT);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+1,o^.Absolute.a.y+6,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+1,COL_3DTEXT);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2,o^.Absolute.a.y+7,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+2,COL_3DTEXT);	
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2-1,o^.Absolute.a.y+8,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+3,COL_3DTEXT);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2-2,o^.Absolute.a.y+9,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+4,COL_3DTEXT);	
		end else begin
			Rect3D(buffer,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.a.y+14,COL_3DLIGHT,COL_3DDARK);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2,o^.Absolute.a.y+5,(o^.Absolute.a.x+o^.Absolute.b.x) div 2,COL_3DTEXT);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2-1,o^.Absolute.a.y+6,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+1,COL_3DTEXT);	
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2-2,o^.Absolute.a.y+7,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+2,COL_3DTEXT);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2-3,o^.Absolute.a.y+8,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+3,COL_3DTEXT);	
		end;
		if ( SLLBAR(o)^.BtnState = 2 ) then begin
			Rect3D(buffer,o^.Absolute.a.x,o^.Absolute.b.y-14,o^.Absolute.b.x,o^.Absolute.b.y,COL_3DDARK,COL_3DLIGHT);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+1,o^.Absolute.b.y-4,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+1,COL_3DTEXT);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2,o^.Absolute.b.y-5,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+2,COL_3DTEXT);	
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2-1,o^.Absolute.b.y-6,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+3,COL_3DTEXT);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2-2,o^.Absolute.b.y-7,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+4,COL_3DTEXT);	
		end else begin
			Rect3D(buffer,o^.Absolute.a.x,o^.Absolute.b.y-14,o^.Absolute.b.x,o^.Absolute.b.y,COL_3DLIGHT,COL_3DDARK);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2,o^.Absolute.b.y-5,(o^.Absolute.a.x+o^.Absolute.b.x) div 2,COL_3DTEXT);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2-1,o^.Absolute.b.y-6,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+1,COL_3DTEXT);	
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2-2,o^.Absolute.b.y-7,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+2,COL_3DTEXT);
			al_hline(buffer,(o^.Absolute.a.x+o^.Absolute.b.x) div 2-3,o^.Absolute.b.y-8,(o^.Absolute.a.x+o^.Absolute.b.x) div 2+3,COL_3DTEXT);			
		end;
	end;
end;
////////////////////////////////////////////////////////////////////////////////
function SkinnedHeight(condition: l_bool; ValueA,ValueB: l_ulong): l_ulong;
begin
  if condition then SkinnedHeight := ValueA else SkinnedHeight := ValueB;
end;

function VerticalScrollbarEventHandler ( o:PWidget; Ev:PEvent ): l_bool;
var
  r: TRect;
  i: PScrollWidget;
  SizeHeightV: l_ulong;
  ButtonHeight: l_ulong;
begin
        VerticalScrollbarEventHandler := false;
	if (Ev^.aType = EV_MOUSE ) then
	begin
		i := SLLBAR(o)^.Scll;
		SizeHeightV := SIZEHEIGHTNOSKIN(o);
		ButtonHeight := 15;

		if ( UseSkins ) then begin
			SizeHeightV := SIZEHEIGHT(o);
			ButtonHeight := SkinVScrollbarUpUp^.Skin^.h;
		end;


		if ( i^.Max.y = 0 ) then VerticalScrollbarEventHandler := true;

		if ( (Ev^.Message = WEvMouseLDown) or (Ev^.Message = WEvMouseLDClk) ) then
		begin
			MasterWidget := o;

			if ( CURSOR_IN_RECT( SLLBAR(o)^.BarCache ) ) then
			begin
				SLLBAR(o)^.W := Mouse^.State.p.y-SLLBAR(o)^.BarCache.a.y;
                        end
                        else if ( Mouse^.State.p.y < o^.Absolute.a.y+ButtonHeight ) then
			begin
				SLLBAR(o)^.BtnState := 1;

				RectAssign(r,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.a.y+ButtonHeight-1);
				WidgetDraw(o, @r);
                        end
                        else if ( Mouse^.State.p.y > o^.Absolute.b.y-ButtonHeight ) then
			begin
				SLLBAR(o)^.BtnState := 2;

				RectAssign(r,o^.Absolute.a.x,o^.Absolute.b.y-ButtonHeight+1,o^.Absolute.b.x,o^.Absolute.b.y);
				WidgetDraw(o, @r);
			end;
		end
		else if (Ev^.Message = WEvMouseMove) and ((Mouse^.State.b and BUTTON_LEFT)=BUTTON_LEFT) and (SLLBAR(o)^.BtnState = 0)  then
		begin
			i^.Pos.y := CalcPos(i^.Max.y,SizeHeightV,Mouse^.State.p.y-SLLBAR(o)^.W-o^.Absolute.a.y-ButtonHeight);
			i^.RPos.y := Mouse^.State.p.y-SLLBAR(o)^.W-o^.Absolute.a.y-ButtonHeight;
			WidgetDraw(o, NIL);
			NotifyScroll(i);
		end
		else if ( Ev^.Message = WEvMouseLUp ) then
		begin
			MasterWidget := NIL;

			if ( SLLBAR(o)^.BtnState = 1 ) then
			begin
			  SLLBAR(o)^.BtnState := 0;
			  i^.Pos.y := max(0,i^.Pos.y-i^.Delta.y);
			  i^.RPos.y := GetBaRPos( i^.Max.y, SizeHeightV,i^.Pos.y);
			  WidgetDraw(o,NIL);
			  NotifyScroll(i);
			end
			else if ( SLLBAR(o)^.BtnState = 2 ) then
			begin
				SLLBAR(o)^.BtnState := 0;
				i^.Pos.y := min(i^.Max.y,i^.Pos.y+i^.Delta.y);
				i^.RPos.y := GetBaRPos( i^.Max.y, SizeHeightV,i^.Pos.y);
				WidgetDraw(o,NIL);
				NotifyScroll(i);
			end
			else if ( not CURSOR_IN_RECT( SLLBAR(o)^.BarCache ) ) then
			begin
				i^.Pos.y := CalcPos(i^.Max.y,SizeHeightV,Mouse^.State.p.y-o^.Absolute.a.y-ButtonHeight);
				i^.RPos.y := Mouse^.State.p.y-o^.Absolute.a.y-ButtonHeight;
				WidgetDraw(o,NIL);
				NotifyScroll(i);
	
			end;
		end;
		VerticalScrollbarEventHandler := true;
	end;
	if ( Ev^.aType = EV_MESSAGE ) then
		if (  Ev^.Message = MSG_NOTIFY_AUTO_RESIZE ) then begin
			i := SLLBAR(o)^.Scll;
			i^.RPos.y := GetBaRPos( i^.Max.y, SkinnedHeight(UseSkins,SIZEHEIGHT(o),SIZEHEIGHTNOSKIN(o)),i^.Pos.y);
			VerticalScrollbarEventHandler := true;
		end;

end;
////////////////////////////////////////////////////////////////////////////////
procedure HorizontalScrollbarDraw ( o:PWidget; buffer:p_bitmap; w:PRect );
var
  i: PScrollWidget;
  r: TRect;
  size: l_ulong;
begin
	i := SLLBAR(o)^.Scll;

	if ( i^.RPos.x < 0 ) then i^.RPos.x := 0;


	if ( UseSkins ) then begin
		size := GetBarSize( i^.Max.x, SIZEWIDTH(o));
		if ( i^.RPos.x > SIZEWIDTH(o)-size ) then i^.RPos.x := SIZEWIDTH(o)-size;
		DrawSkin(buffer, SkinHScrollbarBackground, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y);
		RectAssign(r,
                  o^.Absolute.a.x+SkinHScrollbarLeftUp^.Skin^.w+i^.RPos.x,
                  o^.Absolute.a.y,
                  o^.Absolute.a.x+SkinHScrollbarLeftUp^.Skin^.w+i^.RPos.x+size,
                  o^.Absolute.b.y
                );
		SLLBAR(o)^.BarCache := r;
		DrawSkin(buffer, SkinHScrollbarTracker, r.a.x, r.a.y, r.b.x+1, r.b.y);

		if ( SLLBAR(o)^.BtnState = 1 ) then
			DrawSkin(buffer,
                          SkinHScrollbarLeftDown,
                          o^.Absolute.a.x,
                          o^.Absolute.a.y,
                          o^.Absolute.a.x+SkinHScrollbarLeftDown^.Skin^.w-1,
                          o^.Absolute.b.y
                        )
		else
			DrawSkin(buffer,
                          SkinHScrollbarLeftUp,
                          o^.Absolute.a.x,
                          o^.Absolute.a.y,
                          o^.Absolute.a.x+SkinHScrollbarLeftUp^.Skin^.w-1,
                          o^.Absolute.b.y
                        );

		if ( SLLBAR(o)^.BtnState = 2 ) then
			DrawSkin(buffer,
                          SkinHScrollbarRightDown,
                          o^.Absolute.b.x-SkinHScrollbarRightDown^.Skin^.w+1,
                          o^.Absolute.a.y,
                          o^.Absolute.b.x,
                          o^.Absolute.b.y
                        )
		else
			DrawSkin(buffer,
                          SkinHScrollbarRightUp,
                          o^.Absolute.b.x-SkinHScrollbarRightUp^.Skin^.w+1,
                          o^.Absolute.a.y,
                          o^.Absolute.b.x,
                          o^.Absolute.b.y
                        );
	end else begin
		size := GetBarSize( i^.Max.x, SIZEWIDTHNOSKIN(o));
		if ( i^.RPos.x > SIZEWIDTHNOSKIN(o)-size ) then i^.RPos.x := SIZEWIDTHNOSKIN(o)-size;
		RectAssign(r,o^.Absolute.a.x+15+i^.RPos.x,o^.Absolute.a.y,o^.Absolute.a.x+15+i^.RPos.x+size,o^.Absolute.b.y);
		SLLBAR(o)^.BarCache := r;

		al_rectfill(buffer,w^.a.x,w^.a.y,w^.b.x,w^.b.y,COL_3DFACE);
		Rect3D(buffer,r.a.x,r.a.y,r.b.x+1,r.b.y,COL_3DLIGHT,COL_3DDARK);

		if ( SLLBAR(o)^.BtnState = 1 ) then begin
			Rect3D(buffer,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.a.x+14,o^.Absolute.b.y,COL_3DDARK,COL_3DLIGHT);
			al_vline(buffer,o^.Absolute.a.x+6,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+1,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+1,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.a.x+7,(o^.Absolute.a.y+o^.Absolute.b.y) div 2,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+2,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.a.x+8,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-1,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+3,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.a.x+9,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-2,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+4,COL_3DTEXT);
		end else begin
			Rect3D(buffer,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.a.x+14,o^.Absolute.b.y,COL_3DLIGHT,COL_3DDARK);
			al_vline(buffer,o^.Absolute.a.x+5,(o^.Absolute.a.y+o^.Absolute.b.y) div 2,(o^.Absolute.a.y+o^.Absolute.b.y) div 2,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.a.x+6,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-1,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+1,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.a.x+7,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-2,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+2,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.a.x+8,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-3,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+3,COL_3DTEXT);
		end;
		
		if ( SLLBAR(o)^.BtnState = 2 ) then begin
			Rect3D(buffer,o^.Absolute.b.x-14,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.b.y,COL_3DDARK,COL_3DLIGHT);
			al_vline(buffer,o^.Absolute.b.x-4,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+1,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+1,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.b.x-5,(o^.Absolute.a.y+o^.Absolute.b.y) div 2,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+2,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.b.x-6,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-1,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+3,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.b.x-7,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-2,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+4,COL_3DTEXT);
		end else begin
			Rect3D(buffer,o^.Absolute.b.x-14,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.b.y,COL_3DLIGHT,COL_3DDARK);
			al_vline(buffer,o^.Absolute.b.x-5,(o^.Absolute.a.y+o^.Absolute.b.y) div 2,(o^.Absolute.a.y+o^.Absolute.b.y) div 2,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.b.x-6,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-1,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+1,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.b.x-7,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-2,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+2,COL_3DTEXT);
			al_vline(buffer,o^.Absolute.b.x-8,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-3,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+3,COL_3DTEXT);

		end;
		
			
	end;

end;
////////////////////////////////////////////////////////////////////////////////
function HorizontalScrollbarEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
var
  SizeWidthx: l_ulong;
  ButtonWidth: l_ulong;
  i: PScrollWidget;
  r: TRect;
begin
	HorizontalScrollbarEventHandler := false;

        if (Ev^.aType = EV_MOUSE ) then
	begin
		i := SLLBAR(o)^.Scll;

		SizeWidthx := SIZEWIDTHNOSKIN(o);
		ButtonWidth := 15;

		if ( UseSkins ) then begin
			SizeWidthx := SIZEWIDTH(o);
			ButtonWidth := SkinVScrollbarUpUp^.Skin^.h;
		end;


		if ( i^.Max.x = 0 ) then HorizontalScrollbarEventHandler := true;

		if ( (Ev^.Message = WEvMouseLDown) or (Ev^.Message = WEvMouseLDClk) ) then
		begin
			MasterWidget := o;

			if ( CURSOR_IN_RECT( SLLBAR(o)^.BarCache ) ) then
			begin
				SLLBAR(o)^.W := Mouse^.State.p.x-SLLBAR(o)^.BarCache.a.x;
			end
			else if ( Mouse^.State.p.x < o^.Absolute.a.x+ButtonWidth ) then
			begin
				SLLBAR(o)^.BtnState := 1;

				RectAssign(r,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.a.x+ButtonWidth-1,o^.Absolute.b.y);
				WidgetDraw(o, @r);
			end
			else if ( Mouse^.State.p.x > o^.Absolute.b.x-ButtonWidth ) then
			begin
				SLLBAR(o)^.BtnState := 2;

				RectAssign(r,o^.Absolute.b.x-ButtonWidth+1,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.b.y);
				WidgetDraw(o, @r);
			end;
		end
		else if ( (Ev^.Message = WEvMouseMove) and ((Mouse^.State.b and BUTTON_LEFT) = BUTTON_LEFT) and (SLLBAR(o)^.BtnState = 0) ) then
		begin
			i^.Pos.x := CalcPos(i^.Max.x,SizeWidthx,Mouse^.State.p.x-SLLBAR(o)^.W-o^.Absolute.a.x-ButtonWidth);
			i^.RPos.x := Mouse^.State.p.x-SLLBAR(o)^.W-o^.Absolute.a.x-ButtonWidth;
			WidgetDraw(o,NIL);
			NotifyScroll(i);
		end
		else if ( Ev^.Message = WEvMouseLUp ) then
		begin
			MasterWidget := NIL;

			if ( SLLBAR(o)^.BtnState = 1 ) then
			begin
				SLLBAR(o)^.BtnState := 0;
				i^.Pos.x := max(0,i^.Pos.x-i^.Delta.x);
				i^.RPos.x := GetBaRPos( i^.Max.x, SizeWidthx,i^.Pos.x);
				WidgetDraw(o,NIL);
				NotifyScroll(i);
			end
			else if ( SLLBAR(o)^.BtnState = 2 ) then
			begin
				SLLBAR(o)^.BtnState := 0;
				i^.Pos.x := min(i^.Max.x,i^.Pos.x+i^.Delta.x);
				i^.RPos.x := GetBaRPos( i^.Max.x, SizeWidthx,i^.Pos.x);
				WidgetDraw(o,NIL);
				NotifyScroll(i);
			end
			else if ( not CURSOR_IN_RECT( SLLBAR(o)^.BarCache ) ) then
			begin
				i^.Pos.x := CalcPos(i^.Max.x,SizeWidthx,Mouse^.State.p.x-o^.Absolute.a.x-ButtonWidth);
				i^.RPos.x := Mouse^.State.p.x-o^.Absolute.a.x-ButtonWidth;

				WidgetDraw(o,NIL);
				NotifyScroll(i);
			end;
		end;
		HorizontalScrollbarEventHandler := true;
	end;

	if ( Ev^.aType = EV_MESSAGE ) then
		if (  Ev^.Message = MSG_NOTIFY_AUTO_RESIZE ) then begin
			i := SLLBAR(o)^.Scll;
			i^.RPos.x := GetBaRPos( i^.Max.x, SkinnedHeight(UseSkins,SIZEWIDTH(o),SIZEWIDTHNOSKIN(o)),i^.Pos.x);
			HorizontalScrollbarEventHandler := true;
		end;

end;
////////////////////////////////////////////////////////////////////////////////
function InitScrollbar( App: PApplication; r: TRect; Scll: PScrollWidget; aType: l_uchar ): PScrollbar;
var p: PScrollBar;
begin
  New(p);

  if not assigned(p) then InitScrollbar := NIL else
  begin
    fillchar(p, sizeof(TScrollbar), 0);

	p^.Scll	:= Scll;
        p^.aType := aType;

	IntialiseWidget(App, ScrollbarClass, PWIDGET(p), r);

	PWIDGET(p)^.Flags := PWIDGET(p)^.Flags or WFForceBuffer; // To prevent blicking as scrollbars are very ofen redrawn

	PWIDGET(p)^.FocusBack := PWIDGET(Scll);


    if ( aType = 1 ) then begin
	  PWIDGET(p)^.Draw := {$ifdef fpc}@{$endif}VerticalScrollbarDraw;
	  PWIDGET(p)^.EventHandler := {$ifdef fpc}@{$endif}VerticalScrollbarEventHandler;
    end else if ( aType = 2 ) then begin

	  PWIDGET(p)^.Draw := {$ifdef fpc}@{$endif}HorizontalScrollbarDraw;
	  PWIDGET(p)^.EventHandler := {$ifdef fpc}@{$endif}HorizontalScrollbarEventHandler;
    end;

    InitScrollBar := p;

  end;
end;
////////////////////////////////////////////////////////////////////////////////

(*
procedure TESTDraw ( o:PWidget; buffer:p_bitmap; w:PRect );
var lt: l_text;
begin
      {$ifdef OPascal}
	lt := Format('%d', SLLWDG(o)^.Pos.y);
      {$else}
        FormatStr(lt,'%d', SLLWDG(o)^.Pos.y);
      {$endif}
	al_rectfill(buffer, w^.a.x, w^.a.y, w^.b.x, w^.b.y, o^.BackgroundColor);

	al_textout(buffer, default_font, lt, o^.Absolute.a.x, o^.Absolute.a.y, makecol(255,255,255));

end;

procedure Scrolled ( o:PScrollWidget; Pos:TPoint );
begin

	WidgetDraw(PWIDGET(o), NIL);
end;
*)
////////////////////////////////////////////////////////////////////////////////
procedure ScrollWidgetSmartFlags ( o:PScrollWidget );
begin
	
	if ( PWIDGET(o)^.Flags and WF_AUTORESIZE ) = WF_AUTORESIZE then begin
		
		if assigned( o^.Vertical ) then
			PWIDGET(o^.Vertical)^.Flags := PWIDGET(o^.Vertical)^.Flags or
                        (PWIDGET(o)^.Flags and (WF_FIXEDHEIGHT or WF_MAGNETRIGHT or WF_MAGNETBOTTOM)) or WF_FIXEDWIDTH or WF_AUTORESIZE;
		
		if assigned( o^.Horizontal ) then
			PWIDGET(o^.Horizontal)^.Flags := PWIDGET(o^.Horizontal)^.Flags or
                        (PWIDGET(o)^.Flags and (WF_FIXEDWIDTH or WF_MAGNETRIGHT or WF_MAGNETBOTTOM)) or WF_FIXEDHEIGHT or WF_AUTORESIZE;
	end;

end;


////////////////////////////////////////////////////////////////////////////////
procedure ScrollWidgetSetup ( o:PWidget );
begin
	if assigned( SLLWDG(o)^.Vertical ) then begin
		InsertWidget(o^.Parent, PWIDGET(SLLWDG(o)^.Vertical));
        end;
	if assigned( SLLWDG(o)^.Horizontal ) then begin
		InsertWidget(o^.Parent, PWIDGET(SLLWDG(o)^.Horizontal));
        end;
	ScrollWidgetSmartFlags(SLLWDG(o));
end;
(*
procedure ScrollWidgetFreeEx ( o:PWidget );
begin
  if assigned( SLLWDG(o)^.Vertical ) then WidgetDispose(PWIDGET(SLLWDG(o)^.Vertical));
end;
*)
////////////////////////////////////////////////////////////////////////////////
procedure ScrollBarScrollTo ( i:PScrollWidget; Pos:TPoint );
begin
	if ( (Pos.y <> i^.Pos.y) and assigned(i^.Vertical) ) then begin
		i^.Pos.y := max(0,min(i^.Max.y,Pos.y));
		i^.RPos.y := GetBaRPos( i^.Max.y, SkinnedHeight(UseSkins,SIZEHEIGHT(PWIDGET(i^.Vertical)),SIZEHEIGHTNOSKIN(PWIDGET(i^.Vertical))),i^.Pos.y);
		WidgetDraw(PWIDGET(i^.Vertical),NIL);
        end;
	if ( (Pos.x <> i^.Pos.x) and assigned(i^.Horizontal) ) then begin
		i^.Pos.x := max(0,min(i^.Max.x,Pos.x));
		i^.RPos.x := GetBaRPos( i^.Max.x, SkinnedHeight(UseSkins,SIZEWIDTH(PWIDGET(i^.Horizontal)),SIZEWIDTHNOSKIN(PWIDGET(i^.Horizontal))),i^.Pos.x);
		WidgetDraw(PWIDGET(i^.Horizontal),NIL);
        end;
	NotifyScroll(i);
end;
////////////////////////////////////////////////////////////////////////////////
procedure ScrollBarSetPosY ( i: PScrollWidget; Pos: l_ulong );
begin
	i^.Pos.y := min(i^.Max.y,Pos);
	if assigned( i^.Vertical ) then i^.RPos.y :=
           GetBaRPos(i^.Max.y,SkinnedHeight(UseSkins,SIZEHEIGHT(PWIDGET(i^.Vertical)),SIZEHEIGHTNOSKIN(PWIDGET(i^.Vertical))),i^.Pos.y);
end;
////////////////////////////////////////////////////////////////////////////////
procedure ScrollBarSetPosX ( i:PScrollWidget; Pos:l_ulong );
begin
	i^.Pos.x := min(i^.Max.x,Pos);
	if assigned( i^.Horizontal ) then i^.RPos.x :=
           GetBaRPos( i^.Max.x, SkinnedHeight(UseSkins,SIZEWIDTH(PWIDGET(i^.Horizontal)),SIZEWIDTHNOSKIN(PWIDGET(i^.Horizontal))),i^.Pos.x);
end;
////////////////////////////////////////////////////////////////////////////////
procedure ScrollBarSetMaxY ( i:PScrollWidget; Max:l_ulong );
begin
	i^.Max.y := Max;
	i^.Pos.y := min(i^.Max.y,i^.Pos.y);
	if assigned( i^.Vertical ) then
	   i^.RPos.y := GetBaRPos(i^.Max.y,SkinnedHeight(UseSkins,SIZEHEIGHT(PWIDGET(i^.Vertical)),SIZEHEIGHTNOSKIN(PWIDGET(i^.Vertical))),i^.Pos.y);
end;
////////////////////////////////////////////////////////////////////////////////
procedure ScrollBarSetMaxX ( i:PScrollWidget; Max:l_ulong );
begin
	i^.Max.x := Max;
	i^.Pos.x := min(i^.Max.x,i^.Pos.x);
	if assigned( i^.Horizontal ) then
		i^.RPos.x := GetBaRPos(i^.Max.x, SkinnedHeight(UseSkins,SIZEWIDTH(PWIDGET(i^.Horizontal)),SIZEWIDTHNOSKIN(PWIDGET(i^.Horizontal))),i^.Pos.x);
end;
////////////////////////////////////////////////////////////////////////////////
procedure ScrollBarRedraw ( o:PScrollWidget );
begin
  if assigned( o^.Vertical ) then WidgetDraw(PWIDGET(o^.Vertical),NIL);
  if assigned( o^.Horizontal ) then WidgetDraw(PWIDGET(o^.Horizontal),NIL);
end;
////////////////////////////////////////////////////////////////////////////////
procedure _ScrollBarChangeType ( o:PScrollWidget; aType:l_uchar );
var
	r,rb: TRect;
	SWidth: l_ulong;
	SHeight: l_ulong;
begin
        r := PWIDGET(o)^.Relative;
        SWidth := 15;
        SHeight := 15;
	if ( UseSkins ) then begin
		SHeight := SkinHScrollbarBackground^.Skin^.h;
		SWidth := SkinVScrollbarBackground^.Skin^.w;
	end;

	if ( ((aType and 1)=1) and not assigned(o^.Vertical )) then r.b.x := r.b.x - SWidth+1;
	if ( ((aType and 2)=2) and not assigned(o^.Horizontal )) then r.b.y := r.b.y - SHeight+1;
	
	if ( ((aType and 1)<>1) and assigned(o^.Vertical) ) then r.b.x := r.b.x + SWidth+1;
	if ( ((aType and 2)<>2) and assigned(o^.Horizontal) ) then  r.b.y := r.b.y + SHeight+1;

	if ( aType and 1 ) = 1 then begin
		RectAssign(rb,r.b.x+2, r.a.y, r.b.x+SWidth+1, r.b.y);
		if not assigned( o^.Vertical ) then begin
			o^.Vertical := InitScrollbar(PWIDGET(o)^.AppOwner, rb, o, 1);
			InsertWidget(PWIDGET(o)^.Parent, PWIDGET(o^.Vertical));
		end else
			WidgetSetRect(PWIDGET(o^.Vertical),rb);
	end else if assigned( o^.Vertical ) then begin
		WidgetDispose(PWIDGET(o^.Vertical));
		o^.Vertical := NIL;
	end;

	if ( aType and 2 ) = 2 then begin
		RectAssign(rb,r.a.x,r.b.y+2,r.b.x,r.b.y+SHeight+1);
		if not assigned( o^.Horizontal ) then begin
			o^.Horizontal := InitScrollbar(PWIDGET(o)^.AppOwner,rb,o,2);
			InsertWidget(PWIDGET(o)^.Parent, PWIDGET(o^.Horizontal));
		end else
			WidgetSetRect(PWIDGET(o^.Horizontal),rb);
	end else if assigned( o^.Horizontal ) then begin
		WidgetDispose(PWIDGET(o^.Horizontal));
		o^.Horizontal := NIL;
	end;

	
	WidgetSetRect(PWIDGET(o),r);
	WidgetDrawAll(PWIDGET(o));
	ScrollBarRedraw(o);
	
	ScrollWidgetSmartFlags(SLLWDG(o));
end;
////////////////////////////////////////////////////////////////////////////////
procedure ScrollBarChangeTypeUninserted ( o:PScrollWidget; aType:l_uchar );
var
	r,rb: TRect;
	SWidth: l_ulong;
	SHeight: l_ulong;
begin
        r := PWIDGET(o)^.Relative;
        SWidth := 15;
        SHeight := 15;

	if ( UseSkins ) then begin
		SHeight := SkinHScrollbarBackground^.Skin^.h;
		SWidth := SkinVScrollbarBackground^.Skin^.w;
	end;
	
	if ( ((aType and 1)=1) and not assigned(o^.Vertical) ) then r.b.x := r.b.x - SWidth+1;
	if ( ((aType and 2)=2) and not assigned(o^.Horizontal) ) then r.b.y := r.b.y - SHeight+1;
	
	if ( ((aType and 1)<>1) and assigned(o^.Vertical) ) then r.b.x := r.b.x + SWidth+1;
	if ( ((aType and 2)<>2) and assigned(o^.Horizontal) ) then r.b.y := r.b.y + SHeight+1;

	if ( aType and 1 ) = 1 then begin
		RectAssign(rb,r.b.x+2, r.a.y, r.b.x+SWidth+1, r.b.y);
		if not assigned( o^.Vertical ) then
			o^.Vertical := InitScrollbar(PWIDGET(o)^.AppOwner, rb, o, 1)
		else
			WidgetSetRect(PWIDGET(o^.Vertical),rb);
	end else if assigned( o^.Vertical ) then begin
		Dispose(o^.Vertical);
		o^.Vertical := NIL;
	end;

	if ( aType and 2 ) = 2 then begin
		RectAssign(rb,r.a.x,r.b.y+2,r.b.x,r.b.y+SHeight+1);
		if not assigned( o^.Horizontal ) then
			o^.Horizontal := InitScrollbar(PWIDGET(o)^.AppOwner,rb,o,2)
		else
			WidgetSetRect(PWIDGET(o^.Horizontal),rb);
	end else if assigned( o^.Horizontal ) then begin
		Dispose(o^.Horizontal);
		o^.Horizontal := NIL;
	end;
	WidgetSetRect(PWIDGET(o),r);
	ScrollWidgetSmartFlags(SLLWDG(o));
end;
////////////////////////////////////////////////////////////////////////////////
procedure ScrollBarChangeType ( o: PScrollWidget; aType: l_uchar );
begin
	if assigned( PWIDGET(o)^.Setup ) then
		ScrollBarChangeTypeUninserted(o,aType)
	else
		_ScrollBarChangeType(o,aType);
end;
////////////////////////////////////////////////////////////////////////////////
function ScrollWidgetEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
var
  p: TPoint;
begin
	if ( Ev^.aType = EV_MOUSE ) then begin
		if ( SLLWDG(o)^.Max.y > 0 ) then
			if ( Ev^.Message = MSG_MOUSE_SCLLUP ) then begin
				p := SLLWDG(o)^.Pos;
				if ( p.y > 3*SLLWDG(o)^.Delta.y ) then
					p.y := p.y - 3*SLLWDG(o)^.Delta.y
				else
					p.y := 0;
				ScrollBarScrollTo(SLLWDG(o),p);
				ScrollWidgetEventHandler := true;
			end else if ( Ev^.Message = MSG_MOUSE_SCLLDWN ) then begin
				p := SLLWDG(o)^.Pos;
				if ( p.y < SLLWDG(o)^.Max.y-(3*SLLWDG(o)^.Delta.y) ) then
					p.y := p.y + 3*SLLWDG(o)^.Delta.y
				else
					p.y := SLLWDG(o)^.Max.y;
				ScrollBarScrollTo(SLLWDG(o),p);
				ScrollWidgetEventHandler := true;
			end;
	end else

	ScrollWidgetEventHandler := false;
end;
////////////////////////////////////////////////////////////////////////////////
function wClass(Condition: boolean; widget1,widget2: PWidgetClass): PWidgetClass;
begin
  if Condition then wClass := widget1 else wClass := widget2;
end;

function InitScrollWidget( o:PScrollWidget; aClass:PWidgetClass; App:PApplication; r:TRect; aType:l_uchar ): PScrollWidget;
var
  rb: TRect;
  SWidth: l_ulong;
  SHeight: l_ulong;
  thisClass: PWidgetClass;
begin
	SWidth := 15;
	SHeight := 15;

	if ( UseSkins ) then begin
		SHeight := SkinHScrollbarBackground^.Skin^.h;
		SWidth := SkinVScrollbarBackground^.Skin^.w;
	end;


	if ( aType and 1 ) = 1 then r.b.x := r.b.x-SWidth+1;
	if ( aType and 2 ) = 2 then r.b.y := r.b.y-SHeight+1;

        if assigned(aClass) then
           thisClass := aClass
        else
           thisClass := ScrollWidgetClass;

	IntialiseWidget(App, thisClass, PWIDGET(o), r );

	if ( aType and 1 ) = 1 then begin
		RectAssign(rb,r.b.x+2, r.a.y, r.b.x+1+SWidth, r.b.y);
		o^.Vertical := InitScrollbar(App, rb, o, 1);

	end;

	if ( aType and 2 ) = 2 then begin
		RectAssign(rb,r.a.x,r.b.y+2,r.b.x,r.b.y+1+SHeight);
		o^.Horizontal := InitScrollbar(App,rb,o,2);
	end;


	PWIDGET(o)^.Setup := {$ifdef fpc}@{$endif}ScrollWidgetSetup;
	{PWIDGET(o)^.FreeEx := {$ifdef fpc}@{$endif}ScrollWidgetFreeEx;}

	InitScrollWidget := o;
end;
////////////////////////////////////////////////////////////////////////////////
function LibMain ( argc: integer; argv: l_text ): l_bool;
begin
	ScrollWidgetClass := CreateWidgetClass( ULONG_ID('S','l','W','g'), StdWidgetClass, 0 );
	
	ScrollbarClass := CreateWidgetClass( ULONG_ID('x','S','c','l'), StdWidgetClass, WCF_SATELLITE );


	SkinVScrollbarUpUp		:= GetSkinItem('VScrollbarUpUp');
	SkinVScrollbarUpDown		:= GetSkinItem('VScrollbarUpDown');
	SkinVScrollbarDownUp		:= GetSkinItem('VScrollbarDownUp');
	SkinVScrollbarDownDown		:= GetSkinItem('VScrollbarDownDown');
	SkinVScrollbarBackground	:= GetSkinItem('VScrollbarBackground');
	SkinVScrollbarTracker		:= GetSkinItem('VScrollbarTracker');

	SkinHScrollbarLeftUp		:= GetSkinItem('HScrollbarLeftUp');
	SkinHScrollbarLeftDown		:= GetSkinItem('HScrollbarLeftDown');
	SkinHScrollbarRightUp		:= GetSkinItem('HScrollbarRightUp');
	SkinHScrollbarRightDown		:= GetSkinItem('HScrollbarRightDown');
	SkinHScrollbarBackground	:= GetSkinItem('HScrollbarBackground');
	SkinHScrollbarTracker		:= GetSkinItem('HScrollbarTracker');
        (*
	APPEXPORT(InitScrollWidget);
	APPEXPORT(ScrollBarScrollTo);
	APPEXPORT(ScrollBarSetPosX);
	APPEXPORT(ScrollBarSetPosY);
	APPEXPORT(ScrollBarSetMaxY);
	APPEXPORT(ScrollBarSetMaxX);

	APPEXPORT(ScrollBarChangeType);
	APPEXPORT(ScrollBarRedraw);
	APPEXPORT(ScrollWidgetEventHandler);
	APPEXPORT(ScrollWidgetClass);
	APPEXPORT(ScrollBarChangeTypeUninserted);
	*)
	LibMain := true;
end;
////////////////////////////////////////////////////////////////////////////////
procedure Close;
begin

end;
////////////////////////////////////////////////////////////////////////////////
{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
   AppVersion := ULONG_ID(0,1,0,0);

{$endif}  {$LINK_CLIBS}
end.
