(*
/////////////////////////////////////////////////////////////////////////////////
//
//	Skin Core File 
//
//	Aura Milestone 4 
//
//	(c) Copyright 2013 Finn Technologies and Chase Finn. All rights reserved.
//
/////////////////////////////////////////////////////////////////////////////////
*)
unit skin;
{
 .#include "kernel.h"
 .#include "widget.h"
 .#include "string.h"
}
interface

uses
  guitypes, vfile, widget, grfx, allegro, alblend, registry, ini, debug;

{$i cdefs.inc}

{$ifdef LINK_C_LIBS}
  {$link widget.o}
  {$link string.o}
{$endif}

type
  PSkin = ^TSkin;
  TSkin = record

	Name      : l_text;
	
    Left, 
    Top, 
    Right, 
    Bottom    : l_short;
    
	Skin      : p_bitmap;

	
	{     Font       }
    	
	Font      : PFont;
	FontColor : l_color;

	Next      : PSkin;
	Prev      : PSkin;
  end;

var
  SkinItems: PSkin; 

function GetSkinItem(szName: l_text): PSkin; {$ifdef LINK_C_LIBS}external;{$endif}
procedure DrawSkin(outb: p_bitmap; si: PSkin; x1, y1, x2, y2: l_int); {$ifdef LINK_C_LIBS}external;{$endif}
procedure DrawTransSkin(outb: p_bitmap; si: PSkin; x1, y1, x2, y2, nb, max: l_int); {$ifdef LINK_C_LIBS}external;{$endif}

procedure DrawSkinCustom (outb: p_bitmap; ssi: p_bitmap; x1, y1, x2, y2: l_int); {$ifdef LINK_C_LIBS}external;{$endif}
procedure DrawTransSkinCustom (outb: p_bitmap; ssi: p_bitmap; x1, y1, x2, y2, nb, max: l_int); {$ifdef LINK_C_LIBS}external;{$endif}

procedure SkinSystemInstall; {$ifdef LINK_C_LIBS}external;{$endif}


function GetColor( c: TSColor ): l_color; {$ifdef LINK_C_LIBS}external;{$endif}

var
  SysColors: array[0..13] of l_color; 

var
  COL_3DTEXT 	:	l_color;
  COL_3DFACE 	:	l_color;
  COL_3DLIGHT	:	l_color;
  COL_3DDARK	:	l_color;
  COL_3DOVER	:	l_color;

  COL_BOXFACE	:	l_color;
  COL_BOXTEXT	:	l_color;
  COL_BOXLIGHT	:	l_color;
  COL_BOXDARK	:	l_color;
  COL_BOXSEL	:	l_color;

  COL_WTACT	:	l_color;
  COL_WTACTTEXT	:	l_color;

  COL_WTPAS	:	l_color;
  COL_WTPASTEXT	:	l_color;

  UseSkins: l_bool; SKIN_DIR: l_text;


implementation
{$ifndef LINK_C_LIBS}

const
  AppName: String           = 'Skin System';
  nUID: l_uid               = 'skin';

(*
 * The skin items tree
 *)

(*
 * Global exported variable determining the use of skins
 *)

(*
 * Color support
 *)
type
  l_color_ptr = ^l_color; 
 
var 
  _SysColors: array[0..24] of l_color;
  
const
   SystemColors: l_color_ptr = @_SysColors;

function GetColor( c: TSColor ): l_color;
begin
        if ( c.idx = $FF ) then
                GetColor := makecol(c.r,c.g,c.b)
        else
                GetColor := SysColors[c.idx];
end;

const ssi: p_bitmap = NIL;


function GetSkinItem (szName: l_text): PSkin;
var p: PSkin;
begin
        p := SkinItems;

        GetSkinItem := nil;
        while assigned(p) do
        begin
                if (szName = p^.Name) then begin GetSkinItem := p; exit; end;

                p := p^.Next;
        end;

end;

procedure DrawSkin (outb: p_bitmap; si: PSkin; x1, y1, x2, y2: integer );
begin
        if not assigned( si ) then
        begin
                DebugError('No skin',[]);
                exit;
        end;

        al_masked_blit(si^.Skin, outb, 0, 0, x1, y1, si^.Left, si^.Top);
        al_masked_blit(si^.Skin, outb, 0, si^.Skin^.h-si^.Bottom, x1, y2-si^.Bottom+1, si^.Left, si^.Bottom);
        al_masked_blit(si^.Skin, outb, si^.Skin^.w-si^.Right, 0, x2-si^.Right+1, y1, si^.Right, si^.Top);
        al_masked_blit(si^.Skin, outb, si^.Skin^.w-si^.Right, si^.Skin^.h-si^.Bottom, x2-si^.Right+1, y2-si^.Bottom+1, si^.Right, si^.Bottom);

        al_masked_stretch_blit(si^.Skin, outb, 0, si^.Top, si^.Left, si^.Skin^.h-si^.Top-si^.Bottom, x1, y1+si^.Top, si^.Left, (y2-y1+1)-si^.Top-si^.Bottom);
        al_masked_stretch_blit(si^.Skin, outb, si^.Skin^.w-si^.Right, si^.Top, si^.Right, si^.Skin^.h-si^.Top-si^.Bottom, x2-si^.Right+1, y1+si^.Top, si^.Right, (y2-y1+1)-si^.Top-si^.Bottom);

        al_masked_stretch_blit(si^.Skin, outb, si^.Left, 0, si^.Skin^.w-si^.Left-si^.Right, si^.Top, x1+si^.Left, y1, (x2-x1+1)-si^.Left-si^.Right, si^.Top);
        al_masked_stretch_blit(si^.Skin, outb, si^.Left, si^.Skin^.h-si^.Bottom, si^.Skin^.w-si^.Left-si^.Right, si^.Bottom, x1+si^.Left, y2-si^.Bottom+1, (x2-x1+1)-si^.Left-si^.Right, si^.Bottom);

        al_masked_stretch_blit(si^.Skin, outb, si^.Left, si^.Top, si^.Skin^.w-si^.Left-si^.Right, si^.Skin^.h-si^.Top-si^.Bottom, x1+si^.Left, y1+si^.Top, (x2-x1)-si^.Left-si^.Right+1, (y2-y1)-si^.Top-si^.Bottom+1);

end;

procedure DrawSkinCustom (outb, ssi: p_bitmap; x1, y1, x2, y2: integer);
begin
        if not assigned( ssi )then
        begin
                DebugError('Custom skin file was called, but was unable to be drawn',[]);
                exit;
        end;

        al_masked_blit(ssi, outb, 0, 0, x1, y1, 0, 0);
        al_masked_blit(ssi, outb, 0, ssi^.h-0, x1, y2-0+1, 0, 0);
        al_masked_blit(ssi, outb, ssi^.w-0, 0, x2-0+1, y1, 0, 0);
        al_masked_blit(ssi, outb, ssi^.w-0, ssi^.h-0, x2-0+1, y2-0+1, 0, 0);

        al_masked_stretch_blit(ssi, outb, 0, 0, 0, ssi^.h-0-0, x1, y1+0, 0, (y2-y1+1)-0-0);
        al_masked_stretch_blit(ssi, outb, ssi^.w-0, 0, 0, ssi^.h-0-0, x2-0+1, y1+0, 0, (y2-y1+1)-0-0);

        al_masked_stretch_blit(ssi, outb, 0, 0, ssi^.w-0-0, 0, x1+0, y1, (x2-x1+1)-0-0, 0);
        al_masked_stretch_blit(ssi, outb, 0, ssi^.h-0, ssi^.w-0-0, 0, x1+0, y2-0+1, (x2-x1+1)-0-0, 0);

        al_masked_stretch_blit(ssi, outb, 0, 0, ssi^.w-0-0, ssi^.h-0-0, x1+0, y1+0, (x2-x1)-0-0+1, (y2-y1)-0-0+1);      
		
end;

procedure DrawTransSkin (outb: p_bitmap; si: PSkin; x1, y1, x2, y2, nb, max: integer);
var sprite_buffer: PBitmap;
begin
        if not assigned( si ) then
        begin
                DebugError('No skin',[]);
                exit;
        end;
		
	sprite_buffer := al_create_bitmap(x2-x1+1,y2-y1);
	al_clear_to_color(sprite_buffer,makecol(255,0,255));
	al_masked_stretch_blit(si^.Skin, sprite_buffer, 0,  0, 10, si^.Skin^.h, 0, 0, 10, y2-y1);
	al_masked_stretch_blit(si^.Skin, sprite_buffer, 10, 0, si^.Skin^.w-20, si^.Skin^.h, 10, 0, x2-x1-19, y2-y1);
	al_masked_stretch_blit(si^.Skin, sprite_buffer, si^.Skin^.w-10, 0, 10, si^.Skin^.h, x2-x1-9, 0, 10, y2-y1);

	
    	al_set_trans_blender(0, 0, 0, nb*255 div max);

    	al_draw_trans_sprite(outb, sprite_buffer, x1, y1);

    	al_solid_mode();

        al_destroy_bitmap (sprite_buffer);

end;

const  ResImage: p_bitmap = NIL;

procedure DrawTransSkinCustom (outb, ssi: p_bitmap; x1, y1, x2, y2, nb, max: integer);
var sprite_buffer: PBitmap;
begin
        if not assigned( ssi ) then
        begin
                DebugError('Custom skin file was called, but was unable to be drawn',[]);
                exit;
        end;
		
	sprite_buffer := al_create_bitmap(x2-x1+1,y2-y1);
	al_clear_to_color(sprite_buffer,makecol(255,0,255));
	al_masked_stretch_blit(ssi, sprite_buffer, 0,  0, 10, ssi^.h, 0, 0, 10, y2-y1);
	al_masked_stretch_blit(ssi, sprite_buffer, 10, 0, ssi^.w-20, ssi^.h, 10, 0, x2-x1-19, y2-y1);
	al_masked_stretch_blit(ssi, sprite_buffer, ssi^.w-10, 0, 10, ssi^.h, x2-x1-9, 0, 10, y2-y1);

	
    	al_set_trans_blender(0, 0, 0, nb*255 div max);

    	al_draw_trans_sprite(outb, sprite_buffer, x1, y1);

    	al_solid_mode();

        al_destroy_bitmap (sprite_buffer);

end;

(*
function ReloadSkin (Filename: l_text): l_bool;
{
        PSkin p = SkinItems;
        l_text szFile = malloc(strlen(SKIN_DIR) + strlen(File) + 1);
        l_int Items = 0;

        strcpy(szFile, SKIN_DIR);
        strcat(szFile, File);

        DebugMessage(szFile);

        do
        {
                if (INISectionExists (szFile, p^.Name))
                {
                        l_text szPath = malloc(1024);

                        strcpy(szPath, SKIN_DIR);
                        strcat(szPath, INIGetNew(szFile, p^.Name, "bmp", NULL));
                        if (p^.Skin) destroy_bitmap (p^.Skin);
                        p^.Skin = LoadImage(szPath);

                        p^.Left = INIGetInteger(szFile, p^.Name, "left", 3);
                        p^.Right = INIGetInteger(szFile, p^.Name, "right", 3);
                        p^.Top = INIGetInteger(szFile, p^.Name, "top", 3);
                        p^.Bottom = INIGetInteger(szFile, p^.Name, "bottom", 3);

                        Items++;

                        free (szPath);
                }
                else DebugMessage ("Section %s doesnt exist", p^.Name);

                p = p^.Next;
        }
        while (p);

        DebugMessage("Skin system reloaded a total of '%i' skin items", Items);

        WidgetDrawAll(DeskTop);

        return 0;
} 
*)

function LoadColors (szFile: l_text): l_bool;
begin
        _SysColors[0]  := INIGetColor(szFile, 'colors', '3dtext', _SysColors[0]);
        _SysColors[1]  := INIGetColor(szFile, 'colors', '3dface', _SysColors[1]);
        _SysColors[2]  := INIGetColor(szFile, 'colors', '3dlight', _SysColors[2]);
        _SysColors[3]  := INIGetColor(szFile, 'colors', '3ddark', _SysColors[3]);
        _SysColors[4]  := INIGetColor(szFile, 'colors', '3dover', _SysColors[4]);

        (*_SysColors[5]  := INIGetColor(szFile, 'colors", 'boxface', makecol(0,0,0));
        _SysColors[6]  := INIGetColor(szFile, 'colors', 'boxtext', makecol(0,0,0));
        _SysColors[7]  := INIGetColor(szFile, 'colors', 'boxlight', makecol(0,0,0));
        _SysColors[8]  := INIGetColor(szFile, 'colors', 'boxdark', makecol(0,0,0));
        _SysColors[9]  := INIGetColor(szFile, 'colors', 'boxsel', makecol(0,0,0));

        _SysColors[10] := INIGetColor(szFile, 'colors', 'wtact', makecol(0,0,0));
        _SysColors[11] := INIGetColor(szFile, 'colors', 'wtacttext', makecol(0,0,0));

        _SysColors[12] := INIGetColor(szFile, 'colors', 'wtpas', makecol(0,0,0));
        _SysColors[13] := INIGetColor(szFile, 'colors', 'wtpas', makecol(0,0,0));*)
end;

function strcpy(var d :l_Text; s: l_text): integer;
begin
  d := s;
  strcpy := Length(d);
end;

function strcat(var d: l_text; s: l_text): integer;
begin
  d := d + s;
  strcat := Length(d);
end;

function strlen(s: l_text): integer;
begin
  strlen := Length(s);
end;

function stricmp(c: array of char; s: l_text): Integer;
var v: l_text; i,j: integer;
begin
  v := ''; i := 1; j := 0;
  for j := 0 to high(c) do
    if c[j] <> #0 then v := v + c[j] else exit;
  if v < s then stricmp := -1 else
  if v = s then stricmp :=  0 else stricmp := +1;
end;

type
  PFOpenRec = ^TFOpenRec;
  TFOpenRec = record
     f: file;
     opend: l_bool;
  end;

{$ifdef OPascal}
function fopen(Filename: l_text; mode: integer): TFOpenRec;
var f: file; rec: TFOpenRec;
begin
  Assign(f, Filename); 
  {$i-}
  Reset(f, 1);
  {$i+}
  if ioresult = 0 then
  begin
    rec.f := f;
    rec.opend := true;
    fopen := rec;
  end
  else
  begin
    rec.opend := false;
    fopen := rec;
  end;
end;
{$else}
function fopen(Filename: l_text; mode: integer): PFOpenRec;
var f: file; rec: PFOpenRec;
begin
  New(rec);
  Assign(f, Filename);
  {$i-}
  Reset(f, 1);
  {$i+}
  if ioresult = 0 then
  begin
    rec^.f := f;
    rec^.opend := true;
    fopen := rec;
  end
  else
  begin
    rec^.opend := false;
    fopen := rec;
  end;
end;
{$endif}

procedure fclose(var f: file);
begin
  Close(f);
end;

function fgets(var s: array of char; Count: integer; var f: file): l_bool;
var readed: Integer;
begin
  BlockRead(f,s,Count,readed);
end;

function CharsToString(a: array of char): l_text;
var i: integer; s: l_text;
begin
  for i:=0 to high(a) do
    if a[i] <> #0 then s := s + a[i] else break;
  CharsToString := s;
end;

function LoadSkin (Filename: l_text): l_bool;
var
  f: file;
  szFile,
  szPath,
  szTemp,
  c: array[0..255] of char;
  Items: integer;
  p: PSkin;
  platform: l_bool;
  Rec: TFOpenRec;
begin
        {
        FILE *f = 0;
        l_text szFile = malloc(strlen(SKIN_DIR) + strlen(File) + 1);
        }
  (*
  Items  := 0;

        szFile := SKIN_DIR;
        szFile := szFile + Filename;

        DebugMessage('Trying to load skin file - %s', [szFile]);

        Rec := fopen(szFile, FSF_Read){$ifndef OPascal}^{$endif};
        f := Rec.f;
        if not Rec.opend then
        begin
                DebugWarning ('Skin - %s - not found! Using standard skin', [szFile]);
                { free(szFile); }
                LoadSkin := false;
                exit;
        end;

        LoadColors (szFile);

        while not(eof(f)) do
        begin

                fgets(c, 256, f);
                {
                if (!c[0]) then continue;
                if (c[strlen(c)-1] < 32) then c[strlen(c)-1] = 0;
                }
                if (c[0] = '[') and (c[strlen(c)-1] = ']') or (c[strlen(c)-2] = ']') then
                if stricmp(c[1], 'Information]') then
                begin
                      {
                        l_text szPath           = malloc(1024);
                        l_text szTemp           = malloc(1024);
                        l_bool Platform         = false;
                        PSkin p                 = (PSkin)malloc(sizeof(TSkin));
                      }
                        fillchar(p^, sizeof(TSkin), 0);

                        (*
                         * Fuck this dirty hack :GRR:
                         *)
                        if (c[strlen(c)-2] = ']') then Platform := true;

                        p^.Name := CharsToString(c[1]);

                        if (Platform) then p^.Name[strlen(CharsToString(c[1]))-2] := #0
                        else p^.Name[strlen(CharsToString(c[1]))-1] := #0;

                        (*
                        *       Load image  (bug fixed 19.05.2006)
                        *)

                        strcpy(szPath, SKIN_DIR);
                        DebugMessage('Skin - Load Image - Path: %s , File: %s', [szPath, INIGetNew(szFile, p^.Name, 'bmp', NIL)]);
                        strcpy(szTemp, INIGetNew(szFile, p^.Name, "bmp", NULL));
                        if (strlen(szTemp) > 1) then
                        begin
                          strcat(szPath, szTemp);
                          p^.Skin = LoadImage(szPath);

                         (*
                          *       Load image offsets
                          *)
                          p^.Left = INIGetInteger(szFile, p^.Name, 'left', 3);
                          p^.Right = INIGetInteger(szFile, p^.Name, 'right', 3);
                          p^.Top = INIGetInteger(szFile, p^.Name, 'top', 3);
                          p^.Bottom = INIGetInteger(szFile, p^.Name, 'bottom", 3);

                          (*
                          *       Load skin items font and color
                          *)
                          strcpy(szPath, SKIN_DIR);
                          strcat(szPath, INIGetNew(szFile, p^.Name, 'font', ''));
                          p^.Font := FontLoad(szPath);
                          p^.FontColor = INIGetColor(szFile, p^.Name, 'color', makecol(0,0,0));
                        end;
                        inc(Items);

                        p^.Next := NIL;                        

                        if not assigned(SkinItems) then SkinItems := p;
                        else
                        begin
                                w := SkinItems;

                                if assigned(w^.Next) then
                                begin
                                   while assigned(w^.Next) do w := w^.Next;
                                end;
                                w^.Next := p;
                        end;
                end;
        end; 

        DebugMessage('Skin system loaded a total of '%i' skin items', [Items]);

        fclose(f);
        
        LoadSkin := true;
        *)
end;

procedure SkinSystemInstall;
begin
        _SysColors[0]  := makecol(  0,  0,  0);
        _SysColors[1]  := makecol(246,246,246);
        _SysColors[2]  := makecol(255,255,255);
        _SysColors[3]  := makecol(128,128,128);
        _SysColors[4]  := makecol(255,255,255);

        _SysColors[5]  := makecol(255,255,255);
        _SysColors[6]  := makecol(  0,  0,  0);
        _SysColors[7]  := makecol(195,195,195);
        _SysColors[8]  := makecol(195,195,195);
        _SysColors[9]  := makecol( 64, 64,255);

        _SysColors[10] := makecol(  0,  0,255);
        _SysColors[11] := makecol(255,255,255);

        _SysColors[12] := makecol(192,192,255);
        _SysColors[13] := makecol(  0,  0,  0);

   SKIN_DIR := KeyGetText('/USER/GUI/SKINPATH', './SYSTEM/SKINS/');

   if (strlen(KeyGetText('/USER/GUI/SKIN', '')) = 0) then
   begin
     UseSkins := false;
     DebugMessage('User does not want to use skins. No skin is loaded.',[]);
     exit;
   end;

   UseSkins := LoadSkin (KeyGetText('/USER/GUI/SKIN', 'ozone.ini'));
end;

function LibMain ( argc: integer; argv: l_text ): l_bool;
begin
        SkinSystemInstall;
       (*
        APPEXPORT(GetSkinItem);
        APPEXPORT(DrawSkin);
        APPEXPORT(DrawSkinCustom);
        APPEXPORT(DrawTransSkin);
        APPEXPORT(DrawTransSkinCustom);
        APPEXPORT(SysColors);
        APPEXPORT(_SysColors);
        APPEXPORT(UseSkins);
  
        APPEXPORT(ReloadSkin);

        APPEXPORT(GetColor);
       *)
        LibMain := true;
end;

procedure Close;
var
  p,tmp: PSkin;
  Items: integer;
begin
        p := SkinItems;

        SkinItems := NIL;

        Items := 0;
        while assigned( p ) do
        begin
                tmp := p^.Next;

                if ( p^.Name <> '' ) then {free(p^.Name)};
                if assigned( p^.Skin ) then al_destroy_bitmap(p^.Skin);

                dispose(p);

                p := tmp;
                inc(Items);
        end;

        DebugMessage('Skin system destroyed a total of %i skin items', [Items]);
end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  COL_3DTEXT 	:=		SysColors[0];
  COL_3DFACE 	:=		SysColors[1];
  COL_3DLIGHT	:=		SysColors[2];
  COL_3DDARK	:=		SysColors[3];
  COL_3DOVER	:=		SysColors[4];

  COL_BOXFACE	:=		SysColors[5];
  COL_BOXTEXT	:=		SysColors[6];
  COL_BOXLIGHT	:=		SysColors[7]    ;
  COL_BOXDARK	:=		SysColors[8];
  COL_BOXSEL	:=		SysColors[9];

  COL_WTACT	:=		SysColors[10];
  COL_WTACTTEXT	:=		SysColors[11];

  COL_WTPAS	:=		SysColors[12];
  COL_WTPASTEXT	:=		SysColors[13];

  SKIN_DIR      := './SYSTEM/SKINS/';
  SkinItems     := NIL;
  UseSkins      := true;

{$endif} {LINK_CLIBS}
end.
