(*
////////////////////////////////////////////////////////////////////////////////
//
//  Slider widget - Core File
//
//	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
unit slider;

{
 .#include 'kernel.h'
 .#include 'widget.h'
 .#include 'slider.h'
}

interface

uses guitypes, widget;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
  {$link slider.o}
{$endif}

const
 SL_SCROLLUP	= $00FF1100;
 SL_SCROLLDOWN	= $00FF1200;

 SCTX_START     = $01;
 SCTX_SLIDING   = $02;
 SCTX_SET       = $03;


type
  NotifyProc = procedure(o: PSlider; Ctx: l_uchar; Value: l_ulong);
  PSlider = ^TSlider;
  TSlider = record

	o         : TWidget;

	Value     : l_ulong;

	Steps     : l_ulong;
	
	
	State     : l_char;
	PointValue: l_ulong;
	
	Notify    : NotifyProc;
	
  end;
  SLIDER = PSlider;



function CreateSlider( App: PApplication; r: TRect; Steps: l_ulong ): PSlider; {$ifdef LINK_CLIBS}external;{$endif}

implementation
{$ifndef LINK_CLIBS}

(*////////////////////////////////////////////////////////////////////////////////*)
var	AppVersion: l_ulong;
	
const
  AppName: l_text = 'Slider Widget';
  nUID: l_text = 'slider';
  NeededLibs: l_uid = { 'widget', 'skin','' };
(*////////////////////////////////////////////////////////////////////////////////*)
var
  SliderClass: PWidgetClass;
  SkinSliderLine: PSkin;
  SkinSliderBackground: PSkin;
  SkinSliderButton: PSkin;
  SLIDERWIDTH: l_ulong;
  SLIDERHEIGHT: l_ulong;
(*////////////////////////////////////////////////////////////////////////////////*)
const DOING_DRAG = 1;

function Slider_Width(UseSkins: l_bool): l_ulong;
begin
  if UseSkins then Slider_Width := SkinSliderButton^.Skin^.w else Slider_Width := 10;
end;

function Slider_Height(UseSkins: l_bool): l_ulong;
begin
  if UseSkins then Slider_Height := SkinSliderButton^.Skin^.h else Slider_Height := 20;
end;
{
 .#define SLIDERWIDTH 	(UseSkins?SkinSliderButton^.Skin^.w:10)
 .#define SLIDERHEIGHT 	(UseSkins?SkinSliderButton^.Skin^.h:20)
}
(*////////////////////////////////////////////////////////////////////////////////*)
var  
  p: TPoint;
  Doing: integer;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure SliderDraw ( o: PWidget; buffer: p_bitmap; w: PRect );
var
  Value: longint;
  Pos: longint;
begin
	Value := SLIDER(o)^.Value;
	Pos := o^.Absolute.a.x+10+(((o^.Absolute.b.x-o^.Absolute.a.x-20)*Value) div SLIDER(o)^.Steps);

	if ( SLIDER(o)^.State = DOING_DRAG ) then
	begin
		(*if ( Pos+(Mouse^.State.p.x-p.x) <= o^.Absolute.b.x-10 ) then
		 begin
			if ( Pos+(Mouse^.State.p.x-p.x) >= o^.Absolute.a.x+10 )
				Value := (Mouse^.State.p.x-o^.Absolute.a.x-10)*SLIDER(o)^.Steps div (o^.Absolute.b.x-o^.Absolute.a.x-20);
			else
				Value := 0;
		end else
			Value := SLIDER(o)^.Steps; *)
		Value := SLIDER(o)^.PointValue;
		
		
	end;
	Pos := o^.Absolute.a.x+10+(((o^.Absolute.b.x-o^.Absolute.a.x-20)*Value)/SLIDER(o)^.Steps);

	if ( not UseSkins ) then
	begin
		rectfill(buffer,w^.a.x,w^.a.y,w^.b.x,w^.b.y,COL_3DFACE);
		Rect3D(buffer,o^.Absolute.a.x+5,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-1,o^.Absolute.b.x-5,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+1,COL_3DDARK,COL_3DLIGHT);
		rectfill(buffer,Pos-5,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-10,Pos+5,(o^.Absolute.a.y+o^.Absolute.b.y) div 2+10,COL_3DFACE);
		Rect3D(buffer,Pos-5,(o^.Absolute.a.y+o^.Absolute.b.y) div 2-10,Pos+5,(o^.Absolute.a.y+o^.Absolute.b.y)/2+10,COL_3DLIGHT,COL_3DDARK);
	end else begin
		DrawSkin(buffer, SkinSliderBackground, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y);
		DrawSkin(buffer, SkinSliderLine, o^.Absolute.a.x+10, o^.Absolute.a.y+((o^.Absolute.b.y-o^.Absolute.a.y) div 2-SkinSliderLine^.Skin^.h div 2), o^.Absolute.b.x-10, o^.Absolute.a.y+((o^.Absolute.b.y-o^.Absolute.a.y) div 2+SkinSliderLine^.Skin^.h div 2));
		DrawSkin(buffer, SkinSliderButton, Pos-SkinSliderButton^.Skin^.w div 2, o^.Absolute.a.y+SkinSliderBackground^.Top, Pos+SkinSliderButton^.Skin^.w div 2, o^.Absolute.b.y - SkinSliderBackground^.Bottom);
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function SliderEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
var
  tmp: TRect;
  Pos: longint;
  Value: longint;
begin
	if (Ev^.Type = EV_MOUSE) then
	begin
		if ( ( Ev^.Message = WEvMouseMove ) and ((Mouse^.State.b and BUTTON_LEFT) = BUTTON_LEFT) and (SLIDER(o)^.State = DOING_DRAG) ) then
        begin			
			l_long Value := SLIDER(o)^.Value;
			l_long Pos := o^.Absolute.a.x+10+(((o^.Absolute.b.x-o^.Absolute.a.x-20)*Value)/SLIDER(o)^.Steps);
			if ( Pos+(Mouse^.State.p.x-p.x) <= o^.Absolute.b.x-10 ) then
			begin
				if ( Pos+(Mouse^.State.p.x-p.x) >:= o^.Absolute.a.x+10 ) then
					Value := (Mouse^.State.p.x-o^.Absolute.a.x-10)*SLIDER(o)^.Steps /(o^.Absolute.b.x-o^.Absolute.a.x-20);
				else
					Value := 0;
			end else
				Value := SLIDER(o)^.Steps;
				
			if ( SLIDER(o)^.Notify ) then SLIDER(o)^.Notify(SLIDER(o),SCTX_SLIDING,Value);
		
			SLIDER(o)^.PointValue := Value;
		
			WidgetDraw(o, NIL);
			
			SliderEventHandler := true;
		end;

		if ( Ev^.Message = WEvMouseLUp ) then
		begin
			if ( SLIDER(o)^.State = DOING_DRAG  ) then
			begin
				if (o^.Absolute.a.x+10 + (((o^.Absolute.b.x-o^.Absolute.a.x-20)*(SLIDER(o)^.Value)) div SLIDER(o)^.Steps) + (Mouse^.State.p.x-p.x) <= o^.Absolute.b.x-10) then
                begin
					if (o^.Absolute.a.x+10 + (((o^.Absolute.b.x-o^.Absolute.a.x-20)*(SLIDER(o)^.Value)) div SLIDER(o)^.Steps) + (Mouse^.State.p.x-p.x) >= o^.Absolute.a.x+10) then
						SLIDER(o)^.Value := (Mouse^.State.p.x-o^.Absolute.a.x-10)*SLIDER(o)^.Steps div (o^.Absolute.b.x-o^.Absolute.a.x-20);
					else
						SLIDER(o)^.Value := 0;
				end
				else
					SLIDER(o)^.Value := SLIDER(o)^.Steps;
					
				if assigned( SLIDER(o)^.Notify ) then SLIDER(o)^.Notify(SLIDER(o),SCTX_SET,SLIDER(o)^.Value);
					
				MasterWidget := NIL;
				Doing := 0;
				SliderEventHandler := true;
			end;
		end;

		if (Ev^.Message = WEvMouseLDown) then
		begin
			RectAssign(
              tmp, 
              o^.Absolute.a.x+10+(((o^.Absolute.b.x-o^.Absolute.a.x-20)*(SLIDER(o)^.Value)) div SLIDER(o)^.Steps)-SLIDERWIDTH div 2, 
              o^.Absolute.a.y+((o^.Absolute.b.y-o^.Absolute.a.y) div 2-SLIDERHEIGHT div 2), 
              o^.Absolute.a.x+10+(((o^.Absolute.b.x-o^.Absolute.a.x-20)*(SLIDER(o)^.Value)) div SLIDER(o)^.Steps)+SLIDERWIDTH div 2, 
              o^.Absolute.a.y+((o^.Absolute.b.y-o^.Absolute.a.y) div 2+SLIDERHEIGHT div 2)
            );
			if (CURSOR_IN_RECT(tmp)) then
			begin
				p := Mouse^.State.p;
				MasterWidget := o;
				SLIDER(o)^.State := DOING_DRAG;
				SLIDER(o)^.PointValue := SLIDER(o)^.Value;
				if ( SLIDER(o)^.Notify ) then SLIDER(o)^.Notify(SLIDER(o),SCTX_START,SLIDER(o)^.Value);
				SliderEventHandler := true;
			end
			else if (Mouse^.State.p.x < o^.Absolute.a.x+10+(((o^.Absolute.b.x-o^.Absolute.a.x-10)*(SLIDER(o)^.Value)) div SLIDER(o)^.Steps)) then
			begin
				if ( (SLIDER(o)^.Value - 1) <> -1 )
				begin
					Dec(SLIDER(o)^.Value);
					if assigned( SLIDER(o)^.Notify ) then SLIDER(o)^.Notify(SLIDER(o),SCTX_SET,SLIDER(o)^.Value);
					WidgetDraw(o, NIL);
					SliderEventHandler := true;
				end;
			end;
			else if (Mouse^.State.p.x > o^.Absolute.a.x+10+(((o^.Absolute.b.x-o^.Absolute.a.x-10)*(SLIDER(o)^.Value)) div SLIDER(o)^.Steps))
			begin
				if ( (SLIDER(o)^.Value + 1) <:= SLIDER(o)^.Steps )
				begin
					SLIDER(o)^.Value++;
					if ( SLIDER(o)^.Notify ) SLIDER(o)^.Notify(SLIDER(o),SCTX_SET,SLIDER(o)^.Value);
					WidgetDraw(o, NIL);
					SliderEventHandler := true;
				end;
			end;
		end;
	end;

	SliderEventHandler := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CreateSlider( App: PApplication; r: TRect; Steps: l_ulong ): PSlider;
var
  p: PSlider;
begin
    New(p);
    
	if assigned(p) CreateSlider := NIL else
    begin
    	fillchar(p^, sizeof(TSlider), 0);

    	p^.Steps := Steps;

    	IntialiseWidget(App, SliderClass, PWIDGET(p), r );

    	PWIDGET(p)^.Draw := {ifdef fpc}@{$endif}SliderDraw;
    	PWIDGET(p)^.EventHandler := {ifdef fpc}@{$endif}SliderEventHandler;

    	CreateSlider := p;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function SliderSetRProp ( o:PWidget; Name:l_text; Value:Pointer; Redraw:l_bool ): l_bool;
begin	
	SliderSetRProp := false;

	if ( TextCompare(Name,'value') = 0 ) then
	begin
		SLIDER(o)^.Value := l_ulong(Value)^;
		if ( SLIDER(o)^.Value > SLIDER(o)^.Steps ) then SLIDER(o)^.Value := SLIDER(o)^.Steps;
		if ( Redraw ) then WidgetDraw(PWIDGET(o), NIL);
		SliderSetRProp := true;
	end;
	if ( TextCompare(Name,'max') = 0 ) then
	begin
		SLIDER(o)^.Steps := l_ulong(Value)^;
		if ( SLIDER(o)^.Value > SLIDER(o)^.Steps ) then SLIDER(o)^.Value := SLIDER(o)^.Steps;
		if ( Redraw ) then WidgetDraw(PWIDGET(o), NIL);
		SliderSetRProp := true;
	end;
	
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function SliderGetRProp ( o: PWidget; Name: l_text ): Pointer;
begin	
    if ( TextCompare(Name,'value') = 0 ) then SliderGetRProp := Pointer(SLIDER(o)^.Value);
	if ( TextCompare(Name,'max') = 0 ) then SliderGetRProp := Pointer(SLIDER(o)^.Steps);
	SliderGetRProp := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function StdCreateSlider ( App: PApplication; r: TRect ): PWidget;
begin
	StdCreateSlider := PWIDGET(CreateSlider(App,r,10));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function LibMain ( argc: integer; argv: l_text ): l_bool;
var 
  r: TRect;
  p: PSlider;
begin
	SliderClass := CreateWidgetClass( ULONG_ID('S','l','d','r'), StdWidgetClass, 0 );
	SliderClass^.StdCreate := {ifdef fpc}@{$endif}StdCreateSlider;
	SliderClass^.SetRProp := {ifdef fpc}@{$endif}SliderSetRProp;
	SliderClass^.GetRProp := {ifdef fpc}@{$endif}SliderGetRProp;

	WClassRegisterProp(SliderClass,PTYPE_INT,'value');
	WClassRegisterProp(SliderClass,PTYPE_INT,'max');
	
	(**
	*	Skin items sizes
	*)
	SkinSliderLine			:= GetSkinItem('SliderLine');
	SkinSliderButton		:= GetSkinItem('SliderButton');
	SkinSliderBackground	:= GetSkinItem('SliderBackground');

	{ APPEXPORT(CreateSlider);  }
(*
	RectAssign(r, 750,300,1000,340);
	p := CreateSlider(Me, r);
	InsertWidget(PWIDGET(DeskTop), PWIDGET(p));
	WidgetDraw(WIDGET(p), NIL);
*)
	LibMain := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Close;
begin

end;
(*////////////////////////////////////////////////////////////////////////////////*)
{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion ::= ULONG_ID(0,0,0,1);

  SkinSliderLine	    := NIL;
  SkinSliderBackground	:= NIL;
  SkinSliderButton	    := NIL;
  Doing                 := 0;  
  SLIDERWIDTH           := Slider_Width(UseSkins);
  SLIDERHEIGHT          := Slider_Height(UseSkins);
  
{$endif} {LINK_CLIBS}
end.  
