(*
////////////////////////////////////////////////////////////////////////////////
//
//	Sound system
//
//	Copyright (c) 2004 Point Mad. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
{
 .#include'kernel.h'
 .#include'sound.h'
}
unit sound;

interface

uses
  guitypes,vfile,list,tasks;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
  {$link sound.o}
{$endif}



type
  PSStreamDriver = ^TSStreamDriver;
  PSStream = ^TSStream;

  ExtractSampleFunc = function(s: PSStream; var buffer): integer;
  SetPosMSecFunc = function(s: PSStream; msec: l_ulong): l_bool;
  GetPosMSecFunc = function(s: PSStream; var msec: l_ulong): l_bool;
  GetLengthMSecFunc = function(s: PSStream; var msec: l_ulong): l_bool;
  FreeStreamFunc = function(s: PSStream): l_bool;

  TSStreamDriver = record
	
	Codec         : l_ulong; // DMS linked codec
	
	ExtractSample : ExtractSampleFunc;
	
	SetPosMsec    : SetPosMSecFunc;
	
	GetPosMsec    : GetPosMSecFunc;
	GetLengthMsec : GetLengthMsecFunc;
	
	Free          : FreeStreamFunc;
	
  end;


  TSStream = record
	
	Driver  : PSStreamDriver;
	
	f       : PFileStruct;
	
	Ex      : Pointer;
	
	Finished: l_bool;
	Playing : l_bool;
	
	(* Allegro AudioStream *)
	
	voice   : integer;                       (* the voice we are playing on                         *)
	samp    : Pointer;  { SAMPLE }           (* the sample we are using < TYPE SAMPLE aus Allegro > *)
        len     : integer;                       (* buffer length                                       *)
	bufcount: integer;                       (* number of buffers per sample half                   *)
	bufnum  : integer;                       (* current refill buffer                               *)
	active  : integer;                       (* which half is currently playing                     *)
	locked  : Pointer;                       (* the locked buffer                                   *)
	
  end;
  SSTREAM = PSStream;

function NewSoundStream ( Drv: PSStreamDriver; f: PFileStruct; Ex: Pointer; len,bits,stereo,freq: l_int ): PSStream;
{$ifdef LINK_CLIBS}external;{$endif}

function PlaySoundStream( o: PSStream ): l_bool; {$ifdef LINK_CLIBS}external;{$endif}
function PauseSoundStream( o: PSStream ): l_bool; {$ifdef LINK_CLIBS}external;{$endif}

function SStreamGetPosMsec( o: PSStream; var msec: l_ulong ): l_bool; {$ifdef LINK_CLIBS}external;{$endif}
function SStreamGetLengthMsec( o: PSStream; var msec: l_ulong ): l_bool; {$ifdef LINK_CLIBS}external;{$endif}

function SStreamSetPosMsec( o: PSStream; msec: l_ulong ): l_bool; {$ifdef LINK_CLIBS}external;{$endif}

function SStreamFree( o: PSStream ): l_bool; {$ifdef LINK_CLIBS}external;{$endif}

procedure SetVolumeMidi ( Volume: integer ); {$ifdef LINK_CLIBS}external;{$endif}
procedure SetVolumeDigi ( Volume: integer ); {$ifdef LINK_CLIBS}external;{$endif}

var VolumeDigi, VolumeMidi: integer;

implementation
{$ifndef LINK_CLIBS}

{$ifdef OPascal}
uses Classes;
type
  TSoundStream = class(TFileStream)
    FHeader: PSStream;
    constructor Create(Name: String; Mode: Integer);
    function Header: PSStream;
    destructor Destroy; override;
  end;

  constructor TSoundStream.Create(Name: String; Mode: Integer);
  begin
    inherited Create(Filename, Mode);
    try
      New(FHeader);
      Read(FHeader^,Sizeof(TSStream));
    except
      FHeader := NIL;
    end;
  end;

  function TSoundStream.Header: PSStream;
  begin
    Result := FHeader;
  end;

  destructor TSoundStream.Destroy;
  begin
    Dispose(FHeader);
    FHeader := nil;
    inherited Destroy;
  end;

{$else}
uses Objects;

  TSoundStream = object(TBufStream)
    FHeader: PSStream;
    constructor Init(Filename: String; Mode: Integer);
    destructor Done; virtual;
    function Header: PSStream;
  end;

  constructor TSoundStream.Init(Filename: String; Mode: Integer);
  begin
    FHeader := NIL;
    inherited Init(Filename, Mode);
    if status = stOk then
    begin
      New(FHeader);
      Read(FHeader^,SizeOf(TSStream));
    end
    else fail;
  end;

  function TSoundStream.Header: PSStream:
  begin
    Header := FHeader;
  end;

  destructor TSoundStream.Done;
  begin
    if assigned(FHeader) then Dispose(FHeader);
    FHeader := nil;
    inherited Done;
  end;
{$endif}

(*////////////////////////////////////////////////////////////////////////////////*)
const 
 nUID: l_text 		= 'sound';
 AppName: l_text 	= 'Sound System';
 NeededLibs: array[0..2] of String = ( '', '', '' );

(*////////////////////////////////////////////////////////////////////////////////*)
var
  AppVersion: l_ulong;
  Playing: PList;
  STask  : PTask;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Allegro imported functions                                                   *)
(*////////////////////////////////////////////////////////////////////////////////*)
function GetAudioStreamBuffer( stream: PSStream ): Pointer; {$ifdef OPascal}inline;{$endif}
var
    fac: integer;
    ste: integer;
	pos: integer;
	data: PChar;
begin
  (*
    data := nil;
	if (stream^.bufnum = stream^.active * stream^.bufcount) then
	begin
		pos := voice_get_position(stream^.voice);
		if (stream^.active = 0) then begin
			if ( pos < stream^.len*stream^.bufcount ) then GetAudioStreamBuffer := NIL
		end else begin
			if ( pos >= stream^.len*stream^.bufcount ) then GetAudioStreamBuffer := NIL;
		end;
		stream^.active := 1-stream^.active;
	end;
	
	if (not stream^.locked) then
	begin
		pos := (1-stream^.active) * stream^.bufcount * stream^.len;
		if ( digi_driver^.lock_voice ) then
			data := digi_driver^.lock_voice(stream^.voice, pos, pos+stream^.bufcount*stream^.len);
		if assigned( data ) then
			stream^.locked := data;
		else
			stream^.locked := (char * )stream^.samp^.data + (pos * ((stream^.samp^.bits==8) ? 1 : sizeof(short)) * ((stream^.samp^.stereo) ? 2 : 1));
	end;
	
	if stream^.samp^.bits = 8 then fac := 1 else fac := sizeof(Shortint);
	

	GetAudioStreamBuffer := Pointer(Longint(stream^.locked) + ((stream^.bufnum mod stream^.bufcount) *
		stream^.len * fac *
		stream^.samp^.stereo);
  *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeAudioStreamBuffer(  stream: PSStream ); {$ifdef OPascal}inline;{$endif}
begin
      (*
        inc(stream^.bufnum);
	if (stream^.bufnum >= stream^.bufcount*2) then stream^.bufnum := 0;
	if ((stream^.locked) and ((stream^.bufnum = 0) or (stream^.bufnum = stream^.bufcount))) then
	begin
		if (digi_driver^.unlock_voice) digi_driver^.unlock_voice(stream^.voice);
		stream^.locked := NIL;
	end;
	if (voice_get_position(stream^.voice) == -1) voice_start(stream^.voice);
      *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure _PauseSound ( o: PSStream );
begin
  (*
	if (not o^.Playing ) then exit;
	
    if ((o^.locked) and (digi_driver^.unlock_voice)) then
		digi_driver^.unlock_voice(o^.voice);
	
	o^.locked := NIL;
	o^.Playing := false;
	deallocate_voice(o^.voice);
	DebugMessage('Pause Stream %d', [o]);
    *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function _PlaySound ( o: PSStream ): l_bool;
begin
(*	
	if ( o^.Playing ) then _PlaySound := false;
	o^.Playing := true;
	o^.locked := NIL;
	o^.bufnum := 0;
	o^.active := 1;
	o^.voice := allocate_voice(o^.samp);
	if (o^.voice < 0) return false;
	voice_set_playmode(o^.voice, PLAYMODE_LOOP);
	DebugMessage('Playing Stream %d', [o]);
	_PlaySound := true;
*)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Our global sound stream polling task                                         *)
(*////////////////////////////////////////////////////////////////////////////////*)
procedure SoundTask ( o:PTask );
var
	a, n: PListItem;
	Data: Pointer;
	r: integer;
begin
  (*
  if not assigned( Playing^.Last ) then exit;
	a := Playing^.Last;
	repeat
		n := a^.Next;
		if ( Data = GetAudioStreamBuffer(SSTREAM(a^.Data)) ) then
		begin
			if ( r := SSTREAM(a^.Data)^.Driver^.ExtractSample(SSTREAM(a^.Data),Data) ) then
			begin
				FreeAudioStreamBuffer(SSTREAM(a^.Data));
				if ( r = 2 ) then
				begin
					_PauseSound(SSTREAM(a^.Data));
					SSTREAM(a^.Data)^.Finished = true;
					ListRemoveItem(Playing,a);
				end;
			end else begin
				_PauseSound(SSTREAM(a^.Data));
				SSTREAM(a^.Data)^.Finished = true;
				ListRemoveItem(Playing,a);
			end;
		end;
		a := n;
	until ( a = Playing^.Last and Playing^.Last );
        *)
end;                                                                        
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Play/Pause stream functions                                                  *)
(*////////////////////////////////////////////////////////////////////////////////*)
function PlaySoundStream( o: PSStream ): l_bool;
begin
       (*
        if not assigned(ListFoundItem(Playing,o)) then
	begin
		ListAdd(Playing,NULL,o,NULL);
		_PlaySound(o);
		PlaySoundStream := true;
	end else
		PlaySoundStream := false;
        *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function PauseSoundStream( o: PSStream ): l_bool;
var a: PListItem;
begin
  (*
	a := ListFoundItem(Playing,o);
	if assigned( a ) then
	begin
		ListRemoveItem(Playing,a);
		_PauseSound(o);
		PauseSoundStream := true;
	end else
		PauseSoundStream := false;
  *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// SStream creation/free                                                        *)
(*////////////////////////////////////////////////////////////////////////////////*)
function NewSoundStream ( Drv: PSStreamDriver; f: PFileStruct; Ex: Pointer; len, bits, stereo, freq: integer ): PSStream;
var	
   stream: PSStream;
   i, bufcount: integer;
   
begin
  (*
   if ((digi_driver) and (digi_driver^.buffer_size)) then
      i := digi_driver^.buffer_size();
   else
      i := 2048;

   if (len >= i) then
      bufcount := 1;
   else
      bufcount = (i + len-1) div len;

   stream := malloc(sizeof(TSStream));
   if not assigned(stream) then
      NewSoundStream NIL
   else

   stream^.Driver 	:= Drv;
   stream^.f 		:= f;
   stream^.Ex 		:= Ex;
   stream^.Playing 	:= false;
   stream^.Finished := false;
   stream^.len      := len;
   stream^.bufcount := bufcount;
   stream^.bufnum   := 0;
   stream^.active   := 1;
   stream^.locked   := NIL;
   
   stream^.samp := create_sample(bits, stereo, freq, len*bufcount*2);

   if not assigned(stream^.samp) then
   begin
      dispose(stream);
      NewSoundStream := NIL;
   end;

   if (bits = 16) then
   begin
     unsigned short *p = stream^.samp^.data;
     i := 0;
     while i < len*bufcount*2 * stereo do begin p[i] = $8000; inc(i); end;
   end
   else begin
      unsigned char *p = stream^.samp^.data;
      i := 0; 
      while i < len*bufcount*2 * stereo do begin p[i] = $80; inc(i); end;
   end;

   NewSoundStream := stream;
   *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function SStreamFree( o:PSStream ): l_bool;
begin
  (*
	if ( o^.Playing ) then PauseSoundStream(o);
	if not assigned( o^.Driver^.Free{(o)} ) then
		SStreamFree := false;
	destroy_sample(o^.samp);
	dispose(o);
  *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Stream position functions                                                    *)
(*////////////////////////////////////////////////////////////////////////////////*)
function SStreamGetPosMsec( o: PSStream; var msec: l_ulong ): l_bool;
begin
	SStreamGetPosMsec := o^.Driver^.GetPosMsec(o,msec);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function SStreamGetLengthMsec( o: PSStream; var msec: l_ulong ): l_bool;
begin
	SStreamGetLengthMsec := o^.Driver^.GetLengthMsec(o,msec);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function SStreamSetPosMsec( o: PSStream; msec: l_ulong ): l_bool;
begin
	o^.Finished := false;
	SStreamSetPosMsec := o^.Driver^.SetPosMsec(o,msec);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure SetVolumeDigi ( Volume: l_int );
begin
  (*
	VolumeDigi := Volume;
	KeySetInt('/USER/SOUND/VOLUMEDIGI',VolumeDigi);
	set_volume(VolumeDigi,VolumeMidi);
  *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure SetVolumeMidi ( Volume: integer );
begin
  (*
        VolumeMidi := Volume;
	KeySetInt('/USER/SOUND/VOLUMEMIDI',VolumeMidi);
	set_volume(VolumeDigi,VolumeMidi);
  *)
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function LibMain( argc: Integer; argv: l_text ): l_bool;
begin
    (*
	SYSEXPORT(NewSoundStream);
	SYSEXPORT(PlaySoundStream);
	SYSEXPORT(PauseSoundStream);
	SYSEXPORT(SStreamGetPosMsec);
	SYSEXPORT(SStreamGetLengthMsec);
	SYSEXPORT(SStreamSetPosMsec);
	SYSEXPORT(SStreamFree);
	SYSEXPORT(VolumeDigi);
	SYSEXPORT(VolumeMidi);
	SYSEXPORT(SetVolumeDigi);
	SYSEXPORT(SetVolumeMidi);
	*)
        (*
        VolumeDigi := KeyGetInt('/USER/SOUND/VOLUMEDIGI',255);
	VolumeMidi := KeyGetInt('/USER/SOUND/VOLUMEMIDI',255);
	
	DebugMessage('Installing sound system...',[]);
	Playing := NewList;
	STask := InstallTask(SoundTask);
	reserve_voices (32, -1);
	DebugMessage('Try to initialise hardware',[]);
	if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, '') = -1) then
	begin
		DebugError('Unable to install sound');
		DebugMessage('Sound library still active');
		LibMain := true;
	end;

	set_volume(VolumeDigi,VolumeMidi);
	DebugMessage('Volume set to %d',[VolumeDigi]);
	DebugMessage('Midi Volume set to %d', [VolumeMidi]);
	DebugMessage('Sound system installed',[]);
        *)
	LibMain := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Close;
begin
	if assigned( STask ) then RemoveTask(STask);
	if assigned( Playing ) then FreeList(Playing);
end;
(*////////////////////////////////////////////////////////////////////////////////*)

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  VolumeDigi := 255;
  VolumeMidi := 255;
  PLaying    := NIL;
  STask      := NIL;
  AppVersion := ULONG_ID(0,0,0,0);

{$endif}
end.
