(**
*	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
*
*	FILE:			tasks.c
*
*	PROJECT:		Phoenix engine - Core
*
*	DESCRIPTION:	Tasks handling and switching.
*
*	CONTRIBUTORS:
*					Lukas Lipka
*
*	TODO:			@
*
*	BUG:			@
*
*	MISTAKE:		@
*
*	IDEA:			@
*)
unit tasks;

interface

uses guitypes;

{$i cdefs.inc}
{$IFDEF LINK_CLIBS}
  {$LINK tasks.o}
{$ENDIF}

type 
  PTask = ^TTask;
  TTaskProc = procedure(aTask: PTask);
  TTask = record

    Enabled : l_bool;
    Task    : TTaskProc;

    Prev    : PTask;
    Next    : PTask;
  end;

{
 .#define ENABLE_TASK(t)  t->Enabled = TRUE
 .#define DISABLE_TASK(t) t->Enabled = FALSE
}

(* that generates events without handling them and runs the
 tasks in tasklist. It's HIGHLY recommended to add this in
 big loops *)

var KEEP_MULTITASK	: procedure; 

(* that adds a task to the task list *)
function   InstallTask(Task: TTaskProc): PTask; {$ifdef LINK_CLIBS}external;{$endif}

(* that removes a task from the task list *)
procedure    RemoveTask(Task: PTask);           {$ifdef LINK_CLIBS}external;{$endif}

(* that keeps multitasking. Use KEEP_MULTITASK macro to keep multitasking alive
 when you're using big loops. Instead of big loops, it's recommended to add a
 temporary task *)
procedure  RunTasks(arg: TArguments);           {$ifdef LINK_CLIBS}external;{$endif}


procedure PollTasks();                          {$ifdef LINK_CLIBS}external;{$endif}

(* tasklist contains all installed tasks *)
var TaskList: PTask;

implementation

{$IFNDEF LINK_CLIBS}
uses
  thread;

{
 .#define KERNEL_FILE
 .#include "kernel.h"
}

const
   LastTask: PTask = NIL;

function InstallTask(Task: TTaskProc): PTask;
var t: PTask;
begin
	t 	:= NEW(PTask);  {testen, ob das funktioniert}
	t^.Enabled	:= TRUE;
	t^.Task		:= Task;
	t^.Next		:= NIL;
	t^.Prev		:= LastTask;

	if not assigned(TaskList) then
		TaskList := t
	else
		LastTask^.Next := t;

	LastTask := t;

	InstallTask := t;
end;

procedure RemoveTask(Task: PTask);
begin
	if assigned(Task) then
	begin

    	if assigned(Task^.Prev) then Task^.Prev^.Next := Task^.Next
	    else TaskList := Task^.Next;

	    if assigned(Task^.Next) then Task^.Next^.Prev := Task^.Prev
    	else LastTask := Task^.Prev;

	    Dispose(Task);
	end;
end;

procedure PollTasks();
var p: PTask;
begin
	p := TaskList;

	while Assigned(p) do   // not nice code?	
	begin
		if (p^.Enabled) then p^.Task(p);

		p := p^.Next;
	end;
end;


procedure RunTasks(arg: TArguments);
begin
	while ( true ) do
	begin
		PollTasks();

		Yield();
	end;
end;


{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  {KEEP_MULTITASK := Yield();}
  TaskList := NIL;

{$ENDIF}
end.
