////////////////////////////////////////////////////////////////////////////////
//
//    Text Manipulation - Core file
//
//  (c) Copyright 2003 Point Mad. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
unit text;
{
 .#include "stdio.h"
 .#include "stdarg.h"
 .#include "types.h"
 .#include "text.h"
 .#include "dynld.h"
 .#include "internal.h"
 .#include "debug.h"
 .#include <string.h>
}
interface

uses
  guitypes;
  
{$i cdefs.inc}

{$IFDEF LINK_CLIBS}
   {$LINK dynld.o}
   {$LINK text.o}
   {$link debug.o}   
{$ENDIF}  

function TextLen	    ( t: l_text ): l_ulong; {$IFDEF LINK_CLIBS}external;{$endif}
function TextChr	    ( t: l_text; c: l_char ): Integer; {$IFDEF LINK_CLIBS}external;{$endif}
function TextRChr	    ( t: l_text; c: l_char ): Integer; {$IFDEF LINK_CLIBS}external;{$endif}
function TextCompare	    ( a, b: l_text ): Longint; {$IFDEF LINK_CLIBS}external;{$endif}
function TextNCompare	    ( a, b: l_text; n: l_ulong ): Longint; {$IFDEF LINK_CLIBS}external;{$endif}
function TextCaseCompare    ( a, b: l_text ): Longint; {$IFDEF LINK_CLIBS}external;{$endif}
function TextNCaseCompare   ( a, b: l_text; n: l_ulong ): Longint; {$IFDEF LINK_CLIBS}external;{$endif}
function TextSqNCaseCompare ( sq,t: l_text; n: l_ulong ): Longint; {$IFDEF LINK_CLIBS}external;{$endif}

function TextCopy           ( d,s: l_text ): l_text; {$IFDEF LINK_CLIBS}external;{$endif}
function TextNCopy	    ( d,s: l_text; n: l_ulong ): l_text; {$IFDEF LINK_CLIBS}external;{$endif}
function TextDup            ( t: l_text ): l_text; {$IFDEF LINK_CLIBS}external;{$endif}
function TextNDup           ( t: l_text; n: l_ulong ): l_text; {$IFDEF LINK_CLIBS}external;{$endif}
function TextCat            ( d,t: l_text ): l_text; {$IFDEF LINK_CLIBS}external;{$endif}
function TextArgs           (szFormat: l_text; args: array of const): l_text; {$IFDEF LINK_CLIBS}external;{$endif}


procedure InitText ();  {$IFDEF LINK_CLIBS}external;{$endif}

function ToLower ( c: l_char ): l_char; {$IFDEF LINK_CLIBS}external;{$endif}


implementation
{$IFNDEF LINK_CLIBS}
{$ifdef OPascal}uses Sysutils{$else}uses OSysutil{$endif};
///** @defgroup kerneltext Text manipulation (kernel)
//        A set of functions for string manipulation
//        @{
//*/
///**     @file
//        ASCII Text Manipulation.
//*/

///**
//*       This is a common buffer text, be careful with it.
//*/
var
 CommonBufferText: array[0..COMBUF_TEXT_SIZE] of char;

///**
//*       Return length of text t
//*/
function TextLen( t: l_text ): l_ulong;
begin
  TextLen := Length(t);
{
        l_text l = t;

        if ( !DCkPt("TextLen.t",t) ) return 0;

        while ( *l ) l++;

        return l-t;
}
end;

///**
//*       Return first occurence of char c in text t, if occurence is not found return 0
//*/
function TextChr( t: l_text; c: l_char ): Integer;
var i: Integer;
begin
        if t = '' then TextChr := 0 else
        if ( c = #0 ) then TextChr := Length(t)+1 else
        begin
          i := 1;
          while ( i <= Length(t) ) do
          begin
                if ( t[i] = c ) then 
                begin 
       
                  TextChr := i; break;
                
                end;

                Inc(i);
          end;

        end;
end;

///**
//*       Return last occurence of char c in text t, if occurence not found return 0
//*/
function TextRChr( t: l_text; c: l_char ): Integer;
var r: l_text; i: Integer;
begin
  r := '';
  if ( t = '' ) then TextRChr := 0 else
  if ( c = #0 ) then TextRChr := Length(t)+1 else
  begin
    i := Length(t);
    while ( i > 0 ) do begin
     if ( t[i] = c ) then begin TextRChr := i; break; end;
     Dec(i);
    end;
  end;
end;

///**
//*       Compare strings a and b, return 0 if identical
//*/
function TextCompare ( a, b: l_text ): Longint;
var la,lb,ia,ib: Integer;
begin
  if ( a = b ) then TextCompare := 0 else
  begin
    la := Length(a); ia := 1;
    lb := Length(b); ib := 1;
    while ( a[ia] = b[ib] ) and (ia <= la) and (ib <= lb) do begin
          if (ia = la) then begin TextCompare := 0; break; end;
          Inc(ia);
          Inc(ib);
    end;
    TextCompare := ia-ib;
  end;
end;
//
//      Compare strings a and b on length n, return 0 if identical
//
function TextNCompare ( a,b: l_text; n: l_ulong ): Longint;
var la,lb,ia,ib: Integer; c,v: l_text;
begin
  la := length(a);
  lb := Length(b);
  if la < lb then ia := la else ia := lb;
  if la < lb then begin c := System.Copy(b,1,ia); v := a; end else begin c := System.Copy(a,1,ia); v := b; end;
  TextNCompare := AnsiCompareStr(c, v);
  {
  if ( a=b) ) then TextNCompare := 0 else
  begin
    i := 1;
    while ( i <= n ) do 
    begin
      if (Length(a) <> Length(b)) and ((la < n) or (lb < n)) then begin TextNCompare := la-lb; break; end;
      if ( !*a ) return 0;
      a++;
      b++;
      n--;
    end;
    TextNCompare := 0;
  end;
  }
end;

///**
//*       Compare strings a and b and dont mind the case, return 0 if identical
//*/                                         
function TextCaseCompare ( a, b: l_text ): Longint;
begin
  TextCaseCompare := AnsiCompareText(a, b);

  {      if ( !DCkPt("TextCaseCompare.a",a) or !DCkPt("TextCaseCompare.b",b) ) return 0;
  while ( tolower( *a) == tolower( *b) ) begin
          if (!*a) return 0;
          a++;
          b++;
  end;
        return tolower( *a )-tolower( *b );
  }
end;

///**
//*        Compare strings a and b on length n and dont mind the case, return 0 if identical
//*/
function TextNCaseCompare ( a, b: l_text; n: l_ulong ): Longint;
var la,lb,ia,ib: Integer; c,v: l_text;
begin
  la := length(a);
  lb := Length(b);
  if la < lb then ia := la else ia := lb;
  if la < lb then begin c := System.Copy(b,1,ia); v := a; end else begin c := System.Copy(a,1,ia); v := b; end;
  TextNCaseCompare := AnsiCompareText(c, v);

  {
  if ( !DCkPt("TextNCaseCompare.a",a) || !DCkPt("TextNCaseCompare.b",b) ) return 0;
  while ( n ) begin
          if ( tolower( *a) != tolower( *b) ) return tolower( *a)-tolower( *b);
          if ( !*a ) return 0;
          a++;
          b++;
          n--;
  end;
  return 0;
  }
end;

///**
//*       Compare a sequence (sq, n) and a text t, return 0 if identical,
//*/
function TextSqNCaseCompare ( sq, t: l_text; n: l_ulong ): Longint;
begin
  TextSqNCaseCompare := TextNCaseCompare(sq, t, n);
  (*
        if ( !DCkPt("TextNCaseCompare.sq",sq) || !DCkPt("TextNCaseCompare.t",t) ) return 0;
  while ( n ) do begin
          if ( tolower( *sq) != tolower( *t ) ) return tolower( *sq )-tolower( *t );
          if ( !*sq ) return 0;
          sq++;
          t++;
          n--;
  end;
  return *t;
  *)
end;

///**
//*       Copy text s to text d, return d
//*/
function TextCopy ( d, s: l_text ): l_text;
var l: l_text;
begin
  TextCopy := s;
  {
  l := d;
  if ( d='' ) or ( s='' ) ) Then TextCopy := #0 else l := d + s; TextCopy := l; end;
  
  while ( *d = *s ) begin
          s++;
          d++;
  end;
  *d = 0;
        return l;
  }
end;
///**
//*       Copy text s to text d of n chars, add '\\0' at end ,return d
//*/
function TextNCopy ( d,s: l_text; n: l_ulong  ): l_text;
var
  CopyStr: l_text;

begin
  CopyStr := System.Copy(s, 1, n);
  System.Insert(CopyStr, d, 1);
  TextNCopy := d;
        {
        l_text l = d;
        if ( !DCkPt("TextNCopy.d",d) || !DCkPt("TextNCopy.s",s) ) return 0;
        while ( ( *d = *s ) && n ) begin
          s++;
          d++;
          n--;
        end;
        *d = 0;
        return l;
        }
end;
///**
//*       Make a copy of string t in a new memory allocation
//*/
function TextDup ( t: l_text ): l_text;
begin
  TextDup := t;
       {
        l_text n;
        if ( !DCkPt("TextDup.t",t) ) return NULL;
        n = (l_text)malloc(TextLen(t)+1);
        if ( !n ) return NULL;
        return TextCopy(n,t);
        }
end;
///**
//*       Make a copy of n chars of text t in a new memory allocation (add '\\0' at end)
//*/
function TextNDup ( t: l_text; n: l_ulong ): l_text;
begin
  TextNDup := System.Copy(t, 1, n);   
       {
        l_text l;
        if ( !DCkPt("TextNDup.t",t) ) return NULL;
        l = (l_text)malloc(n+1);
        if ( !l ) return NULL;
        return TextNCopy(l,t,n);
        }
end;
///**
//*       Add text t at the text d end, text d must have enouf memory allocated
//*/
function TextCat ( d,t: l_text ): l_text; 
var l: l_text;
begin   
   l := d;
   TextCat := l + t;
   {  
   if ( !DCkPt("TextCat.d",d) || !DCkPt("TextCat.t",t) ) return d;
   while ( *d ) d++;
   TextCopy(d,t);
   return l;
   }
end;
/// **
// *       Create a string with args
// */
function TextArgs(szFormat: l_text; args: array of const): l_text;
begin
   {$ifdef OPascal}
   TextArgs := Format(szFormat, args);
   {$else}
   FormatStr(TextArgs, szFormat, Args);
   {$endif}
   (*
        {va_list argptr;}

        if ( !DCkPt("TextArgs.szFormat",szFormat) ) return NULL;

        va_start(argptr, szFormat);
        vsprintf(CommonBufferText, szFormat, argptr);
        va_end(argptr);

        return (l_text)TextDup(CommonBufferText);
   *)
end;
///**
//*       Transform text to upper case
//*/
function TextToUpper( d: l_text ): l_text;
begin
  TextToUpper := UpperCase(d);
   {
   l_text l = d;
   if ( !DCkPt("TextToUpper.d",d) ) return 0;
   while ( *d ){ *d = toupper(*d); d++; }
   return l;
   }
end;
///**
//*       Transform text to lower case
//*/
function TextToLower( d: l_text ): l_text;
begin
  TextToLower := LowerCase(d);
  
        {
        l_text l = d;
        if ( !DCkPt("TextToUpper.d",d) ) return 0;
        while ( *d ){ *d = tolower(*d); d++; }
        return l;
        }
end;
////////////////////////////////////////////////////////////////////////////////
function ToLower ( c: l_char ): l_char;
begin
  ToLower := LowerCase(c);
end;
////////////////////////////////////////////////////////////////////////////////
// Hash a string
function TextHash ( t: l_text ): l_ulong;
var
  i, k: l_ulong;
  len:  l_uint;

begin
        i := 0; k := 0;
        len := Length (t);

        if (len <= 0)
               then TextHash := 0;

        while (k < len) do begin
                i := ((i shl 6) + (ord(t[k]) and $7f)); inc(k) end;

        TextHash := i;
end;
///**
//@
//*/
////////////////////////////////////////////////////////////////////////////////
procedure InitText ();
begin
{
        SYSEXPORT(TextLen);
        SYSEXPORT(TextChr);
        SYSEXPORT(TextCompare);
        SYSEXPORT(TextNCompare);
        SYSEXPORT(TextCaseCompare);
        SYSEXPORT(TextNCaseCompare);
        SYSEXPORT(TextCopy);
        SYSEXPORT(TextNCopy);
        SYSEXPORT(TextDup);
        SYSEXPORT(TextNDup);
        SYSEXPORT(TextCat);
        SYSEXPORT(TextRChr);
        SYSEXPORT(TextArgs);
        SYSEXPORT(TextSqNCaseCompare);
        SYSEXPORT(TextToUpper);
        SYSEXPORT(TextToLower);
        SYSEXPORT(TextHash);
        SYSEXPORT(ToLower);
}
end;
////////////////////////////////////////////////////////////////////////////////
{$ENDIF}

end.
