(*
/////////////////////////////////////////////////////////////////////////////////
//
//	Textbox Core File 
//
//	Aura Milestone 3 
//
//	(c) Copyright 2012 Finn Technologies and Chase Finn. All rights reserved.
//
/////////////////////////////////////////////////////////////////////////////////
*)
unit textbox;
{
 .include "kernel.h"
 .include "widget.h"
 .include "textbox.h"
 .include "label.h"
 .include "scrollbar.h"
 .include "menu.h"
}
interface

uses
  guitypes,timer,dynld,metrics,text,widget,skin,ulabel,menu,scrllbar,grfx,font,
  clipbrd,keyboard,events,mousedrv,allegro;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$LINK widget.o}
   {$LINK textbox.o}
   {$LINK label.o}
   {$LINK scrollbar.o}
   {$LINK menu.o}
{$endif}

type
  PTextbox = ^TTextbox;
  TTextbox = record

	o        : TScrollWidget;

	Font     :  PFONT;

	Flags    : l_ulong;

	Text     : l_text;

	SelPos   : l_ulong;    { zusammen mit shedit auf aktuelle Zeile bezogen }
	SelPosEnd: l_ulong;    { zusammen mit shedit auf aktuelle Zeile bezogen }

	DragPos  : l_ulong;

	ValidMsg : l_ulong;    { nur mit C/C++ Bibliotheken von Bedeutung }

	Tmr      : PTimer;

  end;

const
  TBF_EDITABLE		  = $0001;
  TBF_MULTILINE		  = $0002;
  TBF_TIMERHIDECURSOR =	$0004;
  TBF_PASSWORD	      =	$0008;
  TBF_DRAGING		  = $0010;
  TBF_PREDRAG		  = $0020;

function CreateTextbox(App:PApplication; r:TRect; Flags:l_ulong): PTextbox;
{$ifdef LINK_CLIBS}external;{$endif}

procedure TextBoxSetTextEx ( o:PTextbox; Txt:l_text );
{$ifdef LINK_CLIBS}external;{$endif}

procedure TextBoxSetText ( o:PTextbox; StrFormat: l_text; args: {$ifdef OPascal}array of const{$else}OpenArray{$endif} );
{$ifdef LINK_CLIBS}external;{$endif}

implementation
{$ifndef LINK_CLIBS}
uses
{$ifdef OPascal}
  Sysutils
{$else}
  OSysutil,gdrivers
{$endif}
;
////////////////////////////////////////////////////////////////////////////////
var	AppVersion: l_ulong;
  
const  
  AppName: l_text = 'Textbox Widget';
  nUID: l_text = 'textbox';
  NeededLibs: array[0..6] of string = ( 'widget', 'clipbrd','grfx','skin', 'menu', 'scrllbar', '' );

const
  SkinTextbox: PSkin = NIL;
var TextboxClass: PWidgetClass;

////////////////////////////////////////////////////////////////////////////////
procedure TextBoxGetTextArea ( o:PTextbox; r:PRect );
begin
	if ( UseSkins ) then
		RectAssign(r^,
			PWIDGET(o)^.Absolute.a.x+SkinTextbox^.Left,
			PWIDGET(o)^.Absolute.a.y+SkinTextbox^.Top,
			PWIDGET(o)^.Absolute.b.x-SkinTextbox^.Right,
			PWIDGET(o)^.Absolute.b.y-SkinTextbox^.Bottom)
	else
		RectAssign(r^,
			PWIDGET(o)^.Absolute.a.x+2,
			PWIDGET(o)^.Absolute.a.y+2,
			PWIDGET(o)^.Absolute.b.x-2,
			PWIDGET(o)^.Absolute.b.y-2);
end;
////////////////////////////////////////////////////////////////////////////////
procedure Scroll ( o:PScrollWidget; p:TPoint );
begin
	if ( PWIDGET(o)^.Flags and WFForceBuffer ) = WFForceBuffer then
		WidgetDraw(PWIDGET(o),NIL)
	else begin
		PWIDGET(o)^.Flags := PWIDGET(o)^.Flags or WFForceBuffer;
		WidgetDraw(PWIDGET(o),NIL);
		PWIDGET(o)^.Flags := PWIDGET(o)^.Flags and not WFForceBuffer;
        end;
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxDraw ( o:PWidget; buffer:p_bitmap; w:PRect );
var
  a,t: TRect;
  x,y,x1,x2: l_int;
  txt: l_text;
  pxt: l_ulong;
  axt: l_ulong;
  bxt: l_ulong;
  _end_: l_bool;
  ss,se,f: l_ulong;
begin
  TextBoxGetTextArea(PTEXTBOX(o), @a);
	
  if ( UseSkins ) then
	DrawSkin(buffer, SkinTextbox, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y)
  else begin
	al_rectfill(buffer,w^.a.x,w^.a.y,w^.b.x,w^.b.y,COL_BOXFACE);
	Rect3D(buffer, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y,COL_BOXLIGHT,COL_BOXDARK);
  end;
	
  GetClip(buffer,t);
  SetClip(buffer,a);
  al_set_clip_state(buffer, TRUE);

	
  if ( PTEXTBOX(o)^.Text<>'' ) then begin
    x := a.a.x-SLLWDG(o)^.Pos.x;
    y := (a.a.y)-SLLWDG(o)^.Pos.y;
    txt := PTEXTBOX(o)^.Text; pxt := x;
    f  := Length(txt); pxt := 0; { Anfangsposition auszugebender Text }
    ss := min(PTEXTBOX(o)^.SelPosEnd,PTEXTBOX(o)^.SelPos);
    se := max(PTEXTBOX(o)^.SelPosEnd,PTEXTBOX(o)^.SelPos);
    _end_ := false;
    while ( not _end_ ) do begin
      f := system.pos(#10,txt);
      if ( f = 0 ) then begin f := Length(txt); _end_ := true; end;
      if ( y >= w^.a.y - al_text_height(PTEXTBOX(o)^.Font)) and (y <= w^.b.y ) then begin
      if ( not( pxt > se ) or ( f < ss ) ) then begin
	if ( se = ss ) then begin
          if ((PTEXTBOX(o)^.Flags and TBF_TIMERHIDECURSOR)<>TBF_TIMERHIDECURSOR) then begin
	    x1 := x + FontTextLengthEx(PTEXTBOX(o)^.Font,txt,txt[ss],(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
	    al_vline(buffer,x1,y,y+al_text_height(PTEXTBOX(o)^.Font),makecol(128,128,255));
	  end;
	end else begin
	  axt := max(ss,Length(txt)); bxt := min(se,f);
	  x1 := x + FontTextLengthEx(PTEXTBOX(o)^.Font,txt,txt[axt],(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
	  x2 := x + FontTextLengthEx(PTEXTBOX(o)^.Font,txt,txt[bxt],(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
	  al_rectfill(buffer,x1,y,x2,y+al_text_height(PTEXTBOX(o)^.Font),makecol(128,128,255));
	  if ( PTEXTBOX(o)^.Flags and TBF_TIMERHIDECURSOR ) = TBF_TIMERHIDECURSOR then begin
	    if ( not(pxt > PTEXTBOX(o)^.SelPosEnd) or (f < PTEXTBOX(o)^.SelPosEnd)) then begin
	      x1 := x + FontTextLengthEx(PTEXTBOX(o)^.Font,txt,txt[PTEXTBOX(o)^.SelPosEnd],(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
	      al_vline(buffer,x1,y,y+al_text_height(PTEXTBOX(o)^.Font),makecol(255,255,255));
	    end;
	  end;
        end;
      end;
      if ( o^.Flags and WF_DRAGFOCUS ) = WF_DRAGFOCUS then
	if ( not( pxt > PTEXTBOX(o)^.DragPos ) or ( f < PTEXTBOX(o)^.DragPos ) ) then begin
          x1 := x + FontTextLengthEx(PTEXTBOX(o)^.Font,txt,txt[PTEXTBOX(o)^.DragPos],(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
	  al_vline(buffer,x1,y,y+al_text_height(PTEXTBOX(o)^.Font),makecol(128,128,128));
	end;
	DrawTextEx(buffer,PTEXTBOX(o)^.Font,x,y,txt,txt[f],makecol(0,0,0),(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
      end;
      y := x + al_text_height(PTEXTBOX(o)^.Font)+1;
      pxt := f+1;
  end; End;
  SetClip(buffer,t);
  al_set_clip_state(buffer, TRUE);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxWriteChar ( o: PTextbox; c: l_char );
var
  Temp: l_text;
  Ex: l_text;
  ExPos: l_ulong;
  TxPos: l_ulong;
begin
        TxPos := 1;
        Temp := o^.Text[o^.SelPosEnd];
	SetLength(Ex,TextLen(o^.Text)+2+o^.SelPos-o^.SelPosEnd);
	Ex := System.Copy(o^.Text, 1, o^.SelPos);
        Ex := Ex + Temp;
        Temp := '';
        Ex[o^.SelPos] := c;

	o^.SelPos := o^.SelPos+1;
	o^.SelPosEnd := o^.SelPos;
	o^.DragPos := 1;
	o^.Text := Ex;
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxInserText ( o: PTextbox; Txt: l_text );
var
  l: l_ulong;
  Temp,Ex: l_text;
begin
  (*
	l := TextLen(Txt);
	l_text Temp = TextDup(o^.SelPosEnd);
	l_text Ex = realloc(o^.Text,TextLen(o^.Text)+1+l+o^.SelPos-o^.SelPosEnd);
  *)
        system.Insert(Txt,o^.Text,o^.SelPos);
  (*
        TextCopy(Ex+(o^.SelPos-o^.Text),Txt);
	TextCopy(Ex+(o^.SelPos-o^.Text+l),Temp);
  	free(Temp);
	o^.SelPos = Ex+(o^.SelPos-o^.Text+l);
	o^.SelPosEnd = o^.SelPos;
	o^.DragPos = Ex;
	o^.Text = Ex;
  *)
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxDeleteSelection ( o: PTextbox );
var
  Temp,Ex: l_text; ExPos: l_ulong;
begin
	Temp := system.copy(o^.Text, 1, o^.SelPosEnd);
	{Ex = realloc(o^.Text,TextLen(o^.Text)+o^.SelPos-o^.SelPosEnd+1);}
	Ex := System.Copy(o^.Text, 1, o^.SelPos);
  	Temp := '';
  	if ( o^.DragPos >= o^.SelPos ) then begin
	  	if ( o^.DragPos <= o^.SelPosEnd ) then
	  		o^.DragPos := o^.SelPos
		else
	  		o^.DragPos := o^.SelPos + o^.DragPos-o^.SelPosEnd;
        end;
        {
	o^.DragPos = Ex+(o^.DragPos-o^.Text);
	o^.SelPos := Ex+(o^.SelPos-o^.Text);
        }
        o^.SelPosEnd := o^.SelPos;
	o^.Text := Ex;
end;
////////////////////////////////////////////////////////////////////////////////
function TextBoxGetSelection ( o: PTextbox ): l_text;
begin
   TextBoxGetSelection := TextNDup(o^.Text[o^.SelPos],o^.SelPosEnd-o^.SelPos);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxDeleteMove ( o: PTextbox; Pos,EndPos: l_ulong );
var
  Temp,Ex: l_text;
begin
  if Length(o^.Text) >= o^.SelPosEnd+EndPos then
  Temp := System.Copy(o^.Text,EndPos,EndPos-o^.SelPosEnd);
  SetLength(Ex,TextLen(o^.Text)+abs(o^.SelPos-o^.SelPosEnd));
  Ex := system.copy(o^.Text,o^.SelPos,Pos-o^.SelPos);
  {Ex := Ex + System.Copy(o^.Text,abs(o^SelPos-pos));}
  Temp := '';
  o^.SelPos := o^.SelPos+Pos;
  o^.SelPosEnd := o^.SelPos;
  o^.DragPos := Longint(Addr(Ex));
  o^.Text := Ex;
end;
////////////////////////////////////////////////////////////////////////////////
function TextBoxCalculateMax ( o: PTextbox ): l_bool;
var
        a: TRect;
        Old: TPoint;
        x,y: Integer;
        t: l_text;
        _end_: l_bool;
        f: l_text;
        i: integer;
begin
        x := 0; y := 0; i := 1;

        Old := SLLWDG(o)^.Max;

	TextBoxGetTextArea(PTEXTBOX(o),@a);

	
	if ( o^.Text <> '' ) then begin
		t := o^.Text; f := t;
		_end_ := false;
		while ( not _end_ ) do
                begin
		   f := t[Length(t)]; {TextChr(t,'\n');}
		   if ( f = #10 ) then _end_ := true;
		   y := y + al_text_height(o^.Font)+1;
		   x := max(x,FontTextLengthEx(o^.Font,t,f,(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD));
		   t := system.copy(t,i,length(t));
		end;
	end;

	if ( y > a.b.y-a.a.y ) then
		SLLWDG(o)^.Max.y := y - (a.b.y-a.a.y)
	else
		SLLWDG(o)^.Max.y := 0;

	if ( x > a.b.x-a.a.x ) then
		SLLWDG(o)^.Max.x := x - (a.b.x-a.a.x)
	else
		SLLWDG(o)^.Max.x := 0;

	if ( (Old.x = SLLWDG(o)^.Max.x) and (Old.y = SLLWDG(o)^.Max.y) ) then TextBoxCalculateMax := false;

	if ( SLLWDG(o)^.Pos.x > SLLWDG(o)^.Max.x  ) then ScrollBarSetPosX(SLLWDG(o),SLLWDG(o)^.Max.x);
	if ( SLLWDG(o)^.Pos.y > SLLWDG(o)^.Max.y  ) then ScrollBarSetPosY(SLLWDG(o),SLLWDG(o)^.Max.y);

	TextBoxCalculateMax := true;
end;
////////////////////////////////////////////////////////////////////////////////
function ReachStartOfLine ( t: l_text; l: l_ulong ): l_ulong;
begin
	Dec(l);
	while (l <= Length(t)) and (l > 0) do begin
		if ( t[l] = #10 ) then begin ReachStartOfLine := l+1; exit end else
		Dec(l);
	end;
	ReachStartOfLine := l;
end;

function FindNextLine (var t: l_text; start: l_ulong): l_ulong;
var txt: l_text; finish: l_ulong;
begin
  txt := t;
  while (txt[1] = #13) or (txt[1] = #10) do system.Delete(txt,1,1);
  finish := system.Pos(#13,txt);
  txt := system.Copy(t,start,finish);
  t := txt;
end;

function FindPrevLine(var t: l_text; l: l_ulong): l_ulong;
var txt: l_text; start: l_ulong;
begin
  txt := t; start := l;
  while (txt[l] = #13) or (txt[l] = #10) do dec(l);
  while (txt[l] <> #13) and (txt[l] <> #10) do dec(l);
  if txt[l-1] = #13 then dec(l);
  start := ReachStartOfLine(txt, l);
  txt := system.Copy(txt,start,l);
  t := txt;
end;

////////////////////////////////////////////////////////////////////////////////
procedure CheckSelVisibility ( o: PTextbox; RedrawScll: l_bool );
var a: TRect; x,y: l_long; t: l_text;
begin
	TextBoxGetTextArea(PTEXTBOX(o),@a);
	
	
	if ( SLLWDG(o)^.Max.x > 0 )  or ( SLLWDG(o)^.Max.y > 0 ) then begin
                t := system.copy(o^.Text,1,o^.SelPosEnd);
		LocatePointText(o^.Font,o^.Text,t,y,x,(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
		x := x - SLLWDG(o)^.Pos.x;
		y := y - SLLWDG(o)^.Pos.y;
		if ( x < 0 ) then begin
			ScrollBarSetPosX(SLLWDG(o),max(0,SLLWDG(o)^.Pos.x+x));
			if ( RedrawScll ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Horizontal), NIL);
		end else if ( x >= a.b.x-a.a.x ) then begin
			ScrollBarSetPosX(SLLWDG(o),min(SLLWDG(o)^.Max.x,SLLWDG(o)^.Pos.x+x-(a.b.x-a.a.x)));
			if ( RedrawScll ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Horizontal), NIL);
		end;
		if ( y < 0 ) then begin
			ScrollBarSetPosY(SLLWDG(o),max(0,SLLWDG(o)^.Pos.y+y));
			if ( RedrawScll ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Vertical), NIL);
		end else begin
			y := y + al_text_height(o^.Font);
			if ( y >= a.b.y-a.a.y ) then begin
				ScrollBarSetPosY(SLLWDG(o),min(SLLWDG(o)^.Max.y,SLLWDG(o)^.Pos.y+y-(a.b.y-a.a.y)));
				if ( RedrawScll ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Vertical), NIL);
			end;
		end;
	end;
	
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxRefreshFromTo ( o: PTextbox; st,fn: l_text );
var
  t: l_text;
  r,a: TRect;
begin
  TextBoxGetTextArea(PTEXTBOX(o), @a);
	
  if ( Length(st) > Length(fn) ) then begin
		t := fn;
		fn := st;
		st := t;
  end;

  LocatePointText(o^.Font,o^.Text,st,r.a.y,r.a.x,(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
  LocatePointText(o^.Font,o^.Text,fn,r.b.y,r.b.x,(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
  r.b.x := r.b.x + FontCharLength(o^.Font,fn[1]);
  if ( r.a.y <> r.b.y ) then begin
		r.a.x := a.a.x;
		r.b.x := a.b.x;
	end else begin
		r.a.x := max(a.a.x-SLLWDG(o)^.Pos.x+r.a.x,a.a.x);
		r.b.x := min(a.b.x-SLLWDG(o)^.Pos.x+r.b.x,a.b.x);
        end;

  r.b.y += r.b.y + al_text_height(o^.Font);
  r.a.y := max(a.a.y-SLLWDG(o)^.Pos.y+r.a.y,a.a.y);
  r.b.y := max(a.b.y-SLLWDG(o)^.Pos.y+r.b.y,a.b.y);
  WidgetDraw(PWIDGET(o), @r);
end;
////////////////////////////////////////////////////////////////////////////////
function TextBoxDropAccept (
  o,Caller: PWidget;
  aType:l_ulong;
  var Data;
  SupportedMethods:l_ulong;
  var Method: l_ulong
): l_bool;
begin
  TextBoxDropAccept := false;
  if ((SupportedMethods and DMETHOD_MOVE)=DMETHOD_MOVE) and (aType = DTYPE_TEXT) then
    begin
      Method := DMETHOD_MOVE;
      TextBoxDropAccept := true;
    end;
    if ((SupportedMethods and DMETHOD_COPY)=DMETHOD_COPY) and (aType = DTYPE_TEXT) then
    begin
      Method := DMETHOD_COPY;
      TextBoxDropAccept := true;
    end;
end;
////////////////////////////////////////////////////////////////////////////////
function TextBoxDragBeforeDrop(o,Dest:PWidget; var Data; aType,Method:l_ulong): l_bool;
begin
  if ( o = Dest ) then
    if (PTEXTBOX(o)^.DragPos >= PTEXTBOX(o)^.SelPos) and
       (PTEXTBOX(o)^.DragPos <= PTEXTBOX(o)^.SelPosEnd) then
         TextBoxDragBeforeDrop := false;
	if ( Method = DMETHOD_MOVE ) then TextBoxDeleteSelection(PTEXTBOX(o));
	TextBoxDragBeforeDrop := true;
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxDropData(o,Caller: PWidget; var Data; aType,Method:l_ulong);
begin
  if ( o = Caller ) then
    if ((PTEXTBOX(o)^.DragPos >= PTEXTBOX(o)^.SelPos) and
        (PTEXTBOX(o)^.DragPos <= PTEXTBOX(o)^.SelPosEnd)) then exit;
  PTEXTBOX(o)^.SelPos := PTEXTBOX(o)^.DragPos;
  PTEXTBOX(o)^.SelPosEnd := PTEXTBOX(o)^.DragPos;
  TextBoxInserText(PTEXTBOX(o),l_text(Data));
  WidgetDraw(o, NIL);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxDragAfterDrop (o,Dest: PWidget; var Data; aType, Method: l_ulong);
begin
	if ( o <> Dest ) then
		WidgetDraw(o, NIL);
end;
////////////////////////////////////////////////////////////////////////////////
function ConditonedIntValue(Condition: l_bool; Val1,Val2: l_long): l_long;
begin
  if Condition
     then ConditonedIntValue := Val1
  else
     ConditonedIntValue := Val2;
end;

function TextBoxEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
var
  t: l_text; SOL: l_text;
  w: l_text; wi: l_ulong; Temp: l_text; _pos: Longint; _sol,_eol: Longint;
  a: TRect;
  m: PMenu;
begin
	
	if ( Ev^.aType = EV_MESSAGE ) then begin
		if ( Ev^.Message = MSG_COPY ) then begin
			if not((PTEXTBOX(o)^.Flags and TBF_PASSWORD) = TBF_PASSWORD) then
			ClipboardPut(TYPE_TEXT,PString(TextBoxGetSelection(PTEXTBOX(o))),false);
		end
		else if ((Ev^.Message = MSG_CUT) and ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE)) then begin
			if not((PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD) then
		          ClipboardPut(TYPE_TEXT,PString(TextBoxGetSelection(PTEXTBOX(o))),false);
			TextBoxDeleteSelection(PTEXTBOX(o));
			WidgetDraw(o, NIL);
		end
		else if (Ev^.Message = MSG_PASTE) and ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE) then begin
			TextBoxInserText(PTEXTBOX(o),l_text(ClipboardGet(TYPE_TEXT)));
			WidgetDraw(o, NIL);
		end
		else if ( Ev^.Message = MSG_DELETE ) then begin
			TextBoxDeleteSelection(PTEXTBOX(o));
			WidgetDraw(o, NIL);
		end else if ( Ev^.Message = WEvNotifyFocused ) then begin
			PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags and not TBF_TIMERHIDECURSOR;
			WidgetDraw(o, NIL);
	 	end else if ( Ev^.Message = WEvNotifyLostFocus ) then begin
			PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags or TBF_TIMERHIDECURSOR;
			WidgetDraw(o, NIL);
		end else if ( Ev^.Message = MSG_NOTIFY_AUTO_RESIZE ) then begin
			TextBoxCalculateMax(PTEXTBOX(o));
			TextBoxEventHandler := true;
		end;

		
		TextBoxEventHandler := true;
	end;


	if ( Ev^.aType = EV_MOUSE ) then
	begin
		t := PTEXTBOX(o)^.Text;
		
		TextBoxGetTextArea(PTEXTBOX(o),@a);
	
		if ( Ev^.Message = WEvMouseLeft ) then
		begin
			PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags and not TBF_PREDRAG;
			TextBoxEventHandler := true;
		end;
		if ( Ev^.Message = MSG_DRAGCANCEL ) then
		begin
			WidgetDraw(o, NIL);
			TextBoxEventHandler := true;
		end;
	
		if (Ev^.Message = WEvMouseLeft) and ((o^.Flags and WF_DRAGFOCUS)=WF_DRAGFOCUS) then
		begin
		  WidgetDraw(o, NIL);
		  TextBoxEventHandler := true;
		end;
		
		if (Ev^.Message = WEvMouseMove) and ((o^.Flags and WF_DRAGFOCUS)=WF_DRAGFOCUS) then
		begin
		  w := WhereInTextMultiLines(
                      default_font,
                      Mouse^.State.p.x+SLLWDG(o)^.Pos.x-a.a.x,
                      Mouse^.State.p.y+SLLWDG(o)^.Pos.y-a.a.y,t,
                      '',
                      (PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
                  wi:= StrToInt(w);
                  PTEXTBOX(o)^.DragPos := wi;
			WidgetDraw(o, NIL);
			TextBoxEventHandler := true;
		end;
		
		if ( Ev^.Message = WEvMouseLDown ) then
		begin
			w := WhereInTextMultiLines(default_font,Mouse^.State.p.x+SLLWDG(o)^.Pos.x-a.a.x,Mouse^.State.p.y+SLLWDG(o)^.Pos.y-a.a.y,t,'',(PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
			wi := StrToInt(w);
			if ( (wi > PTEXTBOX(o)^.SelPos) and (wi <= PTEXTBOX(o)^.SelPosEnd) and ( PTEXTBOX(o)^.SelPosEnd <> PTEXTBOX(o)^.SelPos )  ) then
				PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags or TBF_PREDRAG
		        else begin
				PTEXTBOX(o)^.SelPos := wi;
				PTEXTBOX(o)^.SelPosEnd := wi;
			end;
			WidgetDraw(o, NIL);
			MasterWidget := o;
			TextBoxEventHandler := true;
		end;
		if ( Ev^.Message = WEvMouseLUp ) then
		begin
			w := WhereInTextMultiLines(
                           default_font,
                           Mouse^.State.p.x+SLLWDG(o)^.Pos.x-a.a.x,
                           Mouse^.State.p.y+SLLWDG(o)^.Pos.y-a.a.y,
                           t,'',
                           (PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD);
                        wi := StrToInt(w);
			(*if ((PTEXTBOX(o)^.Flags and TBF_DRAGING)=TBF_DRAGING) and
                             ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE) then begin
				PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags and not TBF_DRAGING;
				if ((PTEXTBOX(o)^.DragPos < PTEXTBOX(o)^.SelPos) or
                                   (PTEXTBOX(o)^.DragPos > PTEXTBOX(o)^.SelPosEnd) then begin
					Temp := TextBoxGetSelection(PTEXTBOX(o));
					TextBoxDeleteSelection(PTEXTBOX(o));
					PTEXTBOX(o)^.SelPos := PTEXTBOX(o)^.DragPos;
					PTEXTBOX(o)^.SelPosEnd := PTEXTBOX(o)^.DragPos;
					TextBoxInserText(PTEXTBOX(o),Temp);
					Temp := '';
				end;
			end else *)if ((PTEXTBOX(o)^.Flags and TBF_PREDRAG)=TBF_PREDRAG ) then begin
				PTEXTBOX(o)^.SelPos := wi;
				PTEXTBOX(o)^.SelPosEnd := wi;
				PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags and not TBF_PREDRAG;
                        end else begin
				PTEXTBOX(o)^.SelPosEnd := max(PTEXTBOX(o)^.SelPos,wi);
				PTEXTBOX(o)^.SelPos := min(PTEXTBOX(o)^.SelPos,wi);
			end;
			WidgetDraw(o, NIL);
			MasterWidget := NIL;
			TextBoxEventHandler := true;
		end;
		if (Ev^.Message = WEvMouseMove) and ((Mouse^.State.b and BUTTON_LEFT) = BUTTON_LEFT) then begin
			w := WhereInTextMultiLines(
                           default_font,
                           Mouse^.State.p.x+SLLWDG(o)^.Pos.x-a.a.x,
                           Mouse^.State.p.y+SLLWDG(o)^.Pos.y-a.a.y,t,'',
                           (PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD
                        );
                        wi := StrToInt(w);
			(*if (PTEXTBOX(o)^.Flags and TBF_DRAGING ) = TBF_DRAGING then
				PTEXTBOX(o)^.DragPos := wi;
			else *)if ((PTEXTBOX(o)^.Flags and TBF_PREDRAG)=TBF_PREDRAG) and (PTEXTBOX(o)^.SelPosEnd <> PTEXTBOX(o)^.SelPos) then begin
				(*PTEXTBOX(o)^.DragPos := wi;
				PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags and not TBF_PREDRAG;
				PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags or TBF_DRAGING;*)
				PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags and not TBF_PREDRAG;
				t := TextBoxGetSelection(PTEXTBOX(o));
				DragStart(o,DTYPE_TEXT,t,DMETHOD_COPY or DMETHOD_MOVE);
				t := '';
				MasterWidget := NIL;
                        end else
				PTEXTBOX(o)^.SelPosEnd := wi;
			WidgetDraw(o, NIL);
			CheckSelVisibility(PTEXTBOX(o),(PTEXTBOX(o)^.Flags and TBF_MULTILINE)=TBF_MULTILINE);
			TextBoxEventHandler := true;
		end;

		if ( Ev^.Message = WEvMouseRUp ) then begin { // I love right click ! and you ?? }
                (*  auskommentiert ^. Thomas Schönfelder ^. Grund: Syntax nicht übersetzbar ^. Klammern???
			m := NewMenu(
     			  NewMenuItem(
                            'Copy',NIL,MSG_COPY,
                             (PTEXTBOX(o)^.SelPos<>PTEXTBOX(o)^.SelPosEnd) and not
                              ConditonedIntValue(
                                (PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD,
                                 0,MI_DISABLE),
                             NIL,
     			  NewMenuItem(
                            'Cut',NIL,MSG_CUT,
                            (PTEXTBOX(o)^.SelPos<>PTEXTBOX(o)^.SelPosEnd) and
                            ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE) and not
                            ConditonedIntValue(
                              (PTEXTBOX(o)^.Flags and TBF_PASSWORD)=TBF_PASSWORD),
                               0,MI_DISABLE,
                            NIL,
     			  NewMenuItem(
                            'Paste',NIL,MSG_PASTE,
                            ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE) and
                             ConditonedIntValue(ClipboardGet(TYPE_TEXT,NIL),0,MI_DISABLE),
                             NIL,
     			  NewMenuItemSeparator(
     			  NewMenuItem(
                            'Delete',NIL,MSG_DELETE,
                             (PTEXTBOX(o)^.SelPos<>PTEXTBOX(o)^.SelPosEnd) and
                            ConditonedIntValue((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE,0,MI_DISABLE),
                            NIL,
     			NIL))))));

			PopUpMenu(o^.AppOwner,Mouse^.State.p,m,o,0);
                *)
			TextBoxEventHandler := true;
		end;


	end;

	if ( Ev^.aType = EV_KEYBOARD ) then
	begin
		if ( Ev^.Keyb.Shift and KB_CTRL ) = KB_CTRL then begin
			if ( SCANCODE(Ev,ord('C')) ) then
				WidgetSendSEvent(o, EV_MESSAGE, MSG_COPY, NIL)
			else if ( SCANCODE(Ev,AL_KEY_V) ) then
				WidgetSendSEvent(o, EV_MESSAGE, MSG_PASTE, NIL)
			else if ( SCANCODE(Ev,AL_KEY_X) ) then
				WidgetSendSEvent(o, EV_MESSAGE, MSG_CUT, NIL);

			TextBoxEventHandler := true;
		end;

		if SCANCODE(Ev, AL_KEY_ESC) then
                   if (PTEXTBOX(o)^.ValidMsg > 0) then begin
			WidgetSendSEvent(o, EV_MESSAGE, MSG_CANCEL, NIL);
			TextBoxEventHandler := true;
		end;

		if ( SCANCODE(Ev,AL_KEY_LEFT)) then
		begin
			if ( Ev^.Keyb.Shift and KB_SHIFT ) = KB_SHIFT then
			begin
				if ( PTEXTBOX(o)^.SelPosEnd > Longint(Addr(PTEXTBOX(o)^.Text)) ) then Dec(PTEXTBOX(o)^.SelPosEnd);
			end
			else
			begin
				if ( PTEXTBOX(o)^.SelPos > Longint(Addr(PTEXTBOX(o)^.Text)) ) then Dec(PTEXTBOX(o)^.SelPos);
				PTEXTBOX(o)^.SelPosEnd := PTEXTBOX(o)^.SelPos;
			end;
		end
		else if SCANCODE(Ev,AL_KEY_UP) and ((PTEXTBOX(o)^.Flags and TBF_MULTILINE) = TBF_MULTILINE) then
		begin
			_sol := ReachStartOfLine(PTEXTBOX(o)^.Text,PTEXTBOX(o)^.SelPosEnd);
			if ( Longint(Addr(PTEXTBOX(o)^.Text)) <> _sol ) then begin
				_pos := ReachStartOfLine(PTEXTBOX(o)^.Text,_pos-2);
				SOL := IntToStr(_pos);
				if ( PTEXTBOX(o)^.SelPosEnd-_pos <= _pos-_sol ) then _pos := _pos + PTEXTBOX(o)^.SelPosEnd-_pos;
				if ( Ev^.Keyb.Shift and KB_SHIFT ) = KB_SHIFT then
				begin
					PTEXTBOX(o)^.SelPosEnd := _pos;
				end
				else
				begin
					PTEXTBOX(o)^.SelPos := _pos;
					PTEXTBOX(o)^.SelPosEnd := PTEXTBOX(o)^.SelPos;
				end;
			end;
		end
		else if SCANCODE(Ev,AL_KEY_DOWN) and ((PTEXTBOX(o)^.Flags and TBF_MULTILINE)=TBF_MULTILINE) then
		begin
		    _eol := system.POS(#10, PTEXTBOX(o)^.Text);
			if ( _eol > 0 ) then begin
				_pos := _eol+1;
				{l_text eol = TextChr(pos,'\n');}
				_sol := ReachStartOfLine(PTEXTBOX(o)^.Text,PTEXTBOX(o)^.SelPosEnd);
				_eol := FindNextLine(PTEXTBOX(o)^.Text,_sol);
				if ( _eol <= 0 ) then _eol := FindPrevLine(PTEXTBOX(o)^.Text,_eol);
				{if ( PTEXTBOX(o)^.SelPosEnd-_sol <= eol-EOL ) pos += PTEXTBOX(o)^.SelPosEnd-SOL;}
				if ( Ev^.Keyb.Shift and KB_SHIFT ) = KB_SHIFT then
				begin
					PTEXTBOX(o)^.SelPosEnd := _pos;
				end
				else
				begin
					PTEXTBOX(o)^.SelPos := _pos;
					PTEXTBOX(o)^.SelPosEnd := PTEXTBOX(o)^.SelPos;
				end;
			end;
		end
		else if ( SCANCODE(Ev,AL_KEY_RIGHT) ) then
		begin
			if ( Ev^.Keyb.Shift and KB_SHIFT ) = KB_SHIFT then
			begin
				{if ( PTEXTBOX(o)^.SelPosEnd < PTEXTBOX(o)^.Text+TextLen(PTEXTBOX(o)^.Text) ) then inc(PTEXTBOX(o)^.SelPosEnd);}
			end
			else
			begin
				{if ( PTEXTBOX(o)^.SelPos < PTEXTBOX(o)^.Text+TextLen(PTEXTBOX(o)^.Text) ) then inc(PTEXTBOX(o)^.SelPos);}
				PTEXTBOX(o)^.SelPosEnd := PTEXTBOX(o)^.SelPos;
			end;
		end else begin

			if ( PTEXTBOX(o)^.SelPos > PTEXTBOX(o)^.SelPosEnd ) then
			begin
				_pos := PTEXTBOX(o)^.SelPosEnd;
				PTEXTBOX(o)^.SelPosEnd := PTEXTBOX(o)^.SelPos;
				PTEXTBOX(o)^.SelPos := _pos;
			end;

			if (SCANCODE(Ev,AL_KEY_BACKSPACE) and 
               (PTEXTBOX(o)^.SelPosEnd = PTEXTBOX(o)^.SelPos) and 
               ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE)) then
			begin
				if ( PTEXTBOX(o)^.SelPos <> Longint(Addr(PTEXTBOX(o)^.Text)) ) then
				begin
					TextBoxDeleteMove(PTEXTBOX(o),-1,0);
				end;
			end
			else if (SCANCODE(Ev, AL_KEY_HOME) and ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE)) then
			begin
				PTEXTBOX(o)^.SelPos := Longint(Addr(PTEXTBOX(o)^.Text));
				PTEXTBOX(o)^.SelPosEnd := PTEXTBOX(o)^.SelPos;
			end
			else if (SCANCODE(Ev, AL_KEY_END) and ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE)) then
			begin
				PTEXTBOX(o)^.SelPos := Longint(Addr(PTEXTBOX(o)^.Text));
				PTEXTBOX(o)^.SelPosEnd := PTEXTBOX(o)^.SelPos;
			end
			else if (SCANCODE(Ev,AL_KEY_DEL) and 
                    (PTEXTBOX(o)^.SelPosEnd = PTEXTBOX(o)^.SelPos) and 
                   ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE)) then
			begin
				if ( PTEXTBOX(o)^.SelPosEnd <> ((Longint(Addr(PTEXTBOX(o)^.Text))+Length(PTEXTBOX(o)^.Text)))) then
				begin
					TextBoxDeleteMove(PTEXTBOX(o),0,1);
				end;
			end
			else if ( (SCANCODE(Ev,AL_KEY_DEL) or SCANCODE(Ev,AL_KEY_BACKSPACE)) and ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE)) then begin
		  	   TextBoxDeleteSelection(PTEXTBOX(o));
			end
			else if (SCANCODE(Ev, AL_KEY_ENTER) and ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE)) then
			begin

				if ( PTEXTBOX(o)^.Flags and TBF_MULTILINE ) = TBF_MULTILINE then
					TextBoxWriteChar(PTEXTBOX(o),#13)
				else if ( PTEXTBOX(o)^.ValidMsg>0 ) then
					WidgetSendSEvent(o, EV_MESSAGE, PTEXTBOX(o)^.ValidMsg, NIL);

			end
			else if ((ASCII(@Ev^.Keyb)<>0) and ((PTEXTBOX(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE)) then
			begin
				TextBoxWriteChar(PTEXTBOX(o),chr(ASCII(@Ev^.Keyb)));
				TextBoxRefreshFromTo(PTEXTBOX(o),PTextBox(o)^.Text[PTEXTBOX(o)^.SelPos-1],PTextBox(o)^.Text[PTEXTBOX(o)^.SelPos]);
			end;

	 	end;

		if ( PTEXTBOX(o)^.Flags and TBF_MULTILINE ) = TBF_MULTILINE then begin
		 	if ( TextBoxCalculateMax(PTEXTBOX(o)) ) then begin
			 	CheckSelVisibility(PTEXTBOX(o),true);
			 	WidgetDraw(PWIDGET(SLLWDG(o)^.Vertical), NIL);
	            WidgetDraw(PWIDGET(SLLWDG(o)^.Horizontal), NIL);
		 	end else
		 		CheckSelVisibility(PTEXTBOX(o),true);
 		end else begin
			TextBoxCalculateMax(PTEXTBOX(o));
			CheckSelVisibility(PTEXTBOX(o),false);


		end;

		WidgetDraw(o, NIL);
		TextBoxEventHandler := true;
	end else

	TextBoxEventHandler := ScrollWidgetEventHandler(o,Ev);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxFreeEx ( o: PWidget );
begin
	if ( PTEXTBOX(o)^.Text <> '' ) then PTEXTBOX(o)^.Text := '';
	if assigned( PTEXTBOX(o)^.Tmr ) then KillTimer(PTEXTBOX(o)^.Tmr);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxSetTextEx ( o: PTextbox; Txt: l_text );
begin
  if ( o^.Text <> '' ) then o^.Text := '';
  o^.Text := Txt;
  o^.SelPos := 1;
  o^.SelPosEnd := 1;
  o^.DragPos := 1;
  if ( TextBoxCalculateMax(o) ) then begin
    if assigned( SLLWDG(o)^.Vertical ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Vertical), NIL);
    if assigned( SLLWDG(o)^.Horizontal ) then WidgetDraw(PWIDGET(SLLWDG(o)^.Horizontal), NIL);
  end;
  WidgetDraw(PWIDGET(o), NIL);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxSetText(o:PTextbox; StrFormat:l_text; args:{$ifdef OPascal}array of const{$else}OpenArray{$endif});
var
  Buffer: l_text;
begin
  {$ifdef OPascal}
  Buffer := Format(StrFormat, args);
  {$else}
  FormatStr(Buffer, StrFormat, args);
  {$endif}
  TextBoxSetTextEx(o,Buffer);
end;
////////////////////////////////////////////////////////////////////////////////
procedure TextBoxTimer ( o: Pointer );
begin
  if ((PTextBox(o)^.Flags and TBF_EDITABLE)=TBF_EDITABLE) and
     (((PWIDGET(o)^.Flags and WFFocused)=WFFocused) or
     ((PTextBox(o)^.Flags and TBF_TIMERHIDECURSOR)=TBF_TIMERHIDECURSOR)) then
  begin
    PTextBox(o)^.Flags := PTextBox(o)^.Flags xor TBF_TIMERHIDECURSOR;
    TextBoxRefreshFromTo(
       PTEXTBOX(o),
       PTextBox(o)^.Text[PTEXTBOX(o)^.SelPosEnd-1],
       PTextBox(o)^.Text[PTEXTBOX(o)^.SelPosEnd]);
  end;
end;
////////////////////////////////////////////////////////////////////////////////
function CreateTextbox(App: PApplication; r: TRect; Flags: l_ulong): PTextbox;
var p: PTextbox; IsMultiline: l_ulong;
begin
  New(p);

  if not assigned(p) then CreateTextBox := NIL else
  begin
     if ((Flags and TBF_MULTILINE) <> TBF_MULTILINE) then begin
	if ( UseSkins ) then begin
          if ( r.b.x < r.a.y+al_text_height(default_font)+SkinTextbox^.Top+SkinTextbox^.Bottom ) then
	    RectAssign(&r,r.a.x,r.a.y,r.b.x,r.a.y+al_text_height(default_font)+SkinTextbox^.Top+SkinTextbox^.Bottom)
	  end else
	    if ( r.b.x < r.a.y+al_text_height(default_font)+4 ) then
	      RectAssign(r,r.a.x,r.a.y,r.b.x,r.a.y+al_text_height(default_font)+4);
     end;

     fillchar(p^, sizeof(TTextbox), 0);

     p^.Font := default_font;
     p^.Flags := Flags or TBF_TIMERHIDECURSOR;
     p^.Text := '';
     { IntialiseWidget(App, PWIDGET(p), r, 'Textline'); }
     p^.SelPos := 1;
     p^.SelPosEnd := 1;
     p^.DragPos := 1;
     if (Flags and TBF_MULTILINE) = TBF_MULTILINE then IsMultiline := 3 else IsMultiline := 0;
     InitScrollWidget(SLLWDG(p), TextboxClass, App, r, IsMultiline);

     SLLWDG(p)^.Delta.y := al_text_height(p^.Font)+1;
     SLLWDG(p)^.Delta.x := 20;
     SLLWDG(p)^.Scrolled := {$ifdef fpc}@{$endif}Scroll;

     PWIDGET(p)^.Flags := PWIDGET(p)^.Flags or WFFocusable or WFForceBuffer;
     PWIDGET(p)^.Draw := {$ifdef fpc}@{$endif}TextBoxDraw;
     //PWIDGET(p)^.EventHandler := {$ifdef fpc}@{$endif}TextBoxEventHandler;
     PWIDGET(p)^.FreeEx := {$ifdef fpc}@{$endif}TextBoxFreeEx;

     PWIDGET(p)^.DropAccept 	:= {$ifdef fpc}@{$endif}TextBoxDropAccept;
     PWIDGET(p)^.DragBeforeDrop := {$ifdef fpc}@{$endif}TextBoxDragBeforeDrop;
     PWIDGET(p)^.DropData 	:= {$ifdef fpc}@{$endif}TextBoxDropData;
     PWIDGET(p)^.DragAfterDrop 	:= {$ifdef fpc}@{$endif}TextBoxDragAfterDrop;

     p^.Tmr := NewTimer(App, 500, {$ifdef fpc}@{$endif}TextBoxTimer, p );

     CreateTextBox := p;
  end;
end;

var tb:	PTextbox;
	
function TextboxSetRProp ( o: PWidget; Name: l_text; const Value; Redraw: l_bool ): l_bool;
var IsMultiLine: l_ulong;
begin
        TextboxSetRProp := false;
	if ( TextCompare(Name,'editable') = 0 ) then begin
		if ( l_bool(Value) ) then
		  PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags or TBF_EDITABLE
		else
		  PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags and not TBF_EDITABLE;
		TextboxSetRProp := true;
	end;
	if ( TextCompare(Name,'multiline') = 0 ) then begin
		if ( l_bool(Value) ) then
			PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags or TBF_MULTILINE
		else
			PTEXTBOX(o)^.Flags := PTEXTBOX(o)^.Flags and not TBF_MULTILINE;
		if (PTEXTBOX(o)^.Flags and TBF_MULTILINE) = TBF_MULTILINE
                   then IsMultiLine := 3
                else IsMultiLine := 0;
		ScrollBarChangeType(SLLWDG(o),IsMultiLine);
		
		TextboxSetRProp := true;
	end;
end;
	
function TextboxGetRProp ( o:PWidget; Name: l_text ): Pointer;
var tmp: l_bool;
begin
        TextboxGetRProp := NIL;
	if ( TextCompare(Name,'editable') = 0 ) then begin
		tmp := (PTEXTBOX(o)^.Flags and TBF_EDITABLE) = TBF_EDITABLE;
		TextboxGetRProp := DuplicateBool(tmp);
	end;
	if ( TextCompare(Name,'multiline') = 0 ) then begin
		tmp := (PTEXTBOX(o)^.Flags and TBF_MULTILINE) = TBF_MULTILINE;
		TextboxGetRProp := DuplicateBool(tmp);
	end;
end;

////////////////////////////////////////////////////////////////////////////////
function StdCreateTextbox ( App: PApplication; r: TRect ): PWidget;
begin
	StdCreateTextbox := PWIDGET(CreateTextbox( App, r, 0));
end;
////////////////////////////////////////////////////////////////////////////////
function LibMain ( argc: integer; argv: l_text ) : l_bool;
var r: TRect;
begin
	TextboxClass := CreateWidgetClass( ULONG_ID('T','x','t','B'), ScrollWidgetClass, 0 );
	TextboxClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateTextbox;
	TextboxClass^.SetRProp := {$ifdef fpc}@{$endif}TextboxSetRProp;
	TextboxClass^.GetRProp := {$ifdef fpc}@{$endif}TextboxGetRProp;

	WClassRegisterProp(TextboxClass,PTYPE_BOOL,'editable');
	WClassRegisterProp(TextboxClass,PTYPE_BOOL,'multiline');
	
	SkinTextbox := GetSkinItem('Textbox');
        (*
	APPEXPORT(CreateTextbox);
	APPEXPORT(TextBoxSetText);
	APPEXPORT(TextBoxSetTextEx);
        *)
	(*
	TRect r;
	WidgetSize(r,10,100,300,150),
	tb := CreateTextbox(@Me, r, TBF_EDITABLE of TBF_MULTILINE);
	InsertWidget(DeskTop, PWIDGET(tb));
	WidgetDraw(PWIDGET(tb), NIL);
	SLLWDG(tb)^.Scrolled := {$ifdef fpc}@{$endif}Scroll;
	TextBoxSetText(tb,
          'You see, my softwares works most of times...'+13#10+
          'I think this version works fine, but there is still work.'+#13#10+
          'Of course i will do it. So do not try to make changes in TextBox please.'+#13#10+
          'Anyway have a nice day using OUR GREAT sofware O3(oZone)'
        );
        *)
	LibMain := true;
end;
////////////////////////////////////////////////////////////////////////////////
procedure Close;
begin
	{WidgetDispose(PWIDGET(tb));}
end;
////////////////////////////////////////////////////////////////////////////////
{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion := ULONG_ID(0,1,1,0);

{$endif}  {LINK_CLIBS}
end.
