unit thread;

interface

uses
  guitypes;

{$i cdefs.inc}
{$IFDEF LINK_CLIBS}
  {$LINK thread.o}
{$ENDIF}

type
  TThreadProc = procedure(Args: TArguments);

procedure Yield();
  {$ifdef LINK_CLIBS}external;{$endif}
function NewThread ( proc: TThreadProc; arg: TArguments; stack_length: Integer; priority: Cardinal ): Integer;
  {$ifdef LINK_CLIBS}external;{$endif}
function KillThread ( Id: Integer ): Integer;
  {$ifdef LINK_CLIBS}external;{$endif}
function ThreadId(): Integer;
  {$ifdef LINK_CLIBS}external;{$endif}
procedure PauseMultitasking();
  {$ifdef LINK_CLIBS}external;{$endif}
procedure UnPauseMultitasking();
  {$ifdef LINK_CLIBS}external;{$endif}

function ThreadInit: l_bool;
  {$ifdef LINK_CLIBS}external;{$endif}
procedure ThreadUnInit;
  {$ifdef LINK_CLIBS}external;{$endif}


implementation
{$IFNDEF LINK_CLIBS}
uses timer;


{
 .#include"kernel.h"
 .#include <setjmp.h>
}

{$ifndef LWP_THREAD} 
{$ifdef __DJGPP__}
////////////////////////////////////////////////////////////////////////////////
// Simple Thread System
//
// :( Not working :(
////////////////////////////////////////////////////////////////////////////////

type
  PThread = ^TThread;
  TThread = record
	Proc: TThreadProc;
	Arg : PArgsArray;
	Next: PThread;
	Id  : l_uint;
	ProcState: jmp_buf;
	StackSize: Integer;
  end;

const
  Threads: PThread  = NIL;
  LastThread:PThread = NIL;
  CurThread: PThread = NIL;
  OldThread: PThread = NIL;

  MainThread: PThread = NIL;

  Ids: l_uint  = 0;

(*////////////////////////////////////////////////////////////////////////////////*)
procedure Yield; 
var jmptothread: Boolean;
begin
    jmptothread := false;
	if Assigned( Threads ) then
	begin
		DebugMessage('Prev : %x',[CurThread^.Id]);
		OldThread := CurThread;
		if not Assigned( CurThread )
		  then CurThread := Threads;
		else
			CurThread := CurThread^.Next{?CurThread^.Next:Threads};
		DebugMessage('Old : %x, New : %x, Switch to %x',[OldThread,CurThread,CurThread^.Id]);
		if Assigned( OldThread ) then
		begin
			DebugMessage('Save : %x',[OldThread^.Id]);
			if(setjmp(OldThread^.ProcState) = 0) then jmptothread := true;
	    end;
		if jmptothread then DebugMessage('	Go!',[]);

		if jmptothread then longjmp(CurThread^.ProcState, 1);
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Go; 
begin
  DebugMessage('Let's go!',[]);
  {if Assigned( CurThread ) then if Assigned( CurThread^.Proc ) then CurThread^.Proc(CurThread^.Arg);}
	
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function AllocThread ( proc: TThreadProc; arg: PArgsArray; stack_length: Integer ): PThread;
var o: PThread; Stack: Pointer;
begin
  {PThread o = malloc(sizeof(TThread));}
  New(o);
  if not Assigned( o ) then AllocThread := NIL;
  fillchar(o,sizeof(TThread),0);
  getmem(Stack, stack_length);
  if not Assigned( Stack ) then begin Dispose(o); AllocThread := NIL; end else
  begin
	fillchar(Stack,stack_length,0);

    o^.stacksize := stack_length;
	o^.Proc:=proc;
	o^.Arg:=arg;

	setjmp(o^.ProcState);

	o^.ProcState[0].sp := Stack;
    o^.ProcState[0].pc := proc;

    if not Assigned( Threads ) then
    begin
      Threads    := o;
      LastThread := o;
    end else begin
  	  LastThread^.Next := o;
    end;
    o^.Id = Ids + 1;
    AllocThread := o;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function  NewThread ( proc: TThreadProc; arg: PArgsArray; stack_length: Integer; priority: Cardinal ): Integer;
var o: PThread;
begin
  o := AllocThread(proc,arg,stack_length);
	if Assigned( o ) then
	begin
		DebugMessage('NewThread %x %d',[o,o^.Id]);

		NewThread := LongInt(o);
	end else
		NewThread := 0;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function KillThread ( Id: Integer ): Integer;
var o,p: PThread;
begin
	o := Threads;
	p := NIL;
	while Assigned( o ) do
	begin
		if ( Id = Longint( o ) then
		begin
			if ( p = NIL ) then
			begin
				Threads := o^.Next;
				if ( o = LastThread ) then LastThread := NIL;
				if ( o = CurThread ) then CurThread := NIL;
			end else begin
				p^.Next = o^.Next;
				if ( o = LastThread ) then LastThread := p;
				if ( o = CurThread ) then CurThread := NIL;
			end;
			freemem(o^.ProcState{[0]}.sp,o^.stacksize);
			Dispose(o);
			KillThread := 1;
		end;
		p := o;
		o := o^.Next;
	end;
	KillThread := 0;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function ThreadId: Longint;
begin
	ThreadId := Longint(CurThread);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure PauseMultitasking;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure UnPauseMultitasking;
begin

end;
(*////////////////////////////////////////////////////////////////////////////////*)
function ThreadInit: l_bool;
begin
  MainThread := AllocThread(NIL,NIL,4096);
  CurThread := MainThread;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ThreadUnInit;
begin

end;
(*////////////////////////////////////////////////////////////////////////////////*)
{$endif}  { .(defined __DJGPP__). }
{$else} { .(defined LWP_THREAD) && (defined __DJGPP__). }

(*
////////////////////////////////////////////////////////////////////////////////
// LWP Thread Support
////////////////////////////////////////////////////////////////////////////////
 #include"lwp.h" ----  fr Windows u.a. von Haus aus multitaskingfhige Systeme
*)
function ThreadInit: l_bool;

	DebugMessage ('Intialise pre-emptive multitasking kernel',[]);
	if not(lwp_init(8, RTC8192)) then
	begin
	//	ExError(NIL,0,ERROR_ENABLETOSTARTLWP,'Kernel','LWP not started');
		ThreadInit := false;
	end;
	DebugMessage('Initialised at 8192 Mhz on IRQ 8');
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ThreadUnInit;
begin

end;
(*////////////////////////////////////////////////////////////////////////////////*)

{$endif} { .(defined LWP_THREAD) && (defined __DJGPP__). }


{$ifdef _OZONE__WIN32_}
procedure Yield();
begin

end;

function NewThread ( proc: TThreadProc; arg: TArguments; stack_length: Integer; priority: Cardinal ): Integer;
begin

end;

function KillThread ( Id: Integer ): Integer;
begin

end;

function ThreadId(): Integer;
begin

end;

procedure PauseMultitasking();
begin

end;

procedure UnPauseMultitasking();
begin

end;

function ThreadInit: l_bool;
begin

end;

procedure ThreadUnInit;
begin

end;
{$endif} {_OZONE_WIN32_}
{$ifdef _OZONE__LINUX_}
procedure Yield();
begin

end;

function NewThread ( proc: TThreadProc; arg: Arguments; stack_length: Integer; priority: Cardinal ): Integer;
begin

end;

function KillThread ( Id: Integer ): Integer;
begin

end;

function ThreadId(): Integer;
begin

end;

procedure PauseMultitasking();
begin

end;

procedure UnPauseMultitasking();
begin

end;

function ThreadInit: l_bool;
begin

end;

procedure ThreadUnInit;
begin

end;
{$endif} {_OZONE_LINUX_}

{$ENDIF} {LINK_CLIBS}
end.
