(*
/////////////////////////////////////////////////////////////////////////////////
//
//	Timer Core System
//
//	Aura Milestone 3 
//
//	(c) Copyright 2012 Finn Technologies and Chase Finn. All rights reserved.
//
/////////////////////////////////////////////////////////////////////////////////
*)
unit timer;

interface

uses
  guitypes,dynld,debug;

{$i cdefs.inc}
{$IFDEF LINK_CLIBS}
  {$LINK timer.o}
{$ENDIF}


type
  PDateTimeRec = ^TDateTimeRec;
  TDateTimeRec = record
     tm_mday, tm_mon, tm_year : {$ifdef OPascal}integer{$else}word{$endif};
     tm_hour, tm_min, tm_sec  : {$ifdef Opascal}integer{$else}word{$endif};
  end;

  PArgsArray = TArguments;
  TPollProc = procedure(p: Pointer); 
  PTimer = ^TTimer;
  TTimer = record
	App: PApplication;
	LastCall : TDateTime;
	Period   : TDateTime;
	//Msg: l_ulong;
	Enable   : l_bool;
	Args     : Pointer;
	Poll     : TPollProc;
  end;

(*
  Those timers (PTimer) works in an App thread, if there are too long they
   slow down all other app events, in this case use threading timers.
*)
function  NewTimer ( App: PApplication; Period: l_ubig; Poll: TPollProc; Args: Pointer ): PTimer;
procedure KillTimer ( o: PTimer );

(*
 * How make a Threading Timer
  ----------------------------

procedure MyThreadTimer(Args: PArgsArray);
begin
  while ( true ) do
  begin
		ThreadTimerWait([Period]);    Args evtl als Array of l_ubig, falls alle mglichen Argumente von diesem Typ
		// do stuff....
  end;
end;

// Initialise timer
var
  TimerId: l_int;
  
TimerId = NewThread(@MyThreadTimer,NULL,4096,1);

// Kill timer

KillThread(TimerId);

 *
*)

procedure TimerSetPeriod ( o: PTimer; Period: l_ubig );

procedure ThreadTimerWait(Period: l_ubig);

procedure TimersPoll ( {void} );

procedure InitTimer ( {void} );
procedure ShutDownTimer ( {void} );

procedure SysPoll;

var ATime: TDateTime; 

implementation

{$ifndef LINK_CLIBS}

uses thread,list;
{
 .#define KERNEL_FILE
 .#include"kernel.h"
 .#include <string.h>
}

var
  Timers        : PList;
  TimerThreadId : l_int;
  
const  
  LastDeleted   : PListItem = NIL;

procedure AClock;
begin
        ATime := ATime+1;
end;

procedure TimersPoll;
var
  a, n: PListItem;
begin

   if Assigned( Timers^.Last) then
   begin
        a := Timers^.Last;

                repeat
                
                        n := a^.Next;

                        if ( PTimer(a^.Data)^.Enable ) then
                        begin
                                if ( ATime - PTimer(a^.Data)^.LastCall >= PTimer(a^.Data)^.Period ) then
                                begin
                                        PTimer(a^.Data)^.LastCall := ATime;
                                        PTimer(a^.Data)^.Poll(PTimer(a^.Data)^.Args);
                                end;
                        end;

                        a := n;
                
                until ( a = Timers^.Last );
   end;
end;

(*
procedure TimerThread ( Args: PArgsArray ) 
begin
  while ( true ) do begin
          TimersPoll();
          Yield();
  end;
end;
*)

procedure ThreadTimerWait(Period: l_ubig);
var LastCall: TDateTime;
begin
        LastCall := ATime;
        while ( ATime-LastCall < Period ) do SysPoll;  //Yield();
end;

procedure TimerSetPeriod ( o: PTimer; Period: l_ubig );
begin
        o^.Period := Period;
end;

procedure freetimer(p: Pointer);
begin
   Dispose(PTimer(p));
end;

function NewTimer ( App: PApplication; Period: l_ubig; Poll: TPollProc; Args: Pointer ): PTimer;
var o: PTimer;
begin
        New(o);
        {PTimer o = malloc(sizeof(TTimer));}
        if not Assigned( o ) then NewTimer := NIL;
        fillchar(o^,sizeof(TTimer),0);

        o^.App := App;
        o^.Period := Period;
        o^.LastCall := ATime;
        o^.Poll := Poll;
        o^.Args := Args;
        o^.Enable := true;
        ListAdd(Timers, '', o, {$ifdef fpc}@{$endif}freetimer);

        NewTimer := o;
end;

procedure KillTimer ( o: PTimer );
var i: PListItem;
begin
        i := ListFoundItem(Timers,o);
        o^.Enable := false;

        if Assigned( i ) then ListRemoveItem(Timers,i);
        LastDeleted := i;
end;

procedure InitTimer;
begin
        {//install_int({$ifdef fpc}@{$endif}AClock, 1);}

        Timers := NewList;

        if not Assigned( Timers ) then DebugFatal('ERROR_NOTENOUGHMEMORY',[]);

        {//TimerThreadId = NewThread(TimerThread, NULL, 4096, 1);}
       (*
        SYSEXPORT(NewTimer);
        SYSEXPORT(KillTimer);
        SYSEXPORT(ThreadTimerWait);
        SYSEXPORT(ATime);
       *)
end;

procedure ShutDownTimer;
begin
        //KillThread(TimerThreadId);
        FreeList(Timers);
end;

procedure SysPoll;
begin

end;

{$endif}
end.
