unit toolbar;

interface
{
 .#include 'kernel.h'
 .#include 'widget.h'
 .#include 'toolbar.h'
 .#include 'button.h'
}
uses
  guitypes, widget, button;
  
{$i cdefs.inc}

{$ifdef LINK_CLIBS}
  {$link toolbar.o}
{$endif}  

type
  PToolbar = ^TToolbar;
  TToolbar = record
	o: TWidget;
	Items: PList;
  end;
  TOOLBAR = PToolbar;

type
  TToolbarItem = ^TToolbarItem;
  TToolbarItem = record
	Caption   : l_text;
	Icon      : p_bitmap;
	Message   : l_ulong;

	Flags     : l_ulong;

	BufAbs    : TRect;

	Button    : PButton;
  end;
  TOOLBARITEM = PToolbarItem;

const
  TI_DISABLED		= $00000001;
  TI_TEXTUNDERICON	= $00000002;

function CreateToolbar( App: PApplication; r: TRect ): PToolbar; {$ifdef LINK_CLIBS}external;{$endif}
procedure ToolbarRemoveItem ( b: PToolbar; i: PToolbarItem ); {$ifdef LINK_CLIBS}external;{$endif}
procedure ToolbarAddItem ( b: PToolbar; i: PToolbarItem ); {$ifdef LINK_CLIBS}external;{$endif}
function CreateToolbarItem ( Caption: l_text; Icon: p_bitmap; Message,Flags: l_ulong ): PToolbarItem; {$ifdef LINK_CLIBS}external;{$endif}
procedure ToolbarItemEnable ( t: PToolbar; p: PToolbarItem );  {$ifdef LINK_CLIBS}external;{$endif}
procedure ToolbarItemDisable ( t: PToolbar; p: PToolbarItem );  {$ifdef LINK_CLIBS}external;{$endif}

implementation
{$ifndef LINK_CLIBS}

var	
  AppVersion: l_ulong;
  SkinToolbar: PSkin;
  ToolBarClass: PWidgetClass;
  	
const    
  AppName   : l_text = 'Toolbar Widget';
  nUID      : l_text = 'toolbar';
  NeededLibs: l_uid = ( 'widget', 'skin', 'button', '' );
  
  ITEMSPACE = 5;

procedure ToolbarDraw ( o: PWidget; buffer: p_bitmap; w: PRect );
{
	if (UseSkins)
		DrawSkin (buffer, SkinToolbar, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y);
	else
		rectfill (buffer, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y, COL_3DFACE);
}

function ToolbarEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
{
	ToolbarEventHandler := false;
}

function CreateToolbarItem ( Caption: l_text; Icon: p_bitmap; Message, Flags: l_ulong ): PToolbarItem;
var p: PToolbarItem;
begin
    p = malloc(sizeof(TToolbarItem));
	if not assigned(p) CreateToolbarItem := NIL;
	memset(p, sizeof(TToolbarItem), 0);

	p^.Caption := TextDup(Caption);
	p^.Icon := Icon;
	p^.Message := Message;
	p^.Flags := Flags;

	CreateToolbarItem := p;
end;

procedure UpdateBufferAbs ( c: PToolbar );
var a,b: PListItem; x: integer;
begin
	a := c^.Items^.Last^.Next; b := a;
	x := 0;

	if not assigned(a) then exit;

	repeat
	
		RectAssign(TOOLBARITEM(a^.Data)^.BufAbs, x, 0, x + (PWIDGET(c)^.Absolute.b.y - PWIDGET(c)^.Absolute.a.y), (PWIDGET(c)^.Absolute.b.y - PWIDGET(c)^.Absolute.a.y));
		WidgetMoveToPoint(PWIDGET(TOOLBARITEM(a^.Data)^.Button), TOOLBARITEM(a^.Data)^.BufAbs.a.x, TOOLBARITEM(a^.Data)^.BufAbs.a.y);

		x := x + (PWIDGET(c)^.Absolute.b.y - PWIDGET(c)^.Absolute.a.y)+ITEMSPACE;

		a := a^.Next;
	
	until (a = b);

	WidgetDraw(PWIDGET(c), NIL);
end;

procedure ToolbarAddItem ( b: PToolbar; i: PToolbarItem );
var
	r: TRect;
	c: PButton;
begin
	if ( b^.Items^.Last )
		RectAssign(r, 
          TOOLBARITEM(b^.Items^.Last^.Data)^.BufAbs.b.x+ITEMSPACE, 0, 
          TOOLBARITEM(b^.Items^.Last^.Data)^.BufAbs.b.x + ITEMSPACE + (PWIDGET(b)^.Absolute.b.y - PWIDGET(b)^.Absolute.a.y), 
          (PWIDGET(b)^.Absolute.b.y - PWIDGET(b)^.Absolute.a.y)
        );
	else
		RectAssign(r, 0, 0, (PWIDGET(b)^.Absolute.b.y - PWIDGET(b)^.Absolute.a.y), (PWIDGET(b)^.Absolute.b.y - PWIDGET(b)^.Absolute.a.y));

	c = CreateButton(PWIDGET(b)^.AppOwner, r, i^.Caption, i^.Message);
	InsertWidget(PWIDGET(b), PWIDGET(c));
	if ( i^.Flags and TI_DISABLED) = TI_DISABLED then c^.Flags := c^.Flags or BF_DISABLED;
	if ( i^.Flags and TI_TEXTUNDERICON) = TI_TEXTUNDERICON then c^.Flags := c^.Flags or BF_TEXTUNDERICON;
	WidgetDraw(WIDGET(c), NIL);

	i^.BufAbs := r;
	i^.Button := c;

	if ( i^.Icon )
		c^.Icon := i^.Icon;

	ListAdd(b^.Items, i^.Caption, i, {$ifdef fpc}@{$endif}free);
end;

procedure ToolbarRemoveItem ( b: PToolbar; i: PToolbarItem );
begin
	WidgetDispose(PWIDGET(i^.Button));
	ListRemoveItem( b^.Items, ListFoundItem( b^.Items, i ) );
	UpdateBufferAbs(b);
end;

procedure ToolbarFreeEx ( o: PWidget );
begin
	FreeList(TOOLBAR(o)^.Items);
end;

procedure ToolbarItemEnable ( t: PToolbar; p: PToolbarItem )
begin
	p^.Flags := p^.Flags and (not TI_DISABLED);
	p^.Button^.Flags := p^.Button^.Flags and (not BF_DISABLED);
	UpdateBufferAbs(t);
end;

procedure ToolbarItemDisable ( t: PToolbar; p: PToolbarItem );
begin
	p^.Flags := p^.Flags or TI_DISABLED;
	p^.Button^.Flags := p^.Button^.Flags or BF_DISABLED;
	UpdateBufferAbs(t);
end;

function CreateToolbar( App: PApplication; r: TRect ): PToolbar;
var p: PToolBar;
begin
	New(p);

	if not assigned(p) then CreateToolbar := NIL else

	fillchar(p, sizeof(TToolbar), 0);

	IntialiseWidget(App, ToolBarClass, PWIDGET(p), r );

	PWIDGET(p)^.Draw := {$ifdef fpc}@{$endif}ToolbarDraw;

	p^.Items := NewList();

	CreateToolbar := p;
end;

function LibMain ( int argc, l_text *argv ): l_bool;
begin
	ToolBarClass := CreateWidgetClass( ULONG_ID('T','l','B','r'), StdWidgetClass, WCF_BOOK );
	ToolBarClass^.StdCreate := {(void * )}{$ifdef fpc}@{$endif}CreateToolbar;
	
	
	(**
	*	Skin items
	*)
	SkinToolbar := GetSkinItem('Toolbar');
    (*
	APPEXPORT(CreateToolbar);
	APPEXPORT(ToolbarRemoveItem);
	APPEXPORT(ToolbarAddItem);
	APPEXPORT(CreateToolbarItem);
	APPEXPORT(ToolbarItemEnable);
	APPEXPORT(ToolbarItemDisable);
    *)
	(*TRect r;
	PToolbarItem p1, p2, p3, p4;
	RectAssign(r, 50,70,400,110);
	PToolbar b := CreateToolbar(Me, r);
	InsertWidget(PWIDGET(DeskTop), PWIDGET(b));

	p1 := CreateToolbarItem('Open', LoadImage('SYSTEM/ICONS/pc32.bmp'), 0, 0);
	ToolbarAddItem(b,p1);
	p2 := CreateToolbarItem('Close', LoadImage('SYSTEM/ICONS/pc32.bmp'), 0, 0);
	ToolbarAddItem(b,p2);
	p3 := CreateToolbarItem('Help', LoadImage('SYSTEM/ICONS/pc32.bmp'), 0, 0);
	ToolbarAddItem(b,p3);
	p4 := CreateToolbarItem('Exit', LoadImage('SYSTEM/ICONS/pc32.bmp'), 0, 0);
	ToolbarAddItem(b,p4);

	ToolbarRemoveItem(b,p2);

	WidgetDraw(PWIDGET(b), NIL);*)

	LibMain := true;
end;

procedure Close;
begin

end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion  := ULONG_ID(0,0,0,1);
  SkinToolBar := NIL;


{$endif}  {LINK_CLIBS}
end.
