(*
////////////////////////////////////////////////////////////////////////////////
//
//  Treeview Widget - Core File
//
//	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
unit treeview;
(*
 #include "kernel.h"
 #include "widget.h"
 #include "treeview.h"
 #include "textbox.h"

 #include "scrollbar.h"
 #include "menu.h"
*)

interface

uses
  guitypes,dms,dynld,events,metrics,grfx,widget,skin,font,textbox,scrllbar,menu,
  allegro,mousedrv,text;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
{$LINK 'kernel.o'}
{$LINK 'widget.o'}
{$LINK 'treeview.o'}
{$LINK 'textbox.o'}

{$LINK 'scrollbar.o'}
{$LINK 'menu.o'}
{$endif}

{typedef struct TTreeItem *PTreeItem, *PTreeviewItem;}
(**
	Treeview item structure
*)
type
  PTreeItem = ^TTreeItem;
  PTreeviewItem = PTreeItem;
  TFreeExProc = procedure(i: PTreeViewItem);
  TGenerateProc = procedure(p: PTreeItem);
  TTreeItem = record

	Caption: l_text;    { ///< Caption of item                 }
	Key    : l_text;    { ///< Iternal string to identify item }
	Flags  : Byte;
	Icon   : PIcon;     { ///< Icon of item                    }
	BufAbs : TRect;

	GenerateSubs: TGenerateProc;

	FreeEx : TFreeExProc;

	Last   : PTreeItem;
	Next   : PTreeItem;
	Prev   : PTreeItem;
	Parent : PTreeItem;
	
  end;
  TREEVIEWITEM = PTreeviewItem;


(**
	Treeview widget structure
*)
  PTreeView = ^TTreeview;
  TTreeItemStartDragProc = procedure(o: PWidget);

  TTreeItemDropAcceptFunc = function(o,Caller:PWidget; Item:PTreeItem; aType:l_ulong;
    Data:Pointer; SupportedMethods:l_ulong; var Method:l_ulong ): l_bool;

  TTreeItemDragBeforeDropFunc = function(o,Dest:PWidget; Item:PTreeItem; Data: Pointer; aType,Method: l_ulong): l_bool;
  TTreeItemDropDataProc = procedure(o, Caller: PWidget; Item: PTreeItem; Data: Pointer; aType,Method: l_ulong );
  TTreeItemDragAfterDropProc = procedure(o, Dest: PWidget; Item: PTreeItem; Data: Pointer; aType,Method: l_ulong );
  TTreeView = record

	o: TScrollWidget;

	Items: PTreeItem;     { ///< root item (not displayed)                        }
	Sel  : PTreeItem;     { ///< Selected item                                    }

	OnSelMsg : l_ulong;   { ///< Message send when an item is selected            }
	OnMenuMsg: l_ulong;   { ///< Message send when there is a right click on item }
	
	Flags: l_ulong;
	
	
	DragItem: PTreeItem;
	
	ItemStartDrag: TTreeItemStartDragProc;
	ItemDropAccept: TTreeItemDropAcceptFunc;
	ItemDragBeforeDrop: TTreeItemDragBeforeDropFunc;
	ItemDropData: TTreeItemDropDataProc;
	ItemDragAfterDrop: TTreeItemDragAfterDropProc;
	
  end;


const
 NO_SCROLL         = $10;

 TIF_EXPENDED      = $01;
 TIF_SELECTED      = $02;
 TIF_FREEICON      = $04;
 TIF_FREEARGS      = $08;

 TVF_DRAWPLUSMINUS = 1;
 TVF_DRAWICONS     = 2;
 TVF_ALLOWDRAG     = 4;



function CreateTreeView( App: PApplication; r: TRect; OnSelMsg: l_ulong ): PTreeView;

procedure EmptySubTreeItems ( o:PTreeItem );
function AddTreeItem ( o:PTreeItem; Caption, Key:l_text; Icon: PIcon; GenerateSubs: TGenerateProc ): PTreeItem;

procedure TreeviewExpendItem ( o:PTreeView; i:PTreeItem ) ;
procedure TreeviewUnExpendItem ( o:PTreeView; i:PTreeItem );

function TreeViewRenameItemBox ( t:PTreeView; i:PTreeItem ): l_text;
procedure TreeViewRenameItem ( t:PTreeView; i:PTreeItem; Caption: l_text );
procedure TreeViewDeleteItem ( o:PTreeView; i:PTreeItem );

procedure TreeViewSelectItem ( o:PTreeView; i:PTreeItem );
procedure TreeviewScrollBarRecalculate( o:PTreeView );

function InitTreeView(b:PTreeView; aClass:PWidgetClass; App:PApplication; r:TRect; OnSelMsg: l_ulong ): PTreeView;
function TreeViewEventHandler ( o:PWidget; Ev:PEvent ): l_bool;


procedure TreeviewAddItem ( Parent, i: PTreeItem );
function TreeviewInitItem(
    i:PTreeItem;
    Caption,Key: l_text;
    Icon: PIcon;
    GenerateSubs:TGenerateProc;
    FreeEx: TFreeExProc
    ): PTreeItem;


var TreeviewClass: PWidgetClass;

implementation
{$ifndef LINK_CLIBS}
(*////////////////////////////////////////////////////////////////////////////////*)
var	AppVersion: l_ulong;

const
  AppName: l_text = 'Treeview Widget';
  nUID   : l_text = 'treeview';
  NeededLibs: array[0..4] of l_text = ( 'widget', 'skin','textbox','menu','' );

  SkinTreeview: PSkin  = NIL;

(** @defgroup treeview Treeview widget
	@ingroup widgets
	@{
*/

/**	@file
	Treeview core file.
*)

procedure memset(var m; fillwith: byte; size: longint);
begin
   fillchar(m, size, fillwith);
end;

function TreeviewInitItem ( i:PTreeItem; Caption, Key: l_text; Icon: PIcon; GenerateSubs: TGenerateProc; FreeEx: TFreeExProc): PTreeItem;
begin
	memset(i,0,sizeof(TTreeItem));
	i^.Caption := Caption;
	i^.Key := Key;
	i^.Icon := DuplicateIcon(Icon);
	i^.GenerateSubs := GenerateSubs;
	i^.FreeEx := FreeEx;
	TreeviewInitItem := i;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeviewAddItem ( Parent,i: PTreeItem );
begin
	i^.Parent := Parent;
	if assigned( Parent^.Last ) then begin
		i^.Next := Parent^.Last^.Next;
		i^.Prev := Parent^.Last;
		Parent^.Last^.Next^.Prev := i;
		Parent^.Last^.Next := i;
		Parent^.Last := i;
	end else begin
		i^.Next := i;
		i^.Prev := i;
		Parent^.Last := i;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function AddTreeItem ( o:PTreeItem; Caption, Key: l_text; Icon: PIcon; GenerateSubs: TGenerateProc ): PTreeItem;
var i: PTreeItem;
begin
	New(i);
	if not assigned(i) then AddTreeItem := NIL else
        begin
	TreeviewInitItem(i,Caption,Key,Icon,GenerateSubs,NIL);
	TreeviewAddItem(o,i);
	AddTreeItem := i;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DrawTreeItem(
    Outbmp:p_bitmap;
    t:PTreeview;
    o:PTreeItem;
    Abs:TRect;
var x,y:l_int;
    Flags,Delta,DeltaY:byte;
var n:l_ulong;
    ss:l_ulong;
    SelColor:l_int
    );
var a,b: PTreeItem;
begin

  RectAssign(o^.BufAbs,x,y,x+al_text_length(default_font,o^.Caption)+Delta+2,y+18);

  if ( RectOverlay(o^.BufAbs,Abs) ) then begin

	if assigned(o^.Last) or
           assigned(o^.GenerateSubs) and
          (not(o^.Flags and TIF_EXPENDED)=TIF_EXPENDED) and
          ((Flags and TVF_DRAWPLUSMINUS)=TVF_DRAWPLUSMINUS) then begin
 	  al_rect(Outbmp,o^.BufAbs.a.x+3, o^.BufAbs.a.y+3, o^.BufAbs.a.x+11, o^.BufAbs.a.y+11,makecol(128,128,128));
	  al_hline(Outbmp,o^.BufAbs.a.x+5,o^.BufAbs.a.y+7,o^.BufAbs.a.x+9,makecol(0,0,0));
	  if not((o^.Flags and TIF_EXPENDED)=TIF_EXPENDED) then
                        al_vline(Outbmp,o^.BufAbs.a.x+7,o^.BufAbs.a.y+5,o^.BufAbs.a.y+9,makecol(0,0,0));
	end;

	if ( assigned(o^.Icon) and ((Flags and TVF_DRAWICONS)=TVF_DRAWICONS)) then begin
	   DrawIcon16(Outbmp,o^.Icon,o^.BufAbs.a.x+Delta-16,o^.BufAbs.a.y+1);
	end;

	if (((o^.Flags and TIF_SELECTED)=TIF_SELECTED) or (t^.DragItem = o)) then begin
	    al_rectfill(Outbmp,
               Delta+o^.BufAbs.a.x+2, o^.BufAbs.a.y+DeltaY, o^.BufAbs.b.x, o^.BufAbs.a.y+DeltaY+al_text_height(default_font), SelColor);
	    al_textout_ex(Outbmp, default_font, o^.Caption, Delta+3+o^.BufAbs.a.x, o^.BufAbs.a.y+DeltaY, makecol(255,255,255), -1);
	  end else
	    al_textout_ex(Outbmp, default_font, o^.Caption, Delta+3+o^.BufAbs.a.x, o^.BufAbs.a.y+DeltaY, makecol(0,0,0), -1);
  end;

  if (Flags and TVF_DRAWICONS ) = TVF_DRAWICONS then
		y := y + 19
	else
		y := y + al_text_height(default_font)+1;


	inc(n);


	if ( assigned(o^.Last) and ((o^.Flags and TIF_EXPENDED)=TIF_EXPENDED) ) then begin
	  a := o^.Last^.Next;
	  b := a;
	  x := x + 16;
		repeat
			DrawTreeItem(Outbmp,t,a,Abs,x,y,Flags,Delta,DeltaY,n,ss,SelColor);
			a := a^.Next;
		until ( a = b );
		x := x - 16;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeViewDraw ( o:PWidget; buffer:p_bitmap; w:PRect );
var
  a,b: PTreeItem;
  Delta: Byte;
  DeltaY: Byte;
  SelColor,x,y: l_int;
  n: l_ulong;
begin
	x := o^.Absolute.a.x-SLLWDG(o)^.Pos.x; y := o^.Absolute.a.y-(SLLWDG(o)^.Pos.y);
	Delta := 0; DeltaY := 0;
	if (o^.Flags and WFFocused) = WFFocused then SelColor := makecol(0,0,255) else SelColor := makecol(215,215,215);

	if ( UseSkins and assigned(SkinTreeview) ) then
	  DrawSkin(buffer, SkinTreeview, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y)
	else begin
	  al_rectfill(buffer,w^.a.x,w^.a.y,w^.b.x,w^.b.y,COL_BOXFACE);
	  Rect3D(buffer, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y,COL_BOXLIGHT,COL_BOXDARK);
	end;
	


	if ( ((PTREEVIEW(o)^.Flags and TVF_DRAWPLUSMINUS)=TVF_DRAWPLUSMINUS) and
             ((PTREEVIEW(o)^.Flags and TVF_DRAWICONS)=TVF_DRAWICONS) ) then
		Delta := 32
	else
		Delta := 16;

	if (PTREEVIEW(o)^.Flags and TVF_DRAWICONS)=TVF_DRAWICONS then DeltaY := (18-al_text_height(default_font)) div 2;

	if assigned( PTREEVIEW(o)^.Items^.Last  ) then begin
	  a := PTREEVIEW(o)^.Items^.Last^.Next;
	  b := a;
	  n := 0;

	  repeat
	    DrawTreeItem(buffer,PTREEVIEW(o),a,w^,x,y,PTREEVIEW(o)^.Flags,Delta,DeltaY,n, SLLWDG(o)^.Pos.y,SelColor);

	    a := a^.Next;
	  until ( a = b );

        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure CalculateMaxEx ( o:PTreeItem; var n:l_long; Delta:l_long; var x,mx: l_long );
var a,b: PTreeItem;
begin
	mx := max ( mx, x+al_text_length(default_font,o^.Caption)+Delta+4 );

	Inc(n);
	if ( assigned(o^.Last) and (( o^.Flags and TIF_EXPENDED)=TIF_EXPENDED)) then
        begin
		a := o^.Last^.Next;
		b := a;
		x := x + 16;
		repeat
			CalculateMaxEx(a,n,Delta,x,mx);
			a := a^.Next;
		until ( a = b );
		x := x - 16;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CalculateMax ( o: PTreeView ): l_ulong;
var a,b: PTreeItem; n,x,mx: l_long;
begin
	n := 0;
	x := 0;
	mx := 0;

	if assigned( o^.Items^.Last ) then begin
		a := o^.Items^.Last^.Next; b:= a;
		repeat
			CalculateMaxEx(a,n,32,x,mx);
			a := a^.Next;
		until ( a = b );
        end;
	
	ScrollBarSetMaxY(SLLWDG(o),max(0,((l_long(n*19))-((PWIDGET(o)^.Absolute.b.y-PWIDGET(o)^.Absolute.a.y))-1)));
	ScrollBarSetMaxX(SLLWDG(o),max(0,(mx-(PWIDGET(o)^.Absolute.b.x-PWIDGET(o)^.Absolute.a.x))));


	{//*******}
	if ( SLLWDG(o)^.Pos.x > SLLWDG(o)^.Max.x  ) then ScrollBarSetPosX(SLLWDG(o),SLLWDG(o)^.Max.x);
	if ( SLLWDG(o)^.Pos.y > SLLWDG(o)^.Max.y  ) then ScrollBarSetPosY(SLLWDG(o),SLLWDG(o)^.Max.y);
	{//*******}
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeTreeItem ( o:PTreeItem );
var a,b,n: PTreeItem;
begin
	if ( o^.Caption <> '' ) then o^.Caption := '';
	if ( o^.Key <> '' ) then o^.Key := '';
	if assigned( o^.Icon ) then FreeIcon(o^.Icon);
	if assigned( o^.Last ) then begin
		a := o^.Last^.Next;
		b := a; n := a;
		repeat
			n := a^.Next;
			FreeTreeItem(a);
			a := n;
		until ( a = b );
	end;
	if assigned( o^.FreeEx ) then o^.FreeEx(o);
	Dispose(o);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure EmptySubTreeItems (o: PTreeItem );
var n,a,b: PTreeItem;
begin
	if assigned( o^.Last ) then begin
		a := o^.Last^.Next;
		b := a; n := a;
		repeat
			n := a^.Next;
			FreeTreeItem(a);
			a := n;
		until ( a = b );
	end;
	o^.Last := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeViewFreeEx ( o: PWidget );
begin
   if assigned( PTREEVIEW(o)^.Items ) then FreeTreeItem(PTREEVIEW(o)^.Items);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function FoundItemOnPoint ( o:PTreeItem; p:TPoint; var f:PTreeItem ): l_bool;
var a,b: PTreeItem;
begin
	FoundItemOnPoint := false;
	if ( PointInRect(p,o^.BufAbs) ) then begin
	  f := o;
	  FoundItemOnPoint := true;
        end;

	if (assigned(o^.Last) and ((o^.Flags and TIF_EXPENDED)=TIF_EXPENDED)) then
        begin
		a := o^.Last^.Next;
		b := a;
		repeat
		  if ( FoundItemOnPoint(a,p,f) ) then FoundItemOnPoint := true;
		  a := a^.Next;
		until( a = b );
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeviewScrollBarRecalculate( o:PTreeView );
begin
	CalculateMax(o);

end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeviewExpendItem ( o:PTreeView; i:PTreeItem );
begin
	if assigned( i^.GenerateSubs ) then begin
		{//EmptySubTreeItems(i);}
		i^.GenerateSubs(i);
        end;
	i^.Flags := i^.Flags or TIF_EXPENDED;

	CalculateMax(o);

	WidgetDraw(PWIDGET(o), NIL);

	WidgetDraw(PWIDGET(SLLWDG(o)^.Vertical), NIL);
  WidgetDraw(PWIDGET(SLLWDG(o)^.Horizontal), NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeviewUnExpendItem ( o: PTreeView; i: PTreeItem );
begin
	i^.Flags := i^.Flags and not TIF_EXPENDED;

	CalculateMax(o);

	WidgetDraw(PWIDGET(o), NIL);

	WidgetDraw(PWIDGET(SLLWDG(o)^.Vertical), NIL);
  WidgetDraw(PWIDGET(SLLWDG(o)^.Horizontal), NIL);

end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeViewSelectItem ( o:PTreeView; i:PTreeItem );
begin
	if ( o^.Sel <> i ) then begin
		if assigned( o^.Sel ) then begin
			o^.Sel^.Flags := o^.Sel^.Flags and not TIF_SELECTED;
		end;
	end;

  i^.Flags := i^.Flags or TIF_SELECTED;
  o^.Sel := i;

	if ( (o^.Flags and NO_SCROLL) = NO_SCROLL ) then begin
	WidgetDraw(PWIDGET(o), NIL);
	end

	else begin

	WidgetDraw(PWIDGET(o), NIL);
	ScrollBarSetPosX(SLLWDG(o),i^.BufAbs.a.x-PWIDGET(o)^.Absolute.a.x+SLLWDG(o)^.Pos.x);
	ScrollBarSetPosY(SLLWDG(o),i^.BufAbs.a.y-PWIDGET(o)^.Absolute.a.y+SLLWDG(o)^.Pos.y);
	WidgetDraw(PWIDGET(o), NIL);
	ScrollBarRedraw(SLLWDG(o));
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeViewSelectItemAndTell ( o: PTreeView; i: PTreeItem );
begin
	if ( o^.Sel <> i ) then begin
		if assigned( o^.Sel ) then begin
			o^.Sel^.Flags := o^.Sel^.Flags and not TIF_SELECTED;
		end;
	end;

  i^.Flags := o^.Sel^.Flags or TIF_SELECTED;
  o^.Sel := i;
	
	if ( o^.Flags and NO_SCROLL = NO_SCROLL ) then begin
          WidgetDraw(PWIDGET(o), NIL);
	end
	else begin
	  WidgetDraw(PWIDGET(o), NIL);
	  ScrollBarSetPosX(SLLWDG(o),i^.BufAbs.a.x-PWIDGET(o)^.Absolute.a.x+SLLWDG(o)^.Pos.x);
	  ScrollBarSetPosY(SLLWDG(o),i^.BufAbs.a.y-PWIDGET(o)^.Absolute.a.y+SLLWDG(o)^.Pos.y);
	  WidgetDraw(PWIDGET(o), NIL);
	  ScrollBarRedraw(SLLWDG(o));
	end;
	if ( PTREEVIEW(o)^.OnSelMsg <> 0 ) then
		WidgetSendSEvent(PWIDGET(o), EV_MESSAGE, o^.OnSelMsg, o^.Sel);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeViewGetFirstFromByCharEx ( i: PTreeItem; from: PTreeItem; c:l_char; var ok:l_bool ): PTreeItem;
var r,a,b: PTreeItem;
begin
	r := NIL;
	if ( i^.Caption <> '' ) then if (LowerCase(i^.Caption) = c) and ok then
           TreeViewGetFirstFromByCharEx := i;
	if ( i = from ) then ok := true;
	if assigned(i^.Last) and ((i^.Flags and TIF_EXPENDED)=TIF_EXPENDED) then
        begin
		a := i^.Last^.Next;
		b := a;
		repeat
			if ( r = TreeViewGetFirstFromByCharEx(a,from,c,ok) )
                           then begin TreeViewGetFirstFromByCharEx := r; exit; end;
			a := a^.Next;
		until ( a = b );
	end else TreeViewGetFirstFromByCharEx := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeViewGetFirstByCharEx ( i:PTreeItem; c:l_char ): PTreeItem;
var r,a,b:PTreeItem;
begin
        r := NIL;
	if ( i^.Caption<>'' ) then if ( Lowercase(i^.Caption) = c ) then TreeViewGetFirstByCharEx:=i;
	if assigned(i^.Last) and ((i^.Flags and TIF_EXPENDED)=TIF_EXPENDED) then
        begin
		a := i^.Last^.Next;
		b := a;
		repeat
                  if (r = TreeViewGetFirstByCharEx(a,c))
                     then begin TreeViewGetFirstByCharEx := r; exit; end;
		  a := a^.Next;
		until ( a = b );
	end else TreeViewGetFirstByCharEx := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeViewGetNextEx ( i:PTreeItem; who:PTreeItem; var ok:l_bool ): PTreeItem;
var r,a,b: PTreeItem;
begin
	r := NIL;
	if ( ok ) then TreeViewGetNextEx := i;
	if ( i = who ) then ok := true;
	if assigned(i^.Last) and ((i^.Flags and TIF_EXPENDED)=TIF_EXPENDED) then
        begin
		a := i^.Last^.Next;
		b := a;
		repeat
			if ( r = TreeViewGetNextEx(a,who,ok) )
                           then begin TreeViewGetNextEx := r; exit; end;
			a := a^.Next;
		until ( a = b );
        end else TreeViewGetNextEx := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeViewGetPrevEx ( i, who: PTreeItem; var prev: PTreeItem ): PTreeItem;
var a,b,r: PTreeItem;
begin
	r := NIL;
	if ( i = who ) then TreeViewGetPrevEx := prev;
	prev := i;
	if assigned(i^.Last) and ((i^.Flags and TIF_EXPENDED)=TIF_EXPENDED) then
        begin
		a := i^.Last^.Next;
		b := a;
		repeat
		  if ( r = TreeViewGetPrevEx(a,who,prev) )
                     then begin TreeViewGetPrevEx := r; exit; end;
		  a := a^.Next;
		until ( a = b );
	end else
	TreeViewGetPrevEx := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeViewGetNext ( o: PTreeView; ii: PTreeItem ): PTreeItem;
var ok: l_bool; i: PTreeItem;
begin
	TreeViewGetNext := NIL;
	if assigned( ii ) then begin
		ok := false;
		i := TreeViewGetNextEx(o^.Items,ii,ok);
		if not assigned( i ) then i := ii;
		TreeViewGetNext := i;
	end else if assigned( o^.Items^.Last ) then
		TreeViewGetNext := o^.Items^.Last^.Next;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeViewGetPrev ( o: PTreeView; ii: PTreeItem ): PTreeItem;
var tmp,i: PTreeItem;
begin
	TreeViewGetPrev := NIL;
        if assigned( ii ) then begin
		tmp := NIL;
		i := TreeViewGetPrevEx(o^.Items,ii,tmp);
		if (i = o^.Items) or not assigned(i) then i := ii;
		TreeViewGetPrev := i;
	end else if assigned( o^.Items^.Last ) then
		TreeViewGetPrev := o^.Items^.Last^.Next;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeViewGetByCharFrom ( o: PTreeView; ii:PTreeItem; c: l_char ): PTreeItem;
var ok: l_bool; i: PTreeItem;
begin
	ok := not assigned(ii);
	i := TreeViewGetFirstFromByCharEx(o^.Items,ii,c,ok);
	if not assigned( i ) then i := TreeViewGetFirstByCharEx(o^.Items,c);
	if not assigned( i ) then i := ii;
	TreeViewGetByCharFrom := i;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeViewGetLast ( o: PTreeView ): PTreeItem;
var i: PTreeItem;
begin
        i := PTREEVIEW(o)^.Items^.Last;
	if assigned( i ) then TreeViewGetLast :=  NIL;
	while ( assigned(i^.Last) and ((i^.Flags and TIF_EXPENDED )=TIF_EXPENDED)) do begin
		i := i^.Last;
	end;
	TreeViewGetLast :=  i;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeviewGetItemUnderPoint ( o: PTreeView; p: TPoint ): PTreeItem;
var i: PTreeItem;
begin
        i := NIL;
	if ( FoundItemOnPoint(PTREEVIEW(o)^.Items, p, i) ) then TreeviewGetItemUnderPoint := i;
	TreeviewGetItemUnderPoint := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeviewDropAccept (o,Caller:PWidget;aType:l_ulong;var Data;SupportedMethods:l_ulong;var Method:l_ulong):l_bool;
var old,p: PTreeViewItem;
begin
        if ( PTREEVIEW(o)^.Flags and TVF_ALLOWDRAG ) = TVF_ALLOWDRAG then begin

		p := TreeviewGetItemUnderPoint(PTREEVIEW(o), Mouse^.State.p);
		
		if ( p <> PTREEVIEW(o)^.DragItem ) then begin
			old := PTREEVIEW(o)^.DragItem;
			PTREEVIEW(o)^.DragItem := p;
			if assigned( old ) then WidgetDraw(PWIDGET(o), @(old^.BufAbs));
			if assigned( PTREEVIEW(o)^.DragItem ) then WidgetDraw(PWIDGET(o), @(PTREEVIEW(o)^.DragItem^.BufAbs));
		end;
		
		if not assigned( p ) then TreeviewDropAccept := false;
		
		if ( o = Caller ) then
			if ( p^.Flags and TIF_SELECTED ) = TIF_SELECTED then TreeviewDropAccept := false;
			
		if assigned( PTREEVIEW(o)^.ItemDropAccept ) then
			TreeviewDropAccept := PTREEVIEW(o)^.ItemDropAccept(o,Caller,PTREEVIEW(o)^.DragItem,aType,Pointer(Data),SupportedMethods,Method);
	end else TreeviewDropAccept := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeviewDragBeforeDrop ( o,Dest: PWidget; var Data; aType, Method: l_ulong ): l_bool;
var Item: PTreeItem;
begin
        Item := NIL;
	if ( o = Dest ) then
		if assigned( PTREEVIEW(o)^.DragItem ) then begin
                        Item := PTREEVIEW(o)^.DragItem;
                        if ( PTREEVIEW(o)^.DragItem^.Flags and TIF_SELECTED ) = TIF_SELECTED then
				TreeviewDragBeforeDrop := false;
                end;
        if assigned( PTREEVIEW(o)^.ItemDragBeforeDrop ) then
		TreeviewDragBeforeDrop := PTREEVIEW(o)^.ItemDragBeforeDrop(o,Dest,Item,Pointer(Data),aType,Method)
	else TreeviewDragBeforeDrop := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeviewDropData  ( o, Caller: PWidget; var Data; aType,Method: l_ulong );
var old: PTreeviewItem;
begin
        old := PTREEVIEW(o)^.DragItem;
	
	PTREEVIEW(o)^.DragItem := NIL;
	
	TreeViewSelectItem(PTREEVIEW(o),old);
	
	if assigned( PTREEVIEW(o)^.ItemDropData ) then
		PTREEVIEW(o)^.ItemDropData(o,Caller,old,Pointer(Data),aType,Method);
	
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure TreeviewDragAfterDrop ( o,Dest: PWidget; var Data; aType,Method: l_ulong );
var Item: PTreeItem;
begin
        if o=Dest then Item := PTREEVIEW(o)^.DragItem else Item := NIL;
	if assigned( PTREEVIEW(o)^.ItemDragAfterDrop ) then
		PTREEVIEW(o)^.ItemDragAfterDrop(o,Dest,Item,Pointer(Data),aType,Method);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeViewEventHandler ( o:PWidget; Ev:PEvent ): l_bool;
var i: PTreeItem; old,p: PTreeViewItem;
    sel: l_char;
    SendMsg: l_bool;
    r: TRect;
begin
	if (Ev^.aType = EV_KEYBOARD) then begin
		if ( SCANCODE(Ev,AL_KEY_DOWN) ) then begin
			i := TreeViewGetNext(PTREEVIEW(o),PTREEVIEW(o)^.Sel);
			if assigned( i ) then
				TreeViewSelectItemAndTell(PTREEVIEW(o),i);
		end else if ( SCANCODE(Ev,AL_KEY_UP) ) then begin
			i := TreeViewGetPrev(PTREEVIEW(o),PTREEVIEW(o)^.Sel);
			if assigned( i ) then
				TreeViewSelectItemAndTell(PTREEVIEW(o),i);
		end else if ( SCANCODE(Ev,AL_KEY_HOME) ) then begin
			if assigned( PTREEVIEW(o)^.Items^.Last ) then
				TreeViewSelectItemAndTell(PTREEVIEW(o),PTREEVIEW(o)^.Items^.Last^.Next);
		end else if ( SCANCODE(Ev,AL_KEY_END) ) then begin
			i := TreeViewGetLast(PTREEVIEW(o));
			if assigned( i ) then
				TreeViewSelectItemAndTell(PTREEVIEW(o),i);
		end else if ( SCANCODE(Ev,AL_KEY_ENTER) ) then begin
			if assigned( PTREEVIEW(o)^.Sel ) then begin
				if ( PTREEVIEW(o)^.Sel^.Flags and TIF_EXPENDED ) = TIF_EXPENDED then
					TreeviewUnExpendItem(PTREEVIEW(o),PTREEVIEW(o)^.Sel)
				else
					TreeviewExpendItem(PTREEVIEW(o),PTREEVIEW(o)^.Sel);
			end;
		end else if ( Ev^.Keyb.Shift = 0 ) then begin
			sel := Lowercase(Chr(Ev^.Keyb.Key));
			i := TreeViewGetByCharFrom(PTREEVIEW(o),PTREEVIEW(o)^.Sel,sel);
			if assigned( i ) then
				TreeViewSelectItemAndTell(PTREEVIEW(o),i);
                end;
	end else if (Ev^.aType = EV_MESSAGE) then begin

		if ( Ev^.Message = WEvNotifyLostFocus ) then begin
			WidgetDraw(o, NIL);
			TreeViewEventHandler :=  true;
		end;
		if ( Ev^.Message = WEvNotifyFocused ) then begin
			WidgetDraw(o, NIL);
			TreeViewEventHandler :=  true;
		end;
		if ( Ev^.Message = WEvNotifySameFocus ) then begin

			TreeViewEventHandler :=  true;
		end;

		if ( Ev^.Message = MSG_NOTIFY_AUTO_RESIZE ) then begin
			CalculateMax(PTREEVIEW(o));
			TreeViewEventHandler :=  true;
		end;
        end
	else if ( Ev^.aType = EV_MOUSE ) then begin

		
		if ((Ev^.Message = MSG_DRAGCANCEL) or
                    (Ev^.Message = WEvMouseLeft) and
                   ((o^.Flags and WF_DRAGFOCUS)=WF_DRAGFOCUS)) then
		begin
			old := PTREEVIEW(o)^.DragItem;
			PTREEVIEW(o)^.DragItem := NIL;
			if assigned( old ) then
				WidgetDraw(PWIDGET(o), @(old^.BufAbs));
			TreeViewEventHandler :=  true;
		end;
		
		if ( Ev^.Message = WEvMouseLDClk ) then
		begin
			i := NIL;
			{//l_bool SendMsg = 1;}

			if ( FoundItemOnPoint(PTREEVIEW(o)^.Items, Mouse^.State.p, i) ) then
			begin
				if ( i^.Flags and TIF_EXPENDED )  = TIF_EXPENDED then
					TreeviewUnExpendItem(PTREEVIEW(o), i)
				else
					TreeviewExpendItem(PTREEVIEW(o), i);
			end;

			TreeViewEventHandler :=  true;
		end
		else if ((Ev^.Message = WEvMouseMove) and
                         ((Mouse^.State.b and BUTTON_LEFT)=BUTTON_LEFT) and
                         not((o^.Flags and WF_DRAGFOCUS)=WF_DRAGFOCUS)) then
		begin	
			if ( PTREEVIEW(o)^.Flags and TVF_ALLOWDRAG ) = TVF_ALLOWDRAG then
                        begin
				p := TreeviewGetItemUnderPoint(PTREEVIEW(o), Mouse^.State.p);
				if assigned( p ) then
					if ( p^.Flags and TIF_SELECTED ) = TIF_SELECTED then
                                        begin
						PTREEVIEW(o)^.DragItem := NIL;
						
						if assigned( PTREEVIEW(o)^.ItemStartDrag ) then
							PTREEVIEW(o)^.ItemStartDrag(o);
					end;
			end;
		
		end
		else if ( (Ev^.Message = WEvMouseLDown) or (Ev^.Message = WEvMouseRDown) ) then
		begin
			i := NIL;
			SendMsg := true;

			if ( FoundItemOnPoint(PTREEVIEW(o)^.Items,Mouse^.State.p,i) ) then
			begin
				
				RectAssign (r, i^.BufAbs.a.x+3, i^.BufAbs.a.y+3, i^.BufAbs.a.x+11, i^.BufAbs.a.y+11);
				if ( CURSOR_IN_RECT (r) ) then
				begin
					if ( FoundItemOnPoint(PTREEVIEW(o)^.Items, Mouse^.State.p, i) ) then
					begin
						if ( i^.Flags and TIF_EXPENDED ) = TIF_EXPENDED then
							TreeviewUnExpendItem(PTREEVIEW(o), i)
						else
							TreeviewExpendItem(PTREEVIEW(o), i);
					end;
					SendMsg := false;
			  	end else begin
					if ( PTREEVIEW(o)^.Sel <> i ) then
					begin
						if assigned( PTREEVIEW(o)^.Sel ) then
						begin
							PTREEVIEW(o)^.Sel^.Flags := PTREEVIEW(o)^.Sel^.Flags and not TIF_SELECTED;
							WidgetDraw(o, @(PTREEVIEW(o)^.Sel^.BufAbs));
						end;
					end
					else
						SendMsg := false;
					
					i^.Flags := i^.Flags or TIF_SELECTED;
					WidgetDraw(o, @(i^.BufAbs));
					PTREEVIEW(o)^.Sel := i;
		  		end;

			end
			else if assigned( PTREEVIEW(o)^.Sel ) then
			begin
				PTREEVIEW(o)^.Sel^.Flags := PTREEVIEW(o)^.Sel^.Flags and not TIF_SELECTED;
				WidgetDraw(o, @(PTREEVIEW(o)^.Sel^.BufAbs));
				PTREEVIEW(o)^.Sel := NIL;
			end;

			if ( (PTREEVIEW(o)^.OnSelMsg<>0) and SendMsg  ) then
					WidgetSendSEvent(o, EV_MESSAGE, PTREEVIEW(o)^.OnSelMsg, PTREEVIEW(o)^.Sel);

			if ( Ev^.Message = WEvMouseRDown ) then begin
				if ( PTREEVIEW(o)^.OnMenuMsg <> 0 ) then begin
					if assigned( PTREEVIEW(o)^.Sel ) then
						WidgetSendSEvent(o, EV_MESSAGE, PTREEVIEW(o)^.OnMenuMsg, PTREEVIEW(o)^.Sel);
				end;
			end;
			TreeViewEventHandler :=  true;
		end;
	end;
	TreeViewEventHandler := ScrollWidgetEventHandler(o,Ev);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Scroll ( o: PScrollWidget; p: TPoint );
begin
	{// To prevent blicking, force temporaly double buffer ! }

	if ( PWIDGET(o)^.Flags and WFForceBuffer ) = WFForceBuffer then
		WidgetDraw(PWIDGET(o),NIL)
	else begin
		PWIDGET(o)^.Flags := PWIDGET(o)^.Flags or WFForceBuffer;
		WidgetDraw(PWIDGET(o),NIL);
		PWIDGET(o)^.Flags := PWIDGET(o)^.Flags and not WFForceBuffer;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function TreeViewRenameItemBox ( t:PTreeView; i:PTreeItem ): l_text;
var
	tb: PTextbox;
	r: TRect;
	ret: l_text;
begin
        ret := '';
	RectAssign(r,i^.BufAbs.a.x+32,i^.BufAbs.a.y+2,PWIDGET(t)^.Absolute.b.x-32,i^.BufAbs.b.y-2);
	tb := CreateTextbox(@Me,r,TBF_EDITABLE);
	InsertWidget(DeskTop, PWIDGET(tb));
	TextBoxSetTextEx(tb,i^.Caption);

	tb^.SelPos := l_ulong(Addr(tb^.Text));
	tb^.SelPosEnd := l_ulong(Addr(tb^.Text))+Length(i^.Caption);
	WidgetDraw(PWIDGET(tb), NIL);

	tb^.ValidMsg := MSG_OK;


	WidgetSetFocus(PWIDGET(tb));

	if ( WidgetExecute(PWIDGET(tb)) <> MSG_CANCEL ) then begin
		if ( TextCaseCompare(tb^.Text,i^.Caption)<>0 ) then
			ret := tb^.Text;
	end;
	WidgetDispose(PWIDGET(tb));

	TreeViewRenameItemBox := ret;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
(**
	Changes the caption of a treeview item
	@param t Treeview owning the item
	@param i Item to affect
	@param Caption New caption
	@note Redraws scrolls bars and treeview
*)
procedure TreeViewRenameItem ( t:PTreeView; i:PTreeItem; Caption:l_text );
begin
	i^.Caption := '';
	i^.Caption := Caption;

	WidgetDraw(PWIDGET(t), NIL);

	WidgetDraw(PWIDGET(SLLWDG(t)^.Vertical), NIL);
  WidgetDraw(PWIDGET(SLLWDG(t)^.Horizontal), NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(**
	Removes the specified treeview item
	@param o Treeview owning the item
	@param i Item to remove
	@note Redraws scrolls bars and treeview
*)
procedure TreeViewDeleteItem ( o:PTreeView; i:PTreeItem );
var p: PTreeItem;
begin
  p := i^.Parent;

  if ( o^.Sel = i ) then begin
		if ( i^.Next = i ) then begin
			if ( i^.Parent = o^.Items ) then
				o^.Sel := NIL
			else
				o^.Sel := i^.Parent;
		end else
			o^.Sel := i^.Next;

		if assigned( o^.Sel ) then o^.Sel^.Flags := o^.Sel^.Flags or TIF_SELECTED;
  end;

  if ( i^.Next = i ) then begin
	  p^.Last := NIL;
	  if ( p^.Flags and TIF_EXPENDED ) = TIF_EXPENDED then p^.Flags := p^.Flags and not TIF_EXPENDED;
  end else begin
	  i^.Next^.Prev := i^.Prev;
	  i^.Prev^.Next := i^.Next;
	  if ( p^.Last = i ) then p^.Last := i^.Prev;
  end;
  WidgetDraw(PWIDGET(o), NIL);
  WidgetDraw(PWIDGET(SLLWDG(o)^.Vertical), NIL);
  WidgetDraw(PWIDGET(SLLWDG(o)^.Horizontal), NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(**
	Initialises a treeview widget
	@param b Treeview to initialise
	@param Class Class of widget (TreeviewClass by default)
	@param App Application which want to allocate the widget( use &Me in most cases)
	@param r Relative coordinates of new widget
	@param OnSelMsg Message send when an item is selected
	@TreeViewEventHandler :=  b
	@see WidgetInsert
*)
function InitTreeView( b:PTreeView; aClass:PWidgetClass; App:PApplication; r:TRect; OnSelMsg:l_ulong ): PTreeView;
var WClass: PWidgetClass;
begin

	New(b^.Items); {=> Getmem(b^.Items,sizeof(TTreeItem));}
	fillchar(b^.Items^,sizeof(TTreeItem),0);

	b^.Items^.Flags := b^.Items^.Flags or TIF_EXPENDED;

	b^.OnSelMsg := OnSelMsg;
	b^.Flags := TVF_DRAWPLUSMINUS or TVF_DRAWICONS;

	if( PWIDGET(b)^.Flags and NO_SCROLL = NO_SCROLL ) then begin
        	PWIDGET(b)^.Draw := {$ifdef fpc}@{$endif}TreeViewDraw;
        	PWIDGET(b)^.EventHandler := {$ifdef fpc}@{$endif}TreeViewEventHandler;
        	PWIDGET(b)^.FreeEx := {$ifdef fpc}@{$endif}TreeViewFreeEx;
        	PWIDGET(b)^.Flags := PWIDGET(b)^.Flags or WFFocusable;
	end
	else begin
	   PWIDGET(b)^.Draw := {$ifdef fpc}@{$endif}TreeViewDraw;
	   PWIDGET(b)^.EventHandler := {$ifdef fpc}@{$endif}TreeViewEventHandler;
	   PWIDGET(b)^.FreeEx := {$ifdef fpc}@{$endif}TreeViewFreeEx;
	   PWIDGET(b)^.Flags := PWIDGET(b)^.Flags or WFFocusable;
	   if Assigned(aClass) then WClass := aClass else WClass := TreeViewClass;
	   InitScrollWidget(SLLWDG(b), WClass, App, r, 3);
           {Class?Class:TreeviewClass}
	   PWIDGET(b)^.Draw := {$ifdef fpc}@{$endif}TreeViewDraw;
	   PWIDGET(b)^.EventHandler := {$ifdef fpc}@{$endif}TreeViewEventHandler;
	   PWIDGET(b)^.FreeEx := {$ifdef fpc}@{$endif}TreeViewFreeEx;
	   PWIDGET(b)^.Flags := PWIDGET(b)^.Flags or WFFocusable;

	   SLLWDG(b)^.Scrolled := {$ifdef fpc}@{$endif}Scroll;
	   SLLWDG(b)^.Delta.y := 19;
	   SLLWDG(b)^.Delta.x := 16;
	end;
	
	PWIDGET(b)^.DropAccept 		:= {$ifdef fpc}@{$endif}TreeviewDropAccept;
	PWIDGET(b)^.DragBeforeDrop 	:= {$ifdef fpc}@{$endif}TreeviewDragBeforeDrop;
	PWIDGET(b)^.DropData 		:= {$ifdef fpc}@{$endif}TreeviewDropData;
	PWIDGET(b)^.DragAfterDrop 	:= {$ifdef fpc}@{$endif}TreeviewDragAfterDrop;
	
	InitTreeView := b;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(**
	Creates a new treeview widget
	@param App Application which want to allocate the widget( use &Me in most cases)
	@param r Relative coordinates of new widget
	@param OnSelMsg Message send when an item is selected
	@CreateTreeView :=  Pointer to the new treeview
	@see WidgetInsert
*)
function CreateTreeView( App:PApplication; r:TRect; OnSelMsg:l_ulong ): PTreeView;
var b: PTreeView;
begin
	New(b);

	if not assigned(b) then CreateTreeView := NIL else

	{ //r.b.x := r.b.x - 10; }

	fillchar(b^, sizeof(TTreeView), 0);

	InitTreeView(b,TreeviewClass,App,r,OnSelMsg);

	CreateTreeView := b;
end;
(**
@}
*)
(*////////////////////////////////////////////////////////////////////////////////*)
function StdCreateTreeView ( App:PApplication; r:TRect ): PWidget;
begin
	StdCreateTreeView := PWIDGET(CreateTreeView( App, r, 0));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function LibMain ( argc: Integer; argv: l_text ): l_bool;
begin
	
	SkinTreeview := GetSkinItem('Treeview');
	
	TreeviewClass := CreateWidgetClass( ULONG_ID('T','r','e','V'), ScrollWidgetClass, 0 );
	TreeviewClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateTreeView;
        (*
	APPEXPORT(CreateTreeView);
	APPEXPORT(EmptySubTreeItems);
	APPEXPORT(AddTreeItem);
	APPEXPORT(TreeviewExpendItem);
	APPEXPORT(TreeviewUnExpendItem);
	APPEXPORT(TreeViewRenameItemBox);
	APPEXPORT(TreeViewSelectItem);
	APPEXPORT(TreeViewRenameItem);
	APPEXPORT(TreeViewDeleteItem);

	APPEXPORT(TreeviewScrollBarRecalculate);
	APPEXPORT(InitTreeView);
	APPEXPORT(TreeViewEventHandler);
	APPEXPORT(TreeviewClass);
	
	APPEXPORT(TreeviewAddItem);
	APPEXPORT(TreeviewInitItem);
	*)
	LibMain := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Close;
begin

end;
(*////////////////////////////////////////////////////////////////////////////////*)

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
   AppVersion := ULONG_ID(0,0,0,1);


{$endif}  {LINK_CLIBS}
end.
