(*
////////////////////////////////////////////////////////////////////////////////
//
//  Label Widget - Core File
//
//	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
unit ulabel;
{
 .#include 'kernel.h'
 .#include 'widget.h'
 .#include 'label.h'
 .#include 'gfx.h'
 .#include 'internal.h'

 .#include 'widget.h'
}

interface

uses guitypes,dynld,text,metrics,grfx,widget,font,skin,allegro;

{$i cdefs.inc}

{$ifdef LINK_CLIBS}
  {$link widget.o}
  {$link label.o}
  {$link gfx.o}
{$endif}

type
  PLabel = ^TLabel;
  TLabel = record

	o         :	TWidget;

	Caption   : l_text;
	FontColor : l_color;

  end;

{//type LABEL = PLabel;}

type
  PImageLabel = ^TImageLabel;
  TImageLabel = record

	o     :	TWidget;

	Ico   : PBITMAP;
	Flags :l_ulong;

  end;

type IMAGELABEL	= PImageLabel;

const ILF_FRAME	= $00000001;

function CreateLabel( App: PApplication; r: TRect; Caption: l_text ): PLabel;
{$ifdef LINK_CLIBS}external;{$endif}

procedure LabelSetText ( l: PLabel;  StrFormat: l_text; args: {$ifdef OPascal}array of const{$else}OpenArray{$endif} );
{$ifdef LINK_CLIBS}external;{$endif}

procedure LabelSetTextEx ( l: PLabel; Caption: l_text );
{$ifdef LINK_CLIBS}external;{$endif}

function CreateImageLabel( App: PApplication; r: TRect; ico: PBITMAP ): PImageLabel;
{$ifdef LINK_CLIBS}external;{$endif}

function LabelGetRProp ( o:PWidget; Name: l_text ): Pointer;
{$ifdef LINK_CLIBS}external;{$endif}

implementation
{$ifndef LINK_CLIBS}
{$ifdef OPascal}
uses Sysutils;
{$else}
uses OSysutil;
{$endif}

var AppVersion: l_ulong;

const
  AppName   : l_text = 'Label Widget';
  nUID      : l_text = 'label';
  {NeededLibs: l_uid  = ( 'widget', 'grfx', '' );}

var
  LabelClass: PWidgetClass;
  ImageLabelClass: PWidgetClass;

procedure LabelDraw ( o: PWidget; buffer: p_bitmap; w: PRect );
var t,lt,tmp: l_text; y,ln,h,ix_t,ix_e: l_ulong;
begin

	al_rectfill(buffer, w^.a.x, w^.a.y, w^.b.x, w^.b.y, o^.BackgroundColor);
	
	if ( PLABEL(o)^.Caption <> '' ) then
	begin
		t   := #10; {PLABEL(o)^.Caption;}
		lt  := PLABEL(o)^.Caption;
		tmp := #0;
		y   := o^.Absolute.a.y;
		ln  := 0;
		h   := al_text_height(default_font);
        (*
		while ( ln < Length(t) ) do
		begin
			if t[ln+1] = #10 then
			if ( t ) t++;
			inc(ln);
		end;
        

	        t  := PLABEL(o)^.Caption;
        *)
                ix_t := 1;
                (*
                while ( ix_t <= Length(t) ) do
		begin
			lt := t;

			ix_e := system.pos(#10, t);
                end;
                *)

  		if ( t[ix_t]<>'' ) then
                begin
				DrawTextEx(buffer,default_font,o^.Absolute.a.x, y,lt,t,PLABEL(o)^.FontColor,true);
				inc(ix_t);
				y := y + h;
		end else
			  al_textout_ex(buffer, default_font, lt, o^.Absolute.a.x, y, PLABEL(o)^.FontColor, -1);

	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure LabelSetTextEx ( l: PLabel; Caption: l_text ); 
begin
  if ( l^.Caption <> '' ) then l^.Caption := '';
	l^.Caption := Caption;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure LabelSetText ( l: PLabel; StrFormat: l_text; args: {$ifdef OPascal}array of const{$else}OpenArray{$endif} );
var LabelCaption: l_text;
begin
  { vsprintf(CommonBufferText, StrFormat, arg); }
  {$ifdef OPascal}
  LabelCaption := Format(StrFormat, args);
  {$else}
  FormatStr(LabelCaption, StrFormat, args);
  {$endif}
  LabelSetTextEx(l,LabelCaption);
  WidgetDraw(PWIDGET(l),NIL);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CreateLabel( App: PApplication; r: TRect; Caption: l_text ): PLabel;
var
  b: PLabel;
	
begin
  New(b);
  if not assigned(b) then CreateLabel := NIL else
  begin
	fillchar(b^,sizeof(TLabel),0);
	b^.Caption := Caption;
	b^.FontColor := makecol(0,0,0);

	IntialiseWidget(App, LabelClass, PWIDGET(b), r);
	PWIDGET(b)^.Draw := {$ifdef fpc}@{$endif}LabelDraw;
	PWIDGET(b)^.BackgroundColor := COL_3DFACE;
	CreateLabel := b;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure ImageLabelDraw ( o: PWidget; buffer: p_bitmap; w: PRect );
begin
	al_rectfill(buffer, w^.a.x, w^.a.y, w^.b.x, w^.b.y, o^.BackgroundColor);

	if assigned( PIMAGELABEL(o)^.Ico ) then
           al_draw_sprite(buffer,PIMAGELABEL(o)^.Ico,o^.Absolute.a.x,o^.Absolute.a.y);

	if ( PIMAGELABEL(o)^.Flags and ILF_FRAME ) = ILF_FRAME then
      Rect3D(buffer, o^.Absolute.a.x,o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.b.y, COL_3DLIGHT, COL_3DDARK);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function CreateImageLabel( App: PApplication; r: TRect; Ico: PBITMAP ): PImageLabel;
var
  b: PImageLabel;
begin
    New(b);
	if not assigned(b) then CreateImageLabel := NIL else
	begin
    	fillchar(b,sizeof(TImageLabel),0);
	    b^.Ico := Ico;
    	IntialiseWidget(App, ImageLabelClass, PWIDGET(b), r );
    	PWIDGET(b)^.Draw := {$ifdef fpc}@{$endif}ImageLabelDraw;
    	PWIDGET(b)^.BackgroundColor := COL_3DFACE;

    	CreateImageLabel := b;
	end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function StdCreateImageLabel ( App: PApplication; r: TRect ): PWidget;
begin
	StdCreateImageLabel := PWIDGET(CreateImageLabel(App,r,NIL));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function StdCreateLabel ( App: PApplication; r: TRect ): PWidget;
begin
	StdCreateLabel := PWIDGET(CreateLabel(App,r,''));
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function LabelSetRProp ( o: PWidget; Name: l_text; const Value; Redraw: l_bool ): l_bool;
begin	
	if ( TextCompare(Name,'caption') = 0 ) then
	begin
		if ( PLABEL(o)^.Caption <> '' ) then PLABEL(o)^.Caption := '';
		PLABEL(o)^.Caption := l_text(Value);
		if ( Redraw ) then WidgetDraw(PWIDGET(o), NIL);
		LabelSetRProp := true;
	end;
	LabelSetRProp := false;
end;

function LabelGetRProp ( o:PWidget; Name: l_text ): Pointer;
begin
	if ( TextCompare(Name,'caption') = 0 ) then LabelGetRProp := NewStr(PLABEL(o)^.Caption) else
	LabelGetRProp := NIL;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
function LibMain ( argc: integer; argv: l_text ): l_bool;
begin
	LabelClass := CreateWidgetClass( ULONG_ID('L','b','l',' '), StdWidgetClass, 0 );
	LabelClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateLabel;
	LabelClass^.SetRProp := {$ifdef fpc}@{$endif}LabelSetRProp;
	LabelClass^.GetRProp := {$ifdef fpc}@{$endif}LabelGetRProp;

	WClassRegisterProp(LabelClass,PTYPE_STRING,'caption');

	ImageLabelClass := CreateWidgetClass( ULONG_ID('L','b','l','I'), StdWidgetClass, 0 );
	ImageLabelClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateImageLabel;

	
	{
	APPEXPORT(CreateLabel);
	APPEXPORT(LabelSetText);
	APPEXPORT(LabelSetTextEx);
	APPEXPORT(CreateImageLabel);
	}
	LibMain := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure Close;
begin

end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
     AppVersion := ULONG_ID(0,2,0,0);
     
{$endif}  {$LINK_CLIBS}
end.
