//**
//*       Copyright (c) 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//*
//*       FILE:                   vfile.pas
//*
//*       PROJECT:                Phoenix engine - Core
//*
//*       DESCRIPTION:    This file contains virtual filesystem functions and its
//*                                               implementation.
//*
//*       CONTRIBUTORS:
//*                                       Lukas Lipka
//*                                       Etelain Julien
//*
//*       TODO:                   @ Add support for Linux filesystem (EXT2)
//*                                       @ Add drive recognition code (for eg. hdd, fdd, cdrom)
//*
//*       BUG:                    @
//*
//*       MISTAKE:                @ Drive B:\ is recognized during loading even if it doesnt exist,
//*                                               so it has to be skipped.
//*
//*       IDEA:                   @
//*/
unit vfile;
//** @defgroup vfile Virtual File System.
//        This module do an abstraction of local systelm filesystem to ensure platforms compatibilty.
//        Most standart functions are provided but under an unuasual name. (fread becomes FileRead)
//        @{
//*/
//**     @file
//        VFile core file
//*/
interface
{ .include "kernel.h"  }
{ .include <stdio.h>   }
{ .include <stdarg.h>  }
{ .include "internal.h"}
{ .include "string.h"  }
{$MACRO ON}

uses
  Sysutils, Classes, FileUtil, List, guitypes, text;

{$i cdefs.inc}

{$IFDEF LINK_CLIBS}
  {$LINK vfile.o}
  {$LINK internal.o}
{$ENDIF}

const
 TYPE_HDD   = $F0000001;
 TYPE_FDD   = $F0000002;
 TYPE_CDROM = $F0000004;

 SEEK_SET   = soFromBeginning;
 SEEK_CUR   = soFromCurrent;
 SEEK_END   = soFromEnd;

 LocalDrives: PList = NIL;

 DEFAULT_FILE_BUFFER_SIZE = 1024;

type
  PFile = {$ifdef use_stream}^TFileStream{$else}^FILE{$endif};

function file_read_char   ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif}; var o: l_char ): l_bool; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function file_read_short  ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif}; var o: l_short ): l_bool;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function file_read_long   ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif}; var o: l_long ): l_bool; {$IFDEF LINK_CLIBS}external;{$ENDIF}
function file_write_char  ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif}; o: l_char ): l_bool;     {$IFDEF LINK_CLIBS}external;{$ENDIF}
function file_write_short ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif}; o: l_short ): l_bool;    {$IFDEF LINK_CLIBS}external;{$ENDIF}
function file_write_long  ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif}; o: l_long ): l_bool;     {$IFDEF LINK_CLIBS}external;{$ENDIF}
function file_get_char    ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif} ): l_int;                 {$IFDEF LINK_CLIBS}external;{$ENDIF}
function file_get_mlong   ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif} ): l_long;                {$IFDEF LINK_CLIBS}external;{$ENDIF}

function WriteString ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif}; s: l_text ): l_bool;       {$IFDEF LINK_CLIBS}external;{$ENDIF}
function ReadString ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif} ): l_text;                   {$IFDEF LINK_CLIBS}external;{$ENDIF}

function GSVFSResolvePath( szPath: l_text ): l_text;           {$IFDEF LINK_CLIBS}external;{$ENDIF}
function GSFileOpen ( szPath: l_text; iMode: Integer ): PFileStruct; {$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure GSFileClose ( f: PFileStruct );                      {$IFDEF LINK_CLIBS}external;{$ENDIF}
function GSSystemVFSInstall ( ): Integer;                      {$IFDEF LINK_CLIBS}external;{$ENDIF}

function FileSeparator: String;
function NetWDriveSep: String;

{$define _include_in_vfile_}



type
  {.$i vfile.inc}
  {
  PFileDriver = ^TFileDriver;
  PDrive = ^TDrive;
  TDrive = record

	Name      : l_text;

	TotalSpace: l_ulong;
	UsedSpace : l_ulong;
	dType     : l_uchar;

	Ex1       : Pointer;
	Ex2       : Pointer;

	Driver    : PFileDriver;

  end;

  PFileStruct = ^TFileStruct;
  TFileStruct = record
	Drive : PDrive;

	Ex1   : Pointer;
	Ex2   : Pointer;

	FileSize  : l_ulong;
	BufferSize: l_ulong;
	BufferPos : l_ulong;
	Pos       : l_ulong;
	
	Buffer    : PByte;

	Flags     : l_ulong;

  end;

  PFileInfo = ^TFileInfo;
  TFileInfo = record
	Name      : l_text;
	VPathName : l_text;
	LocalName : l_text;
	Attributes: l_uchar;
	fType     : l_uchar;
	MTime     : l_ulong;
	ATime     : l_ulong;
	CTime     : l_ulong;

	Size      : l_ulong;

	Drive     : PDrive;

	FMS       : Pointer; // If set, point to manager needed by this file.
	Ex        : Pointer;
  end;
 }

  CallbackFunc = function( p: Pointer; t: l_text): l_bool;
 {
  TFileDriver = record

	DriverName        : l_text;

	FileOpen          : FileOpenFunc;
	FileClose         : FileCloseFunc;

	FileRead          : FileReadFunc;
	FileWrite         : FileWriteFunc;
	FileGetPos        : FileGetPosFunc;
	FileSeek          : FileSeekFunc;

	FileEOF           : FileEOFFunc;
	
	
	FileDelete        : FileDeleteFunc;
	FileRename        : FileRenameFunc;
	FileGetInfo       : FileGetInfoFunc;

	FileFlush         : FileFlushFunc;

	DirMake           : DirMakeFunc;
	DirDelete         : DirDeleteFunc;
	DirRename         : DirRenameFunc;

	List              : ListFunc;

	RefreshDriveInfo  : RefreshDriveInfoFunc;

	FileSetAttributes : FileSetAttributesFunc;

  end;
}


type
  PFileRef = ^TFileRef;
  TFileRef = record
	
	SName      : l_text; 
	LocalName  : l_text;
	Attributes : l_uchar;
	Size       : l_ulong;
	Drive      : PDrive;
	
  end;

// .#define FILEREF(o) ((PFileRef)(o))

{$ifndef FA_RDONLY}
const	
  FA_RDONLY   =     1; // Read Only
  FA_HIDDEN   =     2; // Hidden
  FA_SYSTEM   =     4; // System
  FA_LABEL    =     8; // Label : Drive i suppose
  FA_DIREC    =    16; // Directory
  FA_ARCH     =    32; // Archive
{$endif} (* FA_RDONLY *)

const
  FT_FILE    =   0;
  FT_FOLDER  =   1;
  FT_LFLOPPY =   2;
  FT_FLOPPY  =   3;
  FT_TAPE    =   4;
  FT_HDD     =   5;
  FT_CD      =   6;
  FT_DVD     =   7;
  FT_RAM     =   8;
  FT_CARD    =   9;
  FT_NETWK   =  10;
  FT_ZIP     =  11;
  FT_VFILE   = 128;
  
  _EOF_      =  -1;

function FileNameToPath ( Dir: l_text; Filename: l_text ): l_text;{$IFDEF LINK_CLIBS}external;{$ENDIF}

procedure FreeDrive ( o: PDrive );{$IFDEF LINK_CLIBS}external;{$ENDIF}

procedure FreeFileInfo ( o: Pointer );{$IFDEF LINK_CLIBS}external;{$ENDIF}

const
  FSF_READ   = $01;
  FSF_WRITE  = $02;

function fopen( Filename: l_text; Mode: Integer ): l_bool;
function ListDir ( Dir: l_text; Attrib: Byte ): PList;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileOpen ( Filename: l_text; Mode: Integer ): PFileStruct;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileOpen2 ( fi: PFileInfo; Mode: Integer ): PFileStruct;{$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure FileClose ( f: PFileStruct );{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileRead( var o; n,s: l_ulong; f: PFileStruct ): l_int;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileWrite( var o; n,s: l_ulong; f: PFileStruct ): l_int;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileSetPos( f: PFileStruct; Pos: l_ulong ): l_int;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileSeek( f: PFileStruct; Pos: l_long; fType: l_int ): l_int;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileGetPos( f: PFileStruct; var Pos: l_ulong ): l_int;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileLength ( f: PFileStruct ): l_ulong;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileWriteBinString ( f: PFileStruct; Txt: l_text ): l_bool;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileReadBinString ( f: PFileStruct ): l_text;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileWriteTextString ( f: PFileStruct; Txt: l_text ): l_bool;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FilePrintf ( f: PFileStruct; Fmt: l_text; args: array of const ): l_bool;{$IFDEF LINK_CLIBS}external;{$ENDIF}

//function FileNameToPath ( Dir: l_text; Filename: l_text ): l_text;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function GetPathParent ( dir: l_text ): l_text;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileGetInfo ( Filename: l_text ): PFileInfo;{$IFDEF LINK_CLIBS}external;{$ENDIF}

//procedure FreeFileInfo ( o: PFileInfo );{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileSetAttributes ( Filename: l_text; Attrib: Byte ): l_bool;{$IFDEF LINK_CLIBS}external;{$ENDIF}


function AddFileInfoToList ( 
    l: PList; Name, VDir: l_text; Attributes: Byte;
    fType: Byte; MTime,ATime,CTime,Size: l_ulong; Drive: PDrive; FMS, Ex: Pointer ): PFileInfo;{$IFDEF LINK_CLIBS}external;{$ENDIF}

function AddDrive ( Name: l_text; fType: Byte; Ex1,Ex2: Pointer; Driver: PFileDriver ): PDrive;{$IFDEF LINK_CLIBS}external;{$ENDIF}

procedure FreeFileRef ( p: Pointer );{$IFDEF LINK_CLIBS}external;{$ENDIF}
function NewFileReference ( f: PFileInfo; SName: l_text ): PFileRef;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function _GenerateFileRefs ( Lst: PList; f: PFileInfo; SDir: l_text; CallBack: CallbackFunc; id: Pointer ): l_bool;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function ListDir2 ( fi: PFileInfo; Attrib: Byte ): PList;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function _GenerateFileRefs2 ( Lst: PList; f: PFileRef; SDir: l_text; CallBack: CallbackFunc; id: Pointer ): l_bool;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function MakeDir ( Path: l_text ): l_bool;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileEOF ( f: PFileStruct ): l_int;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileVPathName ( D: PDrive; LocalName: l_text ): l_text;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function FileRename ( Filename,NewName: l_text ): l_bool;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function DirRename ( Filename,NewName: l_text ): l_bool;{$IFDEF LINK_CLIBS}external;{$ENDIF}

function FileGets ( var s: l_text; n: l_int; f: PFileStruct ): l_text;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function ResolveFileName ( Filename: l_text ): l_text;{$IFDEF LINK_CLIBS}external;{$ENDIF}
function RelativeName ( Dir,AbsFile: l_text ): l_text;{$IFDEF LINK_CLIBS}external;{$ENDIF}

procedure RemoveDrive ( o: PDrive );{$IFDEF LINK_CLIBS}external;{$ENDIF}
procedure GSSystemVFSUnInstall ();{$IFDEF LINK_CLIBS}external;{$ENDIF}


implementation

{$IFNDEF LINK_CLIBS}
uses
  debug;

var
{$ifdef _OZONE__LINUX_}
   LinuxFileDriver: PFileDriver;
  {$define SysFileDriver LinuxFileDriver}
{$else}
   MsDosFileDriver: PFileDriver;
  {$define SysFileDriver := MsDosFileDriver}
{$endif}

var
 RootFileDriver: PFileDriver; {$IFDEF LINK_CLIBS}external;{$ENDIF}

 DefDrive  : PDrive  = NIL;
 RootDrive : PDrive  = NIL;

{$ifdef _OZONE__LINUX_}
{$include <sys/stat.h>}

function filelength( name: l_text ): l_long;
var
  stat: status;
begin
  if (stat(name, &status)=-1) then filelength := 0;
  filelength := status.st_size;
end;
{$endif}

function _FileRead(var Data; CountBytes,CountRecords: int64; var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif}): Integer;
var Count,Recs: l_ulong;
begin
  Recs := 0;
  _FileRead := 1;
  {$IFDEF use_stream}
  try
    for Count := Recs to CountRecords-1 do f.Read(Data,CountBytes);
  except
    _FileRead := 0;
    break;
  end;
  {$else}
  while (not EOF(f)) and (Recs < CountRecords) do
  begin
    try
    BlockRead(f, Data, CountBytes, Count);
    Inc(Recs);
    except
      _FileRead := 0;
      break;
    end;
  end;
  {$endif}
end;

function _FileWrite(var Data; CountBytes,CountRecords: int64; var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif}): Integer;
var Count,Recs: l_ulong;
begin
  Recs := 0;
  _FileWrite := 1;
  {$IFDEF use_stream}
  try
    for Count := Recs to CountRecords-1 do f.Write(Data, CountBytes);
  except
    _FileWrite := 0;
    break;
  end;
  {$else}
  while (Recs < CountRecords) do
  begin
    try
    BlockWrite(f, Data, CountBytes, Count);
    Inc(Recs);
    except
      _FileWrite := 0;
      break;
    end;
  end;
  {$endif}
end;

procedure FreeFileDriver ( o: PFileDriver );
begin
  //if ( o^.DriverName <> nil ) then free(o^.DriverName);
  Dispose(o);
end;


function file_read_char ( var f: {$ifdef use_stream}TFileStream{$else}File{$endif}; var o: l_char ): l_bool;
begin
        if (_FileRead(o, 1, 1, f) <> 1) then file_read_char := false else file_read_char := true;
end;

function file_read_short ( var f: {$ifdef use_stream}TFileStream{$else}File{$endif}; var o: l_short ): l_bool;
begin
        if (_FileRead(o,2,1,f) <> 1) then file_read_short := false else file_read_short := true;
end;

function file_read_long ( var f: {$ifdef use_stream}TFileStream{$else}File{$endif}; var o: l_long ): l_bool;
begin
        if (_FileRead(o,4,1,f) <> 1) then file_read_long := false else file_read_long := true;
end;

function file_write_char ( var f: {$ifdef use_stream}TFileStream{$else}File{$endif}; o: l_char ): l_bool;
begin
        if (_FileWrite(o,1,1,f) <> 1) then file_write_char := false else file_write_char := true;
end;

function file_write_short ( var f: {$ifdef use_stream}TFileStream{$else}File{$endif}; o: l_short ): l_bool;
begin
        if (_FileWrite(o,2,1,f) <> 1) then file_write_short := false else file_write_short := true;
end;

function file_write_long ( var f: {$ifdef use_stream}TFileStream{$else}File{$endif}; o: l_long ): l_bool;
begin
        if (_FileWrite(o,4,1,f) <> 1) then file_write_long := false else file_write_long := true;
end;

function file_get_char ( var f: {$ifdef use_stream}TFileStream{$else}File{$endif} ): l_int;
var
  o: l_uchar;
begin
    if ( _FileRead(o, 1, 1, f) <> 1 ) then file_get_char := _EOF_ else file_get_char := o;
end;

function fread(var inMem; elSize, elCount: l_ulong; var stream: {$ifdef use_stream}TFileStream{$else}FILE{$endif}): l_ulong;
var Count,ReadCount,elcnt: l_ulong;
begin
  if (elSize > 0) and (elCount > 0) then
  begin
    Count := 0;
    elcnt := 0;
    ReadCount := 0;
    {$ifdef use_stream}
    try
      for Count := 0 to elCount-1 do stream.Read(inMem, elSize);
    except
      fread := 0;
    end;
    {$else}
    try
      while elcnt < elCount do
      begin
        Blockread(stream, inMem, elSize, ReadCount);
        Count := Count + ReadCount;
        inc(elcnt);
      end;
    except
      fread := 0;
    end;
    fread := Count;
  end else fread := 0;
{$endif}
end;

function fwrite(var inMem; elSize, elCount: l_ulong; var stream: {$ifdef use_stream}TFileStream{$else}FILE{$endif}): l_ulong;
var Count,WriteCount,elcnt: l_ulong;
begin
  if (elSize > 0) and (elCount > 0) then
  begin
    Count := 0;
    elcnt := 0; 
    WriteCount := 0;
    {$ifdef use_stream}
    try
      for Count := elcnt to elCount-1 do stream.Write(inMem, elSize);
    except
      fwrite := 0;
    end;
    {$else}
    try
      while elcnt < elCount do
      begin
        BlockWrite(stream, inMem, elSize, WriteCount);
        Count := Count + WriteCount;
        inc(elcnt);
      end;
    except
      fwrite := 0;
    end;
    fwrite := Count;
  {$endif}
  end else fwrite := 0;
end;

function ReadString ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif} ): l_text;
var
  len: l_ushort = 0;
  s: l_text     = '';
begin
        fread(len, 1, 2, f);
        //s = (l_text)malloc(length+1);
        if ( s='' ) then ReadString := '';
        fread(s, 1, len, f);
        s := s + char(0);

        ReadString := s;
end;

function WriteString ( var f: {$ifdef use_stream}TFileStream{$else}FILE{$endif}; s: l_text ): l_bool;
var
        len: l_ushort;
begin        
        len := Length(s);

        fwrite(len, 1, 2, f);
        fwrite(s, 1, len, f);
        WriteString := true;
end;

///**
//        Reads a long in motorola format from a file stream
//        @param f File stream
//        @return the value or _EOF_ if end of file is reached
//*/
function file_get_mlong ( var f: {$ifdef use_stream}TFileStream{$else}File{$endif} ): l_long;
var b1, b2, b3, b4: l_int;
begin
        if ((b1 = file_get_char(f)) <> not eof(f)) then
                if ((b2 = file_get_char(f)) and (b2 <> _EOF_)) then
                        if ((b3 = file_get_char(f)) and(b3 <> _EOF_)) then
                                if ((b4 = file_get_char(f)) and (b4 <> _EOF_)) then
                                        file_get_mlong := (longint(b1) shl 24) or (longint(b2) shl 16) or (longint(b3) shl 8) or longint(b4);

        file_get_mlong := _EOF_;
end;

{.$ifdef _OZONE__DJGPP_}

function FileNameToPath ( Dir: l_text; Filename: l_text ): l_text; 
begin   
   FileNameToPath := ExpandUNCFileName(Filename);
   {
   if ( !DCkPt("FileNameToPath.Dir",Dir) ) return TextDup(Filename);
   begin
        l_int s = TextLen(Dir);
        if ( Dir[s-1] == '/' ||  Dir[s-1] == '\\' )
                return TextArgs("%s%s",Dir,File);
        else
                return TextArgs("%s/%s",Dir,File);
   end;
   }
end;

procedure RemoveDrive ( o: PDrive );
var i: PListItem;
begin
        i := ListFoundItem(LocalDrives,o);

        if assigned( i ) then
                ListRemoveItem(LocalDrives,i);
end;

procedure DeleteDrive( d: Pointer);
begin
  RemoveDrive(PDrive(d));
end;

function AddDrive ( Name: l_text; fType: l_uchar; Ex1, Ex2: Pointer; Driver: PFileDriver ): PDrive;
var o: PDrive;
begin        
  New( o );
  if not Assigned(o) then AddDrive := NIL else
  begin
    fillchar(o,sizeof(TDrive),0);


    o^.Name := TextDup(Name);
    o^.dType := fType;
    o^.Ex1 := Ex1;
    o^.Ex2 := Ex2;
    o^.Driver := Driver;

    if Assigned(@o^.Driver^.RefreshDriveInfo) then o^.Driver^.RefreshDriveInfo(o);

    ListAdd(LocalDrives,Name,o,@DeleteDrive);

    AddDrive := PDrive(o);
  end;
end;
///**
//        Free a File Information.
//        @param o FileInfo to free
//        @see FileGetInfo
//*/
procedure FreeFileInfo ( o: Pointer );
begin

   //if ( o^.Name ) free(o^.Name);
   //if ( o^.VPathName ) free(o^.VPathName);
   //if ( o^.LocalName ) free(o^.LocalName);

  Dispose(PFileInfo(o));
end;

procedure FreeDrive ( o: PDrive );
begin    
  //if ( o^.Name ) then Dispose (o^.Name);
  Dispose(o);
end;

function FileSeparator: String;
begin
{$ifdef _OZONE__LINUX_}
  FileSeparator := '/';
{$else}
  FileSeparator := '\';
{$endif}
end;

function NetWDriveSep: String;
begin
{$ifdef _OZONE__LINUX_}
  NetWDriveSep := '//';
{$else}
  NetWDriveSep := '\\';
{$endif}
end;

function FoundDrive ( Filename: l_text; out outp: l_text ): PDrive; 
var O: l_text; Drv: PDrive; F,d: l_text;
begin
   d    := ExtractFileDrive(Filename);
   Drv  := PDrive(ListKey(LocalDrives,d));
   outp := d;
   FoundDrive := Drv;
   
        (*
        O := Filename;
        if (UpperCase(Filename) <> UpperCase('local://') then Filename := System.Copy(Filename,8,System.Pos('/',Filename)-8-1);

        if ( !TextNCaseCompare(File,"local://",8 ) ) File += 7;  {String Positionszeiger -> 7 Zeichen weiter}

        if ( *Filename == '/' || *Filename == '\\' ) {

                l_text F,d;
                PDrive Drv;

                File++;
                F = File;
                while ( *Filename ) {
                        if ( *Filename == '/' || *Filename == '\\' ) break;
                  File++;
                }

                if ( Filename == F ) {
                        ( *Out) = TextDup(O+1);
                  return RootDrive;
                }


                d = TextNDup(F,File-F);
                if ( *Filename == '/' || *Filename == '\\' )File++;
                //DebugMessage(' %S => '%S' '%S' ',O,d,File);

                Drv = ListKey(LocalDrives,d);

                if ( Drv ) {
                        ( *Out) = TextDup(Filename);
                        //DebugMessage("    => Use drive '%s' with '%s' driver ",Drv->Name,Drv->Driver->DriverName);

                        return Drv;

                } else {

                        ( *Out) = TextDup(O+1);
                  return RootDrive;
                }


        }
        
        *Out = TextDup(Filename);
        return DefDrive;
        *)
end;

///**
//        List a directory.
//        @param Dir Name of directory to list
//        @param Attrib Filter by attributes
//        @return Return listing as a PList of PFileInfo
//*/
function ListDir ( Dir: l_text; Attrib: l_uchar ): PList; 
var
  LFile: l_text;
  l: PList;
  Drv: List.PDrive;
begin
  LFile := '';
  l := NIL;
  Drv := List.PDrive(FoundDrive(Dir,LFile));
  if Assigned( Drv^.Driver^.List ) then l := Drv^.Driver^.List(Drv,Dir,LFile,Attrib);
  ListDir := l;
end;

///**
//        List a directory.
//        @param fi PFileInfo of the directory
//        @param Attrib Filter by attributes
//        @return Return listing as a PList of PFileInfo
//*/
function ListDir2 ( fi: List.PFileInfo; Attrib: Byte ): PList;
begin
  if Assigned( fi^.Drive^.Driver^.List ) then
                ListDir2 := fi^.Drive^.Driver^.List(fi^.Drive,fi^.VPathName,fi^.LocalName,Attrib) else
        ListDir2 := NIL;
end;

{$ifdef _OZONE__LINUX_}
procedure InitLinuxFileDriver ();
begin

end;
{$define InitSysFileDriver := InitLinuxFileDriver}

{$else}
procedure InitMsDosFileDriver ();
begin

end;

{$define InitSysFileDriver := InitMsDosFileDriver}
{$endif}


procedure InitRootFileDriver ();
begin

end;

procedure DebugListDir ( Dir: l_text );
var  a, b: PListItem; l: PList;
begin
  l := ListDir(Dir,FA_DIREC);

  if ( l <> NIL ) and (l^.Last <> NIL) then
  begin

    DebugMessage('-> %s',[Dir]);

    a := l^.Last^.Next;
    b := l^.Last^.Next;

    if Assigned( a ) then
    while a <> NIL do
    begin

       DebugMessage('%S %S',[PFILEINFO(a^.Data)^.Attributes and FA_DIREC,  '[Dir]:'     ,PFILEINFO(a^.Data)^.Name]);

       a := a^.Next;
       
    end; // while ( a != b );

    FreeList(l);
  end;
end;
////////////////////////////////////////////////////////////////////////////////
function FileRename ( Filename, NewName: l_text ): l_bool;
var
 LFile1: l_text;
 LFile2: l_text;
 r     : l_bool;
 Drv1,Drv2: List.PDrive;
begin        
  LFile1 := '';
  LFile2 := '';
  r := false;
  Drv1 := List.PDrive(FoundDrive(Filename,LFile1));
  Drv2 := List.PDrive(FoundDrive(NewName,LFile2));
  if ( Drv1 <> Drv2 ) then
  begin
    DebugError('Different drivers %S %S',[Filename,NewName]); 
    FileRename := false; 
  end 
  else
  begin  
    if Assigned( Drv1^.Driver^.FileRename ) then r := Drv1^.Driver^.FileRename(Drv1,LFile1,LFile2);
    FileRename := r;
  end;
end;
////////////////////////////////////////////////////////////////////////////////
function DirRename ( Filename,NewName: l_text ): l_bool;
var
 LFile1: l_text;
 LFile2: l_text;
 r: l_bool;
 Drv1,Drv2: List.PDrive;
begin        
  LFile1 := '';
  LFile2 := '';
  r := false;
  Drv1 := List.PDrive(FoundDrive(Filename,LFile1));
  Drv2 := List.PDrive(FoundDrive(NewName,LFile2));
  if ( Drv1 <> Drv2 ) then
  begin 
    DebugError('Different drivers %s %s',[Filename,NewName]); 
    DirRename := false; 
  end
  else
  begin
    if Assigned( Drv1^.Driver^.DirRename ) then r := Drv1^.Driver^.DirRename(Drv1,LFile1,LFile2);
    DirRename := r;
  end;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Open a file stream
//        @param File Name of the file to open
//        @param Mode Filestream mode
//        @return NULL if file not found (read), or enable to create file (write) or the filestream
//*/
function FileOpen ( Filename: l_text; Mode: Integer ): PFileStruct;
var LFile: l_text; f: PFileStruct; Drv: List.PDrive;
begin        
  f     := NIL;
  LFile := '';
  Drv   := List.PDrive(FoundDrive(Filename,LFile));
  if Assigned( Drv^.Driver^.FileOpen ) then f := Drv^.Driver^.FileOpen(Drv,LFile,Mode);

  FileOpen := f;
end;

function fopen( Filename: l_text; Mode: Integer ): l_bool;
begin
  fOpen := FileOpen(Filename, Mode) <> NIL;
end;

////////////////////////////////////////////////////////////////////////////////
///**
//        Open a file stream from of FileInfo reference
//        @param fi FileInfo reference
//        @param Mode Filestream mode
//        @return NULL if file not found (read), or enable to create file (write) or the filestream
//*/
function FileOpen2 ( fi: List.PFileInfo; Mode: Integer ): PFileStruct;
begin
        if Assigned( fi^.Drive^.Driver^.FileOpen ) 
            then Pointer(FileOpen2) := Pointer(fi^.Drive^.Driver^.FileOpen(fi^.Drive,fi^.LocalName,Mode))
        else FileOpen2 := NIL;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Close a file stream
//        @param f File stream
//*/
procedure FileClose ( f: PFileStruct );
begin
  f^.Drive^.Driver^.FileClose(f^.Handle);
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Determines if pointer is at the end of the file
//        @param f File stream
//        @return true if pointer is at the end of the file, else false
//*/
function FileEOF ( f: PFileStruct ): l_int;
begin
  FileEOF := f^.Drive^.Driver^.FileEOF(f^.Handle);
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Read data from a file stream and store it to memory. Reads n*s bytes.
//        @param o Pointer to the memory to store data
//        @param s Size of a block
//        @param n Number of blocks to read
//        @param f File stream
//        @return Number of bytes read, of _EOF_ if end of file is reached
//*/
function FileRead( var o; n,s: l_ulong; f:PFileStruct ): l_int;
begin
  FileRead := f^.Drive^.Driver^.FileRead(f^.Handle,o,n*s);
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Write data to a file stream from memory. Write n*s bytes.
//        @param o Pointer to the data to write
//        @param s Size of a block
//        @param n Number of blocks to write
//        @param f File stream
//        @return Number of bytes written
//*/
function FileWrite( var o; n,s: l_ulong; f: PFileStruct ): l_int;
begin
  FileWrite := f^.Drive^.Driver^.FileWrite(f^.Handle,o,n*s);
end;
////////////////////////////////////////////////////////////////////////////////
function FileSetPos( f: PFileStruct; Pos: l_ulong ): l_int;
begin
  FileSetPos := f^.Drive^.Driver^.FileSeek(f^.Handle,SEEK_SET,Pos);
end;
////////////////////////////////////////////////////////////////////////////////
function FileSeek( f: PFileStruct; Pos: l_long; fType: l_int ): l_int;
begin
  FileSeek := f^.Drive^.Driver^.FileSeek(f^.Handle,fType,Pos);
end;
////////////////////////////////////////////////////////////////////////////////
function FileGetPos( f: PFileStruct; var Pos: l_ulong ): l_int;
begin
  FileGetPos := f^.Drive^.Driver^.FileGetPos(f^.Handle,Pos);
end;
////////////////////////////////////////////////////////////////////////////////
function FileLength ( f: PFileStruct ): l_ulong;
begin
  FileLength := f^.FileSize;
end;
////////////////////////////////////////////////////////////////////////////////
// Ported form djgpp's fgets, Copyright (C) 1994 DJ Delorie
///**
//        Read a line from a file buffer.
//        Stops if buffer is full or when a end of line or end of file is reached.
//        @param s Pointer to the place to store data (buffer)
//        @param n Size of the reserved place (size of buffer)
//        @param f File stream
//        @return s when a line is read, return NULL when end of file is reached and that no bytes have been stored.
//*/
function FileGets ( var s: l_text; n: l_int; f: PFileStruct ): l_text;
var
   c : ShortInt;  {  c = 0; }
   cs: l_text;    {*cs = s; }
   ls: Integer;
begin
        c := 0; cs := s; ls := Length(s); Dec(n);
        while ( (n > 0) and (c = file_get_char(f^.Handle)) and (c <> _EOF_) ) do begin
                if ( (c = $10) or (c = $13) ) then break else
                cs := cs + char(c);
        end;
        s := cs;
        if ( (c = _EOF_) and (cs = s) ) then FileGets := '';

        if ( (c = $10{'\n'}) or (c = $13{'\r'}) ) then SetLength(cs,Length(cs)-1);//Dec(cs);
        cs[Length(cs)] := #0;
        s := cs;
        FileGets := s;
end;
////////////////////////////////////////////////////////////////////////////////
// Text length must be smaller than 64k
///**
//        Write a string to a file stream in binary mode. The string can be read by FileReadBinString.
//        @param f File stream
//        @param Txt Pointer to the text to write
//        @return true on sucess, or false on error
//        @see FileReadBinString
//*/
function FileWriteBinString ( f: PFileStruct; Txt: l_text ): l_bool;
var
  s: l_ushort;
begin
  s := TextLen(Txt);
  FileWrite(s,1,2,f);
  FileWrite(Txt,1,s,f);
  FileWriteBinString := true;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Read a string from a file stream in binary mode written by FileWriteBinString.
//        @param f File stream
//        @return A pointer to the text (new allocation), or NULL on error
//        @note Don't forget to free the returned data
//        @see FileWriteBinString
//*/
function FileReadBinString ( f: PFileStruct ): l_text;
var
       len,l: l_ushort;
       s: l_text;
begin
        s := ''; len := 0;
        FileRead(len, 1, 2, f);
        //s = (l_text)malloc(length+1);
        for l:=0 to len do s := s + ' ';
        if ( s = '' ) or (Length(s)<len) then FileReadBinString := '' else
        begin
        FileRead(s, 1, len, f);
        s[len+1] := #0;
        FileReadBinString := s;
        end;
end;
////////////////////////////////////////////////////////////////////////////////
function FileWriteTextString ( f: PFileStruct; Txt: l_text ): l_bool;
begin
  FileWrite(Txt,1,TextLen(Txt),f);
        FileWriteTextString := true;
end;
////////////////////////////////////////////////////////////////////////////////
function FilePrintf ( f: PFileStruct; Fmt: l_text; args: array of const ) : l_bool;
var     
  Buffer: array[0..10240-1] of l_char; tmp: l_text;
  i: Integer;
begin        
  {
  va_start(argptr, Format);    
  vsprintf(Buffer, Format, argptr);
  va_end(argptr);
  }
  tmp := Format(Fmt, args);
  write(Format(Fmt, args));
  FileWriteTextString(f,tmp{buffer});
  FilePrintf := true;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Get a File Information from a file.
//        @param File Name of the file
//        @return NULL if file is not found, else return the File Info
//*/
function FileGetInfo ( Filename: l_text ): PFileInfo;
var
  LFile:  l_text;
  i: PFileInfo; 
  Drv: PDrive;
begin    
    LFile := '';
    i := NIL;
    Drv := FoundDrive(Filename,LFile);
        if Assigned( Drv^.Driver^.FileGetInfo ) then i := Drv^.Driver^.FileGetInfo(Drv,Filename,LFile);
        
        FileGetInfo := i;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Change a file attributes
//        @param File Name of the file
//        @param Attrib Attributes to set to the file
//        @return true on success, else false
//*/
function FileSetAttributes ( Filename: l_text; Attrib: Byte ): l_bool;
var
  LFile: l_text;
  Drv: PDrive;
  i: l_bool;
begin        
        LFile := '';
        
        Drv := FoundDrive(Filename,LFile);
        if Assigned( Drv^.Driver^.FileSetAttributes ) then i := Drv^.Driver^.FileSetAttributes(Drv,LFile,Attrib);
        
        FileSetAttributes := i;
end;
////////////////////////////////////////////////////////////////////////////////
///**
//        Determines the parent of a path.
//        Path should be an absolute one (exemple : "/c/my files/stuff")
//        @param dir Path
//        @return the path to the parent (new allocation), or NULL on error
//        @note Don't forget to free the returned text.
//*/
function GetPathParent ( dir: l_text ): l_text;
var
       i: integer;
       a: l_text;
       e: l_text;
      oe: l_text; 
begin
        a  := dir;
        e  := #0;
        oe := #0;
        
        if (a[Length(a)] = ':') or
           ((a[Length(a)] = FileSeparator) and (a[pred(Length(a))] = ':')) or
           ((a[Length(a)] = FileSeparator) and (a[pred(Length(a))] = FileSeparator))
        then GetPathParent := '' else
        begin
          if a[Length(a)] = FileSeparator then System.Delete(a,Length(a),1);
          for i:=Length(a) downto 1 do
          begin
            if a[i] = FileSeparator then
            begin
              e := System.Copy(a, 1, i);
              GetPathParent := e + oe;
              break;
            end;
          end;
        end;
        (*
        while ( *a ) {
                if ( *a == '/' ) { oe = e; e = a; }
                a++;
        }
        if ( a-1 == e  )
                e = oe;

        if ( e && e != dir )
                GetPathParent := TextNDup(dir,e-dir);
        else
                GetPathParent := TextDup('/');
        *)
end;
////////////////////////////////////////////////////////////////////////////////
procedure FreeInfo(p: Pointer);
begin
  //FreeFileInfo(PFileInfo(p));
  Dispose(PFileInfo(p));
end;

function AddFileInfoToList ( 
    l: PList; Name, VDir: l_text; Attributes: Byte;
    fType: Byte; MTime,ATime,CTime,Size: l_ulong; Drive: PDrive; FMS, Ex: Pointer ): PFileInfo;
var o: PFileInfo;    
begin    
        New(o);   // = malloc(sizeof(TFileInfo));
        if ( o=NIL ) then AddFileInfoToList := NIL else
        begin
        fillchar(o,sizeof(TFileInfo),0);

        o^.Name := TextDup(Name);
        o^.VPathName := FileNameToPath(VDir,Name);;
        o^.Attributes := Attributes;
        o^.fType := fType;
        o^.MTime := MTime;
        o^.ATime := ATime;
        o^.CTime := CTime;
        o^.Size  := Size;
        o^.Drive := Drive;
        o^.FMS   := FMS;
        o^.Ex    := Ex;
        ListAdd(l,AnsiString(Name),o,@FreeFileInfo);
        AddFileInfoToList := o;

        end;
end;
////////////////////////////////////////////////////////////////////////////////
function NewFileReference ( f: PFileInfo; SName: l_text ): PFileRef;
var o: PFileRef;
begin        
        New(o); // = malloc(sizeof(TFileRef));
        if ( o = NIL ) then NewFileReference := NIL;
        fillchar(o,sizeof(TFileRef),0);

        o^.Drive         := f^.Drive;
        o^.Size          := f^.Size;
        o^.Attributes    := f^.Attributes;
        o^.LocalName     := TextDup(f^.LocalName);
        o^.SName         := TextDup(SName);

        NewFileReference := o;
end;
////////////////////////////////////////////////////////////////////////////////
procedure FreeFileRef ( p: Pointer );
begin
        //if ( o^.LocalName ) then free(o^.LocalName);
        //if ( o^.SName ) then free(o^.SName);
        Dispose(PFileRef(p));
end;
////////////////////////////////////////////////////////////////////////////////
function _GenerateFileRefs ( Lst: PList; f: PFileInfo; SDir: l_text; CallBack: CallBackFunc; id: Pointer ) : l_bool;
var
  FT: l_text;
  o: PFileRef;
  Dir: PList;
  a,b: PListItem;
begin
        FT := FileNameToPath(SDir,f^.Name);
        
        if Assigned( CallBack ) then if ( not CallBack(id,FT) ) then _GenerateFileRefs := false;

        o := NewFileReference(f,FT);

        if ( o = NIL ) then _GenerateFileRefs := false;

        ListAdd(Lst,SDir,o,nil{@FreeFileRef});
        if ( f^.Attributes and FA_DIREC ) = FA_DIREC then
        begin
                dir := ListDir(f^.VPathName,FA_RDONLY or FA_HIDDEN or FA_SYSTEM or FA_LABEL or FA_DIREC or FA_ARCH);
                if ( dir <> NIL ) then
                begin
                        if ( dir^.Last <> NIL ) then 
                        begin
                                a := dir^.Last^.Next;
                                b := a;
                                while a <> NIL do 
                                begin
                                        if ( not _GenerateFileRefs(Lst,a^.Data,FT,CallBack,id) ) then begin _GenerateFileRefs := false; break end;
                                        a := a^.Next;
                                end;   // while ( a != b);
                        end;
                        FreeList(dir);
                end;
        end;
        //free(F);

      _GenerateFileRefs := true;
end;
////////////////////////////////////////////////////////////////////////////////
function DuplicateFileRef(f: PFileRef): PFileRef;
begin
  DuplicateFileRef := f;
end;

function _GenerateFileRefs2 ( Lst: PList; f: PFileRef; SDir: l_text; CallBack: CallBackFunc; id: Pointer ): l_bool;
var
  a,b: PListItem;
  FT : l_text;
  Dir: PList;
  Ref: PFileRef;
begin        
        
        FT := FileNameToPath(SDir,f^.SName);

//DebugMessage('Start CallBack');
        if Assigned( CallBack ) then if ( not CallBack(id,FT) ) then _GenerateFileRefs2 := false;
//DebugMessage('CallBack returned true');
        Ref := DuplicateFileRef(f);
        ListAdd(Lst,SDir,Ref,nil{@FreeFileRef});
        if ( f^.Attributes and FA_DIREC ) = FA_DIREC then
        begin
                dir := NIL;
                if Assigned( f^.Drive^.Driver^.List ) then dir := f^.Drive^.Driver^.List(f^.Drive,f^.LocalName,f^.LocalName,FA_RDONLY or FA_HIDDEN or FA_SYSTEM or FA_LABEL or FA_DIREC or FA_ARCH);
                if ( dir<>NIL ) then
                begin
                        if Assigned( dir^.Last ) then
                        begin
                                a := dir^.Last^.Next;
                                b := a;
                                while a <> NIL do begin
                                        if ( not _GenerateFileRefs(Lst,a^.Data,FT,CallBack,id) ) then _GenerateFileRefs2 := false;
                                        a := a^.Next;
                                end;   // while ( a != b);
                        end;
                        FreeList(dir);
                end;
        end;
        //free(F);

       _GenerateFileRefs2 := true;
end;

////////////////////////////////////////////////////////////////////////////////
///**
//        Creates a directory
//        @param Path Path of the directory to create
//        @return true on success or false on error
//*/
function MakeDir ( Path: l_text ): l_bool;
var LFile: l_text; i: l_bool; Drv: PDrive;
begin
  //CreateDir(Path);              
  LFile := '';
  i := false;
  Drv := FoundDrive(Path,LFile);
  if Assigned( Drv^.Driver^.DirMake ) then i := Drv^.Driver^.DirMake(Drv,LFile);
  MakeDir := i;
end;
////////////////////////////////////////////////////////////////////////////////
function FileVPathName ( D: PDrive; LocalName: l_text ): l_text;
begin
        if ( D = DefDrive ) then FileVPathName := TextArgs('/system/%s',[LocalName]) else
        if ( D = RootDrive ) then FileVPathName := TextArgs('/%s',[LocalName]) else
        FileVPathName := TextArgs('/%s/%s',[D^.Name,LocalName]);
end;
////////////////////////////////////////////////////////////////////////////////
(* l_text _ResolveFileName ( l_text File ) {

        l_text  t = File, // Current char
                        l = NULL, // Last token
                        c = File; // Current token

        do {
                if ( ( *t == '/') || !*t ) {
                        /*if ( !(t-c) && l ) {
                                l_text n = TextDup(t);
                                free(File);
                                return _ResolveFileName(n);
                        } else */if ( !TextSqNCaseCompare(c,"..",t-c) ) {
                                if ( l ) {
                                        l_text n = TextDup(File);
                                        TextCopy(n+(l-File),n+(t+( *t?1:0)-File));
                                        free(File);
                                        return _ResolveFileName(n);
                                } else {
                                        l_text n = TextDup(File);
                                        TextCopy(n+(c-File),n+(t+( *t?1:0)-File));
                                        free(File);
                                        return _ResolveFileName(n);
                                }
                        } else if ( !TextSqNCaseCompare(c,".",t-c) ) {
                                l_text n = TextDup(File);
                                TextCopy(n+(c-File),n+(t+( *t?1:0)-File));
                                free(File);
                                return _ResolveFileName(n);
                        }

                        if ( t != File ) l = c;
                        c = t+1;
                }

                //t++;
        } while ( *(t++) );
        return File;
} *)
////////////////////////////////////////////////////////////////////////////////
///**
//        Clean a path. Removes and solve the "/./", "/../", "//"
//        @param File Path to clean
//        @return the cleaned path (new allocation)
//        @note don't forget to free the returned text
//*/
function ResolveFileName ( Filename: l_text ): l_text;
begin

  ResolveFileName := ExtractFileName(Filename);
        //return _ResolveFileName(TextDup(File));

end;
////////////////////////////////////////////////////////////////////////////////
function RelativeName ( Dir, AbsFile: l_text ): l_text;
var dt,ldt,ft,lft,a,Rel: l_text;
begin        
  RelativeName := CreateRelativePath(AbsFile,Dir, True, False);
  
  (*
        dt := Dir; ldt := Dir; ft := AbsFile; lft := AbsFile;
        Rel = '';

        while true do 
        begin
                dt := dt+FileSeparator;
                ft := ft+FileSeparator);
                if ( (dt = '') or (ft = '') ) then
                begin
                        dt := ldt;
                        ft := ldt;
                        break;
                end;
                if ( TextNCaseCompare(Dir,AbsFile,system.pos(dt,AbsFile)-system.pos(Dir,AbsFile)) > 0 ) then break; {dt-dir}       //--- system.pos(dt,AbsFile) - system.pos(Dir,AbsFile) ---
                lft := ft;
                ldt := dt;
        end;


        if ( dt <> '' ) then
        begin

                ldt := dt;
                while ( dt := TextChr(dt+1,'/') ) then begin
                        if ( Rel <> '' ) then begin
                                a := Rel;
                                Rel := TextArgs('%s/..',Rel);
                                //free(a);
                        end else
                                Rel := TextDup('..');
                        ldt = dt;
                end;
                if ( TextLen(ldt) > 1 ) then begin
                        if ( Rel <> '' ) then begin
                                a := Rel;
                                Rel := TextArgs('%s/..',Rel);
                                //free(a);
                        end else
                                Rel := TextDup('..');
                end;
        end;

        if ( Rel <> '' ) then begin
                a := Rel;
                Rel := TextArgs('%s/%s',Rel,@ft+1);
                //free(a);
        end else
                Rel := TextDup(@ft+1);
        RelativeName := Rel;
  *)
end;

////////////////////////////////////////////////////////////////////////////////
function _getdrives: l_ulong;
begin
end;

///**
//*       NAME: GSSystemVFSInstall
//*       DESCRIPTION: Installs the VFS, is called during kernel setup.
//*/
function GSSystemVFSInstall: Integer;
var
  d: l_int;
  uDriveMask: l_ulong;
begin
        LocalDrives := NewList();
        InitSysFileDriver();
        InitRootFileDriver();
{$ifdef _OZONE__DJGPP_}
        
                d := 0;

                while ( d < 27 ) do
                begin
                        if ( _is_remote_drive(d) <> -1 ) then
                        begin
                        
                          if ( d <> 1 ) then
                          begin
                            DebugMessage('Drive %c:\\ mapped as /%c/', [char(ord('a')+d)),char(ord('a')+d)]);
                            AddDrive(TextArgs('%c',[char(ord('a')+d)]),FT_HDD,TextArgs('%c:/',[char(ord('a')+d)]),NIL,MsDosFileDriver);
                          end;
                        end;
                        Inc(d);
                end;

                if ( d = 0 ) then DebugError('MS-DOS VFile :: No drives found!',[]);
        
{$endif}

{$ifdef _OZONE__WIN32_}
        
                d := 0;
                uDriveMask := _getdrives();

                if (uDriveMask <> 0) then
                begin
                        while l_bool(uDriveMask) do
                        begin
                          if l_bool(uDriveMask and 1) then
                          begin
                            DebugMessage('Drive %c:\\ mapped as /%c/', [ord('a')+d,ord('a')+d]);
                            //AddDrive(TextArgs('%c',[char(ord('a')+d)]),FT_HDD,TextArgs('%c:/',[char(ord('a')+d)]),NIL,MsDosFileDriver);
                          end;

                          Inc(d);
                          uDriveMask := uDriveMask shr 1;
                        end;
                end;
        
{$endif}

        New(DefDrive); {= malloc(sizeof(TDrive));}
        fillchar(DefDrive,sizeof(TDrive),0);

        DefDrive^.Name   := TextDup('&__basic');
        DefDrive^.dType  := FT_FOLDER;
        DefDrive^.Ex1    := NewStr('./');
        DefDrive^.Ex2    := NIL;
        DefDrive^.Driver := SysFileDriver;

        New(RootDrive); {:= malloc(sizeof(TDrive));}
        fillchar(RootDrive,sizeof(TDrive),0);

        RootDrive^.Name  := TextDup('');
        RootDrive^.dType := FT_FOLDER;
        RootDrive^.Ex1   := NewStr('./SYSTEM/ROOT/');
        RootDrive^.Ex2   := NIL;
        RootDrive^.Driver:= RootFileDriver;

        AddDrive('system',FT_FOLDER,NewStr('.'),NIL,SysFileDriver);
        AddDrive('Applications',FT_FOLDER,NewStr('./Applications/'),NIL,SysFileDriver);
{$ifdef _OZONE__LINUX_}

        AddDrive('local',FT_HDD,NIL,NIL,LinuxFileDriver);
{$endif}

        GSSystemVFSInstall := 0;
end;

function GSVFSResolvePath( szPath: l_text ): l_text;
begin
  {$ifdef use_stream}
  {$else}
  {$endif}
end;

function GSFileOpen ( szPath: l_text; iMode: Integer ): PFileStruct;
{$ifndef fpc}{$ifndef OPascal}type TFileStream = TBufStream;{$endif}{$endif}
var Mode: Integer; fStruct: PFileStruct; Drive: PDrive; DrvNum: byte;
begin
  GSFileOpen := NIL;
  {$ifdef fpc}
  Mode := iMode;
  {$else}
  Mode := iMode;
  {$endif}
  {$ifdef use_stream}
  New(fStruct); New(Drive);
  Drive^.Name := ExtractFileDrive(szPath);
  case UpperCase(Drive^.Name[1]) of
   'A': DrvNum :=  1;
   'B': DrvNum :=  2;
   'C': DrvNum :=  3;
   'D': DrvNum :=  4;
   'E': DrvNum :=  5;
   'F': DrvNum :=  6;
   'G': DrvNum :=  7;
   'H': DrvNum :=  8;
   'I': DrvNum :=  9;
   'J': DrvNum := 10;
   'K': DrvNum := 11;
   'L': DrvNum := 12;
   'M': DrvNum := 13;
   'N': DrvNum := 14;
   'O': DrvNum := 15;
   'P': DrvNum := 16;
   'Q': DrvNum := 17;
   'R': DrvNum := 18;
   'S': DrvNum := 19;
   'T': DrvNum := 20;
   'U': DrvNum := 21;
   'V': DrvNum := 22;
   'W': DrvNum := 23;
   'X': DrvNum := 24;
   'Y': DrvNum := 25;
   'Z': DrvNum := 26;
  end;
  Drive^.dType := 3;
  Drive^.TotalSpace := DiskSize(DrvNum);
  Drive^.UsedSpace:=Drive^.TotalSpace - DiskFree(DrvNum);
  {$ifdef fpc}
  fStruct^.Handle := TFileStream.Create(szPath,Mode);
  {$else}
  fStruct^.Handle.Init(szPath, Mode, 1024);
  {$endif}
  fStruct^.FileSize := fStruct^.Handle.Size;
  {$ifdef fpc}
  {$else}
  {$ifndef OPascal}fStruct^.BufferSize := 1024;{$endif}
  {$endif}
  fStruct^.Drive := Drive;
  fStruct^.FileSize := fStruct^.Handle.Size;
  fStruct^.BufferSize := DEFAULT_FILE_BUFFER_SIZE;
  fStruct^.Pos := fStruct^.Handle.Position;
  fStruct^.BufferPos := fStruct^.Pos mod fStruct^.BufferSize;
  {$ifdef OPascal}
  SetLength(fStruct^.Buffer, fStruct^.BufferSize);
  {$else}
  fStruct^.Buffer := MakeArray(fStruct^.Buffer, fStruct^.BufferSize);
  {$endif}

  GSFileOpen := fStruct;

  {$else}
  New(fStruct); New(Drive);

  Drive^.Name := ExtractFileDrive(szPath);
  case UpperCase(Drive^.Name[1]) of
   'A': DrvNum :=  1;
   'B': DrvNum :=  2;
   'C': DrvNum :=  3;
   'D': DrvNum :=  4;
   'E': DrvNum :=  5;
   'F': DrvNum :=  6;
   'G': DrvNum :=  7;
   'H': DrvNum :=  8;
   'I': DrvNum :=  9;
   'J': DrvNum := 10;
   'K': DrvNum := 11;
   'L': DrvNum := 12;
   'M': DrvNum := 13;
   'N': DrvNum := 14;
   'O': DrvNum := 15;
   'P': DrvNum := 16;
   'Q': DrvNum := 17;
   'R': DrvNum := 18;
   'S': DrvNum := 19;
   'T': DrvNum := 20;
   'U': DrvNum := 21;
   'V': DrvNum := 22;
   'W': DrvNum := 23;
   'X': DrvNum := 24;
   'Y': DrvNum := 25;
   'Z': DrvNum := 26;
  end;
  Drive^.dType := 3;
  Drive^.TotalSpace := DiskSize(DrvNum);
  Drive^.UsedSpace:=Drive^.TotalSpace - DiskFree(DrvNum);

  Assign(fStruct^.Handle, szPath);
  {$i-}
  if Mode = {$ifdef fpc}fmCreate{$else}{$ifdef OPascal}fmCreate{$else}stCreate{$endif}{$endif}
     then Rewrite(fStruct^.Handle, 1)
  else Reset(fStruct^.Handle, 1);
  {$i+}
  if IOResult = 0 then
  begin
    fStruct^.Drive := Drive;
    fStruct^.FileSize := FileSize(fStruct^.Handle);
    fStruct^.BufferSize := DEFAULT_FILE_BUFFER_SIZE;
    fStruct^.Pos := FilePos(fStruct^.Handle);
    fStruct^.BufferPos := fStruct^.Pos mod fStruct^.BufferSize;
    {$ifdef OPascal}
    SetLength(fStruct^.Buffer, fStruct^.BufferSize);
    {$else}
    fStruct^.Buffer := MakeArray(fStruct^.Buffer, fStruct^.BufferSize);
    {$endif}
    GSFileOpen := fStruct;
  end
  else
  begin
    Dispose(Drive);
    Dispose(fStruct);
    fStruct := nil;
    GSFileOpen := fStruct;
  end;
  {$endif}
end;

procedure GSFileClose ( f: PFileStruct );
begin
  {$ifdef Opascal}
  SetLength(f^.Buffer, 0);
  {$else}
  freemem(f^.Buffer, f^.BufferSize);
  {$endif}
  {$ifdef use_stream}
  f^.Handle.Free;
  {$else}
  Close(f^.Handle);
  {$endif}
  Dispose(f);
end;

procedure GSSystemVFSUnInstall ();
begin

        FreeList(LocalDrives);
        FreeFileDriver(SysFileDriver);
        FreeFileDriver(RootFileDriver);

end;

{$ENDIF}

end.
