(*
////////////////////////////////////////////////////////////////////////////////
//
//  widget.pas - Unit
//
//	(c) Copyright 2003,2004 Point Mad, Lukas Lipka. All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////
*)
unit widget;
(*
//**	
//	@addtogroup widgetsys
//	@{
//*/
///**
//	@file
//	Widget system header file.
//	Declarations and structures for widget system.
//*/
*)
{ .#include "metrics.h"}
{ .#include "dynld.h"}

{ .#ifndef _WIDGET_H_INCLUDED_}
{ .#define _WIDGET_H_INCLUDED_}

interface

uses
  guitypes, dynld, metrics, vfile, list, events, mousedrv, dms, text, timer,
  grfx,allegro,font, thread, screen, keyboard, registry, image, debug;

{$i cdefs.inc}

{$IFDEF LINK_CLIBS}
   {$LINK metrics.o}
   {$LINK dynld.o}
   {$Link debug.o}
{$ENDIF}

(*////////////////////////////////////////////////////////////////////////////////*)
(*
////////////////////////////////////////////////////////////////////////////////
//**
//	Generic property class structure
//*/
*)
type
  DuplicateFunc  = function  ( Value: Pointer ): Pointer;
  FreeProc       = procedure ( Value: Pointer );
  WriteProc      = procedure ( f: PFileStruct; Value: Pointer );
  ReadFunc       = function  ( f: PFileStruct ): Pointer;

type                    
  PPropType = ^TPropType;
  TPropType = record

	Id: l_ulong;
	
	Duplicate : DuplicateFunc;
	Free      : FreeProc;
	Write     : WriteProc;
	Read      : ReadFunc;
  end;
	

{ .#define PTYPE(o) ((PPropType)(o))}
(*
////////////////////////////////////////////////////////////////////////////////
//**
//	Widget property structure
//*/
*)
type
  PWidgetProp = ^TWidgetProp;
  TWidgetProp = record

	aType: PPropType;
	
	Extra: Pointer;
	FreeExtra: FreeProc;
  end;

{ .#define WDGPROP(o) ((PWidgetProp)(o))}

type
  PWidgetPropFlag = ^TWidgetPropFlag;
  TWidgetPropFlag = record

	Value: l_ulong;
	Name: l_text;
	Description: l_text;
	
  end;

{ .#define WDGPFLAG(o) ((PWidgetPropFlag)(o))}
(*
////////////////////////////////////////////////////////////////////////////////
//**
//	Widget Class structure
//*/
*)
type
  PiWidget = ^TiWidget;
  PWidget  = ^TWidget;
  PWidgetClass = ^TWidgetClass;
  GetDefaultfunc = function ( Name: l_text ): Pointer;
  SetRPropfunc   = function ( o: PWidget; Name: l_text; const Value; Redraw: l_bool ): l_bool;
  GetRPropfunc   = function ( o: PWidget; Name: l_text ): Pointer;
  CreateFunc     = function ( App: PApplication; r: TRect): PWidget;
    

  //PWidgetClass = ^TWidgetClass;
  TWidgetClass = record

	Id    : l_ulong;
        Parent: PWidgetClass;
	Flags : l_ulong;
	
	Properties: PList; // Associate Property name to Property type
	
	StdCreate : CreateFunc;
	
	GetDefault: GetDefaultfunc;
	SetRProp  : SetRPropfunc;
	GetRProp  : GetRPropfunc;
	
  end;

{ .#define WCLASS(o) ((PWidgetClass)(o)) }
(*
////////////////////////////////////////////////////////////////////////////////
//**
//	Irreal Widget structure
//**/
*)
  TiWidget = record
 
	Name      : l_text;       ///< Widget name
	WClass    : PWidgetClass; ///< Widget class
	r         : TRect;        ///< Widget relative coordinates (in child area)
	Properties: PList;
	
	Last      : PiWidget;
	Next      : PiWidget;
	Prev      : PiWidget;
	Parent    : PiWidget;
	
  end; 

{ .#define IWIDGET(o) ((PiWidget)(o)) }
(*
////////////////////////////////////////////////////////////////////////////////
//**
//	Widget structure
//**/
*)

  EventHandlerFunc   = function ( o: PWidget; Ev: PEvent ): l_bool;
  AppEvHdlFunc       = function ( o: PWidget; Ev: PEvent ): l_bool;
  
  RefreshProc        = procedure ( o: PWidget; w: PRect; List: PList; var bmp: p_bitmap; delta: PPoint );
  DrawProc	     = procedure ( o: PWidget; buffer: p_bitmap; w: PRect );
  FreeExProc 	     = procedure ( o: PWidget );
  SetupProc          = procedure ( o: PWidget );
	                
  InsertProc         = procedure ( o: PWidget; i: PWidget );
  RemoveProc         = procedure ( o: PWidget; i: PWidget );
  SetFirstExProc     = procedure ( o: PWidget; i: PWidget );
  
  DropAcceptFunc     = function ( o: PWidget; Caller: PWidget; aType: l_ulong; var Data; SupportedMethods: l_ulong; var Method: l_ulong ): l_bool;	
  DragBeforeDropFunc = function ( o: PWidget; Dest: PWidget; var Data; aType: l_ulong; Method: l_ulong ): l_bool;
  DropDataProc       = procedure ( o: PWidget; Caller: PWidget; var Data; aType: l_ulong; Method: l_ulong );
  DragAfterDropProc  = procedure ( o: PWidget; Dest: PWidget; var Data; aType: l_ulong; Method: l_ulong );

  widgetfunc = function (o: PWidget; p: TPoint): l_text;


  TTooltipStruct = record
    case Longint of
     0: (t: ShortString);
     1: (func: widgetfunc);
  end;
  
 (* TWidget = {ifdef fpc}class {$else}object {$endif} *) 
  TWidget = record

	WClass  : PWidgetClass; ///< Widget class

	Relative    : TRect; ///< Coordinates of widget coordinates using ChildArea of parent as reference
	Absolute    : TRect; ///< Coordinates of widget  using screen as reference
	DrawAbsolute: TRect; ///< Coordinates of widget using childarea of parent as reference clipped in ChildDrawAbs of parent
	ChildArea   : TRect; ///< Coordinates relative to object position of child spacework (equal to Relative by default)
	ChildDrawAbs: TRect; ///< Coordinates relative to screen of child spacework (equal to Absolute by default)

	Prev        : PWidget;
	Next        : PWidget;
	Last        : PWidget;
	Parent      : PWidget;

	BackgroundColor: l_long; ///< Widget default background color
	(*
	 //**
	 //	Widget flags.
	 //	@see widgetflags
	 //*/
	*)
	Flags: l_ulong;

	AppOwner: PApplication; ///< Owner application

	MessageBuffer: l_ulong;

	EventHandler: EventHandlerFunc;
	AppEvHdl: AppEvHdlFunc;
	Refresh: RefreshProc;
	Draw: DrawProc;
	FreeEx: FreeExProc;
	Setup: SetupProc;
	
	Insert: InsertProc;
	Remove: RemoveProc;
	SetFirstEx: SetFirstExProc;
	
	FocusBack: PWidget; (* ///< If set, when focus will be set on widget, it will focus instead the specified widget *)

    ToolTip: TTooltipStruct;
    
	DropAccept: DropAcceptFunc;
	
	DragBeforeDrop: DragBeforeDropFunc;
	DropData: DropDataProc;
	DragAfterDrop: DragAfterDropProc;
	
	Name: l_text;
	
	Reserved: array [0..3] of l_long;
	
  end;

{ .#define WIDGET(o) ((PWidget)(o)) }
(*
 ////////////////////////////////////////////////////////////////////////////////
 /**
	Tool tip widget
 */
*)
type
  PToolTip = ^TToolTip;
  TToolTip = record

	o: TWidget;

	Text: l_text;

  end;

{ .#define TOOLTIP(o) ((PToolTip)(o)) }
(*////////////////////////////////////////////////////////////////////////////////*)

const
 MSG_NONE = $00;
 MSG_MSGCANCEL = $01;

var
 wallpaper    : p_bitmap;
 DeskTop      : PWidget;
 MasterWidget : PWidget;
 FocusedWidget: PWidget;

procedure InsertWidget ( o,i: PWidget );
function IntialiseWidget( App: PApplication; AClass: PWidgetClass; o: PWidget; r: TRect ): PWidget;
function CreateWidget	( App: PApplication; r: TRect ): PWidget;
procedure WidgetDrawOnRect	( o: PWidget; w: TRect; Ignore: PWidget );
procedure WidgetMove		( o: PWidget; dx, dy: l_long );
procedure WidgetResize		( o: PWidget; dW, dH: l_long );
procedure WidgetResizeToSize( o: PWidget; SizeX, SizeY : l_long );
procedure WidgetMoveToPoint	( o: PWidget; x, y: l_int );
procedure WidgetSetMetrics	( o: PWidget; x, y, w, h: l_int );

function WidgetSendEvent( o: PWidget; Ev: TEvent ): l_bool;

procedure WidgetInsert ( o: PWidget; i: PWidget );
procedure WidgetRemove ( o: PWidget; i: PWidget );
procedure WidgetDrawAll ( o: PWidget );
procedure WidgetSetOSDRect ( r: TRect );
procedure WidgetUnOSD ( );
procedure WidgetDispose ( o: PWidget );
function WidgetExecute ( o: PWidget ): l_ulong;
procedure WidgetSetFirst ( o: PWidget );
procedure WidgetSetFocusEx ( o: PWidget );
procedure WidgetSetFocus ( o: PWidget );
procedure WidgetPreCenter ( o: PWidget ); // Use between insertion in parent and 1st draw
procedure WidgetUnMakeSync ( var o: PWidget );
procedure WidgetMakeSync ( var o: PWidget );
procedure WidgetSetChildArea( o: PWidget; r: TRect );
function WidgetSendPEvent ( o: PWidget; Ev: PEvent ): l_bool;
procedure WidgetSetRect ( o: PWidget; r: TRect );
procedure WidgetSelect ( o: PWidget );
procedure WidgetUnSelectAllSubs ( o: PWidget );
procedure WidgetDraw ( o: PWidget; w: PRect );
procedure WidgetSetFocusExNoNotify ( o: PWidget );
function WidgetSendSEvent ( o: PWidget; aType: l_int; Message: l_ulong; Extra: Pointer ): l_bool;
function WidgetSendEEvent ( o: PWidget; aType: l_int; Message: l_ulong; Extra: Pointer; Keyb: TKeybState; Mouse: TMouseState ): l_bool;
procedure WidgetSetTooltipText ( o: PWidget; t: l_text );
function WidgetGetIndex ( o: PWidget ): l_long;
procedure DragStart ( o: PWidget; DataType: l_ulong; var Data; SupportedMethods: l_ulong );
    procedure WidgetRefreshAbsolutePos ( o: PWidget ); // (not exported)
function WidgetCheckVisible ( o: PWidget ): l_bool;

function WidgetGetName ( o: PWidget; Name: l_text ): PWidget;
(*
////////////////////////////////////////////////////////////////////////////////
// Compatibility with old software parts
// define InsertWidget(o,i) WidgetInsert(o,i)
// define RemoveWidget(o,i) WidgetRemove(o,i)
// define WidgetSize(r,x,y,w,h) RectAssign(r,x,y,x+w,y+h)

////////////////////////////////////////////////////////////////////////////////
// Drag and drop methods
*)
const
 DMETHOD_COPY	=	$01;
 DMETHOD_MOVE	=	$02;
(*
////////////////////////////////////////////////////////////////////////////////
// Default functions for widget structure
*)
procedure _WidgetInsert ( o,i: PWidget );
procedure _WidgetRemove ( o,i: PWidget );
procedure _WidgetSetFirstEx ( o,i: PWidget );
(*
////////////////////////////////////////////////////////////////////////////////
//** @defgroup widgetflags Widget flags
//	Values that can be set into TWidget Flags field.
//    Use 'OR' to associate those flags.
//	@{
//*/
*)
const
 WFVisible		  =	$00000001; ///< Widget is draw
 WFForceBuffer	  =	$00000002; ///< System will always draw the widget using the buffer (do not blink, quite slow compared to non buffer for some graphics methods)
 WFForceNonBuffer =	$00000004; ///< System will always directlt to the screen. (fast for most graphics methods, can blink)
 WFFocused	      =	$00000008; ///< Widget is focused
 WFFocusable	  = $00000010; ///< Widget can be focused
 WFOnTop		  = $00000020;
 WFSelectable     = $00000040; ///< Widget can be selected 
 WFSelected		  = $00000080; ///< Widget is selected

 WF_TOOLTIPFUNC	  = $00000100; ///< Internal use : Widget is a function to set its tooltip
 WF_DRAGFOCUS     = $00000200; ///< Widget is focused in the current drag'n'drop session, read-only
 WFExecuting      = $80000000; ///< Internal use : Widget is waiting for a message (see WidgetExecute)

 WF_AUTORESIZE    = $00100000; ///< System wille resize automaticly widget if its parent is resizer using flolowing flags
 WF_FIXEDWIDTH    = $00200000; ///< Widget width is constant
 WF_FIXEDHEIGHT   = $00400000; ///< Widget height is constant
 WF_MAGNETRIGHT	  = $00800000; ///< Widget right segment has a constant distance to the right segment of the widget parent
 WF_MAGNETBOTTOM  = $01000000; ///< Widget bottom has a constant distance to the bottom of the widget parent

 WF_SCRIPTFLAGS   = (WF_AUTORESIZE or WF_FIXEDWIDTH or WF_FIXEDHEIGHT or WF_MAGNETRIGHT or WF_MAGNETBOTTOM);
(*
///**
//@}
//*/
////////////////////////////////////////////////////////////////////////////////
// Events

// 0xFFFF1XXX reserved for mouse events
*)
const
 WEvMouse		  = $FFFF1000;
 WEvKeyboard	  =	$FFFF2000;
 WEvNotify 		  = $FFFF4000;

 WEvMouseOther	  = $100;
 WEvMouseUp		  = $200;
 WEvMouseDown	  = $400;
 WEvMouseDClk	  = $800;

 WEvMouseR	=	$01;
 WEvMouseM	=	$02;
 WEvMouseL	=	$04;

 WEvMouseLeft	= $FFFF1101;
 WEvMouseEnter	= $FFFF1102;
 WEvMouseMove	= $FFFF1103;
 WEvMouseEnterDelay	= $FFFF1104;

 WEvMouseRUp	=	$FFFF1201;
 WEvMouseMUp	=	$FFFF1202;
 WEvMouseLUp	=	$FFFF1204;

 WEvMouseRDown =	$FFFF1401;
 WEvMouseMDown =	$FFFF1402;
 WEvMouseLDown =	$FFFF1404;

 WEvMouseRDClk =	$FFFF1801;
 WEvMouseMDClk =	$FFFF1802;
 WEvMouseLDClk =	$FFFF1804;

 WEvMouseRClick	= WEvMouseRUp;
 WEvMouseMClick	= WEvMouseMUp;
 WEvMouseLClick	= WEvMouseLUp;

 WEvNotifyLostFocus = $FFFF4001;
 WEvNotifyFocused   = $FFFF4002;
 WEvNotifySameFocus = $FFFF4003; // A focus change was try, but no change (usefull for some widget like menus)

 WEvNotifySelect =	$FFFF4004;
 WEvNotifyUnSelect =	$FFFF4005;
 MSG_NOTIFY_AUTO_RESIZE =	$FFFF4006;

 MSG_DRAGCANCEL 	 =   $FFFF4010;
 (*
 ////////////////////////////////////////////////////////////////////////////////
 ///*
 //*	Default messages (Reserved $FFFF8XXX)
 //*/
 *)
 MSG_OK		= $FFFF8001;
 MSG_CANCEL	= $FFFF8002;
 MSG_YES	= $FFFF8004;
 MSG_NO		= $FFFF8008;
 MSG_APPLY	= $FFFF8016;
 MSG_YESALL	= $FFFF8017;
(*
////////////////////////////////////////////////////////////////////////////////
// Modes switching
*)
procedure SwitchGraphicsMode ( );
procedure SwitchTextMode ( );
(*
////////////////////////////////////////////////////////////////////////////////
// Widgets class 
*)
const
 WCF_SATELLITE = $0001;

 WCF_BOOK      = $0002; // Global book widget composed of page 
 WCF_PAGE      = $0004;

 WCF_CONTAINER = $0008; 

var
 StdWidgetClass: PWidgetClass;
 WidgetClasses : PList;

function CreateWidgetClass ( Id: l_ulong; Parent: PWidgetClass; Flags: l_ulong ): PWidgetClass;
function WidgetIsClassId ( o: PWidget; Id: l_ulong ): l_bool;
function GetWidgetClass ( Id: l_ulong ): PWidgetClass;
procedure WClassRegisterPropEx ( c: PWidgetClass; tid: l_ulong; name: l_text; var Extra; FreeExtra: FreeProc );
procedure WClassRegisterProp ( c: PWidgetClass; tid: l_ulong; name: l_text );
function GetPropType ( Id: l_ulong ): PPropType;
function CreatePropType ( Id: l_ulong; Duplicate: DuplicateFunc; Free: FreeProc; Write: WriteProc; Read: ReadFunc ): PPropType;
procedure WClassResgisterFlag ( c: PWidgetClass; aProperty: l_text; Value: l_ulong; Name,Description: l_text );

const
 PTYPE_STRING  = 1;
 PTYPE_INT     = 2;
 PTYPE_BOOL    = 3;
 PTYPE_MESSAGE = 4;
 PTYPE_FLAGS   = 5;
 PTYPE_MENU    = 6;
(*
////////////////////////////////////////////////////////////////////////////////
// iWidget
*)
function NewiWidget ( C: PWidgetClass ): PiWidget;
procedure InsertiWidgetBefore ( o: PiWidget; i: PiWidget; b: PiWidget );
function ComplieiWidget( App: PApplication; iw: PiWidget ): PWidget;
procedure FreeiWidget ( o: PiWidget );
(*
////////////////////////////////////////////////////////////////////////////////
// Misc helping functions

// Helper function for multi-windows applications
*)
procedure DisposeAllAppWidgets ( App: PApplication );

function DuplicateBool ( var o ): Pointer;
function DuplicateInt ( var o ): Pointer;

procedure iWidgetInstall;
procedure iWidgetUnInstall;
procedure WClassInstall;
procedure WClassUnInstall;

function WClassGetProp ( c: PWidgetClass; name: l_text ): PWidgetProp;
(*
////////////////////////////////////////////////////////////////////////////////
// Spliters and auto-resize
*)
type
  TSpliter = record

	o: TWidget;

	Mode: l_uchar;

	Moving: l_bool;
	
  end;

const
  SP_VERTICAL	= 1;
  SP_HORIZONTAL	= 2;

{ .#define WCLASS_SPLITER ULONG_ID('z','S','p','l')  }
{ .#define WCLASS_HSPLITER ULONG_ID('H','S','p','l') }
{ .#define WCLASS_VSPLITER ULONG_ID('V','S','p','l') }

{ .#define SPLITER(o) ((PSpliter)(o)) }

procedure _WidgetAutosize ( o: PWidget; p: TRect; r: TRect );
procedure InitSpliters;

function CreateVSpliter  ( App: PApplication; r: TRect ): PWidget;
function CreateHSpliter  ( App: PApplication; r: TRect ): PWidget;

(*
///**
//	@}
//*/

//#endif /* _WIDGET_H_INCLUDED_ */
///** @defgroup widgets Widgets
//        System provided widgets.
//*/
///** @defgroup widgetsys Widget system
//        Manages the widgets displayed onto screen.
//        This is the base of all other provided widgets. By exemple a window is a standart widget with special extentions, so can parse it to functions but don't forget to convert the pointer type : use WIDGET(pointer).
//        @{
//*/
///**     @file
//        Widget system core file.
//*/
*)

var OSDRect: TRect;

implementation

{$ifndef LINK_CLIBS}
(*
 .#include "kernel.h"
 .#include "widget.h"
 .#include "grfx.h"
 .#include "stdlib.h"
 .#include <string.h>
////////////////////////////////////////////////////////////////////////////////
*)
var
 AppVersion: l_ulong;
 DoubleClickSpeed: l_int;
const
  nUID: l_uid                      = 'widget';
  AppName: PChar                   = 'Widget System';
  NeededLibs: array[0..2] of l_uid = ( 'grfx','skin', '' );
(*
////////////////////////////////////////////////////////////////////////////////
// Specifics widget management 
*)
  TopWidget: PWidget        = NIL;
  LookFor  : PWidget        = NIL;
(*
////////////////////////////////////////////////////////////////////////////////
// Common parameters *)
  buffer: p_bitmap          = NIL;
  WorkingFlags: l_ulong     = 0;

  WWF_TOOLTIPUP             = $02;
  WWF_DRAGGING              = $04;
(*
////////////////////////////////////////////////////////////////////////////////
// Mouse click and delay related *)
  LastMouseMove: TDateTime     = 0;
  LastNotified: TDateTime      = 0;
  LastToolTip: TDateTime       = 0;
  LastMouseClickLeft: TDateTime  = 0;
  LastMouseClickRight: TDateTime = 0;
  LastMouseClickMiddle: TDateTime = 0;
{ .#define DoubleClickSpeed (100*Mouse->DblClkSpeed)}
(*
////////////////////////////////////////////////////////////////////////////////
// Drag & Drop related defintions *)
  DragDataType: l_ulong     = 0;
  DragCaller: PWidget       = NIL;
  DragData: Pointer         = NIL;
  DragSupportedMethods: l_ulong  = 0;
  DragOldAccept: l_bool     = false;
  DragOldMethod: l_ulong    = 0;
  DragCur: PCursor          = NIL;
  NoDragCur: PCursor        = NIL;
  DragCopyCur: PCursor      = NIL;
(*  
////////////////////////////////////////////////////////////////////////////////
// "OSD" related defintions *)
  OSD: l_bool               = false;
  OSDIsRect: l_bool         = false;
  OSDColor: l_color         = 0;


function CreateWidgetClass ( Id: l_ulong; Parent: PWidgetClass; Flags: l_ulong ): PWidgetClass;
begin

end;

function WidgetIsClassId ( o: PWidget; Id: l_ulong ): l_bool;
begin

end;

function GetWidgetClass ( Id: l_ulong ): PWidgetClass;
begin

end;

procedure WClassRegisterPropEx ( c: PWidgetClass; tid: l_ulong; name: l_text; var Extra; FreeExtra: FreeProc );
begin

end;

procedure WClassRegisterProp ( c: PWidgetClass; tid: l_ulong; name: l_text );
begin

end;

function GetPropType ( Id: l_ulong ): PPropType;
begin

end;

function CreatePropType ( Id: l_ulong; Duplicate: DuplicateFunc; Free: FreeProc; Write: WriteProc; Read: ReadFunc ): PPropType;
begin

end;

procedure WClassResgisterFlag ( c: PWidgetClass; aProperty: l_text; Value: l_ulong; Name,Description: l_text );
begin

end;

function NewiWidget ( C: PWidgetClass ): PiWidget;
begin

end;

procedure InsertiWidgetBefore ( o: PiWidget; i: PiWidget; b: PiWidget );
begin

end;

function ComplieiWidget( App: PApplication; iw: PiWidget ): PWidget;
begin

end;

procedure FreeiWidget ( o: PiWidget );
begin

end;

function DuplicateBool ( var o ): Pointer;
var p: PBoolean;
begin
  New(p);
  p^ := Boolean(o);
  DuplicateBool := p;
end;

function DuplicateInt ( var o ): Pointer;
begin

end;

procedure iWidgetInstall;
begin

end;

procedure iWidgetUnInstall;
begin

end;

procedure WClassInstall;
begin

end;

procedure WClassUnInstall;
begin

end;

function WClassGetProp ( c: PWidgetClass; name: l_text ): PWidgetProp;
begin

end;

procedure _WidgetAutosize ( o: PWidget; p: TRect; r: TRect );
begin

end;

procedure InitSpliters;
begin

end;

function CreateVSpliter  ( App: PApplication; r: TRect ): PWidget;
begin

end;

function CreateHSpliter  ( App: PApplication; r: TRect ): PWidget;
begin

end;

(*
////////////////////////////////////////////////////////////////////////////////
// Drag&Drop related
////////////////////////////////////////////////////////////////////////////////
*)
procedure DragStop;
begin
        DebugMessage('Drag n Drop: DragStop',[]);
        if ( WorkingFlags and WWF_DRAGGING ) = WWF_DRAGGING then
        begin
                DragData := NIL;
                DragCaller := NIL;
                DragData := NIL;
                DragDataType := 0;
                DragSupportedMethods := 0;
                MouseSetCursor(CUR_POINTER);
                WorkingFlags := WorkingFlags and (not WWF_DRAGGING);
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function __WidgetDropAccept ( o: PWidget; var Method: l_ulong ): l_bool;
begin
        if assigned( o^.DropAccept ) then __WidgetDropAccept := o^.DropAccept(o,DragCaller,DragDataType,DragData,DragSupportedMethods,Method);
        __WidgetDropAccept := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DragStart ( o: PWidget; DataType: l_ulong; var Data; SupportedMethods: l_ulong );
var Method: l_ulong;
begin
        if ( WorkingFlags and WWF_DRAGGING ) = WWF_DRAGGING then DragStop();
        {$ifdef Debug}
        {$ifdef fpc}
        DebugMessage('Drag n Drop: DragStop', []);
        {$else}
        Writeln('Drag n Drop: DragStop');
        {$endif}
        {$endif}
        WorkingFlags         := WorkingFlags or WWF_DRAGGING;
        DragCaller           := o;
        DragDataType         := DataType;
        DragData             := DuplicateData2(DataType,Pointer(Data));
        DragSupportedMethods := SupportedMethods;
        o^.Flags := o^.Flags or WF_DRAGFOCUS;

        if ( DragOldAccept = __WidgetDropAccept(TopWidget,&Method) ) then begin
                if ( Method = DMETHOD_COPY ) then
                        MouseSetCursor(DragCopyCur)
                else
                        MouseSetCursor(DragCur);
        end else
                MouseSetCursor(NoDragCur);

        DragOldMethod := Method;
end;
(*
////////////////////////////////////////////////////////////////////////////////
// Mouse related
////////////////////////////////////////////////////////////////////////////////
*)
function TopWidgetUnderPoint ( o: PWidget; p: TPoint ): PWidget;
var a,b: PWidget;
begin
  if assigned( o^.Last ) then
  begin
    a := o^.Last^.Next;
    b := a;
    while a <> NIL do begin
      if ( PointInRect(p,a^.Absolute) and ((a^.Flags and WFVisible)=WFVisible)) then TopWidgetUnderPoint := TopWidgetUnderPoint(a,p);
            a := a^.Next;
    end; {while ( a != b );}
  end;
  TopWidgetUnderPoint := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function GetTopWidgetUnderMouse: PWidget;
begin
  GetTopWidgetUnderMouse := TopWidgetUnderPoint(DeskTop,Mouse^.State.p);
end;
(*
////////////////////////////////////////////////////////////////////////////////
// OSD related
////////////////////////////////////////////////////////////////////////////////
*)
procedure DrawOSDOnRect ( {$ifdef fpc}out{$else}var{$endif} bmp: p_bitmap; r: TRect );
var
  Tmp: TRect;
begin
  if ( OSDIsRect ) then
  begin
    if ( RectOverlay(r, OSDRect ) ) then
    begin
          
      RectIntersept(Tmp,r,OSDRect);
      al_xor_mode(TRUE);
      if ( Tmp.a.x = OSDRect.a.x ) then al_vline(bmp,Tmp.a.x,Tmp.a.y,Tmp.b.y,OSDColor);
      if ( Tmp.b.x = OSDRect.b.x ) then al_vline(bmp,Tmp.b.x,Tmp.a.y,Tmp.b.y,OSDColor);
      if ( Tmp.a.y = OSDRect.a.y ) then al_hline(bmp,Tmp.a.x,Tmp.a.y,Tmp.b.x,OSDColor);
      if ( Tmp.b.y = OSDRect.b.y ) then al_hline(bmp,Tmp.a.x,Tmp.b.y,Tmp.b.x,OSDColor);
      al_xor_mode(FALSE);
    end;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetSetOSDRect ( r: TRect );
begin
  MouseHide();
  PauseMultitasking();
  al_xor_mode(TRUE);
  al_set_clip_rect(scrn,0,0,scrn^.w-1,scrn^.h-1);
  if ( OSD and OSDIsRect ) then begin
  al_rect(scrn,OSDRect.a.x,OSDRect.a.y,OSDRect.b.x,OSDRect.b.y,OSDColor);
  OSDRect := r;
  end else begin
          OSD := true;
          OSDIsRect := true;
          OSDRect := r;
  end;
  al_rect(scrn,OSDRect.a.x,OSDRect.a.y,OSDRect.b.x,OSDRect.b.y,OSDColor);
  al_xor_mode(FALSE);
  UnPauseMultitasking();
  MouseShow();
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetUnOSD; 
begin
  if ( OSD and OSDIsRect ) then 
  begin
    MouseHide();
    PauseMultitasking();
      al_xor_mode(TRUE);
      al_set_clip_rect(scrn,0,0,scrn^.w-1,scrn^.h-1);
      al_rect(scrn,OSDRect.a.x,OSDRect.a.y,OSDRect.b.x,OSDRect.b.y,OSDColor);
      al_xor_mode(FALSE);
    UnPauseMultitasking();
    MouseShow();
  end;
  OSD := false;
  OSDIsRect := false;
end;
(*
////////////////////////////////////////////////////////////////////////////////
// Draw related
////////////////////////////////////////////////////////////////////////////////

        Check if all widget parent are visible. Using this function you ensure that widget is really visible.
        @param o Widget to check
        @return true if widget and all its parents are visible, else return false
*/
*)
function WidgetCheckVisible ( o: PWidget ): l_bool;
begin
        while ( o = o^.Parent ) do begin
                if ( not(o^.Flags and WFVisible)=WFVisible ) then WidgetCheckVisible := false;
        end;
        WidgetCheckVisible := true;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetGenerateCacheExBrothers ( l: PList; o,i: PWidget; w: TRect ); {$ifndef Ver70}inline;{$endif}
var a: PWidget;
begin
        if Assigned( o^.Parent ) then begin
                a := o^.Parent^.Last^.Next;
                while ( a <> o ) do begin
                        if ( ( a^.Flags and WFVisible ) = WFVisible ) and RectOverlay(w, a^.DrawAbsolute) then
                                RemoveZoneInList(l,a^.DrawAbsolute);
                        a := a^.Next;
                end;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetGenerateCacheExSubs ( l: PList; o: PWidget; w: TRect ) {$ifndef Ver70}inline;{$endif}
var a,b: PWidget;
begin
        if Assigned( o^.Last ) then begin
                a := o^.Last^.Next;
                b := o^.Last^.Next;
                while ( a <> nil ) do begin
                        if ((a^.Flags and WFVisible)=WFVisible) and RectOverlay(w, a^.DrawAbsolute) then
                                RemoveZoneInList(l,a^.DrawAbsolute);
                        a := a^.Next;
                end;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function HasMinDimensions(r: TRect): l_bool;
begin
  if (r.b.x - r.a.x >= 5) and (r.b.y - r.a.y >= 5) then HasMinDimensions := true else HasMinDimensions := false;
end;

function WidgetGenerateCache ( o: PWidget; var w: TRect; Clip: PList ): PList;
var
  a: PWidget;
  l: PList;
  Tmp: TRect;      
begin        
        //DebugMessage ("WidgetGenerateCache(%s)",o^.Name);
        a := o;
        l := NIL;

        if Assigned( Clip ) then begin
                w := o^.DrawAbsolute;
                l := NewListFromRectListIntersept(Clip,w);
        end else if HasMinDimensions( w ) then begin // *w and o^.Absolute MUST have a common part
                RectIntersept(Tmp,o^.DrawAbsolute,w);
                l := NewListFromRect(Tmp);
        end else if ( o^.DrawAbsolute.a.x-o^.DrawAbsolute.b.x+1 > 0 ) or ( o^.DrawAbsolute.a.y-o^.DrawAbsolute.b.y+1 > 0) then begin
                l := NewListFromRect(o^.DrawAbsolute);
                w := o^.DrawAbsolute;
        end else
                WidgetGenerateCache := NIL;

        WidgetGenerateCacheExSubs(l,o,w);
        repeat
                WidgetGenerateCacheExBrothers(l,a,o,w);
        until ( a <> a^.Parent );

        WidgetGenerateCache := l;
end;
(*
////////////////////////////////////////////////////////////////////////////////
// Independant function, as it can usefull for some widget
*)
procedure WidgetDrawFromBuffer ( o: PWidget; l: PList; {$ifdef OPascal}out{$else}var{$endif} bmp: p_bitmap; Buffer: p_bitmap; d: PPoint; var w: TRect );
var a,b: PListItem; r: PRect;
begin
  if ((o^.Flags and WFVisible)=WFVisible) then
  if Assigned( l ) then 
  begin
    a := l^.Last;
    b := a;
    if Assigned( a ) then
    if Assigned( d ) then 
    begin
            al_set_clip_rect(bmp,max(0,o^.DrawAbsolute.a.x+d^.x),
                max(0,o^.DrawAbsolute.a.y+d^.y),
                min(bmp^.w-1,o^.DrawAbsolute.b.x+d^.x),
                min(bmp^.h-1,o^.DrawAbsolute.b.y+d^.y));

                  //blit(Buffer,screen,0,screen^.h/2,0,0,screen^.w, screen^.h/2 );
                repeat
                    Pointer(r) := Pointer(a^.Data);
                    al_blit(Buffer,bmp,r^.a.x, r^.a.y,
                                    r^.a.x+d^.x, r^.a.y+d^.y,
                                    r^.b.x-r^.a.x+1, r^.b.y-r^.a.y+1 );
                    a := a^.Next;
                until ( a = b ) ;
    end 
    else 
    begin
                    al_set_clip_rect(bmp,max(0,o^.DrawAbsolute.a.x),
                            max(0,o^.DrawAbsolute.a.y),
                            min(bmp^.w-1,o^.DrawAbsolute.b.x),
                            min(bmp^.h-1,o^.DrawAbsolute.b.y));
            repeat
                          r := PRect(a^.Data);
                          if ( OSD ) then DrawOSDOnRect(Buffer,r^);
                          _MouseRecap(Buffer,r^);
                    al_blit(Buffer,bmp,r^.a.x, r^.a.y,
                                    r^.a.x, r^.a.y,
                                    r^.b.x-r^.a.x+1, r^.b.y-r^.a.y+1 );
                    a := a^.Next;
            until ( a = b ) ;
    end;
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetRefresh ( o:PWidget; w:PRect; List:PList; var bmp:p_bitmap; d:PPoint);
var l: PList; r,t: PRect; Tmp: TRect; a,b: PListItem;
begin
  if ( o^.Flags and WFVisible ) = WFVisible then
  begin
    l := WidgetGenerateCache(o,w^,List);
        
    if ( l <> nil ) then

    if Assigned( l^.Last ) then // There is at least something to draw
    begin

        PauseMultitasking(); // Disable multitasking to prevent graphics bugs
        if not Assigned( o^.Draw ) then
        begin
                        // No custom draw function //////////////////////////////////////////////////
                        // As default is rectfill, no buffer is needed,
                        // It's faster than ever !!
                        // *BUT* No alpha blending support

          a := l^.Last;
          b := a;

          al_acquire_bitmap(bmp); {Bitmap beschaffen, holen}

          if Assigned( d ) then 
          begin
                al_set_clip_rect(bmp,max(0,o^.DrawAbsolute.a.x+d^.x),
                                    max(0,o^.DrawAbsolute.a.y+d^.y),
                                    min(bmp^.w-1,o^.DrawAbsolute.b.x+d^.x),
                                    min(bmp^.h-1,o^.DrawAbsolute.b.y+d^.y));
              repeat
                 r := a^.Data;
                 al_rectfill(bmp,r^.a.x+d^.x,r^.a.y+d^.y,r^.b.x+d^.x,r^.b.y+d^.y,o^.BackgroundColor);
                 a := a^.Next;
              until ( a = b ) ;

          end 
          else 
          begin
            al_set_clip_rect(bmp,max(0,o^.DrawAbsolute.a.x),
                                    max(0,o^.DrawAbsolute.a.y),
                                    min(bmp^.w-1,o^.DrawAbsolute.b.x),
                                    min(bmp^.h-1,o^.DrawAbsolute.b.y));
              repeat
                 Pointer(r) := Pointer(a^.Data);
                 al_rectfill(bmp,r^.a.x,r^.a.y,r^.b.x,r^.b.y,o^.BackgroundColor);
                 if ( OSD ) then DrawOSDOnRect(bmp,r^);
                 _MouseRecap(bmp,r^);
                 a := a^.Next;
              until ( a = b ) ;
          end;
          al_release_bitmap(bmp);
        end 
        else  
        if ((l^.Last = l^.Last^.Next) and not((o^.Flags and WFForceBuffer)=WFForceBuffer) and not Assigned(d)) then
        begin
          (*// Only a rect to draw, so we can draw directly to screen /////////////////*)
          (*// Faster*)
          Pointer(r) := Pointer(l^.Last^.Data);

          al_acquire_bitmap(bmp);

          al_set_clip_rect(bmp,max(0,r^.a.x),
                            max(0,r^.a.y),
                            min(bmp^.w-1,r^.b.x),
                            min(bmp^.h-1,r^.b.y));
            o^.Draw     (o,bmp,r);
            if ( OSD ) then DrawOSDOnRect(bmp,r^);
            _MouseRecap(bmp,r^);

            al_release_bitmap(bmp);
        end 
        else
        if not((o^.Flags and WFForceNonBuffer) = WFForceNonBuffer)  and
           not((o^.Flags and WFForceBuffer) = WFForceBuffer) and (not Assigned(d)) then
        begin
             (*
             Widget do not want to use automatic buffer /////////////////////////////
             Not always faster
             *)
          a := l^.Last;
          b := a;
          al_acquire_bitmap(bmp);

          repeat
             Pointer(r) := Pointer(a^.Data);
                    al_set_clip_rect(bmp,max(0,r^.a.x),
                              max(0,r^.a.y),
                              min(bmp^.w-1,r^.b.x),
                              min(bmp^.h-1,r^.b.y));
                        o^.Draw (o,bmp,r);
                        if ( OSD ) then DrawOSDOnRect(bmp,r^);
                        _MouseRecap(bmp,r^);
              a := a^.Next;
          until ( a = b );

          al_release_bitmap(bmp);
        end;  
    end 
    else 
    if Assigned( buffer ) then
    begin
                Tmp := o^.Absolute;
                if Assigned( w ) then RectIntersept(Tmp,o^.DrawAbsolute,w^);
                al_set_clip_rect(buffer,max(0,o^.DrawAbsolute.a.x),
                        max(0,o^.DrawAbsolute.a.y),
                        min(buffer^.w-1,o^.DrawAbsolute.b.x),
                        min(buffer^.h-1,o^.DrawAbsolute.b.y));

                o^.Draw (o,buffer,@Tmp);
                al_acquire_bitmap(bmp);
                WidgetDrawFromBuffer(o,l,bmp,buffer,d,w^);
                al_release_bitmap(bmp);
    end;
    UnPauseMultitasking;
  end;
  FreeList(l);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetDrawOnRectListEx ( o: PWidget; List: PList; Output: PImage; Delta: PPoint );
var a,b: PWidget;
begin
        if ( o^.Flags and WFVisible ) = WFVisible then
        begin
                if ( RectOverlayList(List,o^.DrawAbsolute) ) then
                begin
                        o^.Refresh(o,@o^.DrawAbsolute,List,Output,Delta);
                        if Assigned( o^.Last ) then
                        begin
                                a := o^.Last^.Next;
                                b := a;
                                repeat
                                        WidgetDrawOnRectListEx(a,List,Output,Delta);
                                        a := a^.Next;
                                until ( a = b );
                        end;
                end;
        end;
end;

(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetDrawOnRectEx ( o: PWidget; w: TRect; Ignore: PWidget );
var a,b: PWidget;
begin
        if ( RectOverlay (o^.DrawAbsolute,w)) and (( o^.Flags and WFVisible ) = WFVisible ) then
        begin
                o^.Refresh(o,@w,NIL,scrn,NIL);
                if Assigned( o^.Last ) then
                begin
                  a := o^.Last;
                  b := a;
                  repeat
                     if ( a <> Ignore ) then WidgetDrawOnRect(a,w,Ignore);
                     a := a^.Next;
                  until ( a = b );
                end;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetDrawAllEx ( o: PWidget );
var a,b: PWidget;
begin
  if ( o^.Flags and WFVisible ) = WFVisible then
  begin
    o^.Refresh(o,NIL,NIL,scrn,NIL);
    if Assigned( o^.Last ) then
    begin
      a := o^.Last;
      b := a;
      repeat
         WidgetDrawAll(a);
         a := a^.Next;
      until ( a = b );
    end;
  end;
end;
(*
////////////////////////////////////////////////////////////////////////////////
/**
        Draw widget and all its childs clipped by specified Rect.
        @param o Widget to redraw
        @param w Pointer to the clipping rect. Can't be NIL use WidgetDrawAll instead.
        @param Ignore A widget to ignore (won't be redraw). Can be NIL.
        @see WidgetDrawAll
**/
*)
procedure WidgetDrawOnRect ( o: PWidget; w: TRect; Ignore: PWidget );
begin
  if WidgetCheckVisible(o) then WidgetDrawOnRectEx(o,w,Ignore);
end;
(*
/**
        Draw widget and all its childs.
        @param o Widget to redraw
*/
////////////////////////////////////////////////////////////////////////////////
*)
procedure WidgetDrawAll ( o: PWidget );
begin
        if WidgetCheckVisible(o) then WidgetDrawAllEx(o);
end;
(*
////////////////////////////////////////////////////////////////////////////////
/**
        Draw widget clipped by specified Rect (can be NIL).
        Do not redraw its childs.
        @param o Widget to redraw
        @param w Pointer to the clipping rect, if NIL use widget DrawAbsolute
        @see WidgetDrawAll
        @see WidgetDrawOnRect
*/
*)
procedure WidgetDraw ( o: PWidget; w: PRect );
begin
  if WidgetCheckVisible(o) then o^.Refresh(o,w,NIL,scrn,NIL);
end;
(*
////////////////////////////////////////////////////////////////////////////////
// Widget Tree related
////////////////////////////////////////////////////////////////////////////////
*)
procedure RemoveWidgetEx ( o, i: PWidget );
begin
  if ( i^.Next = i ) then  {// Alone ...}
  begin
          o^.Last := NIL;
  end else 
  begin
          i^.Next^.Prev := i^.Prev;
          i^.Prev^.Next := i^.Next;
          if ( o^.Last = i ) then o^.Last := i^.Prev;
  end;
  i^.Parent := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure _WidgetRemove ( o,i: PWidget );
var a,b: PWidget;
begin

        RemoveWidgetEx(o,i);
        WidgetDrawOnRect(o,i^.DrawAbsolute,NIL);

        if (( i^.Flags and WFSelected ) = WFSelected)  and assigned(o^.Last) then
        begin
                a := o^.Last^.Next;
                b := a;
                repeat
                        WidgetDrawAll(a);
                        if ( a^.Flags and WFSelectable ) = WFSelectable then
                        begin
                                WidgetSelect(a);
                                {return;}
                        end;
                        a := a^.Next;
                until ( a = b );
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////
/**
        Remove a widget (@a i) from its parent (@a o)
        @param o Parent widget
        @param i Widget to remove
*/
*)
procedure WidgetRemove ( o,i: PWidget );
begin
        o^.Remove(o,i);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure _WidgetInsertBefore ( o,i,b: PWidget );
var t: PWidget;
begin
        t := b;


        if not Assigned( o^.Last ) then
        begin
                i^.Next := i;
                i^.Prev := i;
                o^.Last := i;
        end
        else
        begin
                if Assigned( b ) then
                begin
                        i^.Prev := b^.Prev;
                        i^.Next := b;
                        b^.Prev^.Next := i;
                        b^.Prev := i;
                end
                else
                begin
                        i^.Next := o^.Last^.Next;
                        i^.Prev := o^.Last;
                        o^.Last^.Next^.Prev := i;
                        o^.Last^.Next := i;
                        o^.Last := i;
                end;
        end;

        i^.Parent := o;

        WidgetRefreshAbsolutePos(i);
        (*
        /*
        *       Call setup only once, not on z-order cahnge
        */
        *)
        if assigned( i^.Setup ) then
        begin
                i^.Setup(i);
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure _WidgetInsert ( o,i: PWidget );
begin

        WidgetUnSelectAllSubs(o);
        if ( o^.Flags and WFSelectable ) = WFSelectable then o^.Flags := o^.Flags or WFSelected;

        if assigned( o^.Last ) then
                _WidgetInsertBefore(o, i, o^.Last^.Next)
        else
                _WidgetInsertBefore(o, i, NIL);
end;
(*
////////////////////////////////////////////////////////////////////////////////
/**
        Insert a widget (@a i) into an other (@a o)
        @param o Destination widget
        @param i Widget to insert
*/
*)
procedure InsertWidget ( o,i: PWidget );
begin
  WidgetInsert ( o,i );
end;

procedure WidgetInsert ( o,i: PWidget );
begin
        o^.Insert(o,i);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure _WidgetSetFirstEx ( o,i: PWidget  );
begin
  RemoveWidgetEx(o,i);
  _WidgetInsertBefore(o, i, o^.Last^.Next);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetSetFirst ( o: PWidget );
begin

        if Assigned( o^.Parent ) then o^.Parent^.SetFirstEx(o^.Parent,o);

        WidgetSelect(o);

        WidgetDraw(o,NIL);
        WidgetDrawAll(o);
end;
(*
////////////////////////////////////////////////////////////////////////////////
/**
        Calculate widget index in its parent
        @param o Widget
        @return Widget index or -1 on error (widget not found)
*/
*)
function WidgetGetIndex ( o: PWidget ): l_long;
var a,b: PWidget; idx: l_long;
begin
        idx:=0;
        if not Assigned( o^.Parent ) then WidgetGetIndex := -1;
        if not Assigned( o^.Parent^.Last ) then WidgetGetIndex := -1;
        a := o^.Parent^.Last^.Next;
        b := o^.Parent^.Last^.Next;
        repeat
                if ( a = o ) then begin WidgetGetIndex := idx; break; end;
                Inc(idx);
                a := a^.Next;
        until ( a = b );
        WidgetGetIndex := -1;
end;
(*
////////////////////////////////////////////////////////////////////////////////
/**
        Get a sub-widget by its name
        @param o Widget to search in
        @param Name Case sensitive name of widget to found
        @return Pointer to widget or NIL if not found.
*/
*)
function WidgetGetName ( o: PWidget; Name: l_text ): PWidget;
var a,b,r: PWidget;
begin

        if ( o^.Name <> '' ) then
                if ( TextCompare(o^.Name,Name) = 0 ) then WidgetGetName := o;

        if Assigned( o^.Last ) then
        begin
                a := o^.Last^.Next;
                b := o^.Last^.Next;
                repeat
                        if ( r = WidgetGetName(a,Name) ) then begin WidgetGetName := r; break; end;
                        a := a^.Next;
                until ( a = b );

        end;

        WidgetGetName := NIL;
end;
(*
////////////////////////////////////////////////////////////////////////////////
// Widget Positions and related...
////////////////////////////////////////////////////////////////////////////////
*)
procedure WidgetRefreshAbsolutePos ( o: PWidget );
var a,b: PWidget; Tmp: TRect;
begin
  if Assigned( o^.Parent ) then
  begin
    RectMove(Tmp,o^.Parent^.ChildArea,o^.Parent^.Absolute.a);
    RectMove(o^.Absolute,o^.Relative,Tmp.a);
    if ( RectOverlay(o^.Absolute,o^.Parent^.ChildDrawAbs) )
        then RectIntersept(o^.DrawAbsolute,o^.Absolute,o^.Parent^.ChildDrawAbs) // It can have a part out of it owner !!!
        else RectAssign(o^.DrawAbsolute,0,0,-1,-1);
                
  end;

  RectMove(Tmp,o^.ChildArea,o^.Absolute.a);
  RectIntersept(o^.ChildDrawAbs,Tmp,o^.DrawAbsolute); // It can have a part out of it owner !!!

  if assigned( o^.Last ) then
  begin
    a := o^.Last;
    b := a;
    if Assigned( a ) then
    repeat
      WidgetRefreshAbsolutePos(a);
      a := a^.Next;
    until ( a = b );
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetRefreshAbsolutePosEx ( o: PWidget );
var tmp: TRect;
begin
        if assigned( o^.Parent ) then
        begin
                RectMove(&Tmp,o^.Parent^.ChildArea,o^.Parent^.Absolute.a);
          RectMove(o^.Absolute,o^.Relative,Tmp.a);
                if ( RectOverlay(o^.Absolute,o^.Parent^.ChildDrawAbs) ) then begin
                        RectIntersept(o^.DrawAbsolute,o^.Absolute,o^.Parent^.ChildDrawAbs);  { It can have a part out of it owner !!! }
                end else begin
                        RectAssign(o^.DrawAbsolute,0,0,-1,-1);
                end;
         end;

        RectMove(Tmp,o^.ChildArea,o^.Absolute.a);
        RectIntersept(o^.ChildDrawAbs,Tmp,o^.DrawAbsolute);  { It can have a part out of it owner !!! }

end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetSetChildArea( o: PWidget; r: TRect );
begin
        o^.ChildArea := r;
        WidgetRefreshAbsolutePos(o);

end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetSetRect ( o: PWidget; r: TRect );
var p,n: TRect; a,b: PWidget;
begin
        p := o^.ChildArea;

        o^.ChildArea.b.x := r.b.x-r.a.x-o^.Relative.b.x+o^.ChildArea.b.x+o^.Relative.a.x;
        o^.ChildArea.b.y := r.b.y-r.a.y-o^.Relative.b.y+o^.ChildArea.b.y+o^.Relative.a.y;
        o^.Relative := r;

        WidgetRefreshAbsolutePosEx(o);
(*
          if assigned( o^.Last ) then 
          begin
            a := o^.Last;
            b := a;
            r := o^.ChildArea;
            repeat
                if ( a^.Flags and WF_AUTORESIZE ) = WF_AUTORESIZE then
                begin
                        n := a^.Relative;
                        if ( a^.Flags and WF_MAGNETRIGHT ) = WF_MAGNETRIGHT then
                        begin
                                if ( a^.Flags and WF_FIXEDWIDTH ) = WF_FIXEDWIDTH then n.a.x := n.a.x - p.b.x-o^.ChildArea.b.x;
                                n.b.x := n.b.x - p.b.x-o->ChildArea.b.x;
                        end;
                        if ( a->Flags and WF_MAGNETBOTTOM ) = WF_MAGNETBOTTOM then
                        begin
                                if ( a->Flags & WF_FIXEDHEIGHT ) n.a.y := n.a.y - p.b.y-o->ChildArea.b.y;
                                n.b.y := n.b.y - p.b.y-o->ChildArea.b.y;
                        end;
                        WidgetSetRect(a,n);
                        WidgetSendSEvent(a,EV_MESSAGE,MSG_NOTIFY_AUTO_RESIZE,NIL);
                end else
                        WidgetRefreshAbsolutePos(a);
              a := a^.Next;
            until ( a = b );
          end;*)

         if assigned( o^.Last ) then _WidgetAutosize(o,p,o^.ChildArea);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetMove ( o: PWidget; dX, dY: l_long );
var r,t: TRect; List: PList;
begin
        r := o^.Relative;
        t := o^.DrawAbsolute;
        r.a.x := r.a.x + dX; r.a.y := r.a.y + dY;
        r.b.x := r.b.x + dX; r.b.y := r.b.y + dY;
        WidgetSetRect(o,r);
     al_vsync;
        WidgetDrawAll(o);
        List := NewListFromRect(t);
        RemoveZoneInList(List,o^.DrawAbsolute);
        WidgetDrawOnRectListEx(o^.Parent, List, scrn, NIL);
        FreeList(List);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetMoveToPoint ( o: PWidget; x, y: l_int );
begin
        WidgetMove(o,x-o^.Relative.a.x,y-o^.Relative.a.y);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetSetMetrics ( o: PWidget; x, y, w, h: l_int );
var r,t: TRect; List: PList;
begin
        r := o^.Relative;
        t := o^.DrawAbsolute;
        
        r.a.x := x;
        r.a.y := y;
        r.b.x := w;
        r.b.y := h;
        WidgetSetRect(o, r);
        WidgetDrawAll(o);
        List := NewListFromRect(t);
        RemoveZoneInList(List,o^.DrawAbsolute);
        WidgetDrawOnRectListEx(o^.Parent, List, scrn, NIL);
        FreeList(List);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetResize ( o: PWidget; dW, dH: l_long );
var r,t: TRect; List: PList;
begin        
        
        r := o^.Relative;
        t := o^.DrawAbsolute;
        r.b.x := r.b.x + dW; r.b.y := r.b.y + dH;
        WidgetSetRect(o,r);
        WidgetDrawAll(o);
        List := NewListFromRect(t);
        RemoveZoneInList(List,o^.DrawAbsolute);
        WidgetDrawOnRectListEx(o^.Parent, List, scrn, NIL);
        FreeList(List);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetResizeToSize ( o: PWidget; SizeX, SizeY: l_long );
var r,t: TRect; List: PList;
begin
        r := o^.Relative;
        t := o^.DrawAbsolute;
        
        r.b.x := o^.Absolute.a.x + SizeX;
        r.b.y := o^.Absolute.a.y + SizeY;
        WidgetSetRect(o, r);
        WidgetDrawAll(o);
        List := NewListFromRect(t);
        RemoveZoneInList(List,o^.DrawAbsolute);
        WidgetDrawOnRectListEx(o^.Parent, List, scrn, NIL);
        FreeList(List);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetPreCenter ( o: PWidget );
var r,rr: TRect; w,h,hh,ww: l_long;
begin
        r := o^.Relative;
        rr := o^.Parent^.ChildArea;
        w := r.b.x - r.a.x; h := r.b.y - r.a.y;
        ww := rr.b.x - rr.a.x; hh := rr.b.y - rr.a.y;
        o^.Relative.a.x := (ww-w) div 2;
        o^.Relative.a.y := (hh-h) div 2;
        o^.Relative.b.x := o^.Relative.a.x+w;
        o^.Relative.b.y := o^.Relative.a.y+h;
        WidgetRefreshAbsolutePos(o);
end;

procedure WidgetPostCenter ( o: PWidget );
var r,rr: TRect; w,ww,h,hh: l_long;
begin        
        r := o^.Relative;        
        rr := o^.Parent^.ChildArea;
        w := r.b.x - r.a.x; h := r.b.y - r.a.y;
        ww := rr.b.x - rr.a.x; hh := rr.b.y - rr.a.y;
        o^.Relative.a.x := (ww-w) div 2;
        o^.Relative.a.y := GSScreenHeight-33;
        o^.Relative.b.x := o^.Relative.a.x+w;
        o^.Relative.b.y := GSScreenHeight;
        WidgetRefreshAbsolutePos(o);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Event related                                                                *)
(*////////////////////////////////////////////////////////////////////////////////*)
function WidgetSendEventEx ( o: PWidget; Ev: TEvent ): l_bool;
begin
{ DebugMessage('%x :: %d %x',[o,Ev.Type,Ev.Message]); }

  WidgetSendEventEx := false;

  if ( ( o^.Flags and WFExecuting ) = WFExecuting ) and (Ev.aType = EV_MESSAGE) then
  begin
     DebugMessage('Catch message( %x, %x )',[o,Ev.Message]);
     o^.MessageBuffer := Ev.Message;
     WidgetSendEventEx := true;
  end;

  if assigned( o^.AppEvHdl ) then
    if ( o^.AppEvHdl(o, @Ev) ) then WidgetSendEventEx := true;

        if assigned( o^.EventHandler ) then
          if ( o^.EventHandler(o, @Ev) ) then WidgetSendEventEx := true;

end;
(*////////////////////////////////////////////////////////////////////////////////*)
(* ** Due to unknown threading bugs, this section is unactivated **

procedure WidgetRedirectEvent ( Dest: Pointer; Ev: PDynLdEvent );
begin
  if (not WidgetSendEventEx(WIDGET(Dest), Ev^.Ev )) then
  begin

          if assigned(PWIDGET(Dest)^.Parent) and (Ev^.Ev.Type <> EV_MOUSE) then
            WidgetSendEvent( PWIDGET(Dest)^.Parent, Ev^.Ev );
  end;
end;
*)
(*////////////////////////////////////////////////////////////////////////////////*)
function WidgetSendEvent ( o: PWidget; Ev: TEvent ): l_bool;
begin
 (* ** Due to unknown threading bugs, this section is unactivated **
  if assigned( o^.AppOwner ) then
  begin
    if ( o^.AppOwner^.Type = DYNLD_TYPEAPP ) then           { Only App can handle events, Dl and old style programs can't }
    begin            
            AppSendEventEv( o^.AppOwner, Ev, o, WidgetRedirectEvent );
            WidgetSendEvent := true;
    end;
  end; 
 *)

        if ( WidgetSendEventEx(o,Ev) ) then WidgetSendEvent := true;
        if ( assigned(o^.Parent) and ( Ev.aType <> EV_MOUSE ) and not( (Ev.aType = EV_MESSAGE) and (Ev.Message = MSG_NOTIFY_AUTO_RESIZE) )  ) then
          WidgetSendEvent := WidgetSendEvent( o^.Parent, Ev )
        else
    WidgetSendEvent := false;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function WidgetSendPEvent ( o: PWidget; Ev: PEvent ): l_bool;
var r: l_bool;
begin
        r := WidgetSendEvent(o, Ev^);
        Dispose(Ev);
        WidgetSendPEvent := r;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function WidgetSendEEvent( o: PWidget; aType: l_int; Message: l_ulong; Extra: Pointer; Keyb: TKeybState; Mouse: TMouseState ): l_bool;
var Ev: TEvent;
begin
        Ev.aType := aType;
        Ev.Message := Message;
        Ev.Extra := Extra;
        Ev.Keyb := Keyb;
        Ev.Mouse := Mouse;
        WidgetSendEEvent := WidgetSendEvent(o,Ev);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
function WidgetSendSEvent( o: PWidget; aType: l_int; Message: l_ulong; Extra: Pointer ): l_bool;
var Ev: TEvent;
begin
        Ev.aType := aType;
        Ev.Message := Message;
        Ev.Extra := Extra;
        Ev.Keyb := KState;
        Ev.Mouse := Mouse^.State;
        WidgetSendSEvent := WidgetSendEvent(o,Ev);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetSetFocusExNoNotify ( o: PWidget );
begin
        if assigned( o^.FocusBack ) then o := o^.FocusBack;
        if ( o^.Flags and WFFocusable ) = WFFocusable then
        begin
                if assigned( FocusedWidget ) then
                   FocusedWidget^.Flags := FocusedWidget^.Flags and not WFFocused;
                o^.Flags := o^.Flags or WFFocused;
                FocusedWidget := o;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetSetFocusEx ( o: PWidget );
var Old: PWidget;
begin
        if assigned( o^.FocusBack ) then o := o^.FocusBack;
        if ( o^.Flags and WFFocusable ) = WFFocusable then
        begin
                Old := FocusedWidget;
                if assigned( FocusedWidget ) then FocusedWidget^.Flags := FocusedWidget^.Flags and not WFFocused;
                o^.Flags := o^.Flags or WFFocused;
                FocusedWidget := o;
                if assigned( Old ) then
                        WidgetSendSEvent(Old,EV_MESSAGE,WEvNotifyLostFocus,NIL);
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetSetFocus ( o: PWidget );
begin
        if assigned( o^.FocusBack ) then o := o^.FocusBack;
        
        if not( FocusedWidget = o ) then

        if ( o^.Flags and WFFocusable ) = WFFocusable then
        begin
                WidgetSetFocusEx(o);

                if assigned( FocusedWidget ) then
                        WidgetSendSEvent(FocusedWidget,EV_MESSAGE,WEvNotifyFocused,NIL);
        end
        else
        begin
                if assigned( o^.Parent ) then
                        WidgetSetFocus(o^.Parent)
                else if assigned( FocusedWidget ) then
                        WidgetSendSEvent(FocusedWidget,EV_MESSAGE,WEvNotifySameFocus,NIL);
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetSelect ( o: PWidget );
var a,b: PWidget;
begin
  if ((o^.Flags and WFSelectable) = WFSelectable) and not((o^.Flags and WFSelected) = WFSelected) then
  begin
    o^.Flags := o^.Flags or WFSelected;
    WidgetSendSEvent(o,EV_MESSAGE,WEvNotifySelect,NIL);
  end;

  if assigned( o^.Parent ) then
  begin
    a := o^.Parent^.Last;
    b := a;

    if assigned( a ) then
    repeat
      if ( (a <> o) and ((a^.Flags and WFSelected)=WFSelected) and ((a^.Flags and WFSelectable)=WFSelectable) ) then
      begin
        a^.Flags := a^.Flags and not WFSelected;
        WidgetSendSEvent(a,EV_MESSAGE,WEvNotifyUnSelect,NIL);
      end;

      a := a^.Next;

    until ( a = b );

    WidgetSelect(o^.Parent);
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetUnSelectAllSubs ( o: PWidget );
var a,b: PWidget;
begin
  if assigned( o^.Last ) then
  begin
    a := o^.Last;
    b := a;
    if assigned( a ) then
    repeat
      if ((a^.Flags and WFSelected) = WFSelected) and ((a^.Flags and WFSelectable) = WFSelectable) then
      begin
        a^.Flags := a^.Flags and not WFSelected;
        //if (  )
        WidgetSendSEvent(a,EV_MESSAGE,WEvNotifyUnSelect,NIL);
      end;
      a := a^.Next;
    until ( a = b );
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetEventHandler ( Event: PEvent );
var
  OldTopWidget: PWidget;
  Method: l_ulong;
begin
  (*
   *       Mouse event
   *)
  if ( Event^.aType = EV_MOUSE ) then
        begin
        OldTopWidget := TopWidget;

        if assigned( MasterWidget ) then
                        TopWidget := MasterWidget
                else
                        TopWidget := GetTopWidgetUnderMouse;


                if ( WorkingFlags and WWF_DRAGGING ) = WWF_DRAGGING then
                begin
                        Method := 0;


                        if ( OldTopWidget <> TopWidget ) then
                        begin
                                LastMouseMove := ATime;
                                if assigned( OldTopWidget ) then
                                begin
                                        WidgetSendSEvent(OldTopWidget,EV_MOUSE,WEvMouseLeft,NIL);
                                        OldTopWidget^.Flags := OldTopWidget^.Flags and not WF_DRAGFOCUS;
                                end;
                                TopWidget^.Flags := TopWidget^.Flags or WF_DRAGFOCUS;
                                if ( __WidgetDropAccept(TopWidget, Method) ) then
                                begin
                                        if (not DragOldAccept) or (DragOldMethod <> Method) then
                                        begin
                                                if ( Method = DMETHOD_COPY ) then
                                                        MouseSetCursor(DragCopyCur)
                                                else
                                                        MouseSetCursor(DragCur);
                                        end;
                                        DragOldAccept := true;
                                        DragOldMethod := Method;
                                end else begin
                                        if ( DragOldAccept ) then MouseSetCursor(NoDragCur);
                                        DragOldAccept := false;
                                end;
                                WidgetSendSEvent(TopWidget,EV_MOUSE,WEvMouseEnter,NIL);
                        end else if ( Mouse^.Moved ) then begin
                                LastMouseClickRight := 0;
                                LastMouseClickMiddle := 0;
                                LastMouseClickLeft := 0;
                                LastMouseMove := ATime;
                                if ( __WidgetDropAccept(TopWidget, Method) ) then
                                begin
                                        if ( (not DragOldAccept) or (DragOldMethod <> Method) ) then
                                        begin
                                                if ( Method = DMETHOD_COPY ) then
                                                        MouseSetCursor(DragCopyCur)
                                                else
                                                        MouseSetCursor(DragCur);
                                        end;
                                        DragOldAccept := true;
                                        DragOldMethod := Method;
                                end else begin
                                        if ( DragOldAccept ) then MouseSetCursor(NoDragCur);
                                        DragOldAccept := false;
                                end;
                                WidgetSendSEvent(TopWidget,EV_MOUSE,WEvMouseMove,NIL);
                        end;

                        if ((Mouse^.ButtonChanges and BUTTON_LEFT) = BUTTON_LEFT) and
                           (not(Mouse^.State.b and BUTTON_LEFT) = BUTTON_LEFT)  then
                        begin
                                if ( __WidgetDropAccept(TopWidget, Method) ) then begin
                                        TopWidget^.Flags := TopWidget^.Flags and not WF_DRAGFOCUS;
                                        if ( DragCaller^.DragBeforeDrop(DragCaller,TopWidget,DragData,DragDataType,Method) ) then
                                        begin
                                                TopWidget^.DropData(TopWidget,DragCaller,DragData,DragDataType,Method);
                                                DragCaller^.DragAfterDrop(DragCaller,TopWidget,DragData,DragDataType,Method);
                                        end else
                                                WidgetSendSEvent(DragCaller,EV_MOUSE,MSG_DRAGCANCEL,NIL);

                                end else begin
                                        TopWidget^.Flags := TopWidget^.Flags and not WF_DRAGFOCUS;
                                        WidgetSendSEvent(DragCaller,EV_MOUSE,MSG_DRAGCANCEL,NIL);
                                end;
                                DragStop();
                        end;

                end else begin

                        if ( OldTopWidget <> TopWidget ) then
                        begin
                                LastMouseMove := ATime;
                                if assigned( OldTopWidget ) then
                                        WidgetSendSEvent(OldTopWidget,EV_MOUSE,WEvMouseLeft,NIL);

                                WidgetSendSEvent(TopWidget,EV_MOUSE,WEvMouseEnter,NIL);
                        end;

                        if WidgetSendEventEx(TopWidget,Event^) then // ?removed because TRUE every time. Florian Xaver?
                        begin
                                if ( Event^.Message <> 0 ) then
                                begin
                                        WidgetSendSEvent(TopWidget,EV_MOUSE,Event^.Message,NIL);
                                end;

                                if ( Mouse^.Moved ) then
                                begin
                                        LastMouseClickRight := 0;
                                        LastMouseClickMiddle := 0;
                                        LastMouseClickLeft := 0;
                                        LastMouseMove := ATime;
                                        WidgetSendSEvent(TopWidget,EV_MOUSE,WEvMouseMove,NIL);
                                end;

                                if ( Mouse^.ButtonChanges <> 0 ) then
                                begin
                                        if (((Mouse^.ButtonChanges and BUTTON_LEFT)=BUTTON_LEFT) and
                                              ((Mouse^.State.b and BUTTON_LEFT)=BUTTON_LEFT)) or

                                             (((Mouse^.ButtonChanges and BUTTON_RIGHT)=BUTTON_RIGHT) and
                                             ((Mouse^.State.b and BUTTON_RIGHT)=BUTTON_RIGHT)) then
                                        begin
                                                WidgetSetFocus(TopWidget);
                                                WidgetSelect(TopWidget);
                                        end;

                                        if assigned( TopWidget ) then

                                        { Left mouse down [X| | ] }
                                        if ( ((Mouse^.ButtonChanges and BUTTON_LEFT)=BUTTON_LEFT) and ((Mouse^.State.b and BUTTON_LEFT)=BUTTON_LEFT) ) then
                                        begin
                                                if (ATime-LastMouseClickLeft  <= DoubleClickSpeed) then
                                                begin
                                                        if ( WidgetSendSEvent(TopWidget,EV_MOUSE, WEvMouseLDClk, NIL) ) then exit;
                                                end
                                                else LastMouseClickLeft := ATime;

                                                if ( WidgetSendSEvent(TopWidget,EV_MOUSE, WEvMouseLDown, NIL) ) then exit;
                                        end;

                                        { Middle mouse down [ |X| ] }
                                        if ( ((Mouse^.ButtonChanges and BUTTON_MIDDLE)=BUTTON_MIDDLE) and ((Mouse^.State.b and BUTTON_MIDDLE)=BUTTON_MIDDLE) ) then
                                        begin
                                                if (ATime-LastMouseClickMiddle  <= DoubleClickSpeed) then
                                                begin
                                                        if ( WidgetSendSEvent(TopWidget,EV_MOUSE, WEvMouseMDClk, NIL) ) then exit;
                                                end
                                                else LastMouseClickMiddle := ATime;

                                                if ( WidgetSendSEvent(TopWidget,EV_MOUSE, WEvMouseMDown, NIL) ) then exit;
                                        end;

                                        { Right mouse down [ | |X] }
                                        if ((Mouse^.ButtonChanges and BUTTON_RIGHT) = BUTTON_RIGHT) and
                                        ((Mouse^.State.b and BUTTON_RIGHT) = BUTTON_RIGHT ) then
                                        begin
                                                if (ATime-LastMouseClickRight  <= DoubleClickSpeed) then
                                                begin
                                                        if WidgetSendSEvent(TopWidget,EV_MOUSE, WEvMouseRDClk, NIL) then exit;
                                                end
                                                else LastMouseClickRight := ATime;

                                                if WidgetSendSEvent(TopWidget,EV_MOUSE, WEvMouseRDown, NIL) then exit;
                                        end;

                                        { Left mouse up/click [X| | ] }

                                        if ((Mouse^.ButtonChanges and BUTTON_LEFT)=BUTTON_LEFT) and
                                           not ((Mouse^.State.b and BUTTON_LEFT)=BUTTON_LEFT) then
                                                if ( WidgetSendSEvent(TopWidget,EV_MOUSE,WEvMouseLUp,NIL) ) then exit;

                                        { Middle mouse up/click [ |X| ] }
                                        if ( ((Mouse^.ButtonChanges and BUTTON_RIGHT)=BUTTON_RIGHT) and not((Mouse^.State.b and BUTTON_RIGHT)=BUTTON_RIGHT) ) then
                                                if ( WidgetSendSEvent(TopWidget,EV_MOUSE,WEvMouseRUp,NIL) ) then exit;

                                        { Right mouse up/click [ | |X] }
                                        if ( ((Mouse^.ButtonChanges and BUTTON_MIDDLE)=BUTTON_MIDDLE) and not((Mouse^.State.b and BUTTON_MIDDLE)=BUTTON_MIDDLE) ) then
                                                if ( WidgetSendSEvent(TopWidget,EV_MOUSE,WEvMouseMUp,NIL) ) then exit;

                                end;
                        end;
                end;
        end;

        (*
        *       Keyboard event
        *)
        if ( Event^.aType = EV_KEYBOARD ) then
        begin
                if not assigned(FocusedWidget) then exit;

                WidgetSendEvent(FocusedWidget, Event^);
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Widget Unitilisation                                                         *)
(*////////////////////////////////////////////////////////////////////////////////*)
procedure FreeWidget ( o: PWidget );
begin
        if assigned( o^.FreeEx ) then o^.FreeEx(o);
        if (not(( o^.Flags and WF_TOOLTIPFUNC ) = WF_TOOLTIPFUNC ) and ( o^.ToolTip.t <> '' ) ) then o^.ToolTip.t := '';
        if ( o^.Name <> '' ) then o^.Name := '';
        Dispose(o);
        o := NIL; { Florian Xaver }
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetDisposeEx ( o: PWidget );
var
  a,b,n: PWidget;
begin
        if ( TopWidget = o ) then TopWidget := NIL;
        if ( MasterWidget = o ) then MasterWidget := NIL;
        if ( FocusedWidget = o ) then FocusedWidget := NIL;
        if assigned( LookFor ) then if ( LookFor = o ) then LookFor := NIL;

        if assigned( o^.Last ) then
        begin
                a := o^.Last;
                b := a;

                repeat
                
                        n := a^.Next;
                        WidgetDisposeEx(a);
                        a := n;
                        
                until ( a = b );
        end;

        FreeWidget(o);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(**     Remove and free a widget
        @param o Widget to dispose
*)
procedure WidgetDispose ( o: PWidget );
begin
  if assigned( o ) then
  begin
    if assigned( o^.Parent ) then
            WidgetRemove(o^.Parent,o);
    WidgetDisposeEx(o);
  end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetMakeSync ( var o: PWidget );
begin
        LookFor := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure WidgetUnMakeSync ( var o: PWidget );
begin
        LookFor := NIL;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure _DisposeAllAppWidgets ( o: PWidget; App: PApplication );
var a,n: PWidget;
begin
        if ( o^.AppOwner = App ) then
        begin
                WidgetDispose(o);
                exit;
        end;
        if assigned( o^.Last ) then
        begin
                a := o^.Last; n := a;
                repeat                
                        n := a^.Next;
                        _DisposeAllAppWidgets(a,App);
                        a := n;
                until ( a = o^.Last );
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure DisposeAllAppWidgets ( App: PApplication );
begin
        if assigned( DeskTop ) then
                _DisposeAllAppWidgets(DeskTop,App);
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*//                                                                              *)
(*////////////////////////////////////////////////////////////////////////////////*)
function WidgetExecute ( o: PWidget ): l_ulong;
var
       Msg : l_ulong;
       Ev: TDynLdEvent;
begin        
  Msg  := 0;
  if ( o^.Flags and WFExecuting ) = WFExecuting then WidgetExecute := 0;

        (* ** Due to unknown threading bugs, this section is unactivated **
  if assigned( o^.AppOwner ) then
  begin
    if ( o^.AppOwner^.Type = DYNLD_TYPEAPP ) then
    begin
      fillchar(Ev, sizeof(Ev), 0);
      if o^.AppOwner^.ThreadId <> ThreadId then DebugError('PANIC :: WidegtExecute called by non-widget owner',[]);
      while AppGetNextEvent(o^.AppOwner, Ev) do
      begin
        if (Ev.Ev.aType = EV_MESSAGE) and (Ev.Dest = o)) then
        begin
                            return Ev.Ev.Message;
        end
        else
        begin
          AppRedirectEvent(o^.AppOwner, Ev);
          Yield;
        end;
      end;
      // Application has recieved exit message, but we deleted it, so we resend it
      AppSendEvent(o^.AppOwner,EV_MESSAGE,MSG_KILLAPP,NIL,NIL,NIL);
      WidgetExecute := 0;
    end;
  end;*)

  o^.Flags := o^.Flags or WFExecuting;

  DebugMessage('Executing %x',[Longint(o)]);
  while ( o^.MessageBuffer = 0 ) do
  begin
                (*PollEvents;
                Yield;*)
                SysPoll;
  end;
  o^.Flags := o^.Flags and not WFExecuting;

  DebugMessage('Return Message %x',[o^.MessageBuffer]);
  Msg := o^.MessageBuffer;
  o^.MessageBuffer := 0;
  WidgetExecute := Msg;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Widget Initialisation                                                        *)
(*////////////////////////////////////////////////////////////////////////////////*)
function IntialiseWidget ( App: PApplication; aClass: PWidgetClass; o: PWidget; r: TRect ): PWidget;
begin
        if not assigned( o ) then IntialiseWidget := NIL else
        begin

        if not assigned( aClass ) then aClass := StdWidgetClass;

        o^.WClass                       := aClass;
        o^.Relative                     := r;
        o^.Absolute                     := r;
        RectAssign(o^.ChildArea,0,0,r.b.x-r.a.x,r.b.y-r.a.y);
        o^.ChildDrawAbs         := r;
        o^.DrawAbsolute         := r;
        o^.Flags                        := WFVisible;
        o^.AppOwner                     := App;

        o^.Refresh                      := {$ifdef fpc}@{$endif}WidgetRefresh;

        o^.Remove := @_WidgetRemove;
        o^.Insert := @_WidgetInsert;
        o^.SetFirstEx := @_WidgetSetFirstEx;
        IntialiseWidget := o;
        end;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(** Create a new standart widget
        @param App Application which want to allocate the widget( use &Me in most cases)
        @param r Relative coordinates of new widget
*)
function CreateWidget ( App: PApplication; r: TRect ): PWidget;
var o: PWidget;
begin
        New(o);
        fillchar(o,sizeof(TWidget),0);
        o := IntialiseWidget(App, StdWidgetClass, o, r);
        CreateWidget := o;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure SwitchTextMode;
begin
        if assigned( buffer ) then begin al_destroy_bitmap(buffer); buffer := NIL; end;
        __MouseStop;
        __KeyboardStop;
        __ScreenTextMode;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
procedure SwitchGraphicsMode;
begin
        __ScreenRestartGraphics;
        __KeyboardRestart;
        __MouseRestart;
        buffer := al_create_system_bitmap(GSScreenWidth, GSScreenHeight);
        WidgetDrawAll(DeskTop);
        MouseShow();
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Tool tips system                                                             *)
(*////////////////////////////////////////////////////////////////////////////////*)
(**
*       Set a widget tooltip text
*       @param o Widget to affect
*       @param t Text to set as tooltip text (text is duplicated, so you can free @a t after)
*
*)
procedure WidgetSetTooltipText ( o: PWidget; t: l_text );
begin
        if (not(( o^.Flags and WF_TOOLTIPFUNC ) = WF_TOOLTIPFUNC) and ( o^.ToolTip.t <> '') ) then o^.ToolTip.t := '';
        o^.Flags := o^.Flags and not WF_TOOLTIPFUNC;
        o^.ToolTip.t := t;
end;
(*////////////////////////////////////////////////////////////////////////////////*)
(*// Tooltip widget draw fucntion *)
procedure ToolTipDraw ( o: PToolTip{PWidget}; buffer: p_bitmap; w: PRect );
var tt: PToolTip; t,lt: l_text; tmp: integer; y,h: l_ulong;
begin
        
        al_rectfill(buffer, w^.a.x, w^.a.y, w^.b.x, w^.b.y, al_makecol(255,255,255));
        al_rect(buffer,o^.o.Absolute.a.x,o^.o.Absolute.a.y,o^.o.Absolute.b.x,o^.o.Absolute.b.y,al_makecol(0,0,0));

        if ( o^.Text <> '' ) then
        begin
                t  := o^.Text;
                lt := o^.Text;
                tmp := 0;
                y := o^.o.Absolute.a.y+2;
                h := al_text_height(default_font)+1;



                t  := o^.Text; tmp := 1;

                while ( tmp <= Length(t) ) do
                begin
                        lt := t;

                        {t := TextChr(t, #10);}

                        if system.Pos(#10,t) > 0 then begin
                                SetLength(t, Length(t)-1);
                                DrawTextEx(buffer,default_font,o^.o.Absolute.a.x+2, y,lt,t,al_makecol(0,0,0),false);
                                Inc(tmp);
                                y := y + h;
                        end else
                                al_textout_ex(buffer,default_font, lt, o^.o.Absolute.a.x+2, y, al_makecol(0,0,0),-1);
                end;
        end;

end;
(*
////////////////////////////////////////////////////////////////////////////////
// General widget timer. Used for
//   - Mouse enter delay event
//   - Tooltips
////////////////////////////////////////////////////////////////////////////////
*)
var WidgetTimerId: PTimer;

function MouseDelayEnough(Start,finish: TDateTime): l_bool;
var
       _start: Double;
       _finish: Double;
       _time: Double;
begin
  _start := Frac(Start);
  _finish := Frac(finish);
  _time := _start - _finish;
  MouseDelayEnough := _time >= 2000.0;
end;

function DifferenceBigEnoughBetween(Start,Finish: TDateTime): l_bool;
var
       _start: Double;
       _finish: Double;
begin
  _start := Frac(Start);
  _finish := Frac(Finish);
  DifferenceBigEnoughBetween := _start <> _finish;
end;

function NoToolTip: l_bool;
var Flag: l_bool;
begin
  Flag := (WorkingFlags and WWF_TOOLTIPUP) = WWF_TOOLTIPUP;
  NoToolTip := not Flag;
end;

procedure WidgetTimer ( var Arg: {array of }Pointer );
var
       r: TRect;
       p: TPoint;
       o: PToolTip;
       w,h: l_ulong;
       txt: l_text;
       Me: PApplication;
       _MouseTime: Double;
       _time,_last_move: Double;
       Mousetime: Longint;
begin
        {// Mouse has not moved since 0.2 secconds, so notify widget for mouse enter delay}
        if ( (LastNotified <> LastMouseMove) and (ATime - LastMouseMove >= 200) ) then 
        begin
          LastNotified := LastMouseMove;
          if Assigned( TopWidget ) then WidgetSendSEvent(TopWidget,EV_MOUSE,WEvMouseEnterDelay,NIL);
        end;
        // Mouse has not moved since 2 secons, so show widget tooltip (if it exists)
        _time := Frac(ATime);
        _last_move := Frac(LastMouseMove);
        _MouseTime := _time - _last_move;
        MouseTime  := Round(_MouseTime);
        if DifferenceBigEnoughBetween(LastToolTip, LastMouseMove) and
           MouseDelayEnough(ATime,LastMouseMove) and
           NoToolTip then
        begin
                LastToolTip := LastMouseMove;

                WorkingFlags := WorkingFlags or WWF_TOOLTIPUP;

                if Assigned( TopWidget ) then
                begin
                        if ( TopWidget^.ToolTip.t <> '') then
                        begin
                          p := Mouse^.State.p;
                
                          if ( TopWidget^.Flags and WF_TOOLTIPFUNC ) = WF_TOOLTIPFUNC then
                                txt := TopWidget^.ToolTip.func(TopWidget,p)
                          else
                                txt := TextDup(TopWidget^.ToolTip.t);

                        TextSize(default_font,txt,w,h);


                        RectAssign(r,p.x+Mouse^.Cur^.Image^.w,p.y,p.x+Mouse^.Cur^.Image^.w+w+4,p.y+h+4);

                        {o := malloc(sizeof(TToolTip));}
                        New(o);
                        fillchar(o^,sizeof(TToolTip),0);
                        IntialiseWidget(Me,NIL,@o^.o,r);
                        o^.Text := txt;
                        //o^.o.Draw := @ToolTipDraw;

                        WidgetInsert(DeskTop,@o^.o);
                        WidgetDrawAll(@o^.o);
                        while (Mouse^.State.p.x = p.x) and
                              (Mouse^.State.p.y = p.y) and
                               not l_bool(Mouse^.State.b)  do begin
                                SysPoll();
                        end;
                        WidgetDispose(@o^.o);
                        txt := '';

                        end;
                end;

                WorkingFlags := WorkingFlags and (not WWF_TOOLTIPUP);

        end;
end;
(*
////////////////////////////////////////////////////////////////////////////////
// Helper functions to maintain startup logo
////////////////////////////////////////////////////////////////////////////////
*)
var StartMask: PWidget;
procedure StartMaskDraw ( o: PWidget; buffer: p_bitmap; w: PRect );
begin
  al_blit(StartLogo, buffer, w^.a.x-StartRect.a.x, w^.a.y-StartRect.a.y, w^.a.x, w^.a.y, w^.b.x-w^.a.x+1, w^.b.y-w^.a.y+1);
end;
(*
////////////////////////////////////////////////////////////////////////////////
*)
procedure DisposeStartupLogo();
begin
        WidgetDispose(StartMask);
        al_destroy_bitmap(StartLogo);
        StartLogo := NIL;
end;
(*
////////////////////////////////////////////////////////////////////////////////
// Library initialisation function
*)
const WidgetExtender: PAppExtender = NIL;

function LibMain ( argc: integer; argv: array of l_text ): l_bool;

var r: TRect;

begin

  buffer := al_create_system_bitmap(GSScreenWidth, GSScreenHeight);
  if not Assigned( buffer ) then
  begin
          DebugError('Widget :: Could not init double buffer, disabling widgets',[]);
          LibMain := false;
  end
  else
  begin
        al_clear_bitmap(buffer);


        WClassInstall;

        iWidgetInstall;

        OSDColor := GetColorFromHex(KeyGetText('/USER/COLOR/OSD', '888888'));
        RectAssign(r,0,0,GSScreenWidth-1,GSScreenHeight-1);
        DeskTop := CreateWidget(@Me, r);
        StartMask := CreateWidget(@Me, StartRect);
        StartMask^.Draw := {$ifdef fpc}@{$endif}StartMaskDraw;
        WidgetInsert(DeskTop,StartMask);
        WidgetDraw(DeskTop,NIL);
        OSD := false;
        //al_set_mouse_range(DeskTop^.Absolute.a.x,DeskTop^.Absolute.a.y,DeskTop^.Absolute.b.x,DeskTop^.Absolute.b.y);
        InstallEventHandler({$ifdef fpc}@{$endif}WidgetEventHandler);

        LastMouseMove := ATime;
        LastNotified  := ATime;

        //WidgetTimerId := NewTimer(&Me, 50, {$ifdef fpc}@{$endif}WidgetTimer, NIL);
        DragCur       := NewCursor(LoadImage('SYSTEM/CURSORS/drag.bmp'),0,0);
        NoDragCur     := NewCursor(LoadImage('SYSTEM/CURSORS/nodrag.bmp'),0,0);
        DragCopyCur   := NewCursor(LoadImage('SYSTEM/CURSORS/dragcopy.bmp'),0,0);

        (*// Library exports*)
        
        InitSpliters();
        (*
        APPEXPORT(_WidgetInsert);
        APPEXPORT(_WidgetRemove);
        APPEXPORT(_WidgetSetFirstEx);

        APPEXPORT(WidgetInsert);
        APPEXPORT(WidgetRemove);
        APPEXPORT(CreatePropType);
        APPEXPORT(DeskTop);
        APPEXPORTAS(DeskTop, "Desktop");
        APPEXPORT(IntialiseWidget);
        APPEXPORT(CreateWidget);
        APPEXPORT(WidgetDrawOnRect);
        APPEXPORT(WidgetMove);
        APPEXPORT(WidgetSetOSDRect);
        APPEXPORT(WidgetUnOSD);
        APPEXPORT(MasterWidget);
        APPEXPORT(WidgetResize);
        APPEXPORT(WidgetResizeToSize);
        APPEXPORT(WidgetMoveToPoint);
        APPEXPORT(WidgetSetMetrics);
        APPEXPORT(WidgetSendEvent);
        APPEXPORT(WidgetDispose);
        APPEXPORT(WidgetDrawAll);
        APPEXPORT(WidgetExecute);
        APPEXPORT(WidgetSetFirst);
        APPEXPORT(WidgetSetFocusEx);
        APPEXPORT(WidgetSetFocus);
        APPEXPORT(FocusedWidget);
        APPEXPORT(WidgetPreCenter);
	  APPEXPORT(WidgetPostCenter);
        APPEXPORT(WidgetUnMakeSync);
        APPEXPORT(WidgetMakeSync);
        APPEXPORT(WidgetSetChildArea);
        APPEXPORT(WidgetSendPEvent);
        APPEXPORT(DisposeStartupLogo);
        APPEXPORT(WidgetSetRect);
        APPEXPORT(WidgetSetFocusExNoNotify);
        APPEXPORT(WidgetDraw);
        APPEXPORT(WidgetSendSEvent);
        APPEXPORT(WidgetSendEEvent);
        APPEXPORT(SwitchGraphicsMode);
        APPEXPORT(SwitchTextMode);
        APPEXPORT(WidgetSetTooltipText);
        APPEXPORT(DragStart);
        APPEXPORT(WidgetGetIndex);

        APPEXPORT(StdWidgetClass);
        APPEXPORT(CreateWidgetClass);
        APPEXPORT(WidgetIsClassId);
        APPEXPORT(GetWidgetClass);
        APPEXPORT(WClassRegisterProp);
        APPEXPORT(GetPropType);

        APPEXPORT(NewiWidget);
        APPEXPORT(InsertiWidgetBefore);
        APPEXPORT(ComplieiWidget);
        APPEXPORT(FreeiWidget);

        APPEXPORT(DisposeAllAppWidgets);

        APPEXPORT(DuplicateBool);
        APPEXPORT(DuplicateInt);
        APPEXPORT(WidgetCheckVisible);
        APPEXPORT(WidgetClasses);
        APPEXPORT(WidgetGetName);

        APPEXPORT(CreateVSpliter);
        APPEXPORT(CreateHSpliter);
        *)
        WidgetExtender :=  InstallAppExtender(NIL,@DisposeAllAppWidgets);

        libMain := true;
  end;
end;
(*
////////////////////////////////////////////////////////////////////////////////
// Library unload function
*)
procedure Close;
begin
        RemoveAppExtender(WidgetExtender);

        KillTimer(WidgetTimerId);

        (*
          Dont handle events when the system is unloaded
         *)
        RemoveEventHandler({$ifdef fpc}@{$endif}WidgetEventHandler);

        if Assigned( buffer ) then al_destroy_bitmap(buffer);

        WidgetDisposeEx(DeskTop);
        DeskTop := NIL;

        iWidgetUnInstall();
        WClassUnInstall();
end;
(*////////////////////////////////////////////////////////////////////////////////*)
{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
   AppVersion := ULONG_ID(0,2,0,0);
   MasterWidget := NIL;
   FocusedWidget := NIL;
   fillchar(OSDRect, sizeof(TRect), 0);
   DoubleClickSpeed := 100*Mouse^.DblClkSpeed;

{$endif} {LINK_CLIBS}

end.
