(*
/////////////////////////////////////////////////////////////////////////////////
//
//	Window Library 
//
//	Aura Milestone 4 
//
//	(c) Copyright 2013 Finn Technologies and Chase Finn. All rights reserved.
//
/////////////////////////////////////////////////////////////////////////////////
*)
unit window;

interface

uses
  guitypes,dynld,dms,events,mousedrv,keyboard,widget,ulabel,metrics,progress,
  sound,skin,font,grfx,registry,image,screen,allegro,menu,list,text;
  
{$i cdefs.inc}

{$ifdef LINK_CLIBS}
   {$link widget.o}
   {$link window.o}
   {$link button.o}
   {$link label.o}
   {$link sound.o}
   {$link progress.o}
{$endif}  
{$i alkeyid.inc}
{
 .#include 'kernel.h'
 .#include 'widget.h'
 .#include 'window.h'
 .#include 'button.h'
 .#include 'label.h'
 .#include 'internal.h'

 .#include 'sound.h'


 .#include 'widget.h'
 .#include 'label.h'
 .#include 'progress.h'
}




(**
*	Window structure
*)
type
  PWindow = ^TWindow;
  TWindow = record

	o:		TWidget;

	Caption: l_text;
	WindowFlags : l_ulong;
	WindowStates: l_ulong;
	Bounds      : TRect;
	Background  : p_bitmap; ///< Window custom background
	TitleActive : p_bitmap; ///< Window custom TitleActive
	TitlePassive: p_bitmap; ///< Window custom TitlePassive

	Icon: PIcon;
	Cursor: PCursor;
	MaxSize: TSize;
	MinSize: TSize;

  end;
  PPWINDOW = PWindow;

{ #define PWINDOW(w)	((PWindow)(w)) }

(**
*	Window flags
*)
const
  WF_FRAME        = $00000001;
  WF_CAPTION      = $00000002;
  WF_MAXIMIZE     = $00000004;
  WF_MINIMIZE     = $00000008;
  WF_RESIZE       = $00000010;
  WF_MAINWINDOW   = $00000020;
  WF_UNMOVEABLE   = $00000040;
  WF_UNSELECTABLE = $00000080;
  WF_DOUBLEBUFFER = $00000100;
  WF_CENTERED     = $00000200;
  WF_NORMAL       = WF_FRAME or WF_CAPTION or WF_MAXIMIZE or WF_MINIMIZE;

(**
*	Window states
*)
  WS_MAXIMIZED    = $00000001;
  WS_MINIMIZED    = $00000002;
  WS_MODAL        = $00000008;


(**
*	Window messages
*)
  WM_CLOSE		  = $FFFF4101;
  WM_ABOUT		  = $FFFF4102;
  WM_NEWWINDOW	  = $FFFF4104;
  WM_CLOSEWINDOW  = $FFFF4108;

procedure WindowSetCaptionEx ( o: PWindow; Caption: l_text );
{$ifdef LINK_CLIBS}external;{$endif}

procedure WindowSetCaption ( o: PWindow; Caption: l_text; args: {$ifdef OPascal}array of const{$else}OpenArray{$endif} );
{$ifdef LINK_CLIBS}external;{$endif}

procedure CustomWindowDraw ( App: PApplication; o: PWidget;  buffer: p_bitmap; w: PRect );
{$ifdef LINK_CLIBS}external;{$endif}

function CreatePWINDOW(App: PApplication; r: TRect; szCaption: l_text;  flags: l_ulong): PWindow;
{$ifdef LINK_CLIBS}external;{$endif}

function CreateWindow(App: PApplication; r: TRect; szCaption: l_text;  flags: l_ulong): PWindow;

function MessageBox ( App: PApplication; Title, Message: l_text; Flags: l_ulong ): l_ulong;
{$ifdef LINK_CLIBS}external;{$endif}

function WindowInit(w: PWindow; App: PApplication; r: TRect; szCaption: l_text; flags: l_ulong): PWindow;
{$ifdef LINK_CLIBS}external;{$endif}

function Alert ( App: PApplication; Message: l_text;  flags:l_ulong ): l_ulong;
{$ifdef LINK_CLIBS}external;{$endif}

(**
*	Message box flags
*)
const
  MBB_OK          = $000000100;
  MBB_OKCANCEL    = $000000200;
  MBB_YESNO       = $000000400;
  MBB_YESNOCANCEL = $000000800;
  MBB_YESYESALLNOCANCEL = $000001000;

  MBI_WARNING	  = $000000001;
  MBI_QUESTION	  = $000000002;
  MBI_ERROR		  = $000000004;
  MBI_INFORMATION = $000000008;

var WINDOW_MaximizeArea: TRect;


type 
  PProgressPop = ^TProgressPop;
  TProgressPop = record

	W: PWindow;
	B: PProgressBar;
	L: PLabel;
	T: PLabel;
	Started: TDateTime;
	
  end;

{ #define PROGRESSPOP(o) ((PProgressPop)(o)) }

function ProgressPopUp ( App: PApplication; Flags: l_ulong; Title, Message: l_text; Steps: l_ulong; InitInfo: l_text ): PProgressPop;
{$ifdef LINK_CLIBS}external;{$endif}

procedure ProgressPopSetInfoEx ( p: PProgressPop; AText: l_text );
{$ifdef LINK_CLIBS}external;{$endif}

procedure ProgressPopSetInfo ( p: PProgressPop; StrFormat: l_text; args: {$ifdef OPascal}array of const{$else}OpenArray{$endif} );
{$ifdef LINK_CLIBS}external;{$endif}

procedure ProgressPopStep ( p: PProgressPop; Steps: l_ulong );
{$ifdef LINK_CLIBS}external;{$endif}

procedure ProgressPopKick ( p: PProgressPop );
{$ifdef LINK_CLIBS}external;{$endif}

procedure WindowNotifyError ( App: PApplication; h: PErrorHandler );
{$ifdef LINK_CLIBS}external;{$endif}

procedure MinimizePWINDOW( o: PWidget );
{$ifdef LINK_CLIBS}external;{$endif}

implementation
{$ifndef LINK_CLIBS}
uses button{$ifdef OPascal},Sysutils{$else},OSysutil{$endif};

var
  AppVersion : l_ulong;
  WindowClass: PWidgetClass;
  
const
  AppName: l_text = 'Window Widget';
  nUID = 'window';
  NeededLibs: array[0..7] of String = ( 'widget', 'skin','button','label','progress','mp3','sound','' );

  DefaultIcon: PIcon = NIL; 
 

const
  aw_trans: Integer = 0;
  pw_trans: Integer = 0;


(**
 * Global exported variable defining the
 * maximize area for window
 *)
var  
  WINDOW_MinimizeArea: TRect;

const
  stream: PSStream = NIL;

(**
*	Message box icons
*)
  IconWarning	    : p_bitmap = NIL;
  IconQuestion	    : p_bitmap = NIL;
  IconError		    : p_bitmap = NIL;
  IconInformation   : p_bitmap = NIL;

(**
*	Skin items
*)
  SkinTitleActive	: PSkin = NIL;
  SkinTitlePassive	: PSkin = NIL;
  SkinCloseUp		: PSkin = NIL;
  SkinCloseDown		: PSkin = NIL;
  SkinWindowFace	: PSkin = NIL;
  SkinMaximizeUp	: PSkin = NIL;
  SkinMaximizeDown	: PSkin = NIL;
  SkinMinimizeUp	: PSkin = NIL;
  SkinMinimizeDown	: PSkin = NIL;


  CResTitleActive   : p_bitmap = NIL;
  CResTitlePassive	: p_bitmap = NIL;
  CResWindowFace	: p_bitmap = NIL;

  CSkinTitleActive	: PSkin = NIL;
  CSkinTitlePassive : PSkin	= NIL;
  CSkinWindowFace   : PSkin	= NIL;


(**
*	Window actions
*)
  WDNOTHING		= $00000000;
  WDRESIZING	= $00000001;
  WDMOVING		= $00000002;

(**
*	Window button states
*)
  WBNOTHING		= $FF000000;
  WBCLOSE		= $FF000001;
  WBMAXIMIZE	= $FF000002;
  WBMINIMIZE	= $FF000003;

(**
*	Fixed widget buttons lenghts
*)
  FIXEDWIDGETSPACE		= 3;
  FIXEDWIDGETSEPERATOR	= 5;
const
  WCLOSE = 1;
  WMAXIMIZE = 2;
  WMINIMIZE = 3;
  WRESIZE = 4;
  WICON = 5;
  WMOVE = 6;

function Alert ( App: PApplication; Message: l_text;  flags:l_ulong ): l_ulong;
begin

end;

procedure AssignZone ( var r: TRect; o: PWidget; id: l_ulong );
begin
	if not( UseSkins ) then
	begin
	  case id of
		WCLOSE:
			RectAssign(r,o^.Absolute.b.x-16, o^.Absolute.a.y+4, o^.Absolute.b.x-4, o^.Absolute.a.y + 16 );
		
		WMAXIMIZE:
			RectAssign(r,o^.Absolute.b.x-33, o^.Absolute.a.y+4, o^.Absolute.b.x-21, o^.Absolute.a.y + 16 );
		
		WMINIMIZE:
			if (PWINDOW(o)^.WindowFlags and WF_MAXIMIZE) = WF_MAXIMIZE then
				RectAssign(r,o^.Absolute.b.x-50, o^.Absolute.a.y+4, o^.Absolute.b.x-38, o^.Absolute.a.y + 16 )
			else
				RectAssign(r,o^.Absolute.b.x-33, o^.Absolute.a.y+4, o^.Absolute.b.x-21, o^.Absolute.a.y + 16 );
		
		WRESIZE:
			RectAssign(r,o^.Absolute.b.x-15, o^.Absolute.b.y-15, o^.Absolute.b.x, o^.Absolute.b.y);
		
		WICON:
			RectAssign(r,o^.Absolute.a.x+5, o^.Absolute.a.y+2, o^.Absolute.a.x + 21, o^.Absolute.a.y + 18);
		
		WMOVE:
			RectAssign(r,o^.Absolute.a.x+1, o^.Absolute.a.y+1, o^.Absolute.b.x-1, o^.Absolute.a.y + 19);
			
	  end;	
	end
	else
	begin
	  case id of
		WCLOSE:
			RectAssign(r,o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinCloseUp^.Skin^.h) div 2, o^.Absolute.b.x - (SkinTitleActive^.Right + 3) - SkinCloseUp^.Skin^.w + SkinCloseUp^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinCloseUp^.Skin^.h) div 2 + SkinCloseUp^.Skin^.h);
		
		WMAXIMIZE:
			RectAssign(r,o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMaximizeUp^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMaximizeUp^.Skin^.h) div 2, o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMaximizeUp^.Skin^.h) div 2 + SkinMaximizeUp^.Skin^.h);
		
		WMINIMIZE:
			if (PWINDOW(o)^.WindowFlags and WF_MAXIMIZE) = WF_MAXIMIZE then
				RectAssign(r,o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMaximizeUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMinimizeUp^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMinimizeUp^.Skin^.h) div 2, o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMaximizeUp^.Skin^.w - FIXEDWIDGETSEPERATOR, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMinimizeUp^.Skin^.h) div 2 + SkinMinimizeUp^.Skin^.h)
			else
				RectAssign(r,o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMinimizeUp^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMinimizeUp^.Skin^.h) div 2, o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMinimizeUp^.Skin^.h) div 2 + SkinMinimizeUp^.Skin^.h);
		
		WRESIZE:
			RectAssign(r,o^.Absolute.b.x-15, o^.Absolute.b.y-15, o^.Absolute.b.x, o^.Absolute.b.y);
		
		WICON:
			RectAssign(r,o^.Absolute.a.x + ((SkinTitleActive^.Skin^.h - 16) div 2), o^.Absolute.a.y + ((SkinTitleActive^.Skin^.h - 16) div 2), o^.Absolute.a.x + 16, o^.Absolute.a.y + 16);
		
		WMOVE:
			RectAssign(r,o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.a.y + SkinTitleActive^.Skin^.h);
	  end;
	end;
end;




(**
*	Global variables
*)
var p: TPoint;

const
  WDoing: l_ulong = 0; // Ev^.Type is WDoing master window
  WButton: l_ulong = 0;
  WSelected: l_ulong = 0;

function selected_colors(flag: l_bool; col_selected,col_unselected: longint): integer;
begin
  if flag then selected_colors := col_selected else selected_colors := col_unselected;
end;

function selected_font(aflag: l_bool; fntsel,fntusel: PFont): PFont;
begin
  if aflag then selected_font := fntsel else selected_font := fntusel;
end;

procedure CustomWindowDraw ( App: PApplication; o: PWidget;  buffer: p_bitmap; w: PRect );
begin

end;

procedure WindowDraw ( o: PWidget; buffer: p_bitmap; w: PRect );
var
  r: TPoint; aflags: l_bool; XFlag: integer;
  a,b,c,d,e,f:	TRect;
begin
	AssignZone(a,o,WCLOSE);
	if ( (PWINDOW(o)^.WindowFlags and WF_MAXIMIZE) = WF_MAXIMIZE ) then AssignZone(b,o,WMAXIMIZE);
	if ( (PWINDOW(o)^.WindowFlags and WF_MINIMIZE) = WF_MINIMIZE ) then AssignZone(c,o,WMINIMIZE);
	AssignZone(d,o,WRESIZE);
	AssignZone(e,o,WICON);
	AssignZone(f,o,WMOVE);

	if not( UseSkins ) then
	begin

		al_rectfill(buffer,o^.Absolute.a.x,f.b.y,o^.Absolute.b.x,o^.Absolute.b.y,COL_3DFACE);

        aflags := (o^.Flags and WFSelected) = WFSelected;
        if aflags then XFlag := COL_WTACT else XFlag := COL_WTPAS;
		al_rectfill(buffer,f.a.x,f.a.y,f.b.x,f.b.y,selected_colors(aflags,COL_WTACT,COL_WTPAS));

        aflags := ((o^.Flags and WFSelected) = WFSelected);
        if aflags then XFlag := COL_WTACTTEXT else XFlag := COL_WTPASTEXT;
		al_textout_centre_ex(
                  buffer,
                  default_font,
                  PWindow(o)^.Caption,
                  (f.a.x+f.b.x) div 2,
                  f.a.y+2,
                  selected_colors(aflags,COL_WTACTTEXT,COL_WTPASTEXT),-1);

		Rect3D(buffer,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.b.y,COL_3DLIGHT,COL_3DDARK);

		if (PWINDOW(o)^.WindowFlags and WF_MINIMIZE) = WF_MINIMIZE then
		begin
			al_rectfill(buffer,c.a.x+1,c.a.y+1,c.b.x-1,c.b.y-1,COL_3DFACE);
			if (WButton <> WBMINIMIZE) then
			begin
				Rect3D(buffer,c.a.x,c.a.y,c.b.x,c.b.y,COL_3DLIGHT,COL_3DDARK);
				al_hline(buffer,c.a.x+2,c.b.y-4,c.b.x-3,COL_3DTEXT);
				al_hline(buffer,c.a.x+2,c.b.y-3,c.b.x-3,COL_3DTEXT);
			end else begin
				Rect3D(buffer,c.a.x,c.a.y,c.b.x,c.b.y,COL_3DDARK,COL_3DLIGHT);
				al_hline(buffer,c.a.x+3,c.b.y-3,c.b.x-2,COL_3DTEXT);
				al_hline(buffer,c.a.x+3,c.b.y-2,c.b.x-2,COL_3DTEXT);
			end;
		end;



		if (PWINDOW(o)^.WindowFlags and WF_MAXIMIZE) = WF_MAXIMIZE then
        begin			
			al_rectfill(buffer,b.a.x+1,b.a.y+1,b.b.x-1,b.b.y-1,COL_3DFACE);
			
			if (WButton = WBMAXIMIZE) then
			begin
				Rect3D(buffer,b.a.x,b.a.y,b.b.x,b.b.y,COL_3DDARK,COL_3DLIGHT);
				al_rect(buffer,b.a.x+4,b.a.y+4,b.b.x-2,b.b.y-2,COL_3DTEXT);
				al_hline(buffer,b.a.x+5,b.a.y+5,b.b.x-3,COL_3DTEXT);
			end else begin
				Rect3D(buffer,b.a.x,b.a.y,b.b.x,b.b.y,COL_3DLIGHT,COL_3DDARK);
				al_rect(buffer,b.a.x+3,b.a.y+3,b.b.x-3,b.b.y-3,COL_3DTEXT);
				al_hline(buffer,b.a.x+4,b.a.y+4,b.b.x-4,COL_3DTEXT);
			end;

		end;
		
		al_rectfill(buffer,a.a.x+1,a.a.y+1,a.b.x-1,a.b.y-1,COL_3DFACE);

		if (WButton = WBCLOSE) then
		begin
			Rect3D(buffer,a.a.x,a.a.y,a.b.x,a.b.y,COL_3DDARK,COL_3DLIGHT);
			al_line(buffer,a.a.x+4,a.a.y+4,a.b.x-2,a.b.y-2,COL_3DTEXT);
			al_line(buffer,a.b.x-2,a.a.y+4,a.a.x+4,a.b.y-2,COL_3DTEXT);
		end else begin
			Rect3D(buffer,a.a.x,a.a.y,a.b.x,a.b.y,COL_3DLIGHT,COL_3DDARK);
			al_line(buffer,a.a.x+3,a.a.y+3,a.b.x-3,a.b.y-3,COL_3DTEXT);
			al_line(buffer,a.b.x-3,a.a.y+3,a.a.x+3,a.b.y-3,COL_3DTEXT);
		end;

		DrawIcon16(buffer,PWINDOW(o)^.Icon,e.a.x,e.a.y);

	end else begin
	(**
	*	Caption
	*)
	if (((PWINDOW(o))^.WindowFlags and WF_CAPTION) = WF_CAPTION) then
	begin

	aw_trans := KeyGetInt('/USER/DESKTOP/transparency/active_window', 255);
	pw_trans := KeyGetInt('/USER/DESKTOP/transparency/passive_window', 255);

		if ( o^.Flags and WFSelected ) = WFSelected then
		begin
				if  assigned(PWindow(o)^.TitleActive) then
					DrawTransSkinCustom (buffer, PWindow(o)^.TitleActive,  o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.a.y+SkinTitleActive^.Skin^.h, aw_trans, 255)
				else	
					DrawTransSkin(buffer, SkinTitleActive, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.a.y+SkinTitleActive^.Skin^.h, aw_trans, 255);
		end
		else
				if  assigned(PWINDOW(o)^.TitlePassive) then
					DrawTransSkinCustom( buffer, PWindow(o)^.TitlePassive,  o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.a.y+SkinTitlePassive^.Skin^.h, pw_trans, 255)
				else	
					DrawTransSkin(buffer, SkinTitlePassive, o^.Absolute.a.x, o^.Absolute.a.y, o^.Absolute.b.x, o^.Absolute.a.y+SkinTitleActive^.Skin^.h, pw_trans, 255);
			
		if (WButton <> WBCLOSE) then
			al_draw_sprite( buffer,SkinCloseUp^.Skin,o^.Absolute.b.x - (SkinTitleActive^.Right + 3) - SkinCloseUp^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinCloseUp^.Skin^.h) div 2)
		else
			al_draw_sprite( buffer,SkinCloseDown^.Skin, o^.Absolute.b.x - (SkinTitleActive^.Right + 3) - SkinCloseDown^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinCloseDown^.Skin^.h) div 2);

		if (PWINDOW(o)^.WindowFlags and WF_MAXIMIZE) = WF_MAXIMIZE then
		begin
			if (WButton <> WBMAXIMIZE) then
				al_draw_sprite(buffer,SkinMaximizeUp^.Skin, o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMaximizeUp^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMaximizeUp^.Skin^.h) div 2)
			else
				al_draw_sprite(buffer,SkinMaximizeDown^.Skin, o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMaximizeUp^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMaximizeUp^.Skin^.h) div 2);
		end;

		if ((PWINDOW(o)^.WindowFlags and WF_MINIMIZE)=WF_MINIMIZE) then
		begin

			if (WButton <> WBMINIMIZE) then
			begin
				if (PWINDOW(o)^.WindowFlags and WF_MAXIMIZE) = WF_MAXIMIZE then
					PointAssign(r,o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMaximizeUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMinimizeUp^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMinimizeUp^.Skin^.h) div 2)
				else
					PointAssign(r,o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMinimizeUp^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMinimizeUp^.Skin^.h) div 2);

				al_draw_sprite(buffer, SkinMinimizeUp^.Skin, r.x, r.y);
			end
			else
			begin
				if (PWINDOW(o)^.WindowFlags and WF_MAXIMIZE) = WF_MAXIMIZE then
					PointAssign(r,o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMaximizeUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMinimizeDown^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMinimizeDown^.Skin^.h) div 2)
				else
					PointAssign(r,o^.Absolute.b.x - (SkinTitleActive^.Right + FIXEDWIDGETSPACE) - SkinCloseUp^.Skin^.w - FIXEDWIDGETSEPERATOR - SkinMinimizeDown^.Skin^.w, o^.Absolute.a.y + (SkinTitleActive^.Skin^.h - SkinMinimizeDown^.Skin^.h) div 2);

				al_draw_sprite(buffer,SkinMinimizeDown^.Skin, r.x, r.y);
			end;
		end;

		if ( (PWINDOW(o))^.Caption <> '' ) then
		begin
                  aflags := (o^.Flags and WFSelected) = WFSelected;
                  {selected_font(o^.Flags and WFSelected) = WFSelected, SkinTitleActive^.Font, SkinTitlePassive^.Font)}
                  al_textout_centre_ex(buffer,
                     selected_font(aflags, SkinTitleActive^.Font, SkinTitlePassive^.Font),
                    PWindow(o)^.Caption, o^.Absolute.a.x+((o^.Absolute.b.x-o^.Absolute.a.x) div 2),
                    o^.Absolute.a.y+SkinTitleActive^.Skin^.h div 2-al_text_height(SkinTitleActive^.Font) div 2,
                    selected_colors((o^.Flags and WFSelected)=WFSelected, SkinTitleActive^.FontColor, SkinTitlePassive^.FontColor), -1
                  );
                end;
		(**
		*	Window icon
		*)
		DrawIcon16(buffer,PWINDOW(o)^.Icon,e.a.x+5, e.a.y );
	end;

	(**
	*	Body
	*)
	al_rectfill(buffer,
          o^.Absolute.a.x,
          o^.Absolute.a.y + SkinTitleActive^.Skin^.h,
          o^.Absolute.b.x,
          o^.Absolute.b.y ,
          COL_3DFACE
        );
	if  assigned(PWindow(o)^.Background) then
	  DrawSkinCustom(buffer,
            PWindow(o)^.Background,
            o^.Absolute.a.x, o^.Absolute.a.y + SkinTitleActive^.Skin^.h,
            o^.Absolute.b.x, o^.Absolute.b.y
          )
	else
	  DrawSkin(buffer,
            SkinWindowFace,
            o^.Absolute.a.x,
            o^.Absolute.a.y + SkinTitleActive^.Skin^.h,
            o^.Absolute.b.x,
            o^.Absolute.b.y
          );

	(**
	*	Resize
	*)
	(*if (((PWindow)o)^.WindowFlags & WF_RESIZE )
		draw_sprite(buffer,SkinResize^.Skin, o^.Absolute.b.x - SkinWindowFace^.Right - SkinResize^.Skin^.w, o^.Absolute.b.y - SkinWindowFace^.Bottom - SkinResize^.Skin^.h);*)


	end;
end;

const App: PApplication = NIL;


procedure CloseWindowEx(o: PWidget);
var t: PWidget;
begin
	t := o^.Parent;

	WidgetSendSEvent(o,EV_MESSAGE, WM_CLOSE, NIL);
end;

procedure MaximizePWINDOW(o: PWidget);
var
	tmp: TRect;

begin
   AssignZone(tmp,o,WMAXIMIZE);
   if (PWINDOW(o)^.WindowStates and WS_MAXIMIZED) = WS_MAXIMIZED then
   begin
     PWINDOW(o)^.WindowStates := PWINDOW(o)^.WindowStates and not WS_MAXIMIZED;
     WButton := WBNOTHING;	{ No widget button pressed }
     WidgetSetMetrics(o, PWINDOW(o)^.Bounds.a.x, PWINDOW(o)^.Bounds.a.y, PWINDOW(o)^.Bounds.b.x, PWINDOW(o)^.Bounds.b.y);
   end
   else
   begin
     PWINDOW(o)^.Bounds := o^.Absolute;
     WButton := WBNOTHING;	{ No widget button pressed }
     WidgetSetMetrics(o, WINDOW_MaximizeArea.a.x, WINDOW_MaximizeArea.a.y, WINDOW_MaximizeArea.b.x, WINDOW_MaximizeArea.b.y);
     PWINDOW(o)^.WindowStates := PWINDOW(o)^.WindowStates or WS_MAXIMIZED;
   end;
end;

procedure MinimizePWINDOW(o: PWidget);
var
	tmp: TRect;

begin
  if (PWINDOW(o)^.WindowStates and WS_MINIMIZED) = WS_MINIMIZED then
  begin
    PWINDOW(o)^.WindowStates := PWINDOW(o)^.WindowStates and not WS_MINIMIZED;
    WButton := WBNOTHING;    { No widget button pressed }
    WidgetSetMetrics(o, PWINDOW(o)^.Bounds.a.x, PWINDOW(o)^.Bounds.a.y, PWINDOW(o)^.Bounds.b.x, PWINDOW(o)^.Bounds.b.y);
  end
  else
  begin
    PWINDOW(o)^.Bounds := o^.Absolute;
    WButton := WBNOTHING;    { No widget button pressed }
    WidgetSetMetrics(o,
      PWINDOW(o)^.Bounds.a.x+GSScreenWidth,
      PWINDOW(o)^.Bounds.a.y+GSScreenHeight,
      PWINDOW(o)^.Bounds.b.x+GSScreenWidth, PWINDOW(o)^.Bounds.b.y+GSScreenHeight
    );
    PWINDOW(o)^.WindowStates := PWINDOW(o)^.WindowStates or WS_MINIMIZED;
  end;
end;

procedure ClosePWINDOW(w: PWindow);
begin
	CloseWindowEx(PWIDGET(w));
end;

function WindowEventHandler ( o: PWidget; Ev: PEvent ): l_bool;
var
	tmp: TRect; pp: TPoint; MinButtonBounds: TRect;
begin
	if ( Ev^.aType = EV_MESSAGE ) then
	begin
		if ( Ev^.Message = WM_CLOSE ) then
		begin
			WidgetDispose(o);

			WindowEventHandler := true;
		end;

		if ( Ev^.Message = WEvNotifySelect ) then
		begin
			WidgetSetFirst(o);

			WindowEventHandler := true;
		end;
		if ( Ev^.Message = WEvNotifyUnSelect ) then
		begin
			WidgetDraw(o, NIL);

			WindowEventHandler := true;
		end;
	end;

	if (Ev^.aType = EV_KEYBOARD) then
	begin
		if ( (Ev^.Keyb.Shift and KB_ALT) = KB_ALT ) then
		begin
			if (Ev^.Keyb.Key = AL_KEY_F4) then
			begin
				CloseWindowEx(o);
				WindowEventHandler := true;
			end;
		end;
	end;

	if (Ev^.aType = EV_MOUSE) then
	begin

		AssignZone(&MinButtonBounds,o,WMINIMIZE);

		if (Ev^.Message = WEvMouseLDown) then
		begin
			(**
			*	Close button down
			*)
			AssignZone(tmp,o,WCLOSE);
			if (CURSOR_IN_RECT(tmp)) then
			begin
				{WButton := WSelected = WBCLOSE;}
                                WButton := WBCLOSE; WSelected := WBCLOSE;

				WidgetDraw(o, @tmp);

				WindowEventHandler := true;
			end;

			(**
			*	Maximize
			*)
			AssignZone(tmp,o,WMAXIMIZE);
			if ((PWINDOW(o)^.WindowFlags and WF_MAXIMIZE) = WF_MINIMIZE) and CURSOR_IN_RECT(tmp) then
			begin
				{WButton := WSelected = WBMAXIMIZE;}
                                WButton := WBMAXIMIZE; WSelected := WBMAXIMIZE;
                                WidgetDraw(o, @tmp);
				WindowEventHandler := true;
			end;

			(**
			*	Minimize
			*)

                        AssignZone(tmp,o,WMINIMIZE);
                        if ((PWINDOW(o)^.WindowFlags and WF_MINIMIZE) = WF_MINIMIZE) and CURSOR_IN_RECT(tmp) then
                        begin
                                WButton := WBMINIMIZE;
                                WSelected := WBMINIMIZE;
                                WidgetDraw(o, @tmp);
                                WindowEventHandler := true;
                        end;

			(**
			*	Resize area button down
			*)
			AssignZone(&tmp,o,WRESIZE);
			if (((PWINDOW(o)^.WindowFlags and WF_RESIZE)=WF_RESIZE) and
                                CURSOR_IN_RECT(tmp)) and
                                ((PWINDOW(o)^.WindowStates and WS_MAXIMIZED)<>WS_MAXIMIZED) then
			begin
				MouseSetCursor(CUR_RESIZE);

				MasterWidget := o; // I'm going to get all mouse messages
				p := Mouse^.State.p;
				WDoing := WDRESIZING;
				WindowEventHandler := true;
			end;

			(**
			*	Popup menu from icon
			*)
			AssignZone(tmp,o,WICON);
			if (CURSOR_IN_RECT(tmp)) then
			begin
				PointAssign(pp,tmp.a.x, tmp.b.y);
				PopUpMenu(@Me, pp, NewMenu(NewMenuItem('Close', NIL, WM_CLOSE, 0, NIL, NIL)), o, 0);

				WindowEventHandler := true;
			end;

			(**
			*	Move area button down
			*)
			AssignZone(tmp,o,WMOVE);
			if (CURSOR_IN_RECT(tmp) and not((PWINDOW(o)^.WindowStates and WS_MAXIMIZED)=WS_MAXIMIZED) and not((PWINDOW(o)^.WindowFlags and WF_UNMOVEABLE)=WF_UNMOVEABLE)) then
			begin
				MasterWidget := o; { I'm going to get all mouse messages }
				p := Mouse^.State.p;
				WDoing := WDMOVING;

				WindowEventHandler := true;
			end;

		end;

		if (Ev^.Message = WEvMouseMove) then
		begin
			(**
			*	The cursor has left any of the window widget buttons
			*)
			if (WButton <> WBNOTHING) then
			begin
				(**
				*	Check if it has left the close button
				*)
				AssignZone(&tmp,o,WCLOSE);
				if (WButton = WBCLOSE) and not CURSOR_IN_RECT(tmp) then
				begin
					WButton := WBNOTHING;
					WidgetDraw(o, @tmp);

					WindowEventHandler := true;
				end;

				(**
				*	Check if it has left the maximize button
				*)
				AssignZone(tmp,o,WMAXIMIZE);
				if (WButton = WBMAXIMIZE) and not CURSOR_IN_RECT(tmp) then
				begin
					WButton := WBNOTHING;
					WidgetDraw(o, @tmp);

					WindowEventHandler := true;
				end;

				(**
				*	Check if it has left the maximize button
				*)
				if (WButton = WBMINIMIZE) and not CURSOR_IN_RECT(MinButtonBounds) then
				begin
					WButton := WBNOTHING;
					WidgetDraw(o, @MinButtonBounds);

					WindowEventHandler := true;
				end;
			end;

			if (Mouse^.State.b and BUTTON_LEFT) = BUTTON_LEFT then
			begin
				if (MasterWidget = o) then
				begin
					(**
					*	Move the OSD
					*)
					if ( WDoing = WDMOVING ) then
					begin
						RectAssign(&tmp,o^.Absolute.a.x+(Mouse^.State.p.x-p.x),o^.Absolute.a.y+(Mouse^.State.p.y-p.y),o^.Absolute.b.x+(Mouse^.State.p.x-p.x),o^.Absolute.b.y+(Mouse^.State.p.y-p.y));
						WidgetSetOSDRect(tmp); // Set/Move OSD rect (xor)
					end
					(**
					*	Resize the OSD
					*)
					else if ( WDoing = WDRESIZING ) then
					begin
						RectAssign(tmp,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.b.y);

						if (PWINDOW(o)^.MinSize.w <= (o^.Absolute.b.x-o^.Absolute.a.x-(p.x-Mouse^.State.p.x))) then
						begin
							RectAssign(&tmp,tmp.a.x,tmp.a.y,tmp.b.x-(p.x-Mouse^.State.p.x),tmp.b.y);
							//WidgetSetOSDRect(tmp); // Set/Move OSD rect (xor)
						end;

						if (PWINDOW(o)^.MinSize.h <= (o^.Absolute.b.y-o^.Absolute.a.y-(p.y-Mouse^.State.p.y))) then
						begin
							RectAssign(tmp,tmp.a.x,tmp.a.y,tmp.b.x,tmp.b.y-(p.y-Mouse^.State.p.y));
						end;

						if (tmp.b.x = o^.Absolute.b.x ) then
							RectAssign(tmp,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.a.x+PWINDOW(o)^.MinSize.w,tmp.b.y);

						if (tmp.b.y = o^.Absolute.b.y) then
							RectAssign(tmp,o^.Absolute.a.x,o^.Absolute.a.y,tmp.b.x,o^.Absolute.a.y+PWINDOW(o)^.MinSize.h);

						WidgetSetOSDRect(tmp); // Set/Move OSD rect (xor)
					end;
				end;

				(**
				*	Check if the mouse has enetered any of the window
				*		widget buttons
				*)
				if (WButton = WBNOTHING)  and (WDoing = WDNOTHING) then
				begin
					AssignZone(&tmp,o,WCLOSE);
					if (WSelected = WBCLOSE) and CURSOR_IN_RECT(tmp) then
					begin

						WButton := WBCLOSE;
						WidgetDraw(o, @tmp);

						WindowEventHandler := true;
					end;
					AssignZone(tmp,o,WMAXIMIZE);
					if (WSelected = WBMAXIMIZE) and ((PWINDOW(o)^.WindowFlags and WF_MAXIMIZE) = WF_MAXIMIZE) and CURSOR_IN_RECT(tmp) then
					begin

						WButton := WBMAXIMIZE;
						WidgetDraw(o, @tmp);

						WindowEventHandler := true;
					end;

					if (( WSelected = WBMINIMIZE ) and ((PWINDOW(o)^.WindowFlags and WF_MINIMIZE)=WF_MINIMIZE) and CURSOR_IN_RECT(MinButtonBounds)) then
					begin
						WButton := WBMINIMIZE;
						WidgetDraw(o, @MinButtonBounds);

						WindowEventHandler := true;
					end;

				end;
			end;
		end;

		if (Ev^.Message = WEvMouseLUp) then
		begin
			if (WDoing = WDNOTHING) then
			begin
				(*
				*	Mouse up on the close button, so close window.
				*)
				AssignZone(&tmp,o,WCLOSE);
				if (WSelected = WBCLOSE) and CURSOR_IN_RECT(tmp) then
				begin
					WButton := WBNOTHING;
					CloseWindowEx(o);

					WindowEventHandler := true;
				end;

				AssignZone(&tmp,o,WMAXIMIZE);
				if (WSelected = WBMAXIMIZE) and ((PWINDOW(o)^.WindowFlags and WF_MAXIMIZE)=WF_MAXIMIZE) and CURSOR_IN_RECT(tmp) then
				begin
					if (PWINDOW(o)^.WindowStates and WS_MAXIMIZED) = WS_MAXIMIZED then
					begin
						PWINDOW(o)^.WindowStates := PWINDOW(o)^.WindowStates and not WS_MAXIMIZED;
						WButton := WBNOTHING;	  { No widget button pressed }
						WidgetSetMetrics(o, PWINDOW(o)^.Bounds.a.x, PWINDOW(o)^.Bounds.a.y, PWINDOW(o)^.Bounds.b.x, PWINDOW(o)^.Bounds.b.y);
					end
					else
					begin
						PWINDOW(o)^.Bounds := o^.Absolute;
						WButton := WBNOTHING;	// No widget button pressed
						WidgetSetMetrics(o, WINDOW_MaximizeArea.a.x, WINDOW_MaximizeArea.a.y, WINDOW_MaximizeArea.b.x, WINDOW_MaximizeArea.b.y);
						PWINDOW(o)^.WindowStates := PWINDOW(o)^.WindowStates or WS_MAXIMIZED;
					end;

					WindowEventHandler := true;
				end;

                                AssignZone(tmp,o,WMINIMIZE);
                                if ( (WSelected = WBMINIMIZE) and ((PWINDOW(o)^.WindowFlags and WF_MINIMIZE) = WF_MINIMIZE) and CURSOR_IN_RECT(tmp)) then
                                begin
                                        if (PWINDOW(o)^.WindowStates and WS_MINIMIZED) = WS_MINIMIZED then
                                        begin
                                                PWINDOW(o)^.WindowStates := PWINDOW(o)^.WindowStates and not WS_MINIMIZED;
                                                WButton := WBNOTHING;    // No widget button pressed
                                                WidgetSetMetrics(o, PWINDOW(o)^.Bounds.a.x, PWINDOW(o)^.Bounds.a.y, PWINDOW(o)^.Bounds.b.x, PWINDOW(o)^.Bounds.b.y);
                                        end
                                        else
                                        begin
                                                PWINDOW(o)^.Bounds := o^.Absolute;
                                                WButton := WBNOTHING;    { No widget button pressed }
                                                WidgetSetMetrics(o, PWINDOW(o)^.Bounds.a.x+GSScreenWidth, PWINDOW(o)^.Bounds.a.y+GSScreenHeight, PWINDOW(o)^.Bounds.b.x+GSScreenWidth, PWINDOW(o)^.Bounds.b.y+GSScreenHeight);
                                                PWINDOW(o)^.WindowStates := PWINDOW(o)^.WindowStates or WS_MINIMIZED;
                                        end;

                                        WindowEventHandler := true;
                                end;
			end;

			if (MasterWidget = o) then
			begin
				WidgetUnOSD;  { Unactivate OSD }

				(**
				*	If moving, move the window to the new position.
				*)
				if ( WDoing = WDMOVING ) then
				begin

					WidgetMove(o,Mouse^.State.p.x-p.x,Mouse^.State.p.y-p.y);


				end
				else if (WDoing = WDRESIZING) then
				begin
					RectAssign(&tmp,o^.Absolute.a.x,o^.Absolute.a.y,o^.Absolute.b.x,o^.Absolute.b.y);

					if (PWINDOW(o)^.MinSize.w <= (o^.Absolute.b.x-o^.Absolute.a.x-(p.x-Mouse^.State.p.x))) then
					begin
						RectAssign(tmp,tmp.a.x,tmp.a.y,tmp.b.x-(p.x-Mouse^.State.p.x),tmp.b.y);
					end;

					if (PWINDOW(o)^.MinSize.h <= (o^.Absolute.b.y-o^.Absolute.a.y-(p.y-Mouse^.State.p.y))) then
					begin
						RectAssign(tmp,tmp.a.x,tmp.a.y,tmp.b.x,tmp.b.y-(p.y-Mouse^.State.p.y));
					end;

					if (tmp.b.x = o^.Absolute.b.x ) then
						RectAssign(&tmp,o^.Absolute.a.x,o^.Absolute.a.y,tmp.a.x+PWINDOW(o)^.MinSize.w,tmp.b.y);

					if (tmp.b.y = o^.Absolute.b.y) then
						RectAssign(tmp,o^.Absolute.a.x,o^.Absolute.a.y,tmp.b.x,tmp.a.y+PWINDOW(o)^.MinSize.h);

					WidgetResize(o, tmp.b.x-o^.Absolute.b.x, tmp.b.y-o^.Absolute.b.y);

					MouseSetCursor(CUR_POINTER);
				end;

				MasterWidget := NIL;	{ I will get only my events... Not all mouse ones : }
				WDoing := WDNOTHING;	{ Window is idle           }
				WButton := WBNOTHING;	{ No widget button pressed }
			end;

		end;
	end;

	WindowEventHandler := false;
end;

procedure WindowSetup ( o: PWidget );
begin
	if ( PWINDOW(o)^.WindowFlags and WF_CENTERED ) = WF_CENTERED then
	begin
		WidgetPreCenter(PWIDGET(o));
	end;
end;

procedure WindowSetCaptionEx ( o: PWindow; Caption: l_text );
begin
	if ( o^.Caption <> '' ) then o^.Caption := '';

	o^.Caption := Caption;

	WidgetDraw(PWIDGET(o), NIL);

end;

procedure WindowSetCaption ( o: PWindow; Caption: l_text; args: {$ifdef OPascal}array of const{$else}OpenArray{$endif} );
var S: String;
begin
	{$ifdef OPascal}
	S := Format(Caption, Args);
	WindowSetCaptionEx(o, S);
	{$else}
	FormatStr(S, args);
	WindowSetCaptionEx(o, S);
	{$endif}
end;

procedure WindowFreeEx ( o: PWidget );
begin
	if assigned( PWINDOW(o)^.Icon ) then FreeIcon(PWINDOW(o)^.Icon);
	if ( PWINDOW(o)^.Caption <> '' ) then PWINDOW(o)^.Caption := '';
end;

function WindowInit(w: PWindow; App: PApplication; r: TRect; szCaption: l_text; flags: l_ulong): PWindow;
var
  a: TRect;

begin
	if ( UseSkins ) then
		RectAssign(r, r.a.x, r.a.y, r.b.x+SkinWindowFace^.Left+SkinWindowFace^.Right, r.b.y+SkinTitleActive^.Skin^.h+SkinWindowFace^.Top+SkinWindowFace^.Bottom)
	else
		RectAssign(r, r.a.x, r.a.y, r.b.x+6, r.b.y+22);
		
		
	fillchar(w, sizeof(TWindow), 0);

	w^.Caption	:= szCaption;
	w^.Bounds	:= r;
	w^.Cursor	:= CUR_POINTER;
	
	if assigned( App^.Icon ) then
		w^.Icon		:= DuplicateIcon(App^.Icon)
	else
		w^.Icon		:= DuplicateIcon(DefaultIcon);
	
	SizeAssign(w^.MaxSize,65000,65000);
	SizeAssign(w^.MinSize,100,100);

	if (flags=0) then w^.WindowFlags := WF_NORMAL
	else w^.WindowFlags := flags;

	IntialiseWidget(App, WindowClass, PWIDGET(w), w^.Bounds );
	PWIDGET(w)^.Flags := PWIDGET(w)^.Flags or WFSelectable or WFFocusable;
	PWIDGET(w)^.EventHandler := {$ifdef fpc}@{$endif}WindowEventHandler;
	PWIDGET(w)^.Draw := {$ifdef fpc}@{$endif}WindowDraw;
	PWIDGET(w)^.Setup := {$ifdef fpc}@{$endif}WindowSetup;
	PWIDGET(w)^.FreeEx := {$ifdef fpc}@{$endif}WindowFreeEx;

	if ( UseSkins ) then
		RectAssign(a, SkinWindowFace^.Left, SkinTitleActive^.Skin^.h+SkinWindowFace^.Top, r.b.x-r.a.x-SkinWindowFace^.Right, r.b.y-r.a.y-SkinWindowFace^.Bottom)
	else
		RectAssign(a, 3, 20, r.b.x-r.a.x-3, r.b.y-r.a.y-2);

	WidgetSetChildArea(PWIDGET(w), a );

	WindowInit := w;
end;


function CreatePWINDOW(App:PApplication; r:TRect; szCaption:l_text; flags:l_ulong): PWindow;
var
  a: TRect; w: PWindow;
begin
	New(w);
	if not assigned(w) then CreatePWINDOW := NIL else CreatePWINDOW := WindowInit(w,App,r,szCaption,flags);
end;

function CreateWindow(App: PApplication; r: TRect; szCaption: l_text;  flags: l_ulong): PWindow;
begin
  CreateWindow := CreatePWindow(App, r, szCaption, flags);
end;
(**
*	MessageBox functions
*)
const
  MSGBOX_BTNWIDTH = 80;
  MSGBOX_BTNHEIGHT = 22;
  MSGBOX_BTNSPACE = 6;

(**
*	NAME: MessageBox
*	DESCRIPTION: Pops up a small window with some warning
*	RETURN: A message ID type defined in widget.h
*)
function BtnHeight(Condition: l_bool; aMinHeight,aMaxHeight: Integer): Integer;
begin
  if Condition then BtnHeight := aMinHeight else BtnHeight := aMaxHeight;
end;

function MessageBox ( App: PApplication; Title,Message: l_text; flags: l_ulong ): l_ulong;
var
  Msg:l_ulong = 0;

  w: PWindow;
  b: PButton;
  l: PLabel;
  i: PImageLabel;
  Icon:	p_bitmap;
  AFlags: l_ulong;
  r: TRect;
  t,tb: l_text;
  tix: integer;
  Lines: Integer;
  ww,h:	l_ulong;

  temp: PWidget;
    
begin   
    w      := NIL;
    b      := NIL;
    l      := NIL;
    i      := NIL;
    Icon   := NIL;
    AFlags := flags;
    Lines  := 0;
    ww     := 240;
    h      := 10;
    t      := Message;
    
	if (Flags and MBB_YESYESALLNOCANCEL) = MBB_YESYESALLNOCANCEL then ww := 330;

	(**
	*	Calculate number lines of text
	*)
	tix := 1;
	while (tix <= length(t)) do
	begin
		if (t[tix] = #10) then inc(Lines);
		h := h + al_text_height(default_font);
		ww := max(ww,FontTextLengthEx(default_font,tb,t,false));
		inc(tix);
	end;


	(**
	*	Messagebox icon
	*)
	if (Flags and MBI_INFORMATION) = MBI_INFORMATION then
	begin
		Icon := IconInformation;
		stream := LoadData2('/system/System/sounds/info.mp3',DTYPE_SOUNDSTREAM);
	end
	else if (Flags and MBI_WARNING) = MBI_WARNING then
	begin
		Icon := IconWarning;
		stream := LoadData2('/system/System/sounds/error.mp3',DTYPE_SOUNDSTREAM);
	end
	else if (Flags and MBI_QUESTION) = MBI_QUESTION then
	begin
		Icon := IconQuestion;
		stream := LoadData2('/system/System/sounds/info.mp3',DTYPE_SOUNDSTREAM);
	end
	else if (Flags and MBI_ERROR) = MBI_ERROR then
	begin
		Icon := IconError;
		stream := LoadData2('/system/System/sounds/error.mp3',DTYPE_SOUNDSTREAM);

	end
	else
	begin
		Icon := IconInformation;
		stream := LoadData2('/system/System/sounds/info.mp3',DTYPE_SOUNDSTREAM);
	end;

	RectAssign(r,0 , 0, ww + 50, BtnHeight(40+h+5 > 90, 40+h, 90));
	w := CreatePWINDOW(App, r, Title, WF_FRAME or WF_CAPTION or WF_CENTERED);
	InsertWidget(DeskTop, PWIDGET(w));

	RectAssign(&r,45,5,ww+50,h+5);
	l := CreateLabel(App, r, Message );
	PWIDGET(l)^.BackgroundColor := COL_3DFACE;
	InsertWidget(PWIDGET(w), PWIDGET(l));

	if assigned( Icon ) then
	begin
		RectAssign(r,5,(
          PWIDGET(w)^.ChildArea.b.y-
          PWIDGET(w)^.ChildArea.a.y) div 2-Icon^.h div 2-10,40,(
          PWIDGET(w)^.ChildArea.b.y-PWIDGET(w)^.ChildArea.a.y) div 2+Icon^.h div 2-10);
		i := CreateImageLabel(App, r, Icon);
		PWIDGET(i)^.BackgroundColor := COL_3DFACE;
		InsertWidget(PWIDGET(w), PWIDGET(i));
	end;
	
	if ((Flags and MBB_OK)=MBB_OK) or (Flags = 0) then
	begin
		RectAssign(r,((
          PWIDGET(w)^.ChildArea.b.x - 
          PWIDGET(w)^.ChildArea.a.x) div 2) - MSGBOX_BTNWIDTH div 2,
          PWIDGET(w)^.ChildArea.b.y-
          PWIDGET(w)^.ChildArea.a.y-5-MSGBOX_BTNHEIGHT, ((
          PWIDGET(w)^.ChildArea.b.x - 
          PWIDGET(w)^.ChildArea.a.x) div 2) + MSGBOX_BTNWIDTH div 2,
          PWIDGET(w)^.ChildArea.b.y - 
          PWIDGET(w)^.ChildArea.a.y  - 5
        );
		b := CreateButton(App, r, 'OK', MSG_OK);
		InsertWidget(PWIDGET(w), PWIDGET(b));
	end
	else if (Flags and MBB_OKCANCEL) = MBB_OKCANCEL then
	begin
	  RectAssign(r,((
            PWIDGET(w)^.ChildArea.b.x -
            PWIDGET(w)^.ChildArea.a.x) div 2) - MSGBOX_BTNSPACE div 2 - MSGBOX_BTNWIDTH,
            PWIDGET(w)^.ChildArea.b.y - PWIDGET(w)^.ChildArea.a.y - 5 - MSGBOX_BTNHEIGHT, ((
            PWIDGET(w)^.ChildArea.b.x - PWIDGET(w)^.ChildArea.a.x) div 2) - MSGBOX_BTNSPACE div 2,
            PWIDGET(w)^.ChildArea.b.y - PWIDGET(w)^.ChildArea.a.y - 5
          );

          b := CreateButton(App, r, 'OK', MSG_OK );
	  InsertWidget(PWIDGET(w), PWIDGET(b));

	  RectAssign(r,((
            PWIDGET(w)^.ChildArea.b.x -
            PWIDGET(w)^.ChildArea.a.x) div 2) + MSGBOX_BTNSPACE div 2,
            PWIDGET(w)^.ChildArea.b.y -
            PWIDGET(w)^.ChildArea.a.y - 5 - MSGBOX_BTNHEIGHT, ((
            PWIDGET(w)^.ChildArea.b.x - PWIDGET(w)^.ChildArea.a.x) div 2) + MSGBOX_BTNSPACE div 2 + MSGBOX_BTNWIDTH,
            PWIDGET(w)^.ChildArea.b.y - PWIDGET(w)^.ChildArea.a.y  - 5
          );
	   b := CreateButton(App, r, 'Cancel', MSG_CANCEL );
          InsertWidget(PWIDGET(w), PWIDGET(b));
	end
	else if (Flags and MBB_YESNO) = MBB_YESNO then
	begin
          RectAssign(r,((
            PWIDGET(w)^.ChildArea.b.x -
            PWIDGET(w)^.ChildArea.a.x) div 2) - MSGBOX_BTNSPACE div 2 - MSGBOX_BTNWIDTH,
            PWIDGET(w)^.ChildArea.b.y - PWIDGET(w)^.ChildArea.a.y  - 5 - MSGBOX_BTNHEIGHT, ((
            PWIDGET(w)^.ChildArea.b.x - PWIDGET(w)^.ChildArea.a.x) div 2) - MSGBOX_BTNSPACE div 2,
            PWIDGET(w)^.ChildArea.b.y - PWIDGET(w)^.ChildArea.a.y - 5);
            b := CreateButton(App, r, 'Yes', MSG_YES );
	   InsertWidget(PWIDGET(w), PWIDGET(b));
	   RectAssign(r,((
             PWIDGET(w)^.ChildArea.b.x -
             PWIDGET(w)^.ChildArea.a.x) div 2) + MSGBOX_BTNSPACE div 2,
             PWIDGET(w)^.ChildArea.b.y -
             PWIDGET(w)^.ChildArea.a.y - 5- MSGBOX_BTNHEIGHT, ((
             PWIDGET(w)^.ChildArea.b.x -
             PWIDGET(w)^.ChildArea.a.x) div 2) + MSGBOX_BTNSPACE div 2 + MSGBOX_BTNWIDTH,
             PWIDGET(w)^.ChildArea.b.y -
             PWIDGET(w)^.ChildArea.a.y - 5);
             b := CreateButton(App, r, 'No', MSG_NO );
	     InsertWidget(PWIDGET(w), PWIDGET(b));
	end
	else if (Flags and MBB_YESNOCANCEL) = MBB_YESNOCANCEL then
	begin
	  RectAssign(r,((
            PWIDGET(w)^.ChildArea.b.x -
            PWIDGET(w)^.ChildArea.a.x) div 2)  - MSGBOX_BTNWIDTH div 2 - MSGBOX_BTNSPACE - MSGBOX_BTNWIDTH,
            PWIDGET(w)^.ChildArea.b.y - PWIDGET(w)^.ChildArea.a.y - 5 - MSGBOX_BTNHEIGHT, ((
            PWIDGET(w)^.ChildArea.b.x -
            PWIDGET(w)^.ChildArea.a.x) div 2) - MSGBOX_BTNWIDTH div 2 - MSGBOX_BTNSPACE,
            PWIDGET(w)^.ChildArea.b.y - PWIDGET(w)^.ChildArea.a.y - 5
          );
	  b := CreateButton(App, r, 'Yes', MSG_YES );
	  InsertWidget(PWIDGET(w), PWIDGET(b));
          RectAssign(r,((
            PWIDGET(w)^.ChildArea.b.x - PWIDGET(w)^.ChildArea.a.x) div 2) - MSGBOX_BTNWIDTH div 2,
            PWIDGET(w)^.ChildArea.b.y - PWIDGET(w)^.ChildArea.a.y  - 5 - MSGBOX_BTNHEIGHT, ((
            PWIDGET(w)^.ChildArea.b.x - PWIDGET(w)^.ChildArea.a.x) div 2) + MSGBOX_BTNWIDTH div 2,
            PWIDGET(w)^.ChildArea.b.y - PWIDGET(w)^.ChildArea.a.y - 5
          );
	  b := CreateButton(App, r, 'No', MSG_NO );
	  InsertWidget(PWIDGET(w), PWIDGET(b));
	  RectAssign(r,((
            PWIDGET(w)^.ChildArea.b.x -
            PWIDGET(w)^.ChildArea.a.x) div 2) + MSGBOX_BTNWIDTH div 2 + MSGBOX_BTNSPACE,
            PWIDGET(w)^.ChildArea.b.y -
            PWIDGET(w)^.ChildArea.a.y - 5 - MSGBOX_BTNHEIGHT, ((
            PWIDGET(w)^.ChildArea.b.x -
            PWIDGET(w)^.ChildArea.a.x) div 2) + MSGBOX_BTNWIDTH div 2 + MSGBOX_BTNSPACE + MSGBOX_BTNWIDTH,
            PWIDGET(w)^.ChildArea.b.y -
            PWIDGET(w)^.ChildArea.a.y  - 5
          );
          b := CreateButton(App, r, 'Cancel', MSG_CANCEL );
	  InsertWidget(PWIDGET(w), PWIDGET(b));
	end
	else if (Flags and MBB_YESYESALLNOCANCEL) = MBB_YESYESALLNOCANCEL then
	begin
		RectAssign(r,((
          PWIDGET(w)^.ChildArea.b.x - 
          PWIDGET(w)^.ChildArea.a.x) div 2) - (2*MSGBOX_BTNSPACE) - (2*MSGBOX_BTNWIDTH),
          PWIDGET(w)^.ChildArea.b.y - 
          PWIDGET(w)^.ChildArea.a.y - 5 - MSGBOX_BTNHEIGHT, ((
          PWIDGET(w)^.ChildArea.b.x - PWIDGET(w)^.ChildArea.a.x) div 2) - MSGBOX_BTNWIDTH - (2*MSGBOX_BTNSPACE),
          PWIDGET(w)^.ChildArea.b.y - PWIDGET(w)^.ChildArea.a.y - 5
        );
	b := CreateButton(App, r, 'Yes', MSG_YES );
	InsertWidget(PWIDGET(w), PWIDGET(b));

	RectAssign(r,((
          PWIDGET(w)^.ChildArea.b.x - 
          PWIDGET(w)^.ChildArea.a.x) div 2) - MSGBOX_BTNSPACE - MSGBOX_BTNWIDTH,
          PWIDGET(w)^.ChildArea.b.y - 
          PWIDGET(w)^.ChildArea.a.y  - 5 - MSGBOX_BTNHEIGHT, ((
          PWIDGET(w)^.ChildArea.b.x - 
          PWIDGET(w)^.ChildArea.a.x) div 2) - MSGBOX_BTNSPACE,
          PWIDGET(w)^.ChildArea.b.y - 
          PWIDGET(w)^.ChildArea.a.y - 5
          );
	  b := CreateButton(App, r, 'Yes all', MSG_YESALL );
	  InsertWidget(PWIDGET(w), PWIDGET(b));


	  RectAssign(r,((
          PWIDGET(w)^.ChildArea.b.x - 
          PWIDGET(w)^.ChildArea.a.x) div 2),
          PWIDGET(w)^.ChildArea.b.y - 
          PWIDGET(w)^.ChildArea.a.y  - 5 - MSGBOX_BTNHEIGHT, ((
          PWIDGET(w)^.ChildArea.b.x - 
          PWIDGET(w)^.ChildArea.a.x) div 2) + MSGBOX_BTNWIDTH,
          PWIDGET(w)^.ChildArea.b.y - 
          PWIDGET(w)^.ChildArea.a.y - 5
          );
	  b := CreateButton(App, r, 'No', MSG_NO );
	  InsertWidget(PWIDGET(w), PWIDGET(b));

	  RectAssign(r,
          ((PWIDGET(w)^.ChildArea.b.x - 
            PWIDGET(w)^.ChildArea.a.x) div 2) + MSGBOX_BTNSPACE + MSGBOX_BTNWIDTH,
            PWIDGET(w)^.ChildArea.b.y - 
            PWIDGET(w)^.ChildArea.a.y - 5 - MSGBOX_BTNHEIGHT, ((
            PWIDGET(w)^.ChildArea.b.x - PWIDGET(w)^.ChildArea.a.x) div 2) + MSGBOX_BTNSPACE + (2*MSGBOX_BTNWIDTH),
            PWIDGET(w)^.ChildArea.b.y - PWIDGET(w)^.ChildArea.a.y  - 5
          );
	  b := CreateButton(App, r, 'Cancel', MSG_CANCEL );
	  InsertWidget(PWIDGET(w), PWIDGET(b));

	end;
	PlaySoundStream(stream);
	WidgetDrawAll(PWIDGET(w));
	while ( (Msg <> MSG_CANCEL) and (Msg <> MSG_YES) and (Msg <> MSG_NO) and (Msg <> MSG_OK) and (Msg <> MSG_YESALL) ) do
	begin
		Msg := WidgetExecute(PWIDGET(w));
		if ( Msg = WM_CLOSE ) then Msg := MSG_CANCEL;
	end;

	temp := PWIDGET(w)^.Parent;

	WidgetDispose(PWIDGET(w));

	MessageBox := Msg;
end;

procedure WindowNotifyError ( App: PApplication; h: PErrorHandler );
var txt: l_text;
begin
	{ TO DO : better version, support multiple errors. }
	
	if assigned( h ) then
	if assigned( PBLIST(h)^.Last ) then
	begin
	  txt := 'Widget konnte nicht erzeugt werden!';
          {TextArgs('%s\n\n%s',[ERROR(PBLIST(h)^.Last)^.Description,ERROR(PBLIST(h)^.Last)^.Log]);}
	  MessageBox(App,'Error',txt, MBI_ERROR or MBB_OK);
	  txt := '';
	end;
	
end;

function ProgressPopUp ( App: PApplication; Flags: l_ulong; Title, Message: l_text; Steps: l_ulong; InitInfo: l_text ): PProgressPop;
var
    r: TRect;
    h: l_ulong;
    L1: PLabel;
    Cancel: PButton;
    o: PProgressPop;
begin
  New(o);
  if not assigned( o ) then ProgressPopUp := NIL else
  begin
    fillchar(o,sizeof(TProgressPop),0);

    if ( Flags and 1 ) = 1 then h := h + 25;
    if ( Flags and 2 ) = 2 then h := h + 20;
    if ( Flags and 4 ) = 4 then h := h + 20;

    RectAssign( r, 0 , 0, 310, 70+h );

    o^.W := CreatePWINDOW(App, r, Title, WF_FRAME or WF_CAPTION or WF_CENTERED);
    InsertWidget(DeskTop, PWIDGET(o^.W));

    RectAssign( r, 0, 0, 300, 20);
    L1 := CreateLabel(App, r, Message );
    PWIDGET(L1)^.BackgroundColor := COL_3DFACE;
    InsertWidget(PWIDGET(o^.W),PWIDGET(L1));

    if ( Flags and 1 ) = 1 then
    begin
      RectAssign( r, 0, r.b.y+5, 300, r.b.y+25);
      o^.L := CreateLabel(App, r, InitInfo );
      PWIDGET(o^.L)^.BackgroundColor := COL_3DFACE;
      PWIDGET(o^.L)^.Flags := PWIDGET(o^.L)^.Flags or WFForceBuffer;
      InsertWidget(PWIDGET(o^.W),PWIDGET(o^.L));
    end;

    if ( Flags and 2 ) = 2 then
    begin
      RectAssign( r, 0, r.b.y+5, 300, r.b.y+20);
      o^.B := CreateProgressBar(App, r, Steps );
      PWIDGET(o^.B)^.Flags := PWIDGET(o^.L)^.Flags or WFForceBuffer;
      InsertWidget(PWIDGET(o^.W),PWIDGET(o^.B));

      if ( Flags and 4 ) = 4 then
      begin
	RectAssign( r, 0, r.b.y+5, 300, r.b.y+20);
	o^.T := CreateLabel(App, r, '' );
	PWIDGET(o^.T)^.BackgroundColor := COL_3DFACE;
	PWIDGET(o^.T)^.Flags := PWIDGET(o^.L)^.Flags or WFForceBuffer;
	InsertWidget(PWIDGET(o^.W),PWIDGET(o^.T));
	o^.Started := 0;
      end;
    end;

    RectAssign( r, 200, r.b.y+5, 300, r.b.y+25);
    Cancel := CreateButton(App, r, 'Cancel', WM_CLOSE );
    InsertWidget(PWIDGET(o^.W),PWIDGET(Cancel));

    PWIDGET(o^.W)^.Flags := PWIDGET(o^.L)^.Flags or WFExecuting;

    WidgetDrawAll(PWIDGET(o^.W));

    ProgressPopUp := o;
  end;
end;

procedure ProgressPopSetInfoEx ( p: PProgressPop; AText: l_text );
begin
	if assigned( p^.L ) then
	begin
	  LabelSetTextEx(p^.L,AText);
	  WidgetDraw(PWIDGET(p^.L),NIL);
	end;
end;

procedure ProgressPopSetInfo ( p:PProgressPop; StrFormat:l_text; args:{$ifdef OPascal}array of const{$else}OpenArray{$endif});
var InfoStr: String;
begin
  {vsprintf(CommonBufferText, Format, arg);}
  {$ifdef OPascal}
  InfoStr := Format(StrFormat, args);
  {$else}
  FormatStr(InfoStr, StrFormat, Args);
  {$endif}
  ProgressPopSetInfoEx(p, InfoStr);
end;

procedure ProgressPopStep ( p: PProgressPop; Steps: l_ulong );
var DS,DT,RS,RT: TDateTime;
begin
	if assigned( p^.B ) then
	begin
	  p^.B^.Step(p^.B,Steps);
	  WidgetDraw(PWIDGET(p^.B),NIL);

	  if assigned( p^.T ) then
	  begin
		 	DT := TDateTime(0)-p^.Started;
		 	if ( (DT > 4) and (p^.B^.Promile <> 0) ) then
		 	begin
		 	  DS := p^.B^.Promile;
			  RS := p^.B^.Steps-p^.B^.Promile;
			  RT := DT*RS / DS;

			  if ( RT <= 30 ) then
			  	LabelSetText(p^.T,'Less than 30 secondes left...',[])
			  else if ( RT <= 60 ) then
			  	LabelSetText(p^.T,'Less than 1 minute left...',[])
			  else if ( RT <= 90 ) then
			  	LabelSetText(p^.T,'About 1 minute left...',[])
				else
			  	LabelSetText(p^.T,'About %d minutes left...',[(RT/60)+1]);
	  		end;
	  end;

	end;

end;

procedure ProgressPopKick ( p: PProgressPop );
begin
  WidgetDispose(PWIDGET(p^.W));
  dispose(p);
end;

function  StdCreateWindow ( App: PApplication; r: TRect ): PWidget;
begin
	StdCreateWindow := PWIDGET(CreatePWINDOW( App, r, '', 0));
end;

{ scripting interface }

function WindowSetRProp ( o: PWidget; Name: l_text; const Value; Redraw: l_bool ): l_bool;
begin
	
	WindowSetRProp := false;
	if ( TextCompare(Name,'caption')=0 ) then
	begin
		if ( PWINDOW(o)^.Caption <> '' ) then PWINDOW(o)^.Caption := '';
		PWINDOW(o)^.Caption := l_text(Value);
		if ( Redraw ) then WidgetDraw(PWIDGET(o), NIL);
		WindowSetRProp := true;
	end;
	if ( TextCompare(Name,'maximize')=0 ) then
	begin
		if L_bool(Value) then
			PWINDOW(o)^.WindowFlags := PWINDOW(o)^.WindowFlags or WF_MAXIMIZE
		else
			PWINDOW(o)^.WindowFlags := PWINDOW(o)^.WindowFlags and not WF_MAXIMIZE;
		if ( Redraw ) then WidgetDraw(PWIDGET(o), NIL);
		WindowSetRProp := true;
	end;
	if ( TextCompare(Name,'minimize')=0 ) then
	begin
		if l_bool(Value) then
			PWINDOW(o)^.WindowFlags := PWINDOW(o)^.WindowFlags or WF_MINIMIZE
		else
			PWINDOW(o)^.WindowFlags := PWINDOW(o)^.WindowFlags and not WF_MINIMIZE;
		if ( Redraw ) then WidgetDraw(PWIDGET(o), NIL);
		WindowSetRProp := true;
	end;
	if ( TextCompare(Name,'resize')=0 ) then
	begin
		if l_bool(Value) then
			PWINDOW(o)^.WindowFlags := PWINDOW(o)^.WindowFlags or WF_RESIZE
		else
			PWINDOW(o)^.WindowFlags := PWINDOW(o)^.WindowFlags and not WF_RESIZE;
		if ( Redraw ) then WidgetDraw(PWIDGET(o), NIL);
		WindowSetRProp := true;
	end;
	if ( TextCompare(Name,'center')=0 ) then
	begin
		if l_bool(Value) then
			PWINDOW(o)^.WindowFlags := PWINDOW(o)^.WindowFlags or WF_CENTERED
		else
			PWINDOW(o)^.WindowFlags := PWINDOW(o)^.WindowFlags and not WF_CENTERED;
		WindowSetRProp := true;
	end;
		
end;

function WindowGetRProp ( o: PWidget; Name: l_text ): Pointer; { in c++ -> Pointer }
var tmp: Boolean;
begin
    WindowGetRProp := Pointer(false);
    if ( TextCompare(Name,'caption')=0 ) then
    begin
      tmp := PWINDOW(o)^.Caption <> '';
      WindowGetRProp := DuplicateBool(tmp);
    end;
	
    if ( TextCompare(Name,'maximize')=0 ) then
    begin
      tmp := (PWINDOW(o)^.WindowFlags and WF_MAXIMIZE) = WF_MAXIMIZE;
      WindowGetRProp := DuplicateBool(tmp);
    end;
    if ( TextCompare(Name,'minimize')=0 ) then
    begin
      tmp := (PWINDOW(o)^.WindowFlags and WF_MINIMIZE) = WF_MINIMIZE;
      WindowGetRProp := DuplicateBool(tmp);
    end;
    if ( TextCompare(Name,'resize')=0 ) then
    begin
      tmp := (PWINDOW(o)^.WindowFlags and WF_RESIZE)= WF_RESIZE;
      WindowGetRProp := DuplicateBool(tmp);
    end;
    if ( TextCompare(Name,'center')=0 ) then
    begin
      tmp := (PWINDOW(o)^.WindowFlags and WF_CENTERED) = WF_CENTERED;
      WindowGetRProp := DuplicateBool(tmp);
    end;
end;

function LibMain ( argc: integer; argv: l_text ): integer;
var pp: PProgressPop;
begin

	WindowClass := CreateWidgetClass( ULONG_ID('W','i','n',' '), StdWidgetClass, WCF_CONTAINER );
	WindowClass^.StdCreate := {$ifdef fpc}@{$endif}StdCreateWindow;
	WindowClass^.SetRProp := {$ifdef fpc}@{$endif}WindowSetRProp;
	WindowClass^.GetRProp := {$ifdef fpc}@{$endif}WindowGetRProp;

	WClassRegisterProp(WindowClass,PTYPE_STRING,'caption');
	WClassRegisterProp(WindowClass,PTYPE_BOOL,'maximize');
	WClassRegisterProp(WindowClass,PTYPE_BOOL,'minimize');
	WClassRegisterProp(WindowClass,PTYPE_BOOL,'resize');	
	WClassRegisterProp(WindowClass,PTYPE_BOOL,'center');	
	
	DefaultIcon := NewIcon(LoadImage('SYSTEM/ICONS/app16.bmp'),NIL,NIL);
	
	SkinTitleActive	 := GetSkinItem('WindowTitleActive');
	SkinTitlePassive := GetSkinItem('WindowTitlePassive');
	SkinCloseUp	 := GetSkinItem('WindowCloseUp');
	SkinCloseDown	 := GetSkinItem('WindowCloseDown');
	SkinWindowFace	 := GetSkinItem('WindowFace');
	SkinMaximizeUp	 := GetSkinItem('WindowMaximizeUp');
	SkinMaximizeDown := GetSkinItem('WindowMaximizeDown');
	SkinMinimizeUp	 := GetSkinItem('WindowMinimizeUp');
	SkinMinimizeDown := GetSkinItem('WindowMinimizeDown');

	IconQuestion	 := LoadImage('SYSTEM/ICONS/msg01.bmp');
	IconInformation	 := LoadImage('SYSTEM/ICONS/msg02.bmp');
	IconWarning	 := LoadImage('SYSTEM/ICONS/msg03.bmp');
	IconError	 := LoadImage('SYSTEM/ICONS/msg04.bmp');
                    
	(**
	 * Default maximize area
	 *)
	RectAssign(WINDOW_MaximizeArea, 0, 0,GSScreenWidth, GSScreenHeight);
    (*
	APPEXPORT(CreateWindow);
	APPEXPORT(CloseWindow);
	APPEXPORT(MinimizeWindow);
	APPEXPORT(MaximizeWindow);
	APPEXPORT(CloseWindowEx);
	APPEXPORT(WindowEventHandler);
	APPEXPORT(MessageBox);
	APPEXPORT(WindowDraw);

	APPEXPORT(WINDOW_MaximizeArea);

	APPEXPORT(WindowSetCaptionEx);
	APPEXPORT(WindowSetCaption);

	APPEXPORT(ProgressPopUp);
	APPEXPORT(ProgressPopSetInfoEx);
	APPEXPORT(ProgressPopSetInfo);
	APPEXPORT(ProgressPopStep);
	APPEXPORT(ProgressPopKick);

	APPEXPORT(WindowInit);
	APPEXPORT(WindowNotifyError);
    *)
	LibMain := 1;
end;

procedure Close;
begin

	if assigned( DefaultIcon ) then FreeIcon(DefaultIcon);
	
end;

{$ifdef OPascal}
initialization
{$else}
begin
{$endif}
  AppVersion := ULONG_ID(5, 0, 0, 0);

{$endif} {LINK_CLIBS}
end.
